@@ -7,13 +7,15 @@
  */
 
 /* eslint-env worker */
-/* import-globals-from ../../modules/ical.js */
+/* import-globals-from ../../modules/Ical.jsm */
 
 var NS_OK = 0;
 var NS_ERROR_FAILURE = 2147500037;
 
 // eslint-disable-next-line no-unused-vars
-importScripts("resource://calendar/modules/ical.js");
+importScripts("resource:///modules/calendar/Ical.jsm");
+
+ICAL.design.strict = false;
 
 onmessage = function(event) {
   try {
