@@ -2,9 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from item-editing/calendar-item-editing.js */
-/* import-globals-from calendar-management.js */
-/* import-globals-from import-export.js */
+/* globals getSelectedCalendar, MODE_RDONLY, startBatchTransaction, doTransaction,
+   endBatchTransaction, createEventWithDialog, createTodoWithDialog */
 
 /* exported invokeEventDragSession,
  *          calendarMailButtonDNDObserver, calendarCalendarButtonDNDObserver,
@@ -464,7 +463,6 @@ var calendarTaskButtonDNDObserver;
       inputStream.init(localFileInstance, MODE_RDONLY, parseInt("0444", 8), {});
 
       try {
-        // XXX support csv
         let importer = Cc["@mozilla.org/calendar/import;1?type=ics"].getService(Ci.calIImporter);
         let items = importer.importFromStream(inputStream);
         this.onDropItems(items);
@@ -900,7 +898,7 @@ function invokeEventDragSession(aItem, aXULBox) {
   mutArray.appendElement(transfer);
   aXULBox.sourceObject = aItem;
   try {
-    cal.getDragService().invokeDragSession(aXULBox, null, null, null, mutArray, action);
+    cal.dragService.invokeDragSession(aXULBox, null, null, null, mutArray, action);
   } catch (e) {
     if (e.result != Cr.NS_ERROR_FAILURE) {
       // Pressing Escape on some platforms results in NS_ERROR_FAILURE
