@@ -2,167 +2,21 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* exported getElementValue, setBooleanAttribute, showElement, hideElement,
- *          uncollapseElement, collapseElement, disableElementWithLock,
- *          enableElementWithLock, uncheckChildNodes, removeChildren,
- *          appendCalendarItems, setAttributeToChildren, checkRadioControl,
+/* exported disableElementWithLock,
+ *          enableElementWithLock,
+ *          appendCalendarItems, checkRadioControl,
  *          checkRadioControlAppmenu,
- *          processEnableCheckbox, updateListboxDeleteButton,
- *          updateUnitLabelPlural, updateMenuLabelsPlural, menuListSelectItem,
+ *          updateUnitLabelPlural, updateMenuLabelsPlural,
  *          getOptimalMinimumWidth, getOptimalMinimumHeight,
- *          getOtherOrientation, updateSelectedLabel, setupAttendanceMenu
+ *          setupAttendanceMenu
  */
 
 /* import-globals-from ../../../../toolkit/content/globalOverlay.js */
 
-var { cal } = ChromeUtils.import("resource://calendar/modules/calUtils.jsm");
+var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { PluralForm } = ChromeUtils.import("resource://gre/modules/PluralForm.jsm");
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-
-/**
- * Helper function for filling the form,
- * Set the value of a property of a XUL element
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- * @param aNewValue     value to set property to ( if undefined no change is made )
- * @param aPropertyName OPTIONAL name of property to set, default is "value",
- *                        use "checked" for radios & checkboxes, "data" for
- *                        drop-downs
- */
-function setElementValue(aElement, aNewValue, aPropertyName) {
-  cal.ASSERT(aElement, "aElement");
-
-  if (aNewValue !== undefined) {
-    if (typeof aElement == "string") {
-      aElement = document.getElementById(aElement);
-      cal.ASSERT(aElement, "aElement");
-    }
-
-    if (!aElement) {
-      return;
-    }
-
-    if (aNewValue === false) {
-      try {
-        aElement.removeAttribute(aPropertyName);
-      } catch (e) {
-        cal.ERROR(
-          "setElementValue: aElement.removeAttribute couldn't remove " +
-            aPropertyName +
-            " from " +
-            (aElement && aElement.localName) +
-            " e: " +
-            e +
-            "\n"
-        );
-      }
-    } else if (aPropertyName) {
-      try {
-        aElement.setAttribute(aPropertyName, aNewValue);
-      } catch (e) {
-        cal.ERROR(
-          "setElementValue: aElement.setAttribute couldn't set " +
-            aPropertyName +
-            " from " +
-            (aElement && aElement.localName) +
-            " to " +
-            aNewValue +
-            " e: " +
-            e +
-            "\n"
-        );
-      }
-    } else {
-      aElement.value = aNewValue;
-    }
-  }
-}
-
-/**
- * Helper function for getting data from the form,
- * Get the value of a property of a XUL element
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- * @param propertyName  OPTIONAL name of property to set, default is "value",
- *                        use "checked" for radios & checkboxes, "data" for
- *                        drop-downs
- * @return newValue     Value of property
- *
- */
-function getElementValue(aElement, aPropertyName) {
-  if (typeof aElement == "string") {
-    aElement = document.getElementById(aElement);
-  }
-  return aElement[aPropertyName || "value"];
-}
-
-/**
- * Sets the value of a boolean attribute by either setting the value or
- * removing the attribute
- *
- * @param aXulElement     The XUL element/string ID the attribute is applied to.
- * @param aAttribute      The name of the attribute
- * @param aValue          The boolean value
- * @return                Returns aValue (for chaining)
- */
-function setBooleanAttribute(aXulElement, aAttribute, aValue) {
-  setElementValue(aXulElement, aValue ? "true" : false, aAttribute);
-  return aValue;
-}
-
-/**
- * Unconditionally show the element (hidden attribute)
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- */
-function showElement(aElement) {
-  setElementValue(aElement, false, "hidden");
-}
-
-/**
- * Unconditionally hide the element (hidden attribute)
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- */
-function hideElement(aElement) {
-  setElementValue(aElement, "true", "hidden");
-}
-
-/**
- * Unconditionally show the element (collapsed attribute)
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- */
-function uncollapseElement(aElement) {
-  setElementValue(aElement, false, "collapsed");
-}
-
-/**
- * Unconditionally hide the element (collapsed attribute)
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- */
-function collapseElement(aElement) {
-  setElementValue(aElement, "true", "collapsed");
-}
-
-/**
- * Unconditionally enable the element (hidden attribute)
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- */
-function enableElement(aElement) {
-  setElementValue(aElement, false, "disabled");
-}
-
-/**
- * Unconditionally disable the element (hidden attribute)
- *
- * @param aElement      ID of XUL element to set, or the element node itself
- */
-function disableElement(aElement) {
-  setElementValue(aElement, "true", "disabled");
-}
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 
 /**
  * This function unconditionally disables the element for
@@ -177,7 +31,7 @@ function disableElement(aElement) {
  */
 function disableElementWithLock(elementId, lockId) {
   // unconditionally disable the element.
-  disableElement(elementId);
+  document.getElementById(elementId).setAttribute("disabled", "true");
 
   // remember that this element has been locked with
   // the key passed as argument. we keep a primitive
@@ -217,43 +71,11 @@ function enableElementWithLock(elementId, lockId) {
       element.removeAttribute("lock");
     }
     if (n <= 0) {
-      enableElement(elementId);
+      element.removeAttribute("disabled");
     }
   }
 }
 
-/**
- * Unchecks the commands of the child elements of a DOM-tree-node i.e of a menu
- *
- * @param aEvent    The event from which the target is taken to retrieve the
- *                    child elements
- */
-function uncheckChildNodes(aEvent) {
-  let liveList = aEvent.target.getElementsByAttribute("checked", "true");
-  for (let i = liveList.length - 1; i >= 0; i--) {
-    let commandName = liveList.item(i).getAttribute("command");
-    let command = document.getElementById(commandName);
-    if (command) {
-      command.setAttribute("checked", "false");
-    }
-  }
-}
-
-/**
- * Removes all child nodes of the given node
- *
- * @param aElement  The Node (or its id) to remove children from
- */
-function removeChildren(aElement) {
-  if (typeof aElement == "string") {
-    aElement = document.getElementById(aElement);
-  }
-
-  while (aElement.firstChild) {
-    aElement.lastChild.remove();
-  }
-}
-
 /**
  * Sorts a sorted array of calendars by pref |calendar.list.sortOrder|.
  * Repairs that pref if dangling entries exist.
@@ -283,10 +105,7 @@ function sortCalendarArray(calendars) {
   // check and repair pref when an array of all calendars has been passed:
   let sortOrderString = Services.prefs.getStringPref("calendar.list.sortOrder", "");
   let wantedOrderString = ret.map(calendar => calendar.id).join(" ");
-  if (
-    wantedOrderString != sortOrderString &&
-    cal.getCalendarManager().getCalendars({}).length == ret.length
-  ) {
+  if (wantedOrderString != sortOrderString && cal.manager.getCalendars().length == ret.length) {
     Services.prefs.setStringPref("calendar.list.sortOrder", wantedOrderString);
   }
 
@@ -308,7 +127,7 @@ function sortCalendarArray(calendars) {
  */
 function appendCalendarItems(aItem, aCalendarMenuParent, aCalendarToUse, aOnCommand) {
   let calendarToUse = aCalendarToUse || aItem.calendar;
-  let calendars = sortCalendarArray(cal.getCalendarManager().getCalendars({}));
+  let calendars = sortCalendarArray(cal.manager.getCalendars());
   let indexToSelect = 0;
   let index = -1;
   for (let i = 0; i < calendars.length; ++i) {
@@ -333,6 +152,9 @@ function appendCalendarItems(aItem, aCalendarMenuParent, aCalendarToUse, aOnComm
       if (calendarToUse && calendarToUse.id == calendar.id) {
         indexToSelect = index;
       }
+      let cssSafeId = cal.view.formatStringForCSSRule(calendar.id);
+      menuitem.style.setProperty("--item-color", `var(--calendar-${cssSafeId}-backcolor)`);
+      menuitem.classList.add("menuitem-iconic");
     }
   }
   return indexToSelect;
@@ -365,104 +187,6 @@ function addMenuItem(aParent, aLabel, aValue, aCommand) {
   return item;
 }
 
-/**
- * Sets a given attribute value on the children of a passed node
- *
- * @param aParent           The parent node.
- * @param aAttribute        The name of the attribute to be set.
- * @param aValue            The value of the attribute.
- * @param aFilterAttribute  (optional) The name of an attribute that the child nodes carry
- *                            and that is used to filter the childnodes.
- * @param aFilterValue      (optional) The value of the filterattribute. If set only those
- *                            childnodes are modified that have an attribute
- *                            'aFilterAttribute' with the given value
- *                            'aFilterValue' set.
- */
-function setAttributeToChildren(aParent, aAttribute, aValue, aFilterAttribute, aFilterValue) {
-  for (let i = 0; i < aParent.childNodes.length; i++) {
-    let element = aParent.childNodes[i];
-    if (aFilterAttribute == null) {
-      setElementValue(element, aValue, aAttribute);
-    } else if (element.hasAttribute(aFilterAttribute)) {
-      let compValue = element.getAttribute(aFilterAttribute);
-      if (compValue === aFilterValue) {
-        setElementValue(element, aValue, aAttribute);
-      }
-    }
-  }
-}
-
-/**
- * Checks a radio control or a radio-menuitem.
- *
- * @param {Element} parent  The parent node of the 'radio controls', either:
- *                           - `radio` elements
- *                           - `menuitem` elements of type 'radio'
- *                           - `toolbarbutton` elements of type 'radio' (e.g. in appmenu)
- * @param {string} value    The value of the radio control bound to be checked.
- * @return {boolean}        Whether the 'radio control' could be checked.
- */
-function checkRadioControl(parent, value) {
-  for (const element of parent.childNodes) {
-    if (element.hasAttribute("value")) {
-      let compValue = element.getAttribute("value");
-      if (compValue == value) {
-        if (element.localName == "menuitem" || element.localName == "toolbarbutton") {
-          if (element.getAttribute("type") == "radio") {
-            element.setAttribute("checked", "true");
-            return true;
-          }
-        } else if (element.localName == "radio") {
-          element.radioGroup.selectedItem = element;
-          return true;
-        }
-      }
-    }
-  }
-  return false;
-}
-
-/**
- * Checks a `toolbarbutton` of type "radio" in the appmenu. (For some reason we
- * need to clear all the checked values before checking the new one.)
- *
- * @param {Element} parent  The parent node that contains the radio elements.
- * @param {string} value    The value of the radio element to be checked.
- * @return {boolean}        Whether the radio element could be checked.
- */
-function checkRadioControlAppmenu(parent, value) {
-  for (const node of parent.childNodes) {
-    if (node.localName == "toolbarbutton" && node.getAttribute("type") == "radio") {
-      node.removeAttribute("checked");
-    }
-  }
-  return checkRadioControl(parent, value);
-}
-
-/**
- * Enables or disables the given element depending on the checkbox state.
- *
- * @param checkboxId    The ID of the XUL checkbox element.
- * @param elementId     The element to change the disabled state on.
- */
-function processEnableCheckbox(checkboxId, elementId) {
-  let checked = document.getElementById(checkboxId).checked;
-  setElementValue(elementId, !checked && "true", "disabled");
-}
-
-/**
- * Enable/disable button if there are children in a listbox
- *
- * XXX This function needs renaming, it can do more than just buttons.
- *
- * @param listboxId     The ID of the listbox to check.
- * @param buttonId      The element to change the disabled state on.
- */
-function updateListboxDeleteButton(listboxId, buttonId) {
-  let rowCount = document.getElementById(listboxId).getRowCount();
-  setElementValue(buttonId, rowCount < 1 && "true", "disabled");
-}
-
 /**
  * Gets the correct plural form of a given unit.
  *
@@ -522,47 +246,6 @@ function updateMenuLabelsPlural(aLengthFieldId, aMenuId) {
   menu.selectedIndex = saveSelectedIndex;
 }
 
-/**
- * Select value in menuList. Throws string if no such value.
- *
- * XXX Isn't it enough to just do menuList.value = value ?
- *
- * @param menuListId    The ID of the menulist to check.
- * @param value         The value to set.
- * @throws              String error if value not found.
- */
-function menuListSelectItem(menuListId, value) {
-  let menuList = document.getElementById(menuListId);
-  let index = menuListIndexOf(menuList, value);
-  if (index == -1) {
-    throw new Error("menuListSelectItem: No such Element: " + value);
-  } else {
-    menuList.selectedIndex = index;
-  }
-}
-
-/**
- * Find index of menuitem with the given value, or return -1 if not found.
- *
- * @param menuListId    The XUL menulist node to check.
- * @param value         The value to look for.
- * @return              The child index of the node that matches, or -1.
- */
-function menuListIndexOf(menuList, value) {
-  let items = menuList.menupopup.childNodes;
-  let index = -1;
-  for (let i = 0; i < items.length; i++) {
-    let element = items[i];
-    if (element.nodeName == "menuitem") {
-      index++;
-    }
-    if (element.getAttribute("value") == value) {
-      return index;
-    }
-  }
-  return -1; // not found
-}
-
 /**
  * A helper function to calculate and add up certain css-values of a box.
  * It is required, that all css values can be converted to integers
@@ -626,32 +309,6 @@ function getOptimalMinimumHeight(aXULElement) {
   return firstEntity + secondEntity;
 }
 
-/**
- * Gets the "other" orientation value, i.e if "horizontal" is passed, "vertical"
- * is returned and vice versa.
- *
- * @param aOrientation    The orientation value to turn around.
- * @return                The opposite orientation value.
- */
-function getOtherOrientation(aOrientation) {
-  return aOrientation == "horizontal" ? "vertical" : "horizontal";
-}
-
-/**
- * Setting labels on a menuitem doesn't update the label that is shown when the
- * menuitem is selected. This function takes care by reselecting the item
- *
- * @param aElement  The element to update, or its id as a string
- */
-function updateSelectedLabel(aElement) {
-  if (typeof aElement == "string") {
-    aElement = document.getElementById(aElement);
-  }
-  let selectedIndex = aElement.selectedIndex;
-  aElement.selectedIndex = -1;
-  aElement.selectedIndex = selectedIndex;
-}
-
 /**
  * Sets up the attendance context menu, based on the given items
  *
@@ -843,7 +500,7 @@ function setupAttendanceMenu(aMenu, aItems) {
     let party = null;
     if (cal.itip.isInvitation(aItem)) {
       party = cal.itip.getInvitedAttendee(aItem);
-    } else if (aItem.organizer && aItem.getAttendees({}).length) {
+    } else if (aItem.organizer && aItem.getAttendees().length) {
       let calOrgId = aItem.calendar.getProperty("organizerId");
       if (calOrgId && calOrgId.toLowerCase() == aItem.organizer.id.toLowerCase()) {
         party = aItem.organizer;
