@@ -2,17 +2,15 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* globals calFilter, calFilter, getViewBox, openEventDialogForViewing,
+   modifyEventWithDialog, createEventWithDialog, currentView,
+   calendarController, editSelectedEvents, deleteSelectedEvents,
+   calendarUpdateDeleteCommand, getEventStatusString, goToggleToolbar */
+
 /* exported gCalendarEventTreeClicked, unifinderDoubleClick, unifinderKeyPress,
  *          focusSearch, ensureUnifinderLoaded, toggleUnifinder
  */
 
-/* import-globals-from ../../../mail/base/content/utilityOverlay.js */
-/* import-globals-from ../src/calFilter.js */
-/* import-globals-from item-editing/calendar-item-editing.js */
-/* import-globals-from widgets/mouseoverPreviews.js */
-/* import-globals-from calendar-command-controller.js */
-/* import-globals-from calendar-views-utils.js */
-
 /**
  * U N I F I N D E R
  *
@@ -88,11 +86,7 @@ var unifinderObserver = {
   onLoad() {},
 
   onAddItem(aItem) {
-    if (
-      aItem.isEvent() &&
-      !gUnifinderNeedsRefresh &&
-      unifinderTreeView.mFilter.isItemInFilters(aItem)
-    ) {
+    if (aItem.isEvent() && !gUnifinderNeedsRefresh) {
       this.addItemToTree(aItem);
     }
   },
@@ -313,6 +307,10 @@ function unifinderDoubleClick(event) {
   let calendarEvent = unifinderTreeView.getItemFromEvent(event);
 
   if (calendarEvent) {
+    if (Services.prefs.getBoolPref("calendar.events.defaultActionEdit", true)) {
+      modifyEventWithDialog(calendarEvent, true);
+      return;
+    }
     openEventDialogForViewing(calendarEvent);
   } else {
     createEventWithDialog();
@@ -907,27 +905,12 @@ function addItemsFromSingleCalendarInternal(eventArray) {
   unifinderTreeView.setSelectedItems();
 }
 
-function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
+async function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
   if (isUnifinderHidden()) {
     // If the unifinder is hidden, don't refresh the events to reduce needed
     // getItems calls.
     return;
   }
-  let refreshListener = {
-    QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
-    mEventArray: [],
-
-    onOperationComplete(aOpCalendar, aStatus, aOperationType, aId, aDateTime) {
-      let refreshTreeInternalFunc = function() {
-        aAddItemsInternalFunc(refreshListener.mEventArray);
-      };
-      setTimeout(refreshTreeInternalFunc, 0);
-    },
-
-    onGetResult(aOpCalendar, aStatus, aItemType, aDetail, aItems) {
-      refreshListener.mEventArray = refreshListener.mEventArray.concat(aItems);
-    },
-  };
 
   let filter = 0;
 
@@ -943,13 +926,17 @@ function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
     filter |= aCalendar.ITEM_FILTER_CLASS_OCCURRENCES;
   }
 
-  aCalendar.getItems(
+  let items = await aCalendar.getItemsAsArray(
     filter,
     0,
     unifinderTreeView.mFilter.startDate,
-    unifinderTreeView.mFilter.endDate,
-    refreshListener
+    unifinderTreeView.mFilter.endDate
   );
+
+  let refreshTreeInternalFunc = function() {
+    aAddItemsInternalFunc(items);
+  };
+  setTimeout(refreshTreeInternalFunc, 0);
 }
 
 function removeItemsFromCalendar(aCalendar) {
