@@ -2,7 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* exported switchToView, getSelectedDay, scheduleMidnightUpdate, minimonthPick,
+/* exported switchToView, minimonthPick,
  *          observeViewDaySelect, toggleOrientation,
  *          toggleWorkdaysOnly, toggleTasksInView, toggleShowCompletedInView,
  *          goToDate, gLastShownCalendarView, deleteSelectedEvents,
@@ -105,6 +105,9 @@ var calendarViewController = {
    * @see calICalendarViewController
    */
   deleteOccurrences(occurrencesArg, useParentItems, doNotConfirm, extResponseArg = null) {
+    if (!cal.window.promptDeleteItems(occurrencesArg)) {
+      return;
+    }
     startBatchTransaction();
     let recurringItems = {};
     let extResponse = extResponseArg || { responseMode: Ci.calIItipItem.USER };
@@ -260,20 +263,24 @@ function switchToView(viewType) {
   ids = ["previous-view-button", "today-view-button", "next-view-button"];
   ids.forEach(x => setupViewNode(x, "tooltiptext"));
 
-  try {
-    selectedDay = getSelectedDay();
-    currentSelection = currentView().getSelectedItems();
-  } catch (ex) {
-    // This dies if no view has even been chosen this session, but that's
-    // ok because we'll just use cal.dtz.now() below.
+  // Anyone wanting to plug in a view needs to follow this naming scheme
+  let view = document.getElementById(viewType + "-view");
+  let oldView = currentView();
+  if (oldView?.isActive) {
+    if (oldView == view) {
+      // Not actually changing view, there's nothing else to do.
+      return;
+    }
+
+    selectedDay = oldView.selectedDay;
+    currentSelection = oldView.getSelectedItems();
+    oldView.isActive = false;
+    oldView.clearItems();
   }
 
   if (!selectedDay) {
     selectedDay = cal.dtz.now();
   }
-
-  // Anyone wanting to plug in a view needs to follow this naming scheme
-  let view = document.getElementById(viewType + "-view");
   for (let i = 0; i < viewBox.children.length; i++) {
     if (view.id == viewBox.children[i].id) {
       viewBox.children[i].hidden = false;
@@ -287,9 +294,7 @@ function switchToView(viewType) {
   let viewTabs = document.getElementById("view-tabs");
   viewTabs.selectedIndex = viewBox.getAttribute("selectedIndex");
 
-  let compositeCal = cal.view.getCompositeCalendar(window);
-  if (view.displayCalendar != compositeCal) {
-    view.displayCalendar = compositeCal;
+  if (!view.controller) {
     view.timezone = cal.dtz.defaultTimezone;
     view.controller = calendarViewController;
   }
@@ -298,6 +303,8 @@ function switchToView(viewType) {
   view.setSelectedItems(currentSelection);
 
   view.onResize(view);
+  view.isActive = true;
+  view.refreshItems();
 }
 
 /**
@@ -323,71 +330,6 @@ function currentView() {
   return null;
 }
 
-/**
- * Returns the selected day in the current view.
- *
- * @return      The selected day
- */
-function getSelectedDay() {
-  return currentView().selectedDay;
-}
-
-var gMidnightTimer;
-
-/**
- * Creates a timer that will fire after midnight.  Pass in a function as
- * aRefreshCallback that should be called at that time.
- *
- * XXX This function is not very usable, since there is only one midnight timer.
- * Better would be a function that uses the observer service to notify at
- * midnight.
- *
- * @param refreshCallback      A callback to be called at midnight.
- */
-function scheduleMidnightUpdate(refreshCallback) {
-  let jsNow = new Date();
-  let tomorrow = new Date(jsNow.getFullYear(), jsNow.getMonth(), jsNow.getDate() + 1);
-  let msUntilTomorrow = tomorrow.getTime() - jsNow.getTime();
-
-  // Is an nsITimer/callback extreme overkill here? Yes, but it's necessary to
-  // workaround bug 291386.  If we don't, we stand a decent chance of getting
-  // stuck in an infinite loop.
-  let udCallback = {
-    notify(timer) {
-      refreshCallback();
-    },
-  };
-
-  if (gMidnightTimer) {
-    gMidnightTimer.cancel();
-  } else {
-    // Observer for wake after sleep/hibernate/standby to create new timers and refresh UI
-    let wakeObserver = {
-      observe(subject, topic, data) {
-        if (topic == "wake_notification") {
-          // postpone refresh for another couple of seconds to get netwerk ready:
-          if (this.mTimer) {
-            this.mTimer.cancel();
-          } else {
-            this.mTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
-          }
-          this.mTimer.initWithCallback(udCallback, 10 * 1000, Ci.nsITimer.TYPE_ONE_SHOT);
-        }
-      },
-    };
-
-    // Add observer
-    Services.obs.addObserver(wakeObserver, "wake_notification");
-
-    // Remove observer on unload
-    window.addEventListener("unload", () => {
-      Services.obs.removeObserver(wakeObserver, "wake_notification");
-    });
-    gMidnightTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
-  }
-  gMidnightTimer.initWithCallback(udCallback, msUntilTomorrow, gMidnightTimer.TYPE_ONE_SHOT);
-}
-
 /**
  * Handler function to set the selected day in the minimonth to the currently
  * selected day in the current view.
@@ -585,7 +527,7 @@ function deleteSelectedEvents() {
 function viewSelectedEvents() {
   let items = currentView().getSelectedItems();
   if (items.length >= 1) {
-    openEventDialog(items[0], items[0].calendar, "view");
+    openEventDialogForViewing(items[0]);
   }
 }
 
@@ -602,20 +544,7 @@ function editSelectedEvents() {
 /**
  * Select all events from all calendars. Use with care.
  */
-function selectAllEvents() {
-  let items = [];
-  let listener = {
-    QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
-    onOperationComplete(calendar, status, operationType, id, detail) {
-      currentView().setSelectedItems(items, false);
-    },
-    onGetResult(calendar, status, itemType, detail, itemsArg) {
-      for (let item of itemsArg) {
-        items.push(item);
-      }
-    },
-  };
-
+async function selectAllEvents() {
   let composite = cal.view.getCompositeCalendar(window);
   let filter = composite.ITEM_FILTER_CLASS_OCCURRENCES;
 
@@ -634,7 +563,8 @@ function selectAllEvents() {
   let end = currentView().endDay.clone();
   end.day += 1;
 
-  composite.getItems(filter, 0, currentView().startDay, end, listener);
+  let items = await composite.getItemsAsArray(filter, 0, currentView().startDay, end);
+  currentView().setSelectedItems(items, false);
 }
 
 var calendarNavigationBar = {
@@ -642,14 +572,14 @@ var calendarNavigationBar = {
     let docTitle = "";
     if (startDate) {
       let intervalLabel = document.getElementById("intervalDescription");
-      let firstWeekNo = cal.getWeekInfoService().getWeekTitle(startDate);
+      let firstWeekNo = cal.weekInfoService.getWeekTitle(startDate);
       let secondWeekNo = firstWeekNo;
       let weekLabel = document.getElementById("calendarWeek");
       if (startDate.nativeTime == endDate.nativeTime) {
         intervalLabel.value = cal.dtz.formatter.formatDate(startDate);
       } else {
         intervalLabel.value = currentView().getRangeDescription();
-        secondWeekNo = cal.getWeekInfoService().getWeekTitle(endDate);
+        secondWeekNo = cal.weekInfoService.getWeekTitle(endDate);
       }
       if (secondWeekNo == firstWeekNo) {
         weekLabel.value = cal.l10n.getCalString("singleShortCalendarWeek", [firstWeekNo]);
