@@ -76,9 +76,11 @@ function pickFileToImport() {
       }
     }
 
+    picker.appendFilters(Ci.nsIFilePicker.filterAll);
     picker.open(returnValue => {
       if (returnValue == Ci.nsIFilePicker.returnCancel) {
         resolve();
+        return;
       }
       resolve(picker.file);
     });
@@ -86,33 +88,13 @@ function pickFileToImport() {
 }
 
 /**
- * Given a file, return the contractId for an importer for the file type.
- *
- * @param {nsIFile} file - A file.
- * @return {string} A contract ID.
- */
-function getContractIdForImportingFile(file) {
-  let fileNameLower = file.leafName.toLowerCase();
-  let contractId = "@mozilla.org/calendar/import;1?type=";
-
-  if (fileNameLower.endsWith(".ics")) {
-    return contractId + "ics";
-  } else if (fileNameLower.endsWith(".csv")) {
-    return contractId + "csv";
-  }
-  throw new Error("No importer for this type of file: " + file.leafName);
-}
-
-/**
- * Given a file (e.g. an ICS or CSV file), return an array of calendar items
- * parsed from it.
+ * Given an ICS file, return an array of calendar items parsed from it.
  *
  * @param {nsIFile} file - File to get items from.
  * @return {calIItemBase[]} Array of calendar items.
  */
-function getItemsFromFile(file) {
-  let contractId = getContractIdForImportingFile(file);
-  let importer = Cc[contractId].getService(Ci.calIImporter);
+function getItemsFromIcsFile(file) {
+  let importer = Cc["@mozilla.org/calendar/import;1?type=ics"].getService(Ci.calIImporter);
 
   let inputStream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(
     Ci.nsIFileInputStream
@@ -192,11 +174,9 @@ async function putItemsIntoCal(destCal, aItems, aListener) {
   let count = 0;
   let total = aItems.length;
 
-  // Using wrappedJSObject is a hack that is needed to prevent a proxy error.
-  let pcal = cal.async.promisifyCalendar(destCal.wrappedJSObject);
   for (let item of aItems) {
     try {
-      await pcal.addItem(item);
+      await destCal.addItem(item);
     } catch (e) {
       if (e == Ci.calIErrors.DUPLICATE_ID) {
         await callListener("onDuplicate", item, e);
@@ -287,15 +267,12 @@ function saveEventsToFile(calendarEventArray, aDefaultFileName) {
       filePath += "." + exporter.getFileTypes()[0].defaultExtension;
     }
 
-    const nsIFile = Ci.nsIFile;
-    const nsIFileOutputStream = Ci.nsIFileOutputStream;
-
     let outputStream;
-    let localFileInstance = Cc["@mozilla.org/file/local;1"].createInstance(nsIFile);
+    let localFileInstance = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsIFile);
     localFileInstance.initWithPath(filePath);
 
     outputStream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(
-      nsIFileOutputStream
+      Ci.nsIFileOutputStream
     );
     try {
       outputStream.init(
@@ -322,27 +299,15 @@ function saveEventsToFile(calendarEventArray, aDefaultFileName) {
  * @param aCalendar     (optional) A specific calendar to export
  */
 function exportEntireCalendar(aCalendar) {
-  let itemArray = [];
-  let getListener = {
-    QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
-    onOperationComplete(aOpCalendar, aStatus, aOperationType, aId, aDetail) {
-      saveEventsToFile(itemArray, aOpCalendar.name);
-    },
-    onGetResult(aOpCalendar, aStatus, aItemType, aDetail, aItems) {
-      for (let item of aItems) {
-        itemArray.push(item);
-      }
-    },
-  };
-
-  let getItemsFromCal = function(aCal) {
-    aCal.getItems(Ci.calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null, getListener);
+  let getItemsFromCal = async function(aCal) {
+    let items = await aCal.getItemsAsArray(Ci.calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null);
+    saveEventsToFile(items, aCal.name);
   };
 
   if (aCalendar) {
     getItemsFromCal(aCalendar);
   } else {
-    let calendars = cal.getCalendarManager().getCalendars();
+    let calendars = cal.manager.getCalendars();
 
     if (calendars.length == 1) {
       // There's only one calendar, so it's silly to ask what calendar
