@@ -121,7 +121,7 @@
                       hidden="true"
                       align="center"
                       disable-on-readonly="true">
-                  <html:img class="attachment-icon"
+                  <html:img class="attachment-icon invisible-on-broken"
                             alt="" />
                   <label class="text-link item-attachment-cell-label"
                          crop="end"
@@ -143,9 +143,14 @@
           </vbox>
         </box>
 
+        <splitter id="attendeeDescriptionSplitter"
+                  class="item-summary-splitter"
+                  collapse="after"
+                  orient="vertical"
+                  state="open"/>
+
         <!-- Description -->
         <box class="item-description-box" hidden="true" orient="vertical" flex="1">
-          <spacer class="default-spacer"/>
           <hbox class="calendar-caption" align="center">
             <label value="&read.only.description.label;"
                    class="header"/>
@@ -315,6 +320,11 @@
             (this.mIsInvitation && cal.acl.userCanRespondToInvitation(item)))
         );
       }
+
+      if (!item.descriptionHTML || !item.getAttendees().length) {
+        // Hide the splitter when there is no description or attendees.
+        document.getElementById("attendeeDescriptionSplitter").setAttribute("hidden", "true");
+      }
     }
 
     get item() {
@@ -687,21 +697,8 @@
 
       let link = itemDescription.contentDocument.createElement("link");
       link.rel = "stylesheet";
-      link.href = "chrome://messenger/content/messengercompose/EditorContent.css";
+      link.href = "chrome://messenger/skin/shared/editorContent.css";
       itemDescription.contentDocument.head.appendChild(link);
-
-      let editorLink = itemDescription.contentDocument.createElement("link");
-      editorLink.rel = "stylesheet";
-      editorLink.href = "chrome://messenger/content/messengercompose/calendar-event-editor.css";
-      itemDescription.contentDocument.head.appendChild(editorLink);
-
-      // Layout the dialog,...
-      requestAnimationFrame(() => {
-        // ... then resize the iframe to fit its content. I don't know why, but the scroll height
-        // seems to be a few pixels smaller than we need to avoid vertical scrolling.
-        itemDescription.style.height =
-          Math.min(itemDescription.contentDocument.documentElement.scrollHeight + 10, 500) + "px";
-      });
     }
 
     /**
