@@ -11,19 +11,19 @@ function calICALJSTimezone(innerObject) {
 }
 
 calICALJSTimezone.prototype = {
-  QueryInterface: ChromeUtils.generateQI([Ci.calITimezone]),
+  QueryInterface: ChromeUtils.generateQI(["calITimezone"]),
   classID: Components.ID("{6702eb17-a968-4b43-b562-0d0c5f8e9eb5}"),
 
   innerObject: null,
 
   get provider() {
-    return cal.getTimezoneService();
+    return cal.timezoneService;
   },
   get icalComponent() {
     let innerComp = this.innerObject.component;
     let comp = null;
     if (innerComp) {
-      comp = cal.getIcsService().createIcalComponent("VTIMEZONE");
+      comp = cal.icsService.createIcalComponent("VTIMEZONE");
       comp.icalComponent = innerComp;
     }
     return comp;
@@ -73,7 +73,7 @@ function calLibicalTimezone(tzid, component, latitude, longitude) {
   this.longitude = longitude;
 }
 calLibicalTimezone.prototype = {
-  QueryInterface: ChromeUtils.generateQI([Ci.calITimezone]),
+  QueryInterface: ChromeUtils.generateQI(["calITimezone"]),
   classID: Components.ID("{6702eb17-a968-4b43-b562-0d0c5f8e9eb5}"),
 
   toString() {
@@ -87,8 +87,7 @@ calLibicalTimezone.prototype = {
   get icalComponent() {
     let comp = this.mComponent;
     if (comp && typeof comp == "string") {
-      this.mComponent = cal
-        .getIcsService()
+      this.mComponent = cal.icsService
         .parseICS("BEGIN:VCALENDAR\r\n" + comp + "\r\nEND:VCALENDAR\r\n", null)
         .getFirstSubcomponent("VTIMEZONE");
     }
@@ -98,7 +97,7 @@ calLibicalTimezone.prototype = {
   get displayName() {
     if (this.mDisplayName === undefined) {
       try {
-        let bundle = cal.getTimezoneService().wrappedJSObject.stringBundle;
+        let bundle = cal.timezoneService.wrappedJSObject.stringBundle;
         this.mDisplayName = bundle.GetStringFromName(
           "pref.timezone." + this.tzid.replace(/\//g, ".")
         );
@@ -112,6 +111,6 @@ calLibicalTimezone.prototype = {
   },
 
   get provider() {
-    return cal.getTimezoneService();
+    return cal.timezoneService;
   },
 };
