@@ -2,6 +2,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalEvent: "resource:///modules/CalEvent.jsm",
+});
+
 function run_test() {
   do_calendar_startup(run_next_test);
 }
@@ -21,7 +27,6 @@ function check_delmgr_call(aFunc) {
 }
 
 add_task(async function test_deleted_items() {
-  let calmgr = cal.getCalendarManager();
   let delmgr = Cc["@mozilla.org/calendar/deleted-items-manager;1"].getService(Ci.calIDeletedItems);
   // No items have been deleted, retrieving one should return null.
   equal(delmgr.getDeletedDate("random"), null);
@@ -31,16 +36,16 @@ add_task(async function test_deleted_items() {
   // error.
   await check_delmgr_call(() => delmgr.flush());
 
-  let memory = calmgr.createCalendar("memory", Services.io.newURI("moz-storage-calendar://"));
-  calmgr.registerCalendar(memory);
+  let memory = cal.manager.createCalendar("memory", Services.io.newURI("moz-storage-calendar://"));
+  cal.manager.registerCalendar(memory);
 
-  let item = cal.createEvent();
+  let item = new CalEvent();
   item.id = "test-item-1";
   item.startDate = cal.dtz.now();
   item.endDate = cal.dtz.now();
 
   // Add the item, it still shouldn't be in the deleted database.
-  await check_delmgr_call(() => memory.addItem(item, null));
+  await check_delmgr_call(() => memory.addItem(item));
   equal(delmgr.getDeletedDate(item.id), null);
   equal(delmgr.getDeletedDate(item.id, memory.id), null);
 
@@ -56,7 +61,7 @@ add_task(async function test_deleted_items() {
   };
 
   // Deleting an item should trigger it being marked for deletion.
-  await check_delmgr_call(() => memory.deleteItem(item, null));
+  await check_delmgr_call(() => memory.deleteItem(item));
 
   // Now check if it was deleted at our reference date.
   let deltime = delmgr.getDeletedDate(item.id);
@@ -81,11 +86,11 @@ add_task(async function test_deleted_items() {
   useFutureDate = false;
 
   // Add, delete, add. Item should no longer be deleted.
-  await check_delmgr_call(() => memory.addItem(item, null));
+  await check_delmgr_call(() => memory.addItem(item));
   equal(delmgr.getDeletedDate(item.id), null);
-  await check_delmgr_call(() => memory.deleteItem(item, null));
+  await check_delmgr_call(() => memory.deleteItem(item));
   equal(delmgr.getDeletedDate(item.id).compare(referenceDate), 0);
-  await check_delmgr_call(() => memory.addItem(item, null));
+  await check_delmgr_call(() => memory.addItem(item));
   equal(delmgr.getDeletedDate(item.id), null);
 
   // Revert now function, in case more tests are written.
