@@ -2,6 +2,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalAttachment: "resource:///modules/CalAttachment.jsm",
+  CalAttendee: "resource:///modules/CalAttendee.jsm",
+  CalRelation: "resource:///modules/CalRelation.jsm",
+});
+
 function run_test() {
   do_calendar_startup(really_run_test);
 }
@@ -43,7 +51,7 @@ function test_icalstring() {
   }
 
   let attach = checkComp(
-    cal.createAttachment.bind(cal),
+    icalString => new CalAttachment(icalString),
     "ATTACH;ENCODING=BASE64;FMTTYPE=text/calendar;FILENAME=test.ics:http://example.com/test.ics",
     { formatType: "text/calendar", encoding: "BASE64" },
     { FILENAME: "test.ics" }
@@ -51,7 +59,7 @@ function test_icalstring() {
   equal(attach.uri.spec, "http://example.com/test.ics");
 
   checkComp(
-    cal.createAttendee.bind(cal),
+    icalString => new CalAttendee(icalString),
     "ATTENDEE;RSVP=TRUE;CN=Name;PARTSTAT=ACCEPTED;CUTYPE=RESOURCE;ROLE=REQ-PARTICIPANT;X-THING=BAR:mailto:test@example.com",
     {
       id: "mailto:test@example.com",
@@ -66,7 +74,7 @@ function test_icalstring() {
   );
 
   checkComp(
-    cal.createRelation.bind(cal),
+    icalString => new CalRelation(icalString),
     "RELATED-TO;RELTYPE=SIBLING;FOO=BAR:VALUE",
     { relType: "SIBLING", relId: "VALUE" },
     { FOO: "BAR" }
@@ -106,8 +114,6 @@ function test_icalstring() {
 }
 
 function test_icsservice() {
-  let svc = cal.getIcsService();
-
   function checkProp(createFunc, icalString, members, parameters) {
     let thing = createFunc(icalString);
     equal(ics_unfoldline(thing.icalString), icalString + "\r\n");
@@ -124,14 +130,14 @@ function test_icsservice() {
 
   // Test ::createIcalPropertyFromString
   checkProp(
-    svc.createIcalPropertyFromString.bind(svc),
+    cal.icsService.createIcalPropertyFromString.bind(cal.icsService),
     "ATTACH;ENCODING=BASE64;FMTTYPE=text/calendar;FILENAME=test.ics:http://example.com/test.ics",
     { value: "http://example.com/test.ics", propertyName: "ATTACH" },
     { ENCODING: "BASE64", FMTTYPE: "text/calendar", FILENAME: "test.ics" }
   );
 
   checkProp(
-    svc.createIcalPropertyFromString.bind(svc),
+    cal.icsService.createIcalPropertyFromString.bind(cal.icsService),
     "DESCRIPTION:new\\nlines\\nare\\ngreat\\,eh?",
     {
       value: "new\nlines\nare\ngreat,eh?",
@@ -141,50 +147,48 @@ function test_icsservice() {
   );
 
   // Test ::createIcalProperty
-  let attach2 = svc.createIcalProperty("ATTACH");
+  let attach2 = cal.icsService.createIcalProperty("ATTACH");
   equal(attach2.propertyName, "ATTACH");
   attach2.value = "http://example.com/";
   equal(attach2.icalString, "ATTACH:http://example.com/\r\n");
 }
 
 function test_icalproperty() {
-  let svc = cal.getIcsService();
-  let comp = svc.createIcalComponent("VEVENT");
-  let prop = svc.createIcalProperty("PROP");
+  let comp = cal.icsService.createIcalComponent("VEVENT");
+  let prop = cal.icsService.createIcalProperty("PROP");
   prop.value = "VAL";
 
   comp.addProperty(prop);
   equal(prop.parent.toString(), comp.toString());
   equal(prop.valueAsDatetime, null);
 
-  prop = svc.createIcalProperty("DESCRIPTION");
+  prop = cal.icsService.createIcalProperty("DESCRIPTION");
   prop.value = "A\nB";
   equal(prop.value, "A\nB");
   equal(prop.valueAsIcalString, "A\\nB");
   equal(prop.valueAsDatetime, null);
 
-  prop = svc.createIcalProperty("DESCRIPTION");
+  prop = cal.icsService.createIcalProperty("DESCRIPTION");
   prop.valueAsIcalString = "A\\nB";
   equal(prop.value, "A\nB");
   equal(prop.valueAsIcalString, "A\\nB");
   equal(prop.valueAsDatetime, null);
 
-  prop = svc.createIcalProperty("DESCRIPTION");
+  prop = cal.icsService.createIcalProperty("DESCRIPTION");
   prop.value = "A\\nB";
   equal(prop.value, "A\\nB");
   equal(prop.valueAsIcalString, "A\\\\nB");
   equal(prop.valueAsDatetime, null);
 
-  prop = svc.createIcalProperty("GEO");
+  prop = cal.icsService.createIcalProperty("GEO");
   prop.value = "43.4913662534171;12.085559129715";
   equal(prop.value, "43.4913662534171;12.085559129715");
   equal(prop.valueAsIcalString, "43.4913662534171;12.085559129715");
 }
 
 function test_icalcomponent() {
-  let svc = cal.getIcsService();
-  let event = svc.createIcalComponent("VEVENT");
-  let alarm = svc.createIcalComponent("VALARM");
+  let event = cal.icsService.createIcalComponent("VEVENT");
+  let alarm = cal.icsService.createIcalComponent("VALARM");
   event.addSubcomponent(alarm);
 
   // Check that the parent works and does not appear on cloned instances
@@ -231,8 +235,7 @@ function test_icalcomponent() {
 }
 
 function test_param() {
-  let svc = cal.getIcsService();
-  let prop = svc.createIcalProperty("DTSTART");
+  let prop = cal.icsService.createIcalProperty("DTSTART");
   prop.value = "20120101T010101";
   equal(prop.icalString, "DTSTART:20120101T010101\r\n");
   prop.setParameter("VALUE", "TEXT");
@@ -247,13 +250,11 @@ function test_param() {
 }
 
 function test_iterator() {
-  let svc = cal.getIcsService();
-
   // Property iterator
-  let comp = svc.createIcalComponent("VEVENT");
+  let comp = cal.icsService.createIcalComponent("VEVENT");
   let propNames = ["X-ONE", "X-TWO"];
   for (let i = 0; i < propNames.length; i++) {
-    let prop = svc.createIcalProperty(propNames[i]);
+    let prop = cal.icsService.createIcalProperty(propNames[i]);
     prop.value = "" + (i + 1);
     comp.addProperty(prop);
   }
@@ -270,7 +271,7 @@ function test_iterator() {
 
   // Property iterator with multiple values
   // eslint-disable-next-line no-useless-concat
-  comp = svc.parseICS("BEGIN:VEVENT\r\n" + "CATEGORIES:a,b,c\r\n" + "END:VEVENT", null);
+  comp = cal.icsService.parseICS("BEGIN:VEVENT\r\n" + "CATEGORIES:a,b,c\r\n" + "END:VEVENT", null);
   let propValues = ["a", "b", "c"];
   for (
     let prop = comp.getFirstProperty("CATEGORIES");
@@ -283,7 +284,7 @@ function test_iterator() {
   }
 
   // Param iterator
-  let dtstart = svc.createIcalProperty("DTSTART");
+  let dtstart = cal.icsService.createIcalProperty("DTSTART");
   let params = ["X-ONE", "X-TWO"];
   for (let i = 0; i < params.length; i++) {
     dtstart.setParameter(params[i], "" + (i + 1));
