@@ -2,8 +2,7 @@
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var irc = {};
-Services.scriptloader.loadSubScript("resource:///modules/irc.jsm", irc);
+var { ircMessage } = ChromeUtils.import("resource:///modules/irc.jsm");
 const { ircNonStandard } = ChromeUtils.import(
   "resource:///modules/ircNonStandard.jsm"
 );
@@ -57,7 +56,7 @@ function testSecureList() {
   const kSecureListMsg =
     ":fripp.mozilla.org NOTICE aleth-build :*** You cannot list within the first 60 seconds of connecting. Please try again later.";
 
-  let message = irc.ircMessage(kSecureListMsg, "");
+  let message = ircMessage(kSecureListMsg, "");
   let account = new FakeAccount();
   account.connected = true;
   let result = NOTICE.call(account, message);
@@ -84,7 +83,7 @@ function testZncAuth() {
   ];
 
   for (let msg of kZncMsgs) {
-    let message = irc.ircMessage(msg, "");
+    let message = ircMessage(msg, "");
     // No provided password.
     let account = new FakeAccount();
     let result = NOTICE.call(account, message);
@@ -143,7 +142,7 @@ function testUMich() {
 
   let account = new FakeAccount();
   for (let msg of kMsgs) {
-    let message = irc.ircMessage(msg, "");
+    let message = ircMessage(msg, "");
     let result = NOTICE.call(account, message);
 
     // These initial notices are not handled (i.e. they'll be subject to
@@ -153,7 +152,7 @@ function testUMich() {
 
   // And finally the last one should be printed out, always. It contains the
   // directions of what to do next.
-  let message = irc.ircMessage(kFinalMsg, "");
+  let message = ircMessage(kFinalMsg, "");
   let result = NOTICE.call(account, message);
   ok(result);
   equal(account.convs.length, 1);
@@ -172,7 +171,7 @@ function testAuthNick() {
   let account = new FakeAccount();
   account._nickname = "AUTH";
 
-  let message = irc.ircMessage(kMsg, "");
+  let message = ircMessage(kMsg, "");
   let result = NOTICE.call(account, message);
 
   // Since it is ambiguous if it was an authentication message or a message
@@ -198,7 +197,7 @@ function testIgnoredNotices() {
   for (let msg of kMsgs) {
     let account = new FakeAccount();
 
-    let message = irc.ircMessage(msg, "");
+    let message = ircMessage(msg, "");
     let result = NOTICE.call(account, message);
 
     // This message should *NOT* be shown.
