@@ -64,12 +64,12 @@ function appUpdater(options = {}) {
   );
 
   let manualURL = Services.urlFormatter.formatURLPref("app.update.url.manual");
-  document
-    .getElementById("manualLink")
-    .setAttribute("onclick", 'openURL("' + manualURL + '");');
-  document
-    .getElementById("failedLink")
-    .setAttribute("onclick", 'openURL("' + manualURL + '");');
+  document.getElementById("manualLink").href = manualURL;
+  document.getElementById("failedLink").href = manualURL;
+
+  if (this.updateDisabledByPackage) {
+    return;
+  }
 
   if (this.updateDisabledByPolicy) {
     this.selectPanel("policyDisabled");
@@ -113,6 +113,10 @@ appUpdater.prototype = {
   // true when there is an update check in progress.
   isChecking: false,
 
+  get selectedPanel() {
+    return this.updateDeck.querySelector(".selected");
+  },
+
   // true when there is an update ready to be applied on restart or staged.
   get isPending() {
     if (this.update) {
@@ -123,10 +127,10 @@ appUpdater.prototype = {
       );
     }
     return (
-      this.um.activeUpdate &&
-      (this.um.activeUpdate.state == "pending" ||
-        this.um.activeUpdate.state == "pending-service" ||
-        this.um.activeUpdate.state == "pending-elevate")
+      this.um.readyUpdate &&
+      (this.um.readyUpdate.state == "pending" ||
+        this.um.readyUpdate.state == "pending-service" ||
+        this.um.readyUpdate.state == "pending-elevate")
     );
   },
 
@@ -138,9 +142,9 @@ appUpdater.prototype = {
       );
     }
     return (
-      this.um.activeUpdate &&
-      (this.um.activeUpdate.state == "applied" ||
-        this.um.activeUpdate.state == "applied-service")
+      this.um.readyUpdate &&
+      (this.um.readyUpdate.state == "applied" ||
+        this.um.readyUpdate.state == "applied-service")
     );
   },
 
@@ -151,8 +155,8 @@ appUpdater.prototype = {
     let errorCode;
     if (this.update) {
       errorCode = this.update.errorCode;
-    } else if (this.um.activeUpdate) {
-      errorCode = this.um.activeUpdate.errorCode;
+    } else if (this.um.readyUpdate) {
+      errorCode = this.um.readyUpdate.errorCode;
     }
     // If the state is pending and the error code is not 0, staging must have
     // failed.
@@ -165,8 +169,8 @@ appUpdater.prototype = {
       let errorCode;
       if (this.update) {
         errorCode = this.update.errorCode;
-      } else if (this.um.activeUpdate) {
-        errorCode = this.um.activeUpdate.errorCode;
+      } else if (this.um.readyUpdate) {
+        errorCode = this.um.readyUpdate.errorCode;
       }
       // If the state is pending and the error code is not 0, staging must have
       // failed and Firefox should be restarted to try to apply the update
@@ -181,7 +185,10 @@ appUpdater.prototype = {
     if (this.update) {
       return this.update.state == "downloading";
     }
-    return this.um.activeUpdate && this.um.activeUpdate.state == "downloading";
+    return (
+      this.um.downloadingUpdate &&
+      this.um.downloadingUpdate.state == "downloading"
+    );
   },
 
   // true when updating has been disabled by enterprise policy
@@ -189,9 +196,31 @@ appUpdater.prototype = {
     return Services.policies && !Services.policies.isAllowed("appUpdate");
   },
 
+  // true if updating is disabled because we're running in an app package.
+  // This is distinct from updateDisabledByPolicy because we need to avoid
+  // messages being shown to the user about an "administrator" handling
+  // updates; packaged apps may be getting updated by an administrator or they
+  // may not be, and we don't have a good way to tell the difference from here,
+  // so we err to the side of less confusion for unmanaged users.
+  get updateDisabledByPackage() {
+    try {
+      return Services.sysinfo.getProperty("hasWinPackageId");
+    } catch (_ex) {
+      // The hasWinPackageId property doesn't exist; assume it would be false.
+    }
+    // Check for distributor.id and updates disabled via policy
+    let defaults = Services.prefs.getDefaultBranch(null);
+    let distribId = Boolean(defaults.getCharPref("distribution.id", ""));
+    return distribId && this.updateDisabledByPolicy;
+  },
+
   // true when updating in background is enabled.
   get updateStagingEnabled() {
-    return !this.updateDisabledByPolicy && this.aus.canStageUpdates;
+    return (
+      !this.updateDisabledByPolicy &&
+      !this.updateDisabledByPackage &&
+      this.aus.canStageUpdates
+    );
   },
 
   /**
@@ -214,8 +243,14 @@ appUpdater.prototype = {
           let month = buildID.slice(4, 6);
           let day = buildID.slice(6, 8);
           updateVersion += ` (${year}-${month}-${day})`;
+        } else {
+          let updateNotesLink = document.getElementById("updateNotes");
+          if (updateNotesLink) {
+            updateNotesLink.href = gAppUpdater.update.detailsURL;
+            updateNotesLink.hidden = false;
+          }
         }
-        button.label = this.bundle.formatStringFromName(
+        button.textContent = this.bundle.formatStringFromName(
           "update.downloadAndInstallButton.label",
           [updateVersion]
         );
@@ -223,7 +258,8 @@ appUpdater.prototype = {
           "update.downloadAndInstallButton.accesskey"
         );
       }
-      this.updateDeck.selectedPanel = panel;
+      this.selectedPanel?.classList.remove("selected");
+      panel.classList.add("selected");
       if (
         this.options.buttonAutoFocus &&
         (!document.commandDispatcher.focusedElement || // don't steal the focus
@@ -233,7 +269,8 @@ appUpdater.prototype = {
         button.focus();
       }
     } else {
-      this.updateDeck.selectedPanel = panel;
+      this.selectedPanel?.classList.remove("selected");
+      panel.classList.add("selected");
     }
   },
 
@@ -314,7 +351,7 @@ appUpdater.prototype = {
         if (gAppUpdater.update.detailsURL) {
           unsupportedLink.href = gAppUpdater.update.detailsURL;
         } else {
-          unsupportedLink.setAttribute("hidden", true);
+          unsupportedLink.hidden = true;
         }
 
         gAppUpdater.selectPanel("unsupportedSystem");
@@ -352,7 +389,7 @@ appUpdater.prototype = {
    */
   waitForUpdateToStage() {
     if (!this.update) {
-      this.update = this.um.activeUpdate;
+      this.update = this.um.readyUpdate;
     }
     this.update.QueryInterface(Ci.nsIWritablePropertyBag);
     this.update.setProperty("foregroundDownload", "true");
@@ -365,13 +402,13 @@ appUpdater.prototype = {
    */
   startDownload() {
     if (!this.update) {
-      this.update = this.um.activeUpdate;
+      this.update = this.um.downloadingUpdate;
     }
     this.update.QueryInterface(Ci.nsIWritablePropertyBag);
     this.update.setProperty("foregroundDownload", "true");
 
-    let state = this.aus.downloadUpdate(this.update, false);
-    if (state == "failed") {
+    let success = this.aus.downloadUpdate(this.update, false);
+    if (!success) {
       this.selectPanel("downloadFailed");
       return;
     }
@@ -384,7 +421,7 @@ appUpdater.prototype = {
    */
   setupDownloadingUI() {
     this.downloadStatus = document.getElementById("downloadStatus");
-    this.downloadStatus.value = DownloadUtils.getTransferTotal(
+    this.downloadStatus.textContent = DownloadUtils.getTransferTotal(
       0,
       this.update.selectedPatch.size
     );
@@ -450,7 +487,7 @@ appUpdater.prototype = {
    * See nsIProgressEventSink.idl
    */
   onProgress(aRequest, aProgress, aProgressMax) {
-    this.downloadStatus.value = DownloadUtils.getTransferTotal(
+    this.downloadStatus.textContent = DownloadUtils.getTransferTotal(
       aProgress,
       aProgressMax
     );
@@ -502,3 +539,15 @@ appUpdater.prototype = {
     "nsIRequestObserver",
   ]),
 };
+
+window.addEventListener("load", () => {
+  for (let link of document.querySelectorAll(".download-link")) {
+    link.addEventListener("click", event => {
+      event.preventDefault();
+      let messenger =
+        window.messenger ||
+        Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger);
+      messenger.launchExternalURL(event.target.href);
+    });
+  }
+});
