@@ -5,7 +5,6 @@
 
 /* This is where functions related to the standalone message window are kept */
 
-/* import-globals-from ../../../../toolkit/components/printing/content/printUtils.js */
 /* import-globals-from ../../../../toolkit/content/viewZoomOverlay.js */
 /* import-globals-from ../../../mailnews/base/content/junkCommands.js */
 /* import-globals-from ../../../mailnews/base/prefs/content/accountUtils.js */
@@ -20,30 +19,20 @@
 /* import-globals-from messageDisplay.js */
 /* import-globals-from msgHdrView.js */
 /* import-globals-from msgViewNavigation.js */
-/* import-globals-from phishingDetector.js */
 /* import-globals-from toolbarIconColor.js */
 
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
-);
-var { MailConstants } = ChromeUtils.import(
-  "resource:///modules/MailConstants.jsm"
-);
 var { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
-var { MsgHdrSyntheticView } = ChromeUtils.import(
-  "resource:///modules/MsgHdrSyntheticView.jsm"
-);
-var { BondOpenPGP } = ChromeUtils.import(
-  "chrome://openpgp/content/BondOpenPGP.jsm"
-);
 
 XPCOMUtils.defineLazyModuleGetters(this, {
+  AppConstants: "resource://gre/modules/AppConstants.jsm",
   CustomizableUI: "resource:///modules/CustomizableUI.jsm",
+  MailUtils: "resource:///modules/MailUtils.jsm",
+  MsgHdrSyntheticView: "resource:///modules/MsgHdrSyntheticView.jsm",
   ShortcutUtils: "resource://gre/modules/ShortcutUtils.jsm",
+  UIDensity: "resource:///modules/UIDensity.jsm",
+  UIFontSize: "resource:///modules/UIFontSize.jsm",
 });
 
 // from MailNewsTypes.h
@@ -58,6 +47,8 @@ var gFolderTreeView = null;
 var gFolderDisplay;
 var gMessageDisplay;
 
+this.__defineGetter__("browser", getBrowser);
+
 /**
  * We subclass FolderDisplayWidget:
  * - Because it assumes some thread-pane things that do not apply to us and we
@@ -274,16 +265,15 @@ StandaloneMessageDisplayWidget.prototype = {
     }
 
     document.title = docTitle;
+    if (AppConstants.platform == "macosx") {
+      document.getElementById("titlebar-title-label").value = docTitle;
+    }
 
     this.isDummy = aMsgHdr.folder == null;
     if (!this.isDummy) {
       this.displayedUri = null;
     }
 
-    // Initialize the appmenu.
-    PanelUI.init();
-    gExtensionsNotifications.init();
-
     // We've loaded a message, so this should be set to false
     this.aboutToLoadMessage = false;
   },
@@ -352,7 +342,7 @@ let messagepaneObserver = {
     event.preventDefault();
   },
 
-  onDragExit(event) {
+  onDragLeave(event) {
     let messagepanebox = document.getElementById("messagepanebox");
     messagepanebox.removeAttribute("dragover");
   },
@@ -397,19 +387,17 @@ function OnLoadMessageWindow() {
     "messagepane"
   );
 
+  updateTroubleshootMenuItem();
   ToolbarIconColor.init();
+  PanelUI.init();
+  gExtensionsNotifications.init();
+
   setTimeout(delayedOnLoadMessageWindow, 0); // when debugging, set this to 5000, so you can see what happens after the window comes up.
 
-  if (MailConstants.MOZ_OPENPGP && BondOpenPGP.isEnabled()) {
-    Enigmail.msg.messengerStartup.bind(Enigmail.msg);
-    Enigmail.msg.messengerStartup();
-    Enigmail.hdrView.hdrViewLoad.bind(Enigmail.hdrView);
-    Enigmail.hdrView.hdrViewLoad();
-  } else {
-    for (let item of document.querySelectorAll(".openpgp-item")) {
-      item.hidden = true;
-    }
-  }
+  Enigmail.msg.messengerStartup.bind(Enigmail.msg);
+  Enigmail.msg.messengerStartup();
+  Enigmail.hdrView.hdrViewLoad.bind(Enigmail.hdrView);
+  Enigmail.hdrView.hdrViewLoad();
 
   let messagePaneBrowser = document.getElementById("messagepane");
   messagePaneBrowser.addEventListener(
@@ -427,10 +415,8 @@ function OnLoadMessageWindow() {
     true
   );
 
-  ExtensionParent.apiManager.emit(
-    "extension-browser-inserted",
-    messagePaneBrowser
-  );
+  UIDensity.registerWindow(window);
+  UIFontSize.registerWindow(window);
 }
 
 function delayedOnLoadMessageWindow() {
@@ -438,7 +424,6 @@ function delayedOnLoadMessageWindow() {
   ShowMenus();
   MailOfflineMgr.init();
   CreateMailWindowGlobals();
-  verifyAccounts(null);
 
   /**
    * Create a message listener so that we can update the title once the message
@@ -454,6 +439,11 @@ function delayedOnLoadMessageWindow() {
     },
     onEndAttachments() {},
   });
+  // Run menubar initialization first, to avoid TabsInTitlebar code picking
+  // up mutations from it and causing a reflow.
+  if (AppConstants.platform != "macosx") {
+    AutoHideMenubar.init();
+  }
 
   InitMsgWindow();
 
@@ -461,8 +451,6 @@ function delayedOnLoadMessageWindow() {
   // FIX ME - later we will be able to use onload from the overlay
   OnLoadMsgHeaderPane();
 
-  gPhishingDetector.init();
-
   // initialize the customizeDone method on the customizeable toolbar
   var toolbox = document.getElementById("mail-toolbox");
   toolbox.customizeDone = function(aEvent) {
@@ -569,6 +557,10 @@ function actuallyLoadMessage() {
 
   gFolderDisplay.makeActive();
 
+  // Initialize the folder listener of the message header only after the
+  // gFolderDisplay has been created.
+  initFolderDBListener();
+
   // set focus to the message pane
   window.content.focus();
 }
@@ -938,8 +930,8 @@ function OnUnloadMessageWindow() {
   UnloadCommandUpdateHandlers();
   // FIX ME - later we will be able to use onunload from the overlay
   OnUnloadMsgHeaderPane();
-  gPhishingDetector.shutdown();
   ToolbarIconColor.uninit();
+  PanelUI.uninit();
   OnMailWindowUnload();
 }
 
@@ -1046,6 +1038,7 @@ var MessageWindowController = {
       case "button_forward":
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
+      case "cmd_redirect":
       case "cmd_editAsNew":
       case "cmd_editDraftMsg":
       case "cmd_newMsgFromTemplate":
@@ -1060,8 +1053,6 @@ var MessageWindowController = {
       case "button_getNewMessages":
       case "button_print":
       case "cmd_print":
-      case "cmd_printpreview":
-      case "cmd_printSetup":
       case "cmd_settingsOffline":
       case "cmd_createFilterFromPopup":
       case "cmd_createFilterFromMenu":
@@ -1070,11 +1061,16 @@ var MessageWindowController = {
       case "cmd_fullZoomEnlarge":
       case "cmd_fullZoomReset":
       case "cmd_fullZoomToggle":
+      case "cmd_fontSizeReset":
       case "cmd_viewAllHeader":
       case "cmd_viewNormalHeader":
       case "cmd_stop":
       case "cmd_chat":
         return true;
+      case "cmd_fontSizeReduce":
+        return UIFontSize.size > UIFontSize.MIN_VALUE;
+      case "cmd_fontSizeEnlarge":
+        return UIFontSize.size < UIFontSize.MAX_VALUE;
       case "cmd_synchronizeOffline":
       case "cmd_downloadFlagged":
       case "cmd_downloadSelected":
@@ -1124,10 +1120,18 @@ var MessageWindowController = {
       case "button_reply":
       case "cmd_replyall":
       case "button_replyall":
-        return gFolderDisplay.selectedMessage && IsReplyEnabled();
+        return (
+          gFolderDisplay.selectedMessage &&
+          IsReplyEnabled() &&
+          CanComposeMessages()
+        );
       case "cmd_replylist":
       case "button_replylist":
-        return gFolderDisplay.selectedMessage && IsReplyListEnabled();
+        return (
+          gFolderDisplay.selectedMessage &&
+          IsReplyListEnabled() &&
+          CanComposeMessages()
+        );
       case "cmd_newMessage":
         return CanComposeMessages();
       case "cmd_replySender":
@@ -1137,12 +1141,13 @@ var MessageWindowController = {
       case "button_forward":
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
+      case "cmd_redirect":
       case "cmd_editAsNew":
+        return CanComposeMessages();
       case "cmd_editDraftMsg":
       case "cmd_newMsgFromTemplate":
       case "cmd_editTemplateMsg":
       case "cmd_print":
-      case "cmd_printpreview":
       case "button_print":
       case "cmd_saveAsFile":
       case "cmd_saveAsTemplate":
@@ -1163,7 +1168,6 @@ var MessageWindowController = {
       case "cmd_tag8":
       case "cmd_tag9":
       case "button_mark":
-      case "cmd_markThreadAsRead":
       case "cmd_markReadByDate":
       case "cmd_viewAllHeader":
       case "cmd_viewNormalHeader":
@@ -1172,6 +1176,8 @@ var MessageWindowController = {
         return true;
       case "cmd_markAllRead":
         return false;
+      case "cmd_markThreadAsRead":
+        return gFolderDisplay.canMarkThreadAsRead;
       case "cmd_markAsRead":
         return CanMarkMsgAsRead(true);
       case "cmd_markAsUnread":
@@ -1179,8 +1185,6 @@ var MessageWindowController = {
       case "cmd_markAsFlagged":
       case "button_file":
         return gFolderDisplay.selectedMessage != null;
-      case "cmd_printSetup":
-        return true;
       case "cmd_getNewMessages":
       case "button_getNewMessages":
       case "cmd_getMsgsForAuthAccounts":
@@ -1211,6 +1215,11 @@ var MessageWindowController = {
       case "cmd_fullZoomEnlarge":
       case "cmd_fullZoomReset":
       case "cmd_fullZoomToggle":
+      case "cmd_fontSizeReduce":
+        return UIFontSize.size > UIFontSize.MIN_VALUE;
+      case "cmd_fontSizeEnlarge":
+        return UIFontSize.size < UIFontSize.MAX_VALUE;
+      case "cmd_fontSizeReset":
         return true;
       case "button_goForward":
       case "button_goBack":
@@ -1239,7 +1248,7 @@ var MessageWindowController = {
         ) {
           return false;
         }
-        let targetURI = Services.prefs.getCharPref(
+        let targetURI = Services.prefs.getStringPref(
           "mail.last_msg_movecopy_target_uri"
         );
         if (!targetURI) {
@@ -1312,6 +1321,9 @@ var MessageWindowController = {
       case "cmd_forwardAttachment":
         MsgForwardAsAttachment(null);
         break;
+      case "cmd_redirect":
+        MsgRedirectMessage(null);
+        break;
       case "cmd_editAsNew":
         MsgEditMessageAsNew(null);
         break;
@@ -1326,7 +1338,7 @@ var MessageWindowController = {
         break;
       case "cmd_moveToFolderAgain":
         var folder = MailUtils.getOrCreateFolder(
-          Services.prefs.getCharPref("mail.last_msg_movecopy_target_uri")
+          Services.prefs.getStringPref("mail.last_msg_movecopy_target_uri")
         );
         if (Services.prefs.getBoolPref("mail.last_msg_movecopy_was_move")) {
           MsgMoveMessage(folder);
@@ -1352,14 +1364,9 @@ var MessageWindowController = {
       case "button_junk":
         MsgJunk();
         break;
-      case "cmd_printSetup":
-        PrintUtils.showPageSetup();
-        break;
       case "cmd_print":
-        PrintEnginePrint();
-        break;
-      case "cmd_printpreview":
-        PrintEnginePrintPreview();
+        let messagePaneBrowser = document.getElementById("messagepane");
+        PrintUtils.startPrintWindow(messagePaneBrowser.browsingContext, {});
         break;
       case "cmd_saveAsFile":
         MsgSaveAsFile();
@@ -1508,6 +1515,15 @@ var MessageWindowController = {
       case "cmd_fullZoomToggle":
         ZoomManager.toggleZoom();
         break;
+      case "cmd_fontSizeReduce":
+        UIFontSize.reduceSize();
+        break;
+      case "cmd_fontSizeReset":
+        UIFontSize.resetSize();
+        break;
+      case "cmd_fontSizeEnlarge":
+        UIFontSize.increaseSize();
+        break;
       case "cmd_stop":
         msgWindow.StopUrls();
         break;
