@@ -8,6 +8,7 @@
 /* import-globals-from mailWindow.js */
 /* import-globals-from messageDisplay.js */
 /* import-globals-from utilityOverlay.js */
+/* global EnigmailURIs: false, gEncryptedURIService: true */
 
 var { InlineSpellChecker, SpellCheckHelper } = ChromeUtils.import(
   "resource://gre/modules/InlineSpellChecker.jsm"
@@ -22,94 +23,136 @@ var { AppConstants } = ChromeUtils.import(
 var { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
-var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
-
-XPCOMUtils.defineLazyGetter(this, "PageMenuParent", function() {
-  let { PageMenuParent } = ChromeUtils.import(
-    "resource://gre/modules/PageMenu.jsm"
-  );
-  return new PageMenuParent();
-});
+ChromeUtils.defineModuleGetter(
+  this,
+  "MailUtils",
+  "resource:///modules/MailUtils.jsm"
+);
+var { E10SUtils } = ChromeUtils.import("resource://gre/modules/E10SUtils.jsm");
 
 var gSpellChecker = new InlineSpellChecker();
 
-function nsContextMenu(aXulMenu, aIsShift) {
-  this.target = null;
-  this.menu = null;
-  this.onTextInput = false;
-  this.onEditable = false;
-  this.onImage = false;
-  this.onLoadedImage = false;
-  this.onCanvas = false;
-  this.onVideo = false;
-  this.onAudio = false;
-  this.onPlayableMedia = false;
-  this.onLink = false;
-  this.onMailtoLink = false;
-  this.onSaveableLink = false;
-  this.onMetaDataItem = false;
-  this.onMathML = false;
-  this.link = false;
-  this.linkURL = "";
-  this.linkURI = null;
-  this.linkProtocol = null;
-  this.mediaURL = "";
-  this.isContentSelected = false;
-  this.shouldDisplay = true;
+/** Called by ContextMenuParent.jsm */
+function openContextMenu({ data }, browser, actor) {
+  if (!browser.hasAttribute("context")) {
+    return;
+  }
 
-  // Message Related Items
-  this.inAMessage = false;
-  this.inThreadPane = false;
-  this.inStandaloneWindow = false;
-  this.numSelectedMessages = 0;
-  this.isNewsgroup = false;
-  this.hideMailItems = false;
+  let spellInfo = data.spellInfo;
+  let frameReferrerInfo = data.frameReferrerInfo;
+  let linkReferrerInfo = data.linkReferrerInfo;
+  let principal = data.principal;
+  let storagePrincipal = data.storagePrincipal;
 
-  // Browser related items
-  this.inABrowser = false;
-  this.inABlankBrowser = false;
+  let documentURIObject = makeURI(
+    data.docLocation,
+    data.charSet,
+    makeURI(data.baseURI)
+  );
 
-  this.initMenu(aXulMenu, aIsShift);
+  if (frameReferrerInfo) {
+    frameReferrerInfo = E10SUtils.deserializeReferrerInfo(frameReferrerInfo);
+  }
+
+  if (linkReferrerInfo) {
+    linkReferrerInfo = E10SUtils.deserializeReferrerInfo(linkReferrerInfo);
+  }
+
+  nsContextMenu.contentData = {
+    context: data.context,
+    browser,
+    actor,
+    editFlags: data.editFlags,
+    spellInfo,
+    principal,
+    storagePrincipal,
+    documentURIObject,
+    docLocation: data.docLocation,
+    charSet: data.charSet,
+    referrerInfo: E10SUtils.deserializeReferrerInfo(data.referrerInfo),
+    frameReferrerInfo,
+    linkReferrerInfo,
+    contentType: data.contentType,
+    contentDisposition: data.contentDisposition,
+    frameID: data.frameID,
+    frameOuterWindowID: data.frameID,
+    frameBrowsingContext: BrowsingContext.get(data.frameBrowsingContextID),
+    selectionInfo: data.selectionInfo,
+    disableSetDesktopBackground: data.disableSetDesktopBackground,
+    loginFillInfo: data.loginFillInfo,
+    parentAllowsMixedContent: data.parentAllowsMixedContent,
+    userContextId: data.userContextId,
+    webExtContextData: data.webExtContextData,
+  };
+
+  let popup = browser.ownerDocument.getElementById(
+    browser.getAttribute("context")
+  );
+  let context = nsContextMenu.contentData.context;
+
+  // We don't have access to the original event here, as that happened in
+  // another process. Therefore we synthesize a new MouseEvent to propagate the
+  // inputSource to the subsequently triggered popupshowing event.
+  let newEvent = document.createEvent("MouseEvent");
+  let screenX = context.screenXDevPx / window.devicePixelRatio;
+  let screenY = context.screenYDevPx / window.devicePixelRatio;
+  newEvent.initNSMouseEvent(
+    "contextmenu",
+    true,
+    true,
+    null,
+    0,
+    screenX,
+    screenY,
+    0,
+    0,
+    false,
+    false,
+    false,
+    false,
+    0,
+    null,
+    0,
+    context.mozInputSource
+  );
+  popup.openPopupAtScreen(newEvent.screenX, newEvent.screenY, true, newEvent);
 }
 
-nsContextMenu.prototype = {
-  /**
-   * Init: set properties based on the clicked-on element and the state of
-   * the world, then determine which context menu items to show based on
-   * those properties.
-   */
-  initMenu(aPopup, aIsShift) {
-    this.menu = aPopup;
-
-    let target = this.menu.target || document.popupNode;
-    delete this.menu.target;
+/** Called by a popupshowing event via fillMailContextMenu x3. */
+class nsContextMenu {
+  constructor(aXulMenu, aIsShift) {
+    this.xulMenu = aXulMenu;
 
     // Get contextual info.
-    this.setTarget(target);
-    this.setMessageTargets(target);
+    this.setContext();
 
-    if (this.inABlankBrowser) {
-      this.shouldDisplay = false;
+    if (!this.shouldDisplay) {
       return;
     }
 
-    this.selectionInfo = BrowserUtils.getSelectionDetails(window);
-    this.isContentSelected = !this.selectionInfo.docSelectionIsCollapsed;
-    this.textSelected = this.selectionInfo.text;
-    this.isTextSelected = !!this.textSelected.length;
+    // Message Related Items
+    this.inAMessage = false;
+    this.inThreadPane = false;
+    this.inStandaloneWindow = false;
+    this.numSelectedMessages = 0;
+    this.isNewsgroup = false;
+    this.hideMailItems = false;
+
+    this.isContentSelected =
+      !this.selectionInfo || !this.selectionInfo.docSelectionIsCollapsed;
+
+    this.setMessageTargets();
 
-    this.hasPageMenu = false;
     if (!aIsShift) {
-      let menuObject = PageMenuParent.maybeBuild(this.target);
-      this.hasPageMenu = PageMenuParent.addToPopup(menuObject, null, aPopup);
-
       // The rest of this block sends menu information to WebExtensions.
       let subject = {
-        menu: aPopup,
+        menu: aXulMenu,
         tab: document.getElementById("tabmail")
           ? document.getElementById("tabmail").currentTabInfo
           : window,
+        timeStamp: this.timeStamp,
         isContentSelected: this.isContentSelected,
+        inFrame: this.inFrame,
         isTextSelected: this.isTextSelected,
         onTextInput: this.onTextInput,
         onLink: this.onLink,
@@ -118,34 +161,36 @@ nsContextMenu.prototype = {
         onAudio: this.onAudio,
         onCanvas: this.onCanvas,
         onEditable: this.onEditable,
+        onSpellcheckable: this.onSpellcheckable,
+        onPassword: this.onPassword,
         srcUrl: this.mediaURL,
-        linkText: this.onLink ? this.linkText() : undefined,
+        frameUrl: this.contentData ? this.contentData.docLocation : undefined,
+        pageUrl: this.browser ? this.browser.currentURI.spec : undefined,
+        linkText: this.linkTextStr,
         linkUrl: this.linkURL,
         selectionText: this.isTextSelected
           ? this.selectionInfo.fullText
           : undefined,
+        frameId: this.frameID,
+        webExtBrowserType: this.webExtBrowserType,
+        webExtContextData: this.contentData
+          ? this.contentData.webExtContextData
+          : undefined,
       };
-      if (this.target) {
-        subject.inFrame =
-          this.target.ownerGlobal != this.target.ownerGlobal.top;
-        subject.frameUrl = this.target.ownerGlobal.location.href;
-        subject.pageUrl = this.target.ownerGlobal.top.location.href;
-        subject.principal = this.target.ownerDocument.nodePrincipal;
-      }
-      if (
-        window.gFolderDisplay?.tree &&
-        target.closest("tree") == gFolderDisplay.tree
-      ) {
+
+      if (this.inThreadPane) {
         subject.displayedFolder = gFolderDisplay.view.displayedFolder;
         subject.selectedMessages = gFolderDisplay.selectedMessages;
       }
-      subject.context = subject;
-      subject.wrappedJSObject = subject;
 
-      Services.obs.notifyObservers(subject, "on-prepare-contextmenu");
+      subject.wrappedJSObject = subject;
       Services.obs.notifyObservers(subject, "on-build-contextmenu");
     }
 
+    // Reset after "on-build-contextmenu" notification in case selection was
+    // changed during the notification.
+    this.isContentSelected =
+      !this.selectionInfo || !this.selectionInfo.docSelectionIsCollapsed;
     this.initItems();
 
     // If all items in the menu are hidden, set this.shouldDisplay to false
@@ -156,11 +201,139 @@ nsContextMenu.prototype = {
         return;
       }
     }
+
     // All items must have been hidden.
     this.shouldDisplay = false;
-  },
+  }
+
+  setContext() {
+    let context = Object.create(null);
+
+    if (nsContextMenu.contentData) {
+      this.contentData = nsContextMenu.contentData;
+      context = this.contentData.context;
+      nsContextMenu.contentData = null;
+    }
+
+    this.shouldDisplay = !this.contentData || context.shouldDisplay;
+    this.timeStamp = context.timeStamp;
+
+    // Assign what's _possibly_ needed from `context` sent by ContextMenuChild.jsm
+    // Keep this consistent with the similar code in ContextMenu's _setContext
+    this.bgImageURL = context.bgImageURL;
+    this.imageDescURL = context.imageDescURL;
+    this.imageInfo = context.imageInfo;
+    this.mediaURL = context.mediaURL;
+
+    this.canSpellCheck = context.canSpellCheck;
+    this.hasBGImage = context.hasBGImage;
+    this.hasMultipleBGImages = context.hasMultipleBGImages;
+    this.isDesignMode = context.isDesignMode;
+    this.inFrame = context.inFrame;
+    this.inPDFViewer = context.inPDFViewer;
+    this.inSrcdocFrame = context.inSrcdocFrame;
+    this.inSyntheticDoc = context.inSyntheticDoc;
+
+    this.link = context.link;
+    this.linkDownload = context.linkDownload;
+    this.linkProtocol = context.linkProtocol;
+    this.linkTextStr = context.linkTextStr;
+    this.linkURL = context.linkURL;
+    this.linkURI = this.getLinkURI(); // can't send; regenerate
+
+    this.onAudio = context.onAudio;
+    this.onCanvas = context.onCanvas;
+    this.onCompletedImage = context.onCompletedImage;
+    this.onDRMMedia = context.onDRMMedia;
+    this.onPiPVideo = context.onPiPVideo;
+    this.onEditable = context.onEditable;
+    this.onImage = context.onImage;
+    this.onKeywordField = context.onKeywordField;
+    this.onLink = context.onLink;
+    this.onLoadedImage = context.onLoadedImage;
+    this.onMailtoLink = context.onMailtoLink;
+    this.onMozExtLink = context.onMozExtLink;
+    this.onNumeric = context.onNumeric;
+    this.onPassword = context.onPassword;
+    this.onSaveableLink = context.onSaveableLink;
+    this.onSpellcheckable = context.onSpellcheckable;
+    this.onTextInput = context.onTextInput;
+    this.onVideo = context.onVideo;
+
+    this.target = context.target;
+    this.targetIdentifier = context.targetIdentifier;
+
+    this.principal = context.principal;
+    this.storagePrincipal = context.storagePrincipal;
+    this.frameID = context.frameID;
+    this.frameOuterWindowID = context.frameOuterWindowID;
+    this.frameBrowsingContext = BrowsingContext.get(
+      context.frameBrowsingContextID
+    );
+
+    this.inSyntheticDoc = context.inSyntheticDoc;
+    this.inAboutDevtoolsToolbox = context.inAboutDevtoolsToolbox;
+
+    // Everything after this isn't sent directly from ContextMenu
+    if (this.target) {
+      this.ownerDoc = this.target.ownerDocument;
+    }
+
+    this.csp = E10SUtils.deserializeCSP(context.csp);
+
+    if (!this.contentData) {
+      return;
+    }
+
+    this.browser = this.contentData.browser;
+    if (this.browser && this.browser.currentURI.spec == "about:blank") {
+      this.shouldDisplay = false;
+      return;
+    }
+    this.selectionInfo = this.contentData.selectionInfo;
+    this.actor = this.contentData.actor;
+
+    this.textSelected = this.selectionInfo?.text;
+    this.isTextSelected = !!this.textSelected?.length;
+
+    this.webExtBrowserType = this.browser.getAttribute(
+      "webextension-view-type"
+    );
+
+    if (context.shouldInitInlineSpellCheckerUINoChildren) {
+      gSpellChecker.initFromRemote(
+        this.contentData.spellInfo,
+        this.actor.manager
+      );
+    }
+
+    if (this.contentData.spellInfo) {
+      this.spellSuggestions = this.contentData.spellInfo.spellSuggestions;
+    }
+
+    if (context.shouldInitInlineSpellCheckerUIWithChildren) {
+      gSpellChecker.initFromRemote(
+        this.contentData.spellInfo,
+        this.actor.manager
+      );
+      let canSpell = gSpellChecker.canSpellCheck && this.canSpellCheck;
+      this.showItem("mailContext-spell-check-enabled", canSpell);
+      this.showItem("mailContext-spell-separator", canSpell);
+    }
+  }
+
+  hiding() {
+    if (this.actor) {
+      this.actor.hiding();
+    }
+
+    this.contentData = null;
+    gSpellChecker.clearSuggestionsFromMenu();
+    gSpellChecker.clearDictionaryListFromMenu();
+    gSpellChecker.uninit();
+  }
+
   initItems() {
-    this.initPageMenuSeparator();
     this.initSaveItems();
     this.initClipboardItems();
     this.initMediaPlayerItems();
@@ -168,28 +341,32 @@ nsContextMenu.prototype = {
     this.initMessageItems();
     this.initSpellingItems();
     this.initSeparators();
-  },
+  }
   addDictionaries() {
     openDictionaryList();
-  },
-  initPageMenuSeparator() {
-    this.showItem("page-menu-separator", this.hasPageMenu);
-  },
+  }
   initSpellingItems() {
-    let canSpell = gSpellChecker.canSpellCheck;
+    let canSpell =
+      gSpellChecker.canSpellCheck &&
+      !gSpellChecker.initialSpellCheckPending &&
+      this.canSpellCheck;
+    let showDictionaries = canSpell && gSpellChecker.enabled;
     let onMisspelling = gSpellChecker.overMisspelling;
+    let showUndo = canSpell && gSpellChecker.canUndo();
     this.showItem("mailContext-spell-check-enabled", canSpell);
-    this.showItem("mailContext-spell-separator", canSpell || this.onEditable);
-    if (canSpell) {
-      document
-        .getElementById("mailContext-spell-check-enabled")
-        .setAttribute("checked", gSpellChecker.enabled);
-    }
+    this.showItem("mailContext-spell-separator", canSpell);
+    document
+      .getElementById("mailContext-spell-check-enabled")
+      .setAttribute("checked", canSpell && gSpellChecker.enabled);
 
     this.showItem("mailContext-spell-add-to-dictionary", onMisspelling);
+    this.showItem("mailContext-spell-undo-add-to-dictionary", showUndo);
 
     // suggestion list
-    this.showItem("mailContext-spell-suggestions-separator", onMisspelling);
+    this.showItem(
+      "mailContext-spell-suggestions-separator",
+      onMisspelling || showUndo
+    );
     if (onMisspelling) {
       let addMenuItem = document.getElementById(
         "mailContext-spell-add-to-dictionary"
@@ -197,7 +374,7 @@ nsContextMenu.prototype = {
       let suggestionCount = gSpellChecker.addSuggestionsToMenu(
         addMenuItem.parentNode,
         addMenuItem,
-        5
+        this.spellSuggestions
       );
       this.showItem("mailContext-spell-no-suggestions", suggestionCount == 0);
     } else {
@@ -205,7 +382,7 @@ nsContextMenu.prototype = {
     }
 
     // dictionary list
-    this.showItem("mailContext-spell-dictionaries", gSpellChecker.enabled);
+    this.showItem("mailContext-spell-dictionaries", showDictionaries);
     if (canSpell) {
       let dictMenu = document.getElementById(
         "mailContext-spell-dictionaries-menu"
@@ -213,21 +390,26 @@ nsContextMenu.prototype = {
       let dictSep = document.getElementById(
         "mailContext-spell-language-separator"
       );
-      gSpellChecker.addDictionaryListToMenu(dictMenu, dictSep);
+      let count = gSpellChecker.addDictionaryListToMenu(dictMenu, dictSep);
+      this.showItem(dictSep, count > 0);
       this.showItem("mailContext-spell-add-dictionaries-main", false);
-    } else if (this.onEditable) {
+    } else if (this.onSpellcheckable) {
       // when there is no spellchecker but we might be able to spellcheck
       // add the add to dictionaries item. This will ensure that people
       // with no dictionaries will be able to download them
-      this.showItem("mailContext-spell-add-dictionaries-main", true);
+      this.showItem("mailContext-spell-language-separator", showDictionaries);
+      this.showItem(
+        "mailContext-spell-add-dictionaries-main",
+        showDictionaries
+      );
     } else {
       this.showItem("mailContext-spell-add-dictionaries-main", false);
     }
-  },
+  }
   initSaveItems() {
     this.showItem("mailContext-savelink", this.onSaveableLink);
     this.showItem("mailContext-saveimage", this.onLoadedImage);
-  },
+  }
   initClipboardItems() {
     // Copy depends on whether there is selected text.
     // Enabling this context menu item is now done through the global
@@ -290,7 +472,7 @@ nsContextMenu.prototype = {
       ]);
       searchTheWeb.value = selection;
     }
-  },
+  }
   initMediaPlayerItems() {
     let onMedia = this.onVideo || this.onAudio;
     // Several mutually exclusive items.... play/pause, mute/unmute, show/hide
@@ -307,7 +489,7 @@ nsContextMenu.prototype = {
       this.setItemAttr("mailContext-media-mute", "disabled", hasError);
       this.setItemAttr("mailContext-media-unmute", "disabled", hasError);
     }
-  },
+  }
   initBrowserItems() {
     // Work out if we are a context menu on a special item e.g. an image, link
     // etc.
@@ -317,7 +499,8 @@ nsContextMenu.prototype = {
       this.onCanvas ||
       this.onLink ||
       this.onImage ||
-      this.onPlayableMedia ||
+      this.onAudio ||
+      this.onVideo ||
       this.onTextInput
     );
     // Ensure these commands are updated with their current status.
@@ -331,8 +514,8 @@ nsContextMenu.prototype = {
     this.showItem("mailContext-reload", notOnSpecialItem);
 
     let loadedProtocol = "";
-    if (this.target && this.target.ownerGlobal.top.location) {
-      loadedProtocol = this.target.ownerGlobal.top.location.protocol;
+    if (this.target && this.target.ownerGlobal?.top.location) {
+      loadedProtocol = this.target.ownerGlobal?.top.location.protocol;
     }
 
     // Only show open in browser if we're not on a special item and we're not
@@ -350,7 +533,7 @@ nsContextMenu.prototype = {
       "mailContext-openLinkInBrowser",
       this.onLink && ["http", "https"].includes(this.linkProtocol)
     );
-  },
+  }
   /* eslint-disable complexity */
   initMessageItems() {
     // If we're not in a message related tab, we're just going to bulk hide most
@@ -369,6 +552,7 @@ nsContextMenu.prototype = {
         "mailContext-forward",
         "mailContext-forwardAsMenu",
         "mailContext-multiForwardAsAttachment",
+        "mailContext-redirect",
         "mailContext-editAsNew",
         "mailContext-editDraftMsg",
         "mailContext-newMsgFromTemplate",
@@ -377,13 +561,13 @@ nsContextMenu.prototype = {
         "mailContext-moveMenu",
         "mailContext-copyMenu",
         "mailContext-moveToFolderAgain",
+        "mailContext-decryptToFolder",
         "mailContext-ignoreThread",
         "mailContext-ignoreSubthread",
         "mailContext-watchThread",
         "mailContext-tags",
         "mailContext-mark",
         "mailContext-saveAs",
-        "mailContext-printpreview",
         "mailContext-print",
         "mailContext-delete",
         "downloadSelected",
@@ -413,11 +597,13 @@ nsContextMenu.prototype = {
       "mailContext-openConversation",
       this.numSelectedMessages == 1 &&
         this.inThreadPane &&
-        gConversationOpener.isSelectedMessageIndexed()
+        ConversationOpener.isMessageIndexed(gFolderDisplay.selectedMessage)
     );
     this.showItem(
       "mailContext-openContainingFolder",
-      !gFolderDisplay.folderPaneVisible && this.numSelectedMessages == 1
+      !gFolderDisplay.folderPaneVisible &&
+        this.numSelectedMessages == 1 &&
+        !gMessageDisplay.isDummy
     );
 
     this.setSingleSelection("mailContext-replySender");
@@ -426,22 +612,19 @@ nsContextMenu.prototype = {
     this.setSingleSelection("mailContext-replyList");
     this.setSingleSelection("mailContext-forward");
     this.setSingleSelection("mailContext-forwardAsMenu");
+    this.setSingleSelection("mailContext-redirect");
     this.setSingleSelection("mailContext-editAsNew");
     this.setSingleSelection(
       "mailContext-editDraftMsg",
-      document.getElementById("cmd_editDraftMsg").getAttribute("hidden") !=
-        "true"
+      !document.getElementById("cmd_editDraftMsg").hidden
     );
     this.setSingleSelection(
       "mailContext-newMsgFromTemplate",
-      document
-        .getElementById("cmd_newMsgFromTemplate")
-        .getAttribute("hidden") != "true"
+      !document.getElementById("cmd_newMsgFromTemplate").hidden
     );
     this.setSingleSelection(
       "mailContext-editTemplateMsg",
-      document.getElementById("cmd_editTemplateMsg").getAttribute("hidden") !=
-        "true"
+      !document.getElementById("cmd_editTemplateMsg").hidden
     );
 
     this.showItem(
@@ -483,6 +666,15 @@ nsContextMenu.prototype = {
       goUpdateCommand("cmd_moveToFolderAgain");
     }
 
+    let showDecrypt = this.numSelectedMessages > 1;
+    if (this.numSelectedMessages == 1) {
+      let msgURI = gFolderDisplay.selectedMessageUris[0];
+      showDecrypt =
+        EnigmailURIs.isEncryptedUri(msgURI) ||
+        gEncryptedURIService.isEncrypted(msgURI);
+    }
+    this.showItem("mailContext-decryptToFolder", showDecrypt);
+
     this.showItem("mailContext-tags", msgModifyItems);
 
     this.showItem("mailContext-mark", msgModifyItems);
@@ -521,12 +713,6 @@ nsContextMenu.prototype = {
         !this.onPlayableMedia
     );
 
-    if (AppConstants.platform == "macosx") {
-      this.showItem("mailContext-printpreview", false);
-    } else {
-      this.setSingleSelection("mailContext-printpreview");
-    }
-
     // XXX Not quite modifying the message, but the same rules apply at the
     // moment as we can't print non-message content from the message pane yet.
     this.showItem("mailContext-print", msgModifyItems);
@@ -553,217 +739,36 @@ nsContextMenu.prototype = {
     );
 
     this.setSingleSelection("mailContext-calendar-convert-menu");
-  },
-  /* eslint-enable complexity */
+  }
   initSeparators() {
-    const mailContextSeparators = [
-      "mailContext-sep-open-browser",
-      "mailContext-sep-link",
-      "mailContext-sep-open",
-      "mailContext-sep-open2",
-      "mailContext-sep-reply",
-      "paneContext-afterMove",
-      "mailContext-sep-afterTagAddNew",
-      "mailContext-sep-afterTagRemoveAll",
-      "mailContext-sep-afterMarkAllRead",
-      "mailContext-sep-afterMarkFlagged",
-      "mailContext-sep-afterMarkMenu",
-      "mailContext-afterWatchThread",
-      "mailContext-sep-edit",
-      "mailContext-sep-editTemplate",
-      "mailContext-sep-copy",
-      "mailContext-sep-reportPhishing",
-      "mailContext-sep-undo",
-      "mailContext-sep-clipboard",
-      "mailContext-spell-suggestions-separator",
-      "mailContext-spell-separator",
-    ];
-    mailContextSeparators.forEach(this.hideIfAppropriate, this);
-
-    this.checkLastSeparator(this.menu);
-  },
-
-  /* eslint-disable complexity */
-  /**
-   * Set the nsContextMenu properties based on the selected node and
-   * its ancestors.
-   */
-  setTarget(aNode) {
-    // Clear any old spellchecking items from the menu, this used to
-    // be in the menu hiding code but wasn't getting called in all
-    // situations. Here, we can ensure it gets cleaned up any time the
-    // menu is shown. Note: must be before uninit because that clears the
-    // internal vars
-    // We also need to do that before we possibly bail because we just clicked
-    // on some XUL node. Otherwise, dictionary choices just accumulate until we
-    // right-click on some HTML element again.
-    gSpellChecker.clearSuggestionsFromMenu();
-    gSpellChecker.clearDictionaryListFromMenu();
-    gSpellChecker.uninit();
-
-    if (
-      aNode.namespaceURI ==
-      "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"
-    ) {
-      if (aNode.localName == "treecol") {
-        // The column header was clicked, show the column picker.
-        let treecols = aNode.parentNode;
-        let treeColPicker = treecols.querySelector("treecolpicker");
-        let popup = treeColPicker.querySelector(`menupopup[anonid="popup"]`);
-        treeColPicker.buildPopup(popup);
-        popup.openPopup(aNode, "before_start", 0, 0, true);
-        this.shouldDisplay = false;
-      }
-      return;
-    }
-    this.onImage = false;
-    this.onLoadedImage = false;
-    this.onMetaDataItem = false;
-    this.onTextInput = false;
-    this.onEditable = false;
-    this.imageURL = "";
-    this.onLink = false;
-    this.onVideo = false;
-    this.onAudio = false;
-    this.mediaURL = "";
-    this.linkURL = "";
-    this.linkURI = null;
-    this.linkProtocol = null;
-    this.onMathML = false;
-
-    this.target = aNode;
-
-    // Set up early the right flags for editable / not editable.
-    let editFlags = SpellCheckHelper.isEditable(this.target, window);
-    this.onTextInput = (editFlags & SpellCheckHelper.TEXTINPUT) !== 0;
-    this.onEditable = (editFlags & SpellCheckHelper.EDITABLE) !== 0;
-
-    // First, do checks for nodes that never have children.
-    if (this.target.nodeType == Node.ELEMENT_NODE) {
+    let separators = Array.from(
+      this.xulMenu.querySelectorAll(":scope > menuseparator")
+    );
+    let lastShownSeparator = null;
+    for (let separator of separators) {
+      let shouldShow = this.shouldShowSeparator(separator);
       if (
-        this.target instanceof Ci.nsIImageLoadingContent &&
-        this.target.currentURI
+        !shouldShow &&
+        lastShownSeparator &&
+        separator.classList.contains("webextension-group-separator")
       ) {
-        this.onImage = true;
-        this.onMetaDataItem = true;
-
-        var request = this.target.getRequest(
-          Ci.nsIImageLoadingContent.CURRENT_REQUEST
-        );
-        if (request && request.imageStatus & request.STATUS_SIZE_AVAILABLE) {
-          this.onLoadedImage = true;
-        }
-
-        this.imageURL = this.target.currentURI.spec;
-      } else if (this.target instanceof HTMLInputElement) {
-        this.onTextInput = this.isTargetATextBox(this.target);
-      } else if (
-        editFlags &
-        (SpellCheckHelper.INPUT | SpellCheckHelper.TEXTAREA)
-      ) {
-        if (!this.target.readOnly) {
-          this.onEditable = true;
-          gSpellChecker.init(this.target.editor);
-          gSpellChecker.initFromEvent(
-            document.popupRangeParent,
-            document.popupRangeOffset
-          );
-        }
-      } else if (editFlags & SpellCheckHelper.CONTENTEDITABLE) {
-        let targetWin = this.target.ownerGlobal;
-        let editingSession = targetWin
-          .getInterface(Ci.nsIWebNavigation)
-          .QueryInterface(Ci.nsIInterfaceRequestor)
-          .getInterface(Ci.nsIEditingSession);
-        gSpellChecker.init(editingSession.getEditorForWindow(targetWin));
-        gSpellChecker.initFromEvent(
-          document.popupRangeParent,
-          document.popupRangeOffset
-        );
-      } else if (this.target instanceof HTMLCanvasElement) {
-        this.onCanvas = true;
-      } else if (this.target instanceof HTMLVideoElement) {
-        this.onVideo = true;
-        this.onPlayableMedia = true;
-        this.mediaURL = this.target.currentSrc || this.target.src;
-      } else if (this.target instanceof HTMLAudioElement) {
-        this.onAudio = true;
-        this.onPlayableMedia = true;
-        this.mediaURL = this.target.currentSrc || this.target.src;
-        // Browser supports background images here but we don't need to.
+        // The separator for the WebExtension elements group must be shown, hide
+        // the last shown menu separator instead.
+        lastShownSeparator.hidden = true;
+        shouldShow = true;
       }
-    }
-
-    // Second, bubble out, looking for items of interest that might be
-    // parents of the click target, picking the innermost of each.
-    const XMLNS = "http://www.w3.org/XML/1998/namespace";
-    var elem = this.target;
-    while (elem) {
-      if (elem.nodeType == Node.ELEMENT_NODE) {
-        // Link?
-        if (
-          !this.onLink &&
-          ((elem instanceof HTMLAnchorElement && elem.href) ||
-            (elem instanceof HTMLAreaElement && elem.href) ||
-            elem instanceof HTMLLinkElement ||
-            elem.getAttributeNS("http://www.w3.org/1999/xlink", "type") ==
-              "simple")
-        ) {
-          // Target is a link or a descendant of a link.
-          this.onLink = true;
-          this.onMetaDataItem = true;
-          // Remember corresponding element.
-          this.link = elem;
-          this.linkURL = this.getLinkURL();
-          this.linkURI = this.getLinkURI();
-          this.linkProtocol = this.getLinkProtocol();
-          this.onMailtoLink = this.linkProtocol == "mailto";
-          this.onSaveableLink = this.isLinkSaveable();
-        }
-
-        // Text input?
-        if (!this.onTextInput) {
-          this.onTextInput = this.isTargetATextBox(elem);
-        }
-
-        // Metadata item?
-        if (!this.onMetaDataItem) {
-          if (
-            (elem instanceof HTMLQuoteElement && elem.cite) ||
-            (elem instanceof HTMLTableElement && elem.summary) ||
-            (elem instanceof HTMLModElement && (elem.cite || elem.dateTime)) ||
-            (elem instanceof HTMLElement && (elem.title || elem.lang)) ||
-            elem.getAttributeNS(XMLNS, "lang")
-          ) {
-            this.onMetaDataItem = true;
-          }
-        }
-
-        // Browser supports background images here but we don't need to.
+      if (shouldShow) {
+        lastShownSeparator = separator;
       }
-      elem = elem.parentNode;
+      separator.hidden = !shouldShow;
     }
+    this.checkLastSeparator(this.xulMenu);
+  }
 
-    // See if the user clicked on MathML.
-    const NS_MathML = "http://www.w3.org/1998/Math/MathML";
-    if (
-      (this.target.nodeType == Node.TEXT_NODE &&
-        this.target.parentNode.namespaceURI == NS_MathML) ||
-      this.target.namespaceURI == NS_MathML
-    ) {
-      this.onMathML = true;
-    }
-  },
-  /* eslint-enable complexity */
-
-  setMessageTargets(aNode) {
-    this.inABrowser = aNode.ownerDocument != document;
-    this.inABlankBrowser =
-      this.inABrowser && aNode.ownerGlobal.location.href == "about:blank";
-
-    if (this.inABrowser) {
-      this.inAMessage = ["imap:", "mailbox:", "news:", "snews:"].includes(
-        aNode.ownerGlobal.location.protocol
+  setMessageTargets() {
+    if (this.browser) {
+      this.inAMessage = ["imap", "mailbox", "news", "snews"].includes(
+        this.browser.currentURI.scheme
       );
       this.inThreadPane = false;
       if (!this.inAMessage) {
@@ -774,8 +779,7 @@ nsContextMenu.prototype = {
         return;
       }
     } else {
-      let threadTree = document.getElementById("threadTree");
-      this.inThreadPane = threadTree && threadTree.contains(aNode);
+      this.inThreadPane = true;
     }
 
     this.inAMessage = true;
@@ -784,7 +788,7 @@ nsContextMenu.prototype = {
     this.isNewsgroup = gFolderDisplay.selectedMessageIsNews;
     // Don't show mail items for links/images, just show related items.
     this.hideMailItems = !this.inThreadPane && (this.onImage || this.onLink);
-  },
+  }
 
   /**
    * Get a computed style property for an element.
@@ -796,7 +800,7 @@ nsContextMenu.prototype = {
    */
   getComputedStyle(aElem, aProp) {
     return aElem.ownerGlobal.getComputedStyle(aElem).getPropertyValue(aProp);
-  },
+  }
 
   /**
    * Determine whether the clicked-on link can be saved, and whether it
@@ -806,11 +810,10 @@ nsContextMenu.prototype = {
    */
   isLinkSaveable() {
     try {
-      const nsIScriptSecurityManager = Ci.nsIScriptSecurityManager;
       Services.scriptSecurityManager.checkLoadURIWithPrincipal(
         this.target.nodePrincipal,
         this.linkURI,
-        nsIScriptSecurityManager.STANDARD
+        Ci.nsIScriptSecurityManager.STANDARD
       );
     } catch (e) {
       // Don't save things we can't link to.
@@ -828,21 +831,41 @@ nsContextMenu.prototype = {
         this.linkProtocol == "snews"
       )
     );
-  },
+  }
 
   /**
    * Save URL of clicked-on link.
    */
   saveLink() {
-    saveURL(this.linkURL, this.linkText(), null, true, null, null, document);
-  },
+    saveURL(
+      this.linkURL,
+      null,
+      this.linkTextStr,
+      null,
+      true,
+      null,
+      null,
+      null,
+      document
+    );
+  }
 
   /**
    * Save a clicked-on image.
    */
   saveImage() {
-    saveURL(this.imageURL, null, "SaveImageTitle", false, null, null, document);
-  },
+    saveURL(
+      this.imageInfo.currentSrc,
+      null,
+      null,
+      "SaveImageTitle",
+      false,
+      null,
+      null,
+      null,
+      document
+    );
+  }
 
   /**
    * Extract email addresses from a mailto: link and put them on the
@@ -876,7 +899,7 @@ nsContextMenu.prototype = {
       Ci.nsIClipboardHelper
     );
     clipboard.copyString(addresses);
-  },
+  }
 
   // ---------
   // Utilities
@@ -897,7 +920,7 @@ nsContextMenu.prototype = {
     if (item) {
       item.hidden = !aShow;
     }
-  },
+  }
 
   /**
    * Set a DOM node's disabled property by passing in the node's id or the
@@ -912,7 +935,7 @@ nsContextMenu.prototype = {
         ? document.getElementById(aItemOrId)
         : aItemOrId;
     item.disabled = !aEnabled;
-  },
+  }
 
   /**
    * Most menu items are visible if there's 1 or 0 messages selected, and
@@ -934,7 +957,7 @@ nsContextMenu.prototype = {
         !this.onPlayableMedia
     );
     this.enableItem(aID, this.numSelectedMessages == 1);
-  },
+  }
 
   /**
    * Set given attribute of specified context-menu item. If the
@@ -958,7 +981,7 @@ nsContextMenu.prototype = {
         elem.setAttribute(aAttr, aVal);
       }
     }
-  },
+  }
 
   /**
    * Get an absolute URL for clicked-on link, from the href property or by
@@ -977,7 +1000,7 @@ nsContextMenu.prototype = {
     }
     href = this.makeURLAbsolute(this.link.baseURI, href);
     return href;
-  },
+  }
 
   /**
    * Generate a URI object from the linkURL spec
@@ -990,7 +1013,7 @@ nsContextMenu.prototype = {
       // e.g. empty URL string
     }
     return null;
-  },
+  }
 
   /**
    * Get the scheme for the clicked-on linkURI, if present.
@@ -1002,36 +1025,15 @@ nsContextMenu.prototype = {
     }
 
     return null;
-  },
+  }
 
   /**
-   * Get some text, any text, for the clicked-on link.
-   * @return the link text, title, alt, href, or "" if everything fails
+   * Get the text of the clicked-on link.
+   * @returns {string}
    */
   linkText() {
-    var text = gatherTextUnder(this.link);
-    if (!text || text.trim() == "") {
-      text = this.link.getAttribute("title");
-      if (!text || text.trim() == "") {
-        text = this.link.getAttribute("alt");
-        if (!text || text.trim() == "") {
-          if (this.link.href) {
-            text = this.link.href;
-          } else {
-            text = this.link.getAttributeNS(
-              "http://www.w3.org/1999/xlink",
-              "href"
-            );
-            if (text && !(text.trim() == "")) {
-              text = this.makeURLAbsolute(this.link.baseURI, text);
-            }
-          }
-        }
-      }
-    }
-
-    return text;
-  },
+    return this.linkTextStr;
+  }
 
   /**
    * Determines whether the focused window has something selected.
@@ -1039,7 +1041,7 @@ nsContextMenu.prototype = {
    */
   isContentSelection() {
     return !document.commandDispatcher.focusedWindow.getSelection().isCollapsed;
-  },
+  }
 
   /**
    * Convert relative URL to absolute, using a provided <base>.
@@ -1054,7 +1056,7 @@ nsContextMenu.prototype = {
     var baseURI = Services.io.newURI(aBase);
 
     return Services.io.newURI(baseURI.resolve(aUrl)).spec;
-  },
+  }
 
   /**
    * Determine whether a DOM node is a text or password input, or a textarea.
@@ -1068,38 +1070,26 @@ nsContextMenu.prototype = {
     }
 
     return aNode instanceof HTMLTextAreaElement;
-  },
-
-  /**
-   * Hide a separator based on whether there are any non-hidden items between
-   * it and the previous separator.
-   *
-   * @param aSeparatorID  The id of the separator element.
-   */
-  hideIfAppropriate(aSeparatorID) {
-    this.showItem(aSeparatorID, this.shouldShowSeparator(aSeparatorID));
-  },
+  }
 
   /**
    * Determine whether a separator should be shown based on whether
    * there are any non-hidden items between it and the previous separator.
-   * @param  aSeparatorID
-   *         The id of the separator element
-   * @return true if the separator should be shown, false if not
+   * @param {DomElement} element - The separator element.
+   * @return {boolean} True if the separator should be shown, false if not.
    */
-  shouldShowSeparator(aSeparatorID) {
-    var separator = document.getElementById(aSeparatorID);
-    if (separator) {
-      var sibling = separator.previousElementSibling;
+  shouldShowSeparator(element) {
+    if (element) {
+      let sibling = element.previousElementSibling;
       while (sibling && sibling.localName != "menuseparator") {
-        if (sibling.getAttribute("hidden") != "true") {
+        if (!sibling.hidden) {
           return true;
         }
         sibling = sibling.previousElementSibling;
       }
     }
     return false;
-  },
+  }
 
   /**
    * Ensures that there isn't a separator shown at the bottom of the menu.
@@ -1109,7 +1099,7 @@ nsContextMenu.prototype = {
   checkLastSeparator(aPopup) {
     let sibling = aPopup.lastElementChild;
     while (sibling) {
-      if (sibling.getAttribute("hidden") != "true") {
+      if (!sibling.hidden) {
         if (sibling.localName == "menuseparator") {
           // If we got here then the item is a menuseparator and everything
           // below it hidden.
@@ -1120,10 +1110,10 @@ nsContextMenu.prototype = {
       }
       sibling = sibling.previousElementSibling;
     }
-  },
+  }
 
   openInBrowser() {
-    let url = this.target.ownerGlobal.top.location.href;
+    let url = this.target.ownerGlobal?.top.location.href;
     PlacesUtils.history
       .insert({
         url,
@@ -1137,7 +1127,7 @@ nsContextMenu.prototype = {
     Cc["@mozilla.org/uriloader/external-protocol-service;1"]
       .getService(Ci.nsIExternalProtocolService)
       .loadURI(Services.io.newURI(url));
-  },
+  }
 
   openLinkInBrowser() {
     PlacesUtils.history
@@ -1153,7 +1143,7 @@ nsContextMenu.prototype = {
     Cc["@mozilla.org/uriloader/external-protocol-service;1"]
       .getService(Ci.nsIExternalProtocolService)
       .loadURI(this.linkURI);
-  },
+  }
 
   mediaCommand(command) {
     var media = this.target;
@@ -1174,5 +1164,5 @@ nsContextMenu.prototype = {
       // XXX hide controls & show controls don't work in emails as Javascript is
       // disabled. May want to consider later for RSS feeds.
     }
-  },
-};
+  }
+}
