@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -57,6 +56,19 @@ var gSanitizePromptDialog = {
       "sanitizeButtonOK"
     );
 
+    let warningIcon = document.getElementById("sanitizeEverythingWarningIcon");
+    if (AppConstants.platform === "linux") {
+      warningIcon.setAttribute(
+        "src",
+        "moz-icon://stock/gtk-dialog-warning?size=dialog"
+      );
+    } else {
+      warningIcon.setAttribute(
+        "src",
+        "chrome://global/skin/icons/warning-large.png"
+      );
+    }
+
     if (this.selectedTimespan === Sanitizer.TIMESPAN_EVERYTHING) {
       this.prepareWarning();
       this.warningBox.hidden = false;
@@ -82,7 +94,6 @@ var gSanitizePromptDialog = {
       this.prepareWarning();
       if (warningBox.hidden) {
         warningBox.hidden = false;
-        window.resizeBy(0, warningBox.getBoundingClientRect().height);
       }
       window.document.title = this.bundleBrowser.getString(
         "sanitizeDialog2.everything.title"
@@ -119,12 +130,8 @@ var gSanitizePromptDialog = {
   /**
    * If the panel that displays a warning when the duration is "Everything" is
    * not set up, sets it up.  Otherwise does nothing.
-   *
-   * @param aDontShowItemList Whether only the warning message should be updated.
-   *                          True means the item list visibility status should not
-   *                          be changed.
    */
-  prepareWarning(aDontShowItemList) {
+  prepareWarning() {
     // If the date and time-aware locale warning string is ever used again,
     // initialize it here.  Currently we use the no-visits warning string,
     // which does not include date and time.  See bug 480169 comment 48.
@@ -132,9 +139,6 @@ var gSanitizePromptDialog = {
     var warningStringID;
     if (this.hasNonSelectedItems()) {
       warningStringID = "sanitizeSelectedWarning";
-      if (!aDontShowItemList) {
-        this.showItemList();
-      }
     } else {
       warningStringID = "sanitizeEverythingWarning2";
     }
@@ -166,7 +170,7 @@ var gSanitizePromptDialog = {
     } catch (e) {}
 
     // Update the warning prompt if needed
-    this.prepareWarning(true);
+    this.prepareWarning();
 
     return undefined;
   },
