@@ -5,6 +5,7 @@ module.exports = {
     // These are defined in the WebExtension script scopes by ExtensionCommon.jsm.
     // From toolkit/components/extensions/.eslintrc.js.
     ExtensionAPI: true,
+    ExtensionAPIPersistent: true,
     ExtensionCommon: true,
     ExtensionUtils: true,
     extensions: true,
@@ -33,24 +34,29 @@ module.exports = {
     isValidCookieStoreId: true,
 
     // These are defined in ext-mail.js.
+    ADDRESS_BOOK_WINDOW_URI: true,
     COMPOSE_WINDOW_URI: true,
+    MESSAGE_WINDOW_URI: true,
     MESSAGE_PROTOCOLS: true,
+    NOTIFICATION_COLLAPSE_TIME: true,
     ExtensionError: true,
     Tab: true,
     TabmailTab: true,
     Window: true,
     TabmailWindow: true,
+    MsgHdrToRawMessage: true,
     clickModifiersFromEvent: true,
     convertFolder: true,
+    convertAccount: true,
     traverseSubfolders: true,
     convertMailIdentity: true,
     convertMessage: true,
     folderPathToURI: true,
     folderURIToPath: true,
+    getNormalWindowReady: true,
     getTabBrowser: true,
     getTabTabmail: true,
     getTabWindow: true,
-    makeWidgetId: true,
     messageListTracker: true,
     messageTracker: true,
     tabGetSender: true,
