@@ -3,13 +3,14 @@
  * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
 
 let account, messages;
+let messagePane = document.getElementById("messagepane");
 
 add_task(async () => {
   account = createAccount();
   let rootFolder = account.incomingServer.rootFolder;
   rootFolder.createSubfolder("messageDisplayScripts", null);
   let folder = rootFolder.getChildNamed("messageDisplayScripts");
-  createMessages(folder, 10);
+  createMessages(folder, 11);
   messages = [...folder.messages];
 
   window.gFolderTreeView.selectFolder(folder);
@@ -24,13 +25,13 @@ async function checkMessageBody(expected, message, browser) {
     });
     // Ignore Windows line-endings, they're not important here.
     body = body.replace(/\r/g, "");
-    expected.textContent = `\n${body}\n\n` + expected.textContent;
+    expected.textContent = body + expected.textContent;
   }
   if (!browser) {
-    browser = document.getElementById("messagepane");
+    browser = messagePane;
   }
 
-  checkContent(browser, expected);
+  await checkContent(browser, expected);
 }
 
 /** Tests browser.tabs.insertCSS and browser.tabs.removeCSS. */
@@ -55,9 +56,7 @@ add_task(async function testInsertRemoveCSS() {
         await window.sendMessage();
 
         await browser.tabs.removeCSS(tab.id, { file: "test.css" });
-        await window.sendMessage();
 
-        await browser.tabs.remove(tab.id);
         browser.test.notifyPass("finished");
       },
       "test.css": "body { background-color: green; }",
@@ -70,7 +69,7 @@ add_task(async function testInsertRemoveCSS() {
   });
 
   window.gFolderDisplay.selectViewIndex(0);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
+  await BrowserTestUtils.browserLoaded(messagePane);
 
   await extension.startup();
 
@@ -90,11 +89,9 @@ add_task(async function testInsertRemoveCSS() {
   await checkMessageBody({ backgroundColor: "rgb(0, 128, 0)" }, messages[0]);
   extension.sendMessage();
 
-  await extension.awaitMessage();
-  await checkMessageBody({ backgroundColor: "rgba(0, 0, 0, 0)" }, messages[0]);
-  extension.sendMessage();
-
   await extension.awaitFinish("finished");
+  await checkMessageBody({ backgroundColor: "rgba(0, 0, 0, 0)" }, messages[0]);
+
   await extension.unload();
 });
 
@@ -128,9 +125,6 @@ add_task(async function testInsertRemoveCSSNoPermissions() {
           "insertCSS without permission should throw"
         );
 
-        await window.sendMessage();
-
-        await browser.tabs.remove(tab.id);
         browser.test.notifyPass("finished");
       },
       "test.css": "body { background-color: red; }",
@@ -143,11 +137,11 @@ add_task(async function testInsertRemoveCSSNoPermissions() {
   });
 
   window.gFolderDisplay.selectViewIndex(1);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
+  await BrowserTestUtils.browserLoaded(messagePane);
 
   await extension.startup();
 
-  await extension.awaitMessage();
+  await extension.awaitFinish("finished");
   await checkMessageBody(
     {
       backgroundColor: "rgba(0, 0, 0, 0)",
@@ -155,9 +149,7 @@ add_task(async function testInsertRemoveCSSNoPermissions() {
     },
     messages[1]
   );
-  extension.sendMessage();
 
-  await extension.awaitFinish("finished");
   await extension.unload();
 });
 
@@ -175,13 +167,12 @@ add_task(async function testExecuteScript() {
         await window.sendMessage();
 
         await browser.tabs.executeScript(tab.id, { file: "test.js" });
-        await window.sendMessage();
 
-        await browser.tabs.remove(tab.id);
         browser.test.notifyPass("finished");
       },
       "test.js": () => {
-        document.body.textContent += "Hey look, the script ran!";
+        document.body.querySelector(".moz-text-flowed").textContent +=
+          "Hey look, the script ran!";
       },
       "utils.js": await getUtilsJS(),
     },
@@ -192,7 +183,7 @@ add_task(async function testExecuteScript() {
   });
 
   window.gFolderDisplay.selectViewIndex(2);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
+  await BrowserTestUtils.browserLoaded(messagePane);
 
   await extension.startup();
 
@@ -204,7 +195,7 @@ add_task(async function testExecuteScript() {
   await checkMessageBody({ foo: "bar" }, messages[2]);
   extension.sendMessage();
 
-  await extension.awaitMessage();
+  await extension.awaitFinish("finished");
   await checkMessageBody(
     {
       foo: "bar",
@@ -212,9 +203,7 @@ add_task(async function testExecuteScript() {
     },
     messages[2]
   );
-  extension.sendMessage();
 
-  await extension.awaitFinish("finished");
   await extension.unload();
 });
 
@@ -248,13 +237,11 @@ add_task(async function testExecuteScriptNoPermissions() {
           "executeScript without permission should throw"
         );
 
-        await window.sendMessage();
-
-        await browser.tabs.remove(tab.id);
         browser.test.notifyPass("finished");
       },
       "test.js": () => {
-        document.body.textContent += "Hey look, the script ran!";
+        document.body.querySelector(".moz-text-flowed").textContent +=
+          "Hey look, the script ran!";
       },
       "utils.js": await getUtilsJS(),
     },
@@ -265,15 +252,13 @@ add_task(async function testExecuteScriptNoPermissions() {
   });
 
   window.gFolderDisplay.selectViewIndex(3);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
+  await BrowserTestUtils.browserLoaded(messagePane);
 
   await extension.startup();
 
-  await extension.awaitMessage();
-  await checkMessageBody({ foo: null, textContent: "" }, messages[3]);
-  extension.sendMessage();
-
   await extension.awaitFinish("finished");
+  await checkMessageBody({ foo: null, textContent: "" }, messages[3]);
+
   await extension.unload();
 });
 
@@ -286,24 +271,23 @@ add_task(async function testExecuteScriptAlias() {
         await window.sendMessage();
 
         await browser.tabs.executeScript(tab.id, {
-          code: `document.body.textContent += messenger.runtime.getManifest().applications.gecko.id;`,
+          code: `document.body.querySelector(".moz-text-flowed").textContent +=
+                   messenger.runtime.getManifest().applications.gecko.id;`,
         });
-        await window.sendMessage();
 
-        await browser.tabs.remove(tab.id);
         browser.test.notifyPass("finished");
       },
       "utils.js": await getUtilsJS(),
     },
     manifest: {
-      applications: { gecko: { id: "alias@mochitest" } },
+      applications: { gecko: { id: "message_display_scripts@mochitest" } },
       background: { scripts: ["utils.js", "background.js"] },
       permissions: ["messagesModify"],
     },
   });
 
   window.gFolderDisplay.selectViewIndex(4);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
+  await BrowserTestUtils.browserLoaded(messagePane);
 
   await extension.startup();
 
@@ -311,11 +295,12 @@ add_task(async function testExecuteScriptAlias() {
   await checkMessageBody({ textContent: "" }, messages[4]);
   extension.sendMessage();
 
-  await extension.awaitMessage();
-  await checkMessageBody({ textContent: "alias@mochitest" }, messages[4]);
-  extension.sendMessage();
-
   await extension.awaitFinish("finished");
+  await checkMessageBody(
+    { textContent: "message_display_scripts@mochitest" },
+    messages[4]
+  );
+
   await extension.unload();
 });
 
@@ -328,6 +313,13 @@ add_task(async function testRegister() {
   let extension = ExtensionTestUtils.loadExtension({
     files: {
       "background.js": async () => {
+        // Keep track of registered scrips being executed and ready.
+        browser.runtime.onMessage.addListener((message, sender) => {
+          if (message == "LOADED") {
+            window.sendMessage("ScriptLoaded", sender.tab.id);
+          }
+        });
+
         let registeredScript = await browser.messageDisplayScripts.register({
           css: [{ code: "body { color: white }" }, { file: "test.css" }],
           js: [
@@ -336,16 +328,39 @@ add_task(async function testRegister() {
           ],
         });
 
-        await window.sendMessage();
+        browser.test.onMessage.addListener(async (message, data) => {
+          switch (message) {
+            case "Unregister":
+              await registeredScript.unregister();
+              browser.test.notifyPass("finished");
+              break;
 
-        await registeredScript.unregister();
-        await window.sendMessage();
+            case "RuntimeMessageTest":
+              try {
+                browser.test.assertEq(
+                  `Received: ${data.tabId}`,
+                  await browser.tabs.sendMessage(data.tabId, data.tabId)
+                );
+              } catch (ex) {
+                browser.test.fail(
+                  `Failed to send message to composeScript: ${ex}`
+                );
+              }
+              browser.test.sendMessage("RuntimeMessageTestDone");
+              break;
+          }
+        });
 
-        browser.test.notifyPass("finished");
+        window.sendMessage("Ready");
       },
       "test.css": "body { background-color: green; }",
       "test.js": () => {
-        document.body.textContent += "Hey look, the script ran!";
+        document.body.querySelector(".moz-text-flowed").textContent +=
+          "Hey look, the script ran!";
+        browser.runtime.onMessage.addListener(async message => {
+          return `Received: ${message}`;
+        });
+        browser.runtime.sendMessage("LOADED");
       },
       "utils.js": await getUtilsJS(),
     },
@@ -355,30 +370,27 @@ add_task(async function testRegister() {
     },
   });
 
-  let messagePane = document.getElementById("messagepane");
   let tabmail = document.getElementById("tabmail");
   window.gFolderDisplay.selectViewIndex(5);
-  await awaitBrowserLoaded(messagePane);
+  await BrowserTestUtils.browserLoaded(messagePane);
 
-  await extension.startup();
+  extension.startup();
+  await extension.awaitMessage("Ready");
 
-  // Check a message that was already loaded.
-  await extension.awaitMessage();
-  await BrowserTestUtils.waitForEvent(window, "extension-scripts-added");
+  // Check a message that was already loaded. This tab has not loaded the
+  // registered scripts.
   await checkMessageBody(
     {
-      backgroundColor: "rgb(0, 128, 0)",
-      color: "rgb(255, 255, 255)",
-      foo: "bar",
-      textContent: "Hey look, the script ran!",
+      backgroundColor: "rgba(0, 0, 0, 0)",
+      textContent: "",
     },
     messages[5]
   );
 
   // Load a new message and check it is modified.
+  let loadPromise = extension.awaitMessage("ScriptLoaded");
   window.gFolderDisplay.selectViewIndex(6);
-  await awaitBrowserLoaded(messagePane);
-  await BrowserTestUtils.waitForEvent(window, "extension-scripts-added");
+  let tabId = await loadPromise;
   await checkMessageBody(
     {
       backgroundColor: "rgb(0, 128, 0)",
@@ -388,15 +400,21 @@ add_task(async function testRegister() {
     },
     messages[6]
   );
+  // Check runtime messaging.
+  let testDonePromise = extension.awaitMessage("RuntimeMessageTestDone");
+  extension.sendMessage("RuntimeMessageTest", { tabId });
+  await testDonePromise;
 
   // Open the message in a new tab.
   // First, sabotage the message pane so we can be sure it changed.
   messagePane.contentDocument.body.style.backgroundColor = "red";
   messagePane.contentDocument.body.textContent = "Nope.";
 
-  window.MsgOpenSelectedMessages();
+  loadPromise = extension.awaitMessage("ScriptLoaded");
+  openMessageInTab(messages[6]);
+  tabId = await loadPromise;
   Assert.equal(tabmail.tabInfo.length, 2);
-  await BrowserTestUtils.waitForEvent(window, "extension-scripts-added");
+
   await checkMessageBody(
     {
       backgroundColor: "rgb(0, 128, 0)",
@@ -406,6 +424,10 @@ add_task(async function testRegister() {
     },
     messages[6]
   );
+  // Check runtime messaging.
+  testDonePromise = extension.awaitMessage("RuntimeMessageTestDone");
+  extension.sendMessage("RuntimeMessageTest", { tabId });
+  await testDonePromise;
 
   // Open a content tab. The CSS and script shouldn't apply.
   let newTab = window.openContentTab("http://mochi.test:8888/");
@@ -421,10 +443,15 @@ add_task(async function testRegister() {
     undefined,
     newTab.browser
   );
-  tabmail.closeTab(newTab);
+  // Check runtime messaging.
+  testDonePromise = extension.awaitMessage("RuntimeMessageTestDone");
+  extension.sendMessage("RuntimeMessageTest", { tabId });
+  await testDonePromise;
 
-  // We should be back at the message opened in a tab.
-  await BrowserTestUtils.waitForEvent(window, "extension-scripts-added");
+  // Closing this tab, should bring us back to the message tab.
+  loadPromise = extension.awaitMessage("ScriptLoaded");
+  tabmail.closeTab(newTab);
+  tabId = await loadPromise;
   await checkMessageBody(
     {
       backgroundColor: "rgb(0, 128, 0)",
@@ -434,14 +461,17 @@ add_task(async function testRegister() {
     },
     messages[6]
   );
+  // Check runtime messaging.
+  testDonePromise = extension.awaitMessage("RuntimeMessageTestDone");
+  extension.sendMessage("RuntimeMessageTest", { tabId });
+  await testDonePromise;
 
   // Open the message in a new window.
-  let newWindowPromise = BrowserTestUtils.domWindowOpenedAndLoaded();
-  window.MsgOpenNewWindowForMessage(messages[7]);
-  let newWindow = await newWindowPromise;
+  loadPromise = extension.awaitMessage("ScriptLoaded");
+  let newWindow = await openMessageInWindow(messages[7]);
   let newWindowMessagePane = newWindow.document.getElementById("messagepane");
+  tabId = await loadPromise;
 
-  await BrowserTestUtils.waitForEvent(newWindow, "extension-scripts-added");
   await checkMessageBody(
     {
       backgroundColor: "rgb(0, 128, 0)",
@@ -452,11 +482,15 @@ add_task(async function testRegister() {
     messages[7],
     newWindowMessagePane
   );
-  extension.sendMessage();
+  // Check runtime messaging.
+  testDonePromise = extension.awaitMessage("RuntimeMessageTestDone");
+  extension.sendMessage("RuntimeMessageTest", { tabId });
+  await testDonePromise;
 
   // Unregister.
-  extension.sendMessage();
-  await extension.awaitMessage();
+  extension.sendMessage("Unregister");
+  await extension.awaitFinish("finished");
+  await extension.unload();
 
   // Check the CSS is unloaded from the message in a tab.
   await checkMessageBody(
@@ -470,13 +504,13 @@ add_task(async function testRegister() {
   );
 
   // Close the new tab. The message reloads in the first tab, so the CSS
-  // should't be applied and the script shouldn't have run.
+  // shouldn't be applied and the script shouldn't have run.
   // Sabotage the message pane so we can be sure it changed.
   messagePane.contentDocument.body.style.backgroundColor = "red";
   messagePane.contentDocument.body.textContent = "Nope.";
   tabmail.closeTab(tabmail.tabInfo[1]);
 
-  await awaitBrowserLoaded(messagePane);
+  await BrowserTestUtils.browserLoaded(messagePane);
   // Let's wait a while and see if anything happens:
   // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
   await new Promise(resolve => setTimeout(resolve, 1000));
@@ -503,33 +537,18 @@ add_task(async function testRegister() {
   );
 
   await BrowserTestUtils.closeWindow(newWindow);
-
-  extension.sendMessage();
-
-  await extension.awaitFinish("finished");
-  await extension.unload();
 });
 
 /** Tests content_scripts in the manifest do not affect message display. */
 async function subtestContentScriptManifest(message, ...permissions) {
   let extension = ExtensionTestUtils.loadExtension({
     files: {
-      "background.js": async () => {
-        let [tab] = await browser.tabs.query({ mailTab: true });
-
-        await window.sendMessage();
-
-        await browser.tabs.remove(tab.id);
-        browser.test.notifyPass("finished");
-      },
       "test.css": "body { background-color: red; }",
       "test.js": () => {
         document.body.textContent += "Hey look, the script ran!";
       },
-      "utils.js": await getUtilsJS(),
     },
     manifest: {
-      background: { scripts: ["utils.js", "background.js"] },
       permissions,
       content_scripts: [
         {
@@ -548,7 +567,6 @@ async function subtestContentScriptManifest(message, ...permissions) {
   await extension.startup();
   ExtensionTestUtils.failOnSchemaWarnings(true);
 
-  await extension.awaitMessage();
   await checkMessageBody(
     {
       backgroundColor: "rgba(0, 0, 0, 0)",
@@ -556,21 +574,19 @@ async function subtestContentScriptManifest(message, ...permissions) {
     },
     message
   );
-  extension.sendMessage();
 
-  await extension.awaitFinish("finished");
   await extension.unload();
 }
 
 add_task(async function testContentScriptManifestNoPermission() {
-  window.gFolderDisplay.selectViewIndex(0);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
-  await subtestContentScriptManifest(messages[0]);
+  window.gFolderDisplay.selectViewIndex(7);
+  await BrowserTestUtils.browserLoaded(messagePane);
+  await subtestContentScriptManifest(messages[7]);
 });
 add_task(async function testContentScriptManifest() {
-  window.gFolderDisplay.selectViewIndex(1);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
-  await subtestContentScriptManifest(messages[1], "messagesModify");
+  window.gFolderDisplay.selectViewIndex(8);
+  await BrowserTestUtils.browserLoaded(messagePane);
+  await subtestContentScriptManifest(messages[8], "messagesModify");
 });
 
 /** Tests registered content scripts do not affect message display. */
@@ -585,16 +601,12 @@ async function subtestContentScriptRegister(message, ...permissions) {
           matchAboutBlank: true,
         });
 
-        let [tab] = await browser.tabs.query({ mailTab: true });
-
-        await window.sendMessage();
-
-        await browser.tabs.remove(tab.id);
         browser.test.notifyPass("finished");
       },
       "test.css": "body { background-color: red; }",
       "test.js": () => {
-        document.body.textContent += "Hey look, the script ran!";
+        document.body.querySelector(".moz-text-flowed").textContent +=
+          "Hey look, the script ran!";
       },
       "utils.js": await getUtilsJS(),
     },
@@ -606,7 +618,7 @@ async function subtestContentScriptRegister(message, ...permissions) {
 
   await extension.startup();
 
-  await extension.awaitMessage();
+  await extension.awaitFinish("finished");
   await checkMessageBody(
     {
       backgroundColor: "rgba(0, 0, 0, 0)",
@@ -614,22 +626,20 @@ async function subtestContentScriptRegister(message, ...permissions) {
     },
     message
   );
-  extension.sendMessage();
 
-  await extension.awaitFinish("finished");
   await extension.unload();
 }
 
 add_task(async function testContentScriptRegisterNoPermission() {
-  window.gFolderDisplay.selectViewIndex(2);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
-  await subtestContentScriptRegister(messages[2], "<all_urls>");
+  window.gFolderDisplay.selectViewIndex(9);
+  await BrowserTestUtils.browserLoaded(messagePane);
+  await subtestContentScriptRegister(messages[9], "<all_urls>");
 });
 add_task(async function testContentScriptRegister() {
-  window.gFolderDisplay.selectViewIndex(3);
-  await awaitBrowserLoaded(document.getElementById("messagepane"));
+  window.gFolderDisplay.selectViewIndex(10);
+  await BrowserTestUtils.browserLoaded(messagePane);
   await subtestContentScriptRegister(
-    messages[3],
+    messages[10],
     "<all_urls>",
     "messagesModify"
   );
