@@ -7,9 +7,10 @@
 var { ExtensionTestUtils } = ChromeUtils.import(
   "resource://testing-common/ExtensionXPCShellUtils.jsm"
 );
-var { fixIterator } = ChromeUtils.import(
-  "resource:///modules/iteratorUtils.jsm"
-);
+
+add_task(async function setup() {
+  Services.prefs.setIntPref("ldap_2.servers.osx.dirType", -1);
+});
 
 add_task(async function test_addressBooks() {
   async function background() {
@@ -66,23 +67,7 @@ add_task(async function test_addressBooks() {
           event.args.length,
           "Argument count is correct"
         );
-        for (let i = 0; i < expectedArgs.length; i++) {
-          if (typeof expectedArgs[i] == "object") {
-            for (let k of Object.keys(expectedArgs[i])) {
-              browser.test.assertEq(
-                expectedArgs[i][k],
-                event.args[i][k],
-                `Property ${k} is correct`
-              );
-            }
-          } else {
-            browser.test.assertEq(
-              expectedArgs[i],
-              event.args[i],
-              `Argument ${i + 1} is correct`
-            );
-          }
-        }
+        window.assertDeepEqual(expectedArgs, event.args);
         if (expectedEvents.length == 1) {
           return event.args;
         }
@@ -96,27 +81,28 @@ add_task(async function test_addressBooks() {
       let list = await browser.addressBooks.list();
       browser.test.assertEq(2, list.length);
       for (let b of list) {
-        browser.test.assertEq(4, Object.keys(b).length);
+        browser.test.assertEq(5, Object.keys(b).length);
         browser.test.assertEq(36, b.id.length);
         browser.test.assertEq("addressBook", b.type);
         browser.test.assertTrue("name" in b);
         browser.test.assertFalse(b.readOnly);
+        browser.test.assertFalse(b.remote);
       }
 
       let completeList = await browser.addressBooks.list(true);
       browser.test.assertEq(2, completeList.length);
       for (let b of completeList) {
-        browser.test.assertEq(6, Object.keys(b).length);
+        browser.test.assertEq(7, Object.keys(b).length);
       }
 
       firstBookId = list[0].id;
       secondBookId = list[1].id;
 
       let firstBook = await browser.addressBooks.get(firstBookId);
-      browser.test.assertEq(4, Object.keys(firstBook).length);
+      browser.test.assertEq(5, Object.keys(firstBook).length);
 
       let secondBook = await browser.addressBooks.get(secondBookId, true);
-      browser.test.assertEq(6, Object.keys(secondBook).length);
+      browser.test.assertEq(7, Object.keys(secondBook).length);
       browser.test.assertTrue(Array.isArray(secondBook.contacts));
       browser.test.assertEq(0, secondBook.contacts.length);
       browser.test.assertTrue(Array.isArray(secondBook.mailingLists));
@@ -199,6 +185,8 @@ add_task(async function test_addressBooks() {
       newContactId = await browser.contacts.create(firstBookId, {
         FirstName: "first",
         LastName: "last",
+        Notes: "Notes",
+        SomethingCustom: "Custom property",
       });
       browser.test.assertEq(36, newContactId.length);
       checkEvents([
@@ -219,35 +207,201 @@ add_task(async function test_addressBooks() {
       );
 
       let newContact = await browser.contacts.get(newContactId);
-      browser.test.assertEq(4, Object.keys(newContact).length);
+      browser.test.assertEq(6, Object.keys(newContact).length);
       browser.test.assertEq(newContactId, newContact.id);
       browser.test.assertEq(firstBookId, newContact.parentId);
       browser.test.assertEq("contact", newContact.type);
-      browser.test.assertEq(3, Object.keys(newContact.properties).length);
-      browser.test.assertEq("0", newContact.properties.PreferMailFormat);
+      browser.test.assertEq(false, newContact.readOnly);
+      browser.test.assertEq(false, newContact.remote);
+      browser.test.assertEq(5, Object.keys(newContact.properties).length);
       browser.test.assertEq("first", newContact.properties.FirstName);
       browser.test.assertEq("last", newContact.properties.LastName);
+      browser.test.assertEq("Notes", newContact.properties.Notes);
+      browser.test.assertEq(
+        "Custom property",
+        newContact.properties.SomethingCustom
+      );
+      browser.test.assertEq(
+        `BEGIN:VCARD\r\nVERSION:4.0\r\nNOTE:Notes\r\nN:last;first;;;\r\nUID:${newContactId}\r\nEND:VCARD\r\n`,
+        newContact.properties.vCard
+      );
 
+      // Changing the UID should throw.
+      try {
+        await browser.contacts.update(newContactId, {
+          vCard: `BEGIN:VCARD\r\nVERSION:4.0\r\nN:;first;;;\r\nEMAIL;PREF=1:first@last\r\nUID:SomethingNew\r\nEND:VCARD\r\n`,
+        });
+        browser.test.fail(
+          `Updating a contact with a vCard with a differnt UID should throw`
+        );
+      } catch (ex) {
+        browser.test.assertEq(
+          `The card's UID ${newContactId} may not be changed: BEGIN:VCARD\r\nVERSION:4.0\r\nN:;first;;;\r\nEMAIL;PREF=1:first@last\r\nUID:SomethingNew\r\nEND:VCARD\r\n.`,
+          ex.message,
+          `browser.contacts.update threw exception`
+        );
+      }
+
+      // If a vCard and legacy properties are given, vCard must win.
       await browser.contacts.update(newContactId, {
-        PrimaryEmail: "first@last",
-        LastName: null,
+        vCard: `BEGIN:VCARD\r\nVERSION:4.0\r\nN:;first;;;\r\nEMAIL;PREF=1:first@last\r\nUID:${newContactId}\r\nEND:VCARD\r\n`,
+        FirstName: "Superman",
+        PrimaryEmail: "c.kent@dailyplanet.com",
+        PreferDisplayName: "0",
+        OtherCustom: "Yet another custom property",
+        Notes: "Ignored Notes",
       });
       checkEvents([
         "contacts",
         "onUpdated",
         { type: "contact", parentId: firstBookId, id: newContactId },
+        {
+          PrimaryEmail: { oldValue: null, newValue: "first@last" },
+          LastName: { oldValue: "last", newValue: null },
+          OtherCustom: {
+            oldValue: null,
+            newValue: "Yet another custom property",
+          },
+          PreferDisplayName: { oldValue: null, newValue: "0" },
+        },
       ]);
 
       let updatedContact = await browser.contacts.get(newContactId);
-      browser.test.assertEq(3, Object.keys(updatedContact.properties).length);
-      browser.test.assertEq("0", updatedContact.properties.PreferMailFormat);
+      browser.test.assertEq(6, Object.keys(updatedContact.properties).length);
       browser.test.assertEq("first", updatedContact.properties.FirstName);
       browser.test.assertEq(
         "first@last",
         updatedContact.properties.PrimaryEmail
       );
       browser.test.assertTrue(!("LastName" in updatedContact.properties));
+      browser.test.assertTrue(
+        !("Notes" in updatedContact.properties),
+        "The vCard is not specifying Notes and the specified Notes property should be ignored."
+      );
+      browser.test.assertEq(
+        "Custom property",
+        updatedContact.properties.SomethingCustom,
+        "Untouched custom properties should not be changed by updating the vCard"
+      );
+      browser.test.assertEq(
+        "Yet another custom property",
+        updatedContact.properties.OtherCustom,
+        "Custom properties should be added even while updating a vCard"
+      );
+      browser.test.assertEq(
+        "0",
+        updatedContact.properties.PreferDisplayName,
+        "Setting non-banished properties parallel to a vCard should update"
+      );
+      browser.test.assertEq(
+        `BEGIN:VCARD\r\nVERSION:4.0\r\nN:;first;;;\r\nEMAIL;PREF=1:first@last\r\nUID:${newContactId}\r\nEND:VCARD\r\n`,
+        updatedContact.properties.vCard
+      );
 
+      // Manually Remove properties.
+      await browser.contacts.update(newContactId, {
+        LastName: "lastname",
+        PrimaryEmail: null,
+        SecondEmail: "test@invalid.de",
+        SomethingCustom: null,
+        OtherCustom: null,
+      });
+      checkEvents([
+        "contacts",
+        "onUpdated",
+        { type: "contact", parentId: firstBookId, id: newContactId },
+        {
+          LastName: { oldValue: null, newValue: "lastname" },
+          // It is how it is. Defining a 2nd email with no 1st, will make it the first.
+          PrimaryEmail: { oldValue: "first@last", newValue: "test@invalid.de" },
+          SomethingCustom: { oldValue: "Custom property", newValue: null },
+          OtherCustom: {
+            oldValue: "Yet another custom property",
+            newValue: null,
+          },
+        },
+      ]);
+
+      updatedContact = await browser.contacts.get(newContactId);
+      browser.test.assertEq(5, Object.keys(updatedContact.properties).length);
+      // LastName and FirstName are stored in the same multi field property and changing LastName should not change FirstName.
+      browser.test.assertEq("first", updatedContact.properties.FirstName);
+      browser.test.assertEq("lastname", updatedContact.properties.LastName);
+      browser.test.assertEq(
+        "test@invalid.de",
+        updatedContact.properties.PrimaryEmail
+      );
+      browser.test.assertTrue(
+        !("SomethingCustom" in updatedContact.properties)
+      );
+      browser.test.assertTrue(!("OtherCustom" in updatedContact.properties));
+      browser.test.assertEq(
+        `BEGIN:VCARD\r\nVERSION:4.0\r\nN:lastname;first;;;\r\nEMAIL:test@invalid.de\r\nUID:${newContactId}\r\nEND:VCARD\r\n`,
+        updatedContact.properties.vCard
+      );
+
+      // Add an email address, going from 1 to 2.Also remove FirstName, LastName should stay.
+      await browser.contacts.update(newContactId, {
+        FirstName: null,
+        PrimaryEmail: "new1@invalid.de",
+        SecondEmail: "new2@invalid.de",
+      });
+      checkEvents([
+        "contacts",
+        "onUpdated",
+        { type: "contact", parentId: firstBookId, id: newContactId },
+        {
+          PrimaryEmail: {
+            oldValue: "test@invalid.de",
+            newValue: "new1@invalid.de",
+          },
+          SecondEmail: { oldValue: null, newValue: "new2@invalid.de" },
+          FirstName: { oldValue: "first", newValue: null },
+        },
+      ]);
+
+      updatedContact = await browser.contacts.get(newContactId);
+      browser.test.assertEq(5, Object.keys(updatedContact.properties).length);
+      browser.test.assertEq("lastname", updatedContact.properties.LastName);
+      browser.test.assertEq(
+        "new1@invalid.de",
+        updatedContact.properties.PrimaryEmail
+      );
+      browser.test.assertEq(
+        "new2@invalid.de",
+        updatedContact.properties.SecondEmail
+      );
+      browser.test.assertEq(
+        `BEGIN:VCARD\r\nVERSION:4.0\r\nN:lastname;;;;\r\nEMAIL;PREF=1:new1@invalid.de\r\nUID:${newContactId}\r\nEMAIL:new2@invalid.de\r\nEND:VCARD\r\n`,
+        updatedContact.properties.vCard
+      );
+
+      // Remove and email address, going from 2 to 1.
+      await browser.contacts.update(newContactId, {
+        SecondEmail: null,
+      });
+      checkEvents([
+        "contacts",
+        "onUpdated",
+        { type: "contact", parentId: firstBookId, id: newContactId },
+        {
+          SecondEmail: { oldValue: "new2@invalid.de", newValue: null },
+        },
+      ]);
+
+      updatedContact = await browser.contacts.get(newContactId);
+      browser.test.assertEq(4, Object.keys(updatedContact.properties).length);
+      browser.test.assertEq("lastname", updatedContact.properties.LastName);
+      browser.test.assertEq(
+        "new1@invalid.de",
+        updatedContact.properties.PrimaryEmail
+      );
+      browser.test.assertEq(
+        `BEGIN:VCARD\r\nVERSION:4.0\r\nN:lastname;;;;\r\nEMAIL;PREF=1:new1@invalid.de\r\nUID:${newContactId}\r\nEND:VCARD\r\n`,
+        updatedContact.properties.vCard
+      );
+
+      // Set a fixed UID.
       let fixedContactId = await browser.contacts.create(
         firstBookId,
         "this is a test",
@@ -318,13 +472,15 @@ add_task(async function test_addressBooks() {
       );
 
       let newAddressList = await browser.mailingLists.get(newMailingListId);
-      browser.test.assertEq(6, Object.keys(newAddressList).length);
+      browser.test.assertEq(8, Object.keys(newAddressList).length);
       browser.test.assertEq(newMailingListId, newAddressList.id);
       browser.test.assertEq(firstBookId, newAddressList.parentId);
       browser.test.assertEq("mailingList", newAddressList.type);
       browser.test.assertEq("name", newAddressList.name);
       browser.test.assertEq("", newAddressList.nickName);
       browser.test.assertEq("", newAddressList.description);
+      browser.test.assertEq(false, newAddressList.readOnly);
+      browser.test.assertEq(false, newAddressList.remote);
 
       // Test that a valid name is ensured for an existing mail list
       await browser.test.assertRejects(
@@ -388,7 +544,12 @@ add_task(async function test_addressBooks() {
       checkEvents([
         "contacts",
         "onCreated",
-        { type: "contact", parentId: firstBookId, id: anotherContactId },
+        {
+          type: "contact",
+          parentId: firstBookId,
+          id: anotherContactId,
+          readOnly: false,
+        },
       ]);
 
       await browser.mailingLists.addMember(newMailingListId, anotherContactId);
@@ -576,6 +737,7 @@ add_task(async function test_addressBooks() {
         "contacts",
         "onUpdated",
         { type: "contact", parentId: parentId1, id: contactId },
+        { LastName: { oldValue: "add", newValue: "edit" } },
       ]);
       browser.test.assertEq("external", updatedContact.properties.FirstName);
       browser.test.assertEq("edit", updatedContact.properties.LastName);
@@ -668,7 +830,7 @@ add_task(async function test_addressBooks() {
       return null;
     }
     function findMailingList(id) {
-      for (let list of fixIterator(parent.childNodes, Ci.nsIAbDirectory)) {
+      for (let list of parent.childNodes) {
         if (list.UID == id) {
           return list;
         }
@@ -676,16 +838,13 @@ add_task(async function test_addressBooks() {
       return null;
     }
 
-    let parent = MailServices.ab.directories
-      .getNext()
-      .QueryInterface(Ci.nsIAbDirectory);
+    let parent = MailServices.ab.getDirectory("jsaddrbook://abook.sqlite");
     switch (action) {
       case "createAddressBook": {
-        const kJSDirectory = 101; // defined in nsDirPrefs.h
         let dirPrefId = MailServices.ab.newAddressBook(
           "external add",
           "",
-          kJSDirectory
+          Ci.nsIAbManager.JS_DIRECTORY_TYPE
         );
         let book = MailServices.ab.getDirectoryFromId(dirPrefId);
         extension.sendMessage(book.UID, dirPrefId);
@@ -772,7 +931,7 @@ add_task(async function test_addressBooks() {
 
         if (list && contact) {
           list.addCard(contact);
-          equal(1, [...list.childCards].length);
+          equal(1, list.childCards.length);
           extension.sendMessage();
           return;
         }
@@ -784,7 +943,7 @@ add_task(async function test_addressBooks() {
 
         if (list && contact) {
           list.deleteCards([contact]);
-          equal(0, [...list.childCards].length);
+          equal(0, list.childCards.length);
           ok(findContact(args[1]), "Contact was not removed");
           extension.sendMessage();
           return;
@@ -802,83 +961,9 @@ add_task(async function test_addressBooks() {
   await extension.unload();
 });
 
-add_task(async function test_quickSearch() {
-  async function background() {
-    let book1 = await browser.addressBooks.create({ name: "book1" });
-    let book2 = await browser.addressBooks.create({ name: "book2" });
-
-    let book1contacts = {
-      charlie: await browser.contacts.create(book1, { FirstName: "charlie" }),
-      juliet: await browser.contacts.create(book1, { FirstName: "juliet" }),
-      mike: await browser.contacts.create(book1, { FirstName: "mike" }),
-      oscar: await browser.contacts.create(book1, { FirstName: "oscar" }),
-      papa: await browser.contacts.create(book1, { FirstName: "papa" }),
-      romeo: await browser.contacts.create(book1, { FirstName: "romeo" }),
-      victor: await browser.contacts.create(book1, { FirstName: "victor" }),
-    };
-
-    let book2contacts = {
-      bigBird: await browser.contacts.create(book2, {
-        FirstName: "Big",
-        LastName: "Bird",
-      }),
-      cookieMonster: await browser.contacts.create(book2, {
-        FirstName: "Cookie",
-        LastName: "Monster",
-      }),
-      elmo: await browser.contacts.create(book2, { FirstName: "Elmo" }),
-      grover: await browser.contacts.create(book2, { FirstName: "Grover" }),
-      oscarTheGrouch: await browser.contacts.create(book2, {
-        FirstName: "Oscar",
-        LastName: "The Grouch",
-      }),
-    };
-
-    // A search string without a match in either book.
-    let results = await browser.contacts.quickSearch(book1, "snuffleupagus");
-    browser.test.assertEq(0, results.length);
-
-    // A search string with a match in the book we're searching.
-    results = await browser.contacts.quickSearch(book1, "mike");
-    browser.test.assertEq(1, results.length);
-    browser.test.assertEq(book1contacts.mike, results[0].id);
-
-    // A search string with a match in the book we're not searching.
-    results = await browser.contacts.quickSearch(book1, "elmo");
-    browser.test.assertEq(0, results.length);
-
-    // A search string with a match in both books.
-    results = await browser.contacts.quickSearch(book1, "oscar");
-    browser.test.assertEq(1, results.length);
-    browser.test.assertEq(book1contacts.oscar, results[0].id);
-
-    // A search string with a match in both books. Looking in all books.
-    results = await browser.contacts.quickSearch("oscar");
-    browser.test.assertEq(2, results.length);
-    browser.test.assertEq(book1contacts.oscar, results[0].id);
-    browser.test.assertEq(book2contacts.oscarTheGrouch, results[1].id);
-
-    // No valid search strings.
-    results = await browser.contacts.quickSearch("  ");
-    browser.test.assertEq(0, results.length);
-
-    await browser.addressBooks.delete(book1);
-    await browser.addressBooks.delete(book2);
-
-    browser.test.notifyPass("addressBooks");
-  }
-
-  let extension = ExtensionTestUtils.loadExtension({
-    background,
-    manifest: { permissions: ["addressBooks"] },
-  });
-
-  await extension.startup();
-  await extension.awaitFinish("addressBooks");
-  await extension.unload();
-});
-
 registerCleanupFunction(() => {
   // Make sure any open database is given a chance to close.
-  Services.obs.notifyObservers(null, "quit-application");
+  Services.startup.advanceShutdownPhase(
+    Services.startup.SHUTDOWN_PHASE_APPSHUTDOWNCONFIRMED
+  );
 });
