@@ -190,7 +190,7 @@ var gCookiesWindow = {
   },
 
   _view: {
-    QueryInterface: ChromeUtils.generateQI([Ci.nsITreeView]),
+    QueryInterface: ChromeUtils.generateQI(["nsITreeView"]),
     _filtered: false,
     _filterSet: [],
     _filterValue: "",
@@ -339,7 +339,6 @@ var gCookiesWindow = {
     },
     set selection(val) {
       this._selection = val;
-      return val;
     },
     getRowProperties(aRow) {
       return "";
@@ -773,18 +772,11 @@ var gCookiesWindow = {
       }
     }
 
-    var blockFutureCookies = false;
-    if (Services.prefs.prefHasUserValue("network.cookie.blockFutureCookies")) {
-      blockFutureCookies = Services.prefs.getBoolPref(
-        "network.cookie.blockFutureCookies"
-      );
-    }
     for (let item of deleteItems) {
       Services.cookies.remove(
         item.host,
         item.name,
         item.path,
-        blockFutureCookies,
         item.originAttributes
       );
     }
