@@ -1,5 +1,4 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -46,8 +45,9 @@ var gDownloadDirSection = {
     var downloadFolder = document.getElementById("downloadFolder");
     var chooseFolder = document.getElementById("chooseFolder");
     var preference = Preferences.get("browser.download.useDownloadDir");
-    downloadFolder.disabled = !preference.value;
-    chooseFolder.disabled = !preference.value;
+    var dirPreference = Preferences.get("browser.download.dir");
+    downloadFolder.disabled = !preference.value || dirPreference.locked;
+    chooseFolder.disabled = !preference.value || dirPreference.locked;
     return undefined;
   },
 
@@ -108,13 +108,15 @@ var gDownloadDirSection = {
     var currentDirPref = Preferences.get("browser.download.downloadDir");
     var downloadDir =
       currentDirPref.value || (await this._indexToFile(folderListPref.value));
-    let urlSpec = Services.io
-      .getProtocolHandler("file")
-      .QueryInterface(Ci.nsIFileProtocolHandler)
-      .getURLSpecFromFile(downloadDir);
+    if (downloadDir) {
+      let urlSpec = Services.io
+        .getProtocolHandler("file")
+        .QueryInterface(Ci.nsIFileProtocolHandler)
+        .getURLSpecFromDir(downloadDir);
 
-    downloadFolder.style.backgroundImage =
-      "url(moz-icon://" + urlSpec + "?size=16)";
+      downloadFolder.style.backgroundImage =
+        "url(moz-icon://" + urlSpec + "?size=16)";
+    }
 
     return undefined;
   },
