@@ -25,12 +25,14 @@
 
 using mozilla::ArrayLength;
 
-static const char* const sMailProtocols[] = {"mailto"};
+static const char* const sMailProtocols[] = {"mailto", "mid"};
 
 static const char* const sNewsProtocols[] = {"news", "snews", "nntp"};
 
 static const char* const sFeedProtocols[] = {"feed"};
 
+static const char* const sCalendarProtocols[] = {"webcal", "webcals"};
+
 struct AppTypeAssociation {
   uint16_t type;
   const char* const* protocols;
@@ -63,10 +65,14 @@ static const AppTypeAssociation sAppTypes[] = {
     {nsIShellService::NEWS, sNewsProtocols, ArrayLength(sNewsProtocols),
      nullptr, nullptr},
     {nsIShellService::RSS, sFeedProtocols, ArrayLength(sFeedProtocols),
-     "application/rss+xml", "rss"}};
+     "application/rss+xml", "rss"},
+    {nsIShellService::CALENDAR, sCalendarProtocols,
+     ArrayLength(sCalendarProtocols), "text/calendar", "ics"}};
 
 nsGNOMEShellService::nsGNOMEShellService()
-    : mCheckedThisSession(false), mAppIsInPath(false) {}
+    : mUseLocaleFilenames(false),
+      mCheckedThisSession(false),
+      mAppIsInPath(false) {}
 
 nsresult nsGNOMEShellService::Init() {
   nsresult rv;
@@ -86,7 +92,7 @@ nsresult nsGNOMEShellService::Init() {
                               getter_AddRefs(appPath));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = appPath->AppendNative(NS_LITERAL_CSTRING(MOZ_APP_NAME));
+  rv = appPath->AppendNative(nsLiteralCString(MOZ_APP_NAME));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = appPath->GetNativePath(mAppPath);
@@ -218,16 +224,16 @@ bool nsGNOMEShellService::checkDefault(const char* const* aProtocols,
   nsresult rv;
 
   for (unsigned int i = 0; i < aLength; ++i) {
-
     if (IsRunningAsASnap()) {
-      const gchar* argv[] = {"xdg-settings", "get", "default-url-scheme-handler",
-                             aProtocols[i], nullptr};
+      const gchar* argv[] = {"xdg-settings", "get",
+                             "default-url-scheme-handler", aProtocols[i],
+                             nullptr};
       GSpawnFlags flags = static_cast<GSpawnFlags>(G_SPAWN_SEARCH_PATH |
                                                    G_SPAWN_STDERR_TO_DEV_NULL);
       gchar* output = nullptr;
       gint exit_status = 0;
-      if (!g_spawn_sync(nullptr, (gchar**)argv, nullptr, flags, nullptr, nullptr,
-                        &output, nullptr, &exit_status, nullptr)) {
+      if (!g_spawn_sync(nullptr, (gchar**)argv, nullptr, flags, nullptr,
+                        nullptr, &output, nullptr, &exit_status, nullptr)) {
         return false;
       }
       if (exit_status != 0) {
@@ -235,8 +241,8 @@ bool nsGNOMEShellService::checkDefault(const char* const* aProtocols,
         return false;
       }
       if (strcmp(output, "thunderbird.desktop\n") == 0) {
-          g_free(output);
-          return true;
+        g_free(output);
+        return true;
       }
       g_free(output);
       return false;
@@ -283,8 +289,12 @@ nsresult nsGNOMEShellService::MakeDefault(const char* const* aProtocols,
 
   if (IsRunningAsASnap()) {
     for (unsigned int i = 0; i < aProtocolsLength; ++i) {
-      const gchar* argv[] = {"xdg-settings", "set", "default-url-scheme-handler",
-                             aProtocols[i], "thunderbird.desktop", nullptr};
+      const gchar* argv[] = {"xdg-settings",
+                             "set",
+                             "default-url-scheme-handler",
+                             aProtocols[i],
+                             "thunderbird.desktop",
+                             nullptr};
       GSpawnFlags flags = static_cast<GSpawnFlags>(G_SPAWN_SEARCH_PATH |
                                                    G_SPAWN_STDOUT_TO_DEV_NULL |
                                                    G_SPAWN_STDERR_TO_DEV_NULL);
