@@ -10,9 +10,6 @@ var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { EnigmailDialog } = ChromeUtils.import(
   "chrome://openpgp/content/modules/dialog.jsm"
 );
-var { EnigmailLocale } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/locale.jsm"
-);
 var { EnigmailKey } = ChromeUtils.import(
   "chrome://openpgp/content/modules/key.jsm"
 );
@@ -22,8 +19,8 @@ var { EnigmailKeyRing } = ChromeUtils.import(
 var { EnigmailArmor } = ChromeUtils.import(
   "chrome://openpgp/content/modules/armor.jsm"
 );
-var { EnigmailFiles } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/files.jsm"
+var { MailStringUtils } = ChromeUtils.import(
+  "resource:///modules/MailStringUtils.jsm"
 );
 
 var l10n = new Localization(["messenger/openpgp/openpgp.ftl"], true);
@@ -34,9 +31,7 @@ var l10n = new Localization(["messenger/openpgp/openpgp.ftl"], true);
  * resultFlags.canceled is set to true if the user clicked cancel
  */
 function passphrasePromptCallback(win, keyId, resultFlags) {
-  let p = {};
-  p.value = "";
-  let dummy = {};
+  let password = { value: "" };
   if (
     !Services.prompt.promptPassword(
       win,
@@ -44,9 +39,7 @@ function passphrasePromptCallback(win, keyId, resultFlags) {
       l10n.formatValueSync("passphrase-prompt", {
         key: keyId,
       }),
-      p,
-      null,
-      dummy
+      password
     )
   ) {
     resultFlags.canceled = true;
@@ -54,16 +47,16 @@ function passphrasePromptCallback(win, keyId, resultFlags) {
   }
 
   resultFlags.canceled = false;
-  return p.value;
+  return password.value;
 }
 
-// Return the first block of the wanted type (skip blocks of wrong type)
-function getKeyBlockFromFile(path, wantSecret) {
-  var contents = EnigmailFiles.readFile(path);
-  if (!contents) {
-    return "";
-  }
-
+/**
+ * @param {nsIFile} file
+ * @returns {string} The first block of the wanted type, or empty string.
+ *   Skip blocks of wrong type.
+ */
+async function getKeyBlockFromFile(file, wantSecret) {
+  let contents = await IOUtils.readUTF8(file.path).catch(() => "");
   let searchOffset = 0;
 
   while (searchOffset < contents.length) {
@@ -101,7 +94,7 @@ function getKeyBlockFromFile(path, wantSecret) {
  * import OpenPGP keys from file
  * @param {string} what - "rev" for revocation, "pub" for public keys, "sec" for secret keys.
  */
-function EnigmailCommon_importObjectFromFile(what) {
+async function EnigmailCommon_importObjectFromFile(what) {
   let importingRevocation = what == "rev";
   let promptStr = importingRevocation ? "import-rev-file" : "import-key-file";
 
@@ -131,22 +124,23 @@ function EnigmailCommon_importObjectFromFile(what) {
     let errorMsgObj = {};
 
     if (importingRevocation) {
-      EnigmailKeyRing.importRevFromFile(file);
+      await EnigmailKeyRing.importRevFromFile(file);
       continue;
     }
 
     let isSecret = what == "sec";
     let importBinary = false;
-    let keyBlock = getKeyBlockFromFile(file, isSecret);
+    let keyBlock = await getKeyBlockFromFile(file, isSecret);
 
     // if we don't find an ASCII block, try to import as binary.
     if (!keyBlock) {
       importBinary = true;
-      keyBlock = EnigmailFiles.readFile(file);
+      let data = await IOUtils.read(file.path);
+      keyBlock = MailStringUtils.uint8ArrayToByteString(data);
     }
 
-    // Generat a preview of the imported key.
-    let preview = EnigmailKey.getKeyListFromKeyBlock(
+    // Generate a preview of the imported key.
+    let preview = await EnigmailKey.getKeyListFromKeyBlock(
       keyBlock,
       errorMsgObj,
       true, // interactive
