@@ -8,10 +8,6 @@
 
 "use strict";
 
-var elib = ChromeUtils.import(
-  "resource://testing-common/mozmill/elementslib.jsm"
-);
-
 var {
   add_attachments,
   close_compose_window,
@@ -26,7 +22,6 @@ var {
   create_folder,
   create_message,
   select_click_row,
-
   wait_for_popup_to_open,
 } = ChromeUtils.import(
   "resource://testing-common/mozmill/FolderDisplayHelpers.jsm"
@@ -58,8 +53,8 @@ var b64Attachment =
   "SNQAAlmAY+71EgFoAAAAASUVORK5CYII=";
 var b64Size = 188;
 
-add_task(function setupModule(module) {
-  folder = create_folder("ComposeAttachmentA");
+add_setup(async function() {
+  folder = await create_folder("ComposeAttachmentA");
 
   messenger = Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger);
 
@@ -97,7 +92,7 @@ add_task(function setupModule(module) {
   ];
 
   for (let i = 0; i < messages.length; i++) {
-    add_message_to_folder(folder, create_message(messages[i]));
+    await add_message_to_folder([folder], create_message(messages[i]));
   }
 });
 
@@ -152,8 +147,8 @@ function check_no_attachment_size(controller, index) {
     throw new Error("attachment.size attribute should be -1!");
   }
 
-  // If there's no size, the size attribute is the zero-width space.
-  if (node.getAttribute("size") != "\u200b") {
+  // If there's no size, the size attribute is empty.
+  if (node.getAttribute("size") != "") {
     throw new Error("Attachment size should not be displayed!");
   }
 }
@@ -183,12 +178,11 @@ function check_total_attachment_size(controller, count) {
   }
 
   // Next, make sure that the formatted size in the label is correct
-  let formattedSize = sizeNode.getAttribute("value");
   let expectedFormattedSize = messenger.formatFileSize(size);
-  if (formattedSize != expectedFormattedSize) {
+  if (sizeNode.textContent != expectedFormattedSize) {
     throw new Error(
       "Formatted attachment size (" +
-        formattedSize +
+        sizeNode.textContent +
         ") does not " +
         "match expected value (" +
         expectedFormattedSize +
@@ -275,7 +269,7 @@ add_task(function test_rename_attachment() {
   // Now, rename the attachment.
   let bucket = cwc.e("attachmentBucket");
   let node = bucket.querySelector("richlistitem.attachmentItem");
-  cwc.click(new elib.Elem(node));
+  cwc.click(node);
   plan_for_modal_dialog("commonDialogWindow", subtest_rename_attachment);
   cwc.window.RenameSelectedAttachment();
   wait_for_modal_dialog("commonDialogWindow");
@@ -303,7 +297,7 @@ add_task(function test_open_attachment() {
   let fileHandler = Services.io
     .getProtocolHandler("file")
     .QueryInterface(Ci.nsIFileProtocolHandler);
-  let url = fileHandler.getURLSpecFromFile(file);
+  let url = fileHandler.getURLSpecFromActualFile(file);
   let size = file.fileSize;
 
   add_attachments(cwc, url, size);
@@ -312,7 +306,7 @@ add_task(function test_open_attachment() {
   let bucket = cwc.e("attachmentBucket");
   let node = bucket.querySelector("richlistitem.attachmentItem");
   plan_for_modal_dialog("unknownContentTypeWindow", subtest_open_attachment);
-  cwc.doubleClick(new elib.Elem(node));
+  cwc.doubleClick(node);
   wait_for_modal_dialog("unknownContentTypeWindow");
 
   close_compose_window(cwc);
@@ -376,40 +370,6 @@ function check_attachment_names(aController, aNames) {
   }
 }
 
-/**
- * Execute a test of opening and closing reorderAttachmentsPanel via keyboard.
- *
- * @param aCwc        The controller for the compose window
- * @param aActions    An array of objects specifying an action of opening the
- *                    panel via keyboard.
- *                    { focusEl: the element to be focused for keypresses
- *                      key:     keycode of key to press
- *                      key_modifiers: { accelKey: bool, ctrlKey: bool
- *                                       shiftKey: bool, altKey: bool, etc. },
- *                    }
- */
-function subtest_reordering_panel_keyboard(aCwc, aActions) {
-  let panel = aCwc.e("reorderAttachmentsPanel");
-
-  for (let action of aActions) {
-    aCwc.keypress(
-      new elib.Elem(action.focusEl),
-      action.key,
-      action.key_modifiers
-    );
-    wait_for_popup_to_open(panel);
-    aCwc.sleep(0);
-
-    // Press ESC which should close the panel.
-    aCwc.keypress(null, "VK_ESCAPE", {});
-    aCwc.sleep(0);
-    aCwc.waitFor(
-      () => panel.state == "closed",
-      "Reordering panel didn't close when Escape was pressed."
-    );
-  }
-}
-
 /**
  * Execute a test of attachment reordering actions and check the resulting order.
  *
@@ -427,7 +387,7 @@ function subtest_reordering_panel_keyboard(aCwc, aActions) {
  *                          }
  * @param openPanel {boolean}   Whether to open reorderAttachmentsPanel for the test
  */
-function subtest_reordering(
+async function subtest_reordering(
   aCwc,
   aInitialAttachmentNames,
   aReorder_actions,
@@ -442,7 +402,7 @@ function subtest_reordering(
     add_attachments(aCwc, filePrefix + name, size);
   }
   aCwc.sleep(0);
-  Assert.equal(aCwc.window.attachmentsCount(), aInitialAttachmentNames.length);
+  Assert.equal(bucket.itemCount, aInitialAttachmentNames.length);
   check_attachment_names(aCwc, aInitialAttachmentNames);
 
   if (aOpenPanel) {
@@ -450,7 +410,7 @@ function subtest_reordering(
     aCwc.window.showReorderAttachmentsPanel();
     aCwc.sleep(0);
     panel = aCwc.e("reorderAttachmentsPanel");
-    wait_for_popup_to_open(panel);
+    await wait_for_popup_to_open(panel);
   }
 
   for (let action of aReorder_actions) {
@@ -461,9 +421,9 @@ function subtest_reordering(
     }
     // Take action.
     if ("button" in action) {
-      aCwc.click(aCwc.eid(action.button));
+      aCwc.click(aCwc.e(action.button));
     } else if ("key" in action) {
-      aCwc.keypress(null, action.key, action.key_modifiers);
+      EventUtils.synthesizeKey(action.key, action.key_modifiers, aCwc.window);
     }
     aCwc.sleep(0);
     // Check result.
@@ -488,7 +448,7 @@ function subtest_reordering(
  * Check basic and advanced attachment reordering operations.
  * This is the main function of this test.
  */
-add_task(function test_attachment_reordering() {
+add_task(async function test_attachment_reordering() {
   let cwc = open_compose_new_mail();
   let editorEl = cwc.window.GetCurrentEditorElement();
   let bucket = cwc.e("attachmentBucket");
@@ -507,38 +467,28 @@ add_task(function test_attachment_reordering() {
     add_attachments(cwc, filePrefix + name, size);
     cwc.sleep(0);
   }
-  Assert.equal(cwc.window.attachmentsCount(), initialAttachmentNames_0.length);
+  Assert.equal(bucket.itemCount, initialAttachmentNames_0.length);
   check_attachment_names(cwc, initialAttachmentNames_0);
 
   // Show 'Reorder Attachments' panel via mouse clicks.
-  cwc.rightClick(new elib.Elem(bucket.getItemAtIndex(1)));
-  cwc.click_menus_in_sequence(cwc.e("msgComposeAttachmentItemContext"), [
-    { id: "composeAttachmentContext_reorderItem" },
-  ]);
-  wait_for_popup_to_open(panel);
+  let contextMenu = cwc.e("msgComposeAttachmentItemContext");
+  let shownPromise = BrowserTestUtils.waitForEvent(contextMenu, "popupshown");
+  EventUtils.synthesizeMouseAtCenter(
+    bucket.getItemAtIndex(1),
+    { type: "contextmenu" },
+    cwc.window
+  );
+  await shownPromise;
+  contextMenu.activateItem(cwc.e("composeAttachmentContext_reorderItem"));
+  await wait_for_popup_to_open(panel);
 
   // Click on the editor which should close the panel.
-  cwc.click(new elib.Elem(editorEl));
+  cwc.click(editorEl);
   cwc.waitFor(
     () => panel.state == "closed",
     "Reordering panel didn't close when editor was clicked."
   );
 
-  // Show 'Reorder Attachments' panel via keyboard.
-  // key_reorderAttachments, Bug 1427037
-  // const openPanelActions = [
-  //   { focusEl: editorEl,
-  //     key: "x",
-  //     key_modifiers: openReorderPanelModifiers },
-  //   { focusEl: bucket,
-  //     key: "x",
-  //     key_modifiers: openReorderPanelModifiers },
-  // ];
-
-  // XXX this doesn't work on any platform yet, ESC doesn't close the panel.
-  // Execute test of opening 'Reorder Attachments' panel via keyboard.
-  // subtest_reordering_panel_keyboard(cwc, openPanelActions);
-
   // Clean up for a new set of attachments.
   cwc.window.RemoveAllAttachments();
 
@@ -553,22 +503,22 @@ add_task(function test_attachment_reordering() {
     },
     {
       select: [4],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["a", "b", "B", "bb", "C", "x"],
     },
     {
       select: [5],
-      button: "btn_moveAttachmentTop",
+      button: "btn_moveAttachmentFirst",
       result: ["x", "a", "b", "B", "bb", "C"],
     },
     {
       select: [0],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["a", "x", "b", "B", "bb", "C"],
     },
     {
       select: [1],
-      button: "btn_moveAttachmentBottom",
+      button: "btn_moveAttachmentLast",
       result: ["a", "b", "B", "bb", "C", "x"],
     },
     {
@@ -600,22 +550,22 @@ add_task(function test_attachment_reordering() {
     // For starters: moving a single attachment around in the list.
     {
       select: [1],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["x", "a", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
     {
       select: [0],
-      button: "btn_moveAttachmentBottom",
+      button: "btn_moveAttachmentLast",
       result: ["a", "C", "y1", "y2", "B", "b", "z", "bb", "x"],
     },
     {
       select: [8],
-      button: "btn_moveAttachmentTop",
+      button: "btn_moveAttachmentFirst",
       result: ["x", "a", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
     {
       select: [0],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["a", "x", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
 
@@ -627,56 +577,56 @@ add_task(function test_attachment_reordering() {
     // {description1.txt, photo1.jpg, description2.txt, photo2.txt}.
     {
       select: [1, 3, 4, 7],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["a", "C", "x", "B", "y1", "y2", "b", "bb", "z"],
     },
     {
       select: [2, 4, 5, 8],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["a", "x", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
     {
       select: [1, 3, 4, 7],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["x", "a", "y1", "y2", "C", "B", "z", "b", "bb"],
     },
 
     // Folding multiple, disjunct selection with inner block towards top/bottom.
     {
       select: [0, 2, 3, 6],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["x", "y1", "y2", "a", "C", "z", "B", "b", "bb"],
     },
     {
       select: [0, 1, 2, 5],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["x", "y1", "y2", "a", "z", "C", "B", "b", "bb"],
     },
     {
       select: [0, 1, 2, 4],
-      button: "btn_moveAttachmentUp",
+      button: "btn_moveAttachmentLeft",
       result: ["x", "y1", "y2", "z", "a", "C", "B", "b", "bb"],
     },
     {
       select: [3, 5, 6, 8],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["x", "y1", "y2", "a", "z", "b", "C", "B", "bb"],
     },
     {
       select: [4, 6, 7, 8],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["x", "y1", "y2", "a", "b", "z", "C", "B", "bb"],
     },
 
     // Prepare scenario for and test 'Group together' (upwards).
     {
       select: [1, 2],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["x", "a", "y1", "y2", "b", "z", "C", "B", "bb"],
     },
     {
       select: [0, 2, 3, 5],
-      button: "btn_moveAttachmentDown",
+      button: "btn_moveAttachmentRight",
       result: ["a", "x", "b", "y1", "y2", "C", "z", "B", "bb"],
     },
     {
@@ -728,12 +678,12 @@ add_task(function test_attachment_reordering() {
     // Collapsing multiple, disjunct selection with inner block to top/bottom.
     {
       select: [3, 5, 6, 8],
-      button: "btn_moveAttachmentTop",
+      button: "btn_moveAttachmentFirst",
       result: ["x", "bb", "B", "a", "z", "y2", "y1", "C", "b"],
     },
     {
       select: [0, 2, 3, 7],
-      button: "btn_moveAttachmentBottom",
+      button: "btn_moveAttachmentLast",
       result: ["bb", "z", "y2", "y1", "b", "x", "B", "a", "C"],
     },
   ];
@@ -759,8 +709,8 @@ add_task(function test_attachment_reordering() {
     // For starters: moving a single attachment around in the list.
     {
       select: [1],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["x", "a", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
@@ -794,8 +744,8 @@ add_task(function test_attachment_reordering() {
     },
     {
       select: [0],
-      // key_moveAttachmentDown
-      key: "VK_DOWN",
+      // key_moveAttachmentRight
+      key: "VK_RIGHT",
       key_modifiers: modAlt,
       result: ["a", "x", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
@@ -808,22 +758,22 @@ add_task(function test_attachment_reordering() {
     // {description1.txt, photo1.jpg, description2.txt, photo2.txt}.
     {
       select: [1, 3, 4, 7],
-      // key_moveAttachmentDown
-      key: "VK_DOWN",
+      // key_moveAttachmentRight
+      key: "VK_RIGHT",
       key_modifiers: modAlt,
       result: ["a", "C", "x", "B", "y1", "y2", "b", "bb", "z"],
     },
     {
       select: [2, 4, 5, 8],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["a", "x", "C", "y1", "y2", "B", "b", "z", "bb"],
     },
     {
       select: [1, 3, 4, 7],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["x", "a", "y1", "y2", "C", "B", "z", "b", "bb"],
     },
@@ -831,36 +781,36 @@ add_task(function test_attachment_reordering() {
     // Folding multiple, disjunct selection with inner block towards top/bottom.
     {
       select: [0, 2, 3, 6],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["x", "y1", "y2", "a", "C", "z", "B", "b", "bb"],
     },
     {
       select: [0, 1, 2, 5],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["x", "y1", "y2", "a", "z", "C", "B", "b", "bb"],
     },
     {
       select: [0, 1, 2, 4],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["x", "y1", "y2", "z", "a", "C", "B", "b", "bb"],
     },
     {
       select: [3, 5, 6, 8],
-      // key_moveAttachmentDown
-      key: "VK_DOWN",
+      // key_moveAttachmentRight
+      key: "VK_RIGHT",
       key_modifiers: modAlt,
       result: ["x", "y1", "y2", "a", "z", "b", "C", "B", "bb"],
     },
     {
       select: [4, 6, 7, 8],
-      // key_moveAttachmentDown
-      key: "VK_DOWN",
+      // key_moveAttachmentRight
+      key: "VK_RIGHT",
       key_modifiers: modAlt,
       result: ["x", "y1", "y2", "a", "b", "z", "C", "B", "bb"],
     },
@@ -868,36 +818,36 @@ add_task(function test_attachment_reordering() {
     // Prepare scenario for and test 'Group together' (upwards/downwards).
     {
       select: [1, 2],
-      // key_moveAttachmentDown
-      key: "VK_DOWN",
+      // key_moveAttachmentRight
+      key: "VK_RIGHT",
       key_modifiers: modAlt,
       result: ["x", "a", "y1", "y2", "b", "z", "C", "B", "bb"],
     },
     {
       select: [0, 2, 3, 5],
-      // key_moveAttachmentDown
-      key: "VK_DOWN",
+      // key_moveAttachmentRight
+      key: "VK_RIGHT",
       key_modifiers: modAlt,
       result: ["a", "x", "b", "y1", "y2", "C", "z", "B", "bb"],
     },
     {
       select: [1, 3, 4, 6],
       // key_moveAttachmentBundleUp
-      key: "VK_LEFT",
+      key: "VK_UP",
       key_modifiers: modAlt,
       result: ["a", "x", "y1", "y2", "z", "b", "C", "B", "bb"],
     },
     {
       select: [5, 6],
-      // key_moveAttachmentUp
-      key: "VK_UP",
+      // key_moveAttachmentLeft
+      key: "VK_LEFT",
       key_modifiers: modAlt,
       result: ["a", "x", "y1", "y2", "b", "C", "z", "B", "bb"],
     },
     {
       select: [0, 4, 5, 7],
       // key_moveAttachmentBundleDown
-      key: "VK_RIGHT",
+      key: "VK_DOWN",
       key_modifiers: modAlt,
       result: ["x", "y1", "y2", "z", "a", "b", "C", "B", "bb"],
     },
@@ -957,21 +907,86 @@ add_task(function test_attachment_reordering() {
   ];
 
   // Execute the tests of reordering actions as defined above.
-  subtest_reordering(cwc, initialAttachmentNames_1, reorderActions_1);
-  subtest_reordering(cwc, initialAttachmentNames_2, reorderActions_2);
+  await subtest_reordering(cwc, initialAttachmentNames_1, reorderActions_1);
+  await subtest_reordering(cwc, initialAttachmentNames_2, reorderActions_2);
   // Check 3 (keyboard-only) with panel open.
-  subtest_reordering(cwc, initialAttachmentNames_3, reorderActions_3);
+  await subtest_reordering(cwc, initialAttachmentNames_3, reorderActions_3);
   // Check 3 (keyboard-only) without panel.
-  subtest_reordering(cwc, initialAttachmentNames_3, reorderActions_3, false);
+  await subtest_reordering(
+    cwc,
+    initialAttachmentNames_3,
+    reorderActions_3,
+    false
+  );
   // Check 4 (Alt+Y keyboard shortcut for sorting) without panel.
-  subtest_reordering(cwc, initialAttachmentNames_4, reorderActions_4, false);
+  await subtest_reordering(
+    cwc,
+    initialAttachmentNames_4,
+    reorderActions_4,
+    false
+  );
   // Check 4 (Alt+Y keyboard shortcut for sorting) with panel open.
-  subtest_reordering(cwc, initialAttachmentNames_4, reorderActions_4);
+  await subtest_reordering(cwc, initialAttachmentNames_4, reorderActions_4);
   // XXX When the root problem of bug 1425891 has been found and fixed, we should
   // test here if the panel stays open as it should, esp. on Windows.
 
   close_compose_window(cwc);
 });
 
-// XXX: Test attached emails dragged onto composer and files pulled from other
-// emails (this probably requires better drag-and-drop support from Mozmill)
+add_task(async function test_restore_attachment_bucket_height() {
+  let cwc = open_compose_new_mail();
+
+  let attachmentArea = cwc.window.document.getElementById("attachmentArea");
+  let attachmentBucket = cwc.window.document.getElementById("attachmentBucket");
+
+  Assert.ok(
+    BrowserTestUtils.is_hidden(attachmentArea),
+    "Attachment area should be hidden initially with no attachments"
+  );
+
+  // Add 9 attachments to open a pane least 2 rows height.
+  let files = [
+    { name: "foo.txt", size: 1234 },
+    { name: "bar.txt", size: 5678 },
+    { name: "baz.txt", size: 9012 },
+    { name: "foo2.txt", size: 1234 },
+    { name: "bar2.txt", size: 5678 },
+    { name: "baz2.txt", size: 9012 },
+    { name: "foo3.txt", size: 1234 },
+    { name: "bar3.txt", size: 5678 },
+    { name: "baz3.txt", size: 9012 },
+  ];
+  for (let i = 0; i < files.length; i++) {
+    add_attachments(cwc, filePrefix + files[i].name, files[i].size);
+  }
+
+  // Store the height of the attachment bucket.
+  let heightBefore = attachmentBucket.getBoundingClientRect().height;
+
+  let modifiers =
+    AppConstants.platform == "macosx"
+      ? { accelKey: true, shiftKey: true }
+      : { ctrlKey: true, shiftKey: true };
+
+  let collapsedPromise = BrowserTestUtils.waitForCondition(
+    () => BrowserTestUtils.is_visible(attachmentArea) && !attachmentArea.open,
+    "The attachment area should be visible but closed."
+  );
+
+  // Press Ctrl/Cmd+Shift+M to collapse the attachment pane.
+  EventUtils.synthesizeKey("M", modifiers, cwc.window);
+  await collapsedPromise;
+
+  let visiblePromise = BrowserTestUtils.waitForCondition(
+    () => BrowserTestUtils.is_visible(attachmentArea) && attachmentArea.open,
+    "The attachment area should be visible and open."
+  );
+  // Press Ctrl/Cmd+Shift+M again.
+  EventUtils.synthesizeKey("M", modifiers, cwc.window);
+  await visiblePromise;
+
+  // The height of these elements should have been properly restored.
+  Assert.equal(attachmentBucket.getBoundingClientRect().height, heightBefore);
+
+  close_compose_window(cwc);
+});
