@@ -7,7 +7,7 @@
 var {
   be_in_folder,
   create_folder,
-  make_new_sets_in_folder,
+  make_message_sets_in_folders,
 } = ChromeUtils.import(
   "resource://testing-common/mozmill/FolderDisplayHelpers.jsm"
 );
@@ -42,17 +42,10 @@ var gMsgMinutes = 9000;
 var gMockAlertsService = {
   _doFail: false,
 
-  QueryInterface: ChromeUtils.generateQI([Ci.nsIAlertsService]),
+  QueryInterface: ChromeUtils.generateQI(["nsIAlertsService"]),
 
-  showAlertNotification(
-    imageUrl,
-    title,
-    text,
-    textClickable,
-    cookie,
-    alertListener,
-    name
-  ) {
+  showAlert(alertInfo, alertListener) {
+    let { imageURL, title, text, textClickable, cookie, name } = alertInfo;
     // Setting the _doFail flag allows us to revert to the newmailalert.xhtml
     // notification
     if (this._doFail) {
@@ -60,7 +53,7 @@ var gMockAlertsService = {
       throw Components.Exception("", Cr.NS_ERROR_FAILURE);
     }
     this._didNotify = true;
-    this._imageUrl = imageUrl;
+    this._imageUrl = imageURL;
     this._title = title;
     this._text = text;
     this._textClickable = textClickable;
@@ -98,11 +91,7 @@ var gMockAlertsService = {
 };
 
 var gMockAlertsServiceFactory = {
-  createInstance(aOuter, aIID) {
-    if (aOuter != null) {
-      throw Components.Exception("", Cr.NS_ERROR_NO_AGGREGATION);
-    }
-
+  createInstance(aIID) {
     if (!aIID.equals(Ci.nsIAlertsService)) {
       throw Components.Exception("", Cr.NS_ERROR_NO_INTERFACE);
     }
@@ -111,16 +100,41 @@ var gMockAlertsServiceFactory = {
   },
 };
 
-add_task(function setupModule(module) {
+let gMockWindowsIntegration = {
+  QueryInterface: ChromeUtils.generateQI([
+    "nsIMessengerOSIntegration",
+    "nsIMessengerWindowsIntegration",
+  ]),
+
+  // Do not suppress notification, so that the tests can pass on Treeherder.
+  suppressNotification: false,
+
+  updateUnreadCount() {},
+};
+
+let gMockWindowsIntegrationFactory = {
+  createInstance() {
+    return gMockWindowsIntegration;
+  },
+};
+
+add_setup(async function() {
   // Register the mock alerts service
-  Components.manager
-    .QueryInterface(Ci.nsIComponentRegistrar)
-    .registerFactory(
-      Components.ID("{1bda6c33-b089-43df-a8fd-111907d6385a}"),
-      "Mock Alerts Service",
-      "@mozilla.org/system-alerts-service;1",
-      gMockAlertsServiceFactory
+  let registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
+  registrar.registerFactory(
+    Components.ID("{1bda6c33-b089-43df-a8fd-111907d6385a}"),
+    "Mock Alerts Service",
+    "@mozilla.org/system-alerts-service;1",
+    gMockAlertsServiceFactory
+  );
+  if (Services.appinfo.OS == "WINNT") {
+    registrar.registerFactory(
+      Components.ID("{a1e1bf40-4275-46b5-897f-71c20b6931cd}"),
+      "",
+      "@mozilla.org/messenger/osintegration;1",
+      gMockWindowsIntegrationFactory
     );
+  }
 
   // Ensure we have enabled new mail notifications
   remember_and_set_bool_pref("mail.biff.show_alert", true);
@@ -150,7 +164,7 @@ add_task(function setupModule(module) {
   server.performingBiff = true;
 
   // Create the target folders
-  gFolder = create_folder("My Folder");
+  gFolder = await create_folder("My Folder");
   let localRoot = server.rootFolder.QueryInterface(Ci.nsIMsgLocalMailFolder);
   gFolder2 = localRoot.createLocalSubfolder("Another Folder");
 
@@ -159,11 +173,15 @@ add_task(function setupModule(module) {
   account.addIdentity(identity2);
 });
 
-registerCleanupFunction(function teardownModule(module) {
+registerCleanupFunction(function() {
   put_bool_prefs_back();
   if (Services.appinfo.OS != "Darwin") {
     Services.prefs.setIntPref("alerts.totalOpenTime", gTotalOpenTime);
   }
+
+  // Request focus on something in the main window so the test doesn't time
+  // out waiting for focus.
+  document.getElementById("searchInput").focus();
 });
 
 function setupTest(test) {
@@ -196,55 +214,40 @@ function remember_and_set_bool_pref(aPrefString, aBoolValue) {
   Services.prefs.setBoolPref(aPrefString, aBoolValue);
 }
 
-/* This function wraps up make_new_sets_in_folder, and takes the
+/**
+ * This function wraps up MessageInjection.makeNewSetsInFolders, and takes the
  * same arguments.  The point of this function is to ensure that
  * each sent message is slightly newer than the last.  In this
  * case, each new message set will be sent one minute further
  * into the future than the last message set.
+ *
+ * @see MessageInjection.makeNewSetsInFolders
  */
-function make_gradually_newer_sets_in_folder(aFolder, aArgs) {
+async function make_gradually_newer_sets_in_folder(aFolder, aArgs) {
   gMsgMinutes -= 1;
   if (!aArgs.age) {
     for (let arg of aArgs) {
       arg.age = { minutes: gMsgMinutes };
     }
   }
-  make_new_sets_in_folder(aFolder, aArgs);
+  await make_message_sets_in_folders(aFolder, aArgs);
 }
 
-/**
- * Test that we revert to newmailalert.xhtml if there is no system
- * notification service present.
- */
-add_task(function test_revert_to_newmailalert() {
-  setupTest();
-  // Set up the gMockAlertsService so that it fails
-  // to send a notification.
-  gMockAlertsService._doFail = true;
-
-  // We expect the newmailalert.xhtml window...
-  plan_for_new_window("alert:alert");
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 2 }]);
-  let controller = wait_for_new_window("alert:alert");
-  plan_for_window_close(controller);
-  wait_for_window_close();
-});
-
 /**
  * Test that receiving new mail causes a notification to appear
  */
-add_task(function test_new_mail_received_causes_notification() {
+add_task(async function test_new_mail_received_causes_notification() {
   setupTest();
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1 }]);
-  Assert.ok(gMockAlertsService._didNotify, "Did not show alert notification.");
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1 }]);
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
 });
 
 /**
  * Test that if notification shows, we don't show newmailalert.xhtml
  */
-add_task(function test_dont_show_newmailalert() {
+add_task(async function test_dont_show_newmailalert() {
   setupTest();
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1 }]);
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1 }]);
 
   // Wait for newmailalert.xhtml to show
   plan_for_new_window("alert:alert");
@@ -260,14 +263,15 @@ add_task(function test_dont_show_newmailalert() {
  * Test that we notify, showing the oldest new, unread message received
  * since the last notification.
  */
-add_task(function test_show_oldest_new_unread_since_last_notification() {
+add_task(async function test_show_oldest_new_unread_since_last_notification() {
   setupTest();
   let notifyFirst = "This should notify first";
   Assert.ok(!gMockAlertsService._didNotify, "Should not have notified yet.");
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, body: { body: notifyFirst } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, body: { body: notifyFirst } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(notifyFirst, 1),
     "Should have notified for the first message"
@@ -279,10 +283,11 @@ add_task(function test_show_oldest_new_unread_since_last_notification() {
 
   let notifySecond = "This should notify second";
   Assert.ok(!gMockAlertsService._didNotify, "Should not have notified yet.");
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, body: { body: notifySecond } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, body: { body: notifySecond } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(notifySecond, 1),
     "Should have notified for the second message"
@@ -292,21 +297,22 @@ add_task(function test_show_oldest_new_unread_since_last_notification() {
 /**
  * Test that notifications work across different accounts.
  */
-add_task(function test_notification_works_across_accounts() {
+add_task(async function test_notification_works_across_accounts() {
   setupTest();
   // Cause a notification in the first folder
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1 }]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1 }]);
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
 
   gMockAlertsService._reset();
   // We'll set the time for these messages to be slightly further
   // into the past.  That way, test_notification_independent_across_accounts
   // has an opportunity to send slightly newer messages that are older than
   // the messages sent to gFolder.
-  make_gradually_newer_sets_in_folder(gFolder2, [
-    { count: 2, age: { minutes: gMsgMinutes + 20 } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder2],
+    [{ count: 2, age: { minutes: gMsgMinutes + 20 } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
 });
 
 /* Test that notification timestamps are independent from account
@@ -315,29 +321,30 @@ add_task(function test_notification_works_across_accounts() {
  * account completes, if it has new mail, it should notify, even if second
  * account's newest mail is older than the first account's newest mail.
  */
-add_task(function test_notifications_independent_across_accounts() {
+add_task(async function test_notifications_independent_across_accounts() {
   setupTest();
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1 }]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1 }]);
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
 
   gMockAlertsService._reset();
   // Next, let's make some mail arrive in the second folder, but
   // let's have that mail be slightly older than the mail that
   // landed in the first folder.  We should still notify.
-  make_gradually_newer_sets_in_folder(gFolder2, [
-    { count: 2, age: { minutes: gMsgMinutes + 10 } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder2],
+    [{ count: 2, age: { minutes: gMsgMinutes + 10 } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
 });
 
 /**
  * Test that we can show the message subject in the notification.
  */
-add_task(function test_show_subject() {
+add_task(async function test_show_subject() {
   setupTest();
   let subject = "This should be displayed";
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1, subject }]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1, subject }]);
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(subject),
     "Should have displayed the subject"
@@ -347,12 +354,12 @@ add_task(function test_show_subject() {
 /**
  * Test that we can hide the message subject in the notification.
  */
-add_task(function test_hide_subject() {
+add_task(async function test_hide_subject() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_subject", false);
   let subject = "This should not be displayed";
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1, subject }]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1, subject }]);
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     !gMockAlertsService._text.includes(subject),
     "Should not have displayed the subject"
@@ -362,7 +369,7 @@ add_task(function test_hide_subject() {
 /**
  * Test that we can show just the message sender in the notification.
  */
-add_task(function test_show_only_subject() {
+add_task(async function test_show_only_subject() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_preview", false);
   Services.prefs.setBoolPref("mail.biff.alert.show_sender", false);
@@ -372,10 +379,11 @@ add_task(function test_show_only_subject() {
   let subject = "This should not be displayed";
   let messageBody = "My message preview";
 
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, from: sender, subject, body: { body: messageBody } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, from: sender, subject, body: { body: messageBody } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(subject),
     "Should have displayed the subject"
@@ -393,11 +401,14 @@ add_task(function test_show_only_subject() {
 /**
  * Test that we can show the message sender in the notification.
  */
-add_task(function test_show_sender() {
+add_task(async function test_show_sender() {
   setupTest();
   let sender = ["John Cleese", "john@cleese.invalid"];
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1, from: sender }]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, from: sender }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(sender[0]),
     "Should have displayed the sender"
@@ -407,12 +418,15 @@ add_task(function test_show_sender() {
 /**
  * Test that we can hide the message sender in the notification.
  */
-add_task(function test_hide_sender() {
+add_task(async function test_hide_sender() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_sender", false);
   let sender = ["John Cleese", "john@cleese.invalid"];
-  make_gradually_newer_sets_in_folder(gFolder, [{ count: 1, from: sender }]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, from: sender }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     !gMockAlertsService._text.includes(sender[0]),
     "Should not have displayed the sender"
@@ -422,7 +436,7 @@ add_task(function test_hide_sender() {
 /**
  * Test that we can show just the message sender in the notification.
  */
-add_task(function test_show_only_sender() {
+add_task(async function test_show_only_sender() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_preview", false);
   Services.prefs.setBoolPref("mail.biff.alert.show_sender", true);
@@ -432,10 +446,11 @@ add_task(function test_show_only_sender() {
   let subject = "This should not be displayed";
   let messageBody = "My message preview";
 
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, from: sender, subject, body: { body: messageBody } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, from: sender, subject, body: { body: messageBody } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(sender[0]),
     "Should have displayed the sender"
@@ -453,14 +468,15 @@ add_task(function test_show_only_sender() {
 /**
  * Test that we can show the message preview in the notification.
  */
-add_task(function test_show_preview() {
+add_task(async function test_show_preview() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_preview", true);
   let messageBody = "My message preview";
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, body: { body: messageBody } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, body: { body: messageBody } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(messageBody),
     "Should have displayed the preview"
@@ -470,14 +486,15 @@ add_task(function test_show_preview() {
 /**
  * Test that we can hide the message preview in the notification.
  */
-add_task(function test_hide_preview() {
+add_task(async function test_hide_preview() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_preview", false);
   let messageBody = "My message preview";
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, body: { body: messageBody } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, body: { body: messageBody } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     !gMockAlertsService._text.includes(messageBody),
     "Should not have displayed the preview"
@@ -487,7 +504,7 @@ add_task(function test_hide_preview() {
 /**
  * Test that we can show justthe message preview in the notification.
  */
-add_task(function test_show_only_preview() {
+add_task(async function test_show_only_preview() {
   setupTest();
   Services.prefs.setBoolPref("mail.biff.alert.show_preview", true);
   Services.prefs.setBoolPref("mail.biff.alert.show_sender", false);
@@ -496,10 +513,11 @@ add_task(function test_show_only_preview() {
   let sender = ["John Cleese", "john@cleese.invalid"];
   let subject = "This should not be displayed";
   let messageBody = "My message preview";
-  make_gradually_newer_sets_in_folder(gFolder, [
-    { count: 1, from: sender, subject, body: { body: messageBody } },
-  ]);
-  Assert.ok(gMockAlertsService._didNotify, "Should have notified");
+  await make_gradually_newer_sets_in_folder(
+    [gFolder],
+    [{ count: 1, from: sender, subject, body: { body: messageBody } }]
+  );
+  await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
   Assert.ok(
     gMockAlertsService._text.includes(messageBody),
     "Should have displayed the preview: " + gMockAlertsService._text
@@ -518,7 +536,7 @@ add_task(function test_show_only_preview() {
  * Test that we can receive notifications even when the biff state of
  * the folder has not been changed.
  */
-add_task(function test_still_notify_with_unchanged_biff() {
+add_task(async function test_still_notify_with_unchanged_biff() {
   setupTest();
   // For now, we'll make sure that if we receive 10 pieces
   // of email, one after the other, we'll be notified for all
@@ -530,8 +548,8 @@ add_task(function test_still_notify_with_unchanged_biff() {
   Assert.ok(!gMockAlertsService._didNotify, "Should have notified.");
 
   for (let i = 0; i < HOW_MUCH_MAIL; i++) {
-    make_gradually_newer_sets_in_folder(gFolder, [{ count: 1 }]);
-    Assert.ok(gMockAlertsService._didNotify, "Should have notified.");
+    await make_gradually_newer_sets_in_folder([gFolder], [{ count: 1 }]);
+    await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
     gMockAlertsService._reset();
   }
 });
@@ -540,9 +558,9 @@ add_task(function test_still_notify_with_unchanged_biff() {
  * Test that we don't receive notifications for Draft, Queue, SentMail,
  * Templates or Junk folders.
  */
-add_task(function test_no_notification_for_uninteresting_folders() {
+add_task(async function test_no_notification_for_uninteresting_folders() {
   setupTest();
-  var someFolder = create_folder("Uninteresting Folder");
+  var someFolder = await create_folder("Uninteresting Folder");
   var uninterestingFlags = [
     Ci.nsMsgFolderFlags.Drafts,
     Ci.nsMsgFolderFlags.Queue,
@@ -554,7 +572,9 @@ add_task(function test_no_notification_for_uninteresting_folders() {
 
   for (let i = 0; i < uninterestingFlags.length; i++) {
     someFolder.flags = uninterestingFlags[i];
-    make_gradually_newer_sets_in_folder(someFolder, [{ count: 1 }]);
+    await make_gradually_newer_sets_in_folder([someFolder], [{ count: 1 }]);
+    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+    await new Promise(resolve => setTimeout(resolve, 100));
     Assert.ok(!gMockAlertsService._didNotify, "Showed alert notification.");
   }
 
@@ -564,11 +584,35 @@ add_task(function test_no_notification_for_uninteresting_folders() {
 
   for (let i = 0; i < uninterestingFlags.length; i++) {
     someFolder.flags |= uninterestingFlags[i];
-    make_gradually_newer_sets_in_folder(someFolder, [{ count: 1 }]);
-    Assert.ok(
-      gMockAlertsService._didNotify,
-      "Did not show alert notification."
-    );
+    await make_gradually_newer_sets_in_folder([someFolder], [{ count: 1 }]);
+    await TestUtils.waitForCondition(() => gMockAlertsService._didNotify);
     someFolder.flags = someFolder.flags & ~uninterestingFlags[i];
   }
 });
+
+/**
+ * Test that we revert to newmailalert.xhtml if there is no system notification
+ * service present.
+ *
+ * NOTE: this test should go last because if
+ * nsIAlertsService.showAlertNotification failed for once, we always fallback to
+ * newmailalert.xhtml afterwards.
+ */
+add_task(async function test_revert_to_newmailalert() {
+  setupTest();
+  // Set up the gMockAlertsService so that it fails
+  // to send a notification.
+  gMockAlertsService._doFail = true;
+
+  if (AppConstants.platform == "macosx") {
+    // newmailalert.xhtml doesn't work on macOS.
+    return;
+  }
+
+  // We expect the newmailalert.xhtml window...
+  plan_for_new_window("alert:alert");
+  await make_gradually_newer_sets_in_folder([gFolder], [{ count: 2 }]);
+  let controller = wait_for_new_window("alert:alert");
+  plan_for_window_close(controller);
+  wait_for_window_close();
+});
