@@ -14,7 +14,8 @@ var gDirectory = null;
 var kPersonalAddressbookURI = "jsaddrbook://abook.sqlite";
 var kCollectedAddressbookURI = "jsaddrbook://history.sqlite";
 var kAllDirectoryRoot = "moz-abdirectory://";
-var kJSDirectory = 101; // defined in nsDirPrefs.h
+
+window.addEventListener("load", abNameOnLoad);
 
 function abNameOnLoad() {
   // Get the document elements.
@@ -65,7 +66,10 @@ function abNameOKButton(event) {
   let newDirName = gNameInput.value.trim();
 
   // Do not allow an already existing name.
-  if (MailServices.ab.directoryNameExists(newDirName)) {
+  if (
+    MailServices.ab.directoryNameExists(newDirName) &&
+    (!gDirectory || newDirName != gDirectory.dirName)
+  ) {
     const kAlertTitle = document
       .getElementById("bundle_addressBook")
       .getString("duplicateNameTitle");
@@ -85,10 +89,13 @@ function abNameOKButton(event) {
     let dirPrefId = MailServices.ab.newAddressBook(
       newDirName,
       "",
-      kJSDirectory
+      Ci.nsIAbManager.JS_DIRECTORY_TYPE
     );
     let directory = MailServices.ab.getDirectoryFromId(dirPrefId);
-    window.arguments[0].newDirectoryURI = directory.URI;
+    window.arguments[0].newDirectoryUID = directory.UID;
+    if ("onNewDirectory" in window.arguments[0]) {
+      window.arguments[0].onNewDirectory(directory);
+    }
   }
 }
 
