@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -10,40 +9,34 @@ var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
 
+window.addEventListener("DOMContentLoaded", onInitEditDirectories);
+
 // Listener to refresh the list items if something changes. In all these
 // cases we just rebuild the list as it is easier than searching/adding in the
 // correct places an would be an infrequent operation.
 var gAddressBookAbListener = {
-  onItemAdded(parentDir, item) {
-    try {
-      item.QueryInterface(Ci.nsIAbDirectory);
-    } catch (ex) {
-      return;
+  QueryInterface: ChromeUtils.generateQI([
+    "nsIObserver",
+    "nsISupportsWeakReference",
+  ]),
+
+  init() {
+    for (let topic of [
+      "addrbook-directory-created",
+      "addrbook-directory-updated",
+      "addrbook-directory-deleted",
+    ]) {
+      Services.obs.addObserver(this, topic, true);
     }
-    fillDirectoryList(item);
   },
-  onItemRemoved(parentDir, item) {
-    try {
-      item.QueryInterface(Ci.nsIAbDirectory);
-    } catch (ex) {
-      return;
-    }
-    fillDirectoryList();
-  },
-  onItemPropertyChanged(item, property, oldValue, newValue) {
-    try {
-      item.QueryInterface(Ci.nsIAbDirectory);
-    } catch (ex) {
-      return;
-    }
-    fillDirectoryList(item);
+
+  observe(subject, topic, data) {
+    subject.QueryInterface(Ci.nsIAbDirectory);
+    fillDirectoryList(subject);
   },
 };
 
 function onInitEditDirectories() {
-  // For AbDeleteDirectory in abCommon.js
-  gAddressBookBundle = document.getElementById("bundle_addressBook");
-
   // If the pref is locked disable the "Add" button
   if (Services.prefs.prefIsLocked("ldap_2.disable_button_add")) {
     document.getElementById("addButton").setAttribute("disabled", true);
@@ -52,18 +45,8 @@ function onInitEditDirectories() {
   // Fill out the directory list
   fillDirectoryList();
 
-  const nsIAbListener = Ci.nsIAbListener;
   // Add a listener so we can update correctly if the list should change
-  MailServices.ab.addAddressBookListener(
-    gAddressBookAbListener,
-    nsIAbListener.itemAdded |
-      nsIAbListener.directoryRemoved |
-      nsIAbListener.itemChanged
-  );
-}
-
-function onUninitEditDirectories() {
-  MailServices.ab.removeAddressBookListener(gAddressBookAbListener);
+  gAddressBookAbListener.init();
 }
 
 function fillDirectoryList(aItem = null) {
@@ -77,7 +60,7 @@ function fillDirectoryList(aItem = null) {
   // Init the address book list
   let holdingArray = [];
   for (let ab of MailServices.ab.directories) {
-    if (ab instanceof Ci.nsIAbDirectory && ab.isRemote) {
+    if (ab.isRemote) {
       holdingArray.push(ab);
     }
   }
@@ -141,6 +124,14 @@ function dblClickDirectory(event) {
   editDirectory();
 }
 
+function addDirectory() {
+  window.browsingContext.topChromeWindow.openDialog(
+    "chrome://messenger/content/addressbook/pref-directory-add.xhtml",
+    "",
+    "chrome,resizable=no,centerscreen"
+  );
+}
+
 function editDirectory() {
   var abList = document.getElementById("directoriesList");
 
@@ -148,7 +139,7 @@ function editDirectory() {
     let abURI = abList.value;
     let ab = MailServices.ab.getDirectory(abURI);
 
-    window.docShell.rootTreeItem.domWindow.openDialog(
+    window.browsingContext.topChromeWindow.openDialog(
       ab.propertiesChromeURI,
       "editDirectory",
       "chrome,modal=yes,resizable=no",
@@ -157,10 +148,44 @@ function editDirectory() {
   }
 }
 
-function removeDirectory() {
-  var abList = document.getElementById("directoriesList");
+async function removeDirectory() {
+  let abList = document.getElementById("directoriesList");
 
-  if (abList && abList.selectedItem) {
-    AbDeleteDirectory(abList.value);
+  if (!abList.selectedItem) {
+    return;
+  }
+
+  let directory = GetDirectoryFromURI(abList.value);
+  if (
+    !directory ||
+    ["ldap_2.servers.history", "ldap_2.servers.pab"].includes(
+      directory.dirPrefId
+    )
+  ) {
+    return;
+  }
+
+  let action = "delete-book";
+  if (directory.isMailList) {
+    action = "delete-lists";
+  } else if (
+    [
+      Ci.nsIAbManager.CARDDAV_DIRECTORY_TYPE,
+      Ci.nsIAbManager.LDAP_DIRECTORY_TYPE,
+    ].includes(directory.dirType)
+  ) {
+    action = "remove-remote-book";
+  }
+
+  let [title, message] = await document.l10n.formatValues([
+    { id: `about-addressbook-confirm-${action}-title`, args: { count: 1 } },
+    {
+      id: `about-addressbook-confirm-${action}`,
+      args: { name: directory.dirName, count: 1 },
+    },
+  ]);
+
+  if (Services.prompt.confirm(window, title, message)) {
+    MailServices.ab.deleteAddressBook(directory.URI);
   }
 }
