@@ -15,8 +15,6 @@
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsMemory.h"
-#include "nsIMutableArray.h"
-#include "nsArrayUtils.h"
 #include "mozITXTToHTMLConv.h"
 #include "nsIAbManager.h"
 #include "nsIUUIDGenerator.h"
@@ -24,7 +22,6 @@
 #include "nsVariant.h"
 #include "nsIProperty.h"
 #include "nsCOMArray.h"
-#include "nsArrayEnumerator.h"
 #include "prmem.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Services.h"
@@ -98,8 +95,6 @@ static const AppendItem CHAT_ATTRS_ARRAY[] = {
 
 nsAbCardProperty::nsAbCardProperty() : m_IsMailList(false) {
   // Initialize some default properties
-  SetPropertyAsUint32(kPreferMailFormatProperty,
-                      nsIAbPreferMailFormat::unknown);
   SetPropertyAsUint32(kPopularityIndexProperty, 0);
   // Uninitialized...
   SetPropertyAsUint32(kLastModifiedDateProperty, 0);
@@ -109,37 +104,13 @@ nsAbCardProperty::~nsAbCardProperty(void) {}
 
 NS_IMPL_ISUPPORTS(nsAbCardProperty, nsIAbCard)
 
-NS_IMETHODIMP nsAbCardProperty::GetUuid(nsACString& uuid) {
-  // If we have indeterminate sub-ids, return an empty uuid.
-  if (m_directoryId.IsEmpty() || m_localId.IsEmpty()) {
-    uuid.Truncate();
-    return NS_OK;
-  }
-
-  nsresult rv;
-  nsCOMPtr<nsIAbManager> manager = do_GetService(NS_ABMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return manager->GenerateUUID(m_directoryId, m_localId, uuid);
-}
-
-NS_IMETHODIMP nsAbCardProperty::GetDirectoryId(nsACString& dirId) {
-  dirId = m_directoryId;
+NS_IMETHODIMP nsAbCardProperty::GetDirectoryUID(nsACString& dirUID) {
+  dirUID = m_directoryUID;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsAbCardProperty::SetDirectoryId(const nsACString& aDirId) {
-  m_directoryId = aDirId;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsAbCardProperty::GetLocalId(nsACString& localId) {
-  localId = m_localId;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsAbCardProperty::SetLocalId(const nsACString& aLocalId) {
-  m_localId = aLocalId;
+NS_IMETHODIMP nsAbCardProperty::SetDirectoryUID(const nsACString& aDirUID) {
+  m_directoryUID = aDirUID;
   return NS_OK;
 }
 
@@ -202,13 +173,14 @@ nsAbSimpleProperty::GetValue(nsIVariant** aValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsAbCardProperty::GetProperties(nsISimpleEnumerator** props) {
-  nsCOMArray<nsIProperty> propertyArray(m_properties.Count());
+NS_IMETHODIMP nsAbCardProperty::GetProperties(
+    nsTArray<RefPtr<nsIProperty>>& props) {
+  props.ClearAndRetainStorage();
+  props.SetCapacity(m_properties.Count());
   for (auto iter = m_properties.Iter(); !iter.Done(); iter.Next()) {
-    propertyArray.AppendObject(
-        new nsAbSimpleProperty(iter.Key(), iter.UserData()));
+    props.AppendElement(new nsAbSimpleProperty(iter.Key(), iter.UserData()));
   }
-  return NS_NewArrayEnumerator(props, propertyArray, NS_GET_IID(nsIProperty));
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsAbCardProperty::GetProperty(const nsACString& name,
@@ -260,7 +232,7 @@ NS_IMETHODIMP nsAbCardProperty::GetPropertyAsBool(const char* name,
 
 NS_IMETHODIMP nsAbCardProperty::SetProperty(const nsACString& name,
                                             nsIVariant* value) {
-  m_properties.Put(name, value);
+  m_properties.InsertOrUpdate(name, value);
   return NS_OK;
 }
 
@@ -270,7 +242,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsAString(const char* name,
 
   nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsAString(value);
-  m_properties.Put(nsDependentCString(name), variant);
+  m_properties.InsertOrUpdate(nsDependentCString(name), variant);
   return NS_OK;
 }
 
@@ -280,7 +252,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsAUTF8String(
 
   nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsAUTF8String(value);
-  m_properties.Put(nsDependentCString(name), variant);
+  m_properties.InsertOrUpdate(nsDependentCString(name), variant);
   return NS_OK;
 }
 
@@ -290,7 +262,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsUint32(const char* name,
 
   nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsUint32(value);
-  m_properties.Put(nsDependentCString(name), variant);
+  m_properties.InsertOrUpdate(nsDependentCString(name), variant);
   return NS_OK;
 }
 
@@ -300,7 +272,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsBool(const char* name,
 
   nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsBool(value);
-  m_properties.Put(nsDependentCString(name), variant);
+  m_properties.InsertOrUpdate(nsDependentCString(name), variant);
   return NS_OK;
 }
 
@@ -309,6 +281,17 @@ NS_IMETHODIMP nsAbCardProperty::DeleteProperty(const nsACString& name) {
   return NS_OK;
 }
 
+NS_IMETHODIMP nsAbCardProperty::GetSupportsVCard(bool* aSupportsVCard) {
+  *aSupportsVCard = false;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsAbCardProperty::GetVCardProperties(
+    JS::MutableHandle<JS::Value> properties) {
+  properties.setNull();
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAbCardProperty::GetUID(nsACString& uid) {
   nsAutoString aString;
   nsresult rv = GetPropertyAsAString(kUIDProperty, aString);
@@ -343,22 +326,22 @@ NS_IMETHODIMP nsAbCardProperty::SetUID(const nsACString& aUID) {
   rv = SetPropertyAsAString(kUIDProperty, NS_ConvertUTF8toUTF16(aUID));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (m_directoryId.IsEmpty()) {
+  if (m_directoryUID.IsEmpty()) {
+    // This card's not in a directory.
     return NS_OK;
   }
 
-  int ampIndex = m_directoryId.FindChar('&');
-  const nsACString& directoryId = Substring(m_directoryId, 0, ampIndex);
-
   nsCOMPtr<nsIAbManager> abManager =
       do_GetService(NS_ABMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIAbDirectory> directory = nullptr;
-  rv = abManager->GetDirectoryFromId(directoryId, getter_AddRefs(directory));
+  rv =
+      abManager->GetDirectoryFromUID(m_directoryUID, getter_AddRefs(directory));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!directory) {
+    // This card claims to be in a directory, but we can't find it.
     return NS_OK;
   }
 
@@ -367,8 +350,11 @@ NS_IMETHODIMP nsAbCardProperty::SetUID(const nsACString& aUID) {
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (readOnly) {
-    return NS_ERROR_FAILURE;
+    // The directory is read-only.
+    return NS_OK;
   }
+
+  // Save the new UID so we can use it again in the future.
   return directory->ModifyCard(this);
 }
 
@@ -424,6 +410,26 @@ NS_IMETHODIMP nsAbCardProperty::SetPrimaryEmail(const nsAString& aString) {
   return SetPropertyAsAString(kPriEmailProperty, aString);
 }
 
+NS_IMETHODIMP nsAbCardProperty::GetEmailAddresses(
+    nsTArray<nsString>& aEmailAddresses) {
+  aEmailAddresses.Clear();
+
+  nsresult rv;
+  nsString emailAddress;
+
+  rv = GetPropertyAsAString(kPriEmailProperty, emailAddress);
+  if (rv != NS_ERROR_NOT_AVAILABLE && !emailAddress.IsEmpty()) {
+    aEmailAddresses.AppendElement(emailAddress);
+  }
+
+  rv = GetPropertyAsAString(k2ndEmailProperty, emailAddress);
+  if (rv != NS_ERROR_NOT_AVAILABLE && !emailAddress.IsEmpty()) {
+    aEmailAddresses.AppendElement(emailAddress);
+  }
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAbCardProperty::HasEmailAddress(const nsACString& aEmailAddress,
                                                 bool* aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
@@ -446,24 +452,21 @@ NS_IMETHODIMP nsAbCardProperty::HasEmailAddress(const nsACString& aEmailAddress,
   return NS_OK;
 }
 
+NS_IMETHODIMP nsAbCardProperty::GetPhotoURL(nsAString& aPhotoURL) {
+  aPhotoURL.Truncate();
+  return NS_OK;
+}
+
 // This function may be overridden by derived classes for
 // nsAb*Card specific implementations.
 NS_IMETHODIMP nsAbCardProperty::Copy(nsIAbCard* srcCard) {
   NS_ENSURE_ARG_POINTER(srcCard);
 
-  nsCOMPtr<nsISimpleEnumerator> properties;
-  nsresult rv = srcCard->GetProperties(getter_AddRefs(properties));
+  nsTArray<RefPtr<nsIProperty>> properties;
+  nsresult rv = srcCard->GetProperties(properties);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  bool hasMore;
-  nsCOMPtr<nsISupports> result;
-  while (NS_SUCCEEDED(rv = properties->HasMoreElements(&hasMore)) && hasMore) {
-    rv = properties->GetNext(getter_AddRefs(result));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIProperty> property = do_QueryInterface(result, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-
+  for (nsIProperty* property : properties) {
     nsAutoString name;
     property->GetName(name);
     nsCOMPtr<nsIVariant> value;
@@ -471,7 +474,6 @@ NS_IMETHODIMP nsAbCardProperty::Copy(nsIAbCard* srcCard) {
 
     SetProperty(NS_ConvertUTF16toUTF8(name), value);
   }
-  NS_ENSURE_SUCCESS(rv, rv);
 
   bool isMailList;
   srcCard->GetIsMailList(&isMailList);
@@ -517,8 +519,12 @@ nsresult nsAbCardProperty::ConvertToEscapedVCard(nsACString& aResult) {
       do_GetService(NS_MSGVCARDSERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIAbCard> cardFromVCard;
-  return vCardService->AbCardToEscapedVCard(this, aResult);
+  nsAutoString result;
+  rv = vCardService->AbCardToEscapedVCard(this, result);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  aResult = NS_ConvertUTF16toUTF8(result);
+  return NS_OK;
 }
 
 nsresult nsAbCardProperty::ConvertToBase64EncodedXML(nsACString& result) {
@@ -528,7 +534,7 @@ nsresult nsAbCardProperty::ConvertToBase64EncodedXML(nsACString& result) {
   xmlStr.AppendLiteral(
       "<?xml version=\"1.0\"?>\n"
       "<?xml-stylesheet type=\"text/css\" "
-      "href=\"chrome://messagebody/content/addressbook/print.css\"?>\n"
+      "href=\"chrome://messagebody/skin/abPrint.css\"?>\n"
       "<directory>\n");
 
   // Get Address Book string and set it as title of XML document
@@ -625,19 +631,19 @@ nsresult nsAbCardProperty::ConvertToXMLPrintData(nsAString& aXMLSubstr) {
 
   rv = AppendSection(PHONE_ATTRS_ARRAY,
                      sizeof(PHONE_ATTRS_ARRAY) / sizeof(AppendItem),
-                     NS_LITERAL_STRING("headingPhone"), bundle, conv, xmlStr);
+                     u"headingPhone"_ns, bundle, conv, xmlStr);
 
   if (!m_IsMailList) {
     rv = AppendSection(CUSTOM_ATTRS_ARRAY,
                        sizeof(CUSTOM_ATTRS_ARRAY) / sizeof(AppendItem),
-                       NS_LITERAL_STRING("headingOther"), bundle, conv, xmlStr);
+                       u"headingOther"_ns, bundle, conv, xmlStr);
     rv = AppendSection(CHAT_ATTRS_ARRAY,
                        sizeof(CHAT_ATTRS_ARRAY) / sizeof(AppendItem),
-                       NS_LITERAL_STRING("headingChat"), bundle, conv, xmlStr);
+                       u"headingChat"_ns, bundle, conv, xmlStr);
   } else {
-    rv = AppendSection(
-        CUSTOM_ATTRS_ARRAY, sizeof(CUSTOM_ATTRS_ARRAY) / sizeof(AppendItem),
-        NS_LITERAL_STRING("headingDescription"), bundle, conv, xmlStr);
+    rv = AppendSection(CUSTOM_ATTRS_ARRAY,
+                       sizeof(CUSTOM_ATTRS_ARRAY) / sizeof(AppendItem),
+                       u"headingDescription"_ns, bundle, conv, xmlStr);
 
     xmlStr.AppendLiteral("<section><sectiontitle>");
 
@@ -656,24 +662,14 @@ nsresult nsAbCardProperty::ConvertToXMLPrintData(nsAString& aXMLSubstr) {
     rv = abManager->GetDirectory(m_MailListURI, getter_AddRefs(mailList));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsISimpleEnumerator> mailListAddresses;
-    rv = mailList->GetChildCards(getter_AddRefs(mailListAddresses));
+    nsTArray<RefPtr<nsIAbCard>> mailListAddresses;
+    rv = mailList->GetChildCards(mailListAddresses);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    bool hasMore = false;
-    nsCOMPtr<nsISupports> support;
-    nsCOMPtr<nsIAbCard> listCard;
-    nsAutoString displayName;
-    nsAutoString primaryEmail;
-    while (NS_SUCCEEDED(mailListAddresses->HasMoreElements(&hasMore)) &&
-           hasMore) {
-      rv = mailListAddresses->GetNext(getter_AddRefs(support));
-      NS_ENSURE_SUCCESS(rv, rv);
-      listCard = do_QueryInterface(support, &rv);
-      NS_ENSURE_SUCCESS(rv, rv);
-
+    for (nsIAbCard* listCard : mailListAddresses) {
       xmlStr.AppendLiteral("<PrimaryEmail>\n");
 
+      nsAutoString displayName;
       rv = listCard->GetDisplayName(displayName);
       NS_ENSURE_SUCCESS(rv, rv);
 
@@ -685,12 +681,14 @@ nsresult nsAbCardProperty::ConvertToXMLPrintData(nsAString& aXMLSubstr) {
 
       xmlStr.AppendLiteral(" &lt;");
 
+      nsAutoString primaryEmail;
       listCard->GetPrimaryEmail(primaryEmail);
 
       // use ScanTXT to convert < > & to safe values.
-      rv = conv->ScanTXT(primaryEmail, mozITXTToHTMLConv::kEntities, safeText);
+      nsString safeText2;
+      rv = conv->ScanTXT(primaryEmail, mozITXTToHTMLConv::kEntities, safeText2);
       NS_ENSURE_SUCCESS(rv, rv);
-      xmlStr.Append(safeText);
+      xmlStr.Append(safeText2);
 
       xmlStr.AppendLiteral("&gt;</PrimaryEmail>\n");
     }
@@ -701,10 +699,10 @@ nsresult nsAbCardProperty::ConvertToXMLPrintData(nsAString& aXMLSubstr) {
 
   rv = AppendSection(HOME_ATTRS_ARRAY,
                      sizeof(HOME_ATTRS_ARRAY) / sizeof(AppendItem),
-                     NS_LITERAL_STRING("headingHome"), bundle, conv, xmlStr);
+                     u"headingHome"_ns, bundle, conv, xmlStr);
   rv = AppendSection(WORK_ATTRS_ARRAY,
                      sizeof(WORK_ATTRS_ARRAY) / sizeof(AppendItem),
-                     NS_LITERAL_STRING("headingWork"), bundle, conv, xmlStr);
+                     u"headingWork"_ns, bundle, conv, xmlStr);
 
   xmlStr.AppendLiteral("</td></tr></table>");
 
