@@ -8,7 +8,6 @@
 #include "nsAbDirectoryQueryProxy.h"
 #include "nsAbUtils.h"
 #include "nsAbBooleanExpression.h"
-#include "nsArrayUtils.h"
 #include "nsComponentManagerUtils.h"
 #include "nsString.h"
 #include "nsUnicharUtils.h"
@@ -47,7 +46,7 @@ NS_IMETHODIMP nsAbDirectoryQuerySimpleBooleanExpression::SetOperation(
   return NS_OK;
 }
 
-/* attribute nsIArray expressions; */
+/* attribute Array<nsISupports> expressions; */
 NS_IMETHODIMP nsAbDirectoryQuerySimpleBooleanExpression::GetExpressions(
     nsTArray<RefPtr<nsISupports>>& aExpressions) {
   aExpressions = mExpressions.Clone();
@@ -200,8 +199,9 @@ NS_IMETHODIMP nsAbDirectoryQuery::DoQuery(
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = query(aDirectory, expression, listener, doSubDirectories, &resultLimit);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = NS_FAILED(rv) ? queryError(listener) : queryFinished(listener);
+  rv = listener->OnSearchFinished(rv, true, nullptr, ""_ns);
 
   *_retval = 0;
   return rv;
@@ -220,7 +220,7 @@ nsresult nsAbDirectoryQuery::query(nsIAbDirectory* directory,
   nsresult rv = queryCards(directory, expression, listener, resultLimit);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (doSubDirectories && resultLimit != 0) {
+  if (*resultLimit != 0 && doSubDirectories) {
     rv = queryChildren(directory, expression, listener, doSubDirectories,
                        resultLimit);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -234,22 +234,11 @@ nsresult nsAbDirectoryQuery::queryChildren(nsIAbDirectory* directory,
                                            nsIAbDirSearchListener* listener,
                                            bool doSubDirectories,
                                            int32_t* resultLimit) {
-  nsresult rv = NS_OK;
-
-  nsCOMPtr<nsISimpleEnumerator> subDirectories;
-  rv = directory->GetChildNodes(getter_AddRefs(subDirectories));
+  nsTArray<RefPtr<nsIAbDirectory>> subDirectories;
+  nsresult rv = directory->GetChildNodes(subDirectories);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  bool hasMore;
-  while (NS_SUCCEEDED(rv = subDirectories->HasMoreElements(&hasMore)) &&
-         hasMore) {
-    nsCOMPtr<nsISupports> item;
-    rv = subDirectories->GetNext(getter_AddRefs(item));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIAbDirectory> subDirectory(do_QueryInterface(item, &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
-
+  for (nsIAbDirectory* subDirectory : subDirectories) {
     rv = query(subDirectory, expression, listener, doSubDirectories,
                resultLimit);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -261,28 +250,14 @@ nsresult nsAbDirectoryQuery::queryCards(nsIAbDirectory* directory,
                                         nsIAbBooleanExpression* expression,
                                         nsIAbDirSearchListener* listener,
                                         int32_t* resultLimit) {
-  nsresult rv = NS_OK;
-
-  nsCOMPtr<nsISimpleEnumerator> cards;
-  rv = directory->GetChildCards(getter_AddRefs(cards));
-  if (NS_FAILED(rv)) {
-    if (rv != NS_ERROR_NOT_IMPLEMENTED)
-      NS_ENSURE_SUCCESS(rv, rv);
-    else
-      return NS_OK;
+  nsTArray<RefPtr<nsIAbCard>> cards;
+  nsresult rv = directory->GetChildCards(cards);
+  if (rv == NS_ERROR_NOT_IMPLEMENTED) {
+    return NS_OK;
   }
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  if (!cards) return NS_OK;
-
-  bool more;
-  while (NS_SUCCEEDED(cards->HasMoreElements(&more)) && more) {
-    nsCOMPtr<nsISupports> item;
-    rv = cards->GetNext(getter_AddRefs(item));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIAbCard> card(do_QueryInterface(item, &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
-
+  for (nsIAbCard* card : cards) {
     rv = matchCard(card, expression, listener, resultLimit);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -302,7 +277,7 @@ nsresult nsAbDirectoryQuery::matchCard(nsIAbCard* card,
 
   if (matchFound) {
     (*resultLimit)--;
-    rv = queryMatch(card, listener);
+    rv = listener->OnSearchFoundCard(card);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -445,18 +420,3 @@ nsresult nsAbDirectoryQuery::matchCardCondition(
 
   return rv;
 }
-
-nsresult nsAbDirectoryQuery::queryMatch(nsIAbCard* card,
-                                        nsIAbDirSearchListener* listener) {
-  return listener->OnSearchFoundCard(card);
-}
-
-nsresult nsAbDirectoryQuery::queryFinished(nsIAbDirSearchListener* listener) {
-  return listener->OnSearchFinished(
-      nsIAbDirectoryQueryResultListener::queryResultComplete, EmptyString());
-}
-
-nsresult nsAbDirectoryQuery::queryError(nsIAbDirSearchListener* listener) {
-  return listener->OnSearchFinished(
-      nsIAbDirectoryQueryResultListener::queryResultError, EmptyString());
-}
