@@ -51,22 +51,20 @@ class nsIAbOSXDirectory : public nsISupports {
 NS_DEFINE_STATIC_IID_ACCESSOR(nsIAbOSXDirectory, NS_IABOSXDIRECTORY_IID)
 
 class nsAbOSXDirectory final : public nsAbDirProperty,
-                               public nsIAbDirSearchListener,
                                public nsIAbOSXDirectory {
  public:
   nsAbOSXDirectory();
 
   NS_DECL_ISUPPORTS_INHERITED
-  NS_DECL_NSIABDIRSEARCHLISTENER
 
   // nsIAbOSXDirectory method
   NS_IMETHOD Init(const char* aUri) override;
 
   // nsAbDirProperty methods
   NS_IMETHOD GetReadOnly(bool* aReadOnly) override;
-  NS_IMETHOD GetChildCards(nsISimpleEnumerator** aCards) override;
-  NS_IMETHOD GetChildNodes(nsISimpleEnumerator** aNodes) override;
-  NS_IMETHOD GetIsQuery(bool* aResult) override;
+  NS_IMETHOD GetChildCardCount(uint32_t* aCount) override;
+  NS_IMETHOD GetChildCards(nsTArray<RefPtr<nsIAbCard>>& result) override;
+  NS_IMETHOD GetChildNodes(nsTArray<RefPtr<nsIAbDirectory>>& result) override;
   NS_IMETHOD HasCard(nsIAbCard* aCard, bool* aHasCard) override;
   NS_IMETHOD HasDirectory(nsIAbDirectory* aDirectory,
                           bool* aHasDirectory) override;
@@ -74,12 +72,12 @@ class nsAbOSXDirectory final : public nsAbDirProperty,
   NS_IMETHOD GetCardFromProperty(const char* aProperty,
                                  const nsACString& aValue, bool caseSensitive,
                                  nsIAbCard** aResult) override;
-  NS_IMETHOD GetCardsFromProperty(const char* aProperty,
-                                  const nsACString& aValue, bool aCaseSensitive,
-                                  nsISimpleEnumerator** aResult) override;
+  NS_IMETHOD GetCardsFromProperty(
+      const char* aProperty, const nsACString& aValue, bool aCaseSensitive,
+      nsTArray<RefPtr<nsIAbCard>>& aResult) override;
   NS_IMETHOD CardForEmailAddress(const nsACString& aEmailAddress,
                                  nsIAbCard** aResult) override;
-  NS_IMETHOD Search(const nsAString& query,
+  NS_IMETHOD Search(const nsAString& query, const nsAString& searchString,
                     nsIAbDirSearchListener* listener) override;
 
   // nsIAbOSXDirectory
