@@ -7,7 +7,6 @@
 #include "nsAbOSXCard.h"
 #include "nsAbOSXUtils.h"
 #include "nsAbQueryStringToExpression.h"
-#include "nsArrayEnumerator.h"
 #include "nsCOMArray.h"
 #include "nsEnumeratorUtils.h"
 #include "nsIAbDirectoryQueryProxy.h"
@@ -45,7 +44,7 @@ static nsresult GetOrCreateGroup(NSString* aUid, nsIAbDirectory** aResult) {
 }
 
 static nsresult GetCard(ABRecord* aRecord, nsIAbCard** aResult, nsIAbOSXDirectory* osxDirectory) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   NSString* uid = [aRecord uniqueId];
   NS_ASSERTION(uid, "No UID for card!.");
@@ -63,11 +62,11 @@ static nsresult GetCard(ABRecord* aRecord, nsIAbCard** aResult, nsIAbOSXDirector
   NS_IF_ADDREF(*aResult = card);
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 static nsresult CreateCard(ABRecord* aRecord, nsIAbCard** aResult) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   NSString* uid = [aRecord uniqueId];
   NS_ASSERTION(uid, "No UID for card!.");
@@ -89,11 +88,11 @@ static nsresult CreateCard(ABRecord* aRecord, nsIAbCard** aResult) {
   NS_IF_ADDREF(*aResult = card);
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 static nsresult Sync(NSString* aUid) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   ABAddressBook* addressBook = [ABAddressBook sharedAddressBook];
   ABRecord* card = [addressBook recordForUniqueId:aUid];
@@ -113,7 +112,7 @@ static nsresult Sync(NSString* aUid) {
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIAbDirectory> directory;
-    rv = abManager->GetDirectory(NS_LITERAL_CSTRING(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
+    rv = abManager->GetDirectory(nsLiteralCString(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
                                  getter_AddRefs(directory));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -128,7 +127,7 @@ static nsresult Sync(NSString* aUid) {
   }
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 @interface ABChangedMonitor : NSObject
@@ -147,7 +146,7 @@ static nsresult Sync(NSString* aUid) {
     NS_ENSURE_SUCCESS_VOID(rv);
 
     nsCOMPtr<nsIAbDirectory> directory;
-    rv = abManager->GetDirectory(NS_LITERAL_CSTRING(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
+    rv = abManager->GetDirectory(nsLiteralCString(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
                                  getter_AddRefs(directory));
     NS_ENSURE_SUCCESS_VOID(rv);
 
@@ -190,7 +189,7 @@ static nsresult Sync(NSString* aUid) {
     NS_ENSURE_SUCCESS_VOID(rv);
 
     nsCOMPtr<nsIAbDirectory> directory;
-    rv = abManager->GetDirectory(NS_LITERAL_CSTRING(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
+    rv = abManager->GetDirectory(nsLiteralCString(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
                                  getter_AddRefs(directory));
     NS_ENSURE_SUCCESS_VOID(rv);
 
@@ -228,12 +227,11 @@ nsAbOSXDirectory::~nsAbOSXDirectory() {
   }
 }
 
-NS_IMPL_ISUPPORTS_INHERITED(nsAbOSXDirectory, nsAbDirProperty, nsIAbOSXDirectory,
-                            nsIAbDirSearchListener)
+NS_IMPL_ISUPPORTS_INHERITED(nsAbOSXDirectory, nsAbDirProperty, nsIAbOSXDirectory)
 
 NS_IMETHODIMP
 nsAbOSXDirectory::Init(const char* aUri) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   nsresult rv;
   rv = nsAbDirProperty::Init(aUri);
@@ -253,10 +251,9 @@ nsAbOSXDirectory::Init(const char* aUri) {
   nsCOMPtr<nsIMutableArray> cardList;
   bool isRootOSXDirectory = false;
 
-  if (!mIsQueryURI && mURINoQuery.Length() <= sizeof(NS_ABOSXDIRECTORY_URI_PREFIX))
+  if (mURI.Length() <= sizeof(NS_ABOSXDIRECTORY_URI_PREFIX)) {
     isRootOSXDirectory = true;
 
-  if (mIsQueryURI || isRootOSXDirectory) {
     m_DirPrefId.AssignLiteral("ldap_2.servers.osx");
 
     cards = [[addressBook people] arrayByAddingObjectsFromArray:[addressBook groups]];
@@ -268,7 +265,7 @@ nsAbOSXDirectory::Init(const char* aUri) {
 
     cardList = mCardList;
   } else {
-    nsAutoCString uid(Substring(mURINoQuery, sizeof(NS_ABOSXDIRECTORY_URI_PREFIX) - 1));
+    nsAutoCString uid(Substring(mURI, sizeof(NS_ABOSXDIRECTORY_URI_PREFIX) - 1));
     ABRecord* card = [addressBook recordForUniqueId:[NSString stringWithUTF8String:uid.get()]];
     NS_ASSERTION([card isKindOfClass:[ABGroup class]], "Huh.");
 
@@ -276,8 +273,7 @@ nsAbOSXDirectory::Init(const char* aUri) {
     AppendToString([card valueForProperty:kABGroupNameProperty], m_ListDirName);
 
     ABGroup* group = (ABGroup*)[addressBook
-        recordForUniqueId:[NSString stringWithUTF8String:nsAutoCString(Substring(mURINoQuery, 21))
-                                                             .get()]];
+        recordForUniqueId:[NSString stringWithUTF8String:nsAutoCString(Substring(mURI, 21)).get()]];
     cards = [[group members] arrayByAddingObjectsFromArray:[group subgroups]];
 
     if (!m_AddressList)
@@ -289,9 +285,6 @@ nsAbOSXDirectory::Init(const char* aUri) {
     cardList = m_AddressList;
   }
 
-  nsAutoCString ourUuid;
-  GetUuid(ourUuid);
-
   unsigned int nbCards = [cards count];
   nsCOMPtr<nsIAbCard> card;
   nsCOMPtr<nsIAbManager> abManager = do_GetService(NS_ABMANAGER_CONTRACTID, &rv);
@@ -316,10 +309,9 @@ nsAbOSXDirectory::Init(const char* aUri) {
 
     NS_ENSURE_SUCCESS(rv, rv);
 
-    // If we're not a query directory, we're going to want to
-    // tell the AB Manager that we've added some cards so that they
-    // show up in the address book views.
-    if (!mIsQueryURI) AssertCard(abManager, card);
+    // We're going to want to tell the AB Manager that we've added some cards
+    // so that they show up in the address book views.
+    AssertCard(abManager, card);
   }
 
   if (isRootOSXDirectory) {
@@ -328,7 +320,7 @@ nsAbOSXDirectory::Init(const char* aUri) {
 
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 NS_IMETHODIMP
@@ -367,7 +359,6 @@ static bool CheckRedundantCards(nsIAbManager* aManager, nsIAbDirectory* aDirecto
   }
 
   if (i == count) {
-    aManager->NotifyDirectoryItemDeleted(aDirectory, aCard);
     return true;
   }
 
@@ -382,7 +373,7 @@ nsresult nsAbOSXDirectory::GetRootOSXDirectory(nsIAbOSXDirectory** aResult) {
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIAbDirectory> directory;
-    rv = abManager->GetDirectory(NS_LITERAL_CSTRING(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
+    rv = abManager->GetDirectory(nsLiteralCString(NS_ABOSXDIRECTORY_URI_PREFIX "/"),
                                  getter_AddRefs(directory));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -396,16 +387,12 @@ nsresult nsAbOSXDirectory::GetRootOSXDirectory(nsIAbOSXDirectory** aResult) {
 }
 
 nsresult nsAbOSXDirectory::Update() {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   nsresult rv;
   nsCOMPtr<nsIAbManager> abManager = do_GetService(NS_ABMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (mIsQueryURI) {
-    return NS_OK;
-  }
-
   ABAddressBook* addressBook = [ABAddressBook sharedAddressBook];
   // Due to the horrible way the address book code works wrt mailing lists
   // we have to use a different list depending on what we are. This pointer
@@ -414,8 +401,7 @@ nsresult nsAbOSXDirectory::Update() {
   NSArray *groups, *cards;
   if (m_IsMailList) {
     ABGroup* group = (ABGroup*)[addressBook
-        recordForUniqueId:[NSString stringWithUTF8String:nsAutoCString(Substring(mURINoQuery, 21))
-                                                             .get()]];
+        recordForUniqueId:[NSString stringWithUTF8String:nsAutoCString(Substring(mURI, 21)).get()]];
     groups = nil;
     cards = [[group members] arrayByAddingObjectsFromArray:[group subgroups]];
 
@@ -465,15 +451,11 @@ nsresult nsAbOSXDirectory::Update() {
   }
 
   card = (ABRecord*)[addressBook
-      recordForUniqueId:[NSString
-                            stringWithUTF8String:nsAutoCString(Substring(mURINoQuery, 21)).get()]];
+      recordForUniqueId:[NSString stringWithUTF8String:nsAutoCString(Substring(mURI, 21)).get()]];
   NSString* stringValue = [card valueForProperty:kABGroupNameProperty];
   if (![stringValue isEqualToString:WrapString(m_ListDirName)]) {
     nsAutoString oldValue(m_ListDirName);
     AssignToString(stringValue, m_ListDirName);
-    nsCOMPtr<nsISupports> supports = do_QueryInterface(static_cast<nsIAbDirectory*>(this), &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    abManager->NotifyItemPropertyChanged(supports, "DirName", oldValue, m_ListDirName);
   }
 
   if (groups) {
@@ -519,14 +501,14 @@ nsresult nsAbOSXDirectory::Update() {
 
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 nsresult nsAbOSXDirectory::AssertChildNodes() {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  // Queries and mailing lists can't have childnodes.
-  if (mIsQueryURI || m_IsMailList) {
+  // Mailing lists can't have childnodes.
+  if (m_IsMailList) {
     return NS_OK;
   }
 
@@ -554,7 +536,7 @@ nsresult nsAbOSXDirectory::AssertChildNodes() {
 
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 nsresult nsAbOSXDirectory::AssertDirectory(nsIAbManager* aManager, nsIAbDirectory* aDirectory) {
@@ -572,13 +554,13 @@ nsresult nsAbOSXDirectory::AssertDirectory(nsIAbManager* aManager, nsIAbDirector
   rv = m_AddressList->AppendElement(aDirectory);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return aManager->NotifyDirectoryItemAdded(this, aDirectory);
+  return NS_OK;
 }
 
 nsresult nsAbOSXDirectory::AssertCard(nsIAbManager* aManager, nsIAbCard* aCard) {
-  nsAutoCString ourUuid;
-  GetUuid(ourUuid);
-  aCard->SetDirectoryId(ourUuid);
+  nsAutoCString ourUID;
+  GetUID(ourUID);
+  aCard->SetDirectoryUID(ourUID);
 
   nsresult rv =
       m_IsMailList ? m_AddressList->AppendElement(aCard) : mCardList->AppendElement(aCard);
@@ -592,19 +574,20 @@ nsresult nsAbOSXDirectory::AssertCard(nsIAbManager* aManager, nsIAbCard* aCard)
   rv = osxCard->GetURI(uri);
 
   nsCOMPtr<nsIAbOSXCard> retrievedCard;
-  if (!mCardStore.Get(uri, getter_AddRefs(retrievedCard))) mCardStore.Put(uri, osxCard);
+  if (!mCardStore.Get(uri, getter_AddRefs(retrievedCard))) mCardStore.InsertOrUpdate(uri, osxCard);
 
-  return aManager->NotifyDirectoryItemAdded(this, aCard);
+  return NS_OK;
 }
 
 nsresult nsAbOSXDirectory::UnassertCard(nsIAbManager* aManager, nsIAbCard* aCard,
                                         nsIMutableArray* aCardList) {
-  nsresult rv;
   uint32_t pos;
+  if (NS_SUCCEEDED(aCardList->IndexOf(0, aCard, &pos))) {
+    nsresult rv = aCardList->RemoveElementAt(pos);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
-  if (NS_SUCCEEDED(aCardList->IndexOf(0, aCard, &pos))) rv = aCardList->RemoveElementAt(pos);
-
-  return aManager->NotifyDirectoryItemDeleted(this, aCard);
+  return NS_OK;
 }
 
 nsresult nsAbOSXDirectory::UnassertDirectory(nsIAbManager* aManager, nsIAbDirectory* aDirectory) {
@@ -616,37 +599,52 @@ nsresult nsAbOSXDirectory::UnassertDirectory(nsIAbManager* aManager, nsIAbDirect
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  return aManager->NotifyDirectoryItemDeleted(this, aDirectory);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsAbOSXDirectory::GetChildNodes(nsTArray<RefPtr<nsIAbDirectory>>& aNodes) {
+  aNodes.Clear();
+  // Mailing lists don't have childnodes.
+  if (m_IsMailList || !m_AddressList) {
+    return NS_OK;
+  }
+
+  uint32_t count = 0;
+  nsresult rv = m_AddressList->GetLength(&count);
+  NS_ENSURE_SUCCESS(rv, rv);
+  aNodes.SetCapacity(count);
+  for (uint32_t i = 0; i < count; i++) {
+    nsCOMPtr<nsIAbDirectory> dir = do_QueryElementAt(m_AddressList, i, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    aNodes.AppendElement(&*dir);
+  }
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAbOSXDirectory::GetChildNodes(nsISimpleEnumerator** aNodes) {
-  NS_ENSURE_ARG_POINTER(aNodes);
-
-  // Queries don't have childnodes.
-  if (mIsQueryURI || m_IsMailList || !m_AddressList) return NS_NewEmptyEnumerator(aNodes);
-
-  return NS_NewArrayEnumerator(aNodes, m_AddressList, NS_GET_IID(nsIAbDirectory));
+nsAbOSXDirectory::GetChildCardCount(uint32_t* aCount) {
+  nsIMutableArray* srcCards = m_IsMailList ? m_AddressList : mCardList;
+  return srcCards->GetLength(aCount);
 }
 
 NS_IMETHODIMP
-nsAbOSXDirectory::GetChildCards(nsISimpleEnumerator** aCards) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
-
-  NS_ENSURE_ARG_POINTER(aCards);
+nsAbOSXDirectory::GetChildCards(nsTArray<RefPtr<nsIAbCard>>& aCards) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
+  aCards.Clear();
 
   // Not a search, so just return the appropriate list of items.
-  return m_IsMailList ? NS_NewArrayEnumerator(aCards, m_AddressList, NS_GET_IID(nsIAbCard))
-                      : NS_NewArrayEnumerator(aCards, mCardList, NS_GET_IID(nsIAbCard));
-
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
-}
-
-NS_IMETHODIMP
-nsAbOSXDirectory::GetIsQuery(bool* aResult) {
-  NS_ENSURE_ARG_POINTER(aResult);
-  *aResult = mIsQueryURI;
+  nsIMutableArray* srcCards = m_IsMailList ? m_AddressList : mCardList;
+  uint32_t count = 0;
+  nsresult rv = srcCards->GetLength(&count);
+  NS_ENSURE_SUCCESS(rv, rv);
+  aCards.SetCapacity(count);
+  for (uint32_t i = 0; i < count; i++) {
+    nsCOMPtr<nsIAbCard> card = do_QueryElementAt(srcCards, i, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    aCards.AppendElement(&*card);
+  }
   return NS_OK;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 /* Recursive method that searches for a child card by URI.  If it cannot find
@@ -662,18 +660,12 @@ nsAbOSXDirectory::GetCardByUri(const nsACString& aUri, nsIAbOSXCard** aResult) {
     return NS_OK;
   }
   // Search children
-  nsCOMPtr<nsISimpleEnumerator> enumerator;
-  nsresult rv = this->GetChildNodes(getter_AddRefs(enumerator));
+  nsTArray<RefPtr<nsIAbDirectory>> children;
+  nsresult rv = this->GetChildNodes(children);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISupports> item;
-  bool hasMore = false;
-  while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore) {
-    rv = enumerator->GetNext(getter_AddRefs(item));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIAbOSXDirectory> childDirectory;
-    childDirectory = do_QueryInterface(item, &rv);
+  for (nsIAbDirectory* dir : children) {
+    nsCOMPtr<nsIAbOSXDirectory> childDirectory = do_QueryInterface(dir, &rv);
     if (NS_SUCCEEDED(rv)) {
       rv = childDirectory->GetCardByUri(aUri, getter_AddRefs(osxCard));
       if (NS_SUCCEEDED(rv)) {
@@ -721,38 +713,31 @@ nsAbOSXDirectory::GetCardFromProperty(const char* aProperty, const nsACString& a
 
 NS_IMETHODIMP
 nsAbOSXDirectory::GetCardsFromProperty(const char* aProperty, const nsACString& aValue,
-                                       bool aCaseSensitive, nsISimpleEnumerator** aResult) {
-  NS_ENSURE_ARG_POINTER(aResult);
-
-  *aResult = nullptr;
-
-  if (aValue.IsEmpty()) return NS_NewEmptyEnumerator(aResult);
+                                       bool aCaseSensitive, nsTArray<RefPtr<nsIAbCard>>& aResult) {
+  aResult.Clear();
+  if (aValue.IsEmpty()) return NS_OK;
 
   nsIMutableArray* list = m_IsMailList ? m_AddressList : mCardList;
-
-  if (!list) return NS_NewEmptyEnumerator(aResult);
+  if (!list) return NS_OK;
 
   uint32_t length;
   nsresult rv = list->GetLength(&length);
   NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMArray<nsIAbCard> resultArray;
-  nsCOMPtr<nsIAbCard> card;
-  nsAutoCString cardValue;
-
+  aResult.SetCapacity(length);
   for (uint32_t i = 0; i < length; ++i) {
-    card = do_QueryElementAt(list, i, &rv);
+    nsCOMPtr<nsIAbCard> card = do_QueryElementAt(list, i, &rv);
     if (NS_SUCCEEDED(rv)) {
+      nsAutoCString cardValue;
       rv = card->GetPropertyAsAUTF8String(aProperty, cardValue);
       if (NS_SUCCEEDED(rv)) {
         bool equal = aCaseSensitive ? cardValue.Equals(aValue)
                                     : cardValue.Equals(aValue, nsCaseInsensitiveCStringComparator);
-        if (equal) resultArray.AppendObject(card);
+        if (equal) aResult.AppendElement(&*card);
       }
     }
   }
 
-  return NS_NewArrayEnumerator(aResult, resultArray, NS_GET_IID(nsIAbCard));
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -817,36 +802,8 @@ nsAbOSXDirectory::HasDirectory(nsIAbDirectory* aDirectory, bool* aHasDirectory)
 }
 
 NS_IMETHODIMP
-nsAbOSXDirectory::OnSearchFinished(int32_t aResult, const nsAString& aErrorMsg) { return NS_OK; }
-
-NS_IMETHODIMP
-nsAbOSXDirectory::OnSearchFoundCard(nsIAbCard* aCard) {
-  nsresult rv;
-  if (!m_AddressList) {
-    m_AddressList = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  if (!mCardList) {
-    mCardList = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  rv = m_AddressList->AppendElement(aCard);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = mCardList->AppendElement(aCard);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsAutoCString ourUuid;
-  GetUuid(ourUuid);
-  aCard->SetDirectoryId(ourUuid);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsAbOSXDirectory::Search(const nsAString& query, nsIAbDirSearchListener* listener) {
+nsAbOSXDirectory::Search(const nsAString& query, const nsAString& searchString,
+                         nsIAbDirSearchListener* listener) {
   nsresult rv;
 
   nsCOMPtr<nsIAbBooleanExpression> expression;
