@@ -1,7 +1,14 @@
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
+
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
+var { TestUtils } = ChromeUtils.import(
+  "resource://testing-common/TestUtils.jsm"
+);
 var { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
@@ -17,19 +24,10 @@ registerCleanupFunction(function() {
   load("../../../resources/mailShutdown.js");
 });
 
-function promiseDirectoryRemoved() {
-  return new Promise(resolve => {
-    let observer = {
-      onItemRemoved() {
-        MailServices.ab.removeAddressBookListener(this);
-        resolve();
-      },
-    };
-    MailServices.ab.addAddressBookListener(
-      observer,
-      Ci.nsIAbListener.directoryRemoved
-    );
-  });
+function promiseDirectoryRemoved(uri) {
+  let removePromise = TestUtils.topicObserved("addrbook-directory-deleted");
+  MailServices.ab.deleteAddressBook(uri);
+  return removePromise;
 }
 
 function acObserver() {}
@@ -50,3 +48,20 @@ acObserver.prototype = {
     });
   },
 };
+
+function formatVCard(strings, ...values) {
+  let arr = [];
+  for (let str of strings) {
+    arr.push(str);
+    arr.push(values.shift());
+  }
+  let lines = arr.join("").split("\n");
+  let indent = lines[1].length - lines[1].trimLeft().length;
+  let outLines = [];
+  for (let line of lines) {
+    if (line.length > 0) {
+      outLines.push(line.substring(indent) + "\r\n");
+    }
+  }
+  return outLines.join("");
+}
