@@ -11,22 +11,29 @@
 
 // Wrap in a block to prevent leaking to window scope.
 {
-  const { FeedUtils } = ChromeUtils.import("resource:///modules/FeedUtils.jsm");
-  const {
-    allAccountsSorted,
-    folderNameCompare,
-    getSpecialFolderString,
-    getMostRecentFolders,
-  } = ChromeUtils.import("resource:///modules/folderUtils.jsm");
-  const { fixIterator, toArray } = ChromeUtils.import(
-    "resource:///modules/iteratorUtils.jsm"
+  const { Services } = ChromeUtils.import(
+    "resource://gre/modules/Services.jsm"
   );
   const { MailServices } = ChromeUtils.import(
     "resource:///modules/MailServices.jsm"
   );
-  const { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
-  const { Services } = ChromeUtils.import(
-    "resource://gre/modules/Services.jsm"
+
+  const LazyModules = {};
+
+  ChromeUtils.defineModuleGetter(
+    LazyModules,
+    "FeedUtils",
+    "resource:///modules/FeedUtils.jsm"
+  );
+  ChromeUtils.defineModuleGetter(
+    LazyModules,
+    "FolderUtils",
+    "resource:///modules/FolderUtils.jsm"
+  );
+  ChromeUtils.defineModuleGetter(
+    LazyModules,
+    "MailUtils",
+    "resource:///modules/MailUtils.jsm"
   );
 
   /**
@@ -243,29 +250,28 @@
             }
           },
 
-          _itemAddedOrRemoved(item) {
-            if (!(item instanceof Ci.nsIMsgFolder)) {
-              return;
-            }
-            if (this._filterFunction && !this._filterFunction(item)) {
+          _folderAddedOrRemoved(folder) {
+            if (this._filterFunction && !this._filterFunction(folder)) {
               return;
             }
             // xxx we can optimize this later
             this._clearMenu(this._menu);
           },
 
-          OnItemAdded(ParentItem, item) {
-            this._itemAddedOrRemoved(item);
+          onFolderAdded(parentFolder, child) {
+            this._folderAddedOrRemoved(child);
           },
-          OnItemRemoved(ParentItem, item) {
-            this._itemAddedOrRemoved(item);
+          onMessageAdded(parentFolder, msg) {},
+          onFolderRemoved(parentFolder, child) {
+            this._folderAddedOrRemoved(child);
           },
+          onMessageRemoved(parentFolder, msg) {},
 
           // xxx I stole this listener list from nsMsgFolderDatasource.cpp, but
           // someone should really document what events are fired when, so that
           // we make sure we're updating at the right times.
-          OnItemPropertyChanged(item, property, old, newItem) {},
-          OnItemIntPropertyChanged(item, property, old, aNew) {
+          onFolderPropertyChanged(item, property, old, newItem) {},
+          onFolderIntPropertyChanged(item, property, old, aNew) {
             if (item instanceof Ci.nsIMsgFolder) {
               if (property == "FolderFlag") {
                 if (
@@ -285,15 +291,15 @@
             }
             this._setCssSelectorsForItem(item);
           },
-          OnItemBoolPropertyChanged(item, property, old, newItem) {
+          onFolderBoolPropertyChanged(item, property, old, newItem) {
             this._setCssSelectorsForItem(item);
           },
-          OnItemUnicharPropertyChanged(item, property, old, newItem) {
+          onFolderUnicharPropertyChanged(item, property, old, newItem) {
             this._setCssSelectorsForItem(item);
           },
-          OnItemPropertyFlagChanged(item, property, old, newItem) {},
+          onFolderPropertyFlagChanged(item, property, old, newItem) {},
 
-          OnItemEvent(folder, eventName) {
+          onFolderEvent(folder, eventName) {
             if (eventName == "MRMTimeChanged") {
               if (
                 this._menu.getAttribute("showRecent") != "true" ||
@@ -380,7 +386,7 @@
       }
 
       set parentFolder(val) {
-        return (this._parentFolder = val);
+        this._parentFolder = val;
       }
 
       get parentFolder() {
@@ -450,14 +456,12 @@
 
         // If we have a parent folder, just get the subFolders for that parent.
         if (parentFolder) {
-          folders = toArray(
-            fixIterator(parentFolder.subFolders, Ci.nsIMsgFolder)
-          );
+          folders = parentFolder.subFolders;
         } else {
           // If we don't have a parent, then we assume we should build the
           // top-level accounts. (Actually we build the fake root folders for
           // those accounts.)
-          let accounts = allAccountsSorted(true);
+          let accounts = LazyModules.FolderUtils.allAccountsSorted(true);
 
           // Now generate our folder list. Note that we'll special case this
           // situation elsewhere, to avoid destroying the sort order we just made.
@@ -570,7 +574,7 @@
         switch (specialType) {
           case "recent":
             // Find the most recently modified ones.
-            specialFolders = getMostRecentFolders(
+            specialFolders = LazyModules.FolderUtils.getMostRecentFolders(
               specialFolders,
               Services.prefs.getIntPref("mail.folder_widget.max_recent"),
               "MRMTime"
@@ -620,7 +624,9 @@
         }
 
         // Make sure the entries are sorted alphabetically.
-        specialFoldersMap.sort((a, b) => folderNameCompare(a.label, b.label));
+        specialFoldersMap.sort((a, b) =>
+          LazyModules.FolderUtils.folderNameCompare(a.label, b.label)
+        );
 
         // Create entries for each of the recent folders.
         for (let folderItem of specialFoldersMap) {
@@ -709,7 +715,7 @@
 
         // We need to call this, or hasSubFolders will always return false.
         // Remove this workaround when Bug 502900 is fixed.
-        MailUtils.discoverFolders();
+        LazyModules.MailUtils.discoverFolders();
         this._serversOnly = true;
 
         let [shouldExpand, labels] = this._getShouldExpandAndLabels();
@@ -870,7 +876,9 @@
         let attributes = {};
 
         // First the SpecialFolder attribute.
-        attributes.SpecialFolder = getSpecialFolderString(folder);
+        attributes.SpecialFolder = LazyModules.FolderUtils.getSpecialFolderString(
+          folder
+        );
 
         // Now the biffState.
         let biffStates = ["NewMail", "NoMail", "UnknownMail"];
@@ -884,7 +892,9 @@
         attributes.IsServer = folder.isServer;
         attributes.IsSecure = folder.server.isSecure;
         attributes.ServerType = folder.server.type;
-        attributes.IsFeedFolder = !!FeedUtils.getFeedUrlsInFolder(folder);
+        attributes.IsFeedFolder = !!LazyModules.FeedUtils.getFeedUrlsInFolder(
+          folder
+        );
 
         return attributes;
       }
@@ -913,7 +923,9 @@
         }
 
         if (this._displayformat == "path") {
-          return FeedUtils.getFolderPrettyPath(folder) || folder.name;
+          return (
+            LazyModules.FeedUtils.getFolderPrettyPath(folder) || folder.name
+          );
         }
 
         return folder.name;
@@ -963,11 +975,13 @@
           );
           menulist.setAttribute(
             "SpecialFolder",
-            folder ? getSpecialFolderString(folder) : "none"
+            folder
+              ? LazyModules.FolderUtils.getSpecialFolderString(folder)
+              : "none"
           );
           menulist.setAttribute(
             "IsFeedFolder",
-            Boolean(folder && FeedUtils.getFeedUrlsInFolder(folder))
+            Boolean(folder && LazyModules.FeedUtils.getFeedUrlsInFolder(folder))
           );
         }
 
