@@ -1,26 +1,26 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/*
- * here's how this dialog works:
- * The main dialog contains a tree on the left (accounttree) and a
- * deck on the right. Each card in the deck on the right contains an
- * IFRAME which loads a particular preference document (such as am-main.xhtml)
+/**
+ * Here's how this dialog works:
+ * The main dialog contains a tree on the left (id="accounttree") and an
+ * iframe which loads a particular preference document (such as am-main.xhtml)
+ * on the right.
  *
- * when the user clicks on items in the tree on the right, two things have
+ * When the user clicks on items in the tree on the left, two things have
  * to be determined before the UI can be updated:
  * - the relevant account
  * - the relevant page
  *
- * when both of these are known, this is what happens:
+ * When both of these are known, this is what happens:
  * - every form element of the previous page is saved in the account value
  *   hashtable for the previous account
- * - the card containing the relevant page is brought to the front
+ * - the relevant page is loaded into the iframe
  * - each form element in the page is filled in with an appropriate value
  *   from the current account's hashtable
- * - in the IFRAME inside the page, if there is an onInit() method,
+ * - in the iframe inside the page, if there is an onInit() method,
  *   it is called. The onInit method can further update this page based
  *   on values set in the previous step.
  */
@@ -28,30 +28,59 @@
 /* import-globals-from accountUtils.js */
 /* import-globals-from am-prefs.js */
 /* import-globals-from amUtils.js */
-/* globals gSubDialog */
 
+var { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { BrowserUtils } = ChromeUtils.import(
-  "resource://gre/modules/BrowserUtils.jsm"
-);
+var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
 var { Gloda } = ChromeUtils.import("resource:///modules/gloda/Gloda.jsm");
-var { fixIterator } = ChromeUtils.import(
-  "resource:///modules/iteratorUtils.jsm"
-);
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var { allAccountsSorted } = ChromeUtils.import(
-  "resource:///modules/folderUtils.jsm"
+var { UIDensity } = ChromeUtils.import("resource:///modules/UIDensity.jsm");
+var { UIFontSize } = ChromeUtils.import("resource:///modules/UIFontSize.jsm");
+
+ChromeUtils.defineModuleGetter(
+  this,
+  "FolderUtils",
+  "resource:///modules/FolderUtils.jsm"
 );
 var { cleanUpHostName, isLegalHostNameOrIP } = ChromeUtils.import(
   "resource:///modules/hostnameUtils.jsm"
 );
+var { ChatIcons } = ChromeUtils.import("resource:///modules/chatIcons.jsm");
 
-document.addEventListener("prefchange", event => {
-  onAccept(true);
-  event.stopPropagation();
+XPCOMUtils.defineLazyGetter(this, "gSubDialog", function() {
+  const { SubDialogManager } = ChromeUtils.import(
+    "resource://gre/modules/SubDialog.jsm"
+  );
+  return new SubDialogManager({
+    dialogStack: document.getElementById("dialogStack"),
+    dialogTemplate: document.getElementById("dialogTemplate"),
+    dialogOptions: {
+      styleSheets: [
+        "chrome://messenger/skin/preferences/dialog.css",
+        "chrome://messenger/skin/preferences/preferences.css",
+      ],
+      resizeCallback: ({ title, frame }) => {
+        UIFontSize.registerWindow(frame.contentWindow);
+
+        // Resize the dialog to fit the content with edited font size.
+        requestAnimationFrame(() => {
+          let dialogs = frame.ownerGlobal.gSubDialog._dialogs;
+          let dialog = dialogs.find(
+            d => d._frame.contentDocument == frame.contentDocument
+          );
+          if (dialog) {
+            UIFontSize.resizeSubDialog(dialog);
+          }
+        });
+      },
+    },
+  });
 });
+
 // If Local directory has changed the app needs to restart. Once this is set
 // a restart will be attempted at each attempt to close the Account manager with OK.
 var gRestartNeeded = false;
@@ -165,7 +194,11 @@ function onLoad() {
         });
       }
     }
+    UIFontSize.registerWindow(contentFrame.contentWindow);
   });
+
+  UIDensity.registerWindow(window);
+  UIFontSize.registerWindow(window);
 }
 
 function onUnload() {
@@ -173,49 +206,39 @@ function onUnload() {
 }
 
 function selectServer(server, selectPageId) {
-  let childrenNode = document.getElementById("account-tree-children");
+  let accountTree = document.getElementById("accounttree");
 
   // Default to showing the first account.
-  let accountNode = childrenNode.firstElementChild;
+  let accountRow = accountTree.rows[0];
 
-  // Find the tree-node for the account we want to select
+  // Find the tree-node for the account we want to select.
   if (server) {
-    for (let i = 0; i < childrenNode.children.length; i++) {
-      let account = childrenNode.children[i]._account;
+    for (let row of accountTree.children) {
+      let account = row._account;
       if (account && server == account.incomingServer) {
-        accountNode = childrenNode.children[i];
+        accountRow = row;
         // Make sure all the panes of the account to be selected are shown.
-        accountNode.setAttribute("open", "true");
+        accountTree.expandRow(accountRow);
         break;
       }
     }
   }
 
-  let pageToSelect = accountNode;
+  let pageToSelect = accountRow;
 
   if (selectPageId) {
     // Find the page that also corresponds to this server.
-    // It either is the accountNode itself...
-    let pageId = accountNode.getAttribute("PageTag");
+    // It either is the accountRow itself...
+    let pageId = accountRow.getAttribute("PageTag");
     if (pageId != selectPageId) {
       // ... or one of its children.
-      pageToSelect = accountNode.querySelector(
+      pageToSelect = accountRow.querySelector(
         '[PageTag="' + selectPageId + '"]'
       );
     }
   }
 
-  let accountTree = document.getElementById("accounttree");
-  let index = accountTree.view.getIndexOfItem(pageToSelect);
-  accountTree.view.selection.select(index);
-  accountTree.ensureRowIsVisible(index);
-
-  let lastItem = accountNode.lastElementChild.lastElementChild;
-  if (lastItem.localName == "treeitem") {
-    index = accountTree.view.getIndexOfItem(lastItem);
-  }
-
-  accountTree.ensureRowIsVisible(index);
+  accountTree.selectedIndex = accountTree.rows.indexOf(pageToSelect);
 }
 
 function replaceWithDefaultSmtpServer(deletedSmtpServerKey) {
@@ -288,27 +311,15 @@ function onAccept(aDoChecks) {
   Services.prefs.savePrefFile(null);
 
   if (gRestartNeeded) {
-    gRestartNeeded = !BrowserUtils.restartApplication();
-    // returns false so that Account manager is not exited when restart failed
-    return !gRestartNeeded;
+    MailUtils.restartApplication();
+    // Prevent closing Account manager in case restart failed. If restart did not fail,
+    // return value does not matter, as we are restarting.
+    return false;
   }
 
   return true;
 }
 
-/**
- * Runs when Cancel button is used.
- *
- * This function must not be called onCancel(), because it would call itself
- * recursively for pages that don't have an onCancel() implementation.
- */
-function onNotAccept(event) {
-  // If onCancel() present in current page frame, call it.
-  if ("onCancel" in top.frames.contentFrame) {
-    top.frames.contentFrame.onCancel(event);
-  }
-}
-
 /**
  * See if the given path to a directory is usable on the current OS.
  *
@@ -582,15 +593,15 @@ function checkUserServerChanges(showAlert) {
 
   // Get the new username, hostname and type from the page.
   var typeElem = getPageFormElement("server.type");
-  var hostElem = getPageFormElement("server.realHostName");
-  var userElem = getPageFormElement("server.realUsername");
+  var hostElem = getPageFormElement("server.hostName");
+  var userElem = getPageFormElement("server.username");
   if (typeElem && userElem && hostElem) {
     var newType = getFormElementValue(typeElem);
     var oldHost = getAccountValue(
       currentAccount,
       accountValues,
       "server",
-      "realHostName",
+      "hostName",
       null,
       false
     );
@@ -599,7 +610,7 @@ function checkUserServerChanges(showAlert) {
       currentAccount,
       accountValues,
       "server",
-      "realUsername",
+      "username",
       null,
       false
     );
@@ -621,11 +632,10 @@ function checkUserServerChanges(showAlert) {
       } else if (!isLegalHostNameOrIP(newHost)) {
         alertText = prefBundle.getString("enterValidServerName");
       } else {
-        let sameServer = MailServices.accounts.findRealServer(
+        let sameServer = MailServices.accounts.findServer(
           newUser,
           newHost,
-          newType,
-          0
+          newType
         );
         if (sameServer && sameServer != currentServer) {
           alertText = prefBundle.getString("modifiedAccountExists");
@@ -677,6 +687,26 @@ function checkUserServerChanges(showAlert) {
           Services.prompt.alert(window, alertTitle, changeText.trim());
         }
       }
+
+      let l10n = new Localization(["messenger/accountManager.ftl"], true);
+      let cancel = Services.prompt.confirmEx(
+        window,
+        alertTitle,
+        l10n.formatValueSync("server-change-restart-required"),
+        Services.prompt.BUTTON_POS_0 * Services.prompt.BUTTON_TITLE_IS_STRING +
+          Services.prompt.BUTTON_POS_1 * Services.prompt.BUTTON_TITLE_CANCEL,
+        prefBundle.getString("localDirectoryRestart"),
+        null,
+        null,
+        null,
+        {}
+      );
+      if (cancel) {
+        setFormElementValue(hostElem, oldHost);
+        setFormElementValue(userElem, oldUser);
+        return false;
+      }
+      gRestartNeeded = true;
     }
   }
 
@@ -753,6 +783,14 @@ function checkAccountNameIsValid() {
       alertText = prefBundle.getString("accountNameEmpty");
     } else if (accountNameExists(accountName, currentAccount.key)) {
       alertText = prefBundle.getString("accountNameExists");
+      // Change the account name to prevent UI freeze.
+      let counter = 2;
+      while (
+        accountNameExists(`${accountName}_${counter}`, currentAccount.key)
+      ) {
+        counter++;
+      }
+      serverNameElem.value = `${accountName}_${counter}`;
     }
 
     if (alertText) {
@@ -785,10 +823,6 @@ function onSave() {
   return true;
 }
 
-function onAddAccount() {
-  MsgAccountWizard();
-}
-
 /**
  * Highlight the default account row in the account tree,
  * optionally un-highlight the previous one.
@@ -803,55 +837,27 @@ function markDefaultServer(newDefault, oldDefault) {
     return;
   }
 
-  let accountTree = document.getElementById("account-tree-children");
-  for (let accountNode of accountTree.children) {
-    if (newDefault && newDefault == accountNode._account) {
-      let props = accountNode.firstElementChild.firstElementChild.getAttribute(
-        "properties"
-      );
-      accountNode.firstElementChild.firstElementChild.setAttribute(
-        "properties",
-        props + " isDefaultServer-true"
-      );
+  let accountTree = document.getElementById("accounttree");
+  for (let accountRow of accountTree.children) {
+    if (newDefault && newDefault == accountRow._account) {
+      accountRow.classList.add("isDefaultServer");
     }
-    if (oldDefault && oldDefault == accountNode._account) {
-      let props = accountNode.firstElementChild.firstElementChild.getAttribute(
-        "properties"
-      );
-      props = props.replace(/isDefaultServer-true/, "");
-      accountNode.firstElementChild.firstElementChild.setAttribute(
-        "properties",
-        props
-      );
+    if (oldDefault && oldDefault == accountRow._account) {
+      accountRow.classList.remove("isDefaultServer");
     }
   }
 }
 
 /**
- * Sets the name of the account rowitem in the tree pane.
+ * Notify the UI to rebuild the account tree.
  *
- * @param aAccountKey   the key of the account to change
- * @param aAccountNode  the node on which to change the label
- * @param aLabel        the value of the label to set
+ * @param {boolean} [reloadAccountManager=true] - When set to false, the
+ *   account manager for the current window is not reloaded.
  */
-function setAccountLabel(aAccountKey, aAccountNode, aLabel) {
-  if (!aAccountNode) {
-    // We can't use the current tree selection to determine the current account,
-    // because this function may be called when the selection
-    // is already on another tree item (account) than the one we want to change.
-    // So find the proper node using the account key.
-    let accountTree = document.getElementById("account-tree-children");
-    for (let accountNode of accountTree.children) {
-      if (
-        "_account" in accountNode &&
-        accountNode._account.key == aAccountKey
-      ) {
-        aAccountNode = accountNode.firstElementChild.firstElementChild;
-        break;
-      }
-    }
+function rebuildAccountTree(reloadAccountManager = true) {
+  for (let win of Services.wm.getEnumerator("mail:3pane")) {
+    win.gFolderTreeView._rebuild();
   }
-  aAccountNode.setAttribute("label", aLabel);
 }
 
 /**
@@ -866,11 +872,11 @@ function onSetDefault(event) {
   let previousDefault = MailServices.accounts.defaultAccount;
   MailServices.accounts.defaultAccount = currentAccount;
   markDefaultServer(currentAccount, previousDefault);
+
   // Update gloda's myContact with the new default account's default identity.
   Gloda._initMyIdentities();
 
-  // This is only needed on Seamonkey which has this button.
-  setEnabled(document.getElementById("setDefaultButton"), false);
+  gAccountTree.load();
 }
 
 function onRemoveAccount(event) {
@@ -886,11 +892,11 @@ function onRemoveAccount(event) {
   }
 
   let serverList = [];
-  let accountTreeNode = document.getElementById("account-tree-children");
+  let accountTree = document.getElementById("accounttree");
   // build the list of servers in the account tree (order is important)
-  for (let i = 0; i < accountTreeNode.children.length; i++) {
-    if ("_account" in accountTreeNode.children[i]) {
-      let curServer = accountTreeNode.children[i]._account.incomingServer;
+  for (let row of accountTree.children) {
+    if ("_account" in row) {
+      let curServer = row._account.incomingServer;
       if (!serverList.includes(curServer)) {
         serverList.push(curServer);
       }
@@ -942,9 +948,11 @@ function onRemoveAccount(event) {
 
   gSubDialog.open(
     "chrome://messenger/content/removeAccount.xhtml",
-    "resizable=no",
-    removeArgs,
-    onCloseDialog
+    {
+      features: "resizable=no",
+      closingCallback: onCloseDialog,
+    },
+    removeArgs
   );
 }
 
@@ -958,33 +966,32 @@ function saveAccount(accountValues, account) {
   }
 
   for (var type in accountValues) {
+    var dest;
+    try {
+      if (type == "identity") {
+        dest = identity;
+      } else if (type == "server") {
+        dest = server;
+      } else if (type == "pop3") {
+        dest = server.QueryInterface(Ci.nsIPop3IncomingServer);
+      } else if (type == "imap") {
+        dest = server.QueryInterface(Ci.nsIImapIncomingServer);
+      } else if (type == "none") {
+        dest = server.QueryInterface(Ci.nsINoIncomingServer);
+      } else if (type == "nntp") {
+        dest = server.QueryInterface(Ci.nsINntpIncomingServer);
+      } else if (type == "smtp") {
+        dest = MailServices.smtp.defaultServer;
+      }
+    } catch (ex) {
+      // don't do anything, just means we don't support that
+    }
+    if (dest == undefined) {
+      continue;
+    }
     var typeArray = accountValues[type];
 
     for (var slot in typeArray) {
-      var dest;
-      try {
-        if (type == "identity") {
-          dest = identity;
-        } else if (type == "server") {
-          dest = server;
-        } else if (type == "pop3") {
-          dest = server.QueryInterface(Ci.nsIPop3IncomingServer);
-        } else if (type == "imap") {
-          dest = server.QueryInterface(Ci.nsIImapIncomingServer);
-        } else if (type == "none") {
-          dest = server.QueryInterface(Ci.nsINoIncomingServer);
-        } else if (type == "nntp") {
-          dest = server.QueryInterface(Ci.nsINntpIncomingServer);
-        } else if (type == "smtp") {
-          dest = MailServices.smtp.defaultServer;
-        }
-      } catch (ex) {
-        // don't do anything, just means we don't support that
-      }
-      if (dest == undefined) {
-        continue;
-      }
-
       if (
         type in gGenericAttributeTypes &&
         slot in gGenericAttributeTypes[type]
@@ -1079,24 +1086,6 @@ function saveAccount(accountValues, account) {
   return true;
 }
 
-/**
- * Set enabled/disabled state for account actions buttons.
- * Called by all apps, but if the buttons do not exist, exits early.
- */
-function updateButtons(tree, account) {
-  let addAccountButton = document.getElementById("addAccountButton");
-  let removeButton = document.getElementById("removeButton");
-  let setDefaultButton = document.getElementById("setDefaultButton");
-
-  if (!addAccountButton && !removeButton && !setDefaultButton) {
-    // Thunderbird isn't using these.
-    return;
-  }
-
-  updateItems(tree, account, addAccountButton, setDefaultButton, removeButton);
-  updateBlockedItems([addAccountButton, setDefaultButton, removeButton], false);
-}
-
 /**
  * Set enabled/disabled state for the actions in the Account Actions menu.
  * Called only by Thunderbird.
@@ -1132,7 +1121,7 @@ function updateItems(
   let canSetDefault = false;
   let canDelete = false;
 
-  if (account && tree.view.selection.count >= 1) {
+  if (account && tree.selectedIndex >= 0) {
     // Only try to check properties if there was anything selected in the tree
     // and it belongs to an account.
     // Otherwise we have either selected a SMTP server, or there is some
@@ -1157,8 +1146,6 @@ function updateItems(
 
 /**
  * Disable buttons/menu items if their control preference is locked.
- * SeaMonkey: Not currently handled by WSM or the main loop yet
- * since these buttons aren't under the IFRAME.
  *
  * @param aItems       an array or NodeList of elements to be checked
  * @param aMustBeTrue  if true then the pref must be boolean and set to true
@@ -1203,11 +1190,11 @@ function onAccountTreeSelect(pageId, account) {
 
   let changeView = pageId && account;
   if (!changeView) {
-    if (tree.view.selection.count < 1) {
+    if (tree.selectedIndex < 0) {
       return false;
     }
 
-    let node = tree.view.getItemAtIndex(tree.currentIndex);
+    let node = tree.rows[tree.selectedIndex];
     account = "_account" in node ? node._account : null;
 
     pageId = node.getAttribute("PageTag");
@@ -1282,10 +1269,6 @@ function onAccountTreeSelect(pageId, account) {
     restorePage(pageId, account);
   }
 
-  if (changeAccount) {
-    updateButtons(tree, account);
-  }
-
   return true;
 }
 
@@ -1356,6 +1339,16 @@ function savePage(account) {
   if (!accountValues) {
     return;
   }
+  // Reset accountArray so that only the current page will be saved. This is
+  // needed to prevent resetting prefs unintentionally. An example is when
+  // changing username/hostname, MsgIncomingServer.jsm will modify identities,
+  // without this, identities changes may be reverted to old values in
+  // accountArray.
+  accountArray = {};
+  accountValues = {};
+  let serverId = account.incomingServer.serverURI;
+  accountArray[serverId] = accountValues;
+  accountArray[serverId]._account = account;
 
   var pageElements = getPageFormElements();
   if (!pageElements) {
@@ -1685,10 +1678,60 @@ function getValueArrayFor(account) {
   return accountArray[serverId];
 }
 
+/**
+ * Sets the name of the account rowitem in the tree pane.
+ *
+ * @param aAccountKey   the key of the account to change
+ * @param aLabel        the value of the label to set
+ */
+function setAccountLabel(aAccountKey, aLabel) {
+  let row = document.getElementById(aAccountKey);
+  if (row) {
+    row.setAttribute("aria-label", aLabel);
+    row.title = aLabel;
+    row.querySelector(".name").textContent = aLabel;
+  }
+  rebuildAccountTree(false);
+}
+
 var gAccountTree = {
   load() {
     this._build();
 
+    let mainTree = document.getElementById("accounttree");
+    mainTree.__defineGetter__("_orderableChildren", function() {
+      let rows = [...this.children];
+      rows.pop();
+      return rows;
+    });
+    mainTree.addEventListener("ordering", event => {
+      if (!event.detail || event.detail.id == "smtp") {
+        event.preventDefault();
+      }
+    });
+    mainTree.addEventListener("ordered", event => {
+      let accountKeyList = Array.from(mainTree.children, row => row.id);
+      accountKeyList.pop(); // Remove SMTP.
+      MailServices.accounts.reorderAccounts(accountKeyList);
+      rebuildAccountTree();
+    });
+    mainTree.addEventListener("expanded", event => {
+      this._dataStore.setValue(
+        document.documentURI,
+        event.target.id,
+        "open",
+        "true"
+      );
+    });
+    mainTree.addEventListener("collapsed", event => {
+      this._dataStore.setValue(
+        document.documentURI,
+        event.target.id,
+        "open",
+        "false"
+      );
+    });
+
     MailServices.accounts.addIncomingServerListener(this);
   },
   unload() {
@@ -1739,9 +1782,9 @@ var gAccountTree = {
       { string: getString("prefPanel-junk"), src: "am-junk.xhtml" },
     ];
 
-    let accounts = allAccountsSorted(false);
+    let accounts = FolderUtils.allAccountsSorted(false);
 
-    let mainTree = document.getElementById("account-tree-children");
+    let mainTree = document.getElementById("accounttree");
     // Clear off all children...
     while (mainTree.hasChildNodes()) {
       mainTree.lastChild.remove();
@@ -1838,50 +1881,54 @@ var gAccountTree = {
       }
 
       // Create the top level tree-item.
-      let treeitem = document.createXULElement("treeitem");
+      let treeitem = document
+        .getElementById("accountTreeItem")
+        .content.firstElementChild.cloneNode(true);
       mainTree.appendChild(treeitem);
-      let treerow = document.createXULElement("treerow");
-      treeitem.appendChild(treerow);
-      let treecell = document.createXULElement("treecell");
-      treerow.appendChild(treecell);
-      treecell.setAttribute("label", accountName);
+      treeitem.setAttribute("aria-label", accountName);
+      treeitem.title = accountName;
+      treeitem.querySelector(".name").textContent = accountName;
       treeitem.setAttribute("PageTag", amChrome);
       // Add icons based on account type.
       if (server) {
-        treecell.setAttribute(
-          "properties",
-          "folderNameCol isServer-true serverType-" + server.type
-        );
+        treeitem.classList.add("serverType-" + server.type);
+        if (server.isSecure) {
+          treeitem.classList.add("isSecure");
+        }
         // For IM accounts, we can try to fetch a protocol specific icon.
         if (server.type == "im") {
-          treecell.setAttribute(
-            "src",
-            server.wrappedJSObject.imAccount.protocol.iconBaseURI + "icon.png"
-          );
+          treeitem.querySelector(".icon").style.backgroundImage =
+            "url(" +
+            ChatIcons.getProtocolIconURI(
+              server.wrappedJSObject.imAccount.protocol
+            ) +
+            ")";
+          treeitem.id = accountKey;
         }
       }
 
       if (panelsToKeep.length > 0) {
-        var treekids = document.createXULElement("treechildren");
-        treeitem.appendChild(treekids);
+        let treekids = treeitem.querySelector("ul");
         for (let panel of panelsToKeep) {
-          var kidtreeitem = document.createXULElement("treeitem");
+          let kidtreeitem = document.createElement("li");
+          kidtreeitem.title = panel.string;
           treekids.appendChild(kidtreeitem);
-          var kidtreerow = document.createXULElement("treerow");
+          let kidtreerow = document.createElement("div");
           kidtreeitem.appendChild(kidtreerow);
-          var kidtreecell = document.createXULElement("treecell");
+          let kidtreecell = document.createElement("span");
+          kidtreecell.classList.add("name");
+          kidtreecell.tabIndex = -1;
           kidtreerow.appendChild(kidtreecell);
-          setAccountLabel(null, kidtreecell, panel.string);
+          kidtreecell.textContent = panel.string;
           kidtreeitem.setAttribute("PageTag", panel.src);
           kidtreeitem._account = account;
+          kidtreeitem.id = `${accountKey}/${panel.src}`;
         }
-        treeitem.setAttribute("container", "true");
         treeitem.id = accountKey;
         // Load the 'open' state of the account from XULStore.json.
-        treeitem.setAttribute("open", this._getAccountOpenState(accountKey));
-        // Let the XULStore.json automatically save the 'open' state of the
-        // account when it is changed.
-        treeitem.setAttribute("persist", "open");
+        if (this._getAccountOpenState(accountKey) != "true") {
+          treeitem.classList.add("collapsed");
+        }
       }
       treeitem._account = account;
     }
@@ -1889,18 +1936,14 @@ var gAccountTree = {
     markDefaultServer(MailServices.accounts.defaultAccount, null);
 
     // Now add the outgoing server node.
-    let treeitem = document.createXULElement("treeitem");
+    let treeitem = document
+      .getElementById("accountTreeItem")
+      .content.firstElementChild.cloneNode(true);
     mainTree.appendChild(treeitem);
-    let treerow = document.createXULElement("treerow");
-    treeitem.appendChild(treerow);
-    let treecell = document.createXULElement("treecell");
-    treerow.appendChild(treecell);
-    treecell.setAttribute("label", getString("prefPanel-smtp"));
+    treeitem.id = "smtp";
+    treeitem.querySelector(".name").textContent = getString("prefPanel-smtp");
     treeitem.setAttribute("PageTag", "am-smtp.xhtml");
-    treecell.setAttribute(
-      "properties",
-      "folderNameCol isServer-true serverType-smtp"
-    );
+    treeitem.classList.add("serverType-smtp");
 
     // If a new server was created, select the server after rebuild of the tree.
     if (newServer) {
