@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -25,9 +24,10 @@ var gSmtpSocketType;
 var gPort;
 var gDefaultPort;
 
+window.addEventListener("DOMContentLoaded", onLoad);
 document.addEventListener("dialogaccept", onAccept);
 
-function onLoad(event) {
+function onLoad() {
   gSmtpServer = window.arguments[0].server;
   initSmtpSettings(gSmtpServer);
 }
@@ -62,9 +62,9 @@ function onAccept(event) {
 }
 
 function initSmtpSettings(server) {
-  gSmtpUsername = document.getElementById("smtp.username");
+  gSmtpUsername = document.getElementById("smtpUsername");
   gSmtpDescription = document.getElementById("smtp.description");
-  gSmtpUsernameLabel = document.getElementById("smtp.username.label");
+  gSmtpUsernameLabel = document.getElementById("smtpUsernameLabel");
   gSmtpHostname = document.getElementById("smtp.hostname");
   gSmtpPort = document.getElementById("smtp.port");
   gSmtpAuthMethod = document.getElementById("smtp.authMethod");
@@ -106,7 +106,7 @@ function initSmtpSettings(server) {
   setLabelFromStringBundle("authMethod-anysecure", "authAnySecure");
   setLabelFromStringBundle("authMethod-any", "authAny");
 
-  sizeToContent();
+  window.sizeToContent();
 
   sslChanged(false);
   authMethodChanged(false);
@@ -116,7 +116,9 @@ function initSmtpSettings(server) {
   }
 
   // Hide OAuth2 option if we can't use it.
-  let details = OAuth2Providers.getHostnameDetails(server.hostname);
+  let details = server
+    ? OAuth2Providers.getHostnameDetails(server.hostname)
+    : null;
   document.getElementById("authMethod-oauth2").hidden = !details;
 
   // Hide deprecated/hidden auth options, unless selected
