@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,7 +8,7 @@
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 function onLoad() {
-  parent.onPanelLoaded("am-addressing.xul");
+  parent.onPanelLoaded("am-addressing.xhtml");
 }
 
 function onInit(aPageId, aServerId) {
@@ -22,11 +21,8 @@ function onInitCompositionAndAddressing() {
 }
 
 function onEditDirectories() {
-  window.openDialog(
-    "chrome://messenger/content/addressbook/pref-editdirectories.xul",
-    "editDirectories",
-    "chrome,modal=yes,resizable=no",
-    null
+  parent.gSubDialog.open(
+    "chrome://messenger/content/addressbook/pref-editdirectories.xhtml"
   );
 }
 
@@ -41,11 +37,11 @@ function quoteEnabling() {
   var placebox = document.getElementById("placeBox");
 
   if (document.getElementById("identity.replyOnTop").value == "1") {
-    placebox.firstChild.removeAttribute("disabled");
-    placebox.lastChild.removeAttribute("disabled");
+    placebox.firstElementChild.removeAttribute("disabled");
+    placebox.lastElementChild.removeAttribute("disabled");
   } else {
-    placebox.firstChild.setAttribute("disabled", "true");
-    placebox.lastChild.setAttribute("disabled", "true");
+    placebox.firstElementChild.setAttribute("disabled", "true");
+    placebox.lastElementChild.setAttribute("disabled", "true");
   }
 }
 
@@ -55,7 +51,7 @@ function quoteEnabling() {
 function showGlobalAddressingPrefs() {
   openPrefsFromAccountManager(
     "paneCompose",
-    "addressingTab",
+    "compositionAddressingCategory",
     null,
     "addressing_pane"
   );
@@ -67,7 +63,7 @@ function showGlobalAddressingPrefs() {
 function showGlobalComposingPrefs() {
   openPrefsFromAccountManager(
     "paneCompose",
-    "generalTab",
+    null,
     null,
     "composing_messages_pane"
   );
