@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -37,6 +36,10 @@ function onLoadIdentityProperties() {
   initIdentityValues(gIdentity);
   initCopiesAndFolder(gIdentity);
   initCompositionAndAddressing(gIdentity);
+  initE2EEncryption(gIdentity);
+
+  // E2E needs an email... hide until we have one.
+  document.getElementById("identityE2ETab").hidden = !gIdentity?.email;
 }
 
 // based on the values of gIdentity, initialize the identity fields we expose to the user
@@ -70,7 +73,7 @@ function initIdentityValues(identity) {
     document.getElementById("identity.attachVCard").checked =
       identity.attachVCard;
     document.getElementById("identity.escapedVCard").value =
-      identity.escapedVCard;
+      identity.escapedVCard || "";
 
     document.getElementById("identity.catchAll").checked = identity.catchAll;
     document.getElementById("identity.catchAllHint").value =
@@ -78,9 +81,12 @@ function initIdentityValues(identity) {
 
     initSmtpServer(identity.smtpServerKey);
 
-    let servers = MailServices.accounts.getServersForIdentity(identity);
-    document.getElementById("identityCatchAllBox").hidden =
-      servers.length > 0 && servers[0].type == "nntp";
+    // In am-main.xhtml this field has no ID, because it's hidden by other means.
+    let catchAllBox = document.getElementById("identityCatchAllBox");
+    if (catchAllBox) {
+      let servers = MailServices.accounts.getServersForIdentity(identity);
+      catchAllBox.hidden = servers.length > 0 && servers[0].type == "nntp";
+    }
 
     // This field does not exist for the default identity shown in the am-main.xhtml pane.
     let idLabel = document.getElementById("identity.label");
@@ -212,7 +218,7 @@ function onOk(event) {
   saveIdentitySettings(gIdentity);
   saveCopiesAndFolderSettings(gIdentity);
   saveAddressingAndCompositionSettings(gIdentity);
-  e2eOnAcceptEditor(event);
+  saveE2EEncryptionSettings(gIdentity);
 
   window.arguments[0].result = true;
 }
@@ -458,26 +464,62 @@ function setupSignatureItems() {
   }
 }
 
-function editVCardCallback(escapedVCardStr) {
-  var escapedVCard = document.getElementById("identity.escapedVCard");
-  escapedVCard.value = escapedVCardStr;
+function editVCard() {
+  // Read vCard hidden value from UI.
+  let escapedVCard = document.getElementById("identity.escapedVCard");
+  let dialog = top.document.getElementById("editVCardDialog");
+  let form = dialog.querySelector("form");
+  let vCardEdit = dialog.querySelector("vcard-edit");
+
+  vCardEdit.vCardString = decodeURIComponent(escapedVCard.value);
+
+  top.addEventListener("keydown", editVCardKeyDown, { capture: true });
+  form.addEventListener("submit", editVCardSubmit);
+  form.addEventListener("reset", editVCardReset);
+
+  top.gSubDialog._topDialog?._overlay.removeAttribute("topmost");
+  dialog.showModal();
 }
 
-function editVCard() {
-  var escapedVCard = document.getElementById("identity.escapedVCard");
-  let args = {
-    escapedVCardStr: escapedVCard.value,
-    okCallback: editVCardCallback,
-    titleProperty: "editVCardTitle",
-    hideABPicker: true,
-  };
+function editVCardKeyDown(event) {
+  let dialog = top.document.getElementById("editVCardDialog");
+  if (event.keyCode == KeyboardEvent.DOM_VK_ESCAPE && dialog.open) {
+    // This is a bit of a hack to prevent other dialogs (particularly
+    // SubDialogs) from closing when the vCard dialog is open.
+    event.preventDefault();
+    editVCardReset();
+  }
+}
 
-  // read vCard hidden value from UI
-  parent.gSubDialog.open(
-    "chrome://messenger/content/addressbook/abNewCardDialog.xhtml",
-    "resizable=no",
-    args
-  );
+function editVCardSubmit(event) {
+  let escapedVCard = document.getElementById("identity.escapedVCard");
+  let dialog = top.document.getElementById("editVCardDialog");
+  let form = dialog.querySelector("form");
+  let vCardEdit = dialog.querySelector("vcard-edit");
+
+  vCardEdit.saveVCard();
+  escapedVCard.value = encodeURIComponent(vCardEdit.vCardString);
+  // Trigger a change event so for the am-main view the event listener
+  // set up in AccountManager.js onLoad() can make sure to save the change.
+  escapedVCard.dispatchEvent(new CustomEvent("change"));
+
+  top.gSubDialog._topDialog?._overlay.setAttribute("topmost", "true");
+  dialog.close();
+
+  event.preventDefault();
+  form.removeEventListener("submit", editVCardSubmit);
+  form.removeEventListener("reset", editVCardReset);
+}
+
+function editVCardReset() {
+  let dialog = top.document.getElementById("editVCardDialog");
+  let form = dialog.querySelector("form");
+
+  top.gSubDialog._topDialog?._overlay.setAttribute("topmost", "true");
+  dialog.close();
+
+  form.removeEventListener("submit", editVCardSubmit);
+  form.removeEventListener("reset", editVCardReset);
 }
 
 function getAccountForFolderPickerState() {
@@ -498,25 +540,23 @@ function loadSMTPServerList() {
   }
 
   for (let server of MailServices.smtp.servers) {
-    if (server instanceof Ci.nsISmtpServer) {
-      var serverName = "";
-      if (server.description) {
-        serverName = server.description + " - ";
-      } else if (server.username) {
-        serverName = server.username + " - ";
-      }
-      serverName += server.hostname;
-
-      if (defaultServer.key == server.key) {
-        serverName +=
-          " " +
-          document
-            .getElementById("bundle_messenger")
-            .getString("defaultServerTag");
-      }
-
-      smtpServerList.appendItem(serverName, server.key);
+    let serverName = "";
+    if (server.description) {
+      serverName = server.description + " - ";
+    } else if (server.username) {
+      serverName = server.username + " - ";
     }
+    serverName += server.hostname;
+
+    if (defaultServer.key == server.key) {
+      serverName +=
+        " " +
+        document
+          .getElementById("bundle_messenger")
+          .getString("defaultServerTag");
+    }
+
+    smtpServerList.appendItem(serverName, server.key);
   }
 
   smtpServerList.value = currentValue;
@@ -535,8 +575,7 @@ function editCurrentSMTP() {
 
   parent.gSubDialog.open(
     "chrome://messenger/content/SmtpServerEdit.xhtml",
-    null,
-    args,
-    loadSMTPServerList
+    { closingCallback: loadSMTPServerList },
+    args
   );
 }
