@@ -1,30 +1,34 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
- */
-ChromeUtils.import("resource:///modules/mailServices.js");
-ChromeUtils.import("resource:///modules/iteratorUtils.jsm");
-ChromeUtils.import("resource:///modules/MailUtils.js");
+ * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
+
+/* import-globals-from am-prefs.js */
+/* import-globals-from amUtils.js */
+
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
+var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
 
 var gDeferredToAccount = "";
 
-function onInit(aPageId, aServerId)
-{
+function onInit(aPageId, aServerId) {
   // manually adjust several pref UI elements
-  document.getElementById('server.spamLevel.visible').setAttribute("checked",
-    document.getElementById('server.spamLevel').value > 0);
+  document.getElementById("server.spamLevel.visible").checked =
+    document.getElementById("server.spamLevel").value > 0;
 
   let deferredToURI = null;
-  if (gDeferredToAccount)
-    deferredToURI = MailServices.accounts
-                                .getAccount(gDeferredToAccount)
-                                .incomingServer.serverURI;
+  if (gDeferredToAccount) {
+    deferredToURI = MailServices.accounts.getAccount(gDeferredToAccount)
+      .incomingServer.serverURI;
+  }
 
-  let spamActionTargetAccountElement =
-    document.getElementById("server.spamActionTargetAccount");
-  let spamActionTargetFolderElement =
-    document.getElementById("server.spamActionTargetFolder");
+  let spamActionTargetAccountElement = document.getElementById(
+    "server.spamActionTargetAccount"
+  );
+  let spamActionTargetFolderElement = document.getElementById(
+    "server.spamActionTargetFolder"
+  );
 
   let spamActionTargetAccount = spamActionTargetAccountElement.value;
   let spamActionTargetFolder = spamActionTargetFolderElement.value;
@@ -33,42 +37,50 @@ function onInit(aPageId, aServerId)
   let moveOnSpamValue = moveOnSpamCheckbox.checked;
 
   // Check if there are any invalid junk targets and fix them.
-  [ spamActionTargetAccount, spamActionTargetFolder, moveOnSpamValue ] =
-    sanitizeJunkTargets(spamActionTargetAccount,
-                        spamActionTargetFolder,
-                        deferredToURI || aServerId,
-                        document.getElementById("server.moveTargetMode").value,
-                        MailUtils.getFolderForURI(aServerId, false).server.spamSettings,
-                        moveOnSpamValue);
+  [
+    spamActionTargetAccount,
+    spamActionTargetFolder,
+    moveOnSpamValue,
+  ] = sanitizeJunkTargets(
+    spamActionTargetAccount,
+    spamActionTargetFolder,
+    deferredToURI || aServerId,
+    document.getElementById("server.moveTargetMode").value,
+    MailUtils.getOrCreateFolder(aServerId).server.spamSettings,
+    moveOnSpamValue
+  );
 
   spamActionTargetAccountElement.value = spamActionTargetAccount;
   spamActionTargetFolderElement.value = spamActionTargetFolder;
   moveOnSpamCheckbox.checked = moveOnSpamValue;
 
-  let server = MailUtils.getFolderForURI(spamActionTargetAccount, false);
+  let server = MailUtils.getOrCreateFolder(spamActionTargetAccount);
   document.getElementById("actionAccountPopup").selectFolder(server);
 
-  let folder = MailUtils.getFolderForURI(spamActionTargetFolder, true);
+  let folder = MailUtils.getExistingFolder(spamActionTargetFolder);
   document.getElementById("actionFolderPopup").selectFolder(folder);
 
   var currentArray = [];
-  if (document.getElementById("server.useWhiteList").checked)
-    currentArray = document.getElementById("server.whiteListAbURI").value.split(" ");
+  if (document.getElementById("server.useWhiteList").checked) {
+    currentArray = document
+      .getElementById("server.whiteListAbURI")
+      .value.split(" ");
+  }
 
   // set up the whitelist UI
   var wList = document.getElementById("whiteListAbURI");
   // Ensure the whitelist is empty
-  for (let i = wList.itemCount - 1; i >= 0; i--) {
-    wList.removeItemAt(i);
+  while (wList.hasChildNodes()) {
+    wList.lastChild.remove();
   }
 
   // Populate the listbox with address books
   let abItems = [];
-  for (let ab of fixIterator(MailServices.ab.directories,
-                             Ci.nsIAbDirectory)) {
+  for (let ab of MailServices.ab.directories) {
     // We skip mailing lists and remote address books.
-    if (ab.isMailList || ab.isRemote)
+    if (ab.isMailList || ab.isRemote) {
       continue;
+    }
 
     abItems.push({ label: ab.dirName, URI: ab.URI });
   }
@@ -81,24 +93,35 @@ function onInit(aPageId, aServerId)
 
   // And then append each item to the listbox
   for (let abItem of abItems) {
-    let item = wList.appendItem(abItem.label, abItem.URI);
-    item.setAttribute("type", "checkbox");
-    item.setAttribute("class", "listitem-iconic");
+    let checkbox = document.createXULElement("checkbox");
+    checkbox.setAttribute("label", abItem.label);
+    checkbox.checked = currentArray.includes(abItem.URI);
 
-    // Due to bug 448582, we have to use setAttribute to set the
-    // checked value of the listitem.
-    item.setAttribute("checked", currentArray.includes(abItem.URI));
+    let item = document.createXULElement("richlistitem");
+    item.appendChild(checkbox);
+    item.setAttribute("value", abItem.URI);
+    wList.appendChild(item);
   }
 
+  wList.addEventListener("keypress", event => {
+    if ([" ", "Enter"].includes(event.key)) {
+      let checkbox = wList.currentItem.firstElementChild;
+      checkbox.checked = !checkbox.checked;
+      wList.dispatchEvent(new CustomEvent("command"));
+    }
+  });
+
   // enable or disable the whitelist
   onAdaptiveJunkToggle();
 
   // set up trusted IP headers
   var serverFilterList = document.getElementById("useServerFilterList");
-  serverFilterList.value =
-    document.getElementById("server.serverFilterName").value;
-  if (!serverFilterList.selectedItem)
+  serverFilterList.value = document.getElementById(
+    "server.serverFilterName"
+  ).value;
+  if (!serverFilterList.selectedItem) {
     serverFilterList.selectedIndex = 0;
+  }
 
   // enable or disable the useServerFilter checkbox
   onCheckItem("useServerFilterList", ["server.useServerFilter"]);
@@ -106,12 +129,26 @@ function onInit(aPageId, aServerId)
   updateJunkTargetsAndRetention();
 }
 
-function onPreInit(account, accountValues)
-{
-  if (getAccountValue(account, accountValues, "server", "type", null, false) == "pop3")
-    gDeferredToAccount = getAccountValue(account, accountValues,
-                                         "pop3", "deferredToAccount",
-                                         null, false);
+function onPreInit(account, accountValues) {
+  if (
+    top.getAccountValue(
+      account,
+      accountValues,
+      "server",
+      "type",
+      null,
+      false
+    ) == "pop3"
+  ) {
+    gDeferredToAccount = top.getAccountValue(
+      account,
+      accountValues,
+      "pop3",
+      "deferredToAccount",
+      null,
+      false
+    );
+  }
 
   buildServerFilterMenuList();
 }
@@ -122,9 +159,8 @@ function onPreInit(account, accountValues)
  *
  * @param aValue  the boolean value of the checkbox
  */
-function updateSpamLevel(aValue)
-{
-  document.getElementById('server.spamLevel').value = aValue ? 100 : 0;
+function updateSpamLevel(aValue) {
+  document.getElementById("server.spamLevel").value = aValue ? 100 : 0;
   onAdaptiveJunkToggle();
 }
 
@@ -132,18 +168,17 @@ function updateSpamLevel(aValue)
  * Propagate changes to the server filter menu list back to
  * our hidden wsm element.
  */
-function onServerFilterListChange()
-{
-  document.getElementById('server.serverFilterName').value =
-    document.getElementById("useServerFilterList").value;
+function onServerFilterListChange() {
+  document.getElementById(
+    "server.serverFilterName"
+  ).value = document.getElementById("useServerFilterList").value;
 }
 
 /**
  * Called when someone checks or unchecks the adaptive junk mail checkbox.
  * We need to enable or disable the whitelist accordingly.
  */
-function onAdaptiveJunkToggle()
-{
+function onAdaptiveJunkToggle() {
   onCheckItem("whiteListAbURI", ["server.spamLevel.visible"]);
   onCheckItem("whiteListLabel", ["server.spamLevel.visible"]);
 
@@ -153,7 +188,9 @@ function onAdaptiveJunkToggle()
   let wListDisabled = wList.disabled;
 
   for (let i = 0; i < wList.getRowCount(); i++) {
-    wList.getItemAtIndex(i).setAttribute("disabled", wListDisabled);
+    let item = wList.getItemAtIndex(i);
+    item.setAttribute("disabled", wListDisabled);
+    item.firstElementChild.setAttribute("disabled", wListDisabled);
   }
 }
 
@@ -165,8 +202,9 @@ function updateJunkTargetsAndRetention() {
   onCheckItem("server.moveTargetMode", ["server.moveOnSpam"]);
   updateJunkTargets();
   onCheckItem("server.purgeSpam", ["server.moveOnSpam"]);
-  document.getElementById("purgeLabel").disabled =
-    document.getElementById("server.purgeSpam").disabled;
+  document.getElementById("purgeLabel").disabled = document.getElementById(
+    "server.purgeSpam"
+  ).disabled;
   updateJunkRetention();
 }
 
@@ -175,7 +213,7 @@ function updateJunkTargetsAndRetention() {
  */
 function updateJunkTargets() {
   onCheckItem("actionTargetAccount", ["server.moveOnSpam", "moveTargetMode0"]);
-  onCheckItem("actionTargetFolder",  ["server.moveOnSpam", "moveTargetMode1"]);
+  onCheckItem("actionTargetFolder", ["server.moveOnSpam", "moveTargetMode1"]);
 }
 
 /**
@@ -183,11 +221,13 @@ function updateJunkTargets() {
  * of the controlling checkbox.
  */
 function updateJunkRetention() {
-  onCheckItem("server.purgeSpamInterval", ["server.purgeSpam", "server.moveOnSpam"]);
+  onCheckItem("server.purgeSpamInterval", [
+    "server.purgeSpam",
+    "server.moveOnSpam",
+  ]);
 }
 
-function onSave()
-{
+function onSave() {
   onSaveWhiteList();
 }
 
@@ -195,33 +235,32 @@ function onSave()
  * Propagate changes to the whitelist menu list back to
  * our hidden wsm element.
  */
-function onSaveWhiteList()
-{
+function onSaveWhiteList() {
   var wList = document.getElementById("whiteListAbURI");
   var wlArray = [];
 
-  for (var i = 0; i < wList.getRowCount(); i++)
-  {
+  for (let i = 0; i < wList.getRowCount(); i++) {
     // Due to bug 448582, do not trust any properties of the listitems
     // as they may not return the right value or may even not exist.
     // Always get the attributes only.
     var wlNode = wList.getItemAtIndex(i);
-    if (wlNode.getAttribute("checked") == "true") {
+    if (wlNode.firstElementChild.getAttribute("checked") == "true") {
       let abURI = wlNode.getAttribute("value");
       wlArray.push(abURI);
     }
   }
   var wlValue = wlArray.join(" ");
-  document.getElementById("server.whiteListAbURI").setAttribute("value", wlValue);
-  document.getElementById("server.useWhiteList").checked = (wlValue != "");
+  document
+    .getElementById("server.whiteListAbURI")
+    .setAttribute("value", wlValue);
+  document.getElementById("server.useWhiteList").checked = wlValue != "";
 }
 
 /**
  * Called when a new value is chosen in one of the junk target folder pickers.
  * Sets the menu label according to the folder name.
  */
-function onActionTargetChange(aEvent, aWSMElementId)
-{
+function onActionTargetChange(aEvent, aWSMElementId) {
   let folder = aEvent.target._folder;
   document.getElementById(aWSMElementId).value = folder.URI;
   document.getElementById("actionFolderPopup").selectFolder(folder);
@@ -231,8 +270,7 @@ function onActionTargetChange(aEvent, aWSMElementId)
  * Enumerates over the "ISPDL" directories, calling buildServerFilterListFromDir
  * for each one.
  */
-function buildServerFilterMenuList()
-{
+function buildServerFilterMenuList() {
   const KEY_ISP_DIRECTORY_LIST = "ISPDL";
   let ispHeaderList = document.getElementById("useServerFilterList");
 
@@ -240,16 +278,20 @@ function buildServerFilterMenuList()
   ispHeaderList.removeAllItems();
 
   // Now walk through the isp directories looking for sfd files.
-  let ispDirectories = Services.dirsvc.get(KEY_ISP_DIRECTORY_LIST,
-                                           Ci.nsISimpleEnumerator);
+  let ispDirectories = Services.dirsvc.get(
+    KEY_ISP_DIRECTORY_LIST,
+    Ci.nsISimpleEnumerator
+  );
 
   let menuEntries = [];
-  while (ispDirectories.hasMoreElements())
-  {
-    let ispDirectory = ispDirectories.getNext()
-                                     .QueryInterface(Ci.nsIFile);
-    if (ispDirectory)
-      menuEntries.push.apply(menuEntries, buildServerFilterListFromDir(ispDirectory, menuEntries));
+  while (ispDirectories.hasMoreElements()) {
+    let ispDirectory = ispDirectories.getNext().QueryInterface(Ci.nsIFile);
+    if (ispDirectory) {
+      menuEntries.push.apply(
+        menuEntries,
+        buildServerFilterListFromDir(ispDirectory, menuEntries)
+      );
+    }
   }
 
   menuEntries.sort((a, b) => a.localeCompare(b));
@@ -266,22 +308,19 @@ function buildServerFilterMenuList()
  * @param aDir              directory to look for .sfd files
  * @param aExistingEntries  Filter names already found.
  */
-function buildServerFilterListFromDir(aDir, aExistingEntries)
-{
+function buildServerFilterListFromDir(aDir, aExistingEntries) {
   let newEntries = [];
   // Now iterate over each file in the directory looking for .sfd files.
   const kSuffix = ".sfd";
-  let entries = aDir.directoryEntries
-                    .QueryInterface(Ci.nsIDirectoryEnumerator);
 
-  while (entries.hasMoreElements()) {
-    let entry = entries.nextFile;
+  for (let entry of aDir.directoryEntries) {
     // we only care about files that end in .sfd
     if (entry.isFile() && entry.leafName.endsWith(kSuffix)) {
       let fileName = entry.leafName.slice(0, -kSuffix.length);
       // If we've already added an item with this name, then don't add it again.
-      if (aExistingEntries.indexOf(fileName) == -1)
+      if (!aExistingEntries.includes(fileName)) {
         newEntries.push(fileName);
+      }
     }
   }
   return newEntries;
@@ -290,7 +329,11 @@ function buildServerFilterListFromDir(aDir, aExistingEntries)
 /**
  * Open the Preferences dialog on the Junk settings tab.
  */
-function showGlobalJunkPrefs()
-{
-  openPrefsFromAccountManager("paneSecurity", "junkTab", null, "junk_pane");
+function showGlobalJunkPrefs() {
+  openPrefsFromAccountManager(
+    "panePrivacy",
+    "privacyJunkCategory",
+    null,
+    "junk_pane"
+  );
 }
