@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -7,22 +6,60 @@
 
 var gAccount;
 
-function onInit(aPageId, aServerId) {
-  var accountName = document.getElementById("server.prettyName");
-  var title = document.querySelector("#am-main-title .dialogheader-title");
-  var defaultTitle = title.getAttribute("defaultTitle");
-  var titleValue;
+/**
+ * Initialize am-main account settings page when it gets shown.
+ * Update an account's main settings title and set up signature items.
+ */
+function onInit() {
+  setAccountTitle();
+  setupSignatureItems();
+  Services.obs.addObserver(
+    onDefaultIdentityChange,
+    "account-default-identity-changed"
+  );
+}
 
+window.addEventListener("unload", function() {
+  Services.obs.removeObserver(
+    onDefaultIdentityChange,
+    "account-default-identity-changed"
+  );
+});
+
+/**
+ * If the default identity for the current account changes, loads the values
+ * from the new default identity.
+ */
+function onDefaultIdentityChange(subject, topic, data) {
+  if (data == gAccount.key) {
+    initIdentityValues(subject.QueryInterface(Ci.nsIMsgIdentity));
+  }
+}
+
+/**
+ * Handle the blur event of the #server.prettyName pref input.
+ * Update account name in account manager tree and account settings' main title.
+ *
+ * @param {Event} event - Blur event from the pretty name input.
+ */
+function serverPrettyNameOnBlur(event) {
+  parent.setAccountLabel(gAccount.key, event.target.value);
+  setAccountTitle();
+}
+
+/**
+ * Update an account's main settings title with the account name if applicable.
+ */
+function setAccountTitle() {
+  let accountName = document.getElementById("server.prettyName");
+  let title = document.querySelector("#am-main-title .dialogheader-title");
+  let titleValue = title.getAttribute("defaultTitle");
   if (accountName.value) {
-    titleValue = defaultTitle + " - <" + accountName.value + ">";
-  } else {
-    titleValue = defaultTitle;
+    titleValue += " - " + accountName.value;
   }
 
   title.setAttribute("value", titleValue);
   document.title = titleValue;
-
-  setupSignatureItems();
 }
 
 function onPreInit(account, accountValues) {
@@ -59,16 +96,12 @@ function manageIdentities() {
 
   parent.gSubDialog.open(
     "chrome://messenger/content/am-identities-list.xhtml",
-    null,
-    args,
-    onCloseIdentities
+    { closingCallback: onCloseIdentities },
+    args
   );
 
   function onCloseIdentities() {
     if (args.result) {
-      // now re-initialize the default identity settings in case they changed
-      identity = gAccount.defaultIdentity; // Refetch the default identity in case it changed.
-      initIdentityValues(identity);
       // Refresh the SMTP list in case the user changed server properties
       // from the identity dialog.
       loadSMTPServerList();
