@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,8 +10,14 @@ var { MailServices } = ChromeUtils.import(
 var gServer;
 var gDialog;
 
+window.addEventListener("DOMContentLoaded", onLoad);
+
 document.addEventListener("dialogdisclosure", showInfo);
 document.addEventListener("dialogaccept", onAccept);
+document.subDialogSetDefaultFocus = isInitialFocus => {
+  gDialog.getButton("cancel").focus();
+  delete document.subDialogSetDefaultFocus;
+};
 
 function onLoad(event) {
   gServer = window.arguments[0].account.incomingServer;
@@ -54,7 +59,6 @@ function onLoad(event) {
   }
 
   enableRemove();
-  window.sizeToContent();
 }
 
 function enableRemove() {
@@ -83,7 +87,6 @@ function openLocalDirectory() {
 }
 
 function showInfo() {
-  let removeAccountBox = document.getElementById("removeAccountBox");
   let descs = document.querySelectorAll("vbox.indent");
   for (let desc of descs) {
     desc.collapsed = false;
@@ -98,7 +101,7 @@ function showInfo() {
     document.getElementById("localAccount").collapsed = false;
   }
 
-  window.resizeBy(0, removeAccountBox.getBoundingClientRect().height);
+  parent.gSubDialog._topDialog.resizeDialog();
   gDialog.getButton("disclosure").disabled = true;
   gDialog.getButton("disclosure").blur();
 }
@@ -128,16 +131,13 @@ function removeAccount() {
       window.arguments[0].result = false;
     }
 
-    document.getElementById("status").selectedPanel = document.getElementById(
-      "success"
-    );
+    document.getElementById("success").hidden = false;
   } catch (ex) {
-    document.getElementById("status").selectedPanel = document.getElementById(
-      "failure"
-    );
+    document.getElementById("failure").hidden = false;
     Cu.reportError("Failure to remove account: " + ex);
     window.arguments[0].result = false;
   }
+  document.getElementById("progress").hidden = true;
 }
 
 function onAccept(event) {
@@ -158,7 +158,8 @@ function onAccept(event) {
   gDialog.getButton("accept").removeAttribute("accesskey");
   gDialog.buttons = "accept";
 
-  document.getElementById("infoPane").selectedIndex = 1;
+  document.getElementById("removeAccountSection").hidden = true;
+  document.getElementById("confirmationSection").hidden = false;
   window.sizeToContent();
 
   removeAccount();
