@@ -228,18 +228,6 @@
     }                                              \
   }
 
-//
-// nsMsgKeyArray
-//
-#define NS_MSGKEYARRAY_CONTRACTID "@mozilla.org/messenger/msgkeyarray;1"
-
-#define NS_MSGKEYARRAY_CID                           \
-  {                                                  \
-    0x86989d1d, 0xc8a1, 0x4e8e, {                    \
-      0xaa, 0xe6, 0xd0, 0xda, 0xbc, 0xac, 0xd8, 0xc2 \
-    }                                                \
-  }
-
 //
 // nsMsgWindow
 //
@@ -252,18 +240,6 @@
     }                                              \
   }
 
-//
-// Print Engine...
-//
-#define NS_MSGPRINTENGINE_CONTRACTID "@mozilla.org/messenger/msgPrintEngine;1"
-
-#define NS_MSG_PRINTENGINE_CID                     \
-  {                                                \
-    0x91fd6b19, 0xe0bc, 0x11d3, {                  \
-      0x8f, 0x97, 0x0, 0x0, 0x64, 0x65, 0x73, 0x74 \
-    }                                              \
-  }
-
 #define NS_MSGLOGONREDIRECTORSERVICE_CONTRACTID \
   "@mozilla.org/messenger/msglogonredirector;1"
 
@@ -287,23 +263,14 @@
     }                                                \
   }
 
-#define NS_MSGLOCALFOLDERCOMPACTOR_CONTRACTID \
-  "@mozilla.org/messenger/localfoldercompactor;1"
+// nsMsgFolderCompactor
+#define NS_MSGFOLDERCOMPACTOR_CONTRACTID \
+  "@mozilla.org/messenger/foldercompactor;1"
 
-#define NS_MSGLOCALFOLDERCOMPACTOR_CID               \
+#define NS_MSGFOLDERCOMPACTOR_CID                    \
   {                                                  \
-    0x7d1d315c, 0xe5c6, 0x11d4, {                    \
-      0xa5, 0xb7, 0x00, 0x60, 0xb0, 0xfc, 0x04, 0xb7 \
-    }                                                \
-  }
-
-#define NS_MSGOFFLINESTORECOMPACTOR_CONTRACTID \
-  "@mozilla.org/messenger/offlinestorecompactor;1"
-
-#define NS_MSG_OFFLINESTORECOMPACTOR_CID             \
-  {                                                  \
-    0x2db43d16, 0xe5c8, 0x11d4, {                    \
-      0xa5, 0xb7, 0x00, 0x60, 0xb0, 0xfc, 0x04, 0xb7 \
+    0x56c4c2ac, 0xfe4a, 0x4528, {                    \
+      0xaa, 0x78, 0xf8, 0xfb, 0x57, 0x9b, 0x02, 0x9c \
     }                                                \
   }
 
@@ -380,6 +347,18 @@
     }                                                \
   }
 
+//
+// nsMsgDBViewService
+//
+#define NS_MSGDBVIEWSERVICE_CONTRACTID \
+  "@mozilla.org/msgDBView/msgDBViewService;1"
+#define NS_MSGDBVIEWSERVICE_CID                      \
+  {                                                  \
+    0xbcf6afbe, 0x7d4f, 0x11ec, {                    \
+      0x90, 0x92, 0xeb, 0x4f, 0xed, 0x0a, 0x5a, 0xaa \
+    }                                                \
+  }
+
 //
 // nsMsgAccountManager
 //
@@ -485,19 +464,6 @@
     }                                                \
   }
 
-//
-// nsMessengerContentHandler
-//
-#define NS_MESSENGERCONTENTHANDLER_CID               \
-  {                                                  \
-    0x57e1bcbb, 0x1fba, 0x47e7, {                    \
-      0xb9, 0x6b, 0xf5, 0x9e, 0x39, 0x24, 0x73, 0xb0 \
-    }                                                \
-  }
-
-#define NS_MESSENGERCONTENTHANDLER_CONTRACTID \
-  NS_CONTENT_HANDLER_CONTRACTID_PREFIX "application/x-message-display"
-
 //
 // nsMsgShutdownService
 //
@@ -548,4 +514,30 @@
 #define NS_FOLDER_FACTORY_CONTRACTID_PREFIX \
   NS_FOLDER_FACTORY_CONTRACTID "?name="
 
+#define NS_BASECOMMANDCONTROLLER_CONTRACTID \
+  "@mozilla.org/embedcomp/base-command-controller;1"
+#define NS_BASECOMMANDCONTROLLER_CID                 \
+  {                                                  \
+    0xbf88b48c, 0xfd8e, 0x40b4, {                    \
+      0xba, 0x36, 0xc7, 0xc3, 0xad, 0x6d, 0x8a, 0xc9 \
+    }                                                \
+  }
+
+#define NS_TRANSACTIONMANAGER_CONTRACTID "@mozilla.org/transactionmanager;1"
+#define NS_TRANSACTIONMANAGER_CID                   \
+  {                                                 \
+    0x9c8f9601, 0x801a, 0x11d2, {                   \
+      0x98, 0xba, 0x0, 0x80, 0x5f, 0x29, 0x7d, 0x89 \
+    }                                               \
+  }
+
+#define NS_SYNCSTREAMLISTENER_CONTRACTID \
+  "@mozilla.org/network/sync-stream-listener;1"
+#define NS_SYNCSTREAMLISTENER_CID                    \
+  {                                                  \
+    0x439400d3, 0x6f23, 0x43db, {                    \
+      0x8b, 0x06, 0x8a, 0xaf, 0xe1, 0x86, 0x9b, 0xd8 \
+    }                                                \
+  }
+
 #endif  // nsMessageBaseCID_h__
