@@ -18,15 +18,17 @@
 #include "nsMsgAccount.h"
 #include "nsIMsgAccount.h"
 #include "nsIMsgAccountManager.h"
+#include "nsIObserverService.h"
+#include "mozilla/Services.h"
 #include "nsServiceManagerUtils.h"
 #include "nsMemory.h"
 #include "nsComponentManagerUtils.h"
 #include "nsMsgUtils.h"
-#include "nsArrayUtils.h"
 
 NS_IMPL_ISUPPORTS(nsMsgAccount, nsIMsgAccount)
 
-nsMsgAccount::nsMsgAccount() : mTriedToGetServer(false) {}
+nsMsgAccount::nsMsgAccount()
+    : m_identitiesValid(false), mTriedToGetServer(false) {}
 
 nsMsgAccount::~nsMsgAccount() {}
 
@@ -126,7 +128,7 @@ nsMsgAccount::SetIncomingServer(nsIMsgIncomingServer* aIncomingServer) {
     nsCOMPtr<nsIFolderListener> mailSession =
         do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
-    mailSession->OnItemAdded(nullptr, rootFolder);
+    mailSession->OnFolderAdded(nullptr, rootFolder);
     nsCOMPtr<nsIMsgFolderNotificationService> notifier(
         do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -138,18 +140,12 @@ nsMsgAccount::SetIncomingServer(nsIMsgIncomingServer* aIncomingServer) {
 
     // Force built-in folders to be created and discovered. Then, notify
     // listeners about them.
-    nsCOMPtr<nsISimpleEnumerator> enumerator;
-    rv = rootFolder->GetSubFolders(getter_AddRefs(enumerator));
+    nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+    rv = rootFolder->GetSubFolders(subFolders);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    bool hasMore;
-    while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore) {
-      nsCOMPtr<nsISupports> item;
-      enumerator->GetNext(getter_AddRefs(item));
-
-      nsCOMPtr<nsIMsgFolder> msgFolder(do_QueryInterface(item));
-      if (!msgFolder) continue;
-      mailSession->OnItemAdded(rootFolder, msgFolder);
+    for (nsIMsgFolder* msgFolder : subFolders) {
+      mailSession->OnFolderAdded(rootFolder, msgFolder);
       notifier->NotifyFolderAdded(msgFolder);
     }
   }
@@ -243,7 +239,16 @@ nsMsgAccount::SetDefaultIdentity(nsIMsgIdentity* aDefaultIdentity) {
   m_identities.RemoveElementAt(position);
   m_identities.InsertElementAt(0, aDefaultIdentity);
 
-  return saveIdentitiesPref();
+  nsresult rv = saveIdentitiesPref();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
+  if (obs) {
+    obs->NotifyObservers(aDefaultIdentity, "account-default-identity-changed",
+                         NS_ConvertUTF8toUTF16(m_accountKey).get());
+  }
+
+  return NS_OK;
 }
 
 /* void addIdentity (in nsIMsgIdentity identity); */
@@ -293,10 +298,17 @@ nsMsgAccount::AddIdentity(nsIMsgIdentity* identity) {
     }
 
     m_prefs->SetCharPref("identities", newIdentityList);
+
+    // now add it to the in-memory list
+    m_identities.AppendElement(identity);
+
+    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
+    if (obs) {
+      obs->NotifyObservers(identity, "account-identity-added",
+                           NS_ConvertUTF8toUTF16(key).get());
+    }
   }
 
-  // now add it to the in-memory list
-  m_identities.AppendElement(identity);
   return NS_OK;
 }
 
@@ -309,10 +321,23 @@ nsMsgAccount::RemoveIdentity(nsIMsgIdentity* aIdentity) {
   // At least one identity must stay after the delete.
   NS_ENSURE_TRUE(m_identities.Length() > 1, NS_ERROR_FAILURE);
 
+  nsCString key;
+  nsresult rv = aIdentity->GetKey(key);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   if (!m_identities.RemoveElement(aIdentity)) {
     return NS_ERROR_FAILURE;
   }
-  // clear out the actual pref values associated with the identity
+
+  // Notify before clearing the pref values, so we do not get the superfluous
+  // update notifications.
+  nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
+  if (obs) {
+    obs->NotifyObservers(nullptr, "account-identity-removed",
+                         NS_ConvertUTF8toUTF16(key).get());
+  }
+
+  // Clear out the actual pref values associated with the identity.
   aIdentity->ClearAllValues();
   return saveIdentitiesPref();
 }
@@ -367,8 +392,13 @@ nsMsgAccount::ToString(nsAString& aResult) {
 
 NS_IMETHODIMP
 nsMsgAccount::ClearAllValues() {
-  nsresult rv = getPrefService();
+  nsTArray<nsCString> prefNames;
+  nsresult rv = m_prefs->GetChildList("", prefNames);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return m_prefs->DeleteBranch("");
+  for (auto& prefName : prefNames) {
+    m_prefs->ClearUserPref(prefName.get());
+  }
+
+  return NS_OK;
 }
