@@ -4,6 +4,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMsgContentPolicy.h"
+#include "nsIMsgMailSession.h"
 #include "nsIPermissionManager.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
@@ -22,7 +23,6 @@
 #include "nsContentPolicyUtils.h"
 #include "nsFrameLoaderOwner.h"
 #include "nsFrameLoader.h"
-#include "nsIWebProgress.h"
 #include "nsMsgUtils.h"
 #include "nsThreadUtils.h"
 #include "mozilla/mailnews/MimeHeaderParser.h"
@@ -31,6 +31,7 @@
 #include "nsILoadInfo.h"
 #include "nsSandboxFlags.h"
 #include "nsQueryObject.h"
+#include "mozilla/dom/WindowGlobalParent.h"
 
 static const char kBlockRemoteImages[] =
     "mailnews.message_display.disable_remote_image";
@@ -47,8 +48,8 @@ using namespace mozilla::mailnews;
 #define kBlockRemoteContent 1
 #define kAllowRemoteContent 2
 
-NS_IMPL_ISUPPORTS(nsMsgContentPolicy, nsIContentPolicy, nsIWebProgressListener,
-                  nsIMsgContentPolicy, nsIObserver, nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS(nsMsgContentPolicy, nsIContentPolicy, nsIMsgContentPolicy,
+                  nsIObserver, nsISupportsWeakReference)
 
 nsMsgContentPolicy::nsMsgContentPolicy() { mBlockRemoteImages = true; }
 
@@ -114,8 +115,8 @@ bool nsMsgContentPolicy::ShouldAcceptRemoteContentForSender(
   mozilla::OriginAttributes attrs;
   RefPtr<mozilla::BasePrincipal> principal =
       mozilla::BasePrincipal::CreateContentPrincipal(mailURI, attrs);
-  rv = mPermissionManager->TestPermissionFromPrincipal(
-      principal, NS_LITERAL_CSTRING("image"), &permission);
+  rv = mPermissionManager->TestPermissionFromPrincipal(principal, "image"_ns,
+                                                       &permission);
   NS_ENSURE_SUCCESS(rv, false);
 
   // Only return true if the permission manager has an explicit allow
@@ -143,13 +144,12 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
                                const nsACString& aMimeGuess,
                                int16_t* aDecision) {
   nsresult rv = NS_OK;
-  uint32_t aContentType = aLoadInfo->GetExternalContentPolicyType();
+  ExtContentPolicyType aContentType = aLoadInfo->GetExternalContentPolicyType();
   nsCOMPtr<nsISupports> aRequestingContext;
-  if (aContentType == nsIContentPolicy::TYPE_DOCUMENT)
+  if (aContentType == ExtContentPolicy::TYPE_DOCUMENT)
     aRequestingContext = aLoadInfo->ContextForTopLevelLoad();
   else
     aRequestingContext = aLoadInfo->LoadingNode();
-  nsCOMPtr<nsIPrincipal> aRequestPrincipal = aLoadInfo->TriggeringPrincipal();
   nsCOMPtr<nsIPrincipal> loadingPrincipal = aLoadInfo->GetLoadingPrincipal();
   nsCOMPtr<nsIURI> aRequestingLocation;
   if (loadingPrincipal) {
@@ -174,6 +174,8 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
 #ifdef DEBUG_MsgContentPolicy
   fprintf(stderr, "aContentType: %d\naContentLocation = %s\n", aContentType,
           aContentLocation->GetSpecOrDefault().get());
+  fprintf(stderr, "aRequestingContext is %s\n",
+          aRequestingContext ? "not null" : "null");
 #endif
 
 #ifndef MOZ_THUNDERBIRD
@@ -190,22 +192,12 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
 
   switch (aContentType) {
       // Plugins (nsIContentPolicy::TYPE_OBJECT) are blocked on document load.
-    case nsIContentPolicy::TYPE_DOCUMENT:
+    case ExtContentPolicy::TYPE_DOCUMENT:
       // At this point, we have no intention of supporting a different JS
       // setting on a subdocument, so we don't worry about TYPE_SUBDOCUMENT
       // here.
 
-      // If the timing were right, we'd enable JavaScript on the docshell
-      // for non mailnews URIs here.  However, at this point, the
-      // old document may still be around, so we can't do any enabling just yet.
-      // Instead, we apply the policy in
-      // nsIWebProgressListener::OnLocationChange. For now, we explicitly
-      // disable JavaScript in order to be safe rather than sorry, because
-      // OnLocationChange isn't guaranteed to necessarily be called soon enough
-      // to disable it in time (though bz says it _should_ be called soon enough
-      // "in all sane cases").
-      rv = SetDisableItemsOnMailNewsUrlDocshells(aContentLocation,
-                                                 aRequestingContext);
+      rv = SetDisableItemsOnMailNewsUrlDocshells(aContentLocation, aLoadInfo);
       // if something went wrong during the tweaking, reject this content
       if (NS_FAILED(rv)) {
         NS_WARNING("Failed to set disable items on docShells");
@@ -214,7 +206,7 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
       }
       break;
 
-    case nsIContentPolicy::TYPE_CSP_REPORT:
+    case ExtContentPolicy::TYPE_CSP_REPORT:
       // We cannot block CSP reports.
       *aDecision = nsIContentPolicy::ACCEPT;
       return NS_OK;
@@ -300,20 +292,39 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
   // Protocols like ftp are always blocked.
   if (ShouldBlockUnexposedProtocol(aContentLocation)) return NS_OK;
 
+  // Mailnews URIs are not loaded in child processes, so I think that beyond
+  // here, if we're in a child process, the decision will always be accept.
+  //
+  // targetContext->Canonical does not work in a child process, so we can't
+  // really move on anyway.
+  if (!XRE_IsParentProcess()) {
+    *aDecision = nsIContentPolicy::ACCEPT;
+    return NS_OK;
+  }
+
   // Find out the URI that originally initiated the set of requests for this
   // context.
-  nsCOMPtr<nsIURI> originatorLocation;
-  if (!aRequestingContext && aRequestPrincipal) {
-    // Can get the URI directly from the principal.
-    BasePrincipal::Cast(aRequestPrincipal)
-        ->GetURI(getter_AddRefs(originatorLocation));
-  } else {
-    rv = GetOriginatingURIForContext(aRequestingContext,
-                                     getter_AddRefs(originatorLocation));
-    if (NS_SUCCEEDED(rv) && !originatorLocation) return NS_OK;
-  }
+  RefPtr<mozilla::dom::BrowsingContext> targetContext;
+  rv = aLoadInfo->GetTargetBrowsingContext(getter_AddRefs(targetContext));
   NS_ENSURE_SUCCESS(rv, NS_OK);
 
+  if (!targetContext) {
+    *aDecision = nsIContentPolicy::ACCEPT;
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIURI> originatorLocation;
+  dom::CanonicalBrowsingContext* cbc = targetContext->Canonical();
+  if (cbc) {
+    dom::WindowGlobalParent* wgp = cbc->GetCurrentWindowGlobal();
+    if (wgp) {
+      originatorLocation = wgp->GetDocumentURI();
+    }
+  }
+  if (!originatorLocation) {
+    return NS_OK;
+  }
+
 #ifdef DEBUG_MsgContentPolicy
   fprintf(stderr, "originatorLocation = %s\n",
           originatorLocation->GetSpecOrDefault().get());
@@ -339,34 +350,12 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
     return NS_OK;
   }
 
-  // Extract the windowtype to handle compose windows separately from mail
-  if (aRequestingContext) {
-    nsCOMPtr<nsIMsgCompose> msgCompose =
-        GetMsgComposeForContext(aRequestingContext);
-    // Work out if we're in a compose window or not.
-    if (msgCompose) {
-      ComposeShouldLoad(msgCompose, aRequestingContext, aContentLocation,
-                        aDecision);
-      return NS_OK;
-    }
-  }
-
-  // Allow content when using a remote page.
-  bool isHttp;
-  bool isHttps;
-  rv = originatorLocation->SchemeIs("http", &isHttp);
-  nsresult rv2 = originatorLocation->SchemeIs("https", &isHttps);
-  if (NS_SUCCEEDED(rv) && NS_SUCCEEDED(rv2) && (isHttp || isHttps)) {
-    *aDecision = nsIContentPolicy::ACCEPT;
-    return NS_OK;
-  }
-
   uint32_t permission;
   mozilla::OriginAttributes attrs;
   RefPtr<mozilla::BasePrincipal> principal =
       mozilla::BasePrincipal::CreateContentPrincipal(aContentLocation, attrs);
-  mPermissionManager->TestPermissionFromPrincipal(
-      principal, NS_LITERAL_CSTRING("image"), &permission);
+  mPermissionManager->TestPermissionFromPrincipal(principal, "image"_ns,
+                                                  &permission);
   switch (permission) {
     case nsIPermissionManager::UNKNOWN_ACTION: {
       // No exception was found for this location.
@@ -382,8 +371,28 @@ nsMsgContentPolicy::ShouldLoad(nsIURI* aContentLocation, nsILoadInfo* aLoadInfo,
     }
   }
 
+  // Handle compose windows separately from mail. Work out if we're in a compose
+  // window or not.
+  nsCOMPtr<nsIMsgCompose> msgCompose =
+      GetMsgComposeForBrowsingContext(targetContext);
+  if (msgCompose) {
+    ComposeShouldLoad(msgCompose, aRequestingContext, originatorLocation,
+                      aContentLocation, aDecision);
+    return NS_OK;
+  }
+
+  // Allow content when using a remote page.
+  bool isHttp;
+  bool isHttps;
+  rv = originatorLocation->SchemeIs("http", &isHttp);
+  nsresult rv2 = originatorLocation->SchemeIs("https", &isHttps);
+  if (NS_SUCCEEDED(rv) && NS_SUCCEEDED(rv2) && (isHttp || isHttps)) {
+    *aDecision = nsIContentPolicy::ACCEPT;
+    return NS_OK;
+  }
+
   // The default decision is still to reject.
-  ShouldAcceptContentForPotentialMsg(originatorLocation, aContentLocation,
+  ShouldAcceptContentForPotentialMsg(aRequestingLocation, aContentLocation,
                                      aDecision);
   return NS_OK;
 }
@@ -442,10 +451,18 @@ bool nsMsgContentPolicy::IsExposedProtocol(nsIURI* aContentLocation) {
   // admitted purely based on their scheme.
   // news, snews, nntp, imap and mailbox are checked before the call
   // to this function by matching content location and requesting location.
-  if (contentScheme.LowerCaseEqualsLiteral("mailto") ||
-      contentScheme.LowerCaseEqualsLiteral("addbook") ||
-      contentScheme.LowerCaseEqualsLiteral("about"))
+  if (contentScheme.LowerCaseEqualsLiteral("mailto")) return true;
+
+  if (contentScheme.LowerCaseEqualsLiteral("about")) {
+    // We want to allow about pages to load content freely. But not about:blank.
+    nsAutoCString fullSpec;
+    rv = aContentLocation->GetSpec(fullSpec);
+    NS_ENSURE_SUCCESS(rv, false);
+    if (fullSpec.EqualsLiteral("about:blank")) {
+      return false;
+    }
     return true;
+  }
 
   // check if customized exposed scheme
   if (mCustomExposedProtocols.Contains(contentScheme)) return true;
@@ -476,8 +493,17 @@ bool nsMsgContentPolicy::IsExposedProtocol(nsIURI* aContentLocation) {
 bool nsMsgContentPolicy::ShouldBlockUnexposedProtocol(
     nsIURI* aContentLocation) {
   // Error condition - we must return true so that we block.
+
+  // about:blank is "web", it should not be blocked.
+  nsAutoCString fullSpec;
+  nsresult rv = aContentLocation->GetSpec(fullSpec);
+  NS_ENSURE_SUCCESS(rv, true);
+  if (fullSpec.EqualsLiteral("about:blank")) {
+    return false;
+  }
+
   bool isHttp;
-  nsresult rv = aContentLocation->SchemeIs("http", &isHttp);
+  rv = aContentLocation->SchemeIs("http", &isHttp);
   NS_ENSURE_SUCCESS(rv, true);
 
   bool isHttps;
@@ -591,36 +617,29 @@ void nsMsgContentPolicy::NotifyContentWasBlocked(nsIURI* aOriginatorLocation,
   }
 
   nsCString resourceURI;
-  rv = msgUrl->GetUri(getter_Copies(resourceURI));
+  rv = msgUrl->GetUri(resourceURI);
   NS_ENSURE_SUCCESS_VOID(rv);
 
-  nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl(
-      do_QueryInterface(aOriginatorLocation, &rv));
+  // Get the window from the session since not all urls have a msgWindow.
+  nsCOMPtr<nsIMsgMailSession> mailSession(
+      do_GetService("@mozilla.org/messenger/services/session;1", &rv));
+  NS_ENSURE_SUCCESS_VOID(rv);
+  nsCOMPtr<nsIMsgWindow> msgWindow;
+  rv = mailSession->GetTopmostMsgWindow(getter_AddRefs(msgWindow));
   NS_ENSURE_SUCCESS_VOID(rv);
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  rv = GetMsgDBHdrFromURI(resourceURI.get(), getter_AddRefs(msgHdr));
+  rv = GetMsgDBHdrFromURI(resourceURI, getter_AddRefs(msgHdr));
   if (NS_FAILED(rv)) {
-    // Maybe we can get a dummy header.
-    nsCOMPtr<nsIMsgWindow> msgWindow;
-    rv = mailnewsUrl->GetMsgWindow(getter_AddRefs(msgWindow));
-    if (msgWindow) {
-      nsCOMPtr<nsIMsgHeaderSink> msgHdrSink;
-      rv = msgWindow->GetMsgHeaderSink(getter_AddRefs(msgHdrSink));
-      if (msgHdrSink)
-        rv = msgHdrSink->GetDummyMsgHeader(getter_AddRefs(msgHdr));
-    }
+    nsCOMPtr<nsIMsgHeaderSink> msgHdrSink;
+    msgWindow->GetMsgHeaderSink(getter_AddRefs(msgHdrSink));
+    if (msgHdrSink) msgHdrSink->GetDummyMsgHeader(getter_AddRefs(msgHdr));
   }
 
-  nsCOMPtr<nsIMsgWindow> msgWindow;
-  (void)mailnewsUrl->GetMsgWindow(getter_AddRefs(msgWindow));
-  if (msgWindow) {
-    nsCOMPtr<nsIRunnable> event = new RemoteContentNotifierEvent(
-        msgWindow, msgHdr, aContentLocation, aCanOverride);
-    // Post this as an event because it can cause dom mutations, and we
-    // get called at a bad time to be causing dom mutations.
-    if (event) NS_DispatchToCurrentThread(event);
-  }
+  // Post this as an event because it can cause dom mutations, and we
+  // get called at a bad time to be causing dom mutations.
+  NS_DispatchToCurrentThread(new RemoteContentNotifierEvent(
+      msgWindow, msgHdr, aContentLocation, aCanOverride));
 }
 
 /**
@@ -632,7 +651,7 @@ void nsMsgContentPolicy::NotifyContentWasBlocked(nsIURI* aOriginatorLocation,
  * determine if we are going to allow remote content.
  */
 void nsMsgContentPolicy::ShouldAcceptContentForPotentialMsg(
-    nsIURI* aOriginatorLocation, nsIURI* aContentLocation, int16_t* aDecision) {
+    nsIURI* aRequestingLocation, nsIURI* aContentLocation, int16_t* aDecision) {
   NS_ASSERTION(
       *aDecision == nsIContentPolicy::REJECT_REQUEST,
       "AllowContentForPotentialMessage expects default decision to be reject!");
@@ -640,7 +659,7 @@ void nsMsgContentPolicy::ShouldAcceptContentForPotentialMsg(
   // Is it a mailnews url?
   nsresult rv;
   nsCOMPtr<nsIMsgMessageUrl> msgUrl(
-      do_QueryInterface(aOriginatorLocation, &rv));
+      do_QueryInterface(aRequestingLocation, &rv));
   if (NS_FAILED(rv)) {
     // It isn't a mailnews url - so we accept the load here, and let other
     // content policies make the decision if we should be loading it or not.
@@ -649,120 +668,93 @@ void nsMsgContentPolicy::ShouldAcceptContentForPotentialMsg(
   }
 
   nsCString resourceURI;
-  rv = msgUrl->GetUri(getter_Copies(resourceURI));
-  NS_ENSURE_SUCCESS_VOID(rv);
-
-  nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl(
-      do_QueryInterface(aOriginatorLocation, &rv));
+  rv = msgUrl->GetUri(resourceURI);
   NS_ENSURE_SUCCESS_VOID(rv);
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  rv = GetMsgDBHdrFromURI(resourceURI.get(), getter_AddRefs(msgHdr));
+  rv = GetMsgDBHdrFromURI(resourceURI, getter_AddRefs(msgHdr));
   if (NS_FAILED(rv)) {
     // Maybe we can get a dummy header.
-    nsCOMPtr<nsIMsgWindow> msgWindow;
-    rv = mailnewsUrl->GetMsgWindow(getter_AddRefs(msgWindow));
-    if (msgWindow) {
-      nsCOMPtr<nsIMsgHeaderSink> msgHdrSink;
-      rv = msgWindow->GetMsgHeaderSink(getter_AddRefs(msgHdrSink));
-      if (msgHdrSink)
-        rv = msgHdrSink->GetDummyMsgHeader(getter_AddRefs(msgHdr));
+    // Get the window from the session since not all urls have a msgWindow.
+    nsCOMPtr<nsIMsgMailSession> mailSession(
+        do_GetService("@mozilla.org/messenger/services/session;1"));
+    if (mailSession) {
+      nsCOMPtr<nsIMsgWindow> msgWindow;
+      mailSession->GetTopmostMsgWindow(getter_AddRefs(msgWindow));
+      if (msgWindow) {
+        nsCOMPtr<nsIMsgHeaderSink> msgHdrSink;
+        msgWindow->GetMsgHeaderSink(getter_AddRefs(msgHdrSink));
+        if (msgHdrSink) msgHdrSink->GetDummyMsgHeader(getter_AddRefs(msgHdr));
+      }
     }
   }
 
   // Get a decision on whether or not to allow remote content for this message
   // header.
-  *aDecision = ShouldAcceptRemoteContentForMsgHdr(msgHdr, aOriginatorLocation,
+  *aDecision = ShouldAcceptRemoteContentForMsgHdr(msgHdr, aRequestingLocation,
                                                   aContentLocation);
 
   // If we're not allowing the remote content, tell the nsIMsgWindow loading
   // this url that this is the case, so that the UI knows to show the remote
   // content header bar, so the user can override if they wish.
   if (*aDecision == nsIContentPolicy::REJECT_REQUEST) {
-    nsCOMPtr<nsIMsgWindow> msgWindow;
-    (void)mailnewsUrl->GetMsgWindow(getter_AddRefs(msgWindow));
-    if (msgWindow) {
-      nsCOMPtr<nsIRunnable> event =
-          new RemoteContentNotifierEvent(msgWindow, msgHdr, aContentLocation);
-      // Post this as an event because it can cause dom mutations, and we
-      // get called at a bad time to be causing dom mutations.
-      if (event) NS_DispatchToCurrentThread(event);
-    }
+    NotifyContentWasBlocked(aRequestingLocation, aContentLocation, true);
   }
 }
 
 /**
  * Content policy logic for compose windows
- *
  */
 void nsMsgContentPolicy::ComposeShouldLoad(nsIMsgCompose* aMsgCompose,
                                            nsISupports* aRequestingContext,
+                                           nsIURI* aOriginatorLocation,
                                            nsIURI* aContentLocation,
                                            int16_t* aDecision) {
   NS_ASSERTION(*aDecision == nsIContentPolicy::REJECT_REQUEST,
                "ComposeShouldLoad expects default decision to be reject!");
 
   nsCString originalMsgURI;
-  nsresult rv = aMsgCompose->GetOriginalMsgURI(getter_Copies(originalMsgURI));
+  nsresult rv = aMsgCompose->GetOriginalMsgURI(originalMsgURI);
   NS_ENSURE_SUCCESS_VOID(rv);
 
-  MSG_ComposeType composeType;
-  rv = aMsgCompose->GetType(&composeType);
-  NS_ENSURE_SUCCESS_VOID(rv);
-
-  // Only allow remote content for new mail compositions or mailto
-  // Block remote content for all other types (drafts, templates, forwards,
-  // replies, etc) unless there is an associated msgHdr which allows the load,
-  // or unless the image is being added by the user and not the quoted message
-  // content...
-  if (composeType == nsIMsgCompType::New ||
-      composeType == nsIMsgCompType::MailToUrl)
-    *aDecision = nsIContentPolicy::ACCEPT;
-  else if (!originalMsgURI.IsEmpty()) {
+  if (!originalMsgURI.IsEmpty()) {
     nsCOMPtr<nsIMsgDBHdr> msgHdr;
-    rv = GetMsgDBHdrFromURI(originalMsgURI.get(), getter_AddRefs(msgHdr));
+    rv = GetMsgDBHdrFromURI(originalMsgURI, getter_AddRefs(msgHdr));
     NS_ENSURE_SUCCESS_VOID(rv);
     *aDecision =
         ShouldAcceptRemoteContentForMsgHdr(msgHdr, nullptr, aContentLocation);
 
-    // Special case image elements. When replying to a message, we want to allow
-    // the user to add remote images to the message. But we don't want remote
-    // images that are a part of the quoted content to load. Hence we block them
-    // while the reply is created (insertingQuotedContent==true), but allow them
-    // later when the user inserts them.
-    if (*aDecision == nsIContentPolicy::REJECT_REQUEST) {
-      bool insertingQuotedContent = true;
-      aMsgCompose->GetInsertingQuotedContent(&insertingQuotedContent);
-      nsCOMPtr<mozilla::dom::Element> element =
-          do_QueryInterface(aRequestingContext);
-      RefPtr<mozilla::dom::HTMLImageElement> image =
-          mozilla::dom::HTMLImageElement::FromNodeOrNull(element);
-      if (image) {
-        if (!insertingQuotedContent) {
-          *aDecision = nsIContentPolicy::ACCEPT;
-          return;
-        }
+    if (!aOriginatorLocation->GetSpecOrDefault().EqualsLiteral(
+            "about:blank?compose")) {
+      return;
+    }
+  }
 
-        // Test whitelist.
-        uint32_t permission;
-        mozilla::OriginAttributes attrs;
-        RefPtr<mozilla::BasePrincipal> principal =
-            mozilla::BasePrincipal::CreateContentPrincipal(aContentLocation,
-                                                           attrs);
-        mPermissionManager->TestPermissionFromPrincipal(
-            principal, NS_LITERAL_CSTRING("image"), &permission);
-        if (permission == nsIPermissionManager::ALLOW_ACTION)
-          *aDecision = nsIContentPolicy::ACCEPT;
-      }
+  // We want to allow the user to add remote content, but do that only when
+  // the allowRemoteContent was set. This way quoted remoted content won't
+  // automatically load, but e.g. pasted content will load because the UI
+  // code toggles the flag.
+  nsCOMPtr<mozilla::dom::Element> element =
+      do_QueryInterface(aRequestingContext);
+  RefPtr<mozilla::dom::HTMLImageElement> image =
+      mozilla::dom::HTMLImageElement::FromNodeOrNull(element);
+  if (image) {
+    // Special case image elements.
+    bool allowRemoteContent = false;
+    aMsgCompose->GetAllowRemoteContent(&allowRemoteContent);
+    if (allowRemoteContent) {
+      *aDecision = nsIContentPolicy::ACCEPT;
+      return;
     }
   }
 }
 
-already_AddRefed<nsIMsgCompose> nsMsgContentPolicy::GetMsgComposeForContext(
-    nsISupports* aRequestingContext) {
+already_AddRefed<nsIMsgCompose>
+nsMsgContentPolicy::GetMsgComposeForBrowsingContext(
+    mozilla::dom::BrowsingContext* aBrowsingContext) {
   nsresult rv;
 
-  nsIDocShell* shell = NS_CP_GetDocShellFromContext(aRequestingContext);
+  nsIDocShell* shell = aBrowsingContext->GetDocShell();
   if (!shell) return nullptr;
   nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem(shell);
 
@@ -787,68 +779,80 @@ already_AddRefed<nsIMsgCompose> nsMsgContentPolicy::GetMsgComposeForContext(
 }
 
 nsresult nsMsgContentPolicy::SetDisableItemsOnMailNewsUrlDocshells(
-    nsIURI* aContentLocation, nsISupports* aRequestingContext) {
+    nsIURI* aContentLocation, nsILoadInfo* aLoadInfo) {
   // XXX if this class changes so that this method can be called from
   // ShouldProcess, and if it's possible for this to be null when called from
   // ShouldLoad, but not in the corresponding ShouldProcess call,
   // we need to re-think the assumptions underlying this code.
 
-  // If there's no docshell to get to, there's nowhere for the JavaScript to
-  // run, so we're already safe and don't need to disable anything.
-  if (!aRequestingContext) {
+  NS_ENSURE_ARG_POINTER(aContentLocation);
+  NS_ENSURE_ARG_POINTER(aLoadInfo);
+
+  RefPtr<mozilla::dom::BrowsingContext> browsingContext =
+      aLoadInfo->GetTargetBrowsingContext();
+  if (!browsingContext) {
     return NS_OK;
   }
 
+  // We're only worried about policy settings in content docshells.
+  if (!browsingContext->IsContent()) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIDocShell> docShell = browsingContext->GetDocShell();
+  if (!docShell) {
+    // If there's no docshell to get to, there's nowhere for the JavaScript to
+    // run, so we're already safe and don't need to disable anything.
+    return NS_OK;
+  }
+
+  // Ensure starting off unsandboxed. We sandbox later if needed.
+  MOZ_ALWAYS_SUCCEEDS(browsingContext->SetSandboxFlags(SANDBOXED_NONE));
+
   nsresult rv;
   bool isAllowedContent = !ShouldBlockUnexposedProtocol(aContentLocation);
-  nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(aContentLocation, &rv);
-  mozilla::Unused << msgUrl;
-  if (NS_FAILED(rv) && !isAllowedContent) {
+  nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(aContentLocation);
+  if (!msgUrl && !isAllowedContent) {
     // If it's not a mailnews url or allowed content url (http[s]|file) then
-    // bail; otherwise set whether js and plugins are allowed.
-    return NS_OK;
-  }
-
-  // since NS_CP_GetDocShellFromContext returns the containing docshell rather
-  // than the contained one we need, we can't use that here, so...
-  RefPtr<nsFrameLoaderOwner> flOwner = do_QueryObject(aRequestingContext);
-  NS_ENSURE_TRUE(flOwner, NS_ERROR_INVALID_POINTER);
-
-  RefPtr<nsFrameLoader> frameLoader = flOwner->GetFrameLoader();
-  NS_ENSURE_TRUE(frameLoader, NS_ERROR_INVALID_POINTER);
-
-  nsCOMPtr<nsIDocShell> docShell =
-      frameLoader->GetDocShell(mozilla::IgnoreErrors());
-  NS_ENSURE_TRUE(docShell, NS_ERROR_INVALID_POINTER);
-
-  nsCOMPtr<nsIDocShellTreeItem> docshellTreeItem(docShell);
-
-  // what sort of docshell is this?
-  int32_t itemType;
-  rv = docshellTreeItem->GetItemType(&itemType);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // we're only worried about policy settings in content docshells
-  if (itemType != nsIDocShellTreeItem::typeContent) {
+    // bail; otherwise set whether JavaScript is allowed.
     return NS_OK;
   }
 
   if (!isAllowedContent) {
     // Disable JavaScript on message URLs.
-    rv = docShell->SetAllowJavascript(false);
+    rv = browsingContext->SetAllowJavascript(false);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = docShell->SetAllowContentRetargetingOnChildren(false);
+    rv = browsingContext->SetAllowContentRetargetingOnChildren(false);
     NS_ENSURE_SUCCESS(rv, rv);
+    // NOTE! Do not set single sandboxing flags only. Sandboxing -  when used -
+    // starts off with all things sandboxed, and individual sandbox keywords
+    // will *allow* the specific feature.
+    // Disabling by setting single flags without starting off with all things
+    // sandboxed would the normal assumptions about sandboxing.
+    // The flags - contrary to the keywords - *prevent* a given feature.
+    uint32_t sandboxFlags = SANDBOX_ALL_FLAGS;
 
-    RefPtr<mozilla::dom::BrowsingContext> browsingContext =
-        docShell->GetBrowsingContext();
-    browsingContext->SetSandboxFlags(browsingContext->GetSandboxFlags() |
-                                     SANDBOXED_FORMS);
+    // Do not block links with target attribute from opening (at all).
+    // xref bug 421310 - we would like to prevent using target, but *handle*
+    // links like the target wasn't there.
+    sandboxFlags &= ~SANDBOXED_AUXILIARY_NAVIGATION;
+
+    // For some unexplicable reason, when SANDBOXED_ORIGIN is in affect, then
+    // images will not work with test --verify. So unset it.
+    sandboxFlags &= ~SANDBOXED_ORIGIN;
+
+    // Having both SANDBOXED_TOPLEVEL_NAVIGATION and
+    // SANDBOXED_TOPLEVEL_NAVIGATION_USER_ACTIVATION will generate a warning,
+    // see BothAllowTopNavigationAndUserActivationPresent. So unset it.
+    sandboxFlags &= ~SANDBOXED_TOPLEVEL_NAVIGATION_USER_ACTIVATION;
+
+    rv = browsingContext->SetSandboxFlags(sandboxFlags);
+    NS_ENSURE_SUCCESS(rv, rv);
   } else {
     // JavaScript is allowed on non-message URLs.
-    rv = docShell->SetAllowJavascript(true);
+    rv = browsingContext->SetAllowJavascript(true);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = docShell->SetAllowContentRetargetingOnChildren(true);
+    rv = browsingContext->SetAllowContentRetargetingOnChildren(true);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -940,95 +944,6 @@ NS_IMETHODIMP nsMsgContentPolicy::Observe(nsISupports* aSubject,
   return NS_OK;
 }
 
-/**
- * We implement the nsIWebProgressListener interface in order to enforce
- * settings at onLocationChange time.
- */
-NS_IMETHODIMP
-nsMsgContentPolicy::OnStateChange(nsIWebProgress* aWebProgress,
-                                  nsIRequest* aRequest, uint32_t aStateFlags,
-                                  nsresult aStatus) {
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMsgContentPolicy::OnProgressChange(nsIWebProgress* aWebProgress,
-                                     nsIRequest* aRequest,
-                                     int32_t aCurSelfProgress,
-                                     int32_t aMaxSelfProgress,
-                                     int32_t aCurTotalProgress,
-                                     int32_t aMaxTotalProgress) {
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMsgContentPolicy::OnLocationChange(nsIWebProgress* aWebProgress,
-                                     nsIRequest* aRequest, nsIURI* aLocation,
-                                     uint32_t aFlags) {
-  nsresult rv;
-
-  // If anything goes wrong and/or there's no docshell associated with this
-  // request, just give up.  The behavior ends up being "don't consider
-  // re-enabling JS on the docshell", which is the safe thing to do (and if
-  // the problem was that there's no docshell, that means that there was
-  // nowhere for any JavaScript to run, so we're already safe
-
-  nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(aWebProgress, &rv);
-  if (NS_FAILED(rv)) {
-    return NS_OK;
-  }
-
-#ifdef DEBUG
-  nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest, &rv);
-  if (NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsIDocShell> docShell2;
-    NS_QueryNotificationCallbacks(channel, docShell2);
-    NS_ASSERTION(docShell == docShell2,
-                 "aWebProgress and channel callbacks"
-                 " do not point to the same docshell");
-  }
-#endif
-
-  nsCOMPtr<nsIMsgMessageUrl> messageUrl = do_QueryInterface(aLocation, &rv);
-  mozilla::Unused << messageUrl;
-  if (NS_SUCCEEDED(rv)) {
-    // Disable javascript on message URLs.
-    rv = docShell->SetAllowJavascript(false);
-    NS_ASSERTION(NS_SUCCEEDED(rv),
-                 "Failed to set javascript disabled on docShell");
-  } else {
-    // Javascript is allowed on non-message URLs. Plugins are not.
-    rv = docShell->SetAllowJavascript(true);
-    NS_ASSERTION(NS_SUCCEEDED(rv),
-                 "Failed to set javascript allowed on docShell");
-  }
-
-  rv = docShell->SetAllowPlugins(false);
-  NS_ASSERTION(NS_SUCCEEDED(rv), "Failed to set plugins disabled on docShell");
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMsgContentPolicy::OnStatusChange(nsIWebProgress* aWebProgress,
-                                   nsIRequest* aRequest, nsresult aStatus,
-                                   const char16_t* aMessage) {
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMsgContentPolicy::OnSecurityChange(nsIWebProgress* aWebProgress,
-                                     nsIRequest* aRequest, uint32_t aState) {
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMsgContentPolicy::OnContentBlockingEvent(nsIWebProgress* aWebProgress,
-                                           nsIRequest* aRequest,
-                                           uint32_t aEvent) {
-  return NS_OK;
-}
-
 /**
  * Implementation of nsIMsgContentPolicy
  *
