@@ -10,7 +10,6 @@
 #include "nsIMsgFolderNotificationService.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMutableArray.h"
-#include "nsArrayUtils.h"
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsMsgUtils.h"
@@ -22,19 +21,17 @@ static mozilla::LazyLogModule gCopyServiceLog("MsgCopyService");
 
 nsCopySource::nsCopySource() : m_processed(false) {
   MOZ_COUNT_CTOR(nsCopySource);
-  m_messageArray = do_CreateInstance(NS_ARRAY_CONTRACTID);
 }
 
 nsCopySource::nsCopySource(nsIMsgFolder* srcFolder) : m_processed(false) {
   MOZ_COUNT_CTOR(nsCopySource);
-  m_messageArray = do_CreateInstance(NS_ARRAY_CONTRACTID);
   m_msgFolder = srcFolder;
 }
 
 nsCopySource::~nsCopySource() { MOZ_COUNT_DTOR(nsCopySource); }
 
 void nsCopySource::AddMessage(nsIMsgDBHdr* aMsg) {
-  m_messageArray->AppendElement(aMsg);
+  m_messageArray.AppendElement(aMsg);
 }
 
 // ************ nsCopyRequest *****************
@@ -129,9 +126,9 @@ void nsMsgCopyService::LogCopyRequest(const char* logMsg,
   aRequest->m_dstFolder->GetURI(destFolderUri);
   uint32_t numMsgs = 0;
   if (aRequest->m_requestType == nsCopyMessagesType &&
-      aRequest->m_copySourceArray.Length() > 0 &&
-      aRequest->m_copySourceArray[0]->m_messageArray)
-    aRequest->m_copySourceArray[0]->m_messageArray->GetLength(&numMsgs);
+      aRequest->m_copySourceArray.Length() > 0) {
+    numMsgs = aRequest->m_copySourceArray[0]->m_messageArray.Length();
+  }
   MOZ_LOG(gCopyServiceLog, mozilla::LogLevel::Info,
           ("request %p %s - src %s dest %s numItems %d type=%d", aRequest,
            logMsg, srcFolderUri.get(), destFolderUri.get(), numMsgs,
@@ -187,7 +184,9 @@ nsresult nsMsgCopyService::QueueRequest(nsCopyRequest* aRequest,
   *aCopyImmediately = true;
   nsCopyRequest* copyRequest;
 
+  // Check through previous requests to see if the copy can start immediately.
   uint32_t cnt = m_copyRequests.Length();
+
   for (uint32_t i = 0; i < cnt; i++) {
     copyRequest = m_copyRequests.ElementAt(i);
     if (aRequest->m_requestType == nsCopyFoldersType) {
@@ -205,6 +204,9 @@ nsresult nsMsgCopyService::QueueRequest(nsCopyRequest* aRequest,
       break;
     }
   }
+
+  // Queue it.
+  m_copyRequests.AppendElement(aRequest);
   return NS_OK;
 }
 
@@ -212,7 +214,6 @@ nsresult nsMsgCopyService::DoCopy(nsCopyRequest* aRequest) {
   NS_ENSURE_ARG(aRequest);
   bool copyImmediately;
   QueueRequest(aRequest, &copyImmediately);
-  m_copyRequests.AppendElement(aRequest);
   if (MOZ_LOG_TEST(gCopyServiceLog, mozilla::LogLevel::Info))
     LogCopyRequest(copyImmediately ? "DoCopy" : "QueueRequest", aRequest);
 
@@ -288,7 +289,7 @@ nsresult nsMsgCopyService::DoNextCopy() {
           // with; if we do we shall have an instance of copySource
           nsCOMPtr<nsIMsgDBHdr> aMessage;
           if (copySource) {
-            aMessage = do_QueryElementAt(copySource->m_messageArray, 0, &rv);
+            aMessage = copySource->m_messageArray[0];
             copySource->m_processed = true;
           }
           copyRequest->m_processed = true;
@@ -363,10 +364,10 @@ NS_IMPL_ISUPPORTS(nsMsgCopyService, nsIMsgCopyService)
 
 MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHODIMP nsMsgCopyService::CopyMessages(
     nsIMsgFolder* srcFolder, /* UI src folder */
-    nsIArray* messages, nsIMsgFolder* dstFolder, bool isMove,
-    nsIMsgCopyServiceListener* listener, nsIMsgWindow* window, bool allowUndo) {
+    nsTArray<RefPtr<nsIMsgDBHdr>> const& messages, nsIMsgFolder* dstFolder,
+    bool isMove, nsIMsgCopyServiceListener* listener, nsIMsgWindow* window,
+    bool allowUndo) {
   NS_ENSURE_ARG_POINTER(srcFolder);
-  NS_ENSURE_ARG_POINTER(messages);
   NS_ENSURE_ARG_POINTER(dstFolder);
 
   MOZ_LOG(gCopyServiceLog, mozilla::LogLevel::Debug, ("CopyMessages"));
@@ -377,11 +378,10 @@ MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHODIMP nsMsgCopyService::CopyMessages(
   }
   nsCopyRequest* copyRequest;
   nsCopySource* copySource = nullptr;
-  nsCOMArray<nsIMsgDBHdr> msgArray;
-  uint32_t cnt;
-  nsCOMPtr<nsIMsgDBHdr> msg;
+  nsIMsgDBHdr* msg;
   nsCOMPtr<nsIMsgFolder> curFolder;
   nsCOMPtr<nsISupports> aSupport;
+  int cnt;
   nsresult rv;
 
   // XXX TODO
@@ -390,8 +390,7 @@ MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHODIMP nsMsgCopyService::CopyMessages(
   // and has proper flags, before we start copying?
 
   // bail early if nothing to do
-  messages->GetLength(&cnt);
-  if (!cnt) {
+  if (messages.IsEmpty()) {
     if (listener) {
       listener->OnStartCopy();
       listener->OnStopCopy(NS_OK);
@@ -402,6 +401,7 @@ MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHODIMP nsMsgCopyService::CopyMessages(
   copyRequest = new nsCopyRequest();
   if (!copyRequest) return NS_ERROR_OUT_OF_MEMORY;
 
+  nsTArray<RefPtr<nsIMsgDBHdr>> unprocessed = messages.Clone();
   aSupport = do_QueryInterface(srcFolder, &rv);
 
   rv = copyRequest->Init(nsCopyMessagesType, aSupport, dstFolder, isMove,
@@ -412,21 +412,16 @@ MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHODIMP nsMsgCopyService::CopyMessages(
   if (MOZ_LOG_TEST(gCopyServiceLog, mozilla::LogLevel::Info))
     LogCopyRequest("CopyMessages request", copyRequest);
 
-  // duplicate the message array so we could sort the messages by it's
-  // folder easily
-  for (uint32_t i = 0; i < cnt; i++) {
-    nsCOMPtr<nsIMsgDBHdr> currMsg = do_QueryElementAt(messages, i);
-    msgArray.AppendObject(currMsg);
-  }
-
-  cnt = msgArray.Count();
-
+  // Build up multiple nsCopySource objects. Each holds a single source folder
+  // and all the messages in the folder that are to be copied.
+  cnt = unprocessed.Length();
   while (cnt-- > 0) {
-    msg = msgArray[cnt];
+    msg = unprocessed[cnt];
     rv = msg->GetFolder(getter_AddRefs(curFolder));
 
     if (NS_FAILED(rv)) goto done;
     if (!copySource) {
+      // Begin a folder grouping.
       copySource = copyRequest->AddNewCopySource(curFolder);
       if (!copySource) {
         rv = NS_ERROR_OUT_OF_MEMORY;
@@ -434,13 +429,16 @@ MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHODIMP nsMsgCopyService::CopyMessages(
       }
     }
 
+    // Stash message if in the current folder grouping.
     if (curFolder == copySource->m_msgFolder) {
       copySource->AddMessage(msg);
-      msgArray.RemoveObjectAt(cnt);
+      unprocessed.RemoveElementAt((size_t)cnt);
     }
 
     if (cnt == 0) {
-      cnt = msgArray.Count();
+      // Finished a folder. Start a new pass to handle any remaining messages
+      // in other folders.
+      cnt = unprocessed.Length();
       if (cnt > 0) {
         // Force to create a new one and continue grouping the messages.
         copySource = nullptr;
@@ -466,46 +464,27 @@ done:
 }
 
 NS_IMETHODIMP
-nsMsgCopyService::CopyFolders(nsIArray* folders, nsIMsgFolder* dstFolder,
-                              bool isMove, nsIMsgCopyServiceListener* listener,
-                              nsIMsgWindow* window) {
-  NS_ENSURE_ARG_POINTER(folders);
+nsMsgCopyService::CopyFolder(nsIMsgFolder* srcFolder, nsIMsgFolder* dstFolder,
+                             bool isMove, nsIMsgCopyServiceListener* listener,
+                             nsIMsgWindow* window) {
+  NS_ENSURE_ARG_POINTER(srcFolder);
   NS_ENSURE_ARG_POINTER(dstFolder);
   nsCopyRequest* copyRequest;
-  nsCopySource* copySource = nullptr;
   nsresult rv;
-  uint32_t cnt;
   nsCOMPtr<nsIMsgFolder> curFolder;
+
   nsCOMPtr<nsISupports> support;
-
-  // If cnt is zero it cannot to get this point, will be detected earlier.
-  rv = folders->GetLength(&cnt);
-  if (cnt > 1)
-    NS_ASSERTION((NS_SUCCEEDED(rv)), "More than one folders to copy");
-
-  support = do_QueryElementAt(folders, 0);
+  support = do_QueryInterface(srcFolder, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   copyRequest = new nsCopyRequest();
-  if (!copyRequest) return NS_ERROR_OUT_OF_MEMORY;
-
   rv = copyRequest->Init(nsCopyFoldersType, support, dstFolder, isMove,
                          0 /* new msg flags, not used */, EmptyCString(),
                          listener, window, false);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  curFolder = do_QueryInterface(support, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  copySource = copyRequest->AddNewCopySource(curFolder);
-  if (!copySource) rv = NS_ERROR_OUT_OF_MEMORY;
-
-  if (NS_FAILED(rv)) {
-    delete copyRequest;
-    NS_ENSURE_SUCCESS(rv, rv);
-  } else
-    rv = DoCopy(copyRequest);
-
-  return rv;
+  copyRequest->AddNewCopySource(srcFolder);
+  return DoCopy(copyRequest);
 }
 
 NS_IMETHODIMP
@@ -519,12 +498,10 @@ nsMsgCopyService::CopyFileMessage(nsIFile* file, nsIMsgFolder* dstFolder,
   nsCopyRequest* copyRequest;
   nsCopySource* copySource = nullptr;
   nsCOMPtr<nsISupports> fileSupport;
-  nsCOMPtr<nsITransactionManager> txnMgr;
 
   NS_ENSURE_ARG_POINTER(file);
   NS_ENSURE_ARG_POINTER(dstFolder);
 
-  if (window) window->GetTransactionManager(getter_AddRefs(txnMgr));
   copyRequest = new nsCopyRequest();
   if (!copyRequest) return rv;
   fileSupport = do_QueryInterface(file, &rv);
