@@ -96,10 +96,8 @@ class nsFolderCompactState : public nsIStreamListener,
   nsFolderCompactState(void);
 
   nsresult Compact(nsIMsgFolder* folder,
-                   std::function<void(nsresult)> completionFn,
+                   std::function<void(nsresult, uint64_t)> completionFn,
                    nsIMsgWindow* msgWindow);
-  // Upon completion, access the number of bytes expunged.
-  uint64_t ExpungedBytes() const { return m_totalExpungedBytes; }
 
  protected:
   virtual ~nsFolderCompactState(void);
@@ -153,8 +151,9 @@ class nsFolderCompactState : public nsIStreamListener,
   // Flag to indicate we're starting a new message, and that no data has
   // been written for it yet.
   bool m_startOfMsg;
-  //  Function which will be run when the folder compaction completes.
-  std::function<void(nsresult)> m_completionFn;
+  // Function which will be run when the folder compaction completes.
+  // Takes a result code and the number of bytes which were expunged.
+  std::function<void(nsresult, uint64_t)> m_completionFn;
   bool m_alreadyWarnedDiskSpace{false};
 };
 
@@ -219,7 +218,7 @@ nsresult nsFolderCompactState::InitDB(nsIMsgDatabase* db) {
 }
 
 nsresult nsFolderCompactState::Compact(
-    nsIMsgFolder* folder, std::function<void(nsresult)> completionFn,
+    nsIMsgFolder* folder, std::function<void(nsresult, uint64_t)> completionFn,
     nsIMsgWindow* msgWindow) {
   NS_ENSURE_ARG_POINTER(folder);
   m_completionFn = completionFn;
@@ -247,7 +246,7 @@ nsresult nsFolderCompactState::Compact(
       {
         folder->NotifyCompactCompleted();
         if (m_completionFn) {
-          m_completionFn(NS_OK);
+          m_completionFn(NS_OK, m_totalExpungedBytes);
         }
         return NS_OK;
       }
@@ -342,7 +341,7 @@ nsresult nsFolderCompactState::Compact(
   // Skipped folder, for whatever reason.
   folder->NotifyCompactCompleted();
   if (m_completionFn) {
-    m_completionFn(NS_OK);
+    m_completionFn(NS_OK, m_totalExpungedBytes);
   }
   return NS_OK;
 }
@@ -448,7 +447,7 @@ NS_IMETHODIMP nsFolderCompactState::OnStopRunningUrl(nsIURI* url,
   m_folder->SetMsgDatabase(nullptr);
 
   if (m_completionFn) {
-    m_completionFn(status);
+    m_completionFn(status, m_totalExpungedBytes);
   }
   return NS_OK;
 }
@@ -620,7 +619,7 @@ nsresult nsFolderCompactState::FinishCompact() {
              ? NS_OK
              : rv;
     m_db->SetSummaryValid(true);
-    m_folder->SetDBTransferInfo(transferInfo);
+    if (transferInfo) m_folder->SetDBTransferInfo(transferInfo);
 
     // since we're transferring info from the old db, we need to reset the
     // expunged bytes
@@ -640,7 +639,7 @@ nsresult nsFolderCompactState::FinishCompact() {
 
   m_folder->NotifyCompactCompleted();
   if (m_completionFn) {
-    m_completionFn(rv);
+    m_completionFn(rv, m_totalExpungedBytes);
   }
 
   return NS_OK;
@@ -1050,6 +1049,10 @@ done:
                     // temp folder and database
     ReleaseFolderLock();
     NS_RELEASE_THIS();  // kill self
+
+    if (m_completionFn) {
+      m_completionFn(m_status, m_totalExpungedBytes);
+    }
     return rv;
   }
   return rv;
@@ -1094,7 +1097,7 @@ nsresult nsOfflineStoreCompactState::FinishCompact() {
   ShowStatusMsg(EmptyString());
   m_folder->NotifyCompactCompleted();
   if (m_completionFn) {
-    m_completionFn(NS_OK);
+    m_completionFn(NS_OK, m_totalExpungedBytes);
   }
   return rv;
 }
@@ -1326,17 +1329,19 @@ void nsMsgFolderCompactor::NextFolder() {
     folder->GetURI(uri);
 
     // Callback for when a folder compaction completes.
-    RefPtr<nsMsgFolderCompactor> self(this);
-    auto completionFn = [self](nsresult status) {
-      RefPtr<nsFolderCompactState> compactor(self->mCompactor);
-      if (NS_FAILED(status)) {
-        // Make sure we return a failing code upon overall completion, for
-        // now try to keep going.
+    auto completionFn = [self = RefPtr<nsMsgFolderCompactor>(this),
+                         compactState = mCompactor](nsresult status,
+                                                    uint64_t expungedBytes) {
+      if (NS_SUCCEEDED(status)) {
+        self->mTotalBytesGained += expungedBytes;
+      } else {
+        // Failed. We want to keep going with the next folder, but make sure
+        // we return a failing code upon overall completion.
         self->mOverallStatus = status;
         NS_WARNING("folder compact failed.");
       }
+
       // Release our lock on the compactor - it's done.
-      self->mTotalBytesGained += compactor->ExpungedBytes();
       self->mCompactor = nullptr;
       self->NextFolder();
     };
