@@ -22,10 +22,7 @@
 #define MSGHDR_CACHE_MAX_SIZE 8192
 #define MSGHDR_CACHE_DEFAULT_SIZE 100
 
-nsMsgGroupView::nsMsgGroupView() {
-  m_dayChanged = false;
-  m_lastCurExplodedTime.tm_mday = 0;
-}
+nsMsgGroupView::nsMsgGroupView() { m_dayChanged = false; }
 
 nsMsgGroupView::~nsMsgGroupView() {}
 
@@ -40,7 +37,7 @@ nsMsgGroupView::Open(nsIMsgFolder* aFolder, nsMsgViewSortTypeValue aSortType,
   nsCOMPtr<nsIDBFolderInfo> dbFolderInfo;
   PersistFolderInfo(getter_AddRefs(dbFolderInfo));
 
-  nsCOMPtr<nsISimpleEnumerator> headers;
+  nsCOMPtr<nsIMsgEnumerator> headers;
   rv = m_db->EnumerateMessages(getter_AddRefs(headers));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -286,7 +283,7 @@ nsMsgGroupThread* nsMsgGroupView::AddHdrToThread(nsIMsgDBHdr* msgHdr,
   if (!foundThread) {
     foundThread = CreateGroupThread(m_db);
     msgThread = foundThread;
-    m_groupsTable.Put(hashKey, msgThread);
+    m_groupsTable.InsertOrUpdate(hashKey, msgThread);
     if (GroupViewUsesDummyRow()) {
       foundThread->m_dummy = true;
       msgFlags |= MSG_VIEW_FLAG_DUMMY | MSG_VIEW_FLAG_HASCHILDREN;
@@ -361,7 +358,7 @@ nsMsgGroupThread* nsMsgGroupView::AddHdrToThread(nsIMsgDBHdr* msgHdr,
 }
 
 NS_IMETHODIMP
-nsMsgGroupView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
+nsMsgGroupView::OpenWithHdrs(nsIMsgEnumerator* aHeaders,
                              nsMsgViewSortTypeValue aSortType,
                              nsMsgViewSortOrderValue aSortOrder,
                              nsMsgViewFlagsTypeValue aViewFlags,
@@ -395,10 +392,9 @@ nsMsgGroupView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
   while (NS_SUCCEEDED(rv) &&
          NS_SUCCEEDED(rv = aHeaders->HasMoreElements(&hasMore)) && hasMore) {
-    rv = aHeaders->GetNext(getter_AddRefs(supports));
-    if (NS_SUCCEEDED(rv) && supports) {
+    rv = aHeaders->GetNext(getter_AddRefs(msgHdr));
+    if (NS_SUCCEEDED(rv) && msgHdr) {
       bool notUsed;
-      msgHdr = do_QueryInterface(supports);
       AddHdrToThread(msgHdr, &notUsed);
     }
   }
@@ -461,7 +457,7 @@ nsMsgGroupView::CopyDBView(nsMsgDBView* aNewMsgDBView,
   // If grouped, we need to clone the group thread hash table.
   if (m_viewFlags & nsMsgViewFlagsType::kGroupBySort) {
     for (auto iter = m_groupsTable.Iter(); !iter.Done(); iter.Next()) {
-      newMsgDBView->m_groupsTable.Put(iter.Key(), iter.UserData());
+      newMsgDBView->m_groupsTable.InsertOrUpdate(iter.Key(), iter.UserData());
     }
   }
   return NS_OK;
@@ -474,7 +470,7 @@ nsMsgGroupView::CopyDBView(nsMsgDBView* aNewMsgDBView,
 // rebuild the new view based on the new view type. So we pass the new
 // view flags to OpenWithHdrs.
 nsresult nsMsgGroupView::RebuildView(nsMsgViewFlagsTypeValue newFlags) {
-  nsCOMPtr<nsISimpleEnumerator> headers;
+  nsCOMPtr<nsIMsgEnumerator> headers;
   if (NS_SUCCEEDED(GetMessageEnumerator(getter_AddRefs(headers)))) {
     int32_t count;
     m_dayChanged = false;
@@ -763,7 +759,6 @@ nsMsgGroupView::CellTextForColumn(int32_t aRow, const nsAString& aColumnName,
     bool rcvDate = false;
     msgHdr->GetFlags(&flags);
     aValue.Truncate();
-    nsString tmp_str;
     switch (m_sortType) {
       case nsMsgViewSortType::byReceived:
         rcvDate = true;
@@ -773,41 +768,23 @@ nsMsgGroupView::CellTextForColumn(int32_t aRow, const nsAString& aColumnName,
         GetAgeBucketValue(msgHdr, &ageBucket, rcvDate);
         switch (ageBucket) {
           case 1:
-            if (m_kTodayString.IsEmpty())
-              m_kTodayString.Adopt(GetString(u"today"));
-
-            aValue.Assign(m_kTodayString);
+            aValue.Assign(nsMsgDBView::kTodayString);
             break;
           case 2:
-            if (m_kYesterdayString.IsEmpty())
-              m_kYesterdayString.Adopt(GetString(u"yesterday"));
-
-            aValue.Assign(m_kYesterdayString);
+            aValue.Assign(nsMsgDBView::kYesterdayString);
             break;
           case 3:
-            if (m_kLastWeekString.IsEmpty())
-              m_kLastWeekString.Adopt(GetString(u"last7Days"));
-
-            aValue.Assign(m_kLastWeekString);
+            aValue.Assign(nsMsgDBView::kLastWeekString);
             break;
           case 4:
-            if (m_kTwoWeeksAgoString.IsEmpty())
-              m_kTwoWeeksAgoString.Adopt(GetString(u"last14Days"));
-
-            aValue.Assign(m_kTwoWeeksAgoString);
+            aValue.Assign(nsMsgDBView::kTwoWeeksAgoString);
             break;
           case 5:
-            if (m_kOldMailString.IsEmpty())
-              m_kOldMailString.Adopt(GetString(u"older"));
-
-            aValue.Assign(m_kOldMailString);
+            aValue.Assign(nsMsgDBView::kOldMailString);
             break;
           default:
             // Future date, error/spoofed.
-            if (m_kFutureDateString.IsEmpty())
-              m_kFutureDateString.Adopt(GetString(u"futureDate"));
-
-            aValue.Assign(m_kFutureDateString);
+            aValue.Assign(nsMsgDBView::kFutureDateString);
             break;
         }
         break;
@@ -821,22 +798,19 @@ nsMsgGroupView::CellTextForColumn(int32_t aRow, const nsAString& aColumnName,
       case nsMsgViewSortType::byStatus:
         rv = FetchStatus(m_flags[aRow], aValue);
         if (aValue.IsEmpty()) {
-          tmp_str.Adopt(GetString(u"messagesWithNoStatus"));
-          aValue.Assign(tmp_str);
+          GetString(u"messagesWithNoStatus", aValue);
         }
         break;
       case nsMsgViewSortType::byTags:
         rv = FetchTags(msgHdr, aValue);
         if (aValue.IsEmpty()) {
-          tmp_str.Adopt(GetString(u"untaggedMessages"));
-          aValue.Assign(tmp_str);
+          GetString(u"untaggedMessages", aValue);
         }
         break;
       case nsMsgViewSortType::byPriority:
         FetchPriority(msgHdr, aValue);
         if (aValue.IsEmpty()) {
-          tmp_str.Adopt(GetString(u"noPriority"));
-          aValue.Assign(tmp_str);
+          GetString(u"noPriority", aValue);
         }
         break;
       case nsMsgViewSortType::byAccount:
@@ -846,16 +820,14 @@ nsMsgGroupView::CellTextForColumn(int32_t aRow, const nsAString& aColumnName,
         FetchRecipients(msgHdr, aValue);
         break;
       case nsMsgViewSortType::byAttachments:
-        tmp_str.Adopt(GetString(flags & nsMsgMessageFlags::Attachment
-                                    ? u"attachments"
-                                    : u"noAttachments"));
-        aValue.Assign(tmp_str);
+        GetString(flags & nsMsgMessageFlags::Attachment ? u"attachments"
+                                                        : u"noAttachments",
+                  aValue);
         break;
       case nsMsgViewSortType::byFlagged:
-        tmp_str.Adopt(GetString(flags & nsMsgMessageFlags::Marked
-                                    ? u"groupFlagged"
-                                    : u"notFlagged"));
-        aValue.Assign(tmp_str);
+        GetString(
+            flags & nsMsgMessageFlags::Marked ? u"groupFlagged" : u"notFlagged",
+            aValue);
         break;
       // byLocation is a special case; we don't want to have duplicate
       // all this logic in nsMsgSearchDBView, and its hash key is what we
