@@ -18,10 +18,10 @@
 #include "nsIPrefService.h"
 #include "mozilla/Logging.h"
 #include "nsMsgFolderFlags.h"
+#include "nsITimer.h"
 #include <stdlib.h>
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
-#include "nsArrayUtils.h"
 
 static mozilla::LazyLogModule MsgPurgeLogModule("MsgPurge");
 
@@ -100,10 +100,12 @@ nsresult nsMsgPurgeService::SetupNextPurge() {
   // calling Notify. So, just release the timer here and create a new one.
   if (mPurgeTimer) mPurgeTimer->Cancel();
 
-  mPurgeTimer = do_CreateInstance("@mozilla.org/timer;1");
-  mPurgeTimer->InitWithNamedFuncCallback(
-      OnPurgeTimer, (void*)this, timeInMSUint32, nsITimer::TYPE_ONE_SHOT,
-      "nsMsgPurgeService::OnPurgeTimer");
+  nsresult rv = NS_NewTimerWithFuncCallback(
+      getter_AddRefs(mPurgeTimer), OnPurgeTimer, (void*)this, timeInMSUint32,
+      nsITimer::TYPE_ONE_SHOT, "nsMsgPurgeService::OnPurgeTimer", nullptr);
+  if (NS_FAILED(rv)) {
+    NS_WARNING("Could not start mPurgeTimer timer");
+  }
 
   return NS_OK;
 }
@@ -208,10 +210,10 @@ nsresult nsMsgPurgeService::PerformPurge() {
         nsresult rv = server->GetType(type);
         NS_ENSURE_SUCCESS(rv, rv);
 
-        nsCString realHostName;
-        server->GetRealHostName(realHostName);
+        nsCString hostName;
+        server->GetHostName(hostName);
         MOZ_LOG(MsgPurgeLogModule, mozilla::LogLevel::Info,
-                ("[%d] %s (%s)", serverIndex, realHostName.get(), type.get()));
+                ("[%d] %s (%s)", serverIndex, hostName.get(), type.get()));
 
         nsCOMPtr<nsISpamSettings> spamSettings;
         rv = server->GetSpamSettings(getter_AddRefs(spamSettings));
@@ -238,7 +240,7 @@ nsresult nsMsgPurgeService::PerformPurge() {
         // check if the spam folder uri is set for this server
         // if not skip it.
         nsCString junkFolderURI;
-        rv = spamSettings->GetSpamFolderURI(getter_Copies(junkFolderURI));
+        rv = spamSettings->GetSpamFolderURI(junkFolderURI);
         NS_ENSURE_SUCCESS(rv, rv);
 
         MOZ_LOG(MsgPurgeLogModule, mozilla::LogLevel::Info,
@@ -410,15 +412,7 @@ nsresult nsMsgPurgeService::SearchFolderToPurge(nsIMsgFolder* folder,
 
   // we are about to search
   // create mHdrsToDelete array (if not previously created)
-  if (!mHdrsToDelete) {
-    mHdrsToDelete = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-  } else {
-    uint32_t count;
-    mHdrsToDelete->GetLength(&count);
-    NS_ASSERTION(count == 0, "mHdrsToDelete is not empty");
-    if (count > 0) mHdrsToDelete->Clear();  // this shouldn't happen
-  }
+  NS_ASSERTION(mHdrsToDelete.IsEmpty(), "mHdrsToDelete is not empty");
 
   mSearchFolder = folder;
   return mSearchSession->Search(nullptr);
@@ -472,15 +466,14 @@ NS_IMETHODIMP nsMsgPurgeService::OnSearchHit(nsIMsgDBHdr* aMsgHdr,
   if (atoi(junkScoreStr.get()) == nsIJunkMailPlugin::IS_SPAM_SCORE) {
     MOZ_LOG(MsgPurgeLogModule, mozilla::LogLevel::Info,
             ("added message to delete"));
-    return mHdrsToDelete->AppendElement(aMsgHdr);
+    mHdrsToDelete.AppendElement(aMsgHdr);
   }
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgPurgeService::OnSearchDone(nsresult status) {
   if (NS_SUCCEEDED(status)) {
-    uint32_t count = 0;
-    if (mHdrsToDelete) mHdrsToDelete->GetLength(&count);
+    uint32_t count = mHdrsToDelete.Length();
     MOZ_LOG(MsgPurgeLogModule, mozilla::LogLevel::Info,
             ("%d messages to delete", count));
 
@@ -492,7 +485,7 @@ NS_IMETHODIMP nsMsgPurgeService::OnSearchDone(nsresult status) {
             nullptr, false /*allowUndo*/);
     }
   }
-  if (mHdrsToDelete) mHdrsToDelete->Clear();
+  mHdrsToDelete.Clear();
   if (mSearchSession) mSearchSession->UnregisterListener(this);
   // don't cache the session
   // just create another search session next time we search, rather than
