@@ -12,9 +12,7 @@
 #include "nsImapCore.h"
 #include "nsIMsgHdr.h"
 #include "nsIDBFolderInfo.h"
-#include "nsArrayEnumerator.h"
 #include "nsMsgMessageFlags.h"
-#include "nsIMutableArray.h"
 #include "nsMsgUtils.h"
 
 nsMsgQuickSearchDBView::nsMsgQuickSearchDBView() {
@@ -37,9 +35,12 @@ NS_IMETHODIMP nsMsgQuickSearchDBView::Open(nsIMsgFolder* folder,
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!m_db) return NS_ERROR_NULL_POINTER;
-  if (pCount) *pCount = 0;
   m_viewFolder = nullptr;
-  return InitThreadedView(pCount);
+
+  int32_t count;
+  rv = InitThreadedView(count);
+  if (pCount) *pCount = count;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -70,18 +71,17 @@ nsMsgQuickSearchDBView::CopyDBView(nsMsgDBView* aNewMsgDBView,
   return NS_OK;
 }
 
-nsresult nsMsgQuickSearchDBView::DeleteMessages(nsIMsgWindow* window,
-                                                nsMsgViewIndex* indices,
-                                                int32_t numIndices,
-                                                bool deleteStorage) {
-  for (nsMsgViewIndex i = 0; i < (nsMsgViewIndex)numIndices; i++) {
+nsresult nsMsgQuickSearchDBView::DeleteMessages(
+    nsIMsgWindow* window, nsTArray<nsMsgViewIndex> const& selection,
+    bool deleteStorage) {
+  for (nsMsgViewIndex viewIndex : selection) {
     nsCOMPtr<nsIMsgDBHdr> msgHdr;
-    (void)GetMsgHdrForViewIndex(indices[i], getter_AddRefs(msgHdr));
-    if (msgHdr) RememberDeletedMsgHdr(msgHdr);
+    (void)GetMsgHdrForViewIndex(viewIndex, getter_AddRefs(msgHdr));
+    if (msgHdr) {
+      RememberDeletedMsgHdr(msgHdr);
+    }
   }
-
-  return nsMsgDBView::DeleteMessages(window, indices, numIndices,
-                                     deleteStorage);
+  return nsMsgDBView::DeleteMessages(window, selection, deleteStorage);
 }
 
 NS_IMETHODIMP nsMsgQuickSearchDBView::DoCommand(
@@ -214,6 +214,7 @@ NS_IMETHODIMP nsMsgQuickSearchDBView::OnHdrFlagsChanged(
 
 NS_IMETHODIMP
 nsMsgQuickSearchDBView::OnHdrPropertyChanged(nsIMsgDBHdr* aHdrChanged,
+                                             const nsACString& property,
                                              bool aPreChange, uint32_t* aStatus,
                                              nsIDBChangeListener* aInstigator) {
   // If the junk mail plugin just activated on a message, then
@@ -307,7 +308,7 @@ nsMsgQuickSearchDBView::OnSearchDone(nsresult status) {
     }
     if (m_db) {
       nsTArray<nsMsgKey> staleHits;
-      nsresult rv = m_db->RefreshCache(searchUri.get(), keyArray, staleHits);
+      nsresult rv = m_db->RefreshCache(searchUri, keyArray, staleHits);
       NS_ENSURE_SUCCESS(rv, rv);
       for (nsMsgKey staleKey : staleHits) {
         nsCOMPtr<nsIMsgDBHdr> hdrDeleted;
@@ -361,10 +362,10 @@ nsMsgQuickSearchDBView::OnNewSearch() {
   // check if it's a virtual folder - if so, we should get the cached hits
   // from the db, and set a flag saying that we're using cached values.
   if (folderFlags & nsMsgFolderFlags::Virtual) {
-    nsCOMPtr<nsISimpleEnumerator> cachedHits;
+    nsCOMPtr<nsIMsgEnumerator> cachedHits;
     nsCString searchUri;
     m_viewFolder->GetURI(searchUri);
-    m_db->GetCachedHits(searchUri.get(), getter_AddRefs(cachedHits));
+    m_db->GetCachedHits(searchUri, getter_AddRefs(cachedHits));
     if (cachedHits) {
       bool hasMore;
 
@@ -373,12 +374,10 @@ nsMsgQuickSearchDBView::OnNewSearch() {
       m_cacheEmpty = !hasMore;
       if (mTree) mTree->BeginUpdateBatch();
       while (hasMore) {
-        nsCOMPtr<nsISupports> supports;
-        nsresult rv = cachedHits->GetNext(getter_AddRefs(supports));
-        NS_ASSERTION(NS_SUCCEEDED(rv), "nsMsgDBEnumerator broken");
-        nsCOMPtr<nsIMsgDBHdr> pHeader = do_QueryInterface(supports);
-        if (pHeader && NS_SUCCEEDED(rv))
-          AddHdr(pHeader);
+        nsCOMPtr<nsIMsgDBHdr> header;
+        nsresult rv = cachedHits->GetNext(getter_AddRefs(header));
+        if (header && NS_SUCCEEDED(rv))
+          AddHdr(header);
         else
           break;
         cachedHits->HasMoreElements(&hasMore);
@@ -428,7 +427,7 @@ nsresult nsMsgQuickSearchDBView::GetFirstMessageHdrToDisplayInThread(
         nsCOMPtr<nsIMsgDBHdr> parent;
         // count number of ancestors - that's our level
         while (parentId != nsMsgKey_None) {
-          rv = m_db->GetMsgHdrForKey(parentId, getter_AddRefs(parent));
+          m_db->GetMsgHdrForKey(parentId, getter_AddRefs(parent));
           if (parent) {
             nsMsgKey saveParentId = parentId;
             parent->GetThreadParent(&parentId);
@@ -535,7 +534,7 @@ nsresult nsMsgQuickSearchDBView::SortThreads(
 }
 
 nsresult nsMsgQuickSearchDBView::ListCollapsedChildren(
-    nsMsgViewIndex viewIndex, nsIMutableArray* messageArray) {
+    nsMsgViewIndex viewIndex, nsTArray<RefPtr<nsIMsgDBHdr>>& messageArray) {
   nsCOMPtr<nsIMsgThread> threadHdr;
   nsresult rv = GetThreadContainingIndex(viewIndex, getter_AddRefs(threadHdr));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -556,7 +555,7 @@ nsresult nsMsgQuickSearchDBView::ListCollapsedChildren(
       if (msgKey != rootKey || (GroupViewUsesDummyRow() && rootKeySkipped)) {
         // if this hdr is in the original view, add it to new view.
         if (m_origKeys.BinaryIndexOf(msgKey) != m_origKeys.NoIndex)
-          messageArray->AppendElement(msgHdr);
+          messageArray.AppendElement(msgHdr);
       } else {
         rootKeySkipped = true;
       }
@@ -621,7 +620,7 @@ nsresult nsMsgQuickSearchDBView::ListIdsInThreadOrder(
     nsIMsgThread* threadHdr, nsMsgKey parentKey, uint32_t level,
     uint32_t callLevel, nsMsgKey keyToSkip, nsMsgViewIndex* viewIndex,
     uint32_t* pNumListed) {
-  nsCOMPtr<nsISimpleEnumerator> msgEnumerator;
+  nsCOMPtr<nsIMsgEnumerator> msgEnumerator;
   nsresult rv =
       threadHdr->EnumerateMessages(parentKey, getter_AddRefs(msgEnumerator));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -630,41 +629,39 @@ nsresult nsMsgQuickSearchDBView::ListIdsInThreadOrder(
   uint32_t numChildren;
   (void)threadHdr->GetNumChildren(&numChildren);
   bool hasMore;
-  nsCOMPtr<nsISupports> supports;
-  nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  while (NS_SUCCEEDED(rv) &&
-         NS_SUCCEEDED(rv = msgEnumerator->HasMoreElements(&hasMore)) &&
+  while (NS_SUCCEEDED(rv = msgEnumerator->HasMoreElements(&hasMore)) &&
          hasMore) {
-    rv = msgEnumerator->GetNext(getter_AddRefs(supports));
-    if (NS_SUCCEEDED(rv) && supports) {
-      msgHdr = do_QueryInterface(supports);
-      nsMsgKey msgKey;
-      msgHdr->GetMessageKey(&msgKey);
-      if (msgKey == keyToSkip) continue;
+    nsCOMPtr<nsIMsgDBHdr> msgHdr;
+    rv = msgEnumerator->GetNext(getter_AddRefs(msgHdr));
+    NS_ENSURE_SUCCESS(rv, rv);
 
-      // If we discover depths of more than numChildren, it means we have
-      // some sort of circular thread relationship and we bail out of the
-      // while loop before overflowing the stack with recursive calls.
-      // Technically, this is an error, but forcing a database rebuild
-      // is too destructive so we just return.
-      if (*pNumListed > numChildren || callLevel > numChildren) {
-        NS_ERROR("loop in message threading while listing children");
-        return NS_OK;
-      }
+    nsMsgKey msgKey;
+    msgHdr->GetMessageKey(&msgKey);
+    if (msgKey == keyToSkip) continue;
 
-      int32_t childLevel = level;
-      if (m_origKeys.BinaryIndexOf(msgKey) != m_origKeys.NoIndex) {
-        uint32_t msgFlags;
-        msgHdr->GetFlags(&msgFlags);
-        InsertMsgHdrAt(*viewIndex, msgHdr, msgKey, msgFlags & ~MSG_VIEW_FLAGS,
-                       level);
-        (*pNumListed)++;
-        (*viewIndex)++;
-        childLevel++;
-      }
-      rv = ListIdsInThreadOrder(threadHdr, msgKey, childLevel, callLevel + 1,
-                                keyToSkip, viewIndex, pNumListed);
+    // If we discover depths of more than numChildren, it means we have
+    // some sort of circular thread relationship and we bail out of the
+    // while loop before overflowing the stack with recursive calls.
+    // Technically, this is an error, but forcing a database rebuild
+    // is too destructive so we just return.
+    if (*pNumListed > numChildren || callLevel > numChildren) {
+      NS_ERROR("loop in message threading while listing children");
+      return NS_OK;
     }
+
+    int32_t childLevel = level;
+    if (m_origKeys.BinaryIndexOf(msgKey) != m_origKeys.NoIndex) {
+      uint32_t msgFlags;
+      msgHdr->GetFlags(&msgFlags);
+      InsertMsgHdrAt(*viewIndex, msgHdr, msgKey, msgFlags & ~MSG_VIEW_FLAGS,
+                     level);
+      (*pNumListed)++;
+      (*viewIndex)++;
+      childLevel++;
+    }
+    rv = ListIdsInThreadOrder(threadHdr, msgKey, childLevel, callLevel + 1,
+                              keyToSkip, viewIndex, pNumListed);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
   return rv;
 }
@@ -740,7 +737,7 @@ nsresult nsMsgQuickSearchDBView::ExpansionDelta(nsMsgViewIndex index,
 }
 
 NS_IMETHODIMP
-nsMsgQuickSearchDBView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
+nsMsgQuickSearchDBView::OpenWithHdrs(nsIMsgEnumerator* aHeaders,
                                      nsMsgViewSortTypeValue aSortType,
                                      nsMsgViewSortOrderValue aSortOrder,
                                      nsMsgViewFlagsTypeValue aViewFlags,
@@ -755,15 +752,15 @@ nsMsgQuickSearchDBView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
 
   bool hasMore;
   nsCOMPtr<nsISupports> supports;
-  nsCOMPtr<nsIMsgDBHdr> msgHdr;
   nsresult rv = NS_OK;
   while (NS_SUCCEEDED(rv = aHeaders->HasMoreElements(&hasMore)) && hasMore) {
-    rv = aHeaders->GetNext(getter_AddRefs(supports));
-    if (NS_SUCCEEDED(rv) && supports) {
-      msgHdr = do_QueryInterface(supports);
+    nsCOMPtr<nsIMsgDBHdr> msgHdr;
+    rv = aHeaders->GetNext(getter_AddRefs(msgHdr));
+    if (NS_SUCCEEDED(rv) && msgHdr) {
       AddHdr(msgHdr);
-    } else
+    } else {
       break;
+    }
   }
   *aCount = m_keys.Length();
   return rv;
@@ -782,7 +779,7 @@ NS_IMETHODIMP nsMsgQuickSearchDBView::SetViewFlags(
 }
 
 nsresult nsMsgQuickSearchDBView::GetMessageEnumerator(
-    nsISimpleEnumerator** enumerator) {
+    nsIMsgEnumerator** enumerator) {
   return GetViewEnumerator(enumerator);
 }
 
