@@ -11,12 +11,9 @@
 #include "nsIDBFolderInfo.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgCopyService.h"
-#include "nsICopyMsgStreamListener.h"
 #include "nsMsgUtils.h"
 #include "nsTreeColumns.h"
 #include "nsIMsgMessageService.h"
-#include "nsArrayUtils.h"
-#include "nsIMutableArray.h"
 #include "nsMsgGroupThread.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
@@ -32,6 +29,9 @@ nsMsgSearchDBView::nsMsgSearchDBView() {
   mSuppressMsgDisplay = true;
   m_totalMessagesInView = 0;
   m_nextThreadId = 1;
+  mCurIndex = 0;
+  mTotalIndices = 0;
+  mCommand = -1;
 }
 
 nsMsgSearchDBView::~nsMsgSearchDBView() {}
@@ -100,7 +100,9 @@ nsMsgSearchDBView::CopyDBView(nsMsgDBView* aNewMsgDBView,
   newMsgDBView->mCurIndex = mCurIndex;
   newMsgDBView->m_folders.InsertObjectsAt(m_folders, 0);
   newMsgDBView->m_curCustomColumn = m_curCustomColumn;
-  newMsgDBView->m_hdrsForEachFolder.InsertObjectsAt(m_hdrsForEachFolder, 0);
+  for (auto const& hdrs : m_hdrsForEachFolder) {
+    newMsgDBView->m_hdrsForEachFolder.AppendElement(hdrs.Clone());
+  }
   newMsgDBView->m_uniqueFoldersSelected.InsertObjectsAt(m_uniqueFoldersSelected,
                                                         0);
 
@@ -113,10 +115,10 @@ nsMsgSearchDBView::CopyDBView(nsMsgDBView* aNewMsgDBView,
   if (m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay) {
     // We need to clone the thread and msg hdr hash tables.
     for (auto iter = m_threadsTable.Iter(); !iter.Done(); iter.Next()) {
-      newMsgDBView->m_threadsTable.Put(iter.Key(), iter.UserData());
+      newMsgDBView->m_threadsTable.InsertOrUpdate(iter.Key(), iter.UserData());
     }
     for (auto iter = m_hdrsTable.Iter(); !iter.Done(); iter.Next()) {
-      newMsgDBView->m_hdrsTable.Put(iter.Key(), iter.UserData());
+      newMsgDBView->m_hdrsTable.InsertOrUpdate(iter.Key(), iter.UserData());
     }
   }
 
@@ -327,19 +329,21 @@ void nsMsgSearchDBView::RemoveRows(nsMsgViewIndex viewIndex, int32_t numRows) {
 
 nsresult nsMsgSearchDBView::GetMsgHdrForViewIndex(nsMsgViewIndex index,
                                                   nsIMsgDBHdr** msgHdr) {
-  nsresult rv = NS_MSG_INVALID_DBVIEW_INDEX;
-  if (index == nsMsgViewIndex_None || index >= (uint32_t)m_folders.Count())
-    return rv;
+  if (index == nsMsgViewIndex_None || index >= (uint32_t)m_folders.Count()) {
+    return NS_MSG_INVALID_DBVIEW_INDEX;
+  }
 
   nsIMsgFolder* folder = m_folders[index];
   if (folder) {
     nsCOMPtr<nsIMsgDatabase> db;
-    rv = folder->GetMsgDatabase(getter_AddRefs(db));
+    nsresult rv = folder->GetMsgDatabase(getter_AddRefs(db));
     NS_ENSURE_SUCCESS(rv, rv);
-    if (db) rv = db->GetMsgHdrForKey(m_keys[index], msgHdr);
+    if (db) {
+      return db->GetMsgHdrForKey(m_keys[index], msgHdr);
+    }
   }
 
-  return rv;
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -589,7 +593,7 @@ void nsMsgSearchDBView::MoveThreadAt(nsMsgViewIndex threadIndex) {
 }
 
 nsresult nsMsgSearchDBView::GetMessageEnumerator(
-    nsISimpleEnumerator** enumerator) {
+    nsIMsgEnumerator** enumerator) {
   // We do not have an m_db, so the default behavior (in nsMsgDBView) is not
   // what we want (it will crash).  We just want someone to enumerate the
   // headers that we already have.  Conveniently, nsMsgDBView already knows
@@ -741,21 +745,16 @@ NS_IMETHODIMP nsMsgSearchDBView::DoCommand(nsMsgViewCommandTypeValue command) {
   nsMsgViewIndexArray selection;
   GetIndicesForSelection(selection);
 
-  nsMsgViewIndex* indices = selection.Elements();
-  int32_t numIndices = selection.Length();
-
   // We need to break apart the selection by folders, and then call
   // ApplyCommandToIndices with the command and the indices in the
   // selection that are from that folder.
 
-  mozilla::UniquePtr<nsTArray<uint32_t>[]> indexArrays;
+  mozilla::UniquePtr<nsTArray<nsMsgViewIndex>[]> indexArrays;
   int32_t numArrays;
-  rv = PartitionSelectionByFolder(indices, numIndices, indexArrays, &numArrays);
+  rv = PartitionSelectionByFolder(selection, indexArrays, &numArrays);
   NS_ENSURE_SUCCESS(rv, rv);
   for (int32_t folderIndex = 0; folderIndex < numArrays; folderIndex++) {
-    rv = ApplyCommandToIndices(command,
-                               (indexArrays.get())[folderIndex].Elements(),
-                               indexArrays[folderIndex].Length());
+    rv = ApplyCommandToIndices(command, (indexArrays.get())[folderIndex]);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -810,11 +809,10 @@ nsresult nsMsgSearchDBView::RemoveByIndex(nsMsgViewIndex index) {
   return nsMsgDBView::RemoveByIndex(index);
 }
 
-nsresult nsMsgSearchDBView::DeleteMessages(nsIMsgWindow* window,
-                                           nsMsgViewIndex* indices,
-                                           int32_t numIndices,
-                                           bool deleteStorage) {
-  nsresult rv = GetFoldersAndHdrsForSelection(indices, numIndices);
+nsresult nsMsgSearchDBView::DeleteMessages(
+    nsIMsgWindow* window, nsTArray<nsMsgViewIndex> const& selection,
+    bool deleteStorage) {
+  nsresult rv = GetFoldersAndHdrsForSelection(selection);
   NS_ENSURE_SUCCESS(rv, rv);
   if (mDeleteModel != nsMsgImapDeleteModels::MoveToTrash) deleteStorage = true;
 
@@ -823,13 +821,17 @@ nsresult nsMsgSearchDBView::DeleteMessages(nsIMsgWindow* window,
   // Remember the deleted messages in case the user undoes the delete,
   // and we want to restore the hdr to the view, even if it no
   // longer matches the search criteria.
-  for (nsMsgViewIndex i = 0; i < (nsMsgViewIndex)numIndices; i++) {
+  for (nsMsgViewIndex viewIndex : selection) {
     nsCOMPtr<nsIMsgDBHdr> msgHdr;
-    (void)GetMsgHdrForViewIndex(indices[i], getter_AddRefs(msgHdr));
-    if (msgHdr) RememberDeletedMsgHdr(msgHdr);
+    (void)GetMsgHdrForViewIndex(viewIndex, getter_AddRefs(msgHdr));
+    if (msgHdr) {
+      RememberDeletedMsgHdr(msgHdr);
+    }
 
     // If we are deleting rows, save off the view indices.
-    if (m_deletingRows) mIndicesToNoteChange.AppendElement(indices[i]);
+    if (m_deletingRows) {
+      mIndicesToNoteChange.AppendElement(viewIndex);
+    }
   }
   rv = deleteStorage ? ProcessRequestsInAllFolders(window)
                      : ProcessRequestsInOneFolder(window);
@@ -838,27 +840,25 @@ nsresult nsMsgSearchDBView::DeleteMessages(nsIMsgWindow* window,
   return rv;
 }
 
-nsresult nsMsgSearchDBView::CopyMessages(nsIMsgWindow* window,
-                                         nsMsgViewIndex* indices,
-                                         int32_t numIndices, bool isMove,
-                                         nsIMsgFolder* destFolder) {
-  GetFoldersAndHdrsForSelection(indices, numIndices);
+nsresult nsMsgSearchDBView::CopyMessages(
+    nsIMsgWindow* window, nsTArray<nsMsgViewIndex> const& selection,
+    bool isMove, nsIMsgFolder* destFolder) {
+  GetFoldersAndHdrsForSelection(selection);
   return ProcessRequestsInOneFolder(window);
 }
 
 nsresult nsMsgSearchDBView::PartitionSelectionByFolder(
-    nsMsgViewIndex* indices, int32_t numIndices,
-    mozilla::UniquePtr<nsTArray<uint32_t>[]>& indexArrays, int32_t* numArrays) {
-  nsMsgViewIndex i;
-  int32_t folderIndex;
+    nsTArray<nsMsgViewIndex> const& selection,
+    mozilla::UniquePtr<nsTArray<nsMsgViewIndex>[]>& indexArrays,
+    int32_t* numArrays) {
   nsCOMArray<nsIMsgFolder> uniqueFoldersSelected;
   nsTArray<uint32_t> numIndicesSelected;
   mCurIndex = 0;
 
   // Build unique folder list based on headers selected by the user.
-  for (i = 0; i < (nsMsgViewIndex)numIndices; i++) {
-    nsIMsgFolder* curFolder = m_folders[indices[i]];
-    folderIndex = uniqueFoldersSelected.IndexOf(curFolder);
+  for (nsMsgViewIndex viewIndex : selection) {
+    nsIMsgFolder* curFolder = m_folders[viewIndex];
+    int32_t folderIndex = uniqueFoldersSelected.IndexOf(curFolder);
     if (folderIndex < 0) {
       uniqueFoldersSelected.AppendObject(curFolder);
       numIndicesSelected.AppendElement(1);
@@ -868,45 +868,38 @@ nsresult nsMsgSearchDBView::PartitionSelectionByFolder(
   }
 
   int32_t numFolders = uniqueFoldersSelected.Count();
-  indexArrays = mozilla::MakeUnique<nsTArray<uint32_t>[]>(numFolders);
+  indexArrays = mozilla::MakeUnique<nsTArray<nsMsgViewIndex>[]>(numFolders);
   *numArrays = numFolders;
   NS_ENSURE_TRUE(indexArrays, NS_ERROR_OUT_OF_MEMORY);
-  for (folderIndex = 0; folderIndex < numFolders; folderIndex++) {
+  for (int32_t folderIndex = 0; folderIndex < numFolders; folderIndex++) {
     (indexArrays.get())[folderIndex].SetCapacity(
         numIndicesSelected[folderIndex]);
   }
-  for (i = 0; i < (nsMsgViewIndex)numIndices; i++) {
-    nsIMsgFolder* curFolder = m_folders[indices[i]];
+  for (nsMsgViewIndex viewIndex : selection) {
+    nsIMsgFolder* curFolder = m_folders[viewIndex];
     int32_t folderIndex = uniqueFoldersSelected.IndexOf(curFolder);
-    (indexArrays.get())[folderIndex].AppendElement(indices[i]);
+    (indexArrays.get())[folderIndex].AppendElement(viewIndex);
   }
   return NS_OK;
 }
 
 nsresult nsMsgSearchDBView::GetFoldersAndHdrsForSelection(
-    nsMsgViewIndex* indices, int32_t numIndices) {
+    nsTArray<nsMsgViewIndex> const& selection) {
   nsresult rv = NS_OK;
   mCurIndex = 0;
   m_uniqueFoldersSelected.Clear();
   m_hdrsForEachFolder.Clear();
 
-  nsCOMPtr<nsIMutableArray> messages(
-      do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
+  AutoTArray<RefPtr<nsIMsgDBHdr>, 1> messages;
+  rv = GetHeadersFromSelection(selection, messages);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = GetHeadersFromSelection(indices, numIndices, messages);
-  NS_ENSURE_SUCCESS(rv, rv);
-  uint32_t numMsgs;
-  messages->GetLength(&numMsgs);
 
-  uint32_t i;
   // Build unique folder list based on headers selected by the user.
-  for (i = 0; i < numMsgs; i++) {
-    nsCOMPtr<nsIMsgDBHdr> hdr = do_QueryElementAt(messages, i, &rv);
-    if (hdr) {
-      nsCOMPtr<nsIMsgFolder> curFolder;
-      hdr->GetFolder(getter_AddRefs(curFolder));
-      if (m_uniqueFoldersSelected.IndexOf(curFolder) < 0)
-        m_uniqueFoldersSelected.AppendObject(curFolder);
+  for (nsIMsgDBHdr* hdr : messages) {
+    nsCOMPtr<nsIMsgFolder> curFolder;
+    hdr->GetFolder(getter_AddRefs(curFolder));
+    if (m_uniqueFoldersSelected.IndexOf(curFolder) < 0) {
+      m_uniqueFoldersSelected.AppendObject(curFolder);
     }
   }
 
@@ -914,34 +907,28 @@ nsresult nsMsgSearchDBView::GetFoldersAndHdrsForSelection(
   uint32_t numFolders = m_uniqueFoldersSelected.Count();
   for (uint32_t folderIndex = 0; folderIndex < numFolders; folderIndex++) {
     nsIMsgFolder* curFolder = m_uniqueFoldersSelected[folderIndex];
-    nsCOMPtr<nsIMutableArray> msgHdrsForOneFolder(
-        do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
-    for (i = 0; i < numMsgs; i++) {
-      nsCOMPtr<nsIMsgDBHdr> hdr = do_QueryElementAt(messages, i, &rv);
-      if (hdr) {
-        nsCOMPtr<nsIMsgFolder> msgFolder;
-        hdr->GetFolder(getter_AddRefs(msgFolder));
-        if (NS_SUCCEEDED(rv) && msgFolder && msgFolder == curFolder) {
-          nsCOMPtr<nsISupports> hdrSupports = do_QueryInterface(hdr);
-          msgHdrsForOneFolder->AppendElement(hdrSupports);
-        }
+    nsTArray<RefPtr<nsIMsgDBHdr>> msgHdrsForOneFolder;
+    for (nsIMsgDBHdr* hdr : messages) {
+      nsCOMPtr<nsIMsgFolder> msgFolder;
+      hdr->GetFolder(getter_AddRefs(msgFolder));
+      if (NS_SUCCEEDED(rv) && msgFolder && msgFolder == curFolder) {
+        msgHdrsForOneFolder.AppendElement(hdr);
       }
     }
 
-    m_hdrsForEachFolder.AppendElement(msgHdrsForOneFolder);
+    m_hdrsForEachFolder.AppendElement(msgHdrsForOneFolder.Clone());
   }
 
   return rv;
 }
 
 nsresult nsMsgSearchDBView::ApplyCommandToIndicesWithFolder(
-    nsMsgViewCommandTypeValue command, nsMsgViewIndex* indices,
-    int32_t numIndices, nsIMsgFolder* destFolder) {
+    nsMsgViewCommandTypeValue command,
+    nsTArray<nsMsgViewIndex> const& selection, nsIMsgFolder* destFolder) {
   mCommand = command;
   mDestFolder = destFolder;
-  return nsMsgDBView::ApplyCommandToIndicesWithFolder(command, indices,
-                                                      numIndices, destFolder);
+  return nsMsgDBView::ApplyCommandToIndicesWithFolder(command, selection,
+                                                      destFolder);
 }
 
 // nsIMsgCopyServiceListener methods
@@ -984,12 +971,11 @@ nsresult nsMsgSearchDBView::ProcessRequestsInOneFolder(nsIMsgWindow* window) {
 
   nsIMsgFolder* curFolder = m_uniqueFoldersSelected[mCurIndex];
   NS_ASSERTION(curFolder, "curFolder is null");
-  nsCOMPtr<nsIMutableArray> messageArray = m_hdrsForEachFolder[mCurIndex];
-  NS_ASSERTION(messageArray, "messageArray is null");
+  nsTArray<RefPtr<nsIMsgDBHdr>> const& msgs = m_hdrsForEachFolder[mCurIndex];
 
   // called for delete with trash, copy and move
   if (mCommand == nsMsgViewCommandType::deleteMsg)
-    curFolder->DeleteMessages(messageArray, window, false /* delete storage */,
+    curFolder->DeleteMessages(msgs, window, false /* delete storage */,
                               false /* is move*/, this, true /*allowUndo*/);
   else {
     NS_ASSERTION(!(curFolder == mDestFolder),
@@ -999,11 +985,11 @@ nsresult nsMsgSearchDBView::ProcessRequestsInOneFolder(nsIMsgWindow* window) {
           do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
       if (NS_SUCCEEDED(rv)) {
         if (mCommand == nsMsgViewCommandType::moveMessages)
-          copyService->CopyMessages(curFolder, messageArray, mDestFolder,
+          copyService->CopyMessages(curFolder, msgs, mDestFolder,
                                     true /* isMove */, this, window,
                                     true /*allowUndo*/);
         else if (mCommand == nsMsgViewCommandType::copyMessages)
-          copyService->CopyMessages(curFolder, messageArray, mDestFolder,
+          copyService->CopyMessages(curFolder, msgs, mDestFolder,
                                     false /* isMove */, this, window,
                                     true /*allowUndo*/);
       }
@@ -1018,13 +1004,9 @@ nsresult nsMsgSearchDBView::ProcessRequestsInAllFolders(nsIMsgWindow* window) {
   for (uint32_t folderIndex = 0; folderIndex < numFolders; folderIndex++) {
     nsIMsgFolder* curFolder = m_uniqueFoldersSelected[folderIndex];
     NS_ASSERTION(curFolder, "curFolder is null");
-
-    nsCOMPtr<nsIMutableArray> messageArray = m_hdrsForEachFolder[folderIndex];
-    NS_ASSERTION(messageArray, "messageArray is null");
-
-    curFolder->DeleteMessages(messageArray, window, true /* delete storage */,
-                              false /* is move*/, nullptr /*copyServListener*/,
-                              false /*allowUndo*/);
+    curFolder->DeleteMessages(
+        m_hdrsForEachFolder[folderIndex], window, true /* delete storage */,
+        false /* is move*/, nullptr /*copyServListener*/, false /*allowUndo*/);
   }
 
   return NS_OK;
@@ -1084,7 +1066,7 @@ nsMsgSearchDBView::GetHdrForFirstSelectedMessage(nsIMsgDBHdr** hdr) {
 }
 
 NS_IMETHODIMP
-nsMsgSearchDBView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
+nsMsgSearchDBView::OpenWithHdrs(nsIMsgEnumerator* aHeaders,
                                 nsMsgViewSortTypeValue aSortType,
                                 nsMsgViewSortOrderValue aSortOrder,
                                 nsMsgViewFlagsTypeValue aViewFlags,
@@ -1099,15 +1081,13 @@ nsMsgSearchDBView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
   SaveSortInfo(m_sortType, m_sortOrder);
 
   bool hasMore;
-  nsCOMPtr<nsISupports> supports;
-  nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  nsCOMPtr<nsIMsgFolder> folder;
   nsresult rv = NS_OK;
   while (NS_SUCCEEDED(rv) &&
          NS_SUCCEEDED(rv = aHeaders->HasMoreElements(&hasMore)) && hasMore) {
-    rv = aHeaders->GetNext(getter_AddRefs(supports));
-    if (NS_SUCCEEDED(rv) && supports) {
-      msgHdr = do_QueryInterface(supports);
+    nsCOMPtr<nsIMsgDBHdr> msgHdr;
+    nsCOMPtr<nsIMsgFolder> folder;
+    rv = aHeaders->GetNext(getter_AddRefs(msgHdr));
+    if (NS_SUCCEEDED(rv) && msgHdr) {
       msgHdr->GetFolder(getter_AddRefs(folder));
       AddHdrFromFolder(msgHdr, folder);
     }
@@ -1117,11 +1097,11 @@ nsMsgSearchDBView::OpenWithHdrs(nsISimpleEnumerator* aHeaders,
   return rv;
 }
 
-nsresult nsMsgSearchDBView::GetFolderFromMsgURI(const char* aMsgURI,
+nsresult nsMsgSearchDBView::GetFolderFromMsgURI(const nsACString& aMsgURI,
                                                 nsIMsgFolder** aFolder) {
   nsCOMPtr<nsIMsgMessageService> msgMessageService;
-  nsresult rv = GetMessageServiceFromURI(nsDependentCString(aMsgURI),
-                                         getter_AddRefs(msgMessageService));
+  nsresult rv =
+      GetMessageServiceFromURI(aMsgURI, getter_AddRefs(msgMessageService));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
@@ -1154,6 +1134,7 @@ nsMsgViewIndex nsMsgSearchDBView::FindHdr(nsIMsgDBHdr* msgHdr,
 nsresult nsMsgSearchDBView::GetXFThreadFromMsgHdr(nsIMsgDBHdr* msgHdr,
                                                   nsIMsgThread** pThread,
                                                   bool* foundByMessageId) {
+  NS_ENSURE_ARG_POINTER(msgHdr);
   NS_ENSURE_ARG_POINTER(pThread);
 
   nsAutoCString messageId;
@@ -1206,7 +1187,7 @@ nsresult nsMsgSearchDBView::AddRefToHash(nsCString& reference,
   m_threadsTable.Get(reference, getter_AddRefs(oldThread));
   if (oldThread) return NS_OK;
 
-  m_threadsTable.Put(reference, thread);
+  m_threadsTable.InsertOrUpdate(reference, thread);
   return NS_OK;
 }
 
@@ -1229,7 +1210,7 @@ nsresult nsMsgSearchDBView::AddMsgToHashTables(nsIMsgDBHdr* msgHdr,
 
   nsCString messageId;
   msgHdr->GetMessageId(getter_Copies(messageId));
-  m_hdrsTable.Put(messageId, msgHdr);
+  m_hdrsTable.InsertOrUpdate(messageId, msgHdr);
   if (!gReferenceOnlyThreading) {
     nsCString subject;
     msgHdr->GetSubject(getter_Copies(subject));
