@@ -53,7 +53,7 @@ class nsMsgSearchDBView : public nsMsgGroupView,
   NS_IMETHOD DoCommandWithFolder(nsMsgViewCommandTypeValue command,
                                  nsIMsgFolder* destFolder) override;
   NS_IMETHOD GetHdrForFirstSelectedMessage(nsIMsgDBHdr** hdr) override;
-  NS_IMETHOD OpenWithHdrs(nsISimpleEnumerator* aHeaders,
+  NS_IMETHOD OpenWithHdrs(nsIMsgEnumerator* aHeaders,
                           nsMsgViewSortTypeValue aSortType,
                           nsMsgViewSortOrderValue aSortOrder,
                           nsMsgViewFlagsTypeValue aViewFlags,
@@ -78,7 +78,7 @@ class nsMsgSearchDBView : public nsMsgGroupView,
   NS_IMETHOD OnAnnouncerGoingAway(nsIDBChangeAnnouncer* instigator) override;
 
   virtual nsCOMArray<nsIMsgFolder>* GetFolders() override;
-  virtual nsresult GetFolderFromMsgURI(const char* aMsgURI,
+  virtual nsresult GetFolderFromMsgURI(const nsACString& aMsgURI,
                                        nsIMsgFolder** aFolder) override;
 
   NS_IMETHOD GetThreadContainingMsgHdr(nsIMsgDBHdr* msgHdr,
@@ -96,11 +96,11 @@ class nsMsgSearchDBView : public nsMsgGroupView,
   virtual nsresult GetDBForViewIndex(nsMsgViewIndex index,
                                      nsIMsgDatabase** db) override;
   virtual nsresult RemoveByIndex(nsMsgViewIndex index) override;
-  virtual nsresult CopyMessages(nsIMsgWindow* window, nsMsgViewIndex* indices,
-                                int32_t numIndices, bool isMove,
-                                nsIMsgFolder* destFolder) override;
-  virtual nsresult DeleteMessages(nsIMsgWindow* window, nsMsgViewIndex* indices,
-                                  int32_t numIndices,
+  virtual nsresult CopyMessages(nsIMsgWindow* window,
+                                nsTArray<nsMsgViewIndex> const& selection,
+                                bool isMove, nsIMsgFolder* destFolder) override;
+  virtual nsresult DeleteMessages(nsIMsgWindow* window,
+                                  nsTArray<nsMsgViewIndex> const& selection,
                                   bool deleteStorage) override;
   virtual void InsertMsgHdrAt(nsMsgViewIndex index, nsIMsgDBHdr* hdr,
                               nsMsgKey msgKey, uint32_t flags,
@@ -114,30 +114,33 @@ class nsMsgSearchDBView : public nsMsgGroupView,
   virtual nsMsgViewIndex FindHdr(nsIMsgDBHdr* msgHdr,
                                  nsMsgViewIndex startIndex = 0,
                                  bool allowDummy = false) override;
-  nsresult GetFoldersAndHdrsForSelection(nsMsgViewIndex* indices,
-                                         int32_t numIndices);
+  nsresult GetFoldersAndHdrsForSelection(
+      nsTArray<nsMsgViewIndex> const& selection);
   nsresult GroupSearchResultsByFolder();
   nsresult PartitionSelectionByFolder(
-      nsMsgViewIndex* indices, int32_t numIndices,
+      nsTArray<nsMsgViewIndex> const& selection,
       mozilla::UniquePtr<nsTArray<uint32_t>[]>& indexArrays,
       int32_t* numArrays);
 
   virtual nsresult ApplyCommandToIndicesWithFolder(
-      nsMsgViewCommandTypeValue command, nsMsgViewIndex* indices,
-      int32_t numIndices, nsIMsgFolder* destFolder) override;
+      nsMsgViewCommandTypeValue command,
+      nsTArray<nsMsgViewIndex> const& selection,
+      nsIMsgFolder* destFolder) override;
   void MoveThreadAt(nsMsgViewIndex threadIndex);
 
-  virtual nsresult GetMessageEnumerator(
-      nsISimpleEnumerator** enumerator) override;
+  virtual nsresult GetMessageEnumerator(nsIMsgEnumerator** enumerator) override;
   virtual nsresult InsertHdrFromFolder(nsIMsgDBHdr* msgHdr,
                                        nsIMsgFolder* folder);
 
+  // Holds the original folder of each message in this view.
+  // Augments the existing arrays in nsMsgDBView (m_keys, m_flags and m_levels),
+  // and is kept in sync with them.
   nsCOMArray<nsIMsgFolder> m_folders;
-  nsCOMArray<nsIMutableArray> m_hdrsForEachFolder;
+
+  nsTArray<nsTArray<RefPtr<nsIMsgDBHdr>>> m_hdrsForEachFolder;
   nsCOMArray<nsIMsgFolder> m_uniqueFoldersSelected;
   uint32_t mCurIndex;
 
-  nsMsgViewIndex* mIndicesForChainedDeleteAndFile;
   int32_t mTotalIndices;
   nsCOMArray<nsIMsgDatabase> m_dbToUseList;
   nsMsgViewCommandTypeValue mCommand;
