@@ -22,8 +22,6 @@ class nsMsgThreadedDBView : public nsMsgGroupView {
                          nsIMsgDBViewCommandUpdater* aCommandUpdater,
                          nsIMsgDBView** _retval) override;
   NS_IMETHOD Close() override;
-  int32_t AddKeys(nsMsgKey* pKeys, int32_t* pFlags, const char* pLevels,
-                  nsMsgViewSortTypeValue sortType, int32_t numKeysToAdd);
   NS_IMETHOD Sort(nsMsgViewSortTypeValue sortType,
                   nsMsgViewSortOrderValue sortOrder) override;
   NS_IMETHOD GetViewType(nsMsgViewTypeValue* aViewType) override;
@@ -33,15 +31,12 @@ class nsMsgThreadedDBView : public nsMsgGroupView {
 
  protected:
   virtual const char* GetViewName(void) override { return "ThreadedDBView"; }
-  nsresult InitThreadedView(int32_t* pCount);
+  nsresult InitThreadedView(int32_t& count);
   virtual nsresult OnNewHeader(nsIMsgDBHdr* newHdr, nsMsgKey aParentKey,
                                bool ensureListed) override;
   virtual nsresult AddMsgToThreadNotInView(nsIMsgThread* threadHdr,
                                            nsIMsgDBHdr* msgHdr,
                                            bool ensureListed);
-  nsresult ListThreadIds(nsMsgKey* startMsg, bool unreadOnly, nsMsgKey* pOutput,
-                         int32_t* pFlags, char* pLevels, int32_t numToList,
-                         int32_t* pNumListed, int32_t* pTotalHeaders);
   nsresult InitSort(nsMsgViewSortTypeValue sortType,
                     nsMsgViewSortOrderValue sortOrder);
   virtual nsresult SortThreads(nsMsgViewSortTypeValue sortType,
@@ -62,7 +57,6 @@ class nsMsgThreadedDBView : public nsMsgGroupView {
   nsTArray<nsMsgKey> m_prevKeys;  // this is used for caching non-threaded view.
   nsTArray<uint32_t> m_prevFlags;
   nsTArray<uint8_t> m_prevLevels;
-  nsCOMPtr<nsISimpleEnumerator> m_threadEnumerator;
 };
 
 #endif
