@@ -14,7 +14,6 @@
 #include "nsTArray.h"
 #include "nsInterfaceRequestorAgg.h"
 #include "nsILoadGroup.h"
-#include "nsIArray.h"
 #include "nsINetUtil.h"
 #include "nsIRequest.h"
 #include "nsILoadInfo.h"
@@ -31,16 +30,24 @@ class nsIUrlListener;
 class nsIOutputStream;
 class nsIInputStream;
 class nsIMsgDatabase;
-class nsIMutableArray;
 class nsIProxyInfo;
 class nsIMsgWindow;
 class nsIStreamListener;
 class nsICancelable;
 class nsIProtocolProxyCallback;
+class nsIMsgSearchTerm;
 
 #define FILE_IO_BUFFER_SIZE (16 * 1024)
 #define MSGS_URL "chrome://messenger/locale/messenger.properties"
 
+enum nsDateFormatSelectorComm : long {
+  kDateFormatNone = 0,
+  kDateFormatLong = 1,
+  kDateFormatShort = 2,
+  kDateFormatUnused = 3,
+  kDateFormatWeekday = 4
+};
+
 // These are utility functions that can used throughout the mailnews code
 
 NS_MSG_BASE nsresult GetMessageServiceContractIDForURI(const char* uri,
@@ -49,7 +56,8 @@ NS_MSG_BASE nsresult GetMessageServiceContractIDForURI(const char* uri,
 NS_MSG_BASE nsresult GetMessageServiceFromURI(
     const nsACString& uri, nsIMsgMessageService** aMessageService);
 
-NS_MSG_BASE nsresult GetMsgDBHdrFromURI(const char* uri, nsIMsgDBHdr** msgHdr);
+NS_MSG_BASE nsresult GetMsgDBHdrFromURI(const nsACString& uri,
+                                        nsIMsgDBHdr** msgHdr);
 
 NS_MSG_BASE nsresult NS_MsgGetPriorityFromString(
     const char* const priority, nsMsgPriorityValue& outPriority);
@@ -199,9 +207,6 @@ NS_MSG_BASE nsresult MsgCleanupTempFiles(const char* fileName,
 NS_MSG_BASE nsresult MsgGetFileStream(nsIFile* file,
                                       nsIOutputStream** fileStream);
 
-NS_MSG_BASE nsresult MsgReopenFileStream(nsIFile* file,
-                                         nsIInputStream* fileStream);
-
 // Automatically creates an output stream with a suitable buffer
 NS_MSG_BASE nsresult MsgNewBufferedFileOutputStream(nsIOutputStream** aResult,
                                                     nsIFile* aFile,
@@ -241,17 +246,11 @@ NS_MSG_BASE nsresult MsgUnescapeString(const nsACString& aStr, uint32_t aFlags,
 NS_MSG_BASE nsresult MsgEscapeURL(const nsACString& aStr, uint32_t aFlags,
                                   nsACString& aResult);
 
-// Converts an nsTArray of nsMsgKeys plus a database, to an array of
-// nsIMsgDBHdrs.
-NS_MSG_BASE nsresult MsgGetHeadersFromKeys(nsIMsgDatabase* aDB,
-                                           const nsTArray<nsMsgKey>& aKeys,
-                                           nsIMutableArray* aHeaders);
-
-// Stopgap while we complete removal of nsIArray usage (Bug 1583030).
-// Then this function can replace MsgGetHeadersFromKeys().
+// Given a message db and a set of keys, fetch the corresponding message
+// headers.
 NS_MSG_BASE nsresult
-MsgGetHeadersFromKeys2(nsIMsgDatabase* aDB, const nsTArray<nsMsgKey>& aKeys,
-                       nsTArray<RefPtr<nsIMsgDBHdr>>& aHeaders);
+MsgGetHeadersFromKeys(nsIMsgDatabase* aDB, const nsTArray<nsMsgKey>& aKeys,
+                      nsTArray<RefPtr<nsIMsgDBHdr>>& aHeaders);
 
 NS_MSG_BASE nsresult MsgExamineForProxyAsync(nsIChannel* channel,
                                              nsIProtocolProxyCallback* listener,
@@ -260,7 +259,8 @@ NS_MSG_BASE nsresult MsgExamineForProxyAsync(nsIChannel* channel,
 NS_MSG_BASE int32_t MsgFindCharInSet(const nsCString& aString,
                                      const char* aChars, uint32_t aOffset = 0);
 NS_MSG_BASE int32_t MsgFindCharInSet(const nsString& aString,
-                                     const char* aChars, uint32_t aOffset = 0);
+                                     const char16_t* aChars,
+                                     uint32_t aOffset = 0);
 
 // advances bufferOffset to the beginning of the next line, if we don't
 // get to maxBufferOffset first. Returns false if we didn't get to the
@@ -300,8 +300,8 @@ NS_MSG_BASE PRTime MsgConvertAgeInDaysToCutoffDate(int32_t ageInDays);
  * @param[out] aOutString   result representation of search terms.
  *
  */
-NS_MSG_BASE nsresult MsgTermListToString(nsIArray* aTermList,
-                                         nsCString& aOutString);
+NS_MSG_BASE nsresult MsgTermListToString(
+    nsTArray<RefPtr<nsIMsgSearchTerm>> const& aTermList, nsCString& aOutString);
 
 NS_MSG_BASE nsresult MsgStreamMsgHeaders(nsIInputStream* aInputStream,
                                          nsIStreamListener* aConsumer);
@@ -330,12 +330,11 @@ NS_MSG_BASE nsresult MsgDetectCharsetFromFile(nsIFile* aFile,
  * unknown or deemed of no importance NULL could be passed.
  * @param[in/out] aConBuf        Variable with the text to convert
  * @param         formatFlowed   Use format flowed?
- * @param         delsp          Use delsp=yes when flowed
  * @param         formatOutput   Reformat the output?
  & @param         disallowBreaks Disallow breaks when formatting
  */
 NS_MSG_BASE nsresult ConvertBufToPlainText(nsString& aConBuf, bool formatFlowed,
-                                           bool delsp, bool formatOutput,
+                                           bool formatOutput,
                                            bool disallowBreaks);
 
 #include "nsEscape.h"
@@ -404,17 +403,10 @@ NS_MSG_BASE
 void MsgLogToConsole4(const nsAString& aErrorText, const nsAString& aFilename,
                       uint32_t aLine, uint32_t flags);
 
-/**
- * Stopgap helper for removal of nsIArray usage (Bug 1583030).
- * Copy an nsIArray of nsIMsgDBHdrs into an nsTArray.
- * TODO: Delete this function when we can!
- */
-void MsgHdrsToTArray(nsIArray* messages, nsTArray<RefPtr<nsIMsgDBHdr>>& out);
-
 // Macro with filename and line number
-#define MSG_LOG_TO_CONSOLE(_text, _flag)                                  \
-  MsgLogToConsole4(NS_LITERAL_STRING(_text), NS_LITERAL_STRING(__FILE__), \
-                   __LINE__, _flag)
+#define MSG_LOG_TO_CONSOLE(_text, _flag)                  \
+  MsgLogToConsole4(NS_LITERAL_STRING_FROM_CSTRING(_text), \
+                   NS_LITERAL_STRING_FROM_CSTRING(__FILE__), __LINE__, _flag)
 #define MSG_LOG_ERR_TO_CONSOLE(_text) \
   MSG_LOG_TO_CONSOLE(_text, nsIScriptError::errorFlag)
 #define MSG_LOG_WARN_TO_CONSOLE(_text) \
@@ -446,4 +438,10 @@ void MsgHdrsToTArray(nsIArray* messages, nsTArray<RefPtr<nsIMsgDBHdr>>& out);
     fprintf(stderr, "(info) %s (%s:%d)\n", _txt, __FILE__, __LINE__); \
   } while (0)
 
+/**
+ * Perform C-style string escaping. E.g. "foo\r\n" => "foo\\r\\n"
+ * This is primarily intended for debuggin purposes.
+ */
+nsCString CEscapeString(nsACString const& s);
+
 #endif
