@@ -11,19 +11,19 @@
 #include "nsIDBFolderInfo.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgCopyService.h"
-#include "nsICopyMsgStreamListener.h"
 #include "nsMsgUtils.h"
 #include "nsIMsgSearchSession.h"
+#include "nsIMsgSearchTerm.h"
 #include "nsMsgDBCID.h"
 #include "nsMsgMessageFlags.h"
 #include "nsServiceManagerUtils.h"
-#include "nsIMutableArray.h"
 
 nsMsgXFVirtualFolderDBView::nsMsgXFVirtualFolderDBView() {
   mSuppressMsgDisplay = false;
   m_doingSearch = false;
   m_doingQuickSearch = false;
   m_totalMessagesInView = 0;
+  m_curFolderHasCachedHits = false;
 }
 
 nsMsgXFVirtualFolderDBView::~nsMsgXFVirtualFolderDBView() {}
@@ -136,8 +136,8 @@ nsresult nsMsgXFVirtualFolderDBView::OnNewHeader(nsIMsgDBHdr* newHdr,
 
 NS_IMETHODIMP
 nsMsgXFVirtualFolderDBView::OnHdrPropertyChanged(
-    nsIMsgDBHdr* aHdrChanged, bool aPreChange, uint32_t* aStatus,
-    nsIDBChangeListener* aInstigator) {
+    nsIMsgDBHdr* aHdrChanged, const nsACString& property, bool aPreChange,
+    uint32_t* aStatus, nsIDBChangeListener* aInstigator) {
   // If the junk mail plugin just activated on a message, then
   // we'll allow filters to remove from view.
   // Otherwise, just update the view line.
@@ -189,7 +189,7 @@ void nsMsgXFVirtualFolderDBView::UpdateCacheAndViewForFolder(
     nsCString searchUri;
     m_viewFolder->GetURI(searchUri);
     nsTArray<nsMsgKey> badHits;
-    rv = db->RefreshCache(searchUri.get(), newHits, badHits);
+    rv = db->RefreshCache(searchUri, newHits, badHits);
     if (NS_SUCCEEDED(rv)) {
       nsCOMPtr<nsIMsgDBHdr> badHdr;
       for (nsMsgKey badKey : badHits) {
@@ -261,7 +261,7 @@ nsMsgXFVirtualFolderDBView::OnSearchHit(nsIMsgDBHdr* aMsgHdr,
     if (NS_SUCCEEDED(rv)) {
       nsCString searchUri;
       m_viewFolder->GetURI(searchUri);
-      dbToUse->HdrIsInCache(searchUri.get(), aMsgHdr, &hdrInCache);
+      dbToUse->HdrIsInCache(searchUri, aMsgHdr, &hdrInCache);
     }
   }
 
@@ -381,18 +381,17 @@ nsMsgXFVirtualFolderDBView::OnNewSearch() {
 
   nsCString terms;
   dbFolderInfo->GetCharProperty("searchStr", terms);
-  nsCOMPtr<nsIMutableArray> searchTerms;
-  rv = searchSession->GetSearchTerms(getter_AddRefs(searchTerms));
+  nsTArray<RefPtr<nsIMsgSearchTerm>> searchTerms;
+  rv = searchSession->GetSearchTerms(searchTerms);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCString curSearchAsString;
 
   rv = MsgTermListToString(searchTerms, curSearchAsString);
   // Trim off the initial AND/OR, which is irrelevant and inconsistent between
   // what SearchSpec.jsm generates, and what's in virtualFolders.dat.
-  curSearchAsString.Cut(
-      0,
-      StringBeginsWith(curSearchAsString, NS_LITERAL_CSTRING("AND")) ? 3 : 2);
-  terms.Cut(0, StringBeginsWith(terms, NS_LITERAL_CSTRING("AND")) ? 3 : 2);
+  curSearchAsString.Cut(0,
+                        StringBeginsWith(curSearchAsString, "AND"_ns) ? 3 : 2);
+  terms.Cut(0, StringBeginsWith(terms, "AND"_ns) ? 3 : 2);
 
   NS_ENSURE_SUCCESS(rv, rv);
   // If the search session search string doesn't match the vf search str,
@@ -407,7 +406,6 @@ nsMsgXFVirtualFolderDBView::OnNewSearch() {
     nsCOMPtr<nsIMsgFolder> searchFolder;
     searchSession->GetNthSearchScope(i, &scopeId, getter_AddRefs(searchFolder));
     if (searchFolder) {
-      nsCOMPtr<nsISimpleEnumerator> cachedHits;
       nsCOMPtr<nsIMsgDatabase> searchDB;
       nsCString searchUri;
       m_viewFolder->GetURI(searchUri);
@@ -420,26 +418,25 @@ nsMsgXFVirtualFolderDBView::OnNewSearch() {
         // Ignore cached hits in quick search case.
         if (m_doingQuickSearch) continue;
 
-        searchDB->GetCachedHits(searchUri.get(), getter_AddRefs(cachedHits));
+        nsCOMPtr<nsIMsgEnumerator> cachedHits;
+        searchDB->GetCachedHits(searchUri, getter_AddRefs(cachedHits));
         bool hasMore;
         if (cachedHits) {
           cachedHits->HasMoreElements(&hasMore);
           if (hasMore) {
             mozilla::DebugOnly<nsMsgKey> prevKey = nsMsgKey_None;
             while (hasMore) {
-              nsCOMPtr<nsISupports> supports;
-              nsresult rv = cachedHits->GetNext(getter_AddRefs(supports));
-              NS_ASSERTION(NS_SUCCEEDED(rv), "nsMsgDBEnumerator broken");
-              nsCOMPtr<nsIMsgDBHdr> pHeader = do_QueryInterface(supports);
-              if (pHeader && NS_SUCCEEDED(rv)) {
+              nsCOMPtr<nsIMsgDBHdr> header;
+              nsresult rv = cachedHits->GetNext(getter_AddRefs(header));
+              if (header && NS_SUCCEEDED(rv)) {
                 nsMsgKey msgKey;
-                pHeader->GetMessageKey(&msgKey);
+                header->GetMessageKey(&msgKey);
                 NS_ASSERTION(prevKey == nsMsgKey_None || msgKey > prevKey,
                              "cached Hits not sorted");
 #ifdef DEBUG
                 prevKey = msgKey;
 #endif
-                AddHdrFromFolder(pHeader, searchFolder);
+                AddHdrFromFolder(header, searchFolder);
               } else {
                 break;
               }
@@ -500,6 +497,6 @@ nsMsgXFVirtualFolderDBView::SetViewFlags(nsMsgViewFlagsTypeValue aViewFlags) {
 }
 
 nsresult nsMsgXFVirtualFolderDBView::GetMessageEnumerator(
-    nsISimpleEnumerator** enumerator) {
+    nsIMsgEnumerator** enumerator) {
   return GetViewEnumerator(enumerator);
 }
