@@ -27,8 +27,8 @@ function run_test() {
   // Get messageCount messages into the local filestore.
   do_test_pending();
 
-  // function setupVirtualFolder() continues the testing after CopyFileMessage.
-  MailServices.copy.CopyFileMessage(
+  // function setupVirtualFolder() continues the testing after copyFileMessage.
+  MailServices.copy.copyFileMessage(
     bugmail1,
     localAccountUtils.inboxFolder,
     null,
@@ -51,7 +51,7 @@ var copyListener = {
   SetMessageId(aMessageId) {},
   OnStopCopy(aStatus) {
     if (--messageCount) {
-      MailServices.copy.CopyFileMessage(
+      MailServices.copy.copyFileMessage(
         bugmail1,
         localAccountUtils.inboxFolder,
         null,
@@ -81,29 +81,14 @@ function setupVirtualFolder() {
   MailServices.tags.addTagForKey(tag1, tag1, null, null);
 
   // add tag1 to 4 messages
-  var messages0to3 = Cc["@mozilla.org/array;1"].createInstance(
-    Ci.nsIMutableArray
-  );
-  for (var i = 0; i <= 3; i++) {
-    messages0to3.appendElement(hdrs[i]);
-  }
+  let messages0to3 = [hdrs[0], hdrs[1], hdrs[2], hdrs[3]];
   localAccountUtils.inboxFolder.addKeywordsToMessages(messages0to3, tag1);
 
   // set 3 messages unread, 2 messages read
-  var messages0to2 = Cc["@mozilla.org/array;1"].createInstance(
-    Ci.nsIMutableArray
-  );
-  for (i = 0; i <= 2; i++) {
-    messages0to2.appendElement(hdrs[i]);
-  }
+  let messages0to2 = [hdrs[0], hdrs[1], hdrs[2]];
   localAccountUtils.inboxFolder.markMessagesRead(messages0to2, false);
 
-  var messages3to4 = Cc["@mozilla.org/array;1"].createInstance(
-    Ci.nsIMutableArray
-  );
-  for (i = 3; i <= 4; i++) {
-    messages3to4.appendElement(hdrs[i]);
-  }
+  let messages3to4 = [hdrs[3], hdrs[4]];
   localAccountUtils.inboxFolder.markMessagesRead(messages3to4, true);
 
   // search will look for tag tag1 in the inbox folder
@@ -177,9 +162,7 @@ function testVirtualFolder() {
   Assert.equal(3, virtualFolder.getNumUnread(false));
 
   // change unread of one item in search to decrease count
-  var message0 = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
-  message0.appendElement(hdrs[0]);
-  localAccountUtils.inboxFolder.markMessagesRead(message0, true);
+  localAccountUtils.inboxFolder.markMessagesRead([hdrs[0]], true);
   virtualFolder.updateSummaryTotals(true);
 
   Assert.equal(2, virtualFolder.getNumUnread(false));
@@ -187,9 +170,7 @@ function testVirtualFolder() {
   // failures fixed in this bug
 
   // remove tag from one item to decrease count
-  var message1 = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
-  message1.appendElement(hdrs[1]);
-
+  var message1 = [hdrs[1]];
   localAccountUtils.inboxFolder.removeKeywordsFromMessages(message1, tag1);
   virtualFolder.updateSummaryTotals(true);
   Assert.equal(3, virtualFolder.getTotalMessages(false));
@@ -229,7 +210,7 @@ function CreateVirtualFolder(
   // use acctMgr to setup the virtual folder listener
   var acctMgr = MailServices.accounts.QueryInterface(Ci.nsIFolderListener);
   // print(acctMgr);
-  acctMgr.OnItemAdded(null, newFolder);
+  acctMgr.onFolderAdded(parentFolder, newFolder);
   return newFolder;
 }
 
