@@ -5,21 +5,23 @@
 const { FileUtils } = ChromeUtils.import(
   "resource://gre/modules/FileUtils.jsm"
 );
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
 const { PromiseTestUtils } = ChromeUtils.import(
   "resource://testing-common/mailnews/PromiseTestUtils.jsm"
 );
 var { convertMailStoreTo } = ChromeUtils.import(
   "resource:///modules/mailstoreConverter.jsm"
 );
-const { Log } = ChromeUtils.import("resource://gre/modules/Log.jsm");
-const { allAccountsSorted } = ChromeUtils.import(
-  "resource:///modules/folderUtils.jsm"
+const { FolderUtils } = ChromeUtils.import(
+  "resource:///modules/FolderUtils.jsm"
 );
 
 // XXX: merge into test_converter.js
 
-var log = Log.repository.getLogger("MailStoreConverter");
+var log = console.createInstance({
+  prefix: "mail.mailstoreconverter",
+  maxLogLevel: "Warn",
+  maxLogLevelPref: "mail.mailstoreconverter.loglevel",
+});
 
 Services.prefs.setCharPref(
   "mail.serverDefaultStoreContractID",
@@ -58,7 +60,7 @@ var EventTarget = function() {
 
 function copyFileMessage(file, destFolder, isDraftOrTemplate) {
   let listener = new PromiseTestUtils.PromiseCopyListener(copyListenerWrap);
-  MailServices.copy.CopyFileMessage(
+  MailServices.copy.copyFileMessage(
     file,
     destFolder,
     null,
@@ -81,7 +83,7 @@ function checkConversion(source, target) {
     let sourceContentName = sourceContent.leafName;
     let ext = sourceContentName.substr(-4);
     let targetFile = FileUtils.File(
-      OS.Path.join(target.path, sourceContentName)
+      PathUtils.join(target.path, sourceContentName)
     );
     log.debug("Checking path: " + targetFile.path);
     if (ext == ".dat") {
@@ -91,9 +93,9 @@ function checkConversion(source, target) {
       checkConversion(sourceContent, targetFile);
     } else if (ext != ".msf") {
       Assert.ok(targetFile.exists());
-      let cur = FileUtils.File(OS.Path.join(targetFile.path, "cur"));
+      let cur = FileUtils.File(PathUtils.join(targetFile.path, "cur"));
       Assert.ok(cur.exists());
-      let tmp = FileUtils.File(OS.Path.join(targetFile.path, "tmp"));
+      let tmp = FileUtils.File(PathUtils.join(targetFile.path, "tmp"));
       Assert.ok(tmp.exists());
       if (targetFile.leafName == "Inbox") {
         let curContents = cur.directoryEntries;
@@ -143,7 +145,7 @@ function run_test() {
   // String to hold names of accounts to convert.
   let accountsToConvert = "";
 
-  let accounts = allAccountsSorted(true);
+  let accounts = FolderUtils.allAccountsSorted(true);
   for (let account of accounts) {
     if (
       account.incomingServer.rootFolder.filePath.path == deferredToRootFolder
