@@ -40,7 +40,10 @@ var gMsgHdrs = [];
 var gExpectedInboxSize;
 var gExpectedFolder2Size;
 var gExpectedFolder3Size;
-var gMsgLinebreak = "";
+// Use this to account for the size of the separating line after the message
+// body. The compactor tries to preserve the convention already in the mbox,
+// and we know that our test messages have CRLFs, so that's what we'll use.
+var gSeparatorLine = "\r\n";
 
 // Transfer message keys between function calls.
 var gMsgKeys = [];
@@ -64,14 +67,13 @@ var urlListenerWrap = {
 
     if (gMsgKeys.length > 0) {
       // Bug 854798: Check if the new message keys are the same as before compaction.
-      let folderMsgs = gMsgKeys.folder.messages;
+      let folderMsgs = [...gMsgKeys.folder.messages];
       // First message was deleted so skip it in the old array.
-      for (let i = 1; i < gMsgKeys.length; i++) {
-        Assert.ok(folderMsgs.hasMoreElements());
-        let header = folderMsgs.getNext().QueryInterface(Ci.nsIMsgDBHdr);
-        Assert.equal(header.messageKey, gMsgKeys[i]);
+      let expectedKeys = [...gMsgKeys].slice(1);
+      Assert.equal(folderMsgs.length, expectedKeys.length);
+      for (let i = 1; i < expectedKeys.length; i++) {
+        Assert.equal(folderMsgs[i], expectedKeys[i]);
       }
-      Assert.ok(!folderMsgs.hasMoreElements());
       gMsgKeys.length = 0;
     }
   },
@@ -79,7 +81,7 @@ var urlListenerWrap = {
 
 function copyFileMessage(file, destFolder, isDraftOrTemplate) {
   let listener = new PromiseTestUtils.PromiseCopyListener(copyListenerWrap);
-  MailServices.copy.CopyFileMessage(
+  MailServices.copy.copyFileMessage(
     file,
     destFolder,
     null,
@@ -94,13 +96,9 @@ function copyFileMessage(file, destFolder, isDraftOrTemplate) {
 
 function copyMessages(items, isMove, srcFolder, destFolder) {
   let listener = new PromiseTestUtils.PromiseCopyListener(copyListenerWrap);
-  var array = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
-  items.forEach(function(item) {
-    array.appendElement(item);
-  });
-  MailServices.copy.CopyMessages(
+  MailServices.copy.copyMessages(
     srcFolder,
-    array,
+    items,
     destFolder,
     isMove,
     listener,
@@ -111,31 +109,16 @@ function copyMessages(items, isMove, srcFolder, destFolder) {
 }
 
 function deleteMessages(srcFolder, items) {
-  var array = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
-  items.forEach(function(item) {
-    array.appendElement(item);
-  });
-
   let listener = new PromiseTestUtils.PromiseCopyListener(copyListenerWrap);
-  srcFolder.deleteMessages(array, null, false, true, listener, true);
+  srcFolder.deleteMessages(items, null, false, true, listener, true);
   return listener.promise;
 }
 
 function calculateFolderSize(folder) {
   let msgDB = folder.msgDatabase;
-  let enumerator = msgDB.EnumerateMessages();
   let totalSize = 0;
-  if (enumerator) {
-    if (gMsgLinebreak == "") {
-      // Figure out what the linebreak sequence is on the platform running the tests.
-      gMsgLinebreak =
-        "@mozilla.org/windows-registry-key;1" in Cc ? "\r\n" : "\n";
-    }
-    for (let header of enumerator) {
-      if (header instanceof Ci.nsIMsgDBHdr) {
-        totalSize += header.messageSize + gMsgLinebreak.length;
-      }
-    }
+  for (let header of msgDB.EnumerateMessages()) {
+    totalSize += header.messageSize + gSeparatorLine.length;
   }
   return totalSize;
 }
@@ -218,7 +201,7 @@ var gTestArray = [
     }
 
     // Now delete the message
-    await deleteMessages(gLocalFolder3, [gMsgHdrs[0].hdr], false, false);
+    await deleteMessages(gLocalFolder3, [gMsgHdrs[0].hdr]);
 
     showMessages(gLocalFolder3, "after deleting 1 message to trash");
   },
@@ -244,7 +227,7 @@ var gTestArray = [
     }
 
     // Now delete the message
-    await deleteMessages(gLocalFolder2, [gMsgHdrs[0].hdr], false, false);
+    await deleteMessages(gLocalFolder2, [gMsgHdrs[0].hdr]);
 
     showMessages(gLocalFolder2, "after deleting 1 message");
   },
@@ -288,7 +271,7 @@ var gTestArray = [
       },
     };
     let listener = new PromiseTestUtils.PromiseUrlListener(checkResult);
-    localAccountUtils.inboxFolder.compactAll(listener, null, true);
+    localAccountUtils.inboxFolder.compactAll(listener, null);
     await listener.promise;
 
     showMessages(localAccountUtils.inboxFolder, "after compactAll");
@@ -341,16 +324,10 @@ function run_test() {
 
 // debug utility to show the key/offset/ID relationship of messages in a folder
 function showMessages(folder, text) {
-  dump("Show messages for folder <" + folder.name + "> " + text + "\n");
-  for (let header of folder.messages) {
+  dump(`***** Show messages for folder <${folder.name}> "${text} *****\n`);
+  for (let hdr of folder.messages) {
     dump(
-      "key: " +
-        header.messageKey +
-        " offset: " +
-        header.messageOffset +
-        " ID: " +
-        header.messageId +
-        "\n"
+      `  key: ${hdr.messageKey} offset: ${hdr.messageOffset} size: ${hdr.messageSize} ID: ${hdr.messageId}\n`
     );
   }
 }
