@@ -1,28 +1,27 @@
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 /*
  * Attempt to test nsMsgDBView's handling of sorting by sender/recipients
  * when using a display name from the address book.
  */
 
-/* import-globals-from ../../../test/resources/logHelper.js */
-/* import-globals-from ../../../test/resources/asyncTestUtils.js */
-load("../../../resources/logHelper.js");
-load("../../../resources/asyncTestUtils.js");
-
-/* import-globals-from ../../../test/resources/MessageGenerator.jsm */
-/* import-globals-from ../../../test/resources/messageModifier.js */
-/* import-globals-from ../../../test/resources/messageInjection.js */
 /* import-globals-from ../../../test/resources/abSetup.js */
-load("../../../resources/MessageGenerator.jsm");
-load("../../../resources/messageModifier.js");
-load("../../../resources/messageInjection.js");
 load("../../../resources/abSetup.js");
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-
+var { MessageGenerator, SyntheticMessageSet } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageGenerator.jsm"
+);
+var { MessageInjection } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageInjection.jsm"
+);
 var gMessageGenerator = new MessageGenerator();
+var messageInjection = new MessageInjection({ mode: "local" });
 
 Services.prefs.setBoolPref("mail.showCondensedAddresses", true);
 
@@ -36,9 +35,7 @@ var cards = [
   { email: "ggg@h.invalid", displayName: "1" },
 ];
 
-function run_test() {
-  configure_message_injection({ mode: "local" });
-
+add_task(async function setupModule() {
   // Ensure all the directories are initialised.
   MailServices.ab.directories;
 
@@ -75,11 +72,11 @@ function run_test() {
   );
 
   let msgSet = new SyntheticMessageSet(messages);
+  gTestFolder = await messageInjection.makeEmptyFolder();
+  await messageInjection.addSetsToFolders([gTestFolder], [msgSet]);
+});
 
-  do_test_pending();
-
-  gTestFolder = make_empty_folder();
-  add_sets_to_folders(gTestFolder, [msgSet]);
+add_task(function test_view_sort_by_addresses() {
   // - create the view
   setup_view("threaded", Ci.nsMsgViewFlagsType.kNone);
   // Check that sorting by sender uses the display name
@@ -107,23 +104,7 @@ function run_test() {
   if (recip2 != 3) {
     view_throw("expected recip 2 to be 3");
   }
-
-  do_test_finished();
-}
-
-var gCommandUpdater = {
-  updateCommandStatus() {
-    // the back end is smart and is only telling us to update command status
-    // when the # of items in the selection has actually changed.
-  },
-
-  displayMessageChanged(aFolder, aSubject, aKeywords) {},
-
-  updateNextMessageAfterDelete() {},
-  summarizeSelection() {
-    return false;
-  },
-};
+});
 
 var WHITESPACE = "                                              ";
 /**
@@ -173,7 +154,7 @@ function setup_view(aViewType, aViewFlags, aTestFolder) {
   aViewFlags |= Ci.nsMsgViewFlagsType.kExpandAll;
 
   gDBView = Cc[dbviewContractId].createInstance(Ci.nsIMsgDBView);
-  gDBView.init(null, null, gCommandUpdater);
+  gDBView.init(null, null, null);
   var outCount = {};
   gDBView.open(
     aViewType != "search" ? aTestFolder : null,
