@@ -7,7 +7,7 @@
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
 #include "prmem.h"
-#include "nsMsgSend.h"
+#include "nsIStringBundle.h"
 #include "nsIIOService.h"
 #include "nsIHttpProtocolHandler.h"
 #include "nsMailHeaders.h"
@@ -31,7 +31,9 @@
 #include <ctype.h>
 #include "mozilla/dom/Element.h"
 #include "mozilla/mailnews/Services.h"
+#include "mozilla/EncodingDetector.h"
 #include "mozilla/Services.h"
+#include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
 #include "mozilla/ContentIterator.h"
 #include "mozilla/dom/Document.h"
@@ -69,6 +71,19 @@ NS_IMETHODIMP nsMsgCompUtils::GetMsgMimeConformToStandard(bool* _retval) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgCompUtils::DetectCharset(const nsACString& aContent,
+                              nsACString& aCharset) {
+  mozilla::UniquePtr<mozilla::EncodingDetector> detector =
+      mozilla::EncodingDetector::Create();
+  mozilla::Span<const uint8_t> src = mozilla::AsBytes(
+      mozilla::Span(ToNewCString(aContent), aContent.Length()));
+  mozilla::Unused << detector->Feed(src, true);
+  auto encoding = detector->Guess(nullptr, true);
+  encoding->Name(aCharset);
+  return NS_OK;
+}
+
 //
 // Create a file for the a unique temp file
 // on the local machine. Caller must free memory
@@ -295,7 +310,7 @@ nsresult mime_generate_headers(nsIMsgCompFields* fields,
           "User-Agent", NS_ConvertUTF8toUTF16(userAgentString));
   }
 
-  finalHeaders->SetUnstructuredHeader("MIME-Version", NS_LITERAL_STRING("1.0"));
+  finalHeaders->SetUnstructuredHeader("MIME-Version", u"1.0"_ns);
 
   nsAutoCString newsgroups;
   finalHeaders->GetRawHeader("Newsgroups", newsgroups);
@@ -477,296 +492,6 @@ char* mime_make_separator(const char* prefix) {
       rand_buf[10], rand_buf[11]);
 }
 
-static char* LegacyParmFolding(const nsCString& aCharset,
-                               const nsCString& aFileName,
-                               int32_t aParmFolding);
-
-char* mime_generate_attachment_headers(
-    const char* type, const char* type_param, const char* encoding,
-    const char* description, const char* x_mac_type, const char* x_mac_creator,
-    const char* real_name, const char* base_url, bool /*digest_p*/,
-    nsMsgAttachmentHandler* /*ma*/, const char* attachmentCharset,
-    const char* bodyCharset, bool bodyIsAsciiOnly, const char* content_id,
-    bool aBodyDocument) {
-  NS_ASSERTION(encoding, "null encoding");
-
-  int32_t parmFolding = 2;  // RFC 2231-compliant
-  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
-  if (prefs) prefs->GetIntPref("mail.strictly_mime.parm_folding", &parmFolding);
-
-  /* Let's encode the real name */
-  char* encodedRealName = nullptr;
-  nsAutoString realName;
-  if (real_name) {
-    encodedRealName = RFC2231ParmFolding("filename", real_name);
-    // somehow RFC2231ParamFolding failed. fall back to legacy method
-    if (!encodedRealName || !*encodedRealName) {
-      PR_FREEIF(encodedRealName);
-      parmFolding = 0;
-      // Not RFC 2231 style encoding (it's not standard-compliant)
-      encodedRealName =
-          LegacyParmFolding(NS_LITERAL_CSTRING("UTF-8"),
-                            nsDependentCString(real_name), parmFolding);
-    }
-  }
-
-  nsCString buf;  // very likely to be longer than 64 characters
-  buf.AppendLiteral("Content-Type: ");
-  buf.Append(type);
-  if (type_param && *type_param) {
-    if (*type_param != ';') buf.AppendLiteral("; ");
-    buf.Append(type_param);
-  }
-
-  if (mime_type_needs_charset(type)) {
-    char charset_label[65] = "";  // Content-Type: charset
-    if (attachmentCharset) {
-      PL_strncpy(charset_label, attachmentCharset, sizeof(charset_label) - 1);
-      charset_label[sizeof(charset_label) - 1] = '\0';
-    }
-
-    // If the characters are all 7bit, arguably it's better to
-    // claim the charset to be US-ASCII. However, it causes
-    // a major 'interoperability problem' with MS OE, which makes it hard
-    // to sell Mozilla/TB to people most of whose correspondents use
-    // MS OE. MS OE turns all non-ASCII characters to question marks
-    // in replies to messages labeled as US-ASCII if users select 'send as is'
-    // with MIME turned on. (with MIME turned off, this happens without
-    // any warning.) To avoid this, we use the label 'US-ASCII' only when
-    // it's explicitly requested by setting the hidden pref.
-    // 'mail.label_ascii_only_mail_as_us_ascii'. (bug 247958)
-    bool labelAsciiAsAscii = false;
-    if (prefs)
-      prefs->GetBoolPref("mail.label_ascii_only_mail_as_us_ascii",
-                         &labelAsciiAsAscii);
-    if (labelAsciiAsAscii && encoding && !PL_strcasecmp(encoding, "7bit") &&
-        bodyIsAsciiOnly)
-      PL_strcpy(charset_label, "us-ascii");
-
-    // If charset is multibyte then no charset to be specified (apply base64
-    // instead). The list of file types match with PickEncoding() where we put
-    // base64 label.
-    if (((attachmentCharset &&
-          !nsMsgI18Nmultibyte_charset(attachmentCharset)) ||
-         ((PL_strcasecmp(type, TEXT_HTML) == 0) ||
-          (PL_strcasecmp(type, TEXT_MDL) == 0) ||
-          (PL_strcasecmp(type, TEXT_PLAIN) == 0) ||
-          (PL_strcasecmp(type, TEXT_RICHTEXT) == 0) ||
-          (PL_strcasecmp(type, TEXT_ENRICHED) == 0) ||
-          (PL_strcasecmp(type, TEXT_VCARD) == 0) ||
-          (PL_strcasecmp(type, APPLICATION_DIRECTORY) ==
-           0) || /* text/x-vcard synonym */
-          (PL_strcasecmp(type, TEXT_CSS) == 0) ||
-          (PL_strcasecmp(type, TEXT_JSSS) == 0)) ||
-         (PL_strcasecmp(encoding, ENCODING_BASE64) != 0)) &&
-        (*charset_label)) {
-      buf.AppendLiteral("; charset=");
-      buf.Append(charset_label);
-    }
-  }
-
-  // Only do this if we are in the body of a message
-  if (aBodyDocument) {
-    // Add format=flowed as in RFC 2646 if we are using that
-    if (type && !PL_strcasecmp(type, "text/plain")) {
-      bool flowed, delsp, formatted, disallowBreaks;
-      GetSerialiserFlags(bodyCharset, &flowed, &delsp, &formatted,
-                         &disallowBreaks);
-      if (flowed) buf.AppendLiteral("; format=flowed");
-      if (delsp) buf.AppendLiteral("; delsp=yes");
-      // else
-      // {
-      // Don't add a markup. Could use
-      //        PUSH_STRING ("; format=fixed");
-      // but it is equivalent to nothing at all and we do want
-      // to save bandwidth. Don't we?
-      // }
-    }
-  }
-
-  if (x_mac_type && *x_mac_type) {
-    buf.AppendLiteral("; x-mac-type=\"");
-    buf.Append(x_mac_type);
-    buf.Append('"');
-  }
-
-  if (x_mac_creator && *x_mac_creator) {
-    buf.AppendLiteral("; x-mac-creator=\"");
-    buf.Append(x_mac_creator);
-    buf.Append('"');
-  }
-
-#ifdef EMIT_NAME_IN_CONTENT_TYPE
-  if (encodedRealName && *encodedRealName) {
-    // Note that we don't need to output the name field if the name encoding is
-    // RFC 2231. If the MUA knows the RFC 2231, it should know the RFC 2183 too.
-    if (parmFolding != 2) {
-      // The underlying JS MIME code will only handle UTF-8 here.
-      char* nameValue =
-          LegacyParmFolding(NS_LITERAL_CSTRING("UTF-8"),
-                            nsDependentCString(real_name), parmFolding);
-      if (!nameValue || !*nameValue) {
-        PR_FREEIF(nameValue);
-        nameValue = encodedRealName;
-      }
-      buf.AppendLiteral(";\r\n name=\"");
-      buf.Append(nameValue);
-      buf.Append('"');
-      if (nameValue != encodedRealName) PR_FREEIF(nameValue);
-    }
-  }
-#endif /* EMIT_NAME_IN_CONTENT_TYPE */
-  buf.Append(CRLF);
-
-  buf.AppendLiteral("Content-Transfer-Encoding: ");
-  buf.Append(encoding);
-  buf.Append(CRLF);
-
-  if (description && *description) {
-    char* s = mime_fix_header(description);
-    if (s) {
-      buf.AppendLiteral("Content-Description: ");
-      buf.Append(s);
-      buf.Append(CRLF);
-      PR_Free(s);
-    }
-  }
-
-  if ((content_id) && (*content_id)) {
-    buf.AppendLiteral("Content-ID: <");
-    buf.Append(content_id);
-    buf.Append('>');
-    buf.Append(CRLF);
-  }
-
-  if (encodedRealName && *encodedRealName) {
-    char* period = PL_strrchr(encodedRealName, '.');
-    int32_t pref_content_disposition = 0;
-
-    if (prefs) {
-      mozilla::DebugOnly<nsresult> rv = prefs->GetIntPref(
-          "mail.content_disposition_type", &pref_content_disposition);
-      NS_ASSERTION(NS_SUCCEEDED(rv),
-                   "failed to get mail.content_disposition_type");
-    }
-
-    buf.AppendLiteral("Content-Disposition: ");
-
-    // If this is an attachment which is part of the message body and therefore
-    // has a Content-ID (e.g, image in HTML msg), then Content-Disposition has
-    // to be inline
-    if (content_id && *content_id)
-      buf.AppendLiteral("inline");
-    else if (pref_content_disposition == 1)
-      buf.AppendLiteral("attachment");
-    else if (pref_content_disposition == 2 &&
-             (!PL_strcasecmp(type, TEXT_PLAIN) ||
-              (period && !PL_strcasecmp(period, ".txt"))))
-      buf.AppendLiteral("attachment");
-
-    // If this document is an anonymous binary file or a vcard,
-    // then always show it as an attachment, never inline.
-    else if (!PL_strcasecmp(type, APPLICATION_OCTET_STREAM) ||
-             !PL_strcasecmp(type, TEXT_VCARD) ||
-             !PL_strcasecmp(type,
-                            APPLICATION_DIRECTORY)) /* text/x-vcard synonym */
-      buf.AppendLiteral("attachment");
-    else
-      buf.AppendLiteral("inline");
-
-    buf.AppendLiteral(";\r\n ");
-    buf.Append(encodedRealName);
-    buf.Append(CRLF);
-  } else if (type && (!PL_strcasecmp(type, MESSAGE_RFC822) ||
-                      !PL_strcasecmp(type, MESSAGE_NEWS)))
-    buf.AppendLiteral("Content-Disposition: inline" CRLF);
-
-#ifdef GENERATE_CONTENT_BASE
-  // If this is an HTML document, and we know the URL it originally
-  // came from, write out a Content-Base header.
-  if (type &&
-      (!PL_strcasecmp(type, TEXT_HTML) || !PL_strcasecmp(type, TEXT_MDL)) &&
-      base_url && *base_url) {
-    int32_t col = 0;
-    const char* s = base_url;
-    const char* colon = PL_strchr(s, ':');
-    bool useContentLocation = false; /* rhp - add this  */
-
-    if (!colon) goto GIVE_UP_ON_CONTENT_BASE; /* malformed URL? */
-
-    // Don't emit a content-base that points to (or into) a news or
-    // mail message.
-    if (!PL_strncasecmp(s, "news:", 5) || !PL_strncasecmp(s, "snews:", 6) ||
-        !PL_strncasecmp(s, "IMAP:", 5) ||
-        !PL_strncasecmp(
-            s, "file:", 5) || /* rhp: fix targets from included HTML files */
-        !PL_strncasecmp(s, "mailbox:", 8))
-      goto GIVE_UP_ON_CONTENT_BASE;
-
-    // rhp - Put in a pref for using Content-Location instead of Content-Base.
-    //       This will get tweaked to default to true in 5.0
-    if (prefs)
-      prefs->GetBoolPref("mail.use_content_location_on_send",
-                         &useContentLocation);
-
-    if (useContentLocation)
-      buf.AppendLiteral("Content-Location: \"");
-    else
-      buf.AppendLiteral("Content-Base: \"");
-    /* rhp - Pref for Content-Location usage */
-
-  /* rhp: this is to work with the Content-Location stuff */
-  CONTENT_LOC_HACK:
-
-    while (*s != 0 && *s != '#') {
-      uint32_t ot = buf.Length();
-      char tmp[] = "\x00\x00";
-      /* URLs must be wrapped at 40 characters or less. */
-      if (col >= 38) {
-        buf.Append(CRLF "\t");
-        col = 0;
-      }
-
-      if (*s == ' ')
-        buf.AppendLiteral("%20");
-      else if (*s == '\t')
-        buf.AppendLiteral("%09");
-      else if (*s == '\n')
-        buf.AppendLiteral("%0A");
-      else if (*s == '\r')
-        buf.AppendLiteral("%0D");
-      else {
-        tmp[0] = *s;
-        buf.Append(tmp);
-      }
-      s++;
-      col += (buf.Length() - ot);
-    }
-    buf.AppendLiteral("\"" CRLF);
-
-    // rhp: this is to try to get around this fun problem with Content-Location
-    if (!useContentLocation) {
-      buf.AppendLiteral("Content-Location: \"");
-      s = base_url;
-      col = 0;
-      useContentLocation = true;
-      goto CONTENT_LOC_HACK;
-    }
-    // rhp: this is to try to get around this fun problem with Content-Location
-
-  GIVE_UP_ON_CONTENT_BASE:;
-  }
-#endif /* GENERATE_CONTENT_BASE */
-
-  /* realloc it smaller... */
-
-#ifdef DEBUG_jungshik
-  printf("header=%s\n", buf.get());
-#endif
-  PR_Free(encodedRealName);
-  return PL_strdup(buf.get());
-}
-
 static bool isValidHost(const char* host) {
   if (host)
     for (const char* s = host; *s; ++s)
@@ -922,28 +647,6 @@ char* msg_generate_message_id(nsIMsgIdentity* identity) {
   return foldedParm;
 }
 
-/*static */ char* LegacyParmFolding(const nsCString& aCharset,
-                                    const nsCString& aFileName,
-                                    int32_t aParmFolding) {
-  bool usemime = nsMsgMIMEGetConformToStandard();
-  char* encodedRealName = nsMsgI18NEncodeMimePartIIStr(
-      aFileName.get(), false, aCharset.get(), 0, usemime);
-
-  if (!encodedRealName || !*encodedRealName) {
-    PR_FREEIF(encodedRealName);
-    encodedRealName = (char*)PR_Malloc(aFileName.Length() + 1);
-    if (encodedRealName) PL_strcpy(encodedRealName, aFileName.get());
-  }
-
-  // Now put backslashes before special characters per RFC 822
-  char* qtextName = msg_make_filename_qtext(encodedRealName, aParmFolding == 0);
-  if (qtextName) {
-    PR_FREEIF(encodedRealName);
-    encodedRealName = qtextName;
-  }
-  return encodedRealName;
-}
-
 bool mime_7bit_data_p(const char* string, uint32_t size) {
   if ((!string) || (!*string)) return true;
 
@@ -1158,144 +861,16 @@ char* msg_make_filename_qtext(const char* srcText, bool stripCRLFs) {
   return newString;
 }
 
-// Rip apart the URL and extract a reasonable value for the `real_name' slot.
-void msg_pick_real_name(nsMsgAttachmentHandler* attachment,
-                        const char16_t* proposedName, const char* charset) {
-  const char *s, *s2;
-
-  if (!attachment->m_realName.IsEmpty()) return;
-
-  if (proposedName && *proposedName) {
-    attachment->m_realName.Adopt(ToNewUTF8String(nsAutoString(proposedName)));
-  } else  // Let's extract the name from the URL
-  {
-    nsCString url;
-    nsresult rv = attachment->mURL->GetSpec(url);
-    if (NS_FAILED(rv)) return;
-
-    s = url.get();
-    s2 = PL_strchr(s, ':');
-    if (s2) s = s2 + 1;
-    // If we know the URL doesn't have a sensible file name in it,
-    // don't bother emitting a content-disposition.
-    if (StringBeginsWith(url, NS_LITERAL_CSTRING("news:"),
-                         nsCaseInsensitiveCStringComparator) ||
-        StringBeginsWith(url, NS_LITERAL_CSTRING("snews:"),
-                         nsCaseInsensitiveCStringComparator) ||
-        StringBeginsWith(url, NS_LITERAL_CSTRING("IMAP:"),
-                         nsCaseInsensitiveCStringComparator) ||
-        StringBeginsWith(url, NS_LITERAL_CSTRING("mailbox:"),
-                         nsCaseInsensitiveCStringComparator))
-      return;
-
-    if (StringBeginsWith(url, NS_LITERAL_CSTRING("data:"),
-                         nsCaseInsensitiveCStringComparator)) {
-      int32_t endNonData = url.FindChar(',');
-      if (endNonData == -1) return;
-      nsCString nonDataPart(Substring(url, 5, endNonData - 5));
-      int32_t filenamePos = nonDataPart.Find("filename=");
-      if (filenamePos != -1) {
-        filenamePos += 9;
-        int32_t endFilename = nonDataPart.FindChar(';', filenamePos);
-        if (endFilename == -1) endFilename = endNonData;
-        attachment->m_realName =
-            Substring(nonDataPart, filenamePos, endFilename - filenamePos);
-      } else {
-        // no filename; need to construct one based on the content type.
-        nsCOMPtr<nsIMIMEService> mimeService(
-            do_GetService(NS_MIMESERVICE_CONTRACTID));
-        if (!mimeService) return;
-        nsCOMPtr<nsIMIMEInfo> mimeInfo;
-        nsCString mediaType(
-            Substring(nonDataPart, 0, nonDataPart.FindChar(';')));
-        mimeService->GetFromTypeAndExtension(mediaType, EmptyCString(),
-                                             getter_AddRefs(mimeInfo));
-        if (!mimeInfo) return;
-        nsCString filename;
-        nsCString extension;
-        mimeInfo->GetPrimaryExtension(extension);
-        unsigned char filePrefixBytes[8];
-        GenerateGlobalRandomBytes(filePrefixBytes, 8);
-        // Create a filename prefix with 16 lowercase letters,
-        // representing 8 bytes.
-        for (int32_t i = 0; i < 8; i++) {
-          // A pair of letters, each any of (a-p).
-          filename.Append((filePrefixBytes[i] & 0xF) + 'a');
-          filename.Append((filePrefixBytes[i] >> 4) + 'a');
-        }
-        filename.Append('.');
-        filename.Append(extension);
-        attachment->m_realName = filename;
-      }
-    } else {
-      /* Take the part of the file name after the last / or \ */
-      s2 = PL_strrchr(s, '/');
-      if (s2) s = s2 + 1;
-      s2 = PL_strrchr(s, '\\');
-
-      if (s2) s = s2 + 1;
-      /* Copy it into the attachment struct. */
-      attachment->m_realName = s;
-      int32_t charPos = attachment->m_realName.FindChar('?');
-      if (charPos != -1) attachment->m_realName.SetLength(charPos);
-      /* Now trim off any named anchors or search data. */
-      charPos = attachment->m_realName.FindChar('#');
-      if (charPos != -1) attachment->m_realName.SetLength(charPos);
-    }
-    /* Now lose the %XX crap. */
-    nsCString unescaped_real_name;
-    MsgUnescapeString(attachment->m_realName, 0, unescaped_real_name);
-    attachment->m_realName = unescaped_real_name;
-  }
-
-  // Now a special case for attaching uuencoded files...
-
-  // If we attach a file "foo.txt.uu", we will send it out with
-  // Content-Type: text/plain; Content-Transfer-Encoding: x-uuencode.
-  // When saving such a file, a mail reader will generally decode it first
-  // (thus removing the uuencoding.)  So, let's make life a little easier by
-  // removing the indication of uuencoding from the file name itself.  (This
-  // will presumably make the file name in the Content-Disposition header be
-  // the same as the file name in the "begin" line of the uuencoded data.)
-
-  // However, since there are mailers out there (including earlier versions of
-  // Mozilla) that will use "foo.txt.uu" as the file name, we still need to
-  // cope with that; the code which copes with that is in the MIME parser, in
-  // libmime/mimei.c.
-  if (attachment->m_already_encoded_p && !attachment->m_encoding.IsEmpty()) {
-    // #### XXX TOTAL KLUDGE.
-    // I'd like to ask the mime.types file, "what extensions correspond
-    // to obj->encoding (which happens to be "x-uuencode") but doing that
-    // in a non-sphagetti way would require brain surgery.  So, since
-    // currently uuencode is the only content-transfer-encoding which we
-    // understand which traditionally has an extension, we just special-
-    // case it here!
-
-    // Note that it's special-cased in a similar way in libmime/mimei.c.
-    if (attachment->m_encoding.LowerCaseEqualsLiteral(ENCODING_UUENCODE) ||
-        attachment->m_encoding.LowerCaseEqualsLiteral(ENCODING_UUENCODE2) ||
-        attachment->m_encoding.LowerCaseEqualsLiteral(ENCODING_UUENCODE3) ||
-        attachment->m_encoding.LowerCaseEqualsLiteral(ENCODING_UUENCODE4)) {
-      if (StringEndsWith(attachment->m_realName, NS_LITERAL_CSTRING(".uu")))
-        attachment->m_realName.Cut(attachment->m_realName.Length() - 3, 3);
-      else if (StringEndsWith(attachment->m_realName,
-                              NS_LITERAL_CSTRING(".uue")))
-        attachment->m_realName.Cut(attachment->m_realName.Length() - 4, 4);
-    }
-  }
-}
-
 // Utility to create a nsIURI object...
 nsresult nsMsgNewURL(nsIURI** aInstancePtrResult, const nsCString& aSpec) {
   nsresult rv = NS_OK;
   if (nullptr == aInstancePtrResult) return NS_ERROR_NULL_POINTER;
   nsCOMPtr<nsIIOService> pNetService = mozilla::services::GetIOService();
   NS_ENSURE_TRUE(pNetService, NS_ERROR_UNEXPECTED);
-  if (aSpec.Find("://") == kNotFound &&
-      !StringBeginsWith(aSpec, NS_LITERAL_CSTRING("data:"))) {
+  if (aSpec.Find("://") == kNotFound && !StringBeginsWith(aSpec, "data:"_ns)) {
     // XXXjag Temporary fix for bug 139362 until the real problem(bug 70083) get
     // fixed
-    nsAutoCString uri(NS_LITERAL_CSTRING("http://"));
+    nsAutoCString uri("http://"_ns);
     uri.Append(aSpec);
     rv = pNetService->NewURI(uri, nullptr, nullptr, aInstancePtrResult);
   } else
@@ -1477,12 +1052,9 @@ void GetFolderURIFromUserPrefs(nsMsgDeliverMode aMode, nsIMsgIdentity* identity,
  * In this function we set all the serialiser flags.
  * 'formatted' is always 'true'.
  */
-void GetSerialiserFlags(const char* charset, bool* flowed, bool* delsp,
-                        bool* formatted, bool* disallowBreaks) {
+void GetSerialiserFlags(bool* flowed, bool* formatted) {
   *flowed = false;
-  *delsp = false;
   *formatted = true;
-  *disallowBreaks = true;
 
   // Set format=flowed as in RFC 2646 according to the preference.
   nsresult rv;
@@ -1490,15 +1062,6 @@ void GetSerialiserFlags(const char* charset, bool* flowed, bool* delsp,
   if (NS_SUCCEEDED(rv)) {
     prefs->GetBoolPref("mailnews.send_plaintext_flowed", flowed);
   }
-
-  // We could test statefulCharset(charset) here, but since ISO-2022-JP is the
-  // only one left we support, we might as well check for it directly.
-  if (PL_strcasecmp(charset, "ISO-2022-JP") == 0) {
-    // Make sure we honour RFC 1468. For encoding in ISO-2022-JP we need to
-    // send short lines to allow 7bit transfer encoding.
-    *disallowBreaks = false;
-    if (*flowed) *delsp = true;
-  }
 }
 
 already_AddRefed<nsIArray> GetEmbeddedObjects(
