@@ -23,14 +23,14 @@
 #include "nsIMimeConverter.h"
 #include "nsMsgMimeCID.h"
 #include "nsComposeStrings.h"
-#include "nsIMutableArray.h"
-#include "nsArrayEnumerator.h"
 #include "nsIObserverService.h"
 #include "nsIMsgLocalMailFolder.h"
 #include "nsIMsgDatabase.h"
+#include "nsIInputStream.h"
+#include "nsIOutputStream.h"
 #include "nsMsgMessageFlags.h"
+#include "mozilla/dom/Promise.h"
 #include "mozilla/Services.h"
-#include "nsArrayUtils.h"
 
 // Consts for checking and sending mail in milliseconds
 
@@ -67,6 +67,8 @@ nsMsgSendLater::nsMsgSendLater() {
 
   mIdentityKey = nullptr;
   mAccountKey = nullptr;
+
+  mUserInitiated = false;
 }
 
 nsMsgSendLater::~nsMsgSendLater() {
@@ -364,7 +366,8 @@ SendOperationListener::SendOperationListener(nsMsgSendLater* aSendLater)
 SendOperationListener::~SendOperationListener(void) {}
 
 NS_IMETHODIMP
-SendOperationListener::OnGetDraftFolderURI(const char* aFolderURI) {
+SendOperationListener::OnGetDraftFolderURI(const char* aMsgID,
+                                           const nsACString& aFolderURI) {
   return NS_OK;
 }
 
@@ -502,6 +505,7 @@ nsresult nsMsgSendLater::CompleteMailFileSend() {
   // Create the listener for the send operation...
   RefPtr<SendOperationListener> sendListener = new SendOperationListener(this);
 
+  RefPtr<mozilla::dom::Promise> promise;
   rv = pMsgSend->SendMessageFile(
       identity, mAccountKey,
       compFields,                   // nsIMsgCompFields *fields,
@@ -510,15 +514,12 @@ nsresult nsMsgSendLater::CompleteMailFileSend() {
       false,                        // bool digest_p,
       nsIMsgSend::nsMsgSendUnsent,  // nsMsgDeliverMode mode,
       nullptr,                      // nsIMsgDBHdr *msgToReplace,
-      sendListener, mFeedback, nullptr);
+      sendListener, mFeedback, nullptr, getter_AddRefs(promise));
   return rv;
 }
 
 nsresult nsMsgSendLater::StartNextMailFileSend(nsresult prevStatus) {
-  bool hasMoreElements = false;
-  if ((!mEnumerator) ||
-      NS_FAILED(mEnumerator->HasMoreElements(&hasMoreElements)) ||
-      !hasMoreElements) {
+  if (mTotalSendCount >= (uint32_t)mMessagesToSend.Count()) {
     // Notify that this message has finished being sent.
     NotifyListenersOnProgress(mTotalSendCount, mMessagesToSend.Count(), 100,
                               100);
@@ -534,20 +535,17 @@ nsresult nsMsgSendLater::StartNextMailFileSend(nsresult prevStatus) {
 
   // If we've already sent a message, and are sending more, send out a progress
   // update with 100% for both send and copy as we must have finished by now.
-  if (mTotalSendCount)
+  if (mTotalSendCount > 0) {
     NotifyListenersOnProgress(mTotalSendCount, mMessagesToSend.Count(), 100,
                               100);
+  }
 
-  nsCOMPtr<nsISupports> currentItem;
-  nsresult rv = mEnumerator->GetNext(getter_AddRefs(currentItem));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  mMessage = do_QueryInterface(currentItem);
-  if (!mMessage) return NS_ERROR_NOT_AVAILABLE;
+  mMessage = mMessagesToSend[mTotalSendCount++];
 
   if (!mMessageFolder) return NS_ERROR_UNEXPECTED;
 
   nsCString messageURI;
+  nsresult rv;
   nsCOMPtr<nsIMsgFolder> folder = do_QueryReferent(mMessageFolder, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   folder->GetUriForMsg(mMessage, messageURI);
@@ -559,8 +557,6 @@ nsresult nsMsgSendLater::StartNextMailFileSend(nsresult prevStatus) {
   rv = GetMessageServiceFromURI(messageURI, getter_AddRefs(messageService));
   if (NS_FAILED(rv) && !messageService) return NS_ERROR_FACTORY_NOT_LOADED;
 
-  ++mTotalSendCount;
-
   nsCString identityKey;
   rv = mMessage->GetStringProperty(HEADER_X_MOZILLA_IDENTITY_KEY,
                                    getter_Copies(identityKey));
@@ -589,8 +585,8 @@ nsresult nsMsgSendLater::StartNextMailFileSend(nsresult prevStatus) {
   // operation
   nsCOMPtr<nsIURI> dummyNull;
   rv = messageService->DisplayMessage(
-      messageURI.get(), static_cast<nsIStreamListener*>(this), nullptr, nullptr,
-      nullptr, getter_AddRefs(dummyNull));
+      messageURI, static_cast<nsIStreamListener*>(this), nullptr, nullptr,
+      false, getter_AddRefs(dummyNull));
 
   return rv;
 }
@@ -718,42 +714,32 @@ nsresult nsMsgSendLater::InternalSendMessages(bool aUserInitiated,
   NS_ENSURE_SUCCESS(rv, rv);
   mIdentity = nullptr;  // don't hold onto the identity since we're a service.
 
-  nsCOMPtr<nsISimpleEnumerator> enumerator;
   nsCOMPtr<nsIMsgFolder> folder = do_QueryReferent(mMessageFolder, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIMsgEnumerator> enumerator;
   rv = folder->GetMessages(getter_AddRefs(enumerator));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // copy all the elements in the enumerator into our isupports array....
-
-  nsCOMPtr<nsISupports> currentItem;
-  nsCOMPtr<nsIMsgDBHdr> messageHeader;
+  // Build mMessagesToSend array.
   bool hasMoreElements = false;
   while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMoreElements)) &&
          hasMoreElements) {
-    rv = enumerator->GetNext(getter_AddRefs(currentItem));
+    nsCOMPtr<nsIMsgDBHdr> messageHeader;
+    rv = enumerator->GetNext(getter_AddRefs(messageHeader));
     if (NS_SUCCEEDED(rv)) {
-      messageHeader = do_QueryInterface(currentItem, &rv);
-      if (NS_SUCCEEDED(rv)) {
-        if (aUserInitiated)
-          // If the user initiated the send, add all messages
+      if (aUserInitiated) {
+        // If the user initiated the send, add all messages
+        mMessagesToSend.AppendObject(messageHeader);
+      } else {
+        // Else just send those that are NOT marked as Queued.
+        uint32_t flags;
+        rv = messageHeader->GetFlags(&flags);
+        if (NS_SUCCEEDED(rv) && !(flags & nsMsgMessageFlags::Queued))
           mMessagesToSend.AppendObject(messageHeader);
-        else {
-          // Else just send those that are NOT marked as Queued.
-          uint32_t flags;
-          rv = messageHeader->GetFlags(&flags);
-          if (NS_SUCCEEDED(rv) && !(flags & nsMsgMessageFlags::Queued))
-            mMessagesToSend.AppendObject(messageHeader);
-        }
       }
     }
   }
 
-  // Now get an enumerator for our array.
-  rv = NS_NewArrayEnumerator(getter_AddRefs(mEnumerator), mMessagesToSend,
-                             NS_GET_IID(nsIMsgDBHdr));
-  NS_ENSURE_SUCCESS(rv, rv);
-
   // We're now sending messages so its time to signal that and reset our counts.
   mSendingMessages = true;
   mTotalSentSuccessfully = 0;
@@ -783,8 +769,7 @@ nsresult nsMsgSendLater::SetOrigMsgDisposition() {
     ParseString(originalMsgURIs, ',', uriArray);
     for (uint32_t i = 0; i < uriArray.Length(); i++) {
       nsCOMPtr<nsIMsgDBHdr> msgHdr;
-      nsresult rv =
-          GetMsgDBHdrFromURI(uriArray[i].get(), getter_AddRefs(msgHdr));
+      nsresult rv = GetMsgDBHdrFromURI(uriArray[i], getter_AddRefs(msgHdr));
       NS_ENSURE_SUCCESS(rv, rv);
       if (msgHdr) {
         // get the folder for the message resource
@@ -792,9 +777,13 @@ nsresult nsMsgSendLater::SetOrigMsgDisposition() {
         msgHdr->GetFolder(getter_AddRefs(msgFolder));
         if (msgFolder) {
           nsMsgDispositionState dispositionSetting =
-              nsIMsgFolder::nsMsgDispositionState_Replied;
-          if (queuedDisposition.EqualsLiteral("forwarded"))
+              nsIMsgFolder::nsMsgDispositionState_None;
+          if (queuedDisposition.EqualsLiteral("replied"))
+            dispositionSetting = nsIMsgFolder::nsMsgDispositionState_Replied;
+          else if (queuedDisposition.EqualsLiteral("forwarded"))
             dispositionSetting = nsIMsgFolder::nsMsgDispositionState_Forwarded;
+          else if (queuedDisposition.EqualsLiteral("redirected"))
+            dispositionSetting = nsIMsgFolder::nsMsgDispositionState_Redirected;
 
           msgFolder->AddMessageDispositionState(msgHdr, dispositionSetting);
         }
@@ -811,17 +800,11 @@ nsresult nsMsgSendLater::DeleteCurrentMessage() {
   }
 
   // Get the composition fields interface
-  nsCOMPtr<nsIMutableArray> msgArray(do_CreateInstance(NS_ARRAY_CONTRACTID));
-  if (!msgArray) return NS_ERROR_FACTORY_NOT_LOADED;
-
   if (!mMessageFolder) return NS_ERROR_UNEXPECTED;
-
-  msgArray->InsertElementAt(mMessage, 0);
-
   nsresult rv;
   nsCOMPtr<nsIMsgFolder> folder = do_QueryReferent(mMessageFolder, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = folder->DeleteMessages(msgArray, nullptr, true, false, nullptr,
+  rv = folder->DeleteMessages({&*mMessage}, nullptr, true, false, nullptr,
                               false /*allowUndo*/);
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
@@ -873,7 +856,6 @@ nsresult nsMsgSendLater::BuildHeaders() {
       case 'b':
         if (!PL_strncasecmp("BCC", buf, end - buf)) {
           header = &m_bcc;
-          prune_p = true;
         }
         break;
       case 'C':
@@ -1205,8 +1187,7 @@ void nsMsgSendLater::EndSendMessages(nsresult aStatus, const char16_t* aMsg,
   // We don't need to keep hold of the database now we've finished sending.
   (void)folder->SetMsgDatabase(nullptr);
 
-  // or the enumerator, temp file or output stream
-  mEnumerator = nullptr;
+  // or temp file or output stream
   mTempFile = nullptr;
   mOutFile = nullptr;
 
@@ -1301,8 +1282,7 @@ nsresult nsMsgSendLater::GetIdentityFromKey(const char* aKey,
   return rv;
 }
 
-NS_IMETHODIMP
-nsMsgSendLater::OnItemAdded(nsIMsgFolder* aParentItem, nsISupports* aItem) {
+nsresult nsMsgSendLater::StartTimer() {
   // No need to trigger if timer is already set
   if (mTimerSet) return NS_OK;
 
@@ -1326,50 +1306,69 @@ nsMsgSendLater::OnItemAdded(nsIMsgFolder* aParentItem, nsISupports* aItem) {
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemRemoved(nsIMsgFolder* aParentItem, nsISupports* aItem) {
+nsMsgSendLater::OnFolderAdded(nsIMsgFolder* /*parent*/,
+                              nsIMsgFolder* /*child*/) {
+  return StartTimer();
+}
+
+NS_IMETHODIMP
+nsMsgSendLater::OnMessageAdded(nsIMsgFolder* /*parent*/, nsIMsgDBHdr* /*msg*/) {
+  return StartTimer();
+}
+
+NS_IMETHODIMP
+nsMsgSendLater::OnFolderRemoved(nsIMsgFolder* /*parent*/,
+                                nsIMsgFolder* /*child*/) {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemPropertyChanged(nsIMsgFolder* aItem,
-                                      const nsACString& aProperty,
-                                      const nsACString& aOldValue,
-                                      const nsACString& aNewValue) {
+nsMsgSendLater::OnMessageRemoved(nsIMsgFolder* /*parent*/,
+                                 nsIMsgDBHdr* /*msg*/) {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemIntPropertyChanged(nsIMsgFolder* aItem,
-                                         const nsACString& aProperty,
-                                         int64_t aOldValue, int64_t aNewValue) {
+nsMsgSendLater::OnFolderPropertyChanged(nsIMsgFolder* aFolder,
+                                        const nsACString& aProperty,
+                                        const nsACString& aOldValue,
+                                        const nsACString& aNewValue) {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemBoolPropertyChanged(nsIMsgFolder* aItem,
-                                          const nsACString& aProperty,
-                                          bool aOldValue, bool aNewValue) {
+nsMsgSendLater::OnFolderIntPropertyChanged(nsIMsgFolder* aFolder,
+                                           const nsACString& aProperty,
+                                           int64_t aOldValue,
+                                           int64_t aNewValue) {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemUnicharPropertyChanged(nsIMsgFolder* aItem,
-                                             const nsACString& aProperty,
-                                             const nsAString& aOldValue,
-                                             const nsAString& aNewValue) {
+nsMsgSendLater::OnFolderBoolPropertyChanged(nsIMsgFolder* aFolder,
+                                            const nsACString& aProperty,
+                                            bool aOldValue, bool aNewValue) {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemPropertyFlagChanged(nsIMsgDBHdr* aItem,
-                                          const nsACString& aProperty,
-                                          uint32_t aOldValue,
-                                          uint32_t aNewValue) {
+nsMsgSendLater::OnFolderUnicharPropertyChanged(nsIMsgFolder* aFolder,
+                                               const nsACString& aProperty,
+                                               const nsAString& aOldValue,
+                                               const nsAString& aNewValue) {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgSendLater::OnItemEvent(nsIMsgFolder* aItem, const nsACString& aEvent) {
+nsMsgSendLater::OnFolderPropertyFlagChanged(nsIMsgDBHdr* aMsg,
+                                            const nsACString& aProperty,
+                                            uint32_t aOldValue,
+                                            uint32_t aNewValue) {
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgSendLater::OnFolderEvent(nsIMsgFolder* aFolder, const nsACString& aEvent) {
   return NS_OK;
 }
 
@@ -1400,6 +1399,6 @@ NS_IMETHODIMP
 nsMsgSendLater::GetCurrentTaskName(nsAString& aResult) {
   // XXX Bug 440794 will localize this, left as non-localized whilst we decide
   // on the actual strings and try out the UI.
-  aResult = NS_LITERAL_STRING("Sending Messages");
+  aResult = u"Sending Messages"_ns;
   return NS_OK;
 }
