@@ -2,7 +2,6 @@ var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var { IOUtils } = ChromeUtils.import("resource:///modules/IOUtils.jsm");
 var { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
@@ -108,7 +107,7 @@ var copyListener = {
   onProgress(aMsgID, aProgress, aProgressMax) {},
   onStatus(aMsgID, aMsg) {},
   onStopSending(aMsgID, aStatus, aMsg, aReturnFile) {},
-  onGetDraftFolderURI(aFolderURI) {},
+  onGetDraftFolderURI(aMsgID, aFolderURI) {},
   onSendNotPerformed(aMsgID, aStatus) {},
   onTransportSecurityError(msgID, status, secInfo, location) {},
 
@@ -132,7 +131,7 @@ var copyListener = {
 var progressListener = {
   onStateChange(aWebProgress, aRequest, aStateFlags, aStatus) {
     if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP) {
-      this.resolve(mailTestUtils.firstMsgHdr(gDraftFolder));
+      this.resolve(gDraftFolder && mailTestUtils.firstMsgHdr(gDraftFolder));
     }
   },
 
@@ -203,12 +202,9 @@ function richCreateMessage(
     gDraftFolder = rootFolder.createLocalSubfolder("Drafts");
   }
   // Clear all messages
-  let array = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
-  for (let msg of gDraftFolder.msgDatabase.EnumerateMessages()) {
-    array.appendElement(msg);
-  }
-  if (array.length) {
-    gDraftFolder.deleteMessages(array, null, true, false, null, false);
+  let msgs = [...gDraftFolder.msgDatabase.EnumerateMessages()];
+  if (msgs.length > 0) {
+    gDraftFolder.deleteMessages(msgs, null, true, false, null, false);
   }
 
   // Set attachment
@@ -225,7 +221,7 @@ function richCreateMessage(
     progressListener.reject = reject;
   });
   progress.registerListener(progressListener);
-  msgCompose.SendMsg(
+  msgCompose.sendMsg(
     Ci.nsIMsgSend.nsMsgSaveAsDraft,
     identity,
     account ? account.key : "",
@@ -263,6 +259,17 @@ function getAttachmentFromContent(aContent) {
   return attachments[1];
 }
 
+/**
+ * Get the body part of an MIME message.
+ * @param {string} content - The message content.
+ * @returns {string}
+ */
+function getMessageBody(content) {
+  let separatorIndex = content.indexOf("\r\n\r\n");
+  Assert.equal(content.slice(-2), "\r\n", "Should end with a line break.");
+  return content.slice(separatorIndex + 4, -2);
+}
+
 registerCleanupFunction(function() {
   load(gDEPTH + "mailnews/resources/mailShutdown.js");
 });
