@@ -10,8 +10,6 @@ var server;
 var finished = false;
 
 var sentFolder;
-var originalData;
-var expectedAlertMessage;
 
 var kSender = "from@foo.invalid";
 var kToASCII = "to@foo.invalid";
@@ -19,7 +17,7 @@ var kToValid = "to@v\u00E4lid.foo.invalid";
 var kToValidACE = "to@xn--vlid-loa.foo.invalid";
 var kToInvalid = "b\u00F8rken.to@invalid.foo.invalid";
 var kToInvalidWithoutDomain = "b\u00F8rken.to";
-var NS_ERROR_BUT_DONT_SHOW_ALERT = 0x805530ef;
+var NS_ERROR_ILLEGAL_LOCALPART = 0x80553139;
 
 /* exported alert */
 // for alertTestUtils.js
@@ -29,32 +27,60 @@ function alert(aDialogText, aText) {
     return;
   }
 
+  var composeProps = Services.strings.createBundle(
+    "chrome://messenger/locale/messengercompose/composeMsgs.properties"
+  );
+  var expectedAlertMessage =
+    composeProps.GetStringFromName("sendFailed") +
+    "\n" +
+    composeProps
+      .GetStringFromName("errorIllegalLocalPart2")
+      .replace("%s", kToInvalid);
+
   // we should only get here for the kToInvalid test case
   Assert.equal(test, kToInvalid);
   Assert.equal(aText, expectedAlertMessage);
 }
 
 // message listener implementations
-function msgListener(aRecipient) {
+function MsgSendListener(aRecipient, originalData) {
   this.rcpt = aRecipient;
+  this.originalData = originalData;
 }
 
-msgListener.prototype = {
+/**
+ * @implements {nsIMsgSendListener}
+ * @implements {nsIMsgCopyServiceListener}
+ */
+MsgSendListener.prototype = {
   // nsIMsgSendListener
   onStartSending(aMsgID, aMsgSize) {},
   onProgress(aMsgID, aProgress, aProgressMax) {},
   onStatus(aMsgID, aMsg) {},
   onStopSending(aMsgID, aStatus, aMsg, aReturnFile) {
     try {
-      Assert.equal(aStatus, 0);
-      do_check_transaction(server.playTransaction(), [
-        "EHLO test",
-        "MAIL FROM:<" + kSender + "> BODY=8BITMIME SIZE=" + originalData.length,
-        "RCPT TO:<" + this.rcpt + ">",
-        "DATA",
-      ]);
-      // Compare data file to what the server received
-      Assert.equal(originalData, server._daemon.post);
+      if (test == kToValid || test == kToASCII) {
+        Assert.equal(aStatus, 0);
+        do_check_transaction(server.playTransaction(), [
+          "EHLO test",
+          "MAIL FROM:<" +
+            kSender +
+            "> BODY=8BITMIME SIZE=" +
+            this.originalData.length,
+          "RCPT TO:<" + this.rcpt + ">",
+          "DATA",
+        ]);
+        // Compare data file to what the server received
+        Assert.equal(this.originalData, server._daemon.post);
+      } else {
+        Assert.equal(aStatus, NS_ERROR_ILLEGAL_LOCALPART);
+        do_check_transaction(server.playTransaction(), ["EHLO test"]);
+        // Local address (before the @) has non-ascii char(s) or the @ is
+        // missing from the address. An alert is triggered after the EHLO is
+        // sent. Nothing else occurs so we "finish" the test to avoid
+        // NS_ERROR_ABORT test failure due to timeout waiting for the send
+        // (which doesn't occurs) to complete.
+      }
     } catch (e) {
       do_throw(e);
     } finally {
@@ -63,9 +89,10 @@ msgListener.prototype = {
       while (thread.hasPendingEvents()) {
         thread.processNextEvent(false);
       }
+      do_test_finished();
     }
   },
-  onGetDraftFolderURI(aFolderURI) {},
+  onGetDraftFolderURI(aMsgID, aFolderURI) {},
   onSendNotPerformed(aMsgID, aStatus) {},
   onTransportSecurityError(msgID, status, secInfo, location) {},
 
@@ -86,12 +113,11 @@ msgListener.prototype = {
       var pos = msgData.indexOf("From:");
       Assert.notEqual(pos, -1);
       msgData = msgData.substr(pos);
-      Assert.equal(originalData, msgData);
+      Assert.equal(this.originalData, msgData);
     } catch (e) {
       do_throw(e);
     } finally {
       finished = true;
-      do_test_finished();
     }
   },
 
@@ -102,7 +128,9 @@ msgListener.prototype = {
   ]),
 };
 
-function DoSendTest(aRecipient, aRecipientExpected, aExceptionExpected) {
+async function doSendTest(aRecipient, aRecipientExpected) {
+  info(`Testing send to ${aRecipient} will get sent to ${aRecipientExpected}`);
+  test = aRecipient;
   server = setupServerDaemon();
   server.start();
   var smtpServer = getBasicSmtpServer(server.port);
@@ -111,11 +139,10 @@ function DoSendTest(aRecipient, aRecipientExpected, aExceptionExpected) {
 
   // Random test file with data we don't actually care about. ;-)
   var testFile = do_get_file("data/message1.eml");
-  originalData = IOUtils.loadFileToString(testFile);
+  var originalData = await IOUtils.readUTF8(testFile.path);
 
   // Handle the server in a try/catch/finally loop so that we always will stop
   // the server if something fails.
-  var exceptionCaught = 0;
   try {
     var compFields = Cc[
       "@mozilla.org/messengercompose/composefields;1"
@@ -135,21 +162,20 @@ function DoSendTest(aRecipient, aRecipientExpected, aExceptionExpected) {
       false,
       Ci.nsIMsgSend.nsMsgDeliverNow,
       null,
-      new msgListener(aRecipientExpected),
+      new MsgSendListener(aRecipientExpected, originalData),
       null,
       null
     );
 
     server.performTest();
-
+    do_test_pending();
     do_timeout(10000, function() {
       if (!finished) {
         do_throw("Notifications of message send/copy not received");
       }
     });
-    do_test_pending();
   } catch (e) {
-    exceptionCaught = e.result;
+    Assert.ok(false, "Send fail: " + e);
   } finally {
     server.stop();
     var thread = gThreadManager.currentThread;
@@ -157,28 +183,24 @@ function DoSendTest(aRecipient, aRecipientExpected, aExceptionExpected) {
       thread.processNextEvent(true);
     }
   }
-  Assert.equal(exceptionCaught, aExceptionExpected);
 }
 
-function run_test() {
+add_task(function setup() {
   registerAlertTestUtils();
-  var composeProps = Services.strings.createBundle(
-    "chrome://messenger/locale/messengercompose/composeMsgs.properties"
-  );
-  expectedAlertMessage = composeProps
-    .GetStringFromName("errorIllegalLocalPart")
-    .replace("%s", kToInvalid);
 
   // Ensure we have at least one mail account
   localAccountUtils.loadLocalMailAccount();
   MailServices.accounts.setSpecialFolders();
   sentFolder = localAccountUtils.rootFolder.createLocalSubfolder("Sent");
+});
 
+add_task(async function plainASCIIRecipient() {
   // Test 1:
   // Plain ASCII recipient address.
-  test = kToASCII;
-  DoSendTest(kToASCII, kToASCII, 0);
+  await doSendTest(kToASCII, kToASCII);
+});
 
+add_task(async function domainContainsNonAscii() {
   // Test 2:
   // The recipient's domain part contains a non-ASCII character, hence the
   // address needs to be converted to ACE before sending.
@@ -186,24 +208,21 @@ function run_test() {
   // the message to the remaining - wrong! - address.
   // The new code will translate the domain part to ACE for the SMTP
   // transaction (only), i.e. the To: header will stay as stated by the sender.
-  test = kToValid;
-  DoSendTest(kToValid, kToValidACE, 0);
+  await doSendTest(kToValid, kToValidACE);
+});
 
+add_task(async function localContainsNonAscii() {
   // Test 3:
   // The recipient's local part contains a non-ASCII character, which is not
   // allowed with unextended SMTP.
   // The old code would just strip the invalid character and try to send the
   // message to the remaining - wrong! - address.
   // The new code will present an informational message box and deny sending.
-  test = kToInvalid;
-  DoSendTest(kToInvalid, kToInvalid, NS_ERROR_BUT_DONT_SHOW_ALERT);
+  await doSendTest(kToInvalid, kToInvalid);
+});
 
+add_task(async function invalidCharNoAt() {
   // Test 4:
   // Bug 856506. invalid char without '@' causes crash.
-  test = kToInvalidWithoutDomain;
-  DoSendTest(
-    kToInvalidWithoutDomain,
-    kToInvalidWithoutDomain,
-    NS_ERROR_BUT_DONT_SHOW_ALERT
-  );
-}
+  await doSendTest(kToInvalidWithoutDomain, kToInvalidWithoutDomain);
+});
