@@ -13,39 +13,47 @@
  */
 
 /*
- * Do not include glodaTestHelper because we do not want gloda loaded and it
+ * Do not include GlodaTestHelper because we do not want gloda loaded and it
  *  adds a lot of runtime overhead which makes certain debugging strategies like
  *  using chronicle-recorder impractical.
  */
-/* import-globals-from ../../../../test/resources/logHelper.js */
-/* import-globals-from ../../../../test/resources/asyncTestUtils.js */
-/* import-globals-from ../../../../test/resources/messageGenerator.js */
-/* import-globals-from ../../../../test/resources/messageModifier.js */
-/* import-globals-from ../../../../test/resources/messageInjection.js */
-load("../../../../resources/logHelper.js");
-load("../../../../resources/asyncTestUtils.js");
-
-load("../../../../resources/messageGenerator.js");
-load("../../../../resources/messageModifier.js");
-load("../../../../resources/messageInjection.js");
-
-/* exported scenarios */
-// Create a message generator
-var msgGen = (gMessageGenerator = new MessageGenerator());
-// Create a message scenario generator using that message generator
-var scenarios = new MessageScenarioFactory(msgGen);
 
+var { GlodaDatastore } = ChromeUtils.import(
+  "resource:///modules/gloda/GlodaDatastore.jsm"
+);
 var { MsgHdrToMimeMessage } = ChromeUtils.import(
-  "resource:///modules/gloda/mimemsg.js"
+  "resource:///modules/gloda/MimeMessage.jsm"
+);
+var {
+  MessageGenerator,
+  SyntheticPartLeaf,
+  SyntheticPartMultiAlternative,
+  SyntheticDegeneratePartEmpty,
+  SyntheticPartMultiSignedSMIME,
+  SyntheticPartMultiMixed,
+  SyntheticPartMultiSignedPGP,
+  SyntheticPartMultiRelated,
+  SyntheticPartMultiDigest,
+  SyntheticPartMultiParallel,
+  SyntheticMessageSet,
+} = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageGenerator.jsm"
+);
+var { MessageInjection } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageInjection.jsm"
 );
 
 // While we're at it, we'll also test the correctness of the GlodaAttachment
 // representation, esp. its "I just need the part information to rebuild the
 // URLs" claim.
 var { GlodaFundAttr } = ChromeUtils.import(
-  "resource:///modules/gloda/fundattr.js"
+  "resource:///modules/gloda/GlodaFundAttr.jsm"
 );
 
+const DEATH_TO_NEWLINE_TYPE_THINGS = /[\r\n]+/g;
+var msgGen = new MessageGenerator();
+var messageInjection;
+
 var partText = new SyntheticPartLeaf("I am text! Woo!");
 var partHtml = new SyntheticPartLeaf(
   "<html><head></head><body>I am HTML! Woo! </body></html>",
@@ -125,7 +133,7 @@ var partRelImage = new SyntheticPartLeaf(relImage.body, relImage);
 
 var tachVCard = {
   filename: "bob.vcf",
-  contentType: "text/x-vcard",
+  contentType: "text/vcard",
   encoding: "7bit",
   body: "begin:vcard\nfn:Bob\nend:vcard\n",
 };
@@ -146,7 +154,7 @@ var partTachMessages = [msgGen.makeMessage(), msgGen.makeMessage()];
 var partEmpty = new SyntheticDegeneratePartEmpty();
 
 var messageInfos = [
-  // -- simple
+  // -- Simple
   {
     name: "text/plain",
     bodyPart: partText,
@@ -155,12 +163,12 @@ var messageInfos = [
     name: "text/html",
     bodyPart: partHtml,
   },
-  // -- simply ugly
+  // -- Simply ugly
   {
     name: "text/enriched",
     bodyPart: partEnriched,
   },
-  // -- simple w/attachment
+  // -- Simple w/attachment
   {
     name: "text/plain w/text attachment (=> multipart/mixed)",
     bodyPart: partText,
@@ -201,7 +209,7 @@ var messageInfos = [
     bodyPart: partHtml,
     attachments: [tachApplication],
   },
-  // -- alternatives
+  // -- Alternatives
   {
     name: "multipart/alternative: text/plain, text/html",
     bodyPart: partAlternative,
@@ -233,28 +241,28 @@ var messageInfos = [
   },
   {
     name: "S/MIME alternative with text attachment inside",
-    // we have to do the attachment packing ourselves on this one.
+    // We have to do the attachment packing ourselves on this one.
     bodyPart: new SyntheticPartMultiSignedSMIME(
       new SyntheticPartMultiMixed([partAlternative, partTachText])
     ),
   },
   {
     name: "S/MIME alternative with image attachment inside",
-    // we have to do the attachment packing ourselves on this one.
+    // We have to do the attachment packing ourselves on this one.
     bodyPart: new SyntheticPartMultiSignedSMIME(
       new SyntheticPartMultiMixed([partAlternative, partTachImage])
     ),
   },
   {
     name: "S/MIME alternative with image attachment inside",
-    // we have to do the attachment packing ourselves on this one.
+    // We have to do the attachment packing ourselves on this one.
     bodyPart: new SyntheticPartMultiSignedSMIME(
       new SyntheticPartMultiMixed([partAlternative, partTachVCard])
     ),
   },
   {
     name: "S/MIME alternative with app attachment inside",
-    // we have to do the attachment packing ourselves on this one.
+    // We have to do the attachment packing ourselves on this one.
     bodyPart: new SyntheticPartMultiSignedSMIME(
       new SyntheticPartMultiMixed([partAlternative, partTachApplication])
     ),
@@ -272,16 +280,16 @@ var messageInfos = [
     name: "PGP signed alternative",
     bodyPart: new SyntheticPartMultiSignedPGP(partAlternative),
   },
-  // -- attached RFC822
+  // -- Attached RFC822
   {
-    // not your average attachment, pack ourselves for now
+    // Not your average attachment, pack ourselves for now.
     name: "attached rfc822",
     bodyPart: new SyntheticPartMultiMixed([
       partAlternative,
       partTachMessages[0],
     ]),
   },
-  // -- multipart/related
+  // -- Multipart/related
   {
     name: "multipart/related",
     bodyPart: new SyntheticPartMultiRelated([partHtml, partRelImage]),
@@ -293,23 +301,23 @@ var messageInfos = [
       new SyntheticPartMultiRelated([partHtml, partRelImage]),
     ]),
   },
-  // -- multipart/digest
+  // -- Multipart/digest
   {
     name: "multipart/digest",
     bodyPart: new SyntheticPartMultiDigest(partTachMessages.concat()),
   },
-  // -- multipart/parallel (allegedly the same as mixed)
+  // -- Multipart/parallel (allegedly the same as mixed)
   {
     name: "multipart/parallel",
     bodyPart: new SyntheticPartMultiParallel([partText, partTachImage]),
   },
-  // --- previous bugs
-  // -- bug 495057, text/enriched was being dumb
+  // --- Previous bugs
+  // -- Bug 495057, text/enriched was being dumb
   {
     name: "text/enriched inside related",
     bodyPart: new SyntheticPartMultiRelated([partEnriched]),
   },
-  // -- empty sections
+  // -- Empty sections
   // This was a crasher because the empty part made us try and close the
   //  child preceding the empty part a second time.  The nested multipart led
   //  to the crash providing evidence of the double-close bug but there was
@@ -328,136 +336,23 @@ var messageInfos = [
   },
 ];
 
-function* test_stream_message(info) {
-  let synMsg = gMessageGenerator.makeMessage(info);
-  let synSet = new SyntheticMessageSet([synMsg]);
-  yield add_sets_to_folder(gInbox, [synSet]);
-
-  let msgHdr = synSet.getMsgHdr(0);
-
-  MsgHdrToMimeMessage(msgHdr, null, function(aMsgHdr, aMimeMsg) {
-    verify_stream_message(info, synMsg, aMsgHdr, aMimeMsg);
+add_task(async function setupTest() {
+  messageInjection = new MessageInjection({ mode: "local" }, msgGen);
+  registerCleanupFunction(function() {
+    GlodaDatastore.shutdown();
   });
+});
 
-  yield false;
-}
-
-var deathToNewlineTypeThings = /[\r\n]+/g;
-
-/**
- * Applies any transformations to the synthetic body part that we would expect
- *  to happen to a message during its libmime journey.  It may be better to
- *  just put the expected translations in the synthetic body part instead of
- *  trying to make this method do anything complex.
- */
-function synTransformBody(aSynBodyPart) {
-  let text = aSynBodyPart.body.trim();
-  // this transforms things into HTML apparently...
-  if (aSynBodyPart._contentType == "text/enriched") {
-    // Our job here is just to transform just enough for our example above.
-    // We also could have provided a manual translation on the body part.
-    text = text.replace(/bold/g, "B").replace(/italic/g, "I") + "\n<BR>";
+add_task(async function test_stream_message() {
+  for (let messageInfo of messageInfos) {
+    await stream_message(messageInfo);
   }
-  return text;
-}
-
-function verify_body_part_equivalence(aSynBodyPart, aMimePart) {
-  // the content-type devoid of parameters should match
-  Assert.equal(aSynBodyPart._contentType, aMimePart.contentType);
-
-  // the header representation of the content-type should also match unless
-  //  this is an rfc822 part, in which case it should only match for the
-  //  actual contents.
-  if (aMimePart.contentType != "message/rfc822") {
-    Assert.equal(
-      aSynBodyPart.contentTypeHeaderValue.replace(deathToNewlineTypeThings, ""),
-      aMimePart.get("content-type").replace(deathToNewlineTypeThings, "")
-    );
-  }
-
-  // XXX body part checking will get brittle if we ever actually encode things!
-  if (
-    aSynBodyPart.body &&
-    !aSynBodyPart._filename &&
-    aSynBodyPart._contentType.startsWith("text/")
-  ) {
-    Assert.equal(
-      synTransformBody(aSynBodyPart),
-      aMimePart.body
-        .trim()
-        .replace(/\r/g, "")
-        // Remove stuff added by libmime for HTML parts.
-        .replace(
-          /[\n]*<meta http-equiv="content-type" content="text\/html; .*">[\n]*/g,
-          ""
-        )
-        .replace(/[\n]+<\/body>/, "</body>")
-    );
-  }
-  if (aSynBodyPart.parts) {
-    let iPart;
-    let realPartOffsetCompensator = 0;
-    for (iPart = 0; iPart < aSynBodyPart.parts.length; iPart++) {
-      let subSyn = aSynBodyPart.parts[iPart];
-      // If this is a degenerate empty, it should not produce output, so
-      //  compensate for the offset drift and get on with our lives.
-      if (subSyn instanceof SyntheticDegeneratePartEmpty) {
-        realPartOffsetCompensator--;
-        continue;
-      }
-      let subMime = aMimePart.parts[iPart + realPartOffsetCompensator];
-      // our special case is the signature, which libmime does not expose to us.
-      // ignore! (also, have our too-many-part checker below not trip on this)
-      if (subSyn._contentType != PKCS_SIGNATURE_MIME_TYPE) {
-        if (subMime == null) {
-          do_throw(
-            "No MIME part matching " + subSyn.contentTypeHeaderValue + "\n"
-          );
-        }
-        verify_body_part_equivalence(subSyn, subMime);
-      }
-    }
-    // only check if there are still more mime parts; don't check for a count
-    //  mismatch (the PKCS case from above needs to be handled)
-    if (iPart < aMimePart.parts.length) {
-      do_throw("MIME part has more sub-parts than syn part?");
-    }
-  }
-}
-
-/**
- * Verify the streamed results are what we wanted.  For now, this just means
- *  receiving a representation; we don't check it for correctness.
- */
-function verify_stream_message(aInfo, aSynMsg, aMsgHdr, aMimeMsg) {
-  if (aMimeMsg == null) {
-    do_throw("We really should have gotten a result!");
-  }
-  try {
-    // aMimeMsg is normalized; it only ever actually gets one child...
-    verify_body_part_equivalence(aSynMsg.bodyPart, aMimeMsg.parts[0]);
-  } catch (ex) {
-    dump("Something was wrong with the MIME rep!\n!!!!!!!!\n");
-    dump("Synthetic looks like:\n  " + aSynMsg.prettyString() + "\n\n");
-    dump(
-      "MIME looks like:  \n" + aMimeMsg.prettyString(true, "  ", true) + "\n\n"
-    );
-    do_throw(ex);
-  }
-
-  dump("Everything is just fine.\n");
-  dump("Synthetic looks like:\n  " + aSynMsg.prettyString() + "\n\n");
-  dump(
-    "MIME looks like:\n  " + aMimeMsg.prettyString(true, "  ", false) + "\n\n"
-  );
-
-  async_driver();
-}
+});
 
 /**
  * Stream
  */
-function* test_sane_bodies() {
+add_task(async function test_sane_bodies() {
   // 60 bytes long... (becomes 59 on the other side when \r is dropped)
   let hugeString =
     "don't know what you want but I can't  stream it anymore...\r\n";
@@ -465,17 +360,25 @@ function* test_sane_bodies() {
   for (let i = 0; i < powahsOfTwo; i++) {
     hugeString = hugeString + hugeString;
   }
-  // this will come out to be 60k, of course.
+  // This will come out to be 60k, of course.
   Assert.equal(hugeString.length, 60 * Math.pow(2, powahsOfTwo));
 
-  let synMsg = gMessageGenerator.makeMessage({
+  let synMsg = msgGen.makeMessage({
     body: { body: hugeString, contentType: "text/plain" },
   });
   let synSet = new SyntheticMessageSet([synMsg]);
-  yield add_sets_to_folder(gInbox, [synSet]);
+  await messageInjection.addSetsToFolders(
+    [messageInjection.getInboxFolder()],
+    [synSet]
+  );
 
   let msgHdr = synSet.getMsgHdr(0);
 
+  let promiseResolve;
+  let promise = new Promise(resolve => {
+    promiseResolve = resolve;
+  });
+
   MsgHdrToMimeMessage(
     msgHdr,
     null,
@@ -489,16 +392,16 @@ function* test_sane_bodies() {
             " bytes long but should not be!"
         );
       }
-      async_driver();
+      promiseResolve();
     },
     false,
     { saneBodySize: true }
   );
 
-  yield false;
-}
+  await promise;
+});
 
-// additional testing for the correctness of allAttachments and
+// Additional testing for the correctness of allAttachments and
 // allUserAttachments representation
 
 var partTachNestedMessages = [
@@ -581,14 +484,21 @@ var expectedAttachmentsInfo = [
   },
 ];
 
-function* test_attachments_correctness() {
+add_task(async function test_attachments_correctness() {
   for (let [i, params] of attMessagesParams.entries()) {
-    let synMsg = gMessageGenerator.makeMessage(params);
+    let synMsg = msgGen.makeMessage(params);
     let synSet = new SyntheticMessageSet([synMsg]);
-    yield add_sets_to_folder(gInbox, [synSet]);
+    await messageInjection.addSetsToFolders(
+      [messageInjection.getInboxFolder()],
+      [synSet]
+    );
 
     let msgHdr = synSet.getMsgHdr(0);
-    // dump(synMsg.toMboxString()+"\n\n");
+
+    let promiseResolve;
+    let promise = new Promise(resolve => {
+      promiseResolve = resolve;
+    });
 
     MsgHdrToMimeMessage(
       msgHdr,
@@ -645,14 +555,14 @@ function* test_attachments_correctness() {
           do_throw(e);
         }
 
-        async_driver();
+        promiseResolve();
       },
       false
     );
 
-    yield false;
+    await promise;
   }
-}
+});
 
 var bogusMessage = msgGen.makeMessage({ body: { body: "whatever" } });
 bogusMessage._contentType = "woooooo"; // Breaking abstraction boundaries. Bad.
@@ -675,13 +585,20 @@ var weirdMessageInfos = [
   },
 ];
 
-function* test_part12_not_an_attachment() {
-  let synMsg = gMessageGenerator.makeMessage(weirdMessageInfos[0]);
+add_task(async function test_part12_not_an_attachment() {
+  let synMsg = msgGen.makeMessage(weirdMessageInfos[0]);
   let synSet = new SyntheticMessageSet([synMsg]);
-  yield add_sets_to_folder(gInbox, [synSet]);
+  await messageInjection.addSetsToFolders(
+    [messageInjection.getInboxFolder()],
+    [synSet]
+  );
 
   let msgHdr = synSet.getMsgHdr(0);
-  // dump(synMsg.toMboxString());
+
+  let promiseResolve;
+  let promise = new Promise(resolve => {
+    promiseResolve = resolve;
+  });
 
   MsgHdrToMimeMessage(msgHdr, null, function(aMsgHdr, aMimeMsg) {
     try {
@@ -690,31 +607,140 @@ function* test_part12_not_an_attachment() {
     } catch (e) {
       do_throw(e);
     }
-    async_driver();
+    promiseResolve();
   });
 
-  yield false;
+  await promise;
+});
+
+async function stream_message(info) {
+  let synMsg = msgGen.makeMessage(info);
+  let synSet = new SyntheticMessageSet([synMsg]);
+  await messageInjection.addSetsToFolders(
+    [messageInjection.getInboxFolder()],
+    [synSet]
+  );
+
+  let msgHdr = synSet.getMsgHdr(0);
+
+  let promiseResolve;
+  let promise = new Promise(resolve => {
+    promiseResolve = resolve;
+  });
+  MsgHdrToMimeMessage(msgHdr, null, function(aMsgHdr, aMimeMsg) {
+    verify_stream_message(info, synMsg, aMsgHdr, aMimeMsg);
+    promiseResolve();
+  });
+
+  await promise;
 }
-
-/* ===== Driver ===== */
-
-var tests = [
-  parameterizeTest(test_stream_message, messageInfos),
-  test_sane_bodies,
-  test_attachments_correctness,
-  test_part12_not_an_attachment,
-];
-
-var gInbox;
-
-function run_test() {
-  registerCleanupFunction(function() {
-    let { GlodaDatastore } = ChromeUtils.import(
-      "resource:///modules/gloda/datastore.js"
+/**
+ * Verify the streamed results are what we wanted.  For now, this just means
+ *  receiving a representation; we don't check it for correctness.
+ */
+function verify_stream_message(aInfo, aSynMsg, aMsgHdr, aMimeMsg) {
+  if (aMimeMsg == null) {
+    do_throw("We really should have gotten a result!");
+  }
+  try {
+    // aMimeMsg is normalized; it only ever actually gets one child.
+    verify_body_part_equivalence(aSynMsg.bodyPart, aMimeMsg.parts[0]);
+  } catch (ex) {
+    dump("Something was wrong with the MIME rep!\n!!!!!!!!\n");
+    dump("Synthetic looks like:\n  " + aSynMsg.prettyString() + "\n\n");
+    dump(
+      "MIME looks like:  \n" + aMimeMsg.prettyString(true, "  ", true) + "\n\n"
     );
-    GlodaDatastore.shutdown();
-  });
-  // use mbox injection because the fake server chokes sometimes right now
-  gInbox = configure_message_injection({ mode: "local" });
-  async_run_tests(tests);
+    do_throw(ex);
+  }
+
+  dump("Everything is just fine.\n");
+  dump("Synthetic looks like:\n  " + aSynMsg.prettyString() + "\n\n");
+  dump(
+    "MIME looks like:\n  " + aMimeMsg.prettyString(true, "  ", false) + "\n\n"
+  );
+}
+
+/**
+ * Applies any transformations to the synthetic body part that we would expect
+ *  to happen to a message during its libmime journey.  It may be better to
+ *  just put the expected translations in the synthetic body part instead of
+ *  trying to make this method do anything complex.
+ */
+function synTransformBody(aSynBodyPart) {
+  let text = aSynBodyPart.body.trim();
+  // This transforms things into HTML apparently.
+  if (aSynBodyPart._contentType == "text/enriched") {
+    // Our job here is just to transform just enough for our example above.
+    // We also could have provided a manual translation on the body part.
+    text = text.replace(/bold/g, "B").replace(/italic/g, "I");
+  }
+  return text;
+}
+
+function verify_body_part_equivalence(aSynBodyPart, aMimePart) {
+  // The content-type devoid of parameters should match.
+  Assert.equal(aSynBodyPart._contentType, aMimePart.contentType);
+
+  // The header representation of the content-type should also match unless
+  //  this is an rfc822 part, in which case it should only match for the
+  //  actual contents.
+  if (aMimePart.contentType != "message/rfc822") {
+    Assert.equal(
+      aSynBodyPart.contentTypeHeaderValue.replace(
+        DEATH_TO_NEWLINE_TYPE_THINGS,
+        ""
+      ),
+      aMimePart.get("content-type").replace(DEATH_TO_NEWLINE_TYPE_THINGS, "")
+    );
+  }
+
+  // XXX body part checking will get brittle if we ever actually encode things!
+  if (
+    aSynBodyPart.body &&
+    !aSynBodyPart._filename &&
+    aSynBodyPart._contentType.startsWith("text/")
+  ) {
+    Assert.equal(
+      synTransformBody(aSynBodyPart),
+      aMimePart.body
+        .trim()
+        .replace(/\r/g, "")
+        // Remove stuff added by libmime for HTML parts.
+        .replace(
+          /[\n]*<meta http-equiv="content-type" content="text\/html; .*">[\n]*/g,
+          ""
+        )
+        .replace(/[\n]+<\/body>/, "</body>")
+    );
+  }
+  if (aSynBodyPart.parts) {
+    let iPart;
+    let realPartOffsetCompensator = 0;
+    for (iPart = 0; iPart < aSynBodyPart.parts.length; iPart++) {
+      let subSyn = aSynBodyPart.parts[iPart];
+      // If this is a degenerate empty, it should not produce output, so
+      //  compensate for the offset drift and get on with our lives.
+      if (subSyn instanceof SyntheticDegeneratePartEmpty) {
+        realPartOffsetCompensator--;
+        continue;
+      }
+      let subMime = aMimePart.parts[iPart + realPartOffsetCompensator];
+      // Our special case is the signature, which libmime does not expose to us.
+      // Ignore! (Also, have our too-many-part checker below not trip on this.)
+      if (subSyn._contentType != "application/x-pkcs7-signature") {
+        if (subMime == null) {
+          do_throw(
+            "No MIME part matching " + subSyn.contentTypeHeaderValue + "\n"
+          );
+        }
+        verify_body_part_equivalence(subSyn, subMime);
+      }
+    }
+    // Only check if there are still more mime parts; don't check for a count
+    //  mismatch (the PKCS case from above needs to be handled).
+    if (iPart < aMimePart.parts.length) {
+      do_throw("MIME part has more sub-parts than syn part?");
+    }
+  }
 }
