@@ -13,25 +13,25 @@
 #include "nsIMsgDatabase.h"
 #include "nsMsgHdr.h"
 #include "nsString.h"
-#include "nsIDBChangeListener.h"
 #include "nsIDBChangeAnnouncer.h"
 #include "nsMsgMessageFlags.h"
 #include "nsIMsgFolder.h"
-#include "nsIMutableArray.h"
 #include "nsDBFolderInfo.h"
-#include "nsICollation.h"
-#include "nsIMsgSearchSession.h"
+#include "mozilla/intl/Collator.h"
 #include "nsIMimeConverter.h"
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
 #include "PLDHashTable.h"
 #include "nsTArray.h"
 #include "nsTObserverArray.h"
-#include "nsSimpleEnumerator.h"
+
+using mozilla::intl::Collator;
 
 class nsMsgThread;
 class nsMsgDatabase;
 class nsIMsgThread;
+class nsMsgDBEnumerator;
+class nsMsgDBThreadEnumerator;
 
 const int32_t kMsgDBVersion = 1;
 
@@ -67,56 +67,6 @@ class nsMsgDBService final : public nsIMsgDBService {
   AutoTArray<nsMsgDatabase*, kInitialMsgDBCacheSize> m_dbCache;
 };
 
-class nsMsgDBEnumerator : public nsSimpleEnumerator {
- public:
-  const nsID& DefaultInterface() override { return NS_GET_IID(nsIMsgDBHdr); }
-
-  // nsISimpleEnumerator methods:
-  NS_DECL_NSISIMPLEENUMERATOR
-
-  // nsMsgDBEnumerator methods:
-  typedef nsresult (*nsMsgDBEnumeratorFilter)(nsIMsgDBHdr* hdr, void* closure);
-
-  nsMsgDBEnumerator(nsMsgDatabase* db, nsIMdbTable* table,
-                    nsMsgDBEnumeratorFilter filter, void* closure,
-                    bool iterateForwards = true);
-  void Clear();
-
-  nsresult GetRowCursor();
-  virtual nsresult PrefetchNext();
-  RefPtr<nsMsgDatabase> mDB;
-  nsCOMPtr<nsIMdbTableRowCursor> mRowCursor;
-  mdb_pos mRowPos;
-  nsCOMPtr<nsIMsgDBHdr> mResultHdr;
-  bool mDone;
-  bool mNextPrefetched;
-  bool mIterateForwards;
-  nsMsgDBEnumeratorFilter mFilter;
-  nsCOMPtr<nsIMdbTable> mTable;
-  void* mClosure;
-  // This is used when the caller wants to limit how many headers the
-  // enumerator looks at in any given time slice.
-  mdb_pos mStopPos;
-
- protected:
-  ~nsMsgDBEnumerator() override;
-};
-
-class nsMsgFilteredDBEnumerator : public nsMsgDBEnumerator {
- public:
-  nsMsgFilteredDBEnumerator(nsMsgDatabase* db, nsIMdbTable* table,
-                            bool reverse);
-  virtual ~nsMsgFilteredDBEnumerator();
-  nsresult InitSearchSession(
-      const nsTArray<RefPtr<nsIMsgSearchTerm>>& searchTerms,
-      nsIMsgFolder* folder);
-
- protected:
-  virtual nsresult PrefetchNext() override;
-
-  nsCOMPtr<nsIMsgSearchSession> m_searchSession;
-};
-
 namespace mozilla {
 namespace mailnews {
 class MsgDBReporter;
@@ -139,7 +89,7 @@ class nsMsgDatabase : public nsIMsgDatabase {
    * @param aCreate         Whether or not the file should be created.
    * @param aLeaveInvalidDB Set to true if you do not want the database to be
    *                        deleted if it is invalid.
-   * @exception NS_ERROR_FILE_TARGET_DOES_NOT_EXIST
+   * @exception NS_ERROR_FILE_NOT_FOUND
    *                        The file could not be created.
    * @exception NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE
    *                        The database is present (and was opened), but the
@@ -162,13 +112,11 @@ class nsMsgDatabase : public nsIMsgDatabase {
   virtual nsresult CreateMsgHdr(nsIMdbRow* hdrRow, nsMsgKey key,
                                 nsIMsgDBHdr** result);
   virtual nsresult GetThreadForMsgKey(nsMsgKey msgKey, nsIMsgThread** result);
-  virtual nsresult EnumerateMessagesWithFlag(nsISimpleEnumerator** result,
+  virtual nsresult EnumerateMessagesWithFlag(nsIMsgEnumerator** result,
                                              uint32_t* pFlag);
-  nsresult GetSearchResultsTable(const char* searchFolderUri,
+  nsresult GetSearchResultsTable(const nsACString& searchFolderUri,
                                  bool createIfMissing, nsIMdbTable** table);
 
-  // this might just be for debugging - we'll see.
-  nsresult ListAllThreads(nsTArray<nsMsgKey>* threadIds);
   //////////////////////////////////////////////////////////////////////////////
   // nsMsgDatabase methods:
   nsMsgDatabase();
@@ -257,14 +205,13 @@ class nsMsgDatabase : public nsIMsgDatabase {
 
 #ifdef DEBUG
   virtual nsresult DumpContents();
-  nsresult DumpThread(nsMsgKey threadId);
-  nsresult DumpMsgChildren(nsIMsgDBHdr* msgHdr);
 #endif
 
   friend class nsMsgHdr;     // use this to get access to cached tokens for hdr
                              // fields
   friend class nsMsgThread;  // use this to get access to cached tokens for hdr
                              // fields
+
   friend class nsMsgDBEnumerator;
   friend class nsMsgDBThreadEnumerator;
 
@@ -317,17 +264,17 @@ class nsMsgDatabase : public nsIMsgDatabase {
   virtual nsresult RemoveHeaderFromThread(nsMsgHdr* msgHdr);
   virtual nsresult AdjustExpungedBytesOnDelete(nsIMsgDBHdr* msgHdr);
 
-  nsCOMPtr<nsICollation> m_collationKeyGenerator;
+  mozilla::UniquePtr<mozilla::intl::Collator> m_collationKeyGenerator = nullptr;
   nsCOMPtr<nsIMimeConverter> m_mimeConverter;
   nsCOMPtr<nsIMsgRetentionSettings> m_retentionSettings;
   nsCOMPtr<nsIMsgDownloadSettings> m_downloadSettings;
 
-  nsresult PurgeMessagesOlderThan(uint32_t daysToKeepHdrs,
-                                  bool applyToFlaggedMessages,
-                                  nsIMutableArray* hdrsToDelete);
-  nsresult PurgeExcessMessages(uint32_t numHeadersToKeep,
-                               bool applyToFlaggedMessages,
-                               nsIMutableArray* hdrsToDelete);
+  nsresult FindMessagesOlderThan(uint32_t daysToKeepHdrs,
+                                 bool applyToFlaggedMessages,
+                                 nsTArray<RefPtr<nsIMsgDBHdr>>& hdrsToDelete);
+  nsresult FindExcessMessages(uint32_t numHeadersToKeep,
+                              bool applyToFlaggedMessages,
+                              nsTArray<RefPtr<nsIMsgDBHdr>>& hdrsToDelete);
 
   // mdb bookkeeping stuff
   virtual nsresult InitExistingDB();
@@ -368,7 +315,6 @@ class nsMsgDatabase : public nsIMsgDatabase {
   mdb_token m_flagsColumnToken;
   mdb_token m_priorityColumnToken;
   mdb_token m_labelColumnToken;
-  mdb_token m_statusOffsetColumnToken;
   mdb_token m_numLinesColumnToken;
   mdb_token m_ccListColumnToken;
   mdb_token m_bccListColumnToken;
@@ -409,7 +355,7 @@ class nsMsgDatabase : public nsIMsgDatabase {
   mdb_pos FindInsertIndexInSortedTable(nsIMdbTable* table, mdb_id idToInsert);
 
   void ClearCachedObjects(bool dbGoingAway);
-  void ClearEnumerators();
+  void InvalidateEnumerators();
   // all instantiated headers, but doesn't hold refs.
   PLDHashTable* m_headersInUse;
   static PLDHashNumber HashKey(const void* aKey);
@@ -443,9 +389,11 @@ class nsMsgDatabase : public nsIMsgDatabase {
   nsresult RemoveMsgRefsFromHash(nsIMsgDBHdr* msgHdr);
   nsresult InitRefHash();
 
-  // not-reference holding array of enumerators we've handed out.
-  // If a db goes away, it will clean up the outstanding enumerators.
-  nsTArray<nsMsgDBEnumerator*> m_enumerators;
+  // The enumerators add themselves to these lists.
+  // If a db goes away - via destruction or ForceClosed() - it needs to
+  // invalidate any outstanding enumerators.
+  nsTArray<nsMsgDBEnumerator*> m_msgEnumerators;
+  nsTArray<nsMsgDBThreadEnumerator*> m_threadEnumerators;
 
   // Memory reporter details
  public:
