@@ -8,9 +8,8 @@
  * GUI-side code for managing folder subscriptions.
  */
 
-var { Feed, FeedItem, FeedParser, FeedUtils } = ChromeUtils.import(
-  "resource:///modules/FeedUtils.jsm"
-);
+var { Feed } = ChromeUtils.import("resource:///modules/Feed.jsm");
+var { FeedUtils } = ChromeUtils.import("resource:///modules/FeedUtils.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
@@ -68,15 +67,11 @@ var FeedSubscriptions = {
     FeedUtils.CANCEL_REQUESTED = false;
 
     if (this.mMainWin) {
-      this.mMainWin.FeedFolderNotificationService = MailServices.mfn;
-      this.mMainWin.FeedFolderNotificationService.addListener(
-        this.FolderListener,
-        this.FOLDER_ACTIONS
-      );
+      MailServices.mfn.addListener(this.FolderListener, this.FOLDER_ACTIONS);
     }
   },
 
-  onClose() {
+  onDialogAccept() {
     let dismissDialog = true;
 
     // If we are in the middle of subscribing to a feed, inform the user that
@@ -104,11 +99,10 @@ var FeedSubscriptions = {
     if (dismissDialog) {
       FeedUtils.CANCEL_REQUESTED = this.mActionMode == this.kSubscribeMode;
       if (this.mMainWin) {
-        this.mMainWin.FeedFolderNotificationService.removeListener(
+        MailServices.mfn.removeListener(
           this.FolderListener,
           this.FOLDER_ACTIONS
         );
-        delete this.mMainWin.FeedFolderNotificationService;
       }
     }
 
@@ -174,7 +168,7 @@ var FeedSubscriptions = {
       return this._selection;
     },
     set selection(val) {
-      return (this._selection = val);
+      this._selection = val;
     },
 
     setTree(aTree) {
@@ -236,7 +230,7 @@ var FeedSubscriptions = {
       } else {
         let url = folder ? null : item.url;
         folder = folder || item.parentFolder;
-        properties = mainWin.getFolderProperties(folder, item.open);
+        properties = mainWin.FolderUtils.getFolderProperties(folder, item.open);
         properties += mainWin.FeedUtils.getFolderProperties(folder, url);
         if (
           this.selection.currentIndex == aRow &&
@@ -1164,7 +1158,7 @@ var FeedSubscriptions = {
     if (
       ("button" in aEvent &&
         (aEvent.button != 0 ||
-          aEvent.originalTarget.localName != "div" ||
+          aEvent.target.localName != "div" ||
           target.selectionStart != target.selectionEnd)) ||
       (aEvent.keyCode && aEvent.keyCode != aEvent.DOM_VK_RETURN)
     ) {
@@ -2164,7 +2158,6 @@ var FeedSubscriptions = {
       return;
     }
 
-    let validationSite = "http://validator.w3.org";
     let validationQuery = "http://validator.w3.org/feed/check.cgi?url=";
 
     if (this.mMainWin) {
@@ -2174,7 +2167,7 @@ var FeedSubscriptions = {
         let url = validationQuery + encodeURIComponent(feedLocation);
 
         this.mMainWin.focus();
-        this.mMainWin.openContentTab(url, "tab", "^" + validationSite);
+        this.mMainWin.openContentTab(url);
         FeedUtils.log.debug("checkValidation: query url - " + url);
       }
     }
