@@ -16,11 +16,10 @@
 #include "nsMsgMessageFlags.h"
 #include "nsMsgUtils.h"
 #include "nsIIOService.h"
+#include "nsITimer.h"
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "mozilla/Services.h"
-#include "nsIMutableArray.h"
-#include "nsArrayUtils.h"
 #include "mozilla/Logging.h"
 
 using namespace mozilla;
@@ -207,7 +206,7 @@ nsAutoSyncManager::nsAutoSyncManager() {
   mPaused = false;
 
   nsresult rv;
-  mIdleService = do_GetService("@mozilla.org/widget/idleservice;1", &rv);
+  mIdleService = do_GetService("@mozilla.org/widget/useridleservice;1", &rv);
   if (mIdleService) mIdleService->AddIdleObserver(this, kIdleTimeInSec);
 
   // Observe xpcom-shutdown event and app-idle changes
@@ -225,14 +224,13 @@ nsAutoSyncManager::~nsAutoSyncManager() {}
 
 void nsAutoSyncManager::InitTimer() {
   if (!mTimer) {
-    nsresult rv;
-    mTimer = do_CreateInstance(NS_TIMER_CONTRACTID, &rv);
-    NS_ASSERTION(NS_SUCCEEDED(rv),
-                 "failed to create timer in nsAutoSyncManager");
-
-    mTimer->InitWithNamedFuncCallback(
-        TimerCallback, (void*)this, kTimerIntervalInMs,
-        nsITimer::TYPE_REPEATING_SLACK, "nsAutoSyncManager::TimerCallback");
+    nsresult rv = NS_NewTimerWithFuncCallback(
+        getter_AddRefs(mTimer), TimerCallback, (void*)this, kTimerIntervalInMs,
+        nsITimer::TYPE_REPEATING_SLACK, "nsAutoSyncManager::TimerCallback",
+        nullptr);
+    if (NS_FAILED(rv)) {
+      NS_WARNING("Could not start nsAutoSyncManager timer");
+    }
   }
 }
 
@@ -839,10 +837,10 @@ nsresult nsAutoSyncManager::DownloadMessagesForOffline(
   // see HandleDownloadErrorFor for recovery policy
   if (!count) return NS_ERROR_NOT_AVAILABLE;
 
-  nsCOMPtr<nsIMutableArray> messagesToDownload;
+  nsTArray<RefPtr<nsIMsgDBHdr>> messagesToDownload;
   uint32_t totalSize = 0;
-  rv = aAutoSyncStateObj->GetNextGroupOfMessages(
-      mGroupSize, &totalSize, getter_AddRefs(messagesToDownload));
+  rv = aAutoSyncStateObj->GetNextGroupOfMessages(mGroupSize, &totalSize,
+                                                 messagesToDownload);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // there are pending messages but the cumulative size is zero:
@@ -858,9 +856,7 @@ nsresult nsAutoSyncManager::DownloadMessagesForOffline(
   // ensure that we don't exceed the given size limit for this particular group
   if (aSizeLimit && aSizeLimit < totalSize) return NS_ERROR_FAILURE;
 
-  uint32_t length;
-  rv = messagesToDownload->GetLength(&length);
-  if (NS_SUCCEEDED(rv) && length > 0) {
+  if (!messagesToDownload.IsEmpty()) {
     rv = aAutoSyncStateObj->DownloadMessagesForOffline(messagesToDownload);
 
     int32_t totalCount;
@@ -869,7 +865,8 @@ nsresult nsAutoSyncManager::DownloadMessagesForOffline(
     nsCOMPtr<nsIMsgFolder> folder;
     aAutoSyncStateObj->GetOwnerFolder(getter_AddRefs(folder));
     if (NS_SUCCEEDED(rv) && folder)
-      NOTIFY_LISTENERS(OnDownloadStarted, (folder, length, totalCount));
+      NOTIFY_LISTENERS(OnDownloadStarted,
+                       (folder, messagesToDownload.Length(), totalCount));
   }
 
   return rv;
