@@ -23,10 +23,10 @@
 // UI Thread proxy helper
 #include "nsIImapProtocolSink.h"
 
-#include "nsIIMAPHostSessionList.h"
+#include "nsIImapHostSessionList.h"
 #include "nsImapServerResponseParser.h"
 #include "nsImapFlagAndUidState.h"
-#include "nsIMAPNamespace.h"
+#include "nsImapNamespace.h"
 #include "nsTArray.h"
 #include "nsIWeakReferenceUtils.h"
 #include "nsMsgLineBuffer.h"  // we need this to use the nsMsgLineStreamBuffer helper class...
@@ -49,6 +49,9 @@
 #include "nsICacheEntryOpenCallback.h"
 #include "nsIProtocolProxyCallback.h"
 #include "nsIStringBundle.h"
+#include "nsHashPropertyBag.h"
+
+#include "mozilla/Monitor.h"
 
 class nsIMAPMessagePartID;
 class nsIPrefBranch;
@@ -173,7 +176,6 @@ class nsIMAPMailboxInfo {
  * blocks until the call is completed.
  */
 class nsImapProtocol : public nsIImapProtocol,
-                       public nsIRunnable,
                        public nsIInputStreamCallback,
                        public nsSupportsWeakReference,
                        public nsMsgProtocol,
@@ -198,9 +200,6 @@ class nsImapProtocol : public nsIImapProtocol,
                                         uint64_t sourceOffset,
                                         uint32_t length) override;
 
-  // nsIRunnable method
-  NS_IMETHOD Run() override;
-
   //////////////////////////////////////////////////////////////////////////////////
   // we support the nsIImapProtocol interface
   //////////////////////////////////////////////////////////////////////////////////
@@ -305,7 +304,8 @@ class nsImapProtocol : public nsIImapProtocol,
   void ProgressEventFunctionUsingName(const char* aMsgId);
   void ProgressEventFunctionUsingNameWithString(const char* aMsgName,
                                                 const char* aExtraInfo);
-  void PercentProgressUpdateEvent(const char16_t* message,
+  void PercentProgressUpdateEvent(nsACString const& fmtStringName,
+                                  nsAString const& mailbox,
                                   int64_t currentProgress, int64_t maxProgress);
   void ShowProgress();
 
@@ -358,7 +358,7 @@ class nsImapProtocol : public nsIImapProtocol,
   // Clears all rights for the current folder, for all users.
   void ClearAllFolderRights();
   void RefreshFolderACLView(const char* mailboxName,
-                            nsIMAPNamespace* nsForMailbox);
+                            nsImapNamespace* nsForMailbox);
 
   nsresult SetFolderAdminUrl(const char* mailboxName);
   void HandleMemoryFailure();
@@ -378,6 +378,9 @@ class nsImapProtocol : public nsIImapProtocol,
   const nsString& GetEmptyMimePartString() { return m_emptyMimePartString; }
   void SetCapabilityResponseOccurred();
 
+  // Start event loop. This is called on IMAP thread
+  bool RunImapThreadMainLoop();
+
  private:
   virtual ~nsImapProtocol();
   // the following flag is used to determine when a url is currently being run.
@@ -394,7 +397,6 @@ class nsImapProtocol : public nsIImapProtocol,
   nsCString m_hostName;
   nsCString m_userName;
   nsCString m_serverKey;
-  nsCString m_realHostName;
   char* m_dataOutputBuf;
   RefPtr<nsMsgLineStreamBuffer> m_inputStreamBuffer;
   nsCString m_trashFolderPath;
@@ -417,7 +419,6 @@ class nsImapProtocol : public nsIImapProtocol,
   // we're done with the conn.
 
   // ******* Thread support *******
-  nsCOMPtr<nsIThread> m_iThread;
   PRThread* m_thread;
   mozilla::ReentrantMonitor
       m_dataAvailableMonitor;  // used to notify the arrival of data from the
@@ -446,9 +447,11 @@ class nsImapProtocol : public nsIImapProtocol,
   nsCString m_connectionType;
 
   bool m_nextUrlReadyToRun;
+  bool m_idleResponseReadyToHandle;
   nsWeakPtr m_server;
 
   RefPtr<ImapMailFolderSinkProxy> m_imapMailFolderSink;
+  RefPtr<ImapMailFolderSinkProxy> m_imapMailFolderSinkSelected;
   RefPtr<ImapMessageSinkProxy> m_imapMessageSink;
   RefPtr<ImapServerSinkProxy> m_imapServerSink;
   RefPtr<ImapServerSinkProxy> m_imapServerSinkLatest;
@@ -465,7 +468,7 @@ class nsImapProtocol : public nsIImapProtocol,
   nsImapServerResponseParser m_parser;
   nsImapServerResponseParser& GetServerStateParser() { return m_parser; }
 
-  void HandleIdleResponses();
+  bool HandleIdleResponses();
   virtual bool ProcessCurrentURL();
   void EstablishServerConnection();
   virtual void ParseIMAPandCheckForNewMail(const char* commandString = nullptr,
@@ -518,9 +521,9 @@ class nsImapProtocol : public nsIImapProtocol,
   // output stream but it also informs the consumer that the data has been
   // written to the stream. aSuppressLogging --> set to true if you wish to
   // suppress logging for this particular command. this is useful for making
-  // sure we don't log authentication information like the user's password (which
-  // was encoded anyway), but still we shouldn't add that information to the
-  // log.
+  // sure we don't log authentication information like the user's password
+  // (which was encoded anyway), but still we shouldn't add that information to
+  // the log.
   nsresult SendData(const char* dataBuffer,
                     bool aSuppressLogging = false) override;
 
@@ -561,6 +564,7 @@ class nsImapProtocol : public nsIImapProtocol,
   // All of these methods actually issue protocol
   void Capability();  // query host for capabilities.
   void ID();          // send RFC 2971 app info to server
+  void EnableUTF8Accept();
   void EnableCondStore();
   void StartCompressDeflate();
   nsresult BeginCompressing();
@@ -577,7 +581,7 @@ class nsImapProtocol : public nsIImapProtocol,
   void ProcessSelectedStateURL();
   bool TryToLogon();
 
-  // Process Authenticated State Url used to be one giant if statement. I've
+  // ProcessAuthenticatedStateURL() used to be one giant if statement. I've
   // broken out a set of actions based on the imap action passed into the url.
   // The following functions are imap protocol handlers for each action. They
   // are called by ProcessAuthenticatedStateUrl.
@@ -676,7 +680,7 @@ class nsImapProtocol : public nsIImapProtocol,
   uint64_t mFolderLastModSeq;
   int32_t mFolderTotalMsgCount;
   uint32_t mFolderHighestUID;
-  uint32_t mFolderNumDeleted;
+  bool m_allowUTF8Accept;
 
   bool m_isGmailServer;
   nsTArray<nsCString> mCustomDBHeaders;
@@ -704,9 +708,9 @@ class nsImapProtocol : public nsIImapProtocol,
   RefPtr<nsMsgImapHdrXferInfo> m_hdrDownloadCache;
   nsCOMPtr<nsIImapHeaderInfo> m_curHdrInfo;
   // mapping between mailboxes and the corresponding folder flags
-  nsDataHashtable<nsCStringHashKey, int32_t> m_standardListMailboxes;
+  nsTHashMap<nsCStringHashKey, int32_t> m_standardListMailboxes;
   // mapping between special xlist mailboxes and the corresponding folder flags
-  nsDataHashtable<nsCStringHashKey, int32_t> m_specialXListMailboxes;
+  nsTHashMap<nsCStringHashKey, int32_t> m_specialXListMailboxes;
 
   nsCOMPtr<nsIImapHostSessionList> m_hostSessionList;
 
@@ -726,7 +730,6 @@ class nsImapProtocol : public nsIImapProtocol,
   nsCString m_lastProgressStringName;
   int32_t m_lastPercent;
   int64_t m_lastProgressTime;
-  nsCOMPtr<nsIStringBundle> m_bundle;
 
   bool m_notifySearchHit;
   bool m_needNoop;
@@ -782,11 +785,12 @@ class nsICacheEntry;
 class nsImapMockChannel : public nsIImapMockChannel,
                           public nsICacheEntryOpenCallback,
                           public nsITransportEventSink,
-                          public nsSupportsWeakReference {
+                          public nsSupportsWeakReference,
+                          public nsHashPropertyBag {
  public:
   friend class nsImapProtocol;
 
-  NS_DECL_THREADSAFE_ISUPPORTS
+  NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSIIMAPMOCKCHANNEL
   NS_DECL_NSICHANNEL
   NS_DECL_NSIREQUEST
@@ -822,6 +826,11 @@ class nsImapMockChannel : public nsIImapMockChannel,
   bool mTryingToReadPart;
   int64_t mContentLength;
 
+  mozilla::Monitor mSuspendedMonitor;
+  bool mSuspended;
+
+  nsresult ResumeAndNotifyOne();
+
   // cache related helper methods
   nsresult OpenCacheEntry();  // makes a request to the cache service for a
                               // cache entry for a url
@@ -836,6 +845,8 @@ class nsImapMockChannel : public nsIImapMockChannel,
   // we end up daisy chaining multiple nsIStreamListeners into the load process.
   nsresult SetupPartExtractorListener(nsIImapUrl* aUrl,
                                       nsIStreamListener* aConsumer);
+
+  uint32_t mContentDisposition;
 };
 
 #endif  // nsImapProtocol_h___
