@@ -7,21 +7,21 @@
 #define _nsIMAPServerResponseParser_H_
 
 #include "mozilla/Attributes.h"
-#include "nsIIMAPHostSessionList.h"
+#include "nsIImapHostSessionList.h"
 #include "nsImapSearchResults.h"
 #include "nsString.h"
 #include "MailNewsTypes.h"
 #include "nsTArray.h"
 #include "nsImapUtils.h"
 
-class nsIMAPBodyShell;
+class nsImapBodyShell;
 class nsIMAPBodypart;
 class nsImapSearchResultIterator;
 class nsIImapFlagAndUidState;
 
-#include "nsIMAPGenericParser.h"
+#include "nsImapGenericParser.h"
 
-class nsImapServerResponseParser : public nsIMAPGenericParser {
+class nsImapServerResponseParser : public nsImapGenericParser {
  public:
   explicit nsImapServerResponseParser(nsImapProtocol& imapConnection);
   virtual ~nsImapServerResponseParser();
@@ -40,6 +40,7 @@ class nsImapServerResponseParser : public nsIMAPGenericParser {
   virtual void InitializeState();
   bool CommandFailed();
   void SetCommandFailed(bool failed);
+  bool UntaggedResponse();
 
   enum eIMAPstate { kNonAuthenticated, kAuthenticated, kFolderSelected };
 
@@ -122,16 +123,19 @@ class nsImapServerResponseParser : public nsIMAPGenericParser {
   // Interrupt a Fetch, without really Interrupting (through netlib)
   bool GetLastFetchChunkReceived();
   void ClearLastFetchChunkReceived();
+  int32_t GetNumBytesFetched();
+  void ClearNumBytesFetched();
   virtual uint16_t SupportsUserFlags() { return fSupportsUserDefinedFlags; }
   virtual uint16_t SettablePermanentFlags() { return fSettablePermanentFlags; }
   void SetFlagState(nsIImapFlagAndUidState* state);
   bool GetDownloadingHeaders();
   bool GetFillingInShell();
-  void UseCachedShell(nsIMAPBodyShell* cachedShell);
+  void UseCachedShell(nsImapBodyShell* cachedShell);
   void SetHostSessionList(nsIImapHostSessionList* aHostSession);
-  char* fAuthChallenge;  // the challenge returned by the server in
-                         // response to authenticate using CRAM-MD5 or NTLM
-  bool fCondStoreEnabled;
+  char* fAuthChallenge;    // the challenge returned by the server in
+                           // response to authenticate using CRAM-MD5 or NTLM
+  bool fCondStoreEnabled;  // Not used it seems
+  bool fUtf8AcceptEnabled;
   bool fUseModSeq;  // can use mod seq for currently selected folder
   uint64_t fHighestModSeq;
 
@@ -191,7 +195,7 @@ class nsImapServerResponseParser : public nsIMAPGenericParser {
                                       const char* currentCommand);
   virtual void PostProcessEndOfLine();
 
-  // Overridden from the nsIMAPGenericParser, to retrieve the next line
+  // Overridden from the nsImapGenericParser, to retrieve the next line
   // from the open socket.
   virtual bool GetNextLineForParser(char** nextLine) override;
   // overridden to do logging
@@ -199,6 +203,7 @@ class nsImapServerResponseParser : public nsIMAPGenericParser {
 
  private:
   bool fCurrentCommandFailed;
+  bool fUntaggedResponse;
   bool fReportingErrors;
 
   bool fCurrentFolderReadOnly;
@@ -273,7 +278,7 @@ class nsImapServerResponseParser : public nsIMAPGenericParser {
   bool fNextChunkStartsWithNewline;
 
   // points to the current body shell, if any
-  RefPtr<nsIMAPBodyShell> m_shell;
+  RefPtr<nsImapBodyShell> m_shell;
 
   // The connection object
   nsImapProtocol& fServerConnection;
