@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -30,8 +30,8 @@ nsresult nsImapMoveCopyMsgTxn::Init(nsIMsgFolder* srcFolder,
   m_isMove = isMove;
   m_srcFolder = do_GetWeakReference(srcFolder);
   m_dstFolder = do_GetWeakReference(dstFolder);
-  m_srcKeyArray = *srcKeyArray;
-  m_dupKeyArray = *srcKeyArray;
+  m_srcKeyArray = srcKeyArray->Clone();
+  m_dupKeyArray = srcKeyArray->Clone();
   nsCString uri;
   nsresult rv = srcFolder->GetURI(uri);
   nsCString protocolType(uri);
@@ -48,7 +48,7 @@ nsresult nsImapMoveCopyMsgTxn::Init(nsIMsgFolder* srcFolder,
     rv = srcDB->GetMsgHdrForKey(m_srcKeyArray[i], getter_AddRefs(srcHdr));
     if (NS_SUCCEEDED(rv)) {
       // ** jt -- only do this for mailbox protocol
-      if (MsgLowerCaseEqualsLiteral(protocolType, "mailbox")) {
+      if (protocolType.LowerCaseEqualsLiteral("mailbox")) {
         m_srcIsPop3 = true;
         uint32_t msgSize;
         rv = srcHdr->GetMessageSize(&msgSize);
@@ -116,10 +116,10 @@ nsImapMoveCopyMsgTxn::UndoTransaction(void) {
 
         if (deletedMsgs)
           rv = imapService->SubtractMessageFlags(
-              srcFolder, this, nullptr, m_srcMsgIdString, kImapMsgDeletedFlag,
+              srcFolder, this, m_srcMsgIdString, kImapMsgDeletedFlag,
               m_idsAreUids);
         else
-          rv = imapService->AddMessageFlags(srcFolder, srcListener, nullptr,
+          rv = imapService->AddMessageFlags(srcFolder, srcListener,
                                             m_srcMsgIdString,
                                             kImapMsgDeletedFlag, m_idsAreUids);
         if (NS_FAILED(rv)) return rv;
@@ -144,9 +144,8 @@ nsImapMoveCopyMsgTxn::UndoTransaction(void) {
     rv =
         imapService->LiteSelectFolder(dstFolder, dstListener, nullptr, nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = imapService->AddMessageFlags(dstFolder, dstListener, nullptr,
-                                      m_dstMsgIdString, kImapMsgDeletedFlag,
-                                      m_idsAreUids);
+    rv = imapService->AddMessageFlags(dstFolder, dstListener, m_dstMsgIdString,
+                                      kImapMsgDeletedFlag, m_idsAreUids);
   }
   return rv;
 }
@@ -188,10 +187,10 @@ nsImapMoveCopyMsgTxn::RedoTransaction(void) {
       NS_ENSURE_SUCCESS(rv, rv);
       if (deletedMsgs) {
         rv = imapService->SubtractMessageFlags(
-            srcFolder, srcListener, nullptr, m_srcMsgIdString,
-            kImapMsgDeletedFlag, m_idsAreUids);
+            srcFolder, srcListener, m_srcMsgIdString, kImapMsgDeletedFlag,
+            m_idsAreUids);
       } else {
-        rv = imapService->AddMessageFlags(srcFolder, srcListener, nullptr,
+        rv = imapService->AddMessageFlags(srcFolder, srcListener,
                                           m_srcMsgIdString, kImapMsgDeletedFlag,
                                           m_idsAreUids);
       }
@@ -210,7 +209,7 @@ nsImapMoveCopyMsgTxn::RedoTransaction(void) {
     rv =
         imapService->LiteSelectFolder(dstFolder, dstListener, nullptr, nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = imapService->SubtractMessageFlags(dstFolder, dstListener, nullptr,
+    rv = imapService->SubtractMessageFlags(dstFolder, dstListener,
                                            m_dstMsgIdString,
                                            kImapMsgDeletedFlag, m_idsAreUids);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -235,7 +234,7 @@ nsresult nsImapMoveCopyMsgTxn::SetCopyResponseUid(const char* aMsgIdString) {
 }
 
 nsresult nsImapMoveCopyMsgTxn::GetSrcKeyArray(nsTArray<nsMsgKey>& srcKeyArray) {
-  srcKeyArray = m_srcKeyArray;
+  srcKeyArray = m_srcKeyArray.Clone();
   return NS_OK;
 }
 
@@ -293,8 +292,7 @@ nsresult nsImapMoveCopyMsgTxn::RedoMailboxDelete() {
     if (NS_FAILED(rv) || !srcFolder) return rv;
     rv = srcFolder->GetMsgDatabase(getter_AddRefs(srcDB));
     if (NS_SUCCEEDED(rv)) {
-      srcDB->DeleteMessages(m_srcKeyArray.Length(), m_srcKeyArray.Elements(),
-                            nullptr);
+      srcDB->DeleteMessages(m_srcKeyArray, nullptr);
       srcDB->SetSummaryValid(true);
     }
     return NS_OK;  // always return NS_OK
@@ -364,7 +362,7 @@ NS_IMETHODIMP nsImapMoveCopyMsgTxn::OnStopRunningUrl(nsIURI* aUrl,
         rv = imapService->LiteSelectFolder(dstFolder, dstListener, nullptr,
                                            nullptr);
         NS_ENSURE_SUCCESS(rv, rv);
-        rv = imapService->AddMessageFlags(dstFolder, dstListener, nullptr,
+        rv = imapService->AddMessageFlags(dstFolder, dstListener,
                                           m_dstMsgIdString, kImapMsgDeletedFlag,
                                           m_idsAreUids);
       }
@@ -391,8 +389,7 @@ NS_IMETHODIMP nsImapMoveCopyMsgTxn::OnStopRunningUrl(nsIURI* aUrl,
       }
       if (dstKeys.Length()) {
         nsAutoCString uids;
-        nsImapMailFolder::AllocateUidStringFromKeys(dstKeys.Elements(),
-                                                    dstKeys.Length(), uids);
+        nsImapMailFolder::AllocateUidStringFromKeys(dstKeys, uids);
         rv = imapService->OnlineMessageCopy(dstFolder, uids, srcFolder, true,
                                             true, nullptr, nullptr, nullptr,
                                             nullptr);
@@ -560,10 +557,10 @@ NS_IMETHODIMP nsImapOfflineTxn::RedoTransaction(void) {
             dstFolder->GetURI(folderURI);
 
             if (m_opType == nsIMsgOfflineImapOperation::kMsgMoved)
-              op->SetDestinationFolderURI(folderURI.get());  // offline move
+              op->SetDestinationFolderURI(folderURI);  // offline move
             if (m_opType == nsIMsgOfflineImapOperation::kMsgCopy) {
               op->SetOperation(nsIMsgOfflineImapOperation::kMsgMoved);
-              op->AddMessageCopyOperation(folderURI.get());  // offline copy
+              op->AddMessageCopyOperation(folderURI);  // offline copy
             }
             dstFolder->SummaryChanged();
           }
@@ -589,7 +586,7 @@ NS_IMETHODIMP nsImapOfflineTxn::RedoTransaction(void) {
         if (NS_SUCCEEDED(rv) && op) {
           nsCString folderURI;
           srcFolder->GetURI(folderURI);
-          op->SetSourceFolderURI(folderURI.get());
+          op->SetSourceFolderURI(folderURI);
         }
       }
       dstFolder->SummaryChanged();
