@@ -46,6 +46,7 @@ class nsImapUrl : public nsIImapUrl,
                                            char onlineDelimiter,
                                            char** resultingCanonicalPath);
   static nsresult EscapeSlashes(const char* sourcePath, char** resultPath);
+  static nsresult UnescapeSlashes(nsACString& path);
   static nsresult UnescapeSlashes(char* path);
   static char* ReplaceCharsInCopiedString(const char* stringToCopy,
                                           char oldChar, char newChar);
@@ -125,8 +126,8 @@ class nsImapUrl : public nsIImapUrl,
   bool m_addDummyEnvelope;
   bool m_canonicalLineEnding;  // CRLF
 
-  nsCString mURI;              // the RDF URI associated with this url.
-  nsCString mCharsetOverride;  // used by nsIMsgI18NUrl...
+  nsCString mURI;           // the RDF URI associated with this url.
+  bool mAutodetectCharset;  // used by nsIMsgI18NUrl...
   mozilla::Mutex mLock;
 };
 
