@@ -1,4 +1,7 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 /*
  * Test to ensure that changes made while offline are played back when we
  * go back online.
@@ -52,18 +55,12 @@ var tests = [
     let msgHdr2 = IMAPPump.inbox.msgDatabase.getMsgHdrForMessageID(
       gSynthMessage2.messageId
     );
-    let headers1 = Cc["@mozilla.org/array;1"].createInstance(
-      Ci.nsIMutableArray
-    );
-    let headers2 = Cc["@mozilla.org/array;1"].createInstance(
-      Ci.nsIMutableArray
-    );
-    headers1.appendElement(msgHdr1);
-    headers2.appendElement(msgHdr2);
+    let headers1 = [msgHdr1];
+    let headers2 = [msgHdr2];
     msgHdr1.folder.markMessagesFlagged(headers1, true);
     msgHdr2.folder.markMessagesFlagged(headers2, true);
     let promiseCopyListener1 = new PromiseTestUtils.PromiseCopyListener();
-    MailServices.copy.CopyMessages(
+    MailServices.copy.copyMessages(
       IMAPPump.inbox,
       headers1,
       gSecondFolder,
@@ -73,7 +70,7 @@ var tests = [
       true
     );
     let promiseCopyListener2 = new PromiseTestUtils.PromiseCopyListener();
-    MailServices.copy.CopyMessages(
+    MailServices.copy.copyMessages(
       IMAPPump.inbox,
       headers2,
       gThirdFolder,
@@ -84,7 +81,7 @@ var tests = [
     );
     var file = do_get_file("../../../data/bugmail10");
     let promiseCopyListener3 = new PromiseTestUtils.PromiseCopyListener();
-    MailServices.copy.CopyFileMessage(
+    MailServices.copy.copyFileMessage(
       file,
       IMAPPump.inbox,
       null,
