@@ -6,12 +6,13 @@
 
 "use strict";
 
-/* import-globals-from ../../extensions/newsblog/content/feed-subscriptions.js */
+/* import-globals-from ../../extensions/newsblog/feed-subscriptions.js */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
+var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
 
 var gImportType = null;
 var gImportMsgsBundle;
@@ -25,7 +26,8 @@ var gSelectedModuleName = null;
 var gAddInterface = null;
 var gNewFeedAcctCreated = false;
 
-var nsISupportsString = Ci.nsISupportsString;
+window.addEventListener("DOMContentLoaded", OnLoadImportDialog);
+window.addEventListener("unload", OnUnloadImportDialog);
 
 function OnLoadImportDialog() {
   gImportMsgsBundle = document.getElementById("bundle_importMsgs");
@@ -44,13 +46,13 @@ function OnLoadImportDialog() {
   gProgressInfo.localFolderExists = false;
 
   gSuccessStr = Cc["@mozilla.org/supports-string;1"].createInstance(
-    nsISupportsString
+    Ci.nsISupportsString
   );
   gErrorStr = Cc["@mozilla.org/supports-string;1"].createInstance(
-    nsISupportsString
+    Ci.nsISupportsString
   );
   gInputStr = Cc["@mozilla.org/supports-string;1"].createInstance(
-    nsISupportsString
+    Ci.nsISupportsString
   );
 
   // look in arguments[0] for parameters
@@ -77,6 +79,21 @@ function OnLoadImportDialog() {
   document.getElementById("importFields").focus();
 }
 
+/**
+ * After importing, need to restart so that imported address books and mail
+ * accounts can show up.
+ */
+function OnUnloadImportDialog() {
+  let nextButton = document.getElementById("forward");
+  if (
+    gImportType == "settings" &&
+    !gErrorStr.data &&
+    nextButton.label == nextButton.getAttribute("finishedval")
+  ) {
+    MailUtils.restartApplication();
+  }
+}
+
 function SetUpImportType() {
   // set dialog title
   document.getElementById("importFields").value = gImportType;
@@ -89,12 +106,15 @@ function SetUpImportType() {
     }
   }
 
-  let descriptionDeck = document.getElementById("selectDescriptionDeck");
-  descriptionDeck.selectedIndex = 0;
+  let fileLabel = document.getElementById("fileLabel");
+  let accountLabel = document.getElementById("accountLabel");
   if (gImportType == "feeds") {
-    descriptionDeck.selectedIndex = 1;
+    accountLabel.hidden = false;
+    fileLabel.hidden = true;
     ListFeedAccounts();
   } else {
+    accountLabel.hidden = true;
+    fileLabel.hidden = false;
     ListModules();
   }
 }
@@ -107,7 +127,7 @@ function SetDivText(id, text) {
       var textNode = document.createTextNode(text);
       div.appendChild(textNode);
     } else if (div.childNodes.length == 1) {
-      div.childNodes[0].nodeValue = text;
+      div.firstChild.nodeValue = text;
     }
   }
 }
@@ -122,9 +142,26 @@ function CheckIfLocalFolderExists() {
   }
 }
 
+function showWizardBox(index) {
+  let stateBox = document.getElementById("stateBox");
+  for (let i = 0; i < stateBox.children.length; i++) {
+    stateBox.children[i].hidden = i != index;
+  }
+}
+
+function getWizardBoxIndex() {
+  let selectedIndex = 0;
+  for (let element of document.getElementById("stateBox").children) {
+    if (!element.hidden) {
+      return selectedIndex;
+    }
+    selectedIndex++;
+  }
+  return selectedIndex - 1;
+}
+
 async function ImportDialogOKButton() {
   var listbox = document.getElementById("moduleList");
-  var deck = document.getElementById("stateDeck");
   var header = document.getElementById("header");
   var progressMeterEl = document.getElementById("progressMeter");
   progressMeterEl.value = 0;
@@ -171,7 +208,7 @@ async function ImportDialogOKButton() {
       let error = {};
       switch (gImportType) {
         case "mail":
-          if (ImportMail(module, gSuccessStr, gErrorStr)) {
+          if (await ImportMail(module, gSuccessStr, gErrorStr)) {
             // We think it was a success, either, we need to
             // wait for the import to finish
             // or we are done!
@@ -189,7 +226,7 @@ async function ImportDialogOKButton() {
             progressStatusEl.setAttribute("label", "");
             progressTitleEl.setAttribute("label", meterText);
 
-            deck.selectedIndex = 2;
+            showWizardBox(2);
             gProgressInfo.progressWindow = window;
             gProgressInfo.intervalState = setInterval(
               ContinueImportCallback,
@@ -216,7 +253,7 @@ async function ImportDialogOKButton() {
             progressTitleEl.setAttribute("label", meterText);
             progressMeterEl.removeAttribute("value");
 
-            deck.selectedIndex = 2;
+            showWizardBox(2);
             return true;
           }
 
@@ -246,7 +283,7 @@ async function ImportDialogOKButton() {
             progressStatusEl.setAttribute("label", "");
             progressTitleEl.setAttribute("label", meterText);
 
-            deck.selectedIndex = 2;
+            showWizardBox(2);
             gProgressInfo.progressWindow = window;
             gProgressInfo.intervalState = setInterval(
               ContinueImportCallback,
@@ -473,7 +510,6 @@ function ListFeedAccounts() {
 function ContinueImport(info) {
   var isMail = info.importType == "mail";
   var clear = true;
-  var deck;
   var pcnt;
 
   if (info.importInterface) {
@@ -481,8 +517,7 @@ function ContinueImport(info) {
       info.importSuccess = false;
       clearInterval(info.intervalState);
       if (info.progressWindow != null) {
-        deck = document.getElementById("stateDeck");
-        deck.selectedIndex = 3;
+        showWizardBox(3);
         info.progressWindow = null;
       }
 
@@ -510,8 +545,7 @@ function ContinueImport(info) {
       clearInterval(info.intervalState);
       info.importSuccess = true;
       if (info.progressWindow) {
-        deck = document.getElementById("stateDeck");
-        deck.selectedIndex = 3;
+        showWizardBox(3);
         info.progressWindow = null;
       }
 
@@ -527,7 +561,6 @@ function ContinueImport(info) {
 function ShowResults(doesWantProgress, result) {
   if (result) {
     if (doesWantProgress) {
-      let deck = document.getElementById("stateDeck");
       let header = document.getElementById("header");
       let progressStatusEl = document.getElementById("progressStatus");
       let progressTitleEl = document.getElementById("progressTitle");
@@ -541,7 +574,7 @@ function ShowResults(doesWantProgress, result) {
       progressStatusEl.setAttribute("label", "");
       progressTitleEl.setAttribute("label", meterText);
 
-      deck.selectedIndex = 2;
+      showWizardBox(2);
       gProgressInfo.progressWindow = window;
       gProgressInfo.intervalState = setInterval(ContinueImportCallback, 100);
     } else {
@@ -585,8 +618,7 @@ function ShowImportResultsRaw(title, results, good) {
   header.setAttribute("description", title);
   dump("*** results = " + results + "\n");
   attachStrings("results", results);
-  var deck = document.getElementById("stateDeck");
-  deck.selectedIndex = 3;
+  showWizardBox(3);
   var nextButton = document.getElementById("forward");
   nextButton.label = nextButton.getAttribute("finishedval");
   nextButton.removeAttribute("disabled");
@@ -844,6 +876,18 @@ async function ImportAddress(module, success, error) {
             addressbookBundle.getString("VCFFiles"),
             "*.vcf"
           );
+        } else if (
+          gSelectedModuleName ==
+          document
+            .getElementById("bundle_morkImportMsgs")
+            .getString("morkImportName")
+        ) {
+          filePicker.appendFilter(
+            document
+              .getElementById("bundle_morkImportMsgs")
+              .getString("MABFiles"),
+            "*.mab"
+          );
         } else {
           filePicker.appendFilter(
             addressbookBundle.getString("LDIFFiles"),
@@ -894,7 +938,7 @@ async function ImportAddress(module, success, error) {
     let result = {};
     result.ok = false;
     window.openDialog(
-      "chrome://messenger/content/fieldMapImport.xul",
+      "chrome://messenger/content/fieldMapImport.xhtml",
       "",
       "chrome,modal,titlebar",
       {
@@ -1031,9 +1075,8 @@ function SwitchType(newType) {
 }
 
 function next() {
-  var deck = document.getElementById("stateDeck");
-  switch (deck.selectedIndex) {
-    case "0":
+  switch (getWizardBoxIndex()) {
+    case 0:
       let backButton = document.getElementById("back");
       backButton.removeAttribute("disabled");
       let radioGroup = document.getElementById("importFields");
@@ -1043,14 +1086,14 @@ function next() {
         let SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";
         if (Services.appinfo.ID == SEAMONKEY_ID) {
           window.openDialog(
-            "chrome://communicator/content/migration/migration.xul",
+            "chrome://communicator/content/migration/migration.xhtml",
             "",
             "chrome,dialog,modal,centerscreen"
           );
         } else {
           // Running as Thunderbird or its clone.
           window.openDialog(
-            "chrome://messenger/content/migration/migration.xul",
+            "chrome://messenger/content/migration/migration.xhtml",
             "",
             "chrome,dialog,modal,centerscreen",
             null,
@@ -1064,17 +1107,24 @@ function next() {
         }
       } else {
         SwitchType(radioGroup.value);
-        deck.selectedIndex = 1;
-        document.getElementById("modulesFound").selectedIndex =
-          document.getElementById("moduleList").itemCount > 0 ? 0 : 1;
+        showWizardBox(1);
+        let moduleBox = document.getElementById("moduleBox");
+        let noModuleLabel = document.getElementById("noModuleLabel");
+        if (document.getElementById("moduleList").itemCount > 0) {
+          moduleBox.hidden = false;
+          noModuleLabel.hidden = true;
+        } else {
+          moduleBox.hidden = true;
+          noModuleLabel.hidden = false;
+        }
         SelectFirstItem();
         enableAdvance();
       }
       break;
-    case "1":
+    case 1:
       ImportDialogOKButton();
       break;
-    case "3":
+    case 3:
       close();
       break;
   }
@@ -1099,17 +1149,16 @@ function enableAdvance() {
 }
 
 function back() {
-  var deck = document.getElementById("stateDeck");
   var backButton = document.getElementById("back");
   var nextButton = document.getElementById("forward");
-  switch (deck.selectedIndex) {
-    case "1":
+  switch (getWizardBoxIndex()) {
+    case 1:
       backButton.setAttribute("disabled", "true");
       nextButton.label = nextButton.getAttribute("nextval");
       nextButton.removeAttribute("disabled");
-      deck.selectedIndex = 0;
+      showWizardBox(0);
       break;
-    case "3":
+    case 3:
       // Clear out the results box.
       let results = document.getElementById("results");
       while (results.hasChildNodes()) {
@@ -1129,7 +1178,7 @@ function back() {
       }
 
       // Now go back to the second page.
-      deck.selectedIndex = 1;
+      showWizardBox(1);
       break;
   }
 }
