@@ -15,7 +15,6 @@
 #include "nsIMsgFolder.h"
 #include "nsIMsgHdr.h"
 #include "nsIMsgPluggableStore.h"
-#include "nsIMutableArray.h"
 #include "nsNetUtil.h"
 #include "nsMsgUtils.h"
 #include "mozilla/Services.h"
@@ -159,7 +158,7 @@ NS_IMETHODIMP nsAppleMailImportMail::GetDefaultLocation(nsIFile** aLocation,
   nsCOMPtr<nsIFile> mailFolder(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
   if (mailFolder) {
     nsresult rv =
-        mailFolder->InitWithNativePath(NS_LITERAL_CSTRING(DEFAULT_MAIL_FOLDER));
+        mailFolder->InitWithNativePath(nsLiteralCString(DEFAULT_MAIL_FOLDER));
     if (NS_SUCCEEDED(rv)) {
       *aFound = true;
       *aUserVerify = false;
@@ -172,13 +171,14 @@ NS_IMETHODIMP nsAppleMailImportMail::GetDefaultLocation(nsIFile** aLocation,
 
 // this is the method that initiates all searching for mailboxes.
 // it will assume that it has a directory like ~/Library/Mail/
-NS_IMETHODIMP nsAppleMailImportMail::FindMailboxes(nsIFile* aMailboxFile,
-                                                   nsIArray** aResult) {
+NS_IMETHODIMP nsAppleMailImportMail::FindMailboxes(
+    nsIFile* aMailboxFile,
+    nsTArray<RefPtr<nsIImportMailboxDescriptor>>& boxes) {
   NS_ENSURE_ARG_POINTER(aMailboxFile);
-  NS_ENSURE_ARG_POINTER(aResult);
 
   IMPORT_LOG0("FindMailboxes for Apple mail invoked");
 
+  boxes.Clear();
   bool exists = false;
   nsresult rv = aMailboxFile->Exists(&exists);
   if (NS_FAILED(rv) || !exists) return NS_ERROR_FAILURE;
@@ -187,14 +187,10 @@ NS_IMETHODIMP nsAppleMailImportMail::FindMailboxes(nsIFile* aMailboxFile,
       do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIMutableArray> resultsArray(
-      do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-  if (!resultsArray) return NS_ERROR_OUT_OF_MEMORY;
-
   mCurDepth = 1;
 
   // 1. look for accounts with mailboxes
-  FindAccountMailDirs(aMailboxFile, resultsArray, importService);
+  FindAccountMailDirs(aMailboxFile, boxes, importService);
   mCurDepth--;
 
   if (NS_SUCCEEDED(rv)) {
@@ -205,26 +201,23 @@ NS_IMETHODIMP nsAppleMailImportMail::FindMailboxes(nsIFile* aMailboxFile,
         do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv)) {
       mailboxesDir->InitWithFile(aMailboxFile);
-      rv = mailboxesDir->Append(NS_LITERAL_STRING("Mailboxes"));
+      rv = mailboxesDir->Append(u"Mailboxes"_ns);
       if (NS_SUCCEEDED(rv)) {
         IMPORT_LOG0("Looking for global Apple mailboxes");
 
         mCurDepth++;
-        rv = FindMboxDirs(mailboxesDir, resultsArray, importService);
+        rv = FindMboxDirs(mailboxesDir, boxes, importService);
         mCurDepth--;
       }
     }
   }
-
-  if (NS_SUCCEEDED(rv) && resultsArray) resultsArray.forget(aResult);
-
   return rv;
 }
 
 // operates on the Mail/ directory root, trying to find accounts (which are
 // folders named something like "POP-hwaara@gmail.com") and add their .mbox dirs
 void nsAppleMailImportMail::FindAccountMailDirs(
-    nsIFile* aRoot, nsIMutableArray* aMailboxDescs,
+    nsIFile* aRoot, nsTArray<RefPtr<nsIImportMailboxDescriptor>>& aMailboxDescs,
     nsIImportService* aImportService) {
   nsCOMPtr<nsIDirectoryEnumerator> directoryEnumerator;
   nsresult rv = aRoot->GetDirectoryEntries(getter_AddRefs(directoryEnumerator));
@@ -249,11 +242,11 @@ void nsAppleMailImportMail::FindAccountMailDirs(
       currentEntry->GetLeafName(folderName);
       bool isAccountFolder = false;
 
-      if (StringBeginsWith(folderName, NS_LITERAL_STRING("POP-"))) {
+      if (StringBeginsWith(folderName, u"POP-"_ns)) {
         // cut off "POP-" prefix so we get a nice folder name
         folderName.Cut(0, 4);
         isAccountFolder = true;
-      } else if (StringBeginsWith(folderName, NS_LITERAL_STRING("IMAP-"))) {
+      } else if (StringBeginsWith(folderName, u"IMAP-"_ns)) {
         // cut off "IMAP-" prefix so we get a nice folder name
         folderName.Cut(0, 5);
         isAccountFolder = true;
@@ -266,8 +259,8 @@ void nsAppleMailImportMail::FindAccountMailDirs(
         // create a mailbox for this account, so we get a parent for "Inbox",
         // "Sent Messages", etc.
         nsCOMPtr<nsIImportMailboxDescriptor> desc;
-        nsresult rv =
-            aImportService->CreateNewMailboxDescriptor(getter_AddRefs(desc));
+        rv = aImportService->CreateNewMailboxDescriptor(getter_AddRefs(desc));
+        if (NS_FAILED(rv)) continue;
         desc->SetSize(1);
         desc->SetDepth(mCurDepth);
         desc->SetDisplayName(folderName.get());
@@ -275,12 +268,12 @@ void nsAppleMailImportMail::FindAccountMailDirs(
 
         nsCOMPtr<nsIFile> mailboxDescFile;
         rv = desc->GetFile(getter_AddRefs(mailboxDescFile));
-        if (!mailboxDescFile) continue;
+        if (NS_FAILED(rv) || !mailboxDescFile) continue;
 
         mailboxDescFile->InitWithFile(currentEntry);
 
         // add this mailbox descriptor to the list
-        aMailboxDescs->AppendElement(desc);
+        aMailboxDescs.AppendElement(desc);
 
         // now add all the children mailboxes
         mCurDepth++;
@@ -292,20 +285,23 @@ void nsAppleMailImportMail::FindAccountMailDirs(
 }
 
 // adds the specified file as a mailboxdescriptor to the array
-nsresult nsAppleMailImportMail::AddMboxDir(nsIFile* aFolder,
-                                           nsIMutableArray* aMailboxDescs,
-                                           nsIImportService* aImportService) {
+nsresult nsAppleMailImportMail::AddMboxDir(
+    nsIFile* aFolder,
+    nsTArray<RefPtr<nsIImportMailboxDescriptor>>& aMailboxDescs,
+    nsIImportService* aImportService) {
   nsAutoString folderName;
   aFolder->GetLeafName(folderName);
 
   // cut off the suffix, if any, or prefix if this is an account folder.
-  if (StringEndsWith(folderName, NS_LITERAL_STRING(POP_MBOX_SUFFIX)))
+  if (StringEndsWith(folderName,
+                     NS_LITERAL_STRING_FROM_CSTRING(POP_MBOX_SUFFIX)))
     folderName.SetLength(folderName.Length() - 5);
-  else if (StringEndsWith(folderName, NS_LITERAL_STRING(IMAP_MBOX_SUFFIX)))
+  else if (StringEndsWith(folderName,
+                          NS_LITERAL_STRING_FROM_CSTRING(IMAP_MBOX_SUFFIX)))
     folderName.SetLength(folderName.Length() - 9);
-  else if (StringBeginsWith(folderName, NS_LITERAL_STRING("POP-")))
+  else if (StringBeginsWith(folderName, u"POP-"_ns))
     folderName.Cut(4, folderName.Length());
-  else if (StringBeginsWith(folderName, NS_LITERAL_STRING("IMAP-")))
+  else if (StringBeginsWith(folderName, u"IMAP-"_ns))
     folderName.Cut(5, folderName.Length());
 
   nsCOMPtr<nsIImportMailboxDescriptor> desc;
@@ -318,7 +314,7 @@ nsresult nsAppleMailImportMail::AddMboxDir(nsIFile* aFolder,
       // move to the .mbox's Messages folder
       nsCOMPtr<nsIFile> messagesFolder;
       aFolder->Clone(getter_AddRefs(messagesFolder));
-      nsresult rv = messagesFolder->Append(NS_LITERAL_STRING("Messages"));
+      nsresult rv = messagesFolder->Append(u"Messages"_ns);
       NS_ENSURE_SUCCESS(rv, rv);
 
       // count the number of messages in this folder. it sucks that we have to
@@ -358,7 +354,7 @@ nsresult nsAppleMailImportMail::AddMboxDir(nsIFile* aFolder,
     if (mailboxDescFile) mailboxDescFile->InitWithFile(aFolder);
 
     // add this mailbox descriptor to the list
-    aMailboxDescs->AppendElement(desc);
+    aMailboxDescs.AppendElement(desc);
   }
 
   return NS_OK;
@@ -375,11 +371,11 @@ nsresult nsAppleMailImportMail::AddMboxDir(nsIFile* aFolder,
 //     MyChildMailbox.mbox/
 //     MyOtherChildMailbox.mbox/
 //
-nsresult nsAppleMailImportMail::FindMboxDirs(nsIFile* aFolder,
-                                             nsIMutableArray* aMailboxDescs,
-                                             nsIImportService* aImportService) {
+nsresult nsAppleMailImportMail::FindMboxDirs(
+    nsIFile* aFolder,
+    nsTArray<RefPtr<nsIImportMailboxDescriptor>>& aMailboxDescs,
+    nsIImportService* aImportService) {
   NS_ENSURE_ARG_POINTER(aFolder);
-  NS_ENSURE_ARG_POINTER(aMailboxDescs);
   NS_ENSURE_ARG_POINTER(aImportService);
 
   // make sure this is a directory.
@@ -408,9 +404,9 @@ nsresult nsAppleMailImportMail::FindMboxDirs(nsIFile* aFolder,
     nsAutoString currentFolderName;
     if (NS_SUCCEEDED(currentEntry->GetLeafName(currentFolderName)) &&
         (StringEndsWith(currentFolderName,
-                        NS_LITERAL_STRING(POP_MBOX_SUFFIX)) ||
+                        NS_LITERAL_STRING_FROM_CSTRING(POP_MBOX_SUFFIX)) ||
          StringEndsWith(currentFolderName,
-                        NS_LITERAL_STRING(IMAP_MBOX_SUFFIX)))) {
+                        NS_LITERAL_STRING_FROM_CSTRING(IMAP_MBOX_SUFFIX)))) {
       IMPORT_LOG1("Adding .mbox dir: %s",
                   NS_ConvertUTF16toUTF8(currentFolderName).get());
 
@@ -428,10 +424,10 @@ nsresult nsAppleMailImportMail::FindMboxDirs(nsIFile* aFolder,
 
       // cut off suffix
       if (StringEndsWith(siblingMailboxDirPath,
-                         NS_LITERAL_STRING(IMAP_MBOX_SUFFIX)))
+                         NS_LITERAL_STRING_FROM_CSTRING(IMAP_MBOX_SUFFIX)))
         siblingMailboxDirPath.SetLength(siblingMailboxDirPath.Length() - 9);
       else if (StringEndsWith(siblingMailboxDirPath,
-                              NS_LITERAL_STRING(POP_MBOX_SUFFIX)))
+                              NS_LITERAL_STRING_FROM_CSTRING(POP_MBOX_SUFFIX)))
         siblingMailboxDirPath.SetLength(siblingMailboxDirPath.Length() - 5);
 
       IMPORT_LOG1("trying to locate a '%s'",
@@ -490,7 +486,7 @@ nsAppleMailImportMail::ImportMailbox(nsIImportMailboxDescriptor* aMailbox,
     // move to the .mbox's Messages folder
     nsCOMPtr<nsIFile> messagesFolder;
     mboxFolder->Clone(getter_AddRefs(messagesFolder));
-    rv = messagesFolder->Append(NS_LITERAL_STRING("Messages"));
+    rv = messagesFolder->Append(u"Messages"_ns);
     if (NS_FAILED(rv)) {
       // even if there are no messages, it might still be a valid mailbox, or
       // even a parent for other mailboxes.
@@ -545,7 +541,7 @@ nsAppleMailImportMail::ImportMailbox(nsIImportMailboxDescriptor* aMailbox,
 
       nsAutoString leafName;
       currentEntry->GetLeafName(leafName);
-      if (!StringEndsWith(leafName, NS_LITERAL_STRING(".emlx"))) continue;
+      if (!StringEndsWith(leafName, u".emlx"_ns)) continue;
 
       nsCOMPtr<nsIMsgDBHdr> msgHdr;
       bool reusable;
