@@ -3,7 +3,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsArrayUtils.h"
 #include "nsILineInputStream.h"
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
@@ -60,9 +59,9 @@ nsresult nsBeckyFilters::GetFilterFile(bool aIncoming, nsIFile* aLocation,
   nsCOMPtr<nsIFile> filter;
   aLocation->Clone(getter_AddRefs(filter));
   if (aIncoming)
-    rv = filter->Append(NS_LITERAL_STRING("IFilter.def"));
+    rv = filter->Append(u"IFilter.def"_ns);
   else
-    rv = filter->Append(NS_LITERAL_STRING("OFilter.def"));
+    rv = filter->Append(u"OFilter.def"_ns);
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool exists = false;
@@ -229,7 +228,7 @@ nsresult nsBeckyFilters::SetSearchTerm(const nsCString& aLine,
   if (!searchKeyword.IsEmpty())
     rv = aFilter->SetFilterName(searchKeyword);
   else
-    rv = aFilter->SetFilterName(NS_LITERAL_STRING("No name"));
+    rv = aFilter->SetFilterName(u"No name"_ns);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return aFilter->AppendTerm(term);
@@ -437,11 +436,11 @@ nsresult nsBeckyFilters::CreateFilter(bool aIncoming, nsIMsgFilter** _retval) {
   NS_ENSURE_STATE(mServer);
 
   nsCOMPtr<nsIMsgFilterList> filterList;
-  nsresult rv = mServer->GetFilterList(nullptr, getter_AddRefs(filterList));
-  NS_ENSURE_SUCCESS(rv, rv);
+  mServer->GetFilterList(nullptr, getter_AddRefs(filterList));
+  NS_ENSURE_STATE(filterList);
 
   nsCOMPtr<nsIMsgFilter> filter;
-  rv = filterList->CreateFilter(EmptyString(), getter_AddRefs(filter));
+  nsresult rv = filterList->CreateFilter(EmptyString(), getter_AddRefs(filter));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (aIncoming)
@@ -460,11 +459,11 @@ nsresult nsBeckyFilters::AppendFilter(nsIMsgFilter* aFilter) {
   NS_ENSURE_STATE(mServer);
 
   nsCOMPtr<nsIMsgFilterList> filterList;
-  nsresult rv = mServer->GetFilterList(nullptr, getter_AddRefs(filterList));
-  NS_ENSURE_SUCCESS(rv, rv);
+  mServer->GetFilterList(nullptr, getter_AddRefs(filterList));
+  NS_ENSURE_STATE(filterList);
 
   uint32_t count;
-  rv = filterList->GetFilterCount(&count);
+  nsresult rv = filterList->GetFilterCount(&count);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return filterList->InsertFilterAt(count, aFilter);
@@ -499,7 +498,7 @@ nsresult nsBeckyFilters::ParseFilterFile(nsIFile* aFile, bool aIncoming) {
         SetSearchTerm(line, filter);
         break;
       case '$':  // $X: disabled
-        if (StringBeginsWith(line, NS_LITERAL_CSTRING("$X")) && filter) {
+        if (StringBeginsWith(line, "$X"_ns) && filter) {
           filter->SetEnabled(false);
         }
         break;
@@ -609,26 +608,18 @@ nsresult nsBeckyFilters::FindMessageFolder(const nsAString& aName,
   nsCOMPtr<nsIMsgFolder> found;
   rv = aParentFolder->GetChildNamed(aName, getter_AddRefs(found));
   if (found) {
-    found.forget(_retval);
+    NS_ADDREF(*_retval = found);
     return NS_OK;
   }
 
-  nsCOMPtr<nsISimpleEnumerator> children;
-  rv = aParentFolder->GetSubFolders(getter_AddRefs(children));
+  nsTArray<RefPtr<nsIMsgFolder>> children;
+  rv = aParentFolder->GetSubFolders(children);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  bool more;
-  nsCOMPtr<nsISupports> entry;
-  while (NS_SUCCEEDED(children->HasMoreElements(&more)) && more) {
-    rv = children->GetNext(getter_AddRefs(entry));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIMsgFolder> child = do_QueryInterface(entry, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-
+  for (nsIMsgFolder* child : children) {
     rv = FindMessageFolder(aName, child, getter_AddRefs(found));
     if (found) {
-      found.forget(_retval);
+      NS_ADDREF(*_retval = found);
       return NS_OK;
     }
   }
