@@ -25,7 +25,6 @@
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
 #include "nsISeekableStream.h"
-#include "nsIMutableArray.h"
 #include "nsIStringBundle.h"
 #include "nsLocalUndoTxn.h"
 
@@ -41,9 +40,9 @@ struct nsLocalMailCopyState {
   nsCOMPtr<nsIMsgPluggableStore> m_msgStore;
   nsCOMPtr<nsISupports> m_srcSupport;
   /// Source nsIMsgDBHdr instances.
-  nsCOMPtr<nsIArray> m_messages;
+  nsTArray<RefPtr<nsIMsgDBHdr>> m_messages;
   /// Destination nsIMsgDBHdr instances.
-  nsCOMPtr<nsIMutableArray> m_destMessages;
+  nsTArray<RefPtr<nsIMsgDBHdr>> m_destMessages;
   RefPtr<nsLocalMoveCopyMsgTxn> m_undoMsgTxn;
   nsCOMPtr<nsIMsgDBHdr> m_message;  // current copy message
   nsMsgMessageFlagType m_flags;     // current copy message flags
@@ -83,13 +82,10 @@ struct nsLocalFolderScanState {
   ~nsLocalFolderScanState();
 
   nsCOMPtr<nsIInputStream> m_inputStream;
-  nsCOMPtr<nsISeekableStream> m_seekableStream;
   nsCOMPtr<nsIMsgPluggableStore> m_msgStore;
   nsCString m_header;
   nsCString m_accountKey;
   const char* m_uidl;  // memory is owned by m_header
-  // false if we need a new input stream for each message
-  bool m_streamReusable;
 };
 
 class nsMsgLocalMailFolder : public nsMsgDBFolder,
@@ -107,11 +103,10 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
   NS_IMETHOD OnStopRunningUrl(nsIURI* aUrl, nsresult aExitCode) override;
 
   // nsIMsgFolder methods:
-  NS_IMETHOD GetSubFolders(nsISimpleEnumerator** aResult) override;
+  NS_IMETHOD GetSubFolders(nsTArray<RefPtr<nsIMsgFolder>>& folders) override;
   NS_IMETHOD GetMsgDatabase(nsIMsgDatabase** aMsgDatabase) override;
 
   NS_IMETHOD OnAnnouncerGoingAway(nsIDBChangeAnnouncer* instigator) override;
-  NS_IMETHOD GetMessages(nsISimpleEnumerator** result) override;
   NS_IMETHOD UpdateFolder(nsIMsgWindow* aWindow) override;
 
   NS_IMETHOD CreateSubfolder(const nsAString& folderName,
@@ -119,12 +114,11 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
 
   NS_IMETHOD Compact(nsIUrlListener* aListener,
                      nsIMsgWindow* aMsgWindow) override;
-  NS_IMETHOD CompactAll(nsIUrlListener* aListener, nsIMsgWindow* aMsgWindow,
-                        bool aCompactOfflineAlso) override;
+  NS_IMETHOD CompactAll(nsIUrlListener* aListener,
+                        nsIMsgWindow* aMsgWindow) override;
   NS_IMETHOD EmptyTrash(nsIMsgWindow* msgWindow,
                         nsIUrlListener* aListener) override;
-  NS_IMETHOD DeleteSubFolders(nsIArray* folders,
-                              nsIMsgWindow* msgWindow) override;
+  NS_IMETHOD DeleteSelf(nsIMsgWindow* msgWindow) override;
   NS_IMETHOD CreateStorageIfMissing(nsIUrlListener* urlListener) override;
   NS_IMETHOD Rename(const nsAString& aNewName,
                     nsIMsgWindow* msgWindow) override;
@@ -145,14 +139,14 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
   NS_IMETHOD GetDBFolderInfoAndDB(nsIDBFolderInfo** folderInfo,
                                   nsIMsgDatabase** db) override;
 
-  NS_IMETHOD DeleteMessages(nsIArray* messages, nsIMsgWindow* msgWindow,
-                            bool deleteStorage, bool isMove,
-                            nsIMsgCopyServiceListener* listener,
+  NS_IMETHOD DeleteMessages(nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
+                            nsIMsgWindow* msgWindow, bool deleteStorage,
+                            bool isMove, nsIMsgCopyServiceListener* listener,
                             bool allowUndo) override;
-  MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHOD
-  CopyMessages(nsIMsgFolder* srcFolder, nsIArray* messages, bool isMove,
-               nsIMsgWindow* msgWindow, nsIMsgCopyServiceListener* listener,
-               bool isFolder, bool allowUndo) override;
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHOD CopyMessages(
+      nsIMsgFolder* srcFolder, nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
+      bool isMove, nsIMsgWindow* msgWindow, nsIMsgCopyServiceListener* listener,
+      bool isFolder, bool allowUndo) override;
   NS_IMETHOD CopyFolder(nsIMsgFolder* srcFolder, bool isMoveFolder,
                         nsIMsgWindow* msgWindow,
                         nsIMsgCopyServiceListener* listener) override;
@@ -164,8 +158,9 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
 
   NS_IMETHOD AddMessageDispositionState(
       nsIMsgDBHdr* aMessage, nsMsgDispositionState aDispositionFlag) override;
-  NS_IMETHOD MarkMessagesRead(nsIArray* aMessages, bool aMarkRead) override;
-  NS_IMETHOD MarkMessagesFlagged(nsIArray* aMessages,
+  NS_IMETHOD MarkMessagesRead(const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
+                              bool aMarkRead) override;
+  NS_IMETHOD MarkMessagesFlagged(const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
                                  bool aMarkFlagged) override;
   NS_IMETHOD MarkAllMessagesRead(nsIMsgWindow* aMsgWindow) override;
   NS_IMETHOD MarkThreadRead(nsIMsgThread* thread) override;
@@ -181,20 +176,25 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
   NS_IMETHOD GetName(nsAString& aName) override;
 
   // Used when headers_only is TRUE
-  NS_IMETHOD DownloadMessagesForOffline(nsIArray* aMessages,
-                                        nsIMsgWindow* aWindow) override;
+  NS_IMETHOD DownloadMessagesForOffline(
+      nsTArray<RefPtr<nsIMsgDBHdr>> const& aMessages,
+      nsIMsgWindow* aWindow) override;
+  NS_IMETHOD GetLocalMsgStream(nsIMsgDBHdr* hdr,
+                               nsIInputStream** stream) override;
   NS_IMETHOD FetchMsgPreviewText(nsTArray<nsMsgKey> const& aKeysToFetch,
-                                 bool aLocalOnly, nsIUrlListener* aUrlListener,
+                                 nsIUrlListener* aUrlListener,
                                  bool* aAsyncResults) override;
-  NS_IMETHOD AddKeywordsToMessages(nsIArray* aMessages,
-                                   const nsACString& aKeywords) override;
-  NS_IMETHOD RemoveKeywordsFromMessages(nsIArray* aMessages,
-                                        const nsACString& aKeywords) override;
+  NS_IMETHOD AddKeywordsToMessages(
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
+      const nsACString& aKeywords) override;
+  NS_IMETHOD RemoveKeywordsFromMessages(
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
+      const nsACString& aKeywords) override;
   NS_IMETHOD GetIncomingServerType(nsACString& serverType) override;
 
  protected:
   virtual ~nsMsgLocalMailFolder();
-  nsresult CreateChildFromURI(const nsCString& uri,
+  nsresult CreateChildFromURI(const nsACString& uri,
                               nsIMsgFolder** folder) override;
   nsresult CopyFolderAcrossServer(nsIMsgFolder* srcFolder,
                                   nsIMsgWindow* msgWindow,
@@ -225,8 +225,8 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
   // copy message helper
   nsresult DisplayMoveCopyStatusMsg();
 
-  nsresult CopyMessageTo(nsISupports* message, nsIMsgFolder* dstFolder,
-                         nsIMsgWindow* msgWindow, bool isMove);
+  nsresult CopyMessageTo(nsISupports* message, nsIMsgWindow* msgWindow,
+                         bool isMove);
 
   /**
    * Checks if there's room in the target folder to copy message(s) into.
@@ -237,11 +237,11 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
                            int64_t totalMsgSize);
 
   // copy multiple messages at a time from this folder
-  nsresult CopyMessagesTo(nsIArray* messages, nsTArray<nsMsgKey>& keyArray,
-                          nsIMsgWindow* aMsgWindow, nsIMsgFolder* dstFolder,
-                          bool isMove);
-  nsresult InitCopyState(nsISupports* aSupport, nsIArray* messages, bool isMove,
-                         nsIMsgCopyServiceListener* listener,
+  nsresult CopyMessagesTo(nsTArray<nsMsgKey>& keyArray,
+                          nsIMsgWindow* aMsgWindow, bool isMove);
+  nsresult InitCopyState(nsISupports* aSupport,
+                         nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
+                         bool isMove, nsIMsgCopyServiceListener* listener,
                          nsIMsgWindow* msgWindow, bool isMoveFolder,
                          bool allowUndo);
   nsresult InitCopyMsgHdrAndFileStream();
@@ -249,8 +249,9 @@ class nsMsgLocalMailFolder : public nsMsgDBFolder,
   void CopyPropertiesToMsgHdr(nsIMsgDBHdr* destHdr, nsIMsgDBHdr* srcHdr,
                               bool isMove);
   virtual nsresult CreateBaseMessageURI(const nsACString& aURI) override;
-  nsresult ChangeKeywordForMessages(nsIArray* aMessages,
-                                    const nsACString& aKeyword, bool add);
+  nsresult ChangeKeywordForMessages(
+      nsTArray<RefPtr<nsIMsgDBHdr>> const& aMessages,
+      const nsACString& aKeyword, bool add);
   bool GetDeleteFromServerOnMove();
   void CopyHdrPropertiesWithSkipList(nsIMsgDBHdr* destHdr, nsIMsgDBHdr* srcHdr,
                                      const nsCString& skipList);
