@@ -15,7 +15,7 @@
 #include "nsIMsgFolderNotificationService.h"
 #include "nsThreadUtils.h"
 #include "nsIMsgDatabase.h"
-#include "nsIMutableArray.h"
+#include "nsIMsgHdr.h"
 #include "nsServiceManagerUtils.h"
 #include "nsComponentManagerUtils.h"
 #include "nsMsgUtils.h"
@@ -66,11 +66,6 @@ nsresult nsLocalMoveCopyMsgTxn::AddSrcKey(nsMsgKey aKey) {
   return NS_OK;
 }
 
-nsresult nsLocalMoveCopyMsgTxn::AddSrcStatusOffset(uint32_t aStatusOffset) {
-  m_srcStatusOffsetArray.AppendElement(aStatusOffset);
-  return NS_OK;
-}
-
 nsresult nsLocalMoveCopyMsgTxn::AddDstKey(nsMsgKey aKey) {
   m_dstKeyArray.AppendElement(aKey);
   return NS_OK;
@@ -102,11 +97,11 @@ nsresult nsLocalMoveCopyMsgTxn::UndoImapDeleteFlag(nsIMsgFolder* folder,
     // folder so use lite select to do the trick
     rv = imapService->LiteSelectFolder(folder, urlListener, nullptr, nullptr);
     if (!deleteFlag)
-      rv = imapService->AddMessageFlags(folder, urlListener, nullptr, msgIds,
+      rv = imapService->AddMessageFlags(folder, urlListener, msgIds,
                                         kImapMsgDeletedFlag, true);
     else
-      rv = imapService->SubtractMessageFlags(folder, urlListener, nullptr,
-                                             msgIds, kImapMsgDeletedFlag, true);
+      rv = imapService->SubtractMessageFlags(folder, urlListener, msgIds,
+                                             kImapMsgDeletedFlag, true);
     if (NS_SUCCEEDED(rv) && m_msgWindow) folder->UpdateFolder(m_msgWindow);
     rv = NS_OK;  // always return NS_OK to indicate that the src is imap
   } else
@@ -207,7 +202,6 @@ nsresult nsLocalMoveCopyMsgTxn::UndoTransactionInternal() {
                                              getter_AddRefs(newHdr));
           NS_ASSERTION(newHdr, "fatal ... cannot create new msg header");
           if (NS_SUCCEEDED(rv) && newHdr) {
-            newHdr->SetStatusOffset(m_srcStatusOffsetArray[i]);
             srcDB->UndoDelete(newHdr);
             srcMessages.AppendElement(newHdr);
             // (we want to keep these two lists in sync)
@@ -233,8 +227,7 @@ nsresult nsLocalMoveCopyMsgTxn::UndoTransactionInternal() {
       }
     } else  // undoing a move means moving the messages back.
     {
-      nsCOMPtr<nsIMutableArray> dstMessages =
-          do_CreateInstance(NS_ARRAY_CONTRACTID);
+      nsTArray<RefPtr<nsIMsgDBHdr>> dstMessages(m_dstKeyArray.Length());
       m_numHdrsCopied = 0;
       m_srcKeyArray.Clear();
       for (i = 0; i < count; i++) {
@@ -247,7 +240,7 @@ nsresult nsLocalMoveCopyMsgTxn::UndoTransactionInternal() {
         if (dstHdr) {
           nsCString messageId;
           dstHdr->GetMessageId(getter_Copies(messageId));
-          dstMessages->AppendElement(dstHdr);
+          dstMessages.AppendElement(dstHdr);
           m_copiedMsgIds.AppendElement(messageId);
         } else {
           NS_WARNING("Cannot get old msg header");
@@ -294,17 +287,13 @@ nsLocalMoveCopyMsgTxn::RedoTransaction() {
   nsCOMPtr<nsIMsgDBHdr> oldHdr;
   nsCOMPtr<nsIMsgDBHdr> newHdr;
 
-  nsCOMPtr<nsIMutableArray> srcMessages =
-      do_CreateInstance(NS_ARRAY_CONTRACTID);
-  nsCOMPtr<nsISupports> msgSupports;
-
+  nsTArray<RefPtr<nsIMsgDBHdr>> srcMessages(m_srcKeyArray.Length());
   for (i = 0; i < count; i++) {
     rv = srcDB->GetMsgHdrForKey(m_srcKeyArray[i], getter_AddRefs(oldHdr));
     NS_ASSERTION(oldHdr, "fatal ... cannot get old msg header");
 
     if (NS_SUCCEEDED(rv) && oldHdr) {
-      msgSupports = do_QueryInterface(oldHdr);
-      srcMessages->AppendElement(msgSupports);
+      srcMessages.AppendElement(oldHdr);
 
       if (m_canUndelete) {
         rv = dstDB->CopyHdrFromExistingHdr(m_dstKeyArray[i], oldHdr, true,
@@ -335,9 +324,8 @@ nsLocalMoveCopyMsgTxn::RedoTransaction() {
       nsCOMPtr<nsIMsgLocalMailFolder> localFolder =
           do_QueryInterface(srcFolder);
       if (localFolder) {
-        nsTArray<RefPtr<nsIMsgDBHdr>> hdrs;
-        MsgHdrsToTArray(srcMessages, hdrs);
-        localFolder->MarkMsgsOnPop3Server(hdrs, POP3_DELETE /*deleteMsgs*/);
+        localFolder->MarkMsgsOnPop3Server(srcMessages,
+                                          POP3_DELETE /*deleteMsgs*/);
       }
 
       rv = srcDB->DeleteMessages(m_srcKeyArray, nullptr);
@@ -365,69 +353,76 @@ nsLocalMoveCopyMsgTxn::RedoTransaction() {
   return rv;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemAdded(nsIMsgFolder* parentItem,
-                                                 nsISupports* item) {
-  nsCOMPtr<nsIMsgDBHdr> msgHdr(do_QueryInterface(item));
-  if (msgHdr) {
-    nsresult rv;
-    nsCOMPtr<nsIMsgFolder> folder =
-        do_QueryReferent(m_undoing ? m_srcFolder : m_dstFolder, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-    nsCString messageId;
-    msgHdr->GetMessageId(getter_Copies(messageId));
-    if (m_copiedMsgIds.Contains(messageId)) {
-      nsMsgKey msgKey;
-      msgHdr->GetMessageKey(&msgKey);
-      if (m_undoing)
-        m_srcKeyArray.AppendElement(msgKey);
-      else
-        m_dstKeyArray.AppendElement(msgKey);
-      if (++m_numHdrsCopied == m_copiedMsgIds.Length()) {
-        folder->RemoveFolderListener(this);
-        m_copiedMsgIds.Clear();
-      }
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderAdded(nsIMsgFolder* parent,
+                                                   nsIMsgFolder* child) {
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnMessageAdded(nsIMsgFolder* parent,
+                                                    nsIMsgDBHdr* msgHdr) {
+  nsresult rv;
+  nsCOMPtr<nsIMsgFolder> folder =
+      do_QueryReferent(m_undoing ? m_srcFolder : m_dstFolder, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCString messageId;
+  msgHdr->GetMessageId(getter_Copies(messageId));
+  if (m_copiedMsgIds.Contains(messageId)) {
+    nsMsgKey msgKey;
+    msgHdr->GetMessageKey(&msgKey);
+    if (m_undoing)
+      m_srcKeyArray.AppendElement(msgKey);
+    else
+      m_dstKeyArray.AppendElement(msgKey);
+    if (++m_numHdrsCopied == m_copiedMsgIds.Length()) {
+      folder->RemoveFolderListener(this);
+      m_copiedMsgIds.Clear();
     }
   }
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemRemoved(nsIMsgFolder* parentItem,
-                                                   nsISupports* item) {
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderRemoved(nsIMsgFolder* parent,
+                                                     nsIMsgFolder* child) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemPropertyChanged(
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnMessageRemoved(nsIMsgFolder* parent,
+                                                      nsIMsgDBHdr* msg) {
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, const nsACString& oldValue,
     const nsACString& newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemIntPropertyChanged(
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderIntPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, int64_t oldValue,
     int64_t newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemBoolPropertyChanged(
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderBoolPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, bool oldValue,
     bool newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemUnicharPropertyChanged(
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderUnicharPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, const nsAString& oldValue,
     const nsAString& newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemPropertyFlagChanged(
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderPropertyFlagChanged(
     nsIMsgDBHdr* item, const nsACString& property, uint32_t oldFlag,
     uint32_t newFlag) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnItemEvent(nsIMsgFolder* aItem,
-                                                 const nsACString& aEvent) {
+NS_IMETHODIMP nsLocalMoveCopyMsgTxn::OnFolderEvent(nsIMsgFolder* aItem,
+                                                   const nsACString& aEvent) {
   return NS_OK;
 }
 
@@ -441,48 +436,58 @@ nsLocalUndoFolderListener::nsLocalUndoFolderListener(
 
 nsLocalUndoFolderListener::~nsLocalUndoFolderListener() {}
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemAdded(nsIMsgFolder* parentItem,
-                                                     nsISupports* item) {
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderAdded(nsIMsgFolder* parent,
+                                                       nsIMsgFolder* child) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemRemoved(nsIMsgFolder* parentItem,
-                                                       nsISupports* item) {
+NS_IMETHODIMP nsLocalUndoFolderListener::OnMessageAdded(nsIMsgFolder* parent,
+                                                        nsIMsgDBHdr* msg) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemPropertyChanged(
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderRemoved(nsIMsgFolder* parent,
+                                                         nsIMsgFolder* child) {
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsLocalUndoFolderListener::OnMessageRemoved(nsIMsgFolder* parent,
+                                                          nsIMsgDBHdr* msg) {
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, const nsACString& oldValue,
     const nsACString& newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemIntPropertyChanged(
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderIntPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, int64_t oldValue,
     int64_t newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemBoolPropertyChanged(
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderBoolPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, bool oldValue,
     bool newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemUnicharPropertyChanged(
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderUnicharPropertyChanged(
     nsIMsgFolder* item, const nsACString& property, const nsAString& oldValue,
     const nsAString& newValue) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemPropertyFlagChanged(
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderPropertyFlagChanged(
     nsIMsgDBHdr* item, const nsACString& property, uint32_t oldFlag,
     uint32_t newFlag) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsLocalUndoFolderListener::OnItemEvent(nsIMsgFolder* aItem,
-                                                     const nsACString& aEvent) {
+NS_IMETHODIMP nsLocalUndoFolderListener::OnFolderEvent(
+    nsIMsgFolder* aItem, const nsACString& aEvent) {
   if (mTxn && mFolder && aItem == mFolder) {
     if (aEvent.Equals(kFolderLoaded)) return mTxn->UndoTransactionInternal();
   }
