@@ -47,6 +47,7 @@ nsMailboxUrl::nsMailboxUrl() {
   m_addDummyEnvelope = false;
   m_canonicalLineEnding = false;
   m_curMsgIndex = 0;
+  mAutodetectCharset = false;
 }
 
 nsMailboxUrl::~nsMailboxUrl() { PR_Free(m_messageID); }
@@ -138,12 +139,12 @@ NS_IMETHODIMP nsMailboxUrl::GetNormalizedSpec(nsACString& aPrincipalSpec) {
           folderPath,
           nsINetUtil::ESCAPE_URL_DIRECTORY | nsINetUtil::ESCAPE_URL_FORCED,
           buf);
-      spec = NS_LITERAL_CSTRING("mailbox://") + buf;
+      spec = "mailbox://"_ns + buf;
     }
   }
 
   if (messageKey) {
-    spec += NS_LITERAL_CSTRING("?number=");
+    spec += "?number="_ns;
     spec.Append(messageKey);
     PR_Free(messageKey);
   }
@@ -166,10 +167,9 @@ nsresult nsMailboxUrl::CreateURL(const nsACString& aSpec, nsIURL** aURL) {
   } else {
     // The URL is more like a file URL without a hostname.
     rv = NS_MutateURI(NS_STANDARDURLMUTATOR_CONTRACTID)
-             .Apply(NS_MutatorMethod(&nsIStandardURLMutator::Init,
-                                     nsIStandardURL::URLTYPE_NO_AUTHORITY, -1,
-                                     PromiseFlatCString(aSpec), nullptr,
-                                     nullptr, nullptr))
+             .Apply(&nsIStandardURLMutator::Init,
+                    nsIStandardURL::URLTYPE_NO_AUTHORITY, -1,
+                    PromiseFlatCString(aSpec), nullptr, nullptr, nullptr)
              .Finalize(url);
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -177,7 +177,7 @@ nsresult nsMailboxUrl::CreateURL(const nsACString& aSpec, nsIURL** aURL) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMailboxUrl::SetUri(const char* aURI) {
+NS_IMETHODIMP nsMailboxUrl::SetUri(const nsACString& aURI) {
   mURI = aURI;
   return NS_OK;
 }
@@ -188,16 +188,16 @@ nsresult nsMailboxUrl::Clone(nsIURI** _retval) {
   // also clone the mURI member, because GetUri below won't work if
   // mURI isn't set due to nsIFile fun.
   nsCOMPtr<nsIMsgMessageUrl> clonedUrl = do_QueryInterface(*_retval);
-  if (clonedUrl) clonedUrl->SetUri(mURI.get());
+  if (clonedUrl) clonedUrl->SetUri(mURI);
   return rv;
 }
 
-NS_IMETHODIMP nsMailboxUrl::GetUri(char** aURI) {
+NS_IMETHODIMP nsMailboxUrl::GetUri(nsACString& aURI) {
   // if we have been given a uri to associate with this url, then use it
   // otherwise try to reconstruct a URI on the fly....
 
   if (!mURI.IsEmpty())
-    *aURI = ToNewCString(mURI);
+    aURI = mURI;
   else {
     if (m_filePath) {
       nsAutoCString baseUri;
@@ -215,11 +215,9 @@ NS_IMETHODIMP nsMailboxUrl::GetUri(char** aURI) {
       }
       nsCString baseMessageURI;
       nsCreateLocalBaseMessageURI(baseUri, baseMessageURI);
-      nsAutoCString uriStr;
-      nsBuildLocalMessageURI(baseMessageURI.get(), m_messageKey, uriStr);
-      *aURI = ToNewCString(uriStr);
+      nsBuildLocalMessageURI(baseMessageURI, m_messageKey, aURI);
     } else
-      *aURI = nullptr;
+      aURI = "";
   }
 
   return NS_OK;
@@ -285,14 +283,14 @@ NS_IMPL_GETSET(nsMailboxUrl, AddDummyEnvelope, bool, m_addDummyEnvelope)
 NS_IMPL_GETSET(nsMailboxUrl, CanonicalLineEnding, bool, m_canonicalLineEnding)
 
 NS_IMETHODIMP
-nsMailboxUrl::GetOriginalSpec(char** aSpec) {
-  if (!aSpec || m_originalSpec.IsEmpty()) return NS_ERROR_NULL_POINTER;
-  *aSpec = ToNewCString(m_originalSpec);
+nsMailboxUrl::GetOriginalSpec(nsACString& aSpec) {
+  if (m_originalSpec.IsEmpty()) return NS_ERROR_NULL_POINTER;
+  aSpec = m_originalSpec;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMailboxUrl::SetOriginalSpec(const char* aSpec) {
+nsMailboxUrl::SetOriginalSpec(const nsACString& aSpec) {
   m_originalSpec = aSpec;
   return NS_OK;
 }
@@ -436,8 +434,7 @@ char* extractAttributeValue(const char* searchString,
       startOfAttribute += attributeNameSize;  // skip over the attributeName
       if (startOfAttribute)  // is there something after the attribute name
       {
-        char* endOfAttribute =
-            startOfAttribute ? PL_strchr(startOfAttribute, '&') : nullptr;
+        char* endOfAttribute = PL_strchr(startOfAttribute, '&');
         nsDependentCString attributeValueStr;
         if (startOfAttribute &&
             endOfAttribute)  // is there text after attribute value
@@ -464,49 +461,21 @@ nsresult nsMailboxUrl::GetFolder(nsIMsgFolder** msgFolder) {
   // if we have a RDF URI, then try to get the folder for that URI and then ask
   // the folder for it's charset....
   nsCString uri;
-  GetUri(getter_Copies(uri));
+  GetUri(uri);
   NS_ENSURE_TRUE(!uri.IsEmpty(), NS_ERROR_FAILURE);
   nsCOMPtr<nsIMsgDBHdr> msg;
-  GetMsgDBHdrFromURI(uri.get(), getter_AddRefs(msg));
+  GetMsgDBHdrFromURI(uri, getter_AddRefs(msg));
   if (!msg) return NS_ERROR_FAILURE;
   return msg->GetFolder(msgFolder);
 }
 
-NS_IMETHODIMP nsMailboxUrl::GetFolderCharset(char** aCharacterSet) {
-  NS_ENSURE_ARG_POINTER(aCharacterSet);
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetFolder(getter_AddRefs(folder));
-
-  // In cases where a file is not associated with a folder, for
-  // example standalone .eml files, failure is normal.
-  if (NS_FAILED(rv)) return rv;
-  nsCString tmpStr;
-  folder->GetCharset(tmpStr);
-  *aCharacterSet = ToNewCString(tmpStr);
+NS_IMETHODIMP nsMailboxUrl::GetAutodetectCharset(bool* aAutodetectCharset) {
+  *aAutodetectCharset = mAutodetectCharset;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMailboxUrl::GetFolderCharsetOverride(
-    bool* aCharacterSetOverride) {
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetFolder(getter_AddRefs(folder));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(folder, NS_ERROR_FAILURE);
-  folder->GetCharsetOverride(aCharacterSetOverride);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsMailboxUrl::GetCharsetOverRide(char** aCharacterSet) {
-  if (!mCharsetOverride.IsEmpty())
-    *aCharacterSet = ToNewCString(mCharsetOverride);
-  else
-    *aCharacterSet = nullptr;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsMailboxUrl::SetCharsetOverRide(const char* aCharacterSet) {
-  mCharsetOverride = aCharacterSet;
+NS_IMETHODIMP nsMailboxUrl::SetAutodetectCharset(bool aAutodetectCharset) {
+  mAutodetectCharset = aAutodetectCharset;
   return NS_OK;
 }
 
