@@ -15,11 +15,9 @@
 #include "nsMsgLocalCID.h"
 #include "nsMsgFolderFlags.h"
 #include "nsPop3Protocol.h"
-#include "nsMsgKeyArray.h"
 #include "nsIMsgLocalMailFolder.h"
 #include "nsIMsgAccountManager.h"
 #include "nsServiceManagerUtils.h"
-#include "nsIMutableArray.h"
 #include "nsMsgUtils.h"
 #include "nsMsgDBFolder.h"
 #include "nsComponentManagerUtils.h"
@@ -117,41 +115,36 @@ NS_IMETHODIMP nsPop3IncomingServer::GetDeferredToAccount(nsACString& aRetVal) {
           // think Sent Mail could end up here, but if any folders have
           // messages, might as well copy them to the real Local Folders
           // account.
-          nsCOMPtr<nsISimpleEnumerator> enumerator;
-          rv = hiddenRootFolder->GetSubFolders(getter_AddRefs(enumerator));
+          nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+          rv = hiddenRootFolder->GetSubFolders(subFolders);
           if (NS_SUCCEEDED(rv)) {
-            bool hasMore;
-            while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) &&
-                   hasMore) {
-              nsCOMPtr<nsISupports> item;
-              enumerator->GetNext(getter_AddRefs(item));
-              nsCOMPtr<nsIMsgFolder> subFolder(do_QueryInterface(item));
-              if (subFolder) {
-                nsCOMPtr<nsIMsgDatabase> subFolderDB;
-                subFolder->GetMsgDatabase(getter_AddRefs(subFolderDB));
-                if (subFolderDB) {
-                  // Copy any messages in this sub-folder of the hidden
-                  // account to the corresponding folder in Local Folders.
-                  RefPtr<nsMsgKeyArray> keys = new nsMsgKeyArray;
-                  rv = subFolderDB->ListAllKeys(keys);
-                  nsCOMPtr<nsIMutableArray> hdrsToCopy(
-                      do_CreateInstance(NS_ARRAY_CONTRACTID));
-                  MsgGetHeadersFromKeys(subFolderDB, keys->m_keys, hdrsToCopy);
-                  uint32_t numHdrs = 0;
-                  if (hdrsToCopy) hdrsToCopy->GetLength(&numHdrs);
-                  if (numHdrs) {
-                    // Look for a folder with the same name in Local Folders.
-                    nsCOMPtr<nsIMsgFolder> dest;
-                    nsString folderName;
-                    subFolder->GetName(folderName);
-                    localFoldersRoot->GetChildNamed(folderName,
-                                                    getter_AddRefs(dest));
-                    if (dest)
-                      dest->CopyMessages(subFolder, hdrsToCopy, false, nullptr,
-                                         nullptr, false, false);
-                    // Should we copy the folder if the dest doesn't exist?
-                  }
+            for (nsIMsgFolder* subFolder : subFolders) {
+              nsCOMPtr<nsIMsgDatabase> subFolderDB;
+              subFolder->GetMsgDatabase(getter_AddRefs(subFolderDB));
+              if (!subFolderDB) {
+                continue;
+              }
+              // Copy any messages in this sub-folder of the hidden
+              // account to the corresponding folder in Local Folders.
+              nsTArray<nsMsgKey> keys;
+              rv = subFolderDB->ListAllKeys(keys);
+              if (NS_FAILED(rv)) {
+                continue;  // Next subfolder.
+              }
+              nsTArray<RefPtr<nsIMsgDBHdr>> hdrsToCopy;
+              MsgGetHeadersFromKeys(subFolderDB, keys, hdrsToCopy);
+              if (!hdrsToCopy.IsEmpty()) {
+                // Look for a folder with the same name in Local Folders.
+                nsCOMPtr<nsIMsgFolder> dest;
+                nsString folderName;
+                subFolder->GetName(folderName);
+                localFoldersRoot->GetChildNamed(folderName,
+                                                getter_AddRefs(dest));
+                if (dest) {
+                  dest->CopyMessages(subFolder, hdrsToCopy, false, nullptr,
+                                     nullptr, false, false);
                 }
+                // Should we copy the folder if the dest doesn't exist?
               }
             }
           }
@@ -189,10 +182,10 @@ NS_IMETHODIMP nsPop3IncomingServer::SetDeferredToAccount(
     if (rootFolder) {
       // if isDeferred state has changed, send notification
       if (aAccountKey.IsEmpty() != deferredToAccount.IsEmpty()) {
-        folderListenerManager->OnItemBoolPropertyChanged(
+        folderListenerManager->OnFolderBoolPropertyChanged(
             rootFolder, kIsDeferred, !deferredToAccount.IsEmpty(),
             deferredToAccount.IsEmpty());
-        folderListenerManager->OnItemBoolPropertyChanged(
+        folderListenerManager->OnFolderBoolPropertyChanged(
             rootFolder, kCanFileMessages, deferredToAccount.IsEmpty(),
             !deferredToAccount.IsEmpty());
 
@@ -220,8 +213,7 @@ NS_IMETHODIMP nsPop3IncomingServer::SetDeferredToAccount(
                   rv = server->GetRootFolder(getter_AddRefs(rootFolder));
                   NS_ENSURE_SUCCESS(rv, rv);
                   // this will fail if it already exists, which is fine.
-                  rootFolder->CreateSubfolder(NS_LITERAL_STRING("Inbox"),
-                                              nullptr);
+                  rootFolder->CreateSubfolder(u"Inbox"_ns, nullptr);
                 }
               }
             }
@@ -348,10 +340,11 @@ NS_IMETHODIMP nsPop3IncomingServer::PerformBiff(nsIMsgWindow* aMsgWindow) {
       nsCOMPtr<nsIMsgDatabase> db;
       rv = inbox->GetMsgDatabase(getter_AddRefs(db));
       if (NS_SUCCEEDED(rv) && db) rv = db->GetSummaryValid(&valid);
-      if (NS_SUCCEEDED(rv) && valid)
+      if (NS_SUCCEEDED(rv) && valid) {
+        nsCOMPtr<nsIURI> resultURI;
         rv = pop3Service->GetNewMail(aMsgWindow, urlListener, inbox, this,
-                                     nullptr);
-      else {
+                                     getter_AddRefs(resultURI));
+      } else {
         bool isLocked;
         inbox->GetLocked(&isLocked);
         if (!isLocked)
@@ -384,10 +377,10 @@ nsPop3IncomingServer::SetFlagsOnDefaultMailboxes() {
 }
 
 NS_IMETHODIMP nsPop3IncomingServer::CreateDefaultMailboxes() {
-  nsresult rv = CreateLocalFolder(NS_LITERAL_STRING("Inbox"));
+  nsresult rv = CreateLocalFolder(u"Inbox"_ns);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return CreateLocalFolder(NS_LITERAL_STRING("Trash"));
+  return CreateLocalFolder(u"Trash"_ns);
 }
 
 // override this so we can say that deferred accounts can't have messages
@@ -438,6 +431,9 @@ NS_IMETHODIMP nsPop3IncomingServer::GetNewMail(nsIMsgWindow* aMsgWindow,
                                                nsIMsgFolder* aInbox,
                                                nsIURI** aResult) {
   nsresult rv;
+  if (aResult) {
+    *aResult = nullptr;
+  }
   nsCOMPtr<nsIPop3Service> pop3Service = do_GetService(kCPop3ServiceCID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   return pop3Service->GetNewMail(aMsgWindow, aUrlListener, aInbox, this,
@@ -550,9 +546,9 @@ NS_IMETHODIMP nsPop3IncomingServer::AddUidlToMark(const char* aUidl,
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  uidlEntry->status = (aMark == POP3_DELETE)
-                          ? DELETE_CHAR
-                          : (aMark == POP3_FETCH_BODY) ? FETCH_BODY : KEEP;
+  uidlEntry->status = (aMark == POP3_DELETE)       ? DELETE_CHAR
+                      : (aMark == POP3_FETCH_BODY) ? FETCH_BODY
+                                                   : KEEP;
   m_uidlsToMark.AppendElement(uidlEntry);
   return NS_OK;
 }
