@@ -47,7 +47,6 @@ NS_IMPL_ISUPPORTS(nsPop3Sink, nsIPop3Sink)
 
 nsPop3Sink::nsPop3Sink() {
   m_authed = false;
-  m_downloadingToTempFile = false;
   m_biffState = 0;
   m_numNewMessages = 0;
   m_numNewMessagesInFolder = 0;
@@ -74,7 +73,7 @@ nsresult nsPop3Sink::GetUserAuthenticated(bool* authed) {
   return m_popServer->GetAuthenticated(authed);
 }
 
-nsresult nsPop3Sink::SetSenderAuthedFlag(void* closure, bool authed) {
+nsresult nsPop3Sink::SetSenderAuthedFlag(bool authed) {
   m_authed = authed;
   return NS_OK;
 }
@@ -99,7 +98,7 @@ partialRecord::~partialRecord() {}
 // matches the current Account, then look for the Uidl and save
 // this message for later processing.
 nsresult nsPop3Sink::FindPartialMessages() {
-  nsCOMPtr<nsISimpleEnumerator> messages;
+  nsCOMPtr<nsIMsgEnumerator> messages;
   bool hasMore = false;
   bool isOpen = false;
   nsLocalFolderScanState folderScanState;
@@ -112,10 +111,10 @@ nsresult nsPop3Sink::FindPartialMessages() {
   nsresult rv = db->EnumerateMessages(getter_AddRefs(messages));
   if (messages) messages->HasMoreElements(&hasMore);
   while (hasMore && NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsISupports> aSupport;
     uint32_t flags = 0;
-    rv = messages->GetNext(getter_AddRefs(aSupport));
-    nsCOMPtr<nsIMsgDBHdr> msgDBHdr(do_QueryInterface(aSupport, &rv));
+    nsCOMPtr<nsIMsgDBHdr> msgDBHdr;
+    rv = messages->GetNext(getter_AddRefs(msgDBHdr));
+    if (!NS_SUCCEEDED(rv)) break;
     msgDBHdr->GetFlags(&flags);
     if (flags & nsMsgMessageFlags::Partial) {
       // Open the various streams we need to seek and read from the mailbox
@@ -180,7 +179,6 @@ void nsPop3Sink::CheckPartialMessages(nsIPop3Protocol* protocol) {
 nsresult nsPop3Sink::BeginMailDelivery(bool uidlDownload,
                                        nsIMsgWindow* aMsgWindow, bool* aBool) {
   nsresult rv;
-
   nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(m_popServer);
   if (!server) return NS_ERROR_UNEXPECTED;
 
@@ -196,6 +194,8 @@ nsresult nsPop3Sink::BeginMailDelivery(bool uidlDownload,
   bool isLocked;
   nsCOMPtr<nsISupports> supports =
       do_QueryInterface(static_cast<nsIPop3Sink*>(this));
+
+  NS_ENSURE_STATE(m_folder);
   m_folder->GetLocked(&isLocked);
   if (!isLocked) {
     MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
@@ -235,8 +235,6 @@ nsresult nsPop3Sink::EndMailDelivery(nsIPop3Protocol* protocol) {
     m_outFileStream = nullptr;
   }
 
-  if (m_downloadingToTempFile) m_tmpDownloadFile->Remove(false);
-
   // tell the parser to mark the db valid *after* closing the mailbox.
   if (m_newMailParser) m_newMailParser->UpdateDBFolderInfo();
 
@@ -350,9 +348,6 @@ nsresult nsPop3Sink::AbortMailDelivery(nsIPop3Protocol* protocol) {
     m_outFileStream = nullptr;
   }
 
-  if (m_downloadingToTempFile && m_tmpDownloadFile)
-    m_tmpDownloadFile->Remove(false);
-
   /* tell the parser to mark the db valid *after* closing the mailbox.
   we have truncated the inbox, so berkeley mailbox and msf file are in sync*/
   if (m_newMailParser) m_newMailParser->UpdateDBFolderInfo();
@@ -373,72 +368,25 @@ nsresult nsPop3Sink::AbortMailDelivery(nsIPop3Protocol* protocol) {
 }
 
 NS_IMETHODIMP
-nsPop3Sink::IncorporateBegin(const char* uidlString, nsIURI* aURL,
-                             uint32_t flags, void** closure) {
+nsPop3Sink::IncorporateBegin(const char* uidlString, uint32_t flags) {
 #ifdef DEBUG
   printf("Incorporate message begin:\n");
   if (uidlString) printf("uidl string: %s\n", uidlString);
 #endif
-  nsCOMPtr<nsIFile> path;
-
-  m_folder->GetFilePath(getter_AddRefs(path));
 
   nsresult rv;
-  nsCOMPtr<nsIPrefBranch> pPrefBranch(
-      do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
-  if (pPrefBranch) {
-    nsCOMPtr<nsIMsgIncomingServer> server;
-    m_folder->GetServer(getter_AddRefs(server));
-    nsCString plugStoreContract;
-    server->GetCharValue("storeContractID", plugStoreContract);
-    // Maildir doesn't care about quaranting, but other stores besides berkeley
-    // mailbox might. We should probably make this an attribute on the pluggable
-    // store, though.
-    if (plugStoreContract.Equals(
-            NS_LITERAL_CSTRING("@mozilla.org/msgstore/berkeleystore;1")))
-      pPrefBranch->GetBoolPref("mailnews.downloadToTempFile",
-                               &m_downloadingToTempFile);
-  }
-
   nsCOMPtr<nsIMsgDBHdr> newHdr;
 
   nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(m_popServer);
   if (!server) return NS_ERROR_UNEXPECTED;
 
-  if (m_downloadingToTempFile) {
-    // need to create an nsIOFileStream from a temp file...
-    nsCOMPtr<nsIFile> tmpDownloadFile;
-    rv = GetSpecialDirectoryWithFileName(NS_OS_TEMP_DIR, "newmsg",
-                                         getter_AddRefs(tmpDownloadFile));
-
-    NS_ASSERTION(NS_SUCCEEDED(rv),
-                 "writing tmp pop3 download file: failed to append filename");
-    if (NS_FAILED(rv)) return rv;
-
-    if (!m_tmpDownloadFile) {
-      // need a unique tmp file to prevent dataloss in multiuser environment
-      rv = tmpDownloadFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 00600);
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      m_tmpDownloadFile = tmpDownloadFile;
-    }
-    rv = MsgGetFileStream(m_tmpDownloadFile, getter_AddRefs(m_outFileStream));
-    NS_ENSURE_SUCCESS(rv, rv);
-  } else {
-    rv = server->GetMsgStore(getter_AddRefs(m_msgStore));
-    bool reusable;
-    NS_ENSURE_SUCCESS(rv, rv);
-    m_msgStore->GetNewMsgOutputStream(m_folder, getter_AddRefs(newHdr),
-                                      &reusable,
-                                      getter_AddRefs(m_outFileStream));
-  }
-  // The following (!m_outFileStream etc) was added to make sure that we don't
-  // write somewhere where for some reason or another we can't write to and
-  // lose the messages. See bug 62480
-  if (!m_outFileStream) return NS_ERROR_OUT_OF_MEMORY;
-
-  nsCOMPtr<nsISeekableStream> seekableOutStream =
-      do_QueryInterface(m_outFileStream);
+  rv = server->GetMsgStore(getter_AddRefs(m_msgStore));
+  bool reusable;
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = m_msgStore->GetNewMsgOutputStream(m_folder, getter_AddRefs(newHdr),
+                                         &reusable,
+                                         getter_AddRefs(m_outFileStream));
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // create a new mail parser
   if (!m_newMailParser) m_newMailParser = new nsParseNewMailState;
@@ -459,43 +407,6 @@ nsPop3Sink::IncorporateBegin(const char* uidlString, nsIURI* aURL,
     rv = NS_OK;
   }
 
-  if (closure) *closure = (void*)this;
-
-#ifdef DEBUG
-  // Debugging, see bug 1116055.
-  int64_t first_pre_seek_pos;
-  nsresult rv3 = seekableOutStream->Tell(&first_pre_seek_pos);
-#endif
-
-  // XXX Handle error such as network error for remote file system.
-  seekableOutStream->Seek(nsISeekableStream::NS_SEEK_END, 0);
-
-#ifdef DEBUG
-  // Debugging, see bug 1116055.
-  int64_t first_post_seek_pos;
-  nsresult rv4 = seekableOutStream->Tell(&first_post_seek_pos);
-  if (NS_SUCCEEDED(rv3) && NS_SUCCEEDED(rv4)) {
-    if (first_pre_seek_pos != first_post_seek_pos) {
-      nsString folderName;
-      if (m_folder) m_folder->GetPrettyName(folderName);
-      if (!folderName.IsEmpty()) {
-        fprintf(stderr,
-                "(seekdebug) Seek was necessary in IncorporateBegin() for "
-                "folder %s.\n",
-                NS_ConvertUTF16toUTF8(folderName).get());
-      } else {
-        fprintf(stderr,
-                "(seekdebug) Seek was necessary in IncorporateBegin().\n");
-      }
-      fprintf(stderr,
-              "(seekdebug) first_pre_seek_pos = 0x%016llx, "
-              "first_post_seek_pos=0x%016llx\n",
-              (unsigned long long)first_pre_seek_pos,
-              (unsigned long long)first_post_seek_pos);
-    }
-  }
-#endif
-
   nsCString outputString(GetDummyEnvelope());
   rv = WriteLineToMailbox(outputString);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -524,12 +435,11 @@ nsPop3Sink::IncorporateBegin(const char* uidlString, nsIURI* aURL,
   PR_smprintf_free(statusLine);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = WriteLineToMailbox(
-      NS_LITERAL_CSTRING("X-Mozilla-Status2: 00000000" MSG_LINEBREAK));
+  rv = WriteLineToMailbox("X-Mozilla-Status2: 00000000"_ns MSG_LINEBREAK);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // leave space for 60 bytes worth of keys/tags
-  rv = WriteLineToMailbox(NS_LITERAL_CSTRING(X_MOZILLA_KEYWORDS));
+  rv = WriteLineToMailbox(nsLiteralCString(X_MOZILLA_KEYWORDS));
   return NS_OK;
 }
 
@@ -621,103 +531,69 @@ nsresult nsPop3Sink::WriteLineToMailbox(const nsACString& buffer) {
     nsCOMPtr<nsISeekableStream> seekableOutStream =
         do_QueryInterface(m_outFileStream);
 
-    int64_t before_seek_pos;
-    nsresult rv2 = seekableOutStream->Tell(&before_seek_pos);
-    MOZ_ASSERT(NS_SUCCEEDED(rv2),
-               "seekableOutStream->Tell(&before_seek_pos) failed");
+    if (seekableOutStream) {
+      int64_t before_seek_pos;
+      nsresult rv2 = seekableOutStream->Tell(&before_seek_pos);
+      MOZ_ASSERT(NS_SUCCEEDED(rv2),
+                 "seekableOutStream->Tell(&before_seek_pos) failed");
 
-    // XXX Handle error such as network error for remote file system.
-    seekableOutStream->Seek(nsISeekableStream::NS_SEEK_END, 0);
+      // XXX Handle error such as network error for remote file system.
+      seekableOutStream->Seek(nsISeekableStream::NS_SEEK_END, 0);
 
-    int64_t after_seek_pos;
-    nsresult rv3 = seekableOutStream->Tell(&after_seek_pos);
-    MOZ_ASSERT(NS_SUCCEEDED(rv3),
-               "seekableOutStream->Tell(&after_seek_pos) failed");
+      int64_t after_seek_pos;
+      nsresult rv3 = seekableOutStream->Tell(&after_seek_pos);
+      MOZ_ASSERT(NS_SUCCEEDED(rv3),
+                 "seekableOutStream->Tell(&after_seek_pos) failed");
 
-    if (NS_SUCCEEDED(rv2) && NS_SUCCEEDED(rv3)) {
-      if (before_seek_pos != after_seek_pos) {
-        nsString folderName;
-        if (m_folder) m_folder->GetPrettyName(folderName);
-        // This merits a console message, it's poor man's telemetry.
-        MsgLogToConsole4(
-            NS_LITERAL_STRING("Unexpected file position change detected") +
-                (folderName.IsEmpty() ? EmptyString()
-                                      : NS_LITERAL_STRING(" in folder ")) +
-                (folderName.IsEmpty() ? EmptyString() : folderName) +
-                NS_LITERAL_STRING(
-                    ". "
-                    "If you can reliably reproduce this, please report the "
-                    "steps you used to dev-apps-thunderbird@lists.mozilla.org "
-                    "or to bug 1308335 at bugzilla.mozilla.org. "
-                    "Resolving this problem will allow speeding up message "
-                    "downloads."),
-            NS_LITERAL_STRING(__FILE__), __LINE__, nsIScriptError::errorFlag);
+      if (NS_SUCCEEDED(rv2) && NS_SUCCEEDED(rv3)) {
+        if (before_seek_pos != after_seek_pos) {
+          nsString folderName;
+          if (m_folder) m_folder->GetPrettyName(folderName);
+          // This merits a console message, it's poor man's telemetry.
+          MsgLogToConsole4(
+              u"Unexpected file position change detected"_ns +
+                  (folderName.IsEmpty() ? EmptyString() : u" in folder "_ns) +
+                  (folderName.IsEmpty() ? EmptyString() : folderName) +
+                  u". "
+                  "If you can reliably reproduce this, please report the "
+                  "steps you used to dev-apps-thunderbird@lists.mozilla.org "
+                  "or to bug 1308335 at bugzilla.mozilla.org. "
+                  "Resolving this problem will allow speeding up message "
+                  "downloads."_ns,
+              NS_LITERAL_STRING_FROM_CSTRING(__FILE__), __LINE__,
+              nsIScriptError::errorFlag);
 #  ifdef DEBUG
-        // Debugging, see bug 1116055.
-        if (!folderName.IsEmpty()) {
+          // Debugging, see bug 1116055.
+          if (!folderName.IsEmpty()) {
+            fprintf(stderr,
+                    "(seekdebug) WriteLineToMailbox() detected an unexpected "
+                    "file position change in folder %s.\n",
+                    NS_ConvertUTF16toUTF8(folderName).get());
+          } else {
+            fprintf(stderr,
+                    "(seekdebug) WriteLineToMailbox() detected an unexpected "
+                    "file position change.\n");
+          }
           fprintf(stderr,
-                  "(seekdebug) WriteLineToMailbox() detected an unexpected "
-                  "file position change in folder %s.\n",
-                  NS_ConvertUTF16toUTF8(folderName).get());
-        } else {
-          fprintf(stderr,
-                  "(seekdebug) WriteLineToMailbox() detected an unexpected "
-                  "file position change.\n");
-        }
-        fprintf(
-            stderr,
-            "(seekdebug) before_seek_pos=0x%016llx, after_seek_pos=0x%016llx\n",
-            (long long unsigned int)before_seek_pos,
-            (long long unsigned int)after_seek_pos);
+                  "(seekdebug) before_seek_pos=0x%016llx, "
+                  "after_seek_pos=0x%016llx\n",
+                  (long long unsigned int)before_seek_pos,
+                  (long long unsigned int)after_seek_pos);
 #  endif
+        }
       }
     }
 #endif
 
     uint32_t bytesWritten;
-    m_outFileStream->Write(buffer.BeginReading(), bufferLen, &bytesWritten);
+    nsresult rv =
+        m_outFileStream->Write(buffer.BeginReading(), bufferLen, &bytesWritten);
+    NS_ENSURE_SUCCESS(rv, rv);
     NS_ENSURE_TRUE(bytesWritten == bufferLen, NS_ERROR_FAILURE);
   }
   return NS_OK;
 }
 
-nsresult nsPop3Sink::HandleTempDownloadFailed(nsIMsgWindow* msgWindow) {
-  nsresult rv;
-  nsCOMPtr<nsIStringBundleService> bundleService =
-      mozilla::services::GetStringBundleService();
-  NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
-  nsCOMPtr<nsIStringBundle> bundle;
-  rv = bundleService->CreateBundle(
-      "chrome://messenger/locale/localMsgs.properties", getter_AddRefs(bundle));
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsString fromStr, subjectStr, confirmString;
-
-  m_newMailParser->m_newMsgHdr->GetMime2DecodedSubject(subjectStr);
-  m_newMailParser->m_newMsgHdr->GetMime2DecodedAuthor(fromStr);
-  AutoTArray<nsString, 2> params = {fromStr, subjectStr};
-  bundle->FormatStringFromName("pop3TmpDownloadError", params, confirmString);
-  nsCOMPtr<mozIDOMWindowProxy> parentWindow;
-  nsCOMPtr<nsIPromptService> promptService =
-      do_GetService(NS_PROMPTSERVICE_CONTRACTID);
-  nsCOMPtr<nsIDocShell> docShell;
-  if (msgWindow) {
-    (void)msgWindow->GetRootDocShell(getter_AddRefs(docShell));
-    parentWindow = do_QueryInterface(docShell);
-  }
-  if (promptService && !confirmString.IsEmpty()) {
-    int32_t dlgResult = -1;
-    bool dummyValue = false;
-    rv = promptService->ConfirmEx(parentWindow, nullptr, confirmString.get(),
-                                  nsIPromptService::STD_YES_NO_BUTTONS, nullptr,
-                                  nullptr, nullptr, nullptr, &dummyValue,
-                                  &dlgResult);
-    m_newMailParser->m_newMsgHdr = nullptr;
-
-    return (dlgResult == 0) ? NS_OK : NS_MSG_ERROR_COPYING_FROM_TMP_DOWNLOAD;
-  }
-  return rv;
-}
-
 NS_IMETHODIMP
 nsPop3Sink::IncorporateComplete(nsIMsgWindow* aMsgWindow, int32_t aSize) {
   if (m_buildMessageUri && !m_baseMessageUri.IsEmpty() && m_newMailParser &&
@@ -725,10 +601,10 @@ nsPop3Sink::IncorporateComplete(nsIMsgWindow* aMsgWindow, int32_t aSize) {
     nsMsgKey msgKey;
     m_newMailParser->m_newMsgHdr->GetMessageKey(&msgKey);
     m_messageUri.Truncate();
-    nsBuildLocalMessageURI(m_baseMessageUri.get(), msgKey, m_messageUri);
+    nsBuildLocalMessageURI(m_baseMessageUri, msgKey, m_messageUri);
   }
 
-  nsresult rv = WriteLineToMailbox(NS_LITERAL_CSTRING(MSG_LINEBREAK));
+  nsresult rv = WriteLineToMailbox(nsLiteralCString(MSG_LINEBREAK));
   NS_ENSURE_SUCCESS(rv, rv);
   bool leaveOnServer = false;
   m_popServer->GetLeaveMessagesOnServer(&leaveOnServer);
@@ -758,56 +634,12 @@ nsPop3Sink::IncorporateComplete(nsIMsgWindow* aMsgWindow, int32_t aSize) {
     // header from the old.
     if (!m_origMessageUri.IsEmpty() && localFolder) {
       nsCOMPtr<nsIMsgDBHdr> oldMsgHdr;
-      rv =
-          GetMsgDBHdrFromURI(m_origMessageUri.get(), getter_AddRefs(oldMsgHdr));
+      rv = GetMsgDBHdrFromURI(m_origMessageUri, getter_AddRefs(oldMsgHdr));
       if (NS_SUCCEEDED(rv) && oldMsgHdr)
         localFolder->UpdateNewMsgHdr(oldMsgHdr, hdr);
     }
-
-    if (m_downloadingToTempFile) {
-      // close file to give virus checkers a chance to do their thing...
-      m_outFileStream->Flush();
-      m_outFileStream->Close();
-      m_newMailParser->FinishHeader();
-      // need to re-open the inbox file stream.
-      bool exists;
-      m_tmpDownloadFile->Exists(&exists);
-      if (!exists) return HandleTempDownloadFailed(aMsgWindow);
-
-      nsCOMPtr<nsIInputStream> inboxInputStream =
-          do_QueryInterface(m_outFileStream);
-      rv = MsgReopenFileStream(m_tmpDownloadFile, inboxInputStream);
-      NS_ENSURE_SUCCESS(rv, HandleTempDownloadFailed(aMsgWindow));
-      if (m_outFileStream) {
-        int64_t tmpDownloadFileSize;
-        uint32_t msgSize;
-        hdr->GetMessageSize(&msgSize);
-        // we need to clone because nsLocalFileUnix caches its stat result,
-        // so it doesn't realize the file has changed size.
-        nsCOMPtr<nsIFile> tmpClone;
-        rv = m_tmpDownloadFile->Clone(getter_AddRefs(tmpClone));
-        NS_ENSURE_SUCCESS(rv, rv);
-        tmpClone->GetFileSize(&tmpDownloadFileSize);
-
-        if (msgSize > tmpDownloadFileSize)
-          rv = NS_MSG_ERROR_WRITING_MAIL_FOLDER;
-        else
-          rv = m_newMailParser->AppendMsgFromStream(inboxInputStream, hdr,
-                                                    msgSize, m_folder);
-        if (NS_FAILED(rv)) return HandleTempDownloadFailed(aMsgWindow);
-
-        m_outFileStream->Close();  // close so we can truncate.
-        m_tmpDownloadFile->SetFileSize(0);
-      } else {
-        return HandleTempDownloadFailed(aMsgWindow);
-        // need to give an error here.
-      }
-    } else {
-      m_msgStore->FinishNewMessage(m_outFileStream, hdr);
-    }
+    m_msgStore->FinishNewMessage(m_outFileStream, hdr);
     m_newMailParser->PublishMsgHeader(aMsgWindow);
-    // run any reply/forward filter after we've finished with the
-    // temp quarantine file, and/or moved the message to another folder.
     m_newMailParser->ApplyForwardAndReplyFilter(aMsgWindow);
     if (aSize) hdr->SetUint32Property("onlineSize", aSize);
 
@@ -829,10 +661,10 @@ nsPop3Sink::IncorporateComplete(nsIMsgWindow* aMsgWindow, int32_t aSize) {
 
 NS_IMETHODIMP
 nsPop3Sink::IncorporateAbort(bool uidlDownload) {
+  NS_ENSURE_STATE(m_outFileStream);
   nsresult rv = m_outFileStream->Close();
   NS_ENSURE_SUCCESS(rv, rv);
-  if (!m_downloadingToTempFile && m_msgStore && m_newMailParser &&
-      m_newMailParser->m_newMsgHdr) {
+  if (m_msgStore && m_newMailParser && m_newMailParser->m_newMsgHdr) {
     m_msgStore->DiscardNewMessage(m_outFileStream,
                                   m_newMailParser->m_newMsgHdr);
   }
@@ -880,31 +712,27 @@ nsPop3Sink::SetBuildMessageUri(bool bVal) {
 }
 
 NS_IMETHODIMP
-nsPop3Sink::GetMessageUri(char** messageUri) {
-  NS_ENSURE_ARG_POINTER(messageUri);
+nsPop3Sink::GetMessageUri(nsACString& messageUri) {
   NS_ENSURE_TRUE(!m_messageUri.IsEmpty(), NS_ERROR_FAILURE);
-  *messageUri = ToNewCString(m_messageUri);
+  messageUri = m_messageUri;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Sink::SetMessageUri(const char* messageUri) {
-  NS_ENSURE_ARG_POINTER(messageUri);
+nsPop3Sink::SetMessageUri(const nsACString& messageUri) {
   m_messageUri = messageUri;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Sink::GetBaseMessageUri(char** baseMessageUri) {
-  NS_ENSURE_ARG_POINTER(baseMessageUri);
+nsPop3Sink::GetBaseMessageUri(nsACString& baseMessageUri) {
   NS_ENSURE_TRUE(!m_baseMessageUri.IsEmpty(), NS_ERROR_FAILURE);
-  *baseMessageUri = ToNewCString(m_baseMessageUri);
+  baseMessageUri = m_baseMessageUri;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Sink::SetBaseMessageUri(const char* baseMessageUri) {
-  NS_ENSURE_ARG_POINTER(baseMessageUri);
+nsPop3Sink::SetBaseMessageUri(const nsACString& baseMessageUri) {
   m_baseMessageUri = baseMessageUri;
   return NS_OK;
 }
