@@ -1,13 +1,23 @@
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource:///modules/mailServices.js");
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://testing-common/mailnews/mailTestUtils.js");
-ChromeUtils.import("resource://testing-common/mailnews/localAccountUtils.js");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
+var { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
+var { mailTestUtils } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MailTestUtils.jsm"
+);
+var { localAccountUtils } = ChromeUtils.import(
+  "resource://testing-common/mailnews/LocalAccountUtils.jsm"
+);
 
-var CC = Components.Constructor;
+var test = null;
 
 // WebApps.jsm called by ProxyAutoConfig (PAC) requires a valid nsIXULAppInfo.
-ChromeUtils.import("resource://testing-common/AppInfo.jsm");
+var { getAppInfo, newAppInfo, updateAppInfo } = ChromeUtils.import(
+  "resource://testing-common/AppInfo.jsm"
+);
 updateAppInfo();
 
 // Ensure the profile directory is set up
@@ -16,11 +26,26 @@ do_get_profile();
 var gDEPTH = "../../../../";
 
 // Import the pop3 server scripts
-ChromeUtils.import("resource://testing-common/mailnews/maild.js");
-ChromeUtils.import("resource://testing-common/mailnews/auth.js");
-ChromeUtils.import("resource://testing-common/mailnews/pop3d.js");
-
-ChromeUtils.import("resource:///modules/mailServices.js");
+/* import-globals-from ../../../test/fakeserver/Maild.jsm */
+/* import-globals-from ../../../test/fakeserver/Auth.jsm */
+/* import-globals-from ../../../test/fakeserver/Pop3d.jsm */
+var {
+  nsMailServer,
+  gThreadManager,
+  fsDebugNone,
+  fsDebugAll,
+  fsDebugRecv,
+  fsDebugRecvSend,
+} = ChromeUtils.import("resource://testing-common/mailnews/Maild.jsm");
+var { AuthPLAIN, AuthLOGIN, AuthCRAM } = ChromeUtils.import(
+  "resource://testing-common/mailnews/Auth.jsm"
+);
+var {
+  pop3Daemon,
+  POP3_RFC1939_handler,
+  POP3_RFC2449_handler,
+  POP3_RFC5034_handler,
+} = ChromeUtils.import("resource://testing-common/mailnews/Pop3d.jsm");
 
 // Setup the daemon and server
 // If the debugOption is set, then it will be applied to the server.
@@ -35,44 +60,50 @@ function setupServerDaemon(debugOption) {
     return handler;
   }
   var server = new nsMailServer(createHandler, daemon);
-  if (debugOption)
+  if (debugOption) {
     server.setDebugLevel(debugOption);
+  }
   return [daemon, server, extraProps];
 }
 
-function createPop3ServerAndLocalFolders(port, hostname="localhost") {
+function createPop3ServerAndLocalFolders(port, hostname = "localhost") {
   localAccountUtils.loadLocalMailAccount();
-  let server = localAccountUtils.create_incoming_server("pop3", port,
-    "fred", "wilma", hostname);
+  let server = localAccountUtils.create_incoming_server(
+    "pop3",
+    port,
+    "fred",
+    "wilma",
+    hostname
+  );
   return server;
 }
 
-var gCopyListener =
-{
+var gCopyListener = {
   callbackFunction: null,
   copiedMessageHeaderKeys: [],
-  OnStartCopy: function() {},
-  OnProgress: function(aProgress, aProgressMax) {},
-  SetMessageKey: function(aKey) {
+  OnStartCopy() {},
+  OnProgress(aProgress, aProgressMax) {},
+  SetMessageKey(aKey) {
     try {
       this.copiedMessageHeaderKeys.push(aKey);
     } catch (ex) {
       dump(ex);
     }
   },
-  GetMessageId: function(aMessageId) {},
-  OnStopCopy: function(aStatus) {
+  GetMessageId(aMessageId) {},
+  OnStopCopy(aStatus) {
     if (this.callbackFunction) {
-      mailTestUtils.do_timeout_function(0, this.callbackFunction,
-					null,
-					[ this.copiedMessageHeaderKeys, aStatus ]);
+      mailTestUtils.do_timeout_function(0, this.callbackFunction, null, [
+        this.copiedMessageHeaderKeys,
+        aStatus,
+      ]);
     }
-  }
+  },
 };
 
 /**
  * copyFileMessageInLocalFolder
- * A utility wrapper of nsIMsgCopyService.CopyFileMessage to copy a message
+ * A utility wrapper of nsIMsgCopyService.copyFileMessage to copy a message
  * into local inbox folder.
  *
  * @param aMessageFile     An instance of nsIFile to copy.
@@ -84,36 +115,48 @@ var gCopyListener =
  * @param aCallback        Callback function which will be invoked after
  *                         message is copied
  */
-function copyFileMessageInLocalFolder(aMessageFile,
-                                      aMessageFlags,
-                                      aMessageKeywords,
-                                      aMessageWindow,
-                                      aCallback) {
+function copyFileMessageInLocalFolder(
+  aMessageFile,
+  aMessageFlags,
+  aMessageKeywords,
+  aMessageWindow,
+  aCallback
+) {
   // Set up local folders
   localAccountUtils.loadLocalMailAccount();
 
   gCopyListener.callbackFunction = aCallback;
   // Copy a message into the local folder
-  MailServices.copy.CopyFileMessage(aMessageFile,
-                                    localAccountUtils.inboxFolder,
-                                    null, false,
-                                    aMessageFlags,
-                                    aMessageKeywords,
-                                    gCopyListener,
-                                    aMessageWindow);
+  MailServices.copy.copyFileMessage(
+    aMessageFile,
+    localAccountUtils.inboxFolder,
+    null,
+    false,
+    aMessageFlags,
+    aMessageKeywords,
+    gCopyListener,
+    aMessageWindow
+  );
 }
 
 function do_check_transaction(real, expected) {
   // If we don't spin the event loop before starting the next test, the readers
   // aren't expired. In this case, the "real" real transaction is the last one.
-  if (real instanceof Array)
+  if (Array.isArray(real)) {
     real = real[real.length - 1];
+  }
 
   // real.them may have an extra QUIT on the end, where the stream is only
   // closed after we have a chance to process it and not them. We therefore
   // excise this from the list
-  if (real.them[real.them.length-1] == "QUIT")
+  if (real.them[real.them.length - 1] == "QUIT") {
     real.them.pop();
+  }
+
+  if (expected[0] == "AUTH") {
+    // We don't send initial AUTH command now.
+    expected = expected.slice(1);
+  }
 
   Assert.equal(real.them.join(","), expected.join(","));
   dump("Passed test " + test + "\n");
@@ -134,8 +177,9 @@ function create_sub_folders(parent, subFolders) {
     let subFolder = parent.clone();
     subFolder.append(subFolders[folder].name);
     subFolder.create(Ci.nsIFile.NORMAL_FILE_TYPE, parseInt("0600", 8));
-    if (subFolders[folder].subFolders)
+    if (subFolders[folder].subFolders) {
       create_sub_folders(subFolder, subFolders[folder].subFolders);
+    }
   }
 }
 
@@ -143,8 +187,9 @@ function create_mail_directory(subFolders) {
   let root = create_temporary_directory();
 
   for (let folder in subFolders) {
-    if (!subFolders[folder].subFolders)
+    if (!subFolders[folder].subFolders) {
       continue;
+    }
     let directory = root.clone();
     directory.append(subFolders[folder].name);
     create_sub_folders(directory, subFolders[folder].subFolders);
@@ -154,11 +199,12 @@ function create_mail_directory(subFolders) {
 }
 
 function setup_mailbox(type, mailboxPath) {
-  let user = Cc["@mozilla.org/uuid-generator;1"]
-               .getService(Ci.nsIUUIDGenerator)
-               .generateUUID().toString();
-  let incomingServer =
-    MailServices.accounts.createIncomingServer(user, "Local Folder", type);
+  let user = Services.uuid.generateUUID().toString();
+  let incomingServer = MailServices.accounts.createIncomingServer(
+    user,
+    "Local Folder",
+    type
+  );
   incomingServer.localPath = mailboxPath;
 
   return incomingServer.rootFolder;
@@ -167,3 +213,8 @@ function setup_mailbox(type, mailboxPath) {
 registerCleanupFunction(function() {
   load(gDEPTH + "mailnews/resources/mailShutdown.js");
 });
+
+// Somehow profile-after-change is not triggered in xpcshell tests, here we
+// manually run the getService, so that correct pop3 modules are loaded
+// according to the pref values.
+Cc["@mozilla.org/messenger/pop3-module-loader;1"].getService();
