@@ -30,12 +30,11 @@ function verifyContentLength(aMessageHeaderKeys, aStatus) {
   let messenger = Cc["@mozilla.org/messenger;1"].createInstance(
     Ci.nsIMessenger
   );
-  let neckoURL = {};
   let messageService = messenger.messageServiceFromURI(messageUri);
-  messageService.GetUrlForUri(messageUri, neckoURL, null);
+  let neckoURL = messageService.getUrlForUri(messageUri);
   // Don't use the necko URL directly. Instead, get the spec and create a new
   // URL using the IO service
-  let urlToRun = Services.io.newURI(neckoURL.value.spec);
+  let urlToRun = Services.io.newURI(neckoURL.spec);
 
   // Get a channel from this URI, and check its content length
   let channel = Services.io.newChannelFromURI(
@@ -43,19 +42,19 @@ function verifyContentLength(aMessageHeaderKeys, aStatus) {
     null,
     Services.scriptSecurityManager.getSystemPrincipal(),
     null,
-    Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+    Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
     Ci.nsIContentPolicy.TYPE_OTHER
   );
   Assert.equal(channel.contentLength, gFile.fileSize);
 
   // Now try an attachment. &part=1.2
-  let attachmentURL = Services.io.newURI(neckoURL.value.spec + "&part=1.2");
+  let attachmentURL = Services.io.newURI(neckoURL.spec + "&part=1.2");
   Services.io.newChannelFromURI(
     attachmentURL,
     null,
     Services.scriptSecurityManager.getSystemPrincipal(),
     null,
-    Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+    Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
     Ci.nsIContentPolicy.TYPE_OTHER
   );
   // Currently attachments have their content length set to the length of the
