@@ -36,12 +36,14 @@
 #include "nsThreadUtils.h"
 #include "nsMsgUtils.h"
 #include "nsNetUtil.h"
+#include "mozilla/dom/Promise.h"
 #include "mozilla/ReentrantMonitor.h"
 #include "mozilla/Services.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "nsEmbedCID.h"
 #include "mozilla/Logging.h"
+#include "mozilla/SpinEventLoopUntil.h"
+
+using namespace mozilla::dom;
 
 extern mozilla::LazyLogModule MAPI;  // defined in msgMapiImp.cpp
 
@@ -95,7 +97,10 @@ class MAPISendListener : public nsIMsgSendListener,
   }
 
   /* void OnGetDraftFolderURI (); */
-  NS_IMETHOD OnGetDraftFolderURI(const char* aFolderURI) { return NS_OK; }
+  NS_IMETHOD OnGetDraftFolderURI(const char* aMsgID,
+                                 const nsACString& aFolderURI) {
+    return NS_OK;
+  }
 
   bool IsDone() { return m_done; }
 
@@ -179,10 +184,9 @@ bool nsMapiHook::DisplayLoginDialog(bool aLogin, char16_t** aUsername,
       rv = bundle->GetStringFromName("loginTextwithName", loginText);
       if (NS_FAILED(rv) || loginText.IsEmpty()) return false;
 
-      bool dummyValue = false;
-      rv = dlgService->PromptUsernameAndPassword(
-          nullptr, loginTitle.get(), loginText.get(), aUsername, aPassword,
-          nullptr, &dummyValue, &btnResult);
+      rv = dlgService->PromptUsernameAndPassword(nullptr, loginTitle.get(),
+                                                 loginText.get(), aUsername,
+                                                 aPassword, &btnResult);
     } else {
       // nsString loginString;
       nsString loginText;
@@ -191,10 +195,8 @@ bool nsMapiHook::DisplayLoginDialog(bool aLogin, char16_t** aUsername,
           bundle->FormatStringFromName("loginText", userNameStrings, loginText);
       if (NS_FAILED(rv)) return false;
 
-      bool dummyValue = false;
       rv = dlgService->PromptPassword(nullptr, loginTitle.get(),
-                                      loginText.get(), aPassword, nullptr,
-                                      &dummyValue, &btnResult);
+                                      loginText.get(), aPassword, &btnResult);
     }
   }
 
@@ -335,22 +337,20 @@ nsresult nsMapiHook::BlindSendMail(unsigned long aSession,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // If we're in offline mode, we'll need to queue it for later.
+  RefPtr<Promise> promise;
   rv = pMsgCompose->SendMsg(WeAreOffline() ? nsIMsgSend::nsMsgQueueForLater
                                            : nsIMsgSend::nsMsgDeliverNow,
-                            pMsgId, nullptr, nullptr, nullptr);
+                            pMsgId, nullptr, nullptr, nullptr,
+                            getter_AddRefs(promise));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // We need to wait to make sure that we only return when the send is
-  // completed. If we're offline, we're not sending yet, so don't bother
-  // waiting.
+  // When offline, the message is saved to Outbox and OnStopSending won't be
+  // called.
   if (WeAreOffline()) return NS_OK;
 
-  nsCOMPtr<nsIThread> thread(do_GetCurrentThread());
-  while (!sendListener->IsDone()) {
-    mozilla::ReentrantMonitorAutoEnter mon(*sendListener);
-    sendListener->Wait(PR_MicrosecondsToInterval(1000UL));
-    NS_ProcessPendingEvents(thread);
-  }
+  // Wait for OnStopSending to be called.
+  mozilla::SpinEventLoopUntil("nsIMsgCompose::SendMsg is async"_ns,
+                              [=]() { return sendListener->IsDone(); });
 
   return rv;
 }
@@ -385,8 +385,7 @@ nsresult nsMapiHook::HandleAttachments(nsIMsgCompFields* aCompFields,
           ("nsMapiHook::HandleAttachments: filename: %s path: %s exists = %s\n",
            (const char*)aFiles[i].lpszFileName,
            (const char*)aFiles[i].lpszPathName, bExist ? "true" : "false"));
-      if (NS_FAILED(rv) || (!bExist))
-        return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
+      if (NS_FAILED(rv) || (!bExist)) return NS_ERROR_FILE_NOT_FOUND;
 
       // Temp Directory
       nsCOMPtr<nsIFile> pTempDir;
@@ -394,7 +393,7 @@ nsresult nsMapiHook::HandleAttachments(nsIMsgCompFields* aCompFields,
 
       // create a new sub directory called moz_mapi underneath the temp
       // directory
-      pTempDir->AppendRelativePath(NS_LITERAL_STRING("moz_mapi"));
+      pTempDir->AppendRelativePath(u"moz_mapi"_ns);
       pTempDir->Exists(&bExist);
       if (!bExist) {
         rv = pTempDir->Create(nsIFile::DIRECTORY_TYPE, 777);
@@ -466,7 +465,7 @@ nsresult nsMapiHook::HandleAttachments(nsIMsgCompFields* aCompFields,
       if (NS_FAILED(rv))
         MOZ_LOG(
             MAPI, mozilla::LogLevel::Debug,
-            ("nsMapiHook::HandleAttachments: AddAttachment rv =  %lx\n", rv));
+            ("nsMapiHook::HandleAttachments: AddAttachment rv =  %x\n", rv));
     }
   }
   return rv;
@@ -500,8 +499,7 @@ nsresult nsMapiHook::HandleAttachmentsW(nsIMsgCompFields* aCompFields,
                NS_ConvertUTF16toUTF8(aFiles[i].lpszFileName).get(),
                NS_ConvertUTF16toUTF8(aFiles[i].lpszPathName).get(),
                bExist ? "true" : "false"));
-      if (NS_FAILED(rv) || (!bExist))
-        return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
+      if (NS_FAILED(rv) || (!bExist)) return NS_ERROR_FILE_NOT_FOUND;
 
       // Temp Directory.
       nsCOMPtr<nsIFile> pTempDir;
@@ -509,7 +507,7 @@ nsresult nsMapiHook::HandleAttachmentsW(nsIMsgCompFields* aCompFields,
 
       // Create a new sub directory called moz_mapi underneath the temp
       // directory.
-      pTempDir->AppendRelativePath(NS_LITERAL_STRING("moz_mapi"));
+      pTempDir->AppendRelativePath(u"moz_mapi"_ns);
       pTempDir->Exists(&bExist);
       if (!bExist) {
         rv = pTempDir->Create(nsIFile::DIRECTORY_TYPE, 777);
@@ -575,7 +573,7 @@ nsresult nsMapiHook::HandleAttachmentsW(nsIMsgCompFields* aCompFields,
       if (NS_FAILED(rv))
         MOZ_LOG(
             MAPI, mozilla::LogLevel::Debug,
-            ("nsMapiHook::HandleAttachmentsW: AddAttachment rv =  %lx\n", rv));
+            ("nsMapiHook::HandleAttachmentsW: AddAttachment rv =  %x\n", rv));
     }
   }
   return rv;
@@ -598,7 +596,7 @@ nsresult nsMapiHook::PopulateCompFieldsWithConversion(
   nsAutoString To;
   nsAutoString Cc;
   nsAutoString Bcc;
-  NS_NAMED_LITERAL_STRING(Comma, ",");
+  constexpr auto Comma = u","_ns;
   if (aMessage->lpRecips) {
     for (int i = 0; i < (int)aMessage->nRecipCount; i++) {
       if (aMessage->lpRecips[i].lpszAddress || aMessage->lpRecips[i].lpszName) {
@@ -708,7 +706,7 @@ nsresult nsMapiHook::PopulateCompFieldsW(lpnsMapiMessageW aMessage,
   nsAutoString Cc;
   nsAutoString Bcc;
 
-  NS_NAMED_LITERAL_STRING(Comma, ",");
+  constexpr auto Comma = u","_ns;
 
   if (aMessage->lpRecips) {
     for (int i = 0; i < (int)aMessage->nRecipCount; i++) {
@@ -818,7 +816,7 @@ nsresult nsMapiHook::PopulateCompFieldsForSendDocs(
 
       // if not already existing, create another temp dir for mapi within Win
       // temp dir this is windows only so we can do "\\"
-      pTempDir->AppendRelativePath(NS_LITERAL_STRING("moz_mapi"));
+      pTempDir->AppendRelativePath(u"moz_mapi"_ns);
       pTempDir->Exists(&bExist);
       if (!bExist) {
         rv = pTempDir->Create(nsIFile::DIRECTORY_TYPE, 777);
@@ -850,8 +848,7 @@ nsresult nsMapiHook::PopulateCompFieldsForSendDocs(
       pFile->InitWithNativePath(RemainingPaths);
 
       rv = pFile->Exists(&bExist);
-      if (NS_FAILED(rv) || (!bExist))
-        return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
+      if (NS_FAILED(rv) || (!bExist)) return NS_ERROR_FILE_NOT_FOUND;
 
       // filename of the file attachment
       nsAutoString leafName;
