@@ -48,6 +48,7 @@
 #include "nsIMsgAccountManager.h"
 #include "nsMsgBaseCID.h"
 #include "modmimee.h"  // for MimeConverterOutputCallback
+#include "mozilla/dom/Promise.h"
 #include "mozilla/mailnews/MimeHeaderParser.h"
 
 using namespace mozilla::mailnews;
@@ -90,7 +91,7 @@ mime_draft_data::mime_draft_data()
       forwardInline(false),
       forwardInlineFilter(false),
       overrideComposeFormat(false),
-      originalMsgURI(nullptr) {}
+      autodetectCharset(false) {}
 ////////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////////
 // THIS SHOULD ALL MOVE TO ANOTHER FILE AFTER LANDING!
@@ -166,7 +167,7 @@ nsresult CreateComposeParams(nsCOMPtr<nsIMsgComposeParams>& pMsgComposeParams,
                              MSG_ComposeType composeType,
                              MSG_ComposeFormat composeFormat,
                              nsIMsgIdentity* identity,
-                             const char* originalMsgURI,
+                             const nsACString& originalMsgURI,
                              nsIMsgDBHdr* origMsgHdr) {
 #ifdef NS_DEBUG
   mime_dump_attachments(attachmentList);
@@ -184,8 +185,8 @@ nsresult CreateComposeParams(nsCOMPtr<nsIMsgComposeParams>& pMsgComposeParams,
             do_CreateInstance(NS_MSGATTACHMENT_CONTRACTID, &rv);
         if (NS_SUCCEEDED(rv) && attachment) {
           nsAutoString nameStr;
-          rv = nsMsgI18NConvertToUnicode(NS_LITERAL_CSTRING("UTF-8"),
-                                         curAttachment->m_realName, nameStr);
+          rv = nsMsgI18NConvertToUnicode("UTF-8"_ns, curAttachment->m_realName,
+                                         nameStr);
           if (NS_FAILED(rv))
             CopyASCIItoUTF16(curAttachment->m_realName, nameStr);
           attachment->SetName(nameStr);
@@ -198,14 +199,21 @@ nsresult CreateComposeParams(nsCOMPtr<nsIMsgComposeParams>& pMsgComposeParams,
           if (!curAttachment->m_cloudPartInfo.IsEmpty()) {
             nsCString provider;
             nsCString cloudUrl;
-            attachment->SetSendViaCloud(true);
+            nsCString cloudPartHeaderData;
+
             provider.Adopt(
                 MimeHeaders_get_parameter(curAttachment->m_cloudPartInfo.get(),
                                           "provider", nullptr, nullptr));
             cloudUrl.Adopt(MimeHeaders_get_parameter(
                 curAttachment->m_cloudPartInfo.get(), "url", nullptr, nullptr));
+            cloudPartHeaderData.Adopt(
+                MimeHeaders_get_parameter(curAttachment->m_cloudPartInfo.get(),
+                                          "data", nullptr, nullptr));
+
+            attachment->SetSendViaCloud(true);
             attachment->SetCloudFileAccountKey(provider);
             attachment->SetContentLocation(cloudUrl);
+            attachment->SetCloudPartHeaderData(cloudPartHeaderData);
           }
           compFields->AddAttachment(attachment);
         }
@@ -235,7 +243,8 @@ nsresult CreateComposeParams(nsCOMPtr<nsIMsgComposeParams>& pMsgComposeParams,
   pMsgComposeParams->SetFormat(format);
   pMsgComposeParams->SetIdentity(identity);
   pMsgComposeParams->SetComposeFields(compFields);
-  if (originalMsgURI) pMsgComposeParams->SetOriginalMsgURI(originalMsgURI);
+  if (!originalMsgURI.IsEmpty())
+    pMsgComposeParams->SetOriginalMsgURI(originalMsgURI);
   if (origMsgHdr) pMsgComposeParams->SetOrigMsgHdr(origMsgHdr);
   return NS_OK;
 }
@@ -245,7 +254,7 @@ nsresult CreateTheComposeWindow(nsIMsgCompFields* compFields,
                                 MSG_ComposeType composeType,
                                 MSG_ComposeFormat composeFormat,
                                 nsIMsgIdentity* identity,
-                                const char* originalMsgURI,
+                                const nsACString& originalMsgURI,
                                 nsIMsgDBHdr* origMsgHdr) {
   nsCOMPtr<nsIMsgComposeParams> pMsgComposeParams;
   nsresult rv = CreateComposeParams(pMsgComposeParams, compFields,
@@ -264,7 +273,8 @@ nsresult CreateTheComposeWindow(nsIMsgCompFields* compFields,
 nsresult ForwardMsgInline(nsIMsgCompFields* compFields,
                           nsMsgAttachmentData* attachmentList,
                           MSG_ComposeFormat composeFormat,
-                          nsIMsgIdentity* identity, const char* originalMsgURI,
+                          nsIMsgIdentity* identity,
+                          const nsACString& originalMsgURI,
                           nsIMsgDBHdr* origMsgHdr) {
   nsCOMPtr<nsIMsgComposeParams> pMsgComposeParams;
   nsresult rv =
@@ -286,8 +296,9 @@ nsresult ForwardMsgInline(nsIMsgCompFields* compFields,
   rv = pMsgCompose->Initialize(pMsgComposeParams, nullptr, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  RefPtr<mozilla::dom::Promise> promise;
   rv = pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nullptr,
-                            nullptr, nullptr);
+                            nullptr, nullptr, getter_AddRefs(promise));
   if (NS_SUCCEEDED(rv)) {
     nsCOMPtr<nsIMsgFolder> origFolder;
     origMsgHdr->GetFolder(getter_AddRefs(origFolder));
@@ -303,7 +314,8 @@ nsresult CreateCompositionFields(
     const char* bcc, const char* fcc, const char* newsgroups,
     const char* followup_to, const char* organization, const char* subject,
     const char* references, const char* priority, const char* newspost_url,
-    char* charset, nsIMsgCompFields** _retval) {
+    const nsTArray<nsString>& otherHeaders, char* charset,
+    nsIMsgCompFields** _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
 
   nsresult rv;
@@ -314,10 +326,6 @@ nsresult CreateCompositionFields(
   NS_ENSURE_SUCCESS(rv, rv);
   NS_ENSURE_TRUE(cFields, NS_ERROR_OUT_OF_MEMORY);
 
-  // Now set all of the passed in stuff...
-  cFields->SetCharacterSet(!PL_strcasecmp("us-ascii", charset) ? "ISO-8859-1"
-                                                               : charset);
-
   nsAutoCString val;
   nsAutoString outString;
 
@@ -407,8 +415,19 @@ nsresult CreateCompositionFields(
     cFields->SetNewspostUrl(!val.IsEmpty() ? val.get() : newspost_url);
   }
 
+  nsTArray<nsString> cFieldsOtherHeaders;
+  cFields->GetOtherHeaders(cFieldsOtherHeaders);
+  for (auto otherHeader : otherHeaders) {
+    if (!otherHeader.IsEmpty()) {
+      MIME_DecodeMimeHeader(NS_ConvertUTF16toUTF8(otherHeader).get(), charset,
+                            false, true, val);
+      cFieldsOtherHeaders.AppendElement(NS_ConvertUTF8toUTF16(val));
+    } else {
+      cFieldsOtherHeaders.AppendElement(u""_ns);
+    }
+  }
+  cFields->SetOtherHeaders(cFieldsOtherHeaders);
   cFields.forget(_retval);
-
   return rv;
 }
 
@@ -475,7 +494,8 @@ static nsMsgAttachmentData* mime_draft_process_attachments(
     else
       tmpFile = mdd->attachments[attachmentsIndex++];
 
-    if (tmpFile->m_type.LowerCaseEqualsLiteral("text/x-vcard"))
+    if (tmpFile->m_type.LowerCaseEqualsLiteral("text/vcard") ||
+        tmpFile->m_type.LowerCaseEqualsLiteral("text/x-vcard"))
       tmp->m_realName = tmpFile->m_description;
 
     if (tmpFile->m_origUrl) {
@@ -1142,6 +1162,7 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
   char* draftInfo = 0;
   char* contentLanguage = 0;
   char* identityKey = 0;
+  nsTArray<nsString> readOtherHeaders;
 
   bool forward_inline = false;
   bool bodyAsAttachment = false;
@@ -1248,11 +1269,30 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
           news_host = PR_smprintf("news://%s", host);
         }
       }
+
+      // Other headers via pref.
+      nsCString otherHeaders;
+      nsTArray<nsCString> otherHeadersArray;
+      nsresult rv;
+      nsCOMPtr<nsIPrefBranch> pPrefBranch(
+          do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+      pPrefBranch->GetCharPref("mail.compose.other.header", otherHeaders);
+      if (!otherHeaders.IsEmpty()) {
+        ToLowerCase(otherHeaders);
+        ParseString(otherHeaders, ',', otherHeadersArray);
+        for (auto otherHeader : otherHeadersArray) {
+          otherHeader.Trim(" ");
+          nsAutoCString result;
+          result.Assign(
+              MimeHeaders_get(mdd->headers, otherHeader.get(), false, false));
+          readOtherHeaders.AppendElement(NS_ConvertUTF8toUTF16(result));
+        }
+      }
     }
 
     CreateCompositionFields(from, repl, to, cc, bcc, fcc, grps, foll, org, subj,
-                            refs, priority, news_host, mdd->mailcharset,
-                            getter_AddRefs(fields));
+                            refs, priority, news_host, readOtherHeaders,
+                            mdd->mailcharset, getter_AddRefs(fields));
 
     contentLanguage =
         MimeHeaders_get(mdd->headers, HEADER_CONTENT_LANGUAGE, false, false);
@@ -1317,7 +1357,7 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
       PR_FREEIF(parm);
       parm = MimeHeaders_get_parameter(draftInfo, "deliveryformat", NULL, NULL);
       if (parm) {
-        int32_t deliveryFormat = nsIMsgCompSendFormat::AskUser;
+        int32_t deliveryFormat = nsIMsgCompSendFormat::Unset;
         sscanf(parm, "%d", &deliveryFormat);
         fields->SetDeliveryFormat(deliveryFormat);
       }
@@ -1522,9 +1562,9 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
         // Note that always setting the draft ID here would cause drafts to be
         // overwritten when edited "as new", which is undesired.
         if (msgComposeType == nsIMsgCompType::EditTemplate) {
-          fields->SetDraftId(mdd->url_name);
-          fields->SetTemplateId(
-              mdd->url_name);  // Remember original template ID.
+          fields->SetDraftId(nsDependentCString(mdd->url_name));
+          fields->SetTemplateId(nsDependentCString(
+              mdd->url_name));  // Remember original template ID.
         }
 
         if (convertToPlainText) fields->ConvertBodyToPlainText();
@@ -1549,7 +1589,7 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
                                    mdd->origMsgHdr);
         } else {
           if (convertToPlainText) fields->ConvertBodyToPlainText();
-          fields->SetDraftId(mdd->url_name);
+          fields->SetDraftId(nsDependentCString(mdd->url_name));
           CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Draft,
                                  composeFormat, mdd->identity,
                                  mdd->originalMsgURI, mdd->origMsgHdr);
@@ -1569,7 +1609,7 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
 #endif
         CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Template,
                                nsIMsgCompFormat::Default, mdd->identity,
-                               nullptr, mdd->origMsgHdr);
+                               EmptyCString(), mdd->origMsgHdr);
       } else {
 #ifdef NS_DEBUG
         printf("Time to create the composition window WITHOUT a body!!!!\n");
@@ -1583,21 +1623,21 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
                                  mdd->identity, mdd->originalMsgURI,
                                  mdd->origMsgHdr);
         } else {
-          fields->SetDraftId(mdd->url_name);
+          fields->SetDraftId(nsDependentCString(mdd->url_name));
           CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Draft,
                                  nsIMsgCompFormat::Default, mdd->identity,
-                                 nullptr, mdd->origMsgHdr);
+                                 EmptyCString(), mdd->origMsgHdr);
         }
       }
     }
   } else {
     CreateCompositionFields(from, repl, to, cc, bcc, fcc, grps, foll, org, subj,
-                            refs, priority, news_host, mdd->mailcharset,
-                            getter_AddRefs(fields));
+                            refs, priority, news_host, readOtherHeaders,
+                            mdd->mailcharset, getter_AddRefs(fields));
     if (fields)
       CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::New,
-                             nsIMsgCompFormat::Default, mdd->identity, nullptr,
-                             mdd->origMsgHdr);
+                             nsIMsgCompFormat::Default, mdd->identity,
+                             EmptyCString(), mdd->origMsgHdr);
   }
 
   if (mdd->headers) MimeHeaders_free(mdd->headers);
@@ -1621,7 +1661,6 @@ static void mime_parse_stream_complete(nsMIMESession* stream) {
 
   mdd->identity = nullptr;
   PR_Free(mdd->url_name);
-  PR_Free(mdd->originalMsgURI);
   mdd->origMsgHdr = nullptr;
   PR_Free(mdd);
 
@@ -1725,9 +1764,14 @@ int mime_decompose_file_init_fn(void* stream_closure, MimeHeaders* headers) {
   if (!nAttachments && !mdd->messageBody) {
     // if we've been told to use an override charset then do so....otherwise use
     // the charset inside the message header...
-    if (mdd->options && mdd->options->override_charset)
-      mdd->mailcharset = strdup(mdd->options->default_charset);
-    else {
+    if (mdd->options && mdd->options->override_charset) {
+      if (mdd->options->default_charset)
+        mdd->mailcharset = strdup(mdd->options->default_charset);
+      else {
+        mdd->mailcharset = strdup("");
+        mdd->autodetectCharset = true;
+      }
+    } else {
       char* contentType;
       contentType = MimeHeaders_get(headers, HEADER_CONTENT_TYPE, false, false);
       if (contentType) {
@@ -1802,48 +1846,40 @@ int mime_decompose_file_init_fn(void* stream_closure, MimeHeaders* headers) {
   newAttachment->m_cloudPartInfo.Adopt(
       MimeHeaders_get(headers, HEADER_X_MOZILLA_CLOUD_PART, false, false));
 
-  // There's no file in the message if it's a cloud part.
-  if (!newAttachment->m_cloudPartInfo.IsEmpty()) {
-    nsAutoCString fileURL;
-    fileURL.Adopt(MimeHeaders_get_parameter(
-        newAttachment->m_cloudPartInfo.get(), "file", nullptr, nullptr));
-    if (!fileURL.IsEmpty())
-      nsMimeNewURI(getter_AddRefs(newAttachment->m_origUrl), fileURL.get(),
-                   nullptr);
-    mdd->tmpFile = nullptr;
-    return 0;
-  }
-
   nsCOMPtr<nsIFile> tmpFile = nullptr;
   {
     // Let's build a temp file with an extension based on the content-type:
     // nsmail.<extension>
 
     nsAutoCString newAttachName("nsmail");
-    bool extensionAdded = false;
+    nsAutoCString fileExtension;
     // the content type may contain a charset. i.e. text/html; ISO-2022-JP...we
     // want to strip off the charset before we ask the mime service for a mime
     // info for this content type.
     nsAutoCString contentType(newAttachment->m_type);
     int32_t pos = contentType.FindChar(';');
     if (pos > 0) contentType.SetLength(pos);
-    nsresult rv = NS_OK;
-    nsCOMPtr<nsIMIMEService> mimeFinder(
-        do_GetService(NS_MIMESERVICE_CONTRACTID, &rv));
-    if (NS_SUCCEEDED(rv) && mimeFinder) {
-      nsAutoCString fileExtension;
-      rv = mimeFinder->GetPrimaryExtension(contentType, EmptyCString(),
-                                           fileExtension);
-
-      if (NS_SUCCEEDED(rv) && !fileExtension.IsEmpty()) {
-        newAttachName.Append('.');
-        newAttachName.Append(fileExtension);
-        extensionAdded = true;
+    int32_t extLoc = newAttachment->m_realName.RFindChar('.');
+    int32_t specLength = newAttachment->m_realName.Length();
+    // @see nsExternalHelperAppService::GetTypeFromURI()
+    if (extLoc != -1 && extLoc != specLength - 1 &&
+        // nothing over 20 chars long can be sanely considered an
+        // extension.... Dat dere would be just data.
+        specLength - extLoc < 20) {
+      fileExtension = Substring(newAttachment->m_realName, extLoc + 1);
+    } else {
+      nsCOMPtr<nsIMIMEService> mimeFinder(
+          do_GetService(NS_MIMESERVICE_CONTRACTID));
+      if (mimeFinder) {
+        mimeFinder->GetPrimaryExtension(contentType, ""_ns, fileExtension);
       }
     }
 
-    if (!extensionAdded) {
+    if (fileExtension.IsEmpty()) {
       newAttachName.AppendLiteral(".tmp");
+    } else {
+      newAttachName.Append('.');
+      newAttachName.Append(fileExtension);
     }
 
     nsMsgCreateTempFile(newAttachName.get(), getter_AddRefs(tmpFile));
@@ -1924,6 +1960,16 @@ int mime_decompose_file_output_fn(const char* buf, int32_t size,
 
   if (!mdd->tmpFileStream) return 0;
 
+  if (mdd->autodetectCharset) {
+    nsAutoCString detectedCharset;
+    nsresult res = NS_OK;
+    res = MIME_detect_charset(buf, size, detectedCharset);
+    if (NS_SUCCEEDED(res) && !detectedCharset.IsEmpty()) {
+      mdd->mailcharset = ToNewCString(detectedCharset);
+      mdd->autodetectCharset = false;
+    }
+  }
+
   if (mdd->decoder_data) {
     int32_t outsize;
     ret = MimeDecoderWrite(mdd->decoder_data, buf, size, &outsize);
@@ -1991,7 +2037,7 @@ extern "C" void* mime_bridge_create_draft_stream(
   rv = GetMessageServiceFromURI(turl, getter_AddRefs(msgService));
   if (NS_FAILED(rv)) goto FAIL;
 
-  rv = msgService->GetUrlForUri(turl.get(), getter_AddRefs(aURL), nullptr);
+  rv = msgService->GetUrlForUri(turl, nullptr, getter_AddRefs(aURL));
   if (NS_FAILED(rv)) goto FAIL;
 
   if (NS_SUCCEEDED(aURL->GetSpec(urlString))) {
@@ -2009,7 +2055,7 @@ extern "C" void* mime_bridge_create_draft_stream(
   newPluginObj2->GetForwardToAddress(mdd->forwardToAddress);
   newPluginObj2->GetOverrideComposeFormat(&mdd->overrideComposeFormat);
   newPluginObj2->GetIdentity(getter_AddRefs(mdd->identity));
-  newPluginObj2->GetOriginalMsgURI(&mdd->originalMsgURI);
+  newPluginObj2->GetOriginalMsgURI(mdd->originalMsgURI);
   newPluginObj2->GetOrigMsgHdr(getter_AddRefs(mdd->origMsgHdr));
   mdd->format_out = format_out;
   mdd->options = new MimeDisplayOptions;
@@ -2062,7 +2108,6 @@ extern "C" void* mime_bridge_create_draft_stream(
 FAIL:
   if (mdd) {
     PR_Free(mdd->url_name);
-    PR_Free(mdd->originalMsgURI);
     if (mdd->options) delete mdd->options;
     PR_Free(mdd);
   }
