@@ -155,11 +155,6 @@ static int MimeMessage_parse_line(const char* aLine, int32_t aLength,
       // If we are processing a flowed plain text line, we need to parse the
       // line in mimeInlineTextPlainFlowedClass.
       if (mime_typep(kid, (MimeObjectClass*)&mimeInlineTextPlainFlowedClass)) {
-        // Remove any stuffed space.
-        if (length > 0 && ' ' == *line) {
-          line++;
-          length--;
-        }
         return kid->clazz->parse_line(line, length, kid);
       } else {
         status = obj->options->decompose_file_output_fn(
