@@ -9,11 +9,10 @@
 const { localAccountUtils } = ChromeUtils.import(
   "resource://testing-common/mailnews/LocalAccountUtils.jsm"
 );
-var { IOUtils } = ChromeUtils.import("resource:///modules/IOUtils.jsm");
 
 var gTestFiles = ["../../../data/bug505221", "../../../data/bug513543"];
 
-var gMsgEnumerator;
+var gMessages;
 
 var gMessenger = Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger);
 
@@ -26,17 +25,19 @@ var gUrlListener = {
 
 localAccountUtils.loadLocalMailAccount();
 
-function run_test() {
+add_task(async function run_the_test() {
   do_test_pending();
   localAccountUtils.inboxFolder.QueryInterface(Ci.nsIMsgLocalMailFolder);
   for (let fileName of gTestFiles) {
     localAccountUtils.inboxFolder.addMessage(
-      IOUtils.loadFileToString(do_get_file(fileName))
+      await IOUtils.readUTF8(do_get_file(fileName).path)
     );
   }
-  gMsgEnumerator = localAccountUtils.inboxFolder.msgDatabase.EnumerateMessages();
+  gMessages = [
+    ...localAccountUtils.inboxFolder.msgDatabase.EnumerateMessages(),
+  ];
   doNextTest();
-}
+});
 
 function streamMsg(msgHdr) {
   let msgURI = localAccountUtils.inboxFolder.getUriForMsg(msgHdr);
@@ -54,7 +55,7 @@ function streamMsg(msgHdr) {
 }
 
 var gStreamListener = {
-  QueryInterface: ChromeUtils.generateQI([Ci.nsIStreamListener]),
+  QueryInterface: ChromeUtils.generateQI(["nsIStreamListener"]),
   _stream: null,
   // nsIRequestObserver part
   onStartRequest(aRequest) {},
@@ -78,8 +79,9 @@ var gStreamListener = {
 };
 
 function doNextTest() {
-  if (gMsgEnumerator.hasMoreElements()) {
-    streamMsg(gMsgEnumerator.getNext().QueryInterface(Ci.nsIMsgDBHdr));
+  if (gMessages.length > 0) {
+    let msgHdr = gMessages.shift();
+    streamMsg(msgHdr);
   } else {
     do_test_finished();
   }
