@@ -17,7 +17,10 @@ function run_test() {
     },
     {
       displayString: "test@foo.invalid, test2@foo.invalid",
-      addresses: [["", "test@foo.invalid"], ["", "test2@foo.invalid"]],
+      addresses: [
+        ["", "test@foo.invalid"],
+        ["", "test2@foo.invalid"],
+      ],
     },
     {
       displayString: "John Doe <test@foo.invalid>",
@@ -142,8 +145,11 @@ function run_test() {
 
     // A mail list.
     {
-      displayString: "Holmes and Watson <Tenants221B>",
-      addresses: [["Holmes and Watson", "Tenants221B"]],
+      displayString: "Holmes and Watson <Tenants221B>, foo@example.com",
+      addresses: [
+        ["Holmes and Watson", "Tenants221B"],
+        ["", "foo@example.com"],
+      ],
     },
 
     // A mail list with a space in the name.
@@ -180,10 +186,8 @@ function run_test() {
   // Test -  strings
 
   for (let i = 0; i < checks.length; ++i) {
-    dump("Test " + i + "\n");
     let addrs = MailServices.headerParser.makeFromDisplayAddress(
-      checks[i].displayString,
-      {}
+      checks[i].displayString
     );
     let checkaddrs = checks[i].addresses;
     Assert.equal(addrs.length, checkaddrs.length, "Number of parsed addresses");
