@@ -28,6 +28,7 @@
 #include "nsNetUtil.h"
 #include "nsISimpleEnumerator.h"
 #include "nsMsgUtils.h"
+#include "nsITimer.h"
 #include "mozilla/Services.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/ErrorResult.h"
@@ -41,7 +42,6 @@
 #define NEW_NEWS_DIR_NAME "News"
 #define PREF_MAIL_NEWSRC_ROOT "mail.newsrc_root"
 #define PREF_MAIL_NEWSRC_ROOT_REL "mail.newsrc_root-rel"
-#define PREF_MAILNEWS_VIEW_DEFAULT_CHARSET "mailnews.view_default_charset"
 #define HOSTINFO_FILE_NAME "hostinfo.dat"
 
 #define NEWS_DELIMITER '.'
@@ -132,10 +132,10 @@ NS_IMPL_SERVERPREF_BOOL(nsNntpIncomingServer, SingleSignon, "singleSignon")
 NS_IMPL_SERVERPREF_INT(nsNntpIncomingServer, MaxArticles, "max_articles")
 
 nsresult nsNntpIncomingServer::CreateRootFolderFromUri(
-    const nsCString& serverUri, nsIMsgFolder** rootFolder) {
+    const nsACString& serverUri, nsIMsgFolder** rootFolder) {
   nsMsgNewsFolder* newRootFolder = new nsMsgNewsFolder;
   NS_ADDREF(*rootFolder = newRootFolder);
-  newRootFolder->Init(serverUri.get());
+  newRootFolder->Init(serverUri);
   return NS_OK;
 }
 
@@ -240,17 +240,18 @@ nsNntpIncomingServer::GetNewsrcRootPath(nsIFile** aNewsrcRootPath) {
 }
 
 nsresult nsNntpIncomingServer::SetupNewsrcSaveTimer() {
-  int64_t ms(300000);  // hard code, 5 minutes.
-  // Convert biffDelay into milliseconds
-  uint32_t timeInMSUint32 = (uint32_t)ms;
   // Can't currently reset a timer when it's in the process of
   // calling Notify. So, just release the timer here and create a new one.
   if (mNewsrcSaveTimer) mNewsrcSaveTimer->Cancel();
-  mNewsrcSaveTimer = do_CreateInstance("@mozilla.org/timer;1");
-  mNewsrcSaveTimer->InitWithNamedFuncCallback(
-      OnNewsrcSaveTimer, (void*)this, timeInMSUint32,
-      nsITimer::TYPE_REPEATING_SLACK,
-      "nsNntpIncomingServer::OnNewsrcSaveTimer");
+
+  nsresult rv = NS_NewTimerWithFuncCallback(
+      getter_AddRefs(mNewsrcSaveTimer), OnNewsrcSaveTimer, (void*)this,
+      1000 * 5 * 60 /* 5 minutes */, nsITimer::TYPE_REPEATING_SLACK,
+      "nsNntpIncomingServer::OnNewsrcSaveTimer", nullptr);
+  if (NS_FAILED(rv)) {
+    NS_WARNING("Could not start mNewsrcSaveTimer timer");
+  }
+
   return NS_OK;
 }
 
@@ -265,15 +266,10 @@ nsNntpIncomingServer::GetCharset(nsACString& aCharset) {
   nsresult rv = GetCharValue("charset", aCharset);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // if the per-server setting is empty,we get the default charset from
-  // mailnews.view_default_charset setting and set it as per-server preference.
+  // if the per-server setting is empty, default to UTF-8 and set it as
+  // per-server preference.
   if (aCharset.IsEmpty()) {
-    nsString defaultCharset;
-    rv = NS_GetLocalizedUnicharPreferenceWithDefault(
-        nullptr, PREF_MAILNEWS_VIEW_DEFAULT_CHARSET,
-        NS_LITERAL_STRING("ISO-8859-1"), defaultCharset);
-    NS_ENSURE_SUCCESS(rv, rv);
-    LossyCopyUTF16toASCII(defaultCharset, aCharset);
+    aCharset.AssignLiteral("UTF-8");
     SetCharset(aCharset);
   }
   return NS_OK;
@@ -306,7 +302,6 @@ nsNntpIncomingServer::WriteNewsrcFile() {
                                                  newsrcFile, -1, 00600);
     if (NS_FAILED(rv)) return rv;
 
-    nsCOMPtr<nsISimpleEnumerator> subFolders;
     nsCOMPtr<nsIMsgFolder> rootFolder;
     rv = GetRootFolder(getter_AddRefs(rootFolder));
     if (NS_FAILED(rv)) return rv;
@@ -345,25 +340,18 @@ nsNntpIncomingServer::WriteNewsrcFile() {
     }
 #endif /* DEBUG_NEWS */
 
-    rv = rootFolder->GetSubFolders(getter_AddRefs(subFolders));
-    if (NS_FAILED(rv)) return rv;
-
-    bool moreFolders;
-
-    while (NS_SUCCEEDED(subFolders->HasMoreElements(&moreFolders)) &&
-           moreFolders) {
-      nsCOMPtr<nsISupports> child;
-      rv = subFolders->GetNext(getter_AddRefs(child));
-      if (NS_SUCCEEDED(rv) && child) {
-        newsFolder = do_QueryInterface(child, &rv);
-        if (NS_SUCCEEDED(rv) && newsFolder) {
-          nsCString newsrcLine;
-          rv = newsFolder->GetNewsrcLine(newsrcLine);
-          if (NS_SUCCEEDED(rv) && !newsrcLine.IsEmpty()) {
-            // write the line to the newsrc file
-            newsrcStream->Write(newsrcLine.get(), newsrcLine.Length(),
-                                &bytesWritten);
-          }
+    nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+    rv = rootFolder->GetSubFolders(subFolders);
+    NS_ENSURE_SUCCESS(rv, rv);
+    for (nsIMsgFolder* child : subFolders) {
+      newsFolder = do_QueryInterface(child, &rv);
+      if (NS_SUCCEEDED(rv) && newsFolder) {
+        nsCString newsrcLine;
+        rv = newsFolder->GetNewsrcLine(newsrcLine);
+        if (NS_SUCCEEDED(rv) && !newsrcLine.IsEmpty()) {
+          // write the line to the newsrc file
+          newsrcStream->Write(newsrcLine.get(), newsrcLine.Length(),
+                              &bytesWritten);
         }
       }
     }
@@ -485,6 +473,11 @@ nsresult nsNntpIncomingServer::CreateProtocolInstance(
   return rv;
 }
 
+/**
+ * Find an available nsNNTPConnection. Can create new connections as long as
+ * we stay under the maximum connection limit.
+ * If none are available, returns NS_OK and nullptr.
+ */
 nsresult nsNntpIncomingServer::GetNntpConnection(
     nsIURI* aUri, nsIMsgWindow* aMsgWindow, nsINNTPProtocol** aNntpConnection) {
   int32_t maxConnections;
@@ -510,8 +503,8 @@ nsresult nsNntpIncomingServer::GetNntpConnection(
   }
 
   if (connection) {
+    // We've got a connection, ready to go.
     connection.forget(aNntpConnection);
-    (*aNntpConnection)->SetIsCachedConnection(true);
   } else if (cnt < maxConnections) {
     // We have room for another connection. Create this connection and return
     // it to the caller.
@@ -528,6 +521,11 @@ nsresult nsNntpIncomingServer::GetNntpConnection(
   return (*aNntpConnection)->Initialize(aUri, aMsgWindow);
 }
 
+/**
+ * Returns an nsIChannel to run the given URI.
+ * The returned channel might be either an nsNNTPProtocol or nsNntpMockChannel,
+ * representing a real connection or a queued command, respectively.
+ */
 NS_IMETHODIMP
 nsNntpIncomingServer::GetNntpChannel(nsIURI* aURI, nsIMsgWindow* aMsgWindow,
                                      nsIChannel** aChannel) {
@@ -547,6 +545,10 @@ nsNntpIncomingServer::GetNntpChannel(nsIURI* aURI, nsIMsgWindow* aMsgWindow,
   return NS_OK;
 }
 
+/**
+ * Submits a news URI to run. If no connections are free, the command will
+ * be queued to run when one becomes available.
+ */
 NS_IMETHODIMP
 nsNntpIncomingServer::LoadNewsUrl(nsIURI* aURI, nsIMsgWindow* aMsgWindow,
                                   nsISupports* aConsumer) {
@@ -556,7 +558,7 @@ nsNntpIncomingServer::LoadNewsUrl(nsIURI* aURI, nsIMsgWindow* aMsgWindow,
 
   nsCOMPtr<nsILoadInfo> loadInfo = new mozilla::net::LoadInfo(
       nsContentUtils::GetSystemPrincipal(), nullptr, nullptr,
-      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
       nsIContentPolicy::TYPE_OTHER);
 
   if (protocol) {
@@ -578,6 +580,10 @@ nsNntpIncomingServer::LoadNewsUrl(nsIURI* aURI, nsIMsgWindow* aMsgWindow,
   return NS_OK;
 }
 
+/**
+ * Called when an nsNNTPProtocol finishes running a command and is ready to be
+ * assigned a new one.
+ */
 NS_IMETHODIMP
 nsNntpIncomingServer::PrepareForNextUrl(nsNNTPProtocol* aConnection) {
   NS_ENSURE_ARG(aConnection);
@@ -626,17 +632,10 @@ nsresult nsNntpIncomingServer::DownloadMail(nsIMsgWindow* aMsgWindow) {
   nsresult rv = GetRootFolder(getter_AddRefs(rootFolder));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISimpleEnumerator> groups;
-  rv = rootFolder->GetSubFolders(getter_AddRefs(groups));
+  nsTArray<RefPtr<nsIMsgFolder>> groups;
+  rv = rootFolder->GetSubFolders(groups);
   NS_ENSURE_SUCCESS(rv, rv);
-
-  bool hasNext;
-  while (NS_SUCCEEDED(rv = groups->HasMoreElements(&hasNext)) && hasNext) {
-    nsCOMPtr<nsISupports> nextGroup;
-    rv = groups->GetNext(getter_AddRefs(nextGroup));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIMsgFolder> group(do_QueryInterface(nextGroup));
+  for (nsIMsgFolder* group : groups) {
     rv = group->GetNewMessages(aMsgWindow, nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -702,8 +701,8 @@ nsNntpIncomingServer::ContainsNewsgroup(const nsACString& aName,
     nsCOMPtr<nsIMsgFolder> rootFolder;
     GetRootFolder(getter_AddRefs(rootFolder));
     if (rootFolder) {
-      nsCOMPtr<nsISimpleEnumerator> subfolders;
-      rootFolder->GetSubFolders(getter_AddRefs(subfolders));
+      nsTArray<RefPtr<nsIMsgFolder>> dummy;
+      rootFolder->GetSubFolders(dummy);
     }
   }
   nsAutoCString unescapedName;
@@ -807,7 +806,7 @@ nsresult nsNntpIncomingServer::LoadHostInfoFile() {
   if (NS_FAILED(rv)) return rv;
   if (!mHostInfoFile) return NS_ERROR_FAILURE;
 
-  rv = mHostInfoFile->AppendNative(NS_LITERAL_CSTRING(HOSTINFO_FILE_NAME));
+  rv = mHostInfoFile->AppendNative(nsLiteralCString(HOSTINFO_FILE_NAME));
   if (NS_FAILED(rv)) return rv;
 
   bool exists;
@@ -842,11 +841,7 @@ nsresult nsNntpIncomingServer::LoadHostInfoFile() {
 NS_IMETHODIMP
 nsNntpIncomingServer::StartPopulatingWithUri(nsIMsgWindow* aMsgWindow,
                                              bool aForceToServer,
-                                             const char* uri) {
-#ifdef DEBUG_seth
-  printf("StartPopulatingWithUri(%s)\n", uri);
-#endif
-
+                                             const nsACString& uri) {
   nsresult rv = EnsureInner();
   NS_ENSURE_SUCCESS(rv, rv);
   rv = mInner->StartPopulatingWithUri(aMsgWindow, aForceToServer, uri);
@@ -1229,7 +1224,7 @@ nsNntpIncomingServer::GetFirstChildURI(const nsACString& path,
 
 NS_IMETHODIMP
 nsNntpIncomingServer::GetChildURIs(const nsACString& aPath,
-                                   nsIUTF8StringEnumerator** aResult) {
+                                   nsTArray<nsCString>& aResult) {
   nsresult rv = EnsureInner();
   NS_ENSURE_SUCCESS(rv, rv);
   return mInner->GetChildURIs(aPath, aResult);
@@ -1260,27 +1255,18 @@ nsNntpIncomingServer::ForgetPassword() {
   NS_ENSURE_SUCCESS(rv, rv);
 
   // clear password of all child folders
-  nsCOMPtr<nsISimpleEnumerator> subFolders;
-
-  rv = rootFolder->GetSubFolders(getter_AddRefs(subFolders));
+  nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+  rv = rootFolder->GetSubFolders(subFolders);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  bool moreFolders = false;
-
   nsresult return_rv = NS_OK;
-
-  while (NS_SUCCEEDED(subFolders->HasMoreElements(&moreFolders)) &&
-         moreFolders) {
-    nsCOMPtr<nsISupports> child;
-    rv = subFolders->GetNext(getter_AddRefs(child));
-    if (NS_SUCCEEDED(rv) && child) {
-      newsFolder = do_QueryInterface(child, &rv);
-      if (NS_SUCCEEDED(rv) && newsFolder) {
-        rv = newsFolder->ForgetAuthenticationCredentials();
-        if (NS_FAILED(rv)) return_rv = rv;
-      } else {
-        return_rv = NS_ERROR_FAILURE;
-      }
+  for (nsIMsgFolder* child : subFolders) {
+    newsFolder = do_QueryInterface(child, &rv);
+    if (NS_SUCCEEDED(rv) && newsFolder) {
+      rv = newsFolder->ForgetAuthenticationCredentials();
+      if (NS_FAILED(rv)) return_rv = rv;
+    } else {
+      return_rv = NS_ERROR_FAILURE;
     }
   }
 
@@ -1431,7 +1417,7 @@ nsNntpIncomingServer::GroupNotFound(nsIMsgWindow* aMsgWindow,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCString hostname;
-  rv = GetRealHostName(hostname);
+  rv = GetHostName(hostname);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsString groupName(aName);
@@ -1460,6 +1446,15 @@ nsNntpIncomingServer::SetPrettyNameForGroup(const nsAString& name,
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+nsNntpIncomingServer::GetCanCompactFoldersOnServer(
+    bool* canCompactFoldersOnServer) {
+  NS_ENSURE_ARG_POINTER(canCompactFoldersOnServer);
+  // Initialize canCompactFoldersOnServer false, a default value for NNTP
+  *canCompactFoldersOnServer = false;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsNntpIncomingServer::GetCanSearchMessages(bool* canSearchMessages) {
   NS_ENSURE_ARG_POINTER(canSearchMessages);
@@ -1737,7 +1732,7 @@ nsNntpIncomingServer::SetTree(mozilla::dom::XULTreeElement* tree) {
   if (!element) return NS_OK;
 
   nsAutoString dir;
-  element->GetAttribute(NS_LITERAL_STRING("sortDirection"), dir);
+  element->GetAttribute(u"sortDirection"_ns, dir);
   mSearchResultSortDescending = dir.EqualsLiteral("descending");
   return NS_OK;
 }
@@ -1753,15 +1748,13 @@ nsNntpIncomingServer::CycleHeader(nsTreeColumn* col) {
 
   bool cycler = col->Cycler();
   if (!cycler) {
-    NS_NAMED_LITERAL_STRING(dir, "sortDirection");
+    constexpr auto dir = u"sortDirection"_ns;
     RefPtr<mozilla::dom::Element> element = col->Element();
     mSearchResultSortDescending = !mSearchResultSortDescending;
     mozilla::IgnoredErrorResult rv2;
-    element->SetAttribute(dir,
-                          mSearchResultSortDescending
-                              ? NS_LITERAL_STRING("descending")
-                              : NS_LITERAL_STRING("ascending"),
-                          rv2);
+    element->SetAttribute(
+        dir, mSearchResultSortDescending ? u"descending"_ns : u"ascending"_ns,
+        rv2);
     mTree->Invalidate();
   }
   return NS_OK;
@@ -1889,7 +1882,7 @@ nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   nsCOMPtr<nsIFile> hostInfoFile;
   rv = GetLocalPath(getter_AddRefs(hostInfoFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = hostInfoFile->AppendNative(NS_LITERAL_CSTRING(HOSTINFO_FILE_NAME));
+  rv = hostInfoFile->AppendNative(nsLiteralCString(HOSTINFO_FILE_NAME));
   NS_ENSURE_SUCCESS(rv, rv);
   hostInfoFile->Remove(false);
 
@@ -1902,28 +1895,21 @@ nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   rv = GetRootMsgFolder(getter_AddRefs(serverFolder));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISimpleEnumerator> subFolders;
-  rv = serverFolder->GetSubFolders(getter_AddRefs(subFolders));
+  nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+  rv = serverFolder->GetSubFolders(subFolders);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsTArray<nsString> groupList;
-  nsString folderName;
-
   // Prepare the group list
-  bool hasMore;
-  while (NS_SUCCEEDED(subFolders->HasMoreElements(&hasMore)) && hasMore) {
-    nsCOMPtr<nsISupports> item;
-    subFolders->GetNext(getter_AddRefs(item));
-    nsCOMPtr<nsIMsgFolder> newsgroupFolder(do_QueryInterface(item));
-    if (!newsgroupFolder) continue;
-
+  nsTArray<nsString> groupList(subFolders.Length());
+  for (nsIMsgFolder* newsgroupFolder : subFolders) {
+    nsString folderName;
     rv = newsgroupFolder->GetName(folderName);
     NS_ENSURE_SUCCESS(rv, rv);
     groupList.AppendElement(folderName);
   }
 
   // If nothing subscribed then we're done.
-  if (groupList.Length() == 0) return NS_OK;
+  if (groupList.IsEmpty()) return NS_OK;
 
   // Now unsubscribe & subscribe.
   uint32_t i;
