@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: JavaScript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* Tests that an empty cache entry doesn't return an empty message for news. */
 
 // The basic daemon to use for testing Nntpd.jsm implementations
@@ -11,8 +11,8 @@ var streamListener = {
   _data: "",
 
   QueryInterface: ChromeUtils.generateQI([
-    Ci.nsIStreamListener,
-    Ci.nsIRequestObserver,
+    "nsIStreamListener",
+    "nsIRequestObserver",
   ]),
 
   // nsIRequestObserver
@@ -70,7 +70,7 @@ function run_test() {
       "",
       Ci.nsICacheStorage.OPEN_NORMALLY,
       {
-        onCacheEntryAvailable(cacheEntry, isNew, appCache, status) {
+        onCacheEntryAvailable(cacheEntry, isNew, status) {
           Assert.equal(status, Cr.NS_OK);
 
           cacheEntry.markValid();
@@ -96,8 +96,8 @@ function run_test() {
 
           var messageUri = folder.getUriForMsg(message);
 
-          MailServices.nntp
-            .QueryInterface(Ci.nsIMsgMessageService)
+          Cc["@mozilla.org/messenger/messageservice;1?type=news"]
+            .getService(Ci.nsIMsgMessageService)
             .DisplayMessage(messageUri, streamListener, null, null, null, {});
 
           // Get the server to run
