@@ -6,8 +6,6 @@
 // this file implements the nsMsgFilter interface
 
 #include "msgCore.h"
-#include "nsArray.h"
-#include "nsArrayUtils.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgHdr.h"
 #include "nsMsgFilterList.h"  // for kFileVersion
@@ -20,20 +18,25 @@
 #include "nsMsgSearchValue.h"
 #include "nsMsgI18N.h"
 #include "nsNativeCharsetUtils.h"
-#include "nsIMutableArray.h"
 #include "nsIOutputStream.h"
 #include "nsIStringBundle.h"
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIMsgFilterService.h"
+#include "nsIMsgNewsFolder.h"
 #include "prmem.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Services.h"
+#include "mozilla/intl/AppDateTimeFormat.h"
 
 static const char* kImapPrefix = "//imap:";
 static const char* kWhitespace = "\b\t\r\n ";
 
-nsMsgRuleAction::nsMsgRuleAction() {}
+nsMsgRuleAction::nsMsgRuleAction()
+    : m_type(nsMsgFilterAction::None),
+      m_priority(nsMsgPriority::notSet),
+      m_label(0),
+      m_junkScore(0) {}
 
 nsMsgRuleAction::~nsMsgRuleAction() {}
 
@@ -153,16 +156,12 @@ NS_IMETHODIMP nsMsgRuleAction::GetCustomAction(
 }
 
 nsMsgFilter::nsMsgFilter()
-    : m_temporary(false),
+    : m_type(nsMsgFilterType::InboxRule | nsMsgFilterType::Manual),
+      m_enabled(false),
+      m_temporary(false),
       m_unparseable(false),
       m_filterList(nullptr),
-      m_expressionTree(nullptr) {
-  m_termList = nsArray::Create();
-  NS_ASSERTION(m_termList,
-               "Failed to allocate a nsIMutableArray for m_termList");
-
-  m_type = nsMsgFilterType::InboxRule | nsMsgFilterType::Manual;
-}
+      m_expressionTree(nullptr) {}
 
 nsMsgFilter::~nsMsgFilter() { delete m_expressionTree; }
 
@@ -220,7 +219,8 @@ NS_IMETHODIMP nsMsgFilter::AppendTerm(nsIMsgSearchTerm* aTerm) {
   // invalidate expression tree if we're changing the terms
   delete m_expressionTree;
   m_expressionTree = nullptr;
-  return m_termList->AppendElement(aTerm);
+  m_termList.AppendElement(aTerm);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -363,34 +363,34 @@ NS_IMETHODIMP nsMsgFilter::GetTerm(
     nsACString& arbitraryHeader) /* arbitrary header specified by user.ignore
                                     unless attrib = attribOtherHeader */
 {
-  nsresult rv;
-  nsCOMPtr<nsIMsgSearchTerm> term =
-      do_QueryElementAt(m_termList, termIndex, &rv);
-  if (NS_SUCCEEDED(rv) && term) {
-    if (attrib) term->GetAttrib(attrib);
-    if (op) term->GetOp(op);
-    if (value) term->GetValue(value);
-    if (booleanAnd) term->GetBooleanAnd(booleanAnd);
-    if (attrib && *attrib > nsMsgSearchAttrib::OtherHeader &&
-        *attrib < nsMsgSearchAttrib::kNumMsgSearchAttributes)
-      term->GetArbitraryHeader(arbitraryHeader);
+  if (termIndex >= (int32_t)m_termList.Length()) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  nsIMsgSearchTerm* term = m_termList[termIndex];
+  if (attrib) term->GetAttrib(attrib);
+  if (op) term->GetOp(op);
+  if (value) term->GetValue(value);
+  if (booleanAnd) term->GetBooleanAnd(booleanAnd);
+  if (attrib && *attrib > nsMsgSearchAttrib::OtherHeader &&
+      *attrib < nsMsgSearchAttrib::kNumMsgSearchAttributes) {
+    term->GetArbitraryHeader(arbitraryHeader);
   }
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgFilter::GetSearchTerms(nsIMutableArray** aResult) {
-  NS_ENSURE_ARG_POINTER(aResult);
-  // caller can change m_termList, which can invalidate m_expressionTree.
+NS_IMETHODIMP nsMsgFilter::GetSearchTerms(
+    nsTArray<RefPtr<nsIMsgSearchTerm>>& terms) {
   delete m_expressionTree;
   m_expressionTree = nullptr;
-  NS_IF_ADDREF(*aResult = m_termList);
+  terms = m_termList.Clone();
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgFilter::SetSearchTerms(nsIMutableArray* aSearchList) {
+NS_IMETHODIMP nsMsgFilter::SetSearchTerms(
+    nsTArray<RefPtr<nsIMsgSearchTerm>> const& terms) {
   delete m_expressionTree;
   m_expressionTree = nullptr;
-  m_termList = aSearchList;
+  m_termList = terms.Clone();
   return NS_OK;
 }
 
@@ -451,9 +451,10 @@ nsresult nsMsgFilter::LogRuleHitGeneric(nsIMsgRuleAction* aFilterAction,
   PRExplodedTime exploded;
   PR_ExplodeTime(date, PR_LocalTimeParameters, &exploded);
 
-  mozilla::DateTimeFormat::FormatPRExplodedTime(mozilla::kDateFormatShort,
-                                                mozilla::kTimeFormatSeconds,
-                                                &exploded, dateValue);
+  mozilla::intl::DateTimeFormat::StyleBag style;
+  style.date = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Short);
+  style.time = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Long);
+  mozilla::intl::AppDateTimeFormat::Format(style, &exploded, dateValue);
 
   (void)aMsgHdr->GetMime2DecodedAuthor(authorValue);
   (void)aMsgHdr->GetMime2DecodedSubject(subjectValue);
@@ -550,7 +551,7 @@ nsresult nsMsgFilter::LogRuleHitGeneric(nsIMsgRuleAction* aFilterAction,
   } else {
     nsString actionValue;
     nsAutoCString filterActionID;
-    filterActionID = NS_LITERAL_CSTRING("filterAction");
+    filterActionID = "filterAction"_ns;
     filterActionID.AppendInt(actionType);
     rv = bundle->GetStringFromName(filterActionID.get(), actionValue);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -581,13 +582,12 @@ nsMsgFilter::MatchHdr(nsIMsgDBHdr* msgHdr, nsIMsgFolder* folder,
                       bool* pResult) {
   NS_ENSURE_ARG_POINTER(folder);
   NS_ENSURE_ARG_POINTER(msgHdr);
-  // use offlineMail because
-  nsCString folderCharset;
-  folder->GetCharset(folderCharset);
-  nsresult rv = nsMsgSearchOfflineMail::MatchTermsForFilter(
+  nsCString folderCharset = "UTF-8"_ns;
+  nsCOMPtr<nsIMsgNewsFolder> newsfolder(do_QueryInterface(folder));
+  if (newsfolder) newsfolder->GetCharset(folderCharset);
+  return nsMsgSearchOfflineMail::MatchTermsForFilter(
       msgHdr, m_termList, folderCharset.get(), m_scope, db, headers,
       &m_expressionTree, pResult);
-  return rv;
 }
 
 NS_IMETHODIMP
@@ -649,7 +649,7 @@ nsresult nsMsgFilter::ConvertMoveOrCopyToFolderValue(
       rootFolder->GetURI(folderUri);
       // if the root folder is not imap, than the local mail root is the server
       // root. otherwise, it's the migrated local folders.
-      if (!StringBeginsWith(folderUri, NS_LITERAL_CSTRING("imap:")))
+      if (!StringBeginsWith(folderUri, "imap:"_ns))
         localMailRoot = rootFolder;
       else {
         nsCOMPtr<nsIMsgAccountManager> accountManager =
@@ -803,14 +803,14 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream* aStream) {
       default:
         break;
     }
+    NS_ENSURE_SUCCESS(err, err);
   }
   // and here the fun begins - file out term list...
   nsAutoCString condition;
   err = MsgTermListToString(m_termList, condition);
-  if (NS_SUCCEEDED(err))
-    err = filterList->WriteStrAttr(nsIMsgFilterList::attribCondition,
-                                   condition.get(), aStream);
-  return err;
+  NS_ENSURE_SUCCESS(err, err);
+  return filterList->WriteStrAttr(nsIMsgFilterList::attribCondition,
+                                  condition.get(), aStream);
 }
 
 // for each action, this table encodes the filterTypes that support the action.
