@@ -4,7 +4,16 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsXULAppAPI.h"
+#include "nsXPCOM.h"
+#include "nsISupports.h"
+#include "mozilla/Logging.h"
 #include "mozilla/XREAppData.h"
+#include "mozilla/ArrayUtils.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/Vector.h"
+#include "mozilla/TimeStamp.h"
+#include "XREChildData.h"
+#include "XREShellData.h"
 #include "application.ini.h"
 #include "mozilla/Bootstrap.h"
 #if defined(XP_WIN)
@@ -233,7 +242,7 @@ static int do_main(int argc, char* argv[], char* envp[])
 }
 
 static nsresult
-InitXPCOMGlue()
+InitXPCOMGlue(LibLoadingStrategy aLibLoadingStrategy)
 {
   UniqueFreePtr<char> exePath = BinaryPath::Get();
   if (!exePath) {
@@ -241,12 +250,15 @@ InitXPCOMGlue()
     return NS_ERROR_FAILURE;
   }
 
-  gBootstrap = mozilla::GetBootstrap(exePath.get());
-  if (!gBootstrap) {
+  auto bootstrapResult =
+      mozilla::GetBootstrap(exePath.get(), aLibLoadingStrategy);
+  if (bootstrapResult.isErr()) {
     Output("Couldn't load XPCOM.\n");
     return NS_ERROR_FAILURE;
   }
 
+  gBootstrap = bootstrapResult.unwrap();
+
   // This will set this thread as the main thread.
   gBootstrap->NS_LogInit();
 
@@ -278,7 +290,7 @@ int main(int argc, char* argv[], char* envp[])
     }
 #endif
 
-    nsresult rv = InitXPCOMGlue();
+    nsresult rv = InitXPCOMGlue(LibLoadingStrategy::NoReadAhead);
     if (NS_FAILED(rv)) {
       return 255;
     }
@@ -296,8 +308,7 @@ int main(int argc, char* argv[], char* envp[])
   DllBlocklist_Initialize(gBlocklistInitFlags);
 #endif
 
-
-  nsresult rv = InitXPCOMGlue();
+  nsresult rv = InitXPCOMGlue(LibLoadingStrategy::ReadAhead);
   if (NS_FAILED(rv)) {
     return 255;
   }
