@@ -69,8 +69,6 @@ nsContextMenu.prototype = {
     this.isTextSelected = this.isTextSelection();
     this.isContentSelected = this.isContentSelection();
 
-    this.initPopupPrincipal();
-
     // Initialize gContextMenuContentData.
     if (aEvent)
       this.initContentData(aEvent);
@@ -300,6 +298,7 @@ nsContextMenu.prototype = {
     // Hide Block and Unblock menuitems.
     this.showItem("context-blockimage", false);
     this.showItem("context-unblockimage", false);
+    this.showItem("context-sep-blockimage", false);
 
     // Block image depends on whether an image was clicked on.
     if (this.onImage) {
@@ -324,6 +323,7 @@ nsContextMenu.prototype = {
         this.setItemAttr(id, "accesskey",
                          bundle.getString(attr + ".accesskey"));
         this.showItem(id, true);
+        this.showItem("context-sep-blockimage", true);
       }
     }
   },
@@ -342,20 +342,6 @@ nsContextMenu.prototype = {
       goSetMenuValue("context-saveframe",
                      this.autoDownload ? "valueSave" : "valueSaveAs");
 
-    var blocking = true;
-      if (this.popupPrincipal)
-        try {
-          const PM = Cc["@mozilla.org/PopupWindowManager;1"]
-                     .getService(Ci.nsIPopupWindowManager);
-          blocking = PM.testPermission(this.popupPrincipal) == PM.DENY_POPUP;
-        } catch (e) {
-          Cu.reportError(e);
-        }
-
-    this.showItem("popupwindow-reject", this.popupPrincipal && !blocking);
-    this.showItem("popupwindow-allow", this.popupPrincipal && blocking);
-    this.showItem("context-sep-popup", this.popupPrincipal);
-
     // BiDi UI
     this.showItem("context-sep-bidi", !this.onNumeric && gShowBiDi);
     this.showItem("context-bidi-text-direction-toggle",
@@ -632,7 +618,6 @@ nsContextMenu.prototype = {
     this.inSyntheticDoc        = false;
     this.hasBGImage            = false;
     this.bgImageURL            = "";
-    this.popupPrincipal        = null;
     this.autoDownload          = false;
     this.isTextSelected        = false;
     this.isContentSelected     = false;
@@ -883,33 +868,6 @@ nsContextMenu.prototype = {
     }
   },
 
-  initPopupPrincipal: function() {
-    // quick check: if no opener, it can't be a popup
-    if (!window.content.opener)
-      return;
-    try {
-      var show = false;
-      // is it a popup window?
-      var xulwin = window
-                  .QueryInterface(Ci.nsIInterfaceRequestor)
-                  .getInterface(Ci.nsIWebNavigation)
-                  .QueryInterface(Ci.nsIDocShellTreeItem)
-                  .treeOwner
-                  .QueryInterface(Ci.nsIInterfaceRequestor)
-                  .getInterface(Ci.nsIXULWindow);
-      if (xulwin.contextFlags &
-          Ci.nsIWindowCreator2.PARENT_IS_LOADING_OR_RUNNING_TIMEOUT) {
-        // do the pref settings allow site-by-site popup management?
-        show = !Services.prefs.getBoolPref("dom.disable_open_during_load");
-      }
-      if (show) {
-        // initialize popupPrincipal
-        this.popupPrincipal = window.content.opener.document.nodePrincipal;
-      }
-    } catch(e) {
-    }
-  },
-
   _isSpellCheckEnabled: function(aNode) {
     // We can always force-enable spellchecking on textboxes
     if (this.isTargetATextBox(aNode)) {
@@ -954,21 +912,6 @@ nsContextMenu.prototype = {
            this.linkProtocol != "javascript";
   },
 
-  // Block popup windows
-  rejectPopupWindows: function(aAndClose) {
-    Services.perms.addFromPrincipal(this.popupPrincipal, "popup",
-                                    Services.perms.DENY_ACTION);
-    if (aAndClose)
-      Services.obs.notifyObservers(window, "popup-perm-close",
-                                   this.popupPrincipal.URI.spec);
-  },
-
-  // Unblock popup windows
-  allowPopupWindows: function() {
-    Services.perms.addFromPrincipal(this.popupPrincipal, "popup",
-                                    Services.perms.ALLOW_ACTION);
-  },
-
   // Block/Unblock image from loading in the future.
   toggleImageBlocking: function(aBlock) {
   const uri = Services.io.newURI(this.mediaURL);
@@ -1128,14 +1071,16 @@ nsContextMenu.prototype = {
   viewMedia(e) {
     let doc = this.target.ownerDocument;
     let where = whereToOpenLink(e);
+
     if (this.onCanvas) {
-      let blobUrl = URL.createObjectURL(this.target);
       let systemPrincipal = Services.scriptSecurityManager
                                     .getSystemPrincipal();
-      openUILinkIn(blobUrl, where,
-                   { referrerURI: doc.documentURIObject,
-                     triggeringPrincipal: systemPrincipal,
-                   });
+      this.target.toBlob((blob) => {
+        openUILinkIn(URL.createObjectURL(blob), where,
+                     { referrerURI: doc.documentURIObject,
+                       triggeringPrincipal: systemPrincipal,
+                     });
+     });
     } else {
       urlSecurityCheck(this.mediaURL,
                        this.target.nodePrincipal,
@@ -1304,7 +1249,7 @@ nsContextMenu.prototype = {
     var channel = NetUtil.newChannel({
                     uri: makeURI(linkURL),
                     loadUsingSystemPrincipal: true,
-                    securityFlags: Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL
+                    securityFlags: Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL
                   });
 
     channel.notificationCallbacks = new Callbacks();
@@ -1390,8 +1335,8 @@ nsContextMenu.prototype = {
 
   // Copy email to clipboard
   copyEmail: function() {
-    var clipboard = this.getService("@mozilla.org/widget/clipboardhelper;1",
-                                    Ci.nsIClipboardHelper);
+    var clipboard = Cc["@mozilla.org/widget/clipboardhelper;1"]
+                      .getService(Ci.nsIClipboardHelper);
     clipboard.copyString(this.getEmail());
   },
 
@@ -1433,18 +1378,6 @@ nsContextMenu.prototype = {
   // Utilities //
   ///////////////
 
-  // Create instance of component given contractId and iid (as string).
-  createInstance: function(aContractId, aIIDName) {
-    var iid = Ci[aIIDName];
-    return Cc[aContractId].createInstance(iid);
-  },
-
-  // Get service given contractId and iid (as string).
-  getService: function(aContractId, aIIDName) {
-    var iid = Ci[aIIDName];
-    return Cc[aContractId].getService(iid);
-  },
-
   // Show/hide one item (specified via name or the item element itself).
   showItem: function(aItemOrId, aShow) {
     var item = aItemOrId.constructor == String ? document.getElementById(aItemOrId) : aItemOrId;
@@ -1666,13 +1599,17 @@ nsContextMenu.prototype = {
     return aNode instanceof HTMLTextAreaElement && this.isTextBoxEnabled(aNode);
   },
 
-  // Determines whether or not the separator with the specified ID should be
-  // shown or not by determining if there are any non-hidden items between it
-  // and the previous separator.
+  /**
+   * Determine whether a separator should be shown based on whether
+   * there are any non-hidden items between it and the previous separator.
+   * @param  aSeparatorID
+   *         The id of the separator element
+   * @return true if the separator should be shown, false if not
+   */
   shouldShowSeparator: function(aSeparatorID) {
-    var separator = document.getElementById(aSeparatorID);
+    let separator = document.getElementById(aSeparatorID);
     if (separator) {
-      var sibling = separator.previousSibling;
+      let sibling = separator.previousSibling;
       while (sibling && sibling.localName != "menuseparator") {
         if (sibling.getAttribute("hidden") != "true")
           return true;
