@@ -7,9 +7,12 @@
 /*
  * Command-specific code. This stuff should be called by the widgets
  */
-
-const {fixIterator} = ChromeUtils.import("resource:///modules/iteratorUtils.jsm");
-var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+ 
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+const { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.js");
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
 
 //NOTE: gMessengerBundle and gBrandBundle must be defined and set
 //      for this Overlay to work properly
@@ -23,19 +26,6 @@ var gPrevFolderFlags;
 var gPrevSelectedFolder;
 var gMsgFolderSelected;
 
-function GetServer(uri)
-{
-    if (!uri) return null;
-    try {
-        var folder = GetMsgFolderFromUri(uri, true);
-        return folder.server;
-    }
-    catch (ex) {
-        dump("GetServer("+uri+") failed, ex="+ex+"\n");
-    }
-    return null;
-}
-
 function setTitleFromFolder(msgfolder, subject)
 {
     var title = subject || "";
@@ -260,9 +250,10 @@ function RerootFolder(uri, newFolder, viewType, viewFlags, sortType, sortOrder)
 
   // If this is the  sent, drafts, templates, or send later folder,
   // we show "Recipient" instead of "Author".
-  const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
-  let outgoingFlags = nsMsgFolderFlags.SentMail | nsMsgFolderFlags.Drafts |
-                      nsMsgFolderFlags.Templates | nsMsgFolderFlags.Queue;
+  let outgoingFlags = Ci.nsMsgFolderFlags.SentMail |
+                      Ci.nsMsgFolderFlags.Drafts |
+                      Ci.nsMsgFolderFlags.Templates |
+                      Ci.nsMsgFolderFlags.Queue;
   SetSentFolderColumns(newFolder.isSpecialFolder(outgoingFlags, true));
   ShowLocationColumn(viewType == nsMsgViewType.eShowVirtualFolderResults);
   // Only show 'Received' column for e-mails.  For newsgroup messages, the 'Date' header is as reliable as an e-mail's
@@ -278,15 +269,13 @@ function RerootFolder(uri, newFolder, viewType, viewFlags, sortType, sortOrder)
 
      /*we don't null out the db reference for inbox because inbox is like the "main" folder
        and performance outweighs footprint */
-    if (!oldFolder.isSpecialFolder(nsMsgFolderFlags.Inbox, false))
+    if (!oldFolder.isSpecialFolder(Ci.nsMsgFolderFlags.Inbox, false))
       if (oldFolder.URI != newFolder.URI)
         oldFolder.msgDatabase = null;
   }
   // that should have initialized gDBView, now re-root the thread pane
   RerootThreadPane();
 
-  UpdateLocationBar(gMsgFolderSelected);
-
   UpdateStatusMessageCounts(gMsgFolderSelected);
 
   UpdateMailToolbar("reroot folder in 3 pane");
@@ -412,10 +401,11 @@ function UpdateReceivedColumn(newFolder)
   // 'Received' header, as it is replaced with the news server's (more reliable) date.
   var receivedColumn = document.getElementById("receivedCol");
 
-  const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
-  var newFolderShowsRcvd = (newFolder.flags & nsMsgFolderFlags.Mail) &&
-    !(newFolder.flags & (nsMsgFolderFlags.Queue | nsMsgFolderFlags.Templates |
-                         nsMsgFolderFlags.Drafts | nsMsgFolderFlags.SentMail));
+  var newFolderShowsRcvd = (newFolder.flags & Ci.nsMsgFolderFlags.Mail) &&
+                           !(newFolder.flags & (Ci.nsMsgFolderFlags.Queue |
+                                                Ci.nsMsgFolderFlags.Templates |
+                                                Ci.nsMsgFolderFlags.Drafts |
+                                                Ci.nsMsgFolderFlags.SentMail));
 
   var tempHidden = receivedColumn.getAttribute("temphidden") == "true";
   var isHidden = receivedColumn.getAttribute("hidden") == "true";
@@ -649,8 +639,19 @@ function CreateDBView(msgFolder, viewType, viewFlags, sortType, sortOrder)
 
 function FolderPaneSelectionChange()
 {
-    var folderTree = GetFolderTree();
-    var folderSelection = folderTree.view.selection;
+    let folders = GetSelectedMsgFolders();
+    if (folders.length) {
+      let locationItem = document.getElementById("locationFolders");
+      if (locationItem &&
+          locationItem.parentNode.parentNode.localName != "toolbarpalette") {
+        let msgFolder = folders[0];
+        locationItem.setAttribute("label", msgFolder.prettyName);
+        document.getElementById("folderLocationPopup")
+                ._setCssSelectors(msgFolder, locationItem);
+      }
+    }
+
+    let folderSelection = gFolderTreeView.selection;
 
     // This prevents a folder from being loaded in the case that the user
     // has right-clicked on a folder different from the one that was
@@ -664,7 +665,6 @@ function FolderPaneSelectionChange()
     gVirtualFolderTerms = null;
     gXFVirtualFolderTerms = null;
 
-    let folders = GetSelectedMsgFolders();
     if (folders.length == 1)
     {
         let msgFolder = folders[0];
@@ -677,7 +677,6 @@ function FolderPaneSelectionChange()
         let realFolder = msgFolder;
         gPrevSelectedFolder = gMsgFolderSelected;
         gMsgFolderSelected = msgFolder;
-        UpdateLocationBar(gMsgFolderSelected);
         var folderFlags = msgFolder.flags;
         const kVirtual = Ci.nsMsgFolderFlags.Virtual;
         // if this is same folder, and we're not showing a virtual folder
@@ -721,8 +720,7 @@ function FolderPaneSelectionChange()
                     var srchFolderUriArray = srchFolderUri.split('|');
                     var searchOnline = dbFolderInfo.getBooleanProperty("searchOnline", false);
                     // cross folder search
-                    var filterService = Cc["@mozilla.org/messenger/services/filters;1"].getService(Ci.nsIMsgFilterService);
-                    var filterList = filterService.getTempFilterList(msgFolder);
+                    var filterList = MailServices.filters.getTempFilterList(msgFolder);
                     var tempFilter = filterList.createFilter("temp");
                     filterList.parseCondition(tempFilter, searchTermString);
                     if (srchFolderUriArray.length > 1)
@@ -737,7 +735,7 @@ function FolderPaneSelectionChange()
                       uriToLoad = srchFolderUri;
                       // we need to load the db for the actual folder so that many hdrs to download
                       // will return false...
-                      realFolder = GetMsgFolderFromUri(uriToLoad);
+                      realFolder = MailUtils.getFolderForURI(uriToLoad);
                       msgDatabase = realFolder.msgDatabase;
 //                      dump("search term string = " + searchTermString + "\n");
 
@@ -834,7 +832,7 @@ function getSearchTermString(searchTerms)
   var count = searchTerms.length;
   for (searchIndex = 0; searchIndex < count; )
   {
-    var term = searchTerms.queryElementAt(searchIndex++, Ci.nsIMsgSearchTerm);
+    var term = searchTerms[searchIndex++];
 
     if (condition.length > 1)
       condition += ' ';
@@ -873,9 +871,8 @@ function  CreateVirtualFolder(newName, parentFolder, searchFolderURIs, searchTer
       dbFolderInfo.setBooleanProperty("searchOnline", searchOnline);
       vfdb.summaryValid = true;
       vfdb.Close(true);
-      parentFolder.NotifyItemAdded(newFolder);
-      var accountManager = Cc["@mozilla.org/messenger/account-manager;1"].getService(Ci.nsIMsgAccountManager);
-      accountManager.saveVirtualFolders();
+      parentFolder.notifyFolderAdded(newFolder);
+      MailServices.accounts.saveVirtualFolders();
     }
     catch(e)
     {
@@ -890,21 +887,12 @@ function  CreateVirtualFolder(newName, parentFolder, searchFolderURIs, searchTer
 }
 
 var searchSessionContractID = "@mozilla.org/messenger/searchSession;1";
-var gSearchView;
 var gSearchSession;
 
-var nsIMsgFolder = Ci.nsIMsgFolder;
-var nsIMsgWindow = Ci.nsIMsgWindow;
 var nsMsgSearchScope = Ci.nsMsgSearchScope;
 
-var gFolderDatasource;
-var gFolderPicker;
-var gStatusBar = null;
 var gMessengerBundle = null;
 
-// Datasource search listener -- made global as it has to be registered
-// and unregistered in different functions.
-var gDataSourceSearchListener;
 var gViewSearchListener;
 
 function GetScopeForFolder(folder)
@@ -914,20 +902,20 @@ function GetScopeForFolder(folder)
 
 function setupXFVirtualFolderSearch(folderUrisToSearch, searchTerms, searchOnline)
 {
-    var count = new Object;
+  var count = new Object;
   var i;
 
-    gSearchSession = Cc[searchSessionContractID].createInstance(Ci.nsIMsgSearchSession);
+  gSearchSession = Cc[searchSessionContractID]
+                     .createInstance(Ci.nsIMsgSearchSession);
 
   for (i in folderUrisToSearch)
     {
-      let realFolder = GetMsgFolderFromUri(folderUrisToSearch[i]);
+      let realFolder = MailUtils.getFolderForURI(folderUrisToSearch[i]);
       if (!realFolder.isServer)
         gSearchSession.addScopeTerm(!searchOnline ? nsMsgSearchScope.offlineMail : GetScopeForFolder(realFolder), realFolder);
     }
 
-    const nsIMsgSearchTerm = Ci.nsIMsgSearchTerm;
-    for (let term of fixIterator(searchTerms, nsIMsgSearchTerm)) {
+    for (let term of searchTerms) {
       gSearchSession.appendTerm(term);
     }
 }
@@ -952,7 +940,7 @@ function CreateGroupedSearchTerms(searchTermsArray)
 
   var numEntries = searchTermsArray.length;
   for (let i = 0; i < numEntries; i++) {
-    let searchTerm = searchTermsArray.queryElementAt(i, Ci.nsIMsgSearchTerm);
+    let searchTerm = searchTermsArray[i];
 
     // clone the term, since we might be modifying it
     var searchTermForQS = searchSession.createTerm();
