@@ -3,6 +3,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+const { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.js");
+
 /* This is where functions related to the standalone message window are kept */
 
 // from MailNewsTypes.h
@@ -22,31 +24,30 @@ var gNextMessageAfterLoad = null;
 
 // the folderListener object
 var folderListener = {
-  OnItemAdded: function(parentItem, item) {},
-
-  OnItemRemoved: function(parentItem, item)
+  onFolderAdded: function(parentFolder, child) {},
+  onMessageAdded: function(parentFolder, msg) {},
+  onFolderRemoved: function(parentFolder, child) {},
+  onMessageRemoved: function(parentFolder, msg)
   {
-    if (parentItem.URI != gCurrentFolderUri)
+    if (parentFolder.URI != gCurrentFolderUri)
       return;
-
-    if (item instanceof Ci.nsIMsgDBHdr &&
-        extractMsgKeyFromURI() == item.messageKey)
+    if (extractMsgKeyFromURI() == msg.messageKey)
       gCurrentMessageIsDeleted = true;
   },
 
-  OnItemPropertyChanged: function(item, property, oldValue, newValue) {},
-  OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {
+  onFolderPropertyChanged: function(item, property, oldValue, newValue) {},
+  onFolderIntPropertyChanged: function(item, property, oldValue, newValue) {
     if (item.URI == gCurrentFolderUri) {
       if (property == "TotalMessages" || property == "TotalUnreadMessages") {
         UpdateStandAloneMessageCounts();
       }
     }
   },
-  OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {},
-  OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue){},
-  OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},
+  onFolderBoolPropertyChanged: function(item, property, oldValue, newValue) {},
+  onFolderUnicharPropertyChanged: function(item, property, oldValue, newValue){},
+  onFolderPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},
 
-  OnItemEvent: function(folder, event) {
+  onFolderEvent: function(folder, event) {
     if (event == "DeleteOrMoveMsgCompleted")
       HandleDeleteOrMoveMsgCompleted(folder);
     else if (event == "DeleteOrMoveMsgFailed")
@@ -70,12 +71,14 @@ var folderListener = {
 }
 
 var messagepaneObserver = {
-
-  canHandleMultipleItems: false,
-
-  onDrop: function (aEvent, aData, aDragSession)
-  {
-    var sourceUri = aData.data;
+  onDrop(aEvent) {
+    let dragSession = Cc["@mozilla.org/widget/dragservice;1"]
+                        .getService(Ci.nsIDragService)
+                        .getCurrentSession();
+    if (!this.canDrop(aEvent, dragSession)) {
+      return;
+    }
+    let sourceUri = aEvent.dataTransfer.getData("text/x-moz-message");
     if (sourceUri != gCurrentMessageUri)
     {
       var msgHdr = GetMsgHdrFromUri(sourceUri);
@@ -91,36 +94,30 @@ var messagepaneObserver = {
       // (msgHdr.folder.URI == windowID.gCurrentFolderUri)
       // the reason is quick search and mail views.
       // see bug #187673
-      CreateView(aDragSession.sourceNode.ownerDocument.defaultView.gDBView);
+      CreateView(dragSession.sourceNode.ownerDocument.defaultView.gDBView);
       LoadMessageByMsgKey(msgHdr.messageKey);
     }
+    aEvent.stopPropagation();
   },
 
-  onDragOver: function (aEvent, aFlavour, aDragSession)
-  {
+  onDragOver(aEvent) {
     var messagepanebox = document.getElementById("messagepanebox");
     messagepanebox.setAttribute("dragover", "true");
+    aEvent.stopPropagation();
+    aEvent.preventDefault();
   },
 
-  onDragExit: function (aEvent, aDragSession)
-  {
+  onDragExit(aEvent) {
     var messagepanebox = document.getElementById("messagepanebox");
     messagepanebox.removeAttribute("dragover");
   },
 
-  canDrop: function(aEvent, aDragSession)  //allow drop from mail:3pane window only - 4xp
-  {
+  canDrop(aEvent, aDragSession) {
+    // Allow drop from mail:3pane window only - 4xp.
     var doc = aDragSession.sourceNode.ownerDocument;
     var elem = doc.getElementById("messengerWindow");
     return (elem && (elem.getAttribute("windowtype") == "mail:3pane"));
   },
-
-  getSupportedFlavours: function ()
-  {
-    var flavourSet = new FlavourSet();
-    flavourSet.appendFlavour("text/x-moz-message");
-    return flavourSet;
-  }
 };
 
 function nsMsgDBViewCommandUpdater()
@@ -434,7 +431,8 @@ function GetSelectedIndices(dbView)
 
 function GetLoadedMsgFolder()
 {
-  return gCurrentFolderUri ? GetMsgFolderFromUri(gCurrentFolderUri) : null;
+  return gCurrentFolderUri ? MailUtils.getFolderForURI(gCurrentFolderUri)
+                           : null;
 }
 
 function GetLoadedMessage()
@@ -455,13 +453,12 @@ function SetNextMessageAfterDelete()
   gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete;
 }
 
-function SelectFolder(folderUri)
-{
-  if (folderUri == gCurrentFolderUri)
+function SelectMsgFolder(msgfolder) {
+  if (!msgfolder || msgfolder.isServer)
     return;
 
-  let msgfolder = GetMsgFolderFromUri(folderUri);
-  if (!msgfolder || msgfolder.isServer)
+  let folderUri = msgfolder.URI;
+  if (folderUri == gCurrentFolderUri)
     return;
 
   // close old folder view
@@ -556,6 +553,7 @@ var MessageWindowController =
       case "cmd_getMsgsForAuthAccounts":
       case "button_mark":
       case "cmd_markAsRead":
+      case "cmd_markAsUnread":
       case "cmd_markAllRead":
       case "cmd_markThreadAsRead":
       case "cmd_markReadByDate":
@@ -599,6 +597,9 @@ var MessageWindowController =
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
       case "cmd_editAsNew":
+      case "cmd_editDraftMsg":
+      case "cmd_newMsgFromTemplate":
+      case "cmd_editTemplateMsg":
       case "cmd_getNextNMessages":
       case "cmd_find":
       case "cmd_findNext":
@@ -616,6 +617,8 @@ var MessageWindowController =
       case "cmd_settingsOffline":
       case "cmd_createFilterFromPopup":
       case "cmd_createFilterFromMenu":
+      case "cmd_viewAllHeader":
+      case "cmd_viewNormalHeader":
         return true;
       case "cmd_synchronizeOffline":
       case "cmd_downloadFlagged":
@@ -676,6 +679,9 @@ var MessageWindowController =
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
       case "cmd_editAsNew":
+      case "cmd_editDraftMsg":
+      case "cmd_newMsgFromTemplate":
+      case "cmd_editTemplateMsg":
       case "cmd_print":
       case "cmd_printpreview":
       case "button_print":
@@ -688,11 +694,16 @@ var MessageWindowController =
       case "cmd_reload":
       case "cmd_find":
       case "button_mark":
-      case "cmd_markAsRead":
       case "cmd_markAllRead":
       case "cmd_markThreadAsRead":
       case "cmd_markReadByDate":
+      case "cmd_viewAllHeader":
+      case "cmd_viewNormalHeader":
         return true;
+      case "cmd_markAsRead":
+        return CanMarkMsgAsRead(true);
+      case "cmd_markAsUnread":
+        return CanMarkMsgAsRead(false);
       case "cmd_markAsFlagged":
       case "button_file":
         return (gCurrentMessageUri != null);
@@ -807,7 +818,16 @@ var MessageWindowController =
         MsgForwardAsAttachment(null);
         break;
       case "cmd_editAsNew":
-        MsgEditMessageAsNew();
+        MsgEditMessageAsNew(null);
+        break;
+      case "cmd_editDraftMsg":
+        MsgEditDraftMessage(null);
+        break;
+      case "cmd_newMsgFromTemplate":
+        MsgNewMessageFromTemplate(null);
+        break;
+      case "cmd_editTemplateMsg":
+        MsgEditTemplateMessage(null);
         break;
       case "cmd_createFilterFromPopup":
         CreateFilter(document.popupNode);
@@ -865,8 +885,13 @@ var MessageWindowController =
         MsgSearchMessages();
         break;
       case "button_mark":
+        MsgMarkMsgAsRead();
+        return;
       case "cmd_markAsRead":
-        MsgMarkMsgAsRead(null);
+        MsgMarkMsgAsRead(true);
+        return;
+      case "cmd_markAsUnread":
+        MsgMarkMsgAsRead(false);
         return;
       case "cmd_markThreadAsRead":
         MsgMarkThreadAsRead();
@@ -877,8 +902,14 @@ var MessageWindowController =
       case "cmd_markReadByDate":
         MsgMarkReadByDate();
         return;
+      case "cmd_viewAllHeader":
+        MsgViewAllHeaders();
+        return;
+      case "cmd_viewNormalHeader":
+        MsgViewNormalHeaders();
+        return;
       case "cmd_markAsFlagged":
-        MsgMarkAsFlagged(null);
+        MsgMarkAsFlagged();
         return;
       case "cmd_markAsJunk":
         JunkSelectedMessages(true);
