@@ -113,15 +113,17 @@ function ScrollToMessageAfterFolderLoad(folder)
 // the folderListener object
 var folderListener =
 {
-  OnItemAdded:   function(parentItem, item) {},
-  OnItemRemoved: function(parentItem, item) {},
+  onFolderAdded: function(parentFolder, child) {},
+  onMessageAdded: function(parentFolder, msg) {},
+  onFolderRemoved: function(parentFolder, child) {},
+  onMessageRemoved: function(parentFolder, msg) {},
 
-  OnItemPropertyChanged:        function(item, property, oldValue, newValue) {},
-  OnItemBoolPropertyChanged:    function(item, property, oldValue, newValue) {},
-  OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue) {},
-  OnItemPropertyFlagChanged:    function(item, property, oldFlag,  newFlag)  {},
+  onFolderPropertyChanged:        function(item, property, oldValue, newValue) {},
+  onFolderBoolPropertyChanged:    function(item, property, oldValue, newValue) {},
+  onFolderUnicharPropertyChanged: function(item, property, oldValue, newValue) {},
+  onFolderPropertyFlagChanged:    function(item, property, oldFlag,  newFlag)  {},
 
-  OnItemIntPropertyChanged: function(item, property, oldValue, newValue)
+  onFolderIntPropertyChanged: function(item, property, oldValue, newValue)
   {
     // handle the currently visible folder
     if (item == gMsgFolderSelected)
@@ -151,7 +153,7 @@ var folderListener =
     }
   },
 
-    OnItemEvent: function(folder, event) {
+    onFolderEvent: function(folder, event) {
       if (event == "FolderLoaded") {
         if (folder) {
           var scrolled = false;
@@ -1176,6 +1178,24 @@ function SetNextMessageAfterDelete()
     gNextMessageViewIndexAfterDelete = treeSelection.currentIndex;
 }
 
+function EnsureFolderIndex(treeView, msgFolder) {
+  // Try to get the index of the folder in the tree.
+  let index = treeView.getIndexOfFolder(msgFolder);
+  if (!index) {
+    // If we couldn't find the folder, open the parents.
+    let folder = msgFolder;
+    while (!index && folder) {
+      folder = folder.parent;
+      index = EnsureFolderIndex(treeView, folder);
+    }
+    if (index) {
+      treeView.toggleOpenState(index);
+      index = treeView.getIndexOfFolder(msgFolder);
+    }
+  }
+  return index;
+}
+
 function SelectMsgFolder(msgFolder) {
   gFolderTreeView.selectFolder(msgFolder);
 }
