@@ -4,12 +4,12 @@
 
 from __future__ import absolute_import, print_function, unicode_literals
 
-from taskgraph.actions.registry import register_callback_action
-from taskgraph.actions.util import (
+from gecko_taskgraph.actions.registry import register_callback_action
+from gecko_taskgraph.actions.util import (
     create_tasks,
     fetch_graph_and_labels,
 )
-from taskgraph.util.attributes import RELEASE_PROMOTION_PROJECTS
+from gecko_taskgraph.util.attributes import RELEASE_PROMOTION_PROJECTS
 
 
 def is_release_promotion_available(parameters):
@@ -30,10 +30,13 @@ def l10n_bump_action(parameters, graph_config, _input, task_group_id, task_id):
     Runs the 'l10n_bump' task.
     """
     decision_task_id, full_task_graph, label_to_taskid = fetch_graph_and_labels(
-        parameters, graph_config)
-    to_run = [label
-              for label, entry
-              in full_task_graph.tasks.items() if 'l10n-bump' in entry.kind]
+        parameters, graph_config
+    )
+    to_run = [
+        label
+        for label, entry in full_task_graph.tasks.items()
+        if "l10n-bump" in entry.kind
+    ]
     create_tasks(
         graph_config,
         to_run,
