@@ -49,10 +49,10 @@ UniqueCERTCertList FindClientCertificatesWithPrivateKeys();
     }                                                \
   }
 
-extern bool EnsureNSSInitializedChromeOrContent();
-extern bool HandleTLSPrefChange(const nsCString& aPref);
-extern void SetValidationOptionsCommon();
-extern void NSSShutdownForSocketProcess();
+bool EnsureNSSInitializedChromeOrContent();
+bool HandleTLSPrefChange(const nsCString& aPref);
+void SetValidationOptionsCommon();
+void NSSShutdownForSocketProcess();
 
 // Implementation of the PSM component interface.
 class nsNSSComponent final : public nsINSSComponent, public nsIObserver {
@@ -100,8 +100,7 @@ class nsNSSComponent final : public nsINSSComponent, public nsIObserver {
       /*out*/ mozilla::psm::CertVerifier::OcspStrictConfig* osc,
       /*out*/ uint32_t* certShortLifetimeInDays,
       /*out*/ TimeDuration& softTimeout,
-      /*out*/ TimeDuration& hardTimeout,
-      const mozilla::MutexAutoLock& proofOfLock);
+      /*out*/ TimeDuration& hardTimeout);
   void UpdateCertVerifierWithEnterpriseRoots();
   nsresult RegisterObservers();
 
@@ -117,8 +116,8 @@ class nsNSSComponent final : public nsINSSComponent, public nsIObserver {
 
   // mLoadableCertsLoadedMonitor protects mLoadableCertsLoaded.
   mozilla::Monitor mLoadableCertsLoadedMonitor;
-  bool mLoadableCertsLoaded;
-  nsresult mLoadableCertsLoadedResult;
+  bool mLoadableCertsLoaded GUARDED_BY(mLoadableCertsLoadedMonitor);
+  nsresult mLoadableCertsLoadedResult GUARDED_BY(mLoadableCertsLoadedMonitor);
 
   // mMutex protects all members that are accessed from more than one thread.
   mozilla::Mutex mMutex;
@@ -126,13 +125,13 @@ class nsNSSComponent final : public nsINSSComponent, public nsIObserver {
   // The following members are accessed from more than one thread:
 
 #ifdef DEBUG
-  nsString mTestBuiltInRootHash;
+  nsCString mTestBuiltInRootHash GUARDED_BY(mMutex);
 #endif
-  nsCString mContentSigningRootHash;
-  RefPtr<mozilla::psm::SharedCertVerifier> mDefaultCertVerifier;
-  nsString mMitmCanaryIssuer;
-  bool mMitmDetecionEnabled;
-  mozilla::Vector<EnterpriseCert> mEnterpriseCerts;
+  RefPtr<mozilla::psm::SharedCertVerifier> mDefaultCertVerifier
+      GUARDED_BY(mMutex);
+  nsString mMitmCanaryIssuer GUARDED_BY(mMutex);
+  bool mMitmDetecionEnabled GUARDED_BY(mMutex);
+  mozilla::Vector<EnterpriseCert> mEnterpriseCerts GUARDED_BY(mMutex);
 
   // The following members are accessed only on the main thread:
   static int mInstanceCount;
