@@ -17,8 +17,6 @@
 #include "mozilla/ModuleUtils.h"
 #include "mozilla/SyncRunnable.h"
 #include "nsCURILoader.h"
-#include "nsCryptoHash.h"
-#include "nsKeyModule.h"
 #include "nsNSSCertificate.h"
 #include "nsNSSCertificateDB.h"
 #include "nsNSSComponent.h"
@@ -30,9 +28,7 @@
 #include "nsSecureBrowserUI.h"
 #include "nsXULAppAPI.h"
 
-#ifdef MOZ_XUL
-#  include "nsCertTree.h"
-#endif
+#include "nsCertTree.h"
 
 namespace mozilla {
 namespace psm {
@@ -74,12 +70,8 @@ template <class InstanceClass,
           ProcessRestriction processRestriction =
               ProcessRestriction::ParentProcessOnly,
           ThreadRestriction threadRestriction = ThreadRestriction::AnyThread>
-static nsresult Constructor(nsISupports* aOuter, REFNSIID aIID,
-                            void** aResult) {
+static nsresult Constructor(REFNSIID aIID, void** aResult) {
   *aResult = nullptr;
-  if (aOuter != nullptr) {
-    return NS_ERROR_NO_AGGREGATION;
-  }
 
   if (processRestriction == ProcessRestriction::ParentProcessOnly &&
       !XRE_IsParentProcess()) {
@@ -98,11 +90,10 @@ static nsresult Constructor(nsISupports* aOuter, REFNSIID aIID,
   return Instantiate<InstanceClass, InitMethod>(aIID, aResult);
 }
 
-#define IMPL(type, ...)                                                  \
-  template <>                                                            \
-  nsresult NSSConstructor<type>(nsISupports * aOuter, const nsIID& aIID, \
-                                void** aResult) {                        \
-    return Constructor<type, __VA_ARGS__>(aOuter, aIID, aResult);        \
+#define IMPL(type, ...)                                              \
+  template <>                                                        \
+  nsresult NSSConstructor<type>(const nsIID& aIID, void** aResult) { \
+    return Constructor<type, __VA_ARGS__>(aIID, aResult);            \
   }
 
 // Components that require main thread initialization could cause a deadlock
@@ -114,13 +105,7 @@ IMPL(nsPK11TokenDB, nullptr)
 IMPL(PKCS11ModuleDB, nullptr)
 IMPL(nsNSSCertificate, nullptr, ProcessRestriction::AnyProcess)
 IMPL(nsNSSCertificateDB, nullptr)
-#ifdef MOZ_XUL
 IMPL(nsCertTree, nullptr)
-#endif
-IMPL(nsCryptoHash, nullptr, ProcessRestriction::AnyProcess)
-IMPL(nsCryptoHMAC, nullptr, ProcessRestriction::AnyProcess)
-IMPL(nsKeyObject, nullptr, ProcessRestriction::AnyProcess)
-IMPL(nsKeyObjectFactory, nullptr, ProcessRestriction::AnyProcess)
 IMPL(ContentSignatureVerifier, nullptr)
 IMPL(nsRandomGenerator, nullptr, ProcessRestriction::AnyProcess)
 IMPL(TransportSecurityInfo, nullptr, ProcessRestriction::AnyProcess)
