@@ -6,6 +6,9 @@
 #![allow(non_camel_case_types)]
 
 use pkcs11::types::*;
+use rsclientcerts::error::{Error, ErrorType};
+use rsclientcerts::manager::{ClientCertsBackend, CryptokiObject, Sign, SlotType};
+use rsclientcerts::util::*;
 use sha2::{Digest, Sha256};
 use std::convert::TryInto;
 use std::ffi::{c_void, CStr, CString};
@@ -17,9 +20,6 @@ use winapi::um::errhandlingapi::GetLastError;
 use winapi::um::ncrypt::*;
 use winapi::um::wincrypt::{HCRYPTHASH, HCRYPTPROV, *};
 
-use crate::manager::SlotType;
-use crate::util::*;
-
 // winapi has some support for ncrypt.h, but not for this function.
 extern "system" {
     fn NCryptSignHash(
@@ -37,7 +37,7 @@ extern "system" {
 /// Given a `CERT_INFO`, tries to return the bytes of the subject distinguished name as formatted by
 /// `CertNameToStrA` using the flag `CERT_SIMPLE_NAME_STR`. This is used as the label for the
 /// certificate.
-fn get_cert_subject_dn(cert_info: &CERT_INFO) -> Result<Vec<u8>, ()> {
+fn get_cert_subject_dn(cert_info: &CERT_INFO) -> Result<Vec<u8>, Error> {
     let mut cert_info_subject = cert_info.Subject;
     let subject_dn_len = unsafe {
         CertNameToStrA(
@@ -56,11 +56,14 @@ fn get_cert_subject_dn(cert_info: &CERT_INFO) -> Result<Vec<u8>, ()> {
             &mut cert_info_subject,
             CERT_SIMPLE_NAME_STR,
             subject_dn_string_bytes.as_mut_ptr() as *mut i8,
-            subject_dn_string_bytes.len().try_into().map_err(|_| ())?,
+            subject_dn_string_bytes
+                .len()
+                .try_into()
+                .map_err(|_| error_here!(ErrorType::ValueTooLarge))?,
         )
     };
     if subject_dn_len as usize != subject_dn_string_bytes.len() {
-        return Err(());
+        return Err(error_here!(ErrorType::ExternalError));
     }
     Ok(subject_dn_string_bytes)
 }
@@ -88,14 +91,14 @@ pub struct Cert {
 }
 
 impl Cert {
-    fn new(cert_context: PCCERT_CONTEXT) -> Result<Cert, ()> {
+    fn new(cert_context: PCCERT_CONTEXT) -> Result<Cert, Error> {
         let cert = unsafe { &*cert_context };
         let cert_info = unsafe { &*cert.pCertInfo };
         let value =
             unsafe { slice::from_raw_parts(cert.pbCertEncoded, cert.cbCertEncoded as usize) };
         let value = value.to_vec();
         let id = Sha256::digest(&value).to_vec();
-        let label = get_cert_subject_dn(&cert_info)?;
+        let label = get_cert_subject_dn(cert_info)?;
         let (serial_number, issuer, subject) = read_encoded_certificate_identifiers(&value)?;
         Ok(Cert {
             class: serialize_uint(CKO_CERTIFICATE)?,
@@ -118,7 +121,7 @@ impl Cert {
         &self.token
     }
 
-    pub fn id(&self) -> &[u8] {
+    fn id(&self) -> &[u8] {
         &self.id
     }
 
@@ -141,7 +144,9 @@ impl Cert {
     fn subject(&self) -> &[u8] {
         &self.subject
     }
+}
 
+impl CryptokiObject for Cert {
     fn matches(&self, slot_type: SlotType, attrs: &[(CK_ATTRIBUTE_TYPE, Vec<u8>)]) -> bool {
         if slot_type != self.slot_type {
             return false;
@@ -211,7 +216,7 @@ enum KeyHandle {
 }
 
 impl KeyHandle {
-    fn from_cert(cert: &CertContext) -> Result<KeyHandle, ()> {
+    fn from_cert(cert: &CertContext) -> Result<KeyHandle, Error> {
         let mut key_handle = 0;
         let mut key_spec = 0;
         let mut must_free = 0;
@@ -225,16 +230,14 @@ impl KeyHandle {
                 &mut must_free,
             ) != 1
             {
-                error!(
-                    "CryptAcquireCertificatePrivateKey failed: 0x{:x}",
-                    GetLastError()
-                );
-                return Err(());
+                return Err(error_here!(
+                    ErrorType::ExternalError,
+                    GetLastError().to_string()
+                ));
             }
         }
         if must_free == 0 {
-            error!("CryptAcquireCertificatePrivateKey returned shared key handle");
-            return Err(());
+            return Err(error_here!(ErrorType::ExternalError));
         }
         if key_spec == CERT_NCRYPT_KEY_SPEC {
             Ok(KeyHandle::NCrypt(key_handle as NCRYPT_KEY_HANDLE))
@@ -249,7 +252,7 @@ impl KeyHandle {
         params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
         do_signature: bool,
         key_type: KeyType,
-    ) -> Result<Vec<u8>, ()> {
+    ) -> Result<Vec<u8>, Error> {
         match &self {
             KeyHandle::NCrypt(ncrypt_handle) => {
                 sign_ncrypt(ncrypt_handle, data, params, do_signature, key_type)
@@ -280,7 +283,7 @@ fn sign_ncrypt(
     params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
     do_signature: bool,
     key_type: KeyType,
-) -> Result<Vec<u8>, ()> {
+) -> Result<Vec<u8>, Error> {
     let mut sign_params = SignParams::new(key_type, params)?;
     let params_ptr = sign_params.params_ptr();
     let flags = sign_params.flags();
@@ -293,7 +296,9 @@ fn sign_ncrypt(
             *ncrypt_handle,
             params_ptr,
             data.as_mut_ptr(),
-            data.len().try_into().map_err(|_| ())?,
+            data.len()
+                .try_into()
+                .map_err(|_| error_here!(ErrorType::ValueTooLarge))?,
             std::ptr::null_mut(),
             0,
             &mut signature_len,
@@ -302,11 +307,7 @@ fn sign_ncrypt(
     };
     // 0 is "ERROR_SUCCESS" (but "ERROR_SUCCESS" is unsigned, whereas SECURITY_STATUS is signed)
     if status != 0 {
-        error!(
-            "NCryptSignHash failed trying to get signature buffer length, {}",
-            status
-        );
-        return Err(());
+        return Err(error_here!(ErrorType::ExternalError, status.to_string()));
     }
     let mut signature = vec![0; signature_len as usize];
     if !do_signature {
@@ -318,7 +319,9 @@ fn sign_ncrypt(
             *ncrypt_handle,
             params_ptr,
             data.as_mut_ptr(),
-            data.len().try_into().map_err(|_| ())?,
+            data.len()
+                .try_into()
+                .map_err(|_| error_here!(ErrorType::ValueTooLarge))?,
             signature.as_mut_ptr(),
             signature_len,
             &mut final_signature_len,
@@ -326,15 +329,10 @@ fn sign_ncrypt(
         )
     };
     if status != 0 {
-        error!("NCryptSignHash failed signing data {}", status);
-        return Err(());
+        return Err(error_here!(ErrorType::ExternalError, status.to_string()));
     }
     if final_signature_len != signature_len {
-        error!(
-            "NCryptSignHash: inconsistent signature lengths? {} != {}",
-            final_signature_len, signature_len
-        );
-        return Err(());
+        return Err(error_here!(ErrorType::ExternalError));
     }
     Ok(signature)
 }
@@ -345,10 +343,9 @@ fn sign_cryptoapi(
     data: &[u8],
     params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
     do_signature: bool,
-) -> Result<Vec<u8>, ()> {
+) -> Result<Vec<u8>, Error> {
     if params.is_some() {
-        error!("non-None signature params cannot be used with CryptoAPI");
-        return Err(());
+        return Err(error_here!(ErrorType::LibraryFailure));
     }
     // data will be an encoded DigestInfo, which specifies the hash algorithm and bytes of the hash
     // to sign. However, CryptoAPI requires directly specifying the bytes of the hash, so it must
@@ -367,11 +364,10 @@ fn sign_cryptoapi(
         )
     } != 1
     {
-        error!(
-            "CryptSignHash failed trying to get signature buffer length: 0x{:x}",
-            unsafe { GetLastError() }
-        );
-        return Err(());
+        return Err(error_here!(
+            ErrorType::ExternalError,
+            unsafe { GetLastError() }.to_string()
+        ));
     }
     let mut signature = vec![0; signature_len as usize];
     if !do_signature {
@@ -389,17 +385,13 @@ fn sign_cryptoapi(
         )
     } != 1
     {
-        error!("CryptSignHash failed signing data: 0x{:x}", unsafe {
-            GetLastError()
-        });
-        return Err(());
+        return Err(error_here!(
+            ErrorType::ExternalError,
+            unsafe { GetLastError() }.to_string()
+        ));
     }
     if final_signature_len != signature_len {
-        error!(
-            "CryptSignHash: inconsistent signature lengths? {} != {}",
-            final_signature_len, signature_len
-        );
-        return Err(());
+        return Err(error_here!(ErrorType::ExternalError));
     }
     // CryptoAPI returns the signature with the most significant byte last (little-endian),
     // whereas PKCS#11 expects the most significant byte first (big-endian).
@@ -410,30 +402,28 @@ fn sign_cryptoapi(
 struct HCryptHash(HCRYPTHASH);
 
 impl HCryptHash {
-    fn new(hcryptprov: &HCRYPTPROV, hash_bytes: &[u8]) -> Result<HCryptHash, ()> {
+    fn new(hcryptprov: &HCRYPTPROV, hash_bytes: &[u8]) -> Result<HCryptHash, Error> {
         let alg = match hash_bytes.len() {
             20 => CALG_SHA1,
             32 => CALG_SHA_256,
             48 => CALG_SHA_384,
             64 => CALG_SHA_512,
             _ => {
-                error!(
-                    "HCryptHash::new: invalid hash of length {}",
-                    hash_bytes.len()
-                );
-                return Err(());
+                return Err(error_here!(ErrorType::UnsupportedInput));
             }
         };
         let mut hash: HCRYPTHASH = 0;
         if unsafe { CryptCreateHash(*hcryptprov, alg, 0, 0, &mut hash) } != 1 {
-            error!("CryptCreateHash failed: 0x{:x}", unsafe { GetLastError() });
-            return Err(());
+            return Err(error_here!(
+                ErrorType::ExternalError,
+                unsafe { GetLastError() }.to_string()
+            ));
         }
         if unsafe { CryptSetHashParam(hash, HP_HASHVAL, hash_bytes.as_ptr(), 0) } != 1 {
-            error!("CryptSetHashParam failed: 0x{:x}", unsafe {
-                GetLastError()
-            });
-            return Err(());
+            return Err(error_here!(
+                ErrorType::ExternalError,
+                unsafe { GetLastError() }.to_string()
+            ));
         }
         Ok(HCryptHash(hash))
     }
@@ -477,7 +467,10 @@ enum SignParams {
 }
 
 impl SignParams {
-    fn new(key_type: KeyType, params: &Option<CK_RSA_PKCS_PSS_PARAMS>) -> Result<SignParams, ()> {
+    fn new(
+        key_type: KeyType,
+        params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
+    ) -> Result<SignParams, Error> {
         // EC is easy, so handle that first.
         match key_type {
             KeyType::EC => return Ok(SignParams::EC),
@@ -500,11 +493,7 @@ impl SignParams {
             CKM_SHA384 => SHA384_ALGORITHM_STRING,
             CKM_SHA512 => SHA512_ALGORITHM_STRING,
             _ => {
-                error!(
-                    "unsupported algorithm to use with RSA-PSS: {}",
-                    unsafe_packed_field_access!(pss_params.hashAlg)
-                );
-                return Err(());
+                return Err(error_here!(ErrorType::UnsupportedInput));
             }
         };
         Ok(SignParams::RSA_PSS(BCRYPT_PSS_PADDING_INFO {
@@ -526,15 +515,16 @@ impl SignParams {
     }
 
     fn flags(&self) -> u32 {
-        match self {
-            &SignParams::EC => 0,
-            &SignParams::RSA_PKCS1(_) => NCRYPT_PAD_PKCS1_FLAG,
-            &SignParams::RSA_PSS(_) => NCRYPT_PAD_PSS_FLAG,
+        match *self {
+            SignParams::EC => 0,
+            SignParams::RSA_PKCS1(_) => NCRYPT_PAD_PKCS1_FLAG,
+            SignParams::RSA_PSS(_) => NCRYPT_PAD_PSS_FLAG,
         }
     }
 }
 
 /// A helper enum to identify a private key's type. We support EC and RSA.
+#[allow(clippy::upper_case_acronyms)]
 #[derive(Clone, Copy, Debug)]
 pub enum KeyType {
     EC,
@@ -569,7 +559,7 @@ pub struct Key {
 }
 
 impl Key {
-    fn new(cert_context: PCCERT_CONTEXT) -> Result<Key, ()> {
+    fn new(cert_context: PCCERT_CONTEXT) -> Result<Key, Error> {
         let cert = unsafe { *cert_context };
         let cert_der =
             unsafe { slice::from_raw_parts(cert.pbCertEncoded, cert.cbCertEncoded as usize) };
@@ -581,10 +571,10 @@ impl Key {
         let spki = &cert_info.SubjectPublicKeyInfo;
         let algorithm_oid = unsafe { CStr::from_ptr(spki.Algorithm.pszObjId) }
             .to_str()
-            .map_err(|_| ())?;
+            .map_err(|_| error_here!(ErrorType::ExternalError))?;
         let (key_type_enum, key_type_attribute) = if algorithm_oid == szOID_RSA_RSA {
             if spki.PublicKey.cUnusedBits != 0 {
-                return Err(());
+                return Err(error_here!(ErrorType::ExternalError));
             }
             let public_key_bytes = unsafe {
                 std::slice::from_raw_parts(spki.PublicKey.pbData, spki.PublicKey.cbData as usize)
@@ -600,7 +590,7 @@ impl Key {
             );
             (KeyType::EC, CKK_EC)
         } else {
-            return Err(());
+            return Err(error_here!(ErrorType::LibraryFailure));
         };
         let cert = CertContext::new(cert_context);
         Ok(Key {
@@ -626,7 +616,7 @@ impl Key {
         &self.token
     }
 
-    pub fn id(&self) -> &[u8] {
+    fn id(&self) -> &[u8] {
         &self.id
     }
 
@@ -652,6 +642,48 @@ impl Key {
         }
     }
 
+    fn sign_with_retry(
+        &mut self,
+        data: &[u8],
+        params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
+        do_signature: bool,
+    ) -> Result<Vec<u8>, Error> {
+        let result = self.sign_internal(data, params, do_signature);
+        if result.is_ok() {
+            return result;
+        }
+        // Some devices appear to not work well when the key handle is held for too long or if a
+        // card is inserted/removed while Firefox is running. Try refreshing the key handle.
+        debug!("sign failed: refreshing key handle");
+        let _ = self.key_handle.take();
+        self.sign_internal(data, params, do_signature)
+    }
+
+    /// data: the data to sign
+    /// do_signature: if true, actually perform the signature. Otherwise, return a `Vec<u8>` of the
+    /// length the signature would be, if performed.
+    fn sign_internal(
+        &mut self,
+        data: &[u8],
+        params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
+        do_signature: bool,
+    ) -> Result<Vec<u8>, Error> {
+        // If this key hasn't been used for signing yet, there won't be a cached key handle. Obtain
+        // and cache it if this is the case. Doing so can cause the underlying implementation to
+        // show an authentication or pin prompt to the user. Caching the handle can avoid causing
+        // multiple prompts to be displayed in some cases.
+        if self.key_handle.is_none() {
+            let _ = self.key_handle.replace(KeyHandle::from_cert(&self.cert)?);
+        }
+        let key = match &self.key_handle {
+            Some(key) => key,
+            None => return Err(error_here!(ErrorType::LibraryFailure)),
+        };
+        key.sign(data, params, do_signature, self.key_type_enum)
+    }
+}
+
+impl CryptokiObject for Key {
     fn matches(&self, slot_type: SlotType, attrs: &[(CK_ATTRIBUTE_TYPE, Vec<u8>)]) -> bool {
         if slot_type != self.slot_type {
             return false;
@@ -698,91 +730,27 @@ impl Key {
             _ => None,
         }
     }
+}
 
-    pub fn get_signature_length(
+impl Sign for Key {
+    fn get_signature_length(
         &mut self,
         data: &[u8],
         params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
-    ) -> Result<usize, ()> {
+    ) -> Result<usize, Error> {
         match self.sign_with_retry(data, params, false) {
             Ok(dummy_signature_bytes) => Ok(dummy_signature_bytes.len()),
-            Err(()) => Err(()),
+            Err(e) => Err(e),
         }
     }
 
-    pub fn sign(
+    fn sign(
         &mut self,
         data: &[u8],
         params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
-    ) -> Result<Vec<u8>, ()> {
+    ) -> Result<Vec<u8>, Error> {
         self.sign_with_retry(data, params, true)
     }
-
-    fn sign_with_retry(
-        &mut self,
-        data: &[u8],
-        params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
-        do_signature: bool,
-    ) -> Result<Vec<u8>, ()> {
-        let result = self.sign_internal(data, params, do_signature);
-        if result.is_ok() {
-            return result;
-        }
-        // Some devices appear to not work well when the key handle is held for too long or if a
-        // card is inserted/removed while Firefox is running. Try refreshing the key handle.
-        debug!("sign failed: refreshing key handle");
-        let _ = self.key_handle.take();
-        self.sign_internal(data, params, do_signature)
-    }
-
-    /// data: the data to sign
-    /// do_signature: if true, actually perform the signature. Otherwise, return a `Vec<u8>` of the
-    /// length the signature would be, if performed.
-    fn sign_internal(
-        &mut self,
-        data: &[u8],
-        params: &Option<CK_RSA_PKCS_PSS_PARAMS>,
-        do_signature: bool,
-    ) -> Result<Vec<u8>, ()> {
-        // If this key hasn't been used for signing yet, there won't be a cached key handle. Obtain
-        // and cache it if this is the case. Doing so can cause the underlying implementation to
-        // show an authentication or pin prompt to the user. Caching the handle can avoid causing
-        // multiple prompts to be displayed in some cases.
-        if self.key_handle.is_none() {
-            let _ = self.key_handle.replace(KeyHandle::from_cert(&self.cert)?);
-        }
-        let key = match &self.key_handle {
-            Some(key) => key,
-            None => {
-                error!("key_handle not set when it should have just been set?");
-                return Err(());
-            }
-        };
-        key.sign(data, params, do_signature, self.key_type_enum)
-    }
-}
-
-/// A helper enum that represents the two types of PKCS #11 objects we support: certificates and
-/// keys.
-pub enum Object {
-    Cert(Cert),
-    Key(Key),
-}
-
-impl Object {
-    pub fn matches(&self, slot_type: SlotType, attrs: &[(CK_ATTRIBUTE_TYPE, Vec<u8>)]) -> bool {
-        match self {
-            Object::Cert(cert) => cert.matches(slot_type, attrs),
-            Object::Key(key) => key.matches(slot_type, attrs),
-        }
-    }
-
-    pub fn get_attribute(&self, attribute: CK_ATTRIBUTE_TYPE) -> Option<&[u8]> {
-        match self {
-            Object::Cert(cert) => cert.get_attribute(attribute),
-            Object::Key(key) => key.get_attribute(attribute),
-        }
-    }
 }
 
 struct CertStore {
@@ -813,21 +781,6 @@ impl CertStore {
     }
 }
 
-pub const SUPPORTED_ATTRIBUTES: &[CK_ATTRIBUTE_TYPE] = &[
-    CKA_CLASS,
-    CKA_TOKEN,
-    CKA_LABEL,
-    CKA_ID,
-    CKA_VALUE,
-    CKA_ISSUER,
-    CKA_SERIAL_NUMBER,
-    CKA_SUBJECT,
-    CKA_PRIVATE,
-    CKA_KEY_TYPE,
-    CKA_MODULUS,
-    CKA_EC_PARAMS,
-];
-
 // Given a pointer to a CERT_CHAIN_CONTEXT, enumerates each chain in the context and each element
 // in each chain to gather every CERT_CONTEXT pointed to by the CERT_CHAIN_CONTEXT.
 // https://docs.microsoft.com/en-us/windows/win32/api/wincrypt/ns-wincrypt-cert_chain_context says
@@ -868,89 +821,94 @@ fn gather_cert_contexts(cert_chain_context: *const CERT_CHAIN_CONTEXT) -> Vec<*c
     cert_contexts
 }
 
-/// Attempts to enumerate certificates with private keys exposed by the OS. Currently only looks in
-/// the "My" cert store of the current user. In the future this may look in more locations.
-pub fn list_objects() -> Vec<Object> {
-    let mut objects = Vec::new();
-    let location_flags = CERT_SYSTEM_STORE_CURRENT_USER // TODO: loop over multiple locations
-        | CERT_STORE_OPEN_EXISTING_FLAG
-        | CERT_STORE_READONLY_FLAG;
-    let store_name = match CString::new("My") {
-        Ok(store_name) => store_name,
-        Err(null_error) => {
-            error!("CString::new given input with a null byte: {}", null_error);
-            return objects;
-        }
-    };
-    let store = CertStore::new(unsafe {
-        CertOpenStore(
-            CERT_STORE_PROV_SYSTEM_REGISTRY_A,
-            0,
-            0,
-            location_flags,
-            store_name.as_ptr() as *const winapi::ctypes::c_void,
-        )
-    });
-    if store.is_null() {
-        error!("CertOpenStore failed");
-        return objects;
-    }
-    let find_params = CERT_CHAIN_FIND_ISSUER_PARA {
-        cbSize: std::mem::size_of::<CERT_CHAIN_FIND_ISSUER_PARA>() as u32,
-        pszUsageIdentifier: std::ptr::null(),
-        dwKeySpec: 0,
-        dwAcquirePrivateKeyFlags: 0,
-        cIssuer: 0,
-        rgIssuer: std::ptr::null_mut(),
-        pfnFindCallback: None,
-        pvFindArg: std::ptr::null_mut(),
-        pdwIssuerChainIndex: std::ptr::null_mut(),
-        pdwIssuerElementIndex: std::ptr::null_mut(),
-    };
-    let mut cert_chain_context: PCCERT_CHAIN_CONTEXT = std::ptr::null_mut();
-    loop {
-        // CertFindChainInStore finds all certificates with private keys in the store. It also
-        // attempts to build a verified certificate chain to a trust anchor for each certificate.
-        // We gather and hold onto these extra certificates so that gecko can use them when
-        // filtering potential client certificates according to the acceptable CAs list sent by
-        // servers when they request client certificates.
-        cert_chain_context = unsafe {
-            CertFindChainInStore(
-                *store,
-                X509_ASN_ENCODING,
-                CERT_CHAIN_FIND_BY_ISSUER_CACHE_ONLY_FLAG
-                    | CERT_CHAIN_FIND_BY_ISSUER_CACHE_ONLY_URL_FLAG,
-                CERT_CHAIN_FIND_BY_ISSUER,
-                &find_params as *const CERT_CHAIN_FIND_ISSUER_PARA as *const winapi::ctypes::c_void,
-                cert_chain_context,
+pub struct Backend {}
+
+impl ClientCertsBackend for Backend {
+    type Cert = Cert;
+    type Key = Key;
+
+    /// Attempts to enumerate certificates with private keys exposed by the OS. Currently only looks in
+    /// the "My" cert store of the current user. In the future this may look in more locations.
+    fn find_objects(&self) -> Result<(Vec<Cert>, Vec<Key>), Error> {
+        let mut certs = Vec::new();
+        let mut keys = Vec::new();
+        let location_flags = CERT_SYSTEM_STORE_CURRENT_USER
+            | CERT_STORE_OPEN_EXISTING_FLAG
+            | CERT_STORE_READONLY_FLAG;
+        let store_name = match CString::new("My") {
+            Ok(store_name) => store_name,
+            Err(_) => return Err(error_here!(ErrorType::LibraryFailure)),
+        };
+        let store = CertStore::new(unsafe {
+            CertOpenStore(
+                CERT_STORE_PROV_SYSTEM_REGISTRY_A,
+                0,
+                0,
+                location_flags,
+                store_name.as_ptr() as *const winapi::ctypes::c_void,
             )
-        };
-        if cert_chain_context.is_null() {
-            break;
+        });
+        if store.is_null() {
+            return Err(error_here!(ErrorType::ExternalError));
         }
-        let cert_contexts = gather_cert_contexts(cert_chain_context);
-        // The 0th CERT_CONTEXT is the end-entity (i.e. the certificate with the private key we're
-        // after).
-        match cert_contexts.get(0) {
-            Some(cert_context) => {
-                let key = match Key::new(*cert_context) {
-                    Ok(key) => key,
-                    Err(()) => continue,
-                };
-                let cert = match Cert::new(*cert_context) {
-                    Ok(cert) => cert,
-                    Err(()) => continue,
-                };
-                objects.push(Object::Cert(cert));
-                objects.push(Object::Key(key));
-            }
-            None => {}
+        let find_params = CERT_CHAIN_FIND_ISSUER_PARA {
+            cbSize: std::mem::size_of::<CERT_CHAIN_FIND_ISSUER_PARA>() as u32,
+            pszUsageIdentifier: std::ptr::null(),
+            dwKeySpec: 0,
+            dwAcquirePrivateKeyFlags: 0,
+            cIssuer: 0,
+            rgIssuer: std::ptr::null_mut(),
+            pfnFindCallback: None,
+            pvFindArg: std::ptr::null_mut(),
+            pdwIssuerChainIndex: std::ptr::null_mut(),
+            pdwIssuerElementIndex: std::ptr::null_mut(),
         };
-        for cert_context in cert_contexts.iter().skip(1) {
-            if let Ok(cert) = Cert::new(*cert_context) {
-                objects.push(Object::Cert(cert));
+        let mut cert_chain_context: PCCERT_CHAIN_CONTEXT = std::ptr::null_mut();
+        loop {
+            // CertFindChainInStore finds all certificates with private keys in the store. It also
+            // attempts to build a verified certificate chain to a trust anchor for each certificate.
+            // We gather and hold onto these extra certificates so that gecko can use them when
+            // filtering potential client certificates according to the acceptable CAs list sent by
+            // servers when they request client certificates.
+            cert_chain_context = unsafe {
+                CertFindChainInStore(
+                    *store,
+                    X509_ASN_ENCODING,
+                    CERT_CHAIN_FIND_BY_ISSUER_CACHE_ONLY_FLAG
+                        | CERT_CHAIN_FIND_BY_ISSUER_CACHE_ONLY_URL_FLAG,
+                    CERT_CHAIN_FIND_BY_ISSUER,
+                    &find_params as *const CERT_CHAIN_FIND_ISSUER_PARA
+                        as *const winapi::ctypes::c_void,
+                    cert_chain_context,
+                )
+            };
+            if cert_chain_context.is_null() {
+                break;
+            }
+            let cert_contexts = gather_cert_contexts(cert_chain_context);
+            // The 0th CERT_CONTEXT is the end-entity (i.e. the certificate with the private key we're
+            // after).
+            match cert_contexts.get(0) {
+                Some(cert_context) => {
+                    let key = match Key::new(*cert_context) {
+                        Ok(key) => key,
+                        Err(_) => continue,
+                    };
+                    let cert = match Cert::new(*cert_context) {
+                        Ok(cert) => cert,
+                        Err(_) => continue,
+                    };
+                    certs.push(cert);
+                    keys.push(key);
+                }
+                None => {}
+            };
+            for cert_context in cert_contexts.iter().skip(1) {
+                if let Ok(cert) = Cert::new(*cert_context) {
+                    certs.push(cert);
+                }
             }
         }
+        Ok((certs, keys))
     }
-    objects
 }
