@@ -90,7 +90,7 @@ function check_telemetry() {
   );
   equal(
     histogram.values[16],
-    2,
+    3,
     "Actual and expected SEC_ERROR_INVALID_TIME values should match"
   );
   equal(
@@ -100,7 +100,7 @@ function check_telemetry() {
   );
   equal(
     histogram.values[19],
-    3,
+    4,
     "Actual and expected MOZILLA_PKIX_ERROR_SELF_SIGNED_CERT values should match"
   );
   equal(
@@ -129,7 +129,7 @@ function check_telemetry() {
   );
   equal(
     keySizeHistogram.values[3],
-    68,
+    70,
     "Actual and expected verification failures unrelated to key size should match"
   );
 
@@ -246,6 +246,12 @@ function add_simple_tests() {
     Ci.nsICertOverrideService.ERROR_TIME,
     SEC_ERROR_INVALID_TIME
   );
+  add_cert_override_test(
+    "before-epoch-self-signed.example.com",
+    Ci.nsICertOverrideService.ERROR_TIME |
+      Ci.nsICertOverrideService.ERROR_UNTRUSTED,
+    MOZILLA_PKIX_ERROR_SELF_SIGNED_CERT
+  );
   add_cert_override_test(
     "selfsigned.example.com",
     Ci.nsICertOverrideService.ERROR_UNTRUSTED,
@@ -566,6 +572,14 @@ function add_simple_tests() {
       expectedBits,
       false
     );
+    certOverrideService.rememberValidityOverride(
+      "::1",
+      80,
+      {},
+      cert,
+      expectedBits,
+      false
+    );
     Assert.ok(
       certOverrideService.hasMatchingOverride(
         "example.com",
@@ -596,6 +610,10 @@ function add_simple_tests() {
       ),
       "Should have added override for example.org:443"
     );
+    Assert.ok(
+      certOverrideService.hasMatchingOverride("::1", 80, {}, cert, {}, {}),
+      "Should have added override for [::1]:80"
+    );
     Assert.ok(
       !certOverrideService.hasMatchingOverride(
         "example.org",
