@@ -14,11 +14,13 @@
 #include "sandbox/win/src/interception.h"
 #include "sandbox/win/src/internal_types.h"
 #include "sandbox/win/src/ipc_tags.h"
+#include "sandbox/win/src/line_break_dispatcher.h"
 #include "sandbox/win/src/named_pipe_dispatcher.h"
 #include "sandbox/win/src/process_mitigations_win32k_dispatcher.h"
 #include "sandbox/win/src/process_thread_dispatcher.h"
 #include "sandbox/win/src/registry_dispatcher.h"
 #include "sandbox/win/src/sandbox_policy_base.h"
+#include "sandbox/win/src/signed_dispatcher.h"
 #include "sandbox/win/src/sync_dispatcher.h"
 
 namespace sandbox {
@@ -29,58 +31,75 @@ TopLevelDispatcher::TopLevelDispatcher(PolicyBase* policy) : policy_(policy) {
   Dispatcher* dispatcher;
 
   dispatcher = new FilesystemDispatcher(policy_);
-  ipc_targets_[IPC_NTCREATEFILE_TAG] = dispatcher;
-  ipc_targets_[IPC_NTOPENFILE_TAG] = dispatcher;
-  ipc_targets_[IPC_NTSETINFO_RENAME_TAG] = dispatcher;
-  ipc_targets_[IPC_NTQUERYATTRIBUTESFILE_TAG] = dispatcher;
-  ipc_targets_[IPC_NTQUERYFULLATTRIBUTESFILE_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTCREATEFILE)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTOPENFILE)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTSETINFO_RENAME)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTQUERYATTRIBUTESFILE)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTQUERYFULLATTRIBUTESFILE)] =
+      dispatcher;
   filesystem_dispatcher_.reset(dispatcher);
 
   dispatcher = new NamedPipeDispatcher(policy_);
-  ipc_targets_[IPC_CREATENAMEDPIPEW_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::CREATENAMEDPIPEW)] = dispatcher;
   named_pipe_dispatcher_.reset(dispatcher);
 
   dispatcher = new ThreadProcessDispatcher(policy_);
-  ipc_targets_[IPC_NTOPENTHREAD_TAG] = dispatcher;
-  ipc_targets_[IPC_NTOPENPROCESS_TAG] = dispatcher;
-  ipc_targets_[IPC_CREATEPROCESSW_TAG] = dispatcher;
-  ipc_targets_[IPC_NTOPENPROCESSTOKEN_TAG] = dispatcher;
-  ipc_targets_[IPC_NTOPENPROCESSTOKENEX_TAG] = dispatcher;
-  ipc_targets_[IPC_CREATETHREAD_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTOPENTHREAD)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTOPENPROCESS)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::CREATEPROCESSW)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTOPENPROCESSTOKEN)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTOPENPROCESSTOKENEX)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::CREATETHREAD)] = dispatcher;
   thread_process_dispatcher_.reset(dispatcher);
 
   dispatcher = new SyncDispatcher(policy_);
-  ipc_targets_[IPC_CREATEEVENT_TAG] = dispatcher;
-  ipc_targets_[IPC_OPENEVENT_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::CREATEEVENT)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::OPENEVENT)] = dispatcher;
   sync_dispatcher_.reset(dispatcher);
 
   dispatcher = new RegistryDispatcher(policy_);
-  ipc_targets_[IPC_NTCREATEKEY_TAG] = dispatcher;
-  ipc_targets_[IPC_NTOPENKEY_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTCREATEKEY)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::NTOPENKEY)] = dispatcher;
   registry_dispatcher_.reset(dispatcher);
 
   dispatcher = new HandleDispatcher(policy_);
-  ipc_targets_[IPC_DUPLICATEHANDLEPROXY_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::DUPLICATEHANDLEPROXY)] = dispatcher;
   handle_dispatcher_.reset(dispatcher);
 
   dispatcher = new ProcessMitigationsWin32KDispatcher(policy_);
-  ipc_targets_[IPC_GDI_GDIDLLINITIALIZE_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_GETSTOCKOBJECT_TAG] = dispatcher;
-  ipc_targets_[IPC_USER_REGISTERCLASSW_TAG] = dispatcher;
-  ipc_targets_[IPC_USER_ENUMDISPLAYMONITORS_TAG] = dispatcher;
-  ipc_targets_[IPC_USER_ENUMDISPLAYDEVICES_TAG] = dispatcher;
-  ipc_targets_[IPC_USER_GETMONITORINFO_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_CREATEOPMPROTECTEDOUTPUTS_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_GETCERTIFICATE_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_GETCERTIFICATESIZE_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_DESTROYOPMPROTECTEDOUTPUT_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_CONFIGUREOPMPROTECTEDOUTPUT_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_GETOPMINFORMATION_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_GETOPMRANDOMNUMBER_TAG] = dispatcher;
-  ipc_targets_[IPC_GDI_GETSUGGESTEDOPMPROTECTEDOUTPUTARRAYSIZE_TAG] =
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_GDIDLLINITIALIZE)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_GETSTOCKOBJECT)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::USER_REGISTERCLASSW)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::USER_ENUMDISPLAYMONITORS)] =
       dispatcher;
-  ipc_targets_[IPC_GDI_SETOPMSIGNINGKEYANDSEQUENCENUMBERS_TAG] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::USER_ENUMDISPLAYDEVICES)] =
+      dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::USER_GETMONITORINFO)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_CREATEOPMPROTECTEDOUTPUTS)] =
+      dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_GETCERTIFICATE)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_GETCERTIFICATESIZE)] =
+      dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_DESTROYOPMPROTECTEDOUTPUT)] =
+      dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_CONFIGUREOPMPROTECTEDOUTPUT)] =
+      dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_GETOPMINFORMATION)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(IpcTag::GDI_GETOPMRANDOMNUMBER)] =
+      dispatcher;
+  ipc_targets_[static_cast<size_t>(
+      IpcTag::GDI_GETSUGGESTEDOPMPROTECTEDOUTPUTARRAYSIZE)] = dispatcher;
+  ipc_targets_[static_cast<size_t>(
+      IpcTag::GDI_SETOPMSIGNINGKEYANDSEQUENCENUMBERS)] = dispatcher;
   process_mitigations_win32k_dispatcher_.reset(dispatcher);
+
+  dispatcher = new SignedDispatcher(policy_);
+  ipc_targets_[static_cast<size_t>(IpcTag::NTCREATESECTION)] = dispatcher;
+  signed_dispatcher_.reset(dispatcher);
+
+  dispatcher = new LineBreakDispatcher(policy_);
+  ipc_targets_[static_cast<size_t>(IpcTag::GETCOMPLEXLINEBREAKS)] = dispatcher;
+  line_break_dispatcher_.reset(dispatcher);
 }
 
 TopLevelDispatcher::~TopLevelDispatcher() {}
@@ -91,8 +110,8 @@ TopLevelDispatcher::~TopLevelDispatcher() {}
 Dispatcher* TopLevelDispatcher::OnMessageReady(IPCParams* ipc,
                                                CallbackGeneric* callback) {
   DCHECK(callback);
-  static const IPCParams ping1 = {IPC_PING1_TAG, {UINT32_TYPE}};
-  static const IPCParams ping2 = {IPC_PING2_TAG, {INOUTPTR_TYPE}};
+  static const IPCParams ping1 = {IpcTag::PING1, {UINT32_TYPE}};
+  static const IPCParams ping2 = {IpcTag::PING2, {INOUTPTR_TYPE}};
 
   if (ping1.Matches(ipc) || ping2.Matches(ipc)) {
     *callback = reinterpret_cast<CallbackGeneric>(
@@ -110,8 +129,8 @@ Dispatcher* TopLevelDispatcher::OnMessageReady(IPCParams* ipc,
 
 // Delegate to the appropriate dispatcher.
 bool TopLevelDispatcher::SetupService(InterceptionManager* manager,
-                                      int service) {
-  if (IPC_PING1_TAG == service || IPC_PING2_TAG == service)
+                                      IpcTag service) {
+  if (IpcTag::PING1 == service || IpcTag::PING2 == service)
     return true;
 
   Dispatcher* dispatcher = GetDispatcher(service);
@@ -122,12 +141,12 @@ bool TopLevelDispatcher::SetupService(InterceptionManager* manager,
   return dispatcher->SetupService(manager, service);
 }
 
-// We service IPC_PING_TAG message which is a way to test a round trip of the
+// We service PING message which is a way to test a round trip of the
 // IPC subsystem. We receive a integer cookie and we are expected to return the
 // cookie times two (or three) and the current tick count.
 bool TopLevelDispatcher::Ping(IPCInfo* ipc, void* arg1) {
   switch (ipc->ipc_tag) {
-    case IPC_PING1_TAG: {
+    case IpcTag::PING1: {
       IPCInt ipc_int(arg1);
       uint32_t cookie = ipc_int.As32Bit();
       ipc->return_info.extended_count = 2;
@@ -135,7 +154,7 @@ bool TopLevelDispatcher::Ping(IPCInfo* ipc, void* arg1) {
       ipc->return_info.extended[1].unsigned_int = 2 * cookie;
       return true;
     }
-    case IPC_PING2_TAG: {
+    case IpcTag::PING2: {
       CountedBuffer* io_buffer = reinterpret_cast<CountedBuffer*>(arg1);
       if (sizeof(uint32_t) != io_buffer->Size())
         return false;
@@ -149,11 +168,11 @@ bool TopLevelDispatcher::Ping(IPCInfo* ipc, void* arg1) {
   }
 }
 
-Dispatcher* TopLevelDispatcher::GetDispatcher(int ipc_tag) {
-  if (ipc_tag >= IPC_LAST_TAG || ipc_tag <= IPC_UNUSED_TAG)
+Dispatcher* TopLevelDispatcher::GetDispatcher(IpcTag ipc_tag) {
+  if (ipc_tag >= IpcTag::LAST || ipc_tag <= IpcTag::UNUSED)
     return nullptr;
 
-  return ipc_targets_[ipc_tag];
+  return ipc_targets_[static_cast<size_t>(ipc_tag)];
 }
 
 }  // namespace sandbox
