@@ -27,7 +27,8 @@ class RemoteSandboxBrokerParent
   // Note: we rely on the caller to keep this instance alive
   // until this promise resolves.
   // aThread is the thread to use to resolve the promise on if needed.
-  RefPtr<GenericPromise> Launch(const nsTArray<uint64_t>& aHandlesToShare,
+  RefPtr<GenericPromise> Launch(uint32_t aLaunchArch,
+                                const nsTArray<uint64_t>& aHandlesToShare,
                                 nsISerialEventTarget* aThread);
 
  private:
