@@ -3,29 +3,28 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsAutoPtr.h"
 #include "nsViewManager.h"
+
+#include "mozilla/MouseEvents.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/PresShellInlines.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/ProfilerLabels.h"
+#include "mozilla/StartupTimeline.h"
+#include "mozilla/dom/Document.h"
 #include "nsGfxCIID.h"
 #include "nsView.h"
 #include "nsCOMPtr.h"
-#include "mozilla/MouseEvents.h"
 #include "nsRegion.h"
 #include "nsCOMArray.h"
-#include "nsIPluginWidget.h"
 #include "nsXULPopupManager.h"
-#include "nsIPresShell.h"
-#include "nsIPresShellInlines.h"
 #include "nsPresContext.h"
-#include "mozilla/StartupTimeline.h"
-#include "GeckoProfiler.h"
 #include "nsRefreshDriver.h"
-#include "mozilla/Preferences.h"
 #include "nsContentUtils.h"  // for nsAutoScriptBlocker
 #include "nsLayoutUtils.h"
 #include "Layers.h"
 #include "gfxPlatform.h"
-#include "gfxPrefs.h"
-#include "nsIDocument.h"
+#include "WindowRenderer.h"
 
 /**
    XXX TODO XXX
@@ -52,14 +51,13 @@ using namespace mozilla::layers;
 #undef DEBUG_MOUSE_LOCATION
 
 // Weakly held references to all of the view managers
-nsTArray<nsViewManager*>* nsViewManager::gViewManagers = nullptr;
+StaticAutoPtr<nsTArray<nsViewManager*>> nsViewManager::gViewManagers;
 uint32_t nsViewManager::gLastUserEventTime = 0;
 
 nsViewManager::nsViewManager()
     : mPresShell(nullptr),
       mDelayedResize(NSCOORD_NONE, NSCOORD_NONE),
       mRootView(nullptr),
-      mRootViewManager(this),
       mRefreshDisableCount(0),
       mPainting(false),
       mRecursiveRefreshPending(false),
@@ -79,10 +77,7 @@ nsViewManager::~nsViewManager() {
     mRootView = nullptr;
   }
 
-  if (!IsRootVM()) {
-    // We have a strong ref to mRootViewManager
-    NS_RELEASE(mRootViewManager);
-  }
+  mRootViewManager = nullptr;
 
   NS_ASSERTION(gViewManagers != nullptr, "About to use null gViewManagers");
 
@@ -97,7 +92,6 @@ nsViewManager::~nsViewManager() {
   if (gViewManagers->IsEmpty()) {
     // There aren't any more view managers so
     // release the global array of view managers
-    delete gViewManagers;
     gViewManagers = nullptr;
   }
 
@@ -109,7 +103,7 @@ nsViewManager::~nsViewManager() {
 // We don't hold a reference to the presentation context because it
 // holds a reference to us.
 nsresult nsViewManager::Init(nsDeviceContext* aContext) {
-  NS_PRECONDITION(nullptr != aContext, "null ptr");
+  MOZ_ASSERT(nullptr != aContext, "null ptr");
 
   if (nullptr == aContext) {
     return NS_ERROR_NULL_POINTER;
@@ -133,8 +127,8 @@ nsView* nsViewManager::CreateView(const nsRect& aBounds, nsView* aParent,
 }
 
 void nsViewManager::SetRootView(nsView* aView) {
-  NS_PRECONDITION(!aView || aView->GetViewManager() == this,
-                  "Unexpected viewmanager on root view");
+  MOZ_ASSERT(!aView || aView->GetViewManager() == this,
+             "Unexpected viewmanager on root view");
 
   // Do NOT destroy the current root view. It's the caller's responsibility
   // to destroy it
@@ -171,16 +165,22 @@ void nsViewManager::GetWindowDimensions(nscoord* aWidth, nscoord* aHeight) {
   }
 }
 
-void nsViewManager::DoSetWindowDimensions(nscoord aWidth, nscoord aHeight) {
+void nsViewManager::DoSetWindowDimensions(nscoord aWidth, nscoord aHeight,
+                                          bool aDoReflow) {
   nsRect oldDim = mRootView->GetDimensions();
   nsRect newDim(0, 0, aWidth, aHeight);
   // We care about resizes even when one dimension is already zero.
-  if (!oldDim.IsEqualEdges(newDim)) {
-    // Don't resize the widget. It is already being set elsewhere.
-    mRootView->SetDimensions(newDim, true, false);
-    if (mPresShell)
-      mPresShell->ResizeReflow(aWidth, aHeight, oldDim.Width(),
-                               oldDim.Height());
+  if (oldDim.IsEqualEdges(newDim)) {
+    return;
+  }
+  // Don't resize the widget. It is already being set elsewhere.
+  mRootView->SetDimensions(newDim, true, false);
+  if (RefPtr<PresShell> presShell = mPresShell) {
+    auto options = ResizeReflowOptions::NoOption;
+    if (!aDoReflow) {
+      options |= ResizeReflowOptions::SuppressReflow;
+    }
+    presShell->ResizeReflow(aWidth, aHeight, options);
   }
 }
 
@@ -213,7 +213,7 @@ void nsViewManager::SetWindowDimensions(nscoord aWidth, nscoord aHeight,
         FlushDelayedResize(false);
       }
       mDelayedResize.SizeTo(NSCOORD_NONE, NSCOORD_NONE);
-      DoSetWindowDimensions(aWidth, aHeight);
+      DoSetWindowDimensions(aWidth, aHeight, /* aDoReflow = */ true);
     } else {
       mDelayedResize.SizeTo(aWidth, aHeight);
       if (mPresShell) {
@@ -226,15 +226,9 @@ void nsViewManager::SetWindowDimensions(nscoord aWidth, nscoord aHeight,
 
 void nsViewManager::FlushDelayedResize(bool aDoReflow) {
   if (mDelayedResize != nsSize(NSCOORD_NONE, NSCOORD_NONE)) {
-    if (aDoReflow) {
-      DoSetWindowDimensions(mDelayedResize.width, mDelayedResize.height);
-      mDelayedResize.SizeTo(NSCOORD_NONE, NSCOORD_NONE);
-    } else if (mPresShell && !mPresShell->GetIsViewportOverridden()) {
-      nsPresContext* presContext = mPresShell->GetPresContext();
-      if (presContext) {
-        presContext->SetVisibleArea(nsRect(nsPoint(0, 0), mDelayedResize));
-      }
-    }
+    DoSetWindowDimensions(mDelayedResize.width, mDelayedResize.height,
+                          aDoReflow);
+    mDelayedResize.SizeTo(NSCOORD_NONE, NSCOORD_NONE);
   }
 }
 
@@ -262,7 +256,7 @@ nsView* nsViewManager::GetDisplayRootFor(nsView* aView) {
     // If we have a combobox dropdown popup within a panel popup, both the view
     // for the dropdown popup and its parent will be floating, so we need to
     // distinguish this situation. We do this by looking for a widget. Any view
-    // with a widget is a display root, except for plugins.
+    // with a widget is a display root.
     nsIWidget* widget = displayRoot->GetWidget();
     if (widget && widget->WindowType() == eWindowType_popup) {
       NS_ASSERTION(displayRoot->GetFloating() && displayParent->GetFloating(),
@@ -288,22 +282,7 @@ void nsViewManager::Refresh(nsView* aView,
     return;
   }
 
-  // damageRegion is the damaged area, in twips, relative to the view origin
-  nsRegion damageRegion = aRegion.ToAppUnits(AppUnitsPerDevPixel());
-
-  // move region from widget coordinates into view coordinates
-  damageRegion.MoveBy(-aView->ViewToWidgetOffset());
-
-  if (damageRegion.IsEmpty()) {
-#ifdef DEBUG_roc
-    nsRect viewRect = aView->GetDimensions();
-    nsRect damageRect = damageRegion.GetBounds();
-    printf_stderr(
-        "XXX Damage rectangle (%d,%d,%d,%d) does not intersect the widget's "
-        "view (%d,%d,%d,%d)!\n",
-        damageRect.x, damageRect.y, damageRect.width, damageRect.height,
-        viewRect.x, viewRect.y, viewRect.width, viewRect.height);
-#endif
+  if (aRegion.IsEmpty()) {
     return;
   }
 
@@ -325,18 +304,17 @@ void nsViewManager::Refresh(nsView* aView,
     NS_ASSERTION(GetDisplayRootFor(aView) == aView,
                  "Widgets that we paint must all be display roots");
 
-    if (mPresShell) {
+    if (RefPtr<PresShell> presShell = mPresShell) {
 #ifdef MOZ_DUMP_PAINTING
       if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
-        printf_stderr("--COMPOSITE-- %p\n", mPresShell);
+        printf_stderr("--COMPOSITE-- %p\n", presShell.get());
       }
 #endif
-      uint32_t paintFlags = nsIPresShell::PAINT_COMPOSITE;
-      LayerManager* manager = widget->GetLayerManager();
-      if (!manager->NeedsWidgetInvalidation()) {
-        manager->FlushRendering();
+      WindowRenderer* renderer = widget->GetWindowRenderer();
+      if (!renderer->NeedsWidgetInvalidation()) {
+        renderer->FlushRendering(wr::RenderReasons::WIDGET);
       } else {
-        mPresShell->Paint(aView, damageRegion, paintFlags);
+        presShell->SyncPaintFallback(aView);
       }
 #ifdef MOZ_DUMP_PAINTING
       if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
@@ -363,7 +341,7 @@ void nsViewManager::ProcessPendingUpdatesForView(nsView* aView,
     return;
   }
 
-  nsCOMPtr<nsIPresShell> rootShell(mPresShell);
+  RefPtr<PresShell> rootPresShell = mPresShell;
   AutoTArray<nsCOMPtr<nsIWidget>, 1> widgets;
   aView->GetViewManager()->ProcessPendingUpdatesRecurse(aView, widgets);
   for (uint32_t i = 0; i < widgets.Length(); ++i) {
@@ -372,8 +350,8 @@ void nsViewManager::ProcessPendingUpdatesForView(nsView* aView,
       if (view->mNeedsWindowPropertiesSync) {
         view->mNeedsWindowPropertiesSync = false;
         if (nsViewManager* vm = view->GetViewManager()) {
-          if (nsIPresShell* ps = vm->GetPresShell()) {
-            ps->SyncWindowProperties(view);
+          if (PresShell* presShell = vm->GetPresShell()) {
+            presShell->SyncWindowProperties(view);
           }
         }
       }
@@ -383,7 +361,7 @@ void nsViewManager::ProcessPendingUpdatesForView(nsView* aView,
       view->ResetWidgetBounds(false, true);
     }
   }
-  if (rootShell->GetViewManager() != this) {
+  if (rootPresShell->GetViewManager() != this) {
     return;  // presentation might have been torn down
   }
   if (aFlushDirtyRegion) {
@@ -393,7 +371,8 @@ void nsViewManager::ProcessPendingUpdatesForView(nsView* aView,
       nsIWidget* widget = widgets[i];
       nsView* view = nsView::GetViewFor(widget);
       if (view) {
-        view->GetViewManager()->ProcessPendingUpdatesPaint(widget);
+        RefPtr<nsViewManager> viewManager = view->GetViewManager();
+        viewManager->ProcessPendingUpdatesPaint(MOZ_KnownLive(widget));
       }
     }
     SetPainting(false);
@@ -449,16 +428,16 @@ void nsViewManager::ProcessPendingUpdatesPaint(nsIWidget* aWidget) {
       return;
     }
 
-    if (mPresShell) {
+    if (RefPtr<PresShell> presShell = mPresShell) {
 #ifdef MOZ_DUMP_PAINTING
       if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
         printf_stderr(
             "---- PAINT START ----PresShell(%p), nsView(%p), nsIWidget(%p)\n",
-            mPresShell, view, aWidget);
+            presShell.get(), view, aWidget);
       }
 #endif
 
-      mPresShell->Paint(view, nsRegion(), nsIPresShell::PAINT_LAYERS);
+      presShell->PaintAndRequestComposite(view, PaintFlags::None);
       view->SetForcedRepaint(false);
 
 #ifdef MOZ_DUMP_PAINTING
@@ -475,20 +454,22 @@ void nsViewManager::FlushDirtyRegionToWidget(nsView* aView) {
   NS_ASSERTION(aView->GetViewManager() == this,
                "FlushDirtyRegionToWidget called on view we don't own");
 
-  if (!aView->HasNonEmptyDirtyRegion()) return;
+  if (!aView->HasNonEmptyDirtyRegion()) {
+    return;
+  }
 
-  nsRegion* dirtyRegion = aView->GetDirtyRegion();
+  nsRegion& dirtyRegion = aView->GetDirtyRegion();
   nsView* nearestViewWithWidget = aView;
   while (!nearestViewWithWidget->HasWidget() &&
          nearestViewWithWidget->GetParent()) {
     nearestViewWithWidget = nearestViewWithWidget->GetParent();
   }
   nsRegion r =
-      ConvertRegionBetweenViews(*dirtyRegion, aView, nearestViewWithWidget);
+      ConvertRegionBetweenViews(dirtyRegion, aView, nearestViewWithWidget);
 
   nsViewManager* widgetVM = nearestViewWithWidget->GetViewManager();
   widgetVM->InvalidateWidgetArea(nearestViewWithWidget, r);
-  dirtyRegion->SetEmpty();
+  dirtyRegion.SetEmpty();
 }
 
 void nsViewManager::InvalidateView(nsView* aView) {
@@ -497,11 +478,9 @@ void nsViewManager::InvalidateView(nsView* aView) {
 }
 
 static void AddDirtyRegion(nsView* aView, const nsRegion& aDamagedRegion) {
-  nsRegion* dirtyRegion = aView->GetDirtyRegion();
-  if (!dirtyRegion) return;
-
-  dirtyRegion->Or(*dirtyRegion, aDamagedRegion);
-  dirtyRegion->SimplifyOutward(8);
+  nsRegion& dirtyRegion = aView->GetDirtyRegion();
+  dirtyRegion.Or(dirtyRegion, aDamagedRegion);
+  dirtyRegion.SimplifyOutward(8);
 }
 
 void nsViewManager::PostPendingUpdate() {
@@ -541,46 +520,8 @@ void nsViewManager::InvalidateWidgetArea(nsView* aWidgetView,
     return;
   }
 
-  // Update all child widgets with the damage. In the process,
-  // accumulate the union of all the child widget areas, or at least
-  // some subset of that.
-  nsRegion children;
-  if (widget->GetTransparencyMode() != eTransparencyTransparent) {
-    for (nsIWidget* childWidget = widget->GetFirstChild(); childWidget;
-         childWidget = childWidget->GetNextSibling()) {
-      nsView* view = nsView::GetViewFor(childWidget);
-      NS_ASSERTION(view != aWidgetView, "will recur infinitely");
-      nsWindowType type = childWidget->WindowType();
-      if (view && childWidget->IsVisible() && type != eWindowType_popup) {
-        NS_ASSERTION(childWidget->IsPlugin(),
-                     "Only plugin or popup widgets can be children!");
-
-        // We do not need to invalidate in plugin widgets, but we should
-        // exclude them from the invalidation region IF we're not on
-        // Mac. On Mac we need to draw under plugin widgets, because
-        // plugin widgets are basically invisible
-#ifndef XP_MACOSX
-        // GetBounds should compensate for chrome on a toplevel widget
-        LayoutDeviceIntRect bounds = childWidget->GetBounds();
-
-        nsTArray<LayoutDeviceIntRect> clipRects;
-        childWidget->GetWindowClipRegion(&clipRects);
-        for (uint32_t i = 0; i < clipRects.Length(); ++i) {
-          nsRect rr = LayoutDeviceIntRect::ToAppUnits(
-              clipRects[i] + bounds.TopLeft(), AppUnitsPerDevPixel());
-          children.Or(children, rr - aWidgetView->ViewToWidgetOffset());
-          children.SimplifyInward(20);
-        }
-#endif
-      }
-    }
-  }
-
-  nsRegion leftOver;
-  leftOver.Sub(aDamagedRegion, children);
-
-  if (!leftOver.IsEmpty()) {
-    for (auto iter = leftOver.RectIter(); !iter.Done(); iter.Next()) {
+  if (!aDamagedRegion.IsEmpty()) {
+    for (auto iter = aDamagedRegion.RectIter(); !iter.Done(); iter.Next()) {
       LayoutDeviceIntRect bounds = ViewToWidget(aWidgetView, iter.Get());
       widget->Invalidate(bounds);
     }
@@ -589,8 +530,8 @@ void nsViewManager::InvalidateWidgetArea(nsView* aWidgetView,
 
 static bool ShouldIgnoreInvalidation(nsViewManager* aVM) {
   while (aVM) {
-    nsIPresShell* shell = aVM->GetPresShell();
-    if (!shell || shell->ShouldIgnoreInvalidation()) {
+    PresShell* presShell = aVM->GetPresShell();
+    if (!presShell || presShell->ShouldIgnoreInvalidation()) {
       return true;
     }
     nsView* view = aVM->GetRootView()->GetParent();
@@ -611,7 +552,7 @@ void nsViewManager::InvalidateView(nsView* aView, const nsRect& aRect) {
 
 void nsViewManager::InvalidateViewNoSuppression(nsView* aView,
                                                 const nsRect& aRect) {
-  NS_PRECONDITION(nullptr != aView, "null view");
+  MOZ_ASSERT(nullptr != aView, "null view");
 
   NS_ASSERTION(aView->GetViewManager() == this,
                "InvalidateViewNoSuppression called on view we don't own");
@@ -657,26 +598,20 @@ void nsViewManager::InvalidateViews(nsView* aView) {
 }
 
 void nsViewManager::WillPaintWindow(nsIWidget* aWidget) {
-  RefPtr<nsIWidget> widget(aWidget);
-  if (widget) {
-    nsView* view = nsView::GetViewFor(widget);
-    LayerManager* manager = widget->GetLayerManager();
+  if (aWidget) {
+    nsView* view = nsView::GetViewFor(aWidget);
+    WindowRenderer* renderer = aWidget->GetWindowRenderer();
     if (view &&
-        (view->ForcedRepaint() || !manager->NeedsWidgetInvalidation())) {
+        (view->ForcedRepaint() || !renderer->NeedsWidgetInvalidation())) {
       ProcessPendingUpdates();
       // Re-get the view pointer here since the ProcessPendingUpdates might have
       // destroyed it during CallWillPaintOnObservers.
-      view = nsView::GetViewFor(widget);
+      view = nsView::GetViewFor(aWidget);
       if (view) {
         view->SetForcedRepaint(false);
       }
     }
   }
-
-  nsCOMPtr<nsIPresShell> shell = mPresShell;
-  if (shell) {
-    shell->WillPaintWindow();
-  }
 }
 
 bool nsViewManager::PaintWindow(nsIWidget* aWidget,
@@ -698,15 +633,14 @@ bool nsViewManager::PaintWindow(nsIWidget* aWidget,
 }
 
 void nsViewManager::DidPaintWindow() {
-  nsCOMPtr<nsIPresShell> shell = mPresShell;
-  if (shell) {
-    shell->DidPaintWindow();
+  if (RefPtr<PresShell> presShell = mPresShell) {
+    presShell->DidPaintWindow();
   }
 }
 
 void nsViewManager::DispatchEvent(WidgetGUIEvent* aEvent, nsView* aView,
                                   nsEventStatus* aStatus) {
-  AUTO_PROFILER_LABEL("nsViewManager::DispatchEvent", EVENTS);
+  AUTO_PROFILER_LABEL("nsViewManager::DispatchEvent", OTHER);
 
   WidgetMouseEvent* mouseEvent = aEvent->AsMouseEvent();
   if ((mouseEvent &&
@@ -717,8 +651,7 @@ void nsViewManager::DispatchEvent(WidgetGUIEvent* aEvent, nsView* aView,
        // create and destroy widgets.
        mouseEvent->mMessage != eMouseExitFromWidget &&
        mouseEvent->mMessage != eMouseEnterIntoWidget) ||
-      aEvent->HasKeyEventMessage() || aEvent->HasIMEEventMessage() ||
-      aEvent->mMessage == ePluginInputEvent) {
+      aEvent->HasKeyEventMessage() || aEvent->HasIMEEventMessage()) {
     gLastUserEventTime = PR_IntervalToMicroseconds(PR_IntervalNow());
   }
 
@@ -734,9 +667,7 @@ void nsViewManager::DispatchEvent(WidgetGUIEvent* aEvent, nsView* aView,
   // If the view has no frame, look for a view that does.
   nsIFrame* frame = view->GetFrame();
   if (!frame && (dispatchUsingCoordinates || aEvent->HasKeyEventMessage() ||
-                 aEvent->IsIMERelatedEvent() ||
-                 aEvent->IsNonRetargetedNativeEventDelivererForPlugin() ||
-                 aEvent->HasPluginActivationEventMessage())) {
+                 aEvent->IsIMERelatedEvent())) {
     while (view && !view->GetFrame()) {
       view = view->GetParent();
     }
@@ -750,17 +681,8 @@ void nsViewManager::DispatchEvent(WidgetGUIEvent* aEvent, nsView* aView,
     // Hold a refcount to the presshell. The continued existence of the
     // presshell will delay deletion of this view hierarchy should the event
     // want to cause its destruction in, say, some JavaScript event handler.
-    nsCOMPtr<nsIPresShell> shell = view->GetViewManager()->GetPresShell();
-    if (shell) {
-      if (aEvent->mMessage == eMouseDown || aEvent->mMessage == eMouseUp) {
-        AutoWeakFrame weakFrame(frame);
-        shell->FlushPendingNotifications(FlushType::Layout);
-        if (!weakFrame.IsAlive()) {
-          *aStatus = nsEventStatus_eIgnore;
-          return;
-        }
-      }
-      shell->HandleEvent(frame, aEvent, false, aStatus);
+    if (RefPtr<PresShell> presShell = view->GetViewManager()->GetPresShell()) {
+      presShell->HandleEvent(frame, aEvent, false, aStatus);
       return;
     }
   }
@@ -771,7 +693,7 @@ void nsViewManager::DispatchEvent(WidgetGUIEvent* aEvent, nsView* aView,
 // Recursively reparent widgets if necessary
 
 void nsViewManager::ReparentChildWidgets(nsView* aView, nsIWidget* aNewWidget) {
-  NS_PRECONDITION(aNewWidget, "");
+  MOZ_ASSERT(aNewWidget, "null widget");
 
   if (aView->HasWidget()) {
     // Check to see if the parent widget is the
@@ -803,8 +725,8 @@ void nsViewManager::ReparentChildWidgets(nsView* aView, nsIWidget* aNewWidget) {
 // Reparent a view and its descendant views widgets if necessary
 
 void nsViewManager::ReparentWidgets(nsView* aView, nsView* aParent) {
-  NS_PRECONDITION(aParent, "Must have a parent");
-  NS_PRECONDITION(aView, "Must have a view");
+  MOZ_ASSERT(aParent, "Must have a parent");
+  MOZ_ASSERT(aView, "Must have a view");
 
   // Quickly determine whether the view has pre-existing children or a
   // widget. In most cases the view will not have any pre-existing
@@ -825,8 +747,8 @@ void nsViewManager::ReparentWidgets(nsView* aView, nsView* aParent) {
 
 void nsViewManager::InsertChild(nsView* aParent, nsView* aChild,
                                 nsView* aSibling, bool aAfter) {
-  NS_PRECONDITION(nullptr != aParent, "null ptr");
-  NS_PRECONDITION(nullptr != aChild, "null ptr");
+  MOZ_ASSERT(nullptr != aParent, "null ptr");
+  MOZ_ASSERT(nullptr != aChild, "null ptr");
   NS_ASSERTION(aSibling == nullptr || aSibling->GetParent() == aParent,
                "tried to insert view with invalid sibling");
   NS_ASSERTION(!IsViewInserted(aChild),
@@ -981,21 +903,16 @@ void nsViewManager::DecrementDisableRefreshCount() {
   NS_ASSERTION(mRefreshDisableCount >= 0, "Invalid refresh disable count!");
 }
 
-void nsViewManager::GetRootWidget(nsIWidget** aWidget) {
-  if (!mRootView) {
-    *aWidget = nullptr;
-    return;
+already_AddRefed<nsIWidget> nsViewManager::GetRootWidget() {
+  nsCOMPtr<nsIWidget> rootWidget;
+  if (mRootView) {
+    if (mRootView->HasWidget()) {
+      rootWidget = mRootView->GetWidget();
+    } else if (mRootView->GetParent()) {
+      rootWidget = mRootView->GetParent()->GetViewManager()->GetRootWidget();
+    }
   }
-  if (mRootView->HasWidget()) {
-    *aWidget = mRootView->GetWidget();
-    NS_ADDREF(*aWidget);
-    return;
-  }
-  if (mRootView->GetParent()) {
-    mRootView->GetParent()->GetViewManager()->GetRootWidget(aWidget);
-    return;
-  }
-  *aWidget = nullptr;
+  return rootWidget.forget();
 }
 
 LayoutDeviceIntRect nsViewManager::ViewToWidget(nsView* aView,
@@ -1016,7 +933,8 @@ void nsViewManager::IsPainting(bool& aIsPainting) {
 
 void nsViewManager::ProcessPendingUpdates() {
   if (!IsRootVM()) {
-    RootViewManager()->ProcessPendingUpdates();
+    RefPtr<nsViewManager> rootViewManager = RootViewManager();
+    rootViewManager->ProcessPendingUpdates();
     return;
   }
 
@@ -1028,24 +946,29 @@ void nsViewManager::ProcessPendingUpdates() {
     CallWillPaintOnObservers();
 
     ProcessPendingUpdatesForView(mRootView, true);
+    if (mPresShell) {
+      if (nsPresContext* pc = mPresShell->GetPresContext()) {
+        pc->RefreshDriver()->ClearHasScheduleFlush();
+      }
+    }
   }
 }
 
 void nsViewManager::UpdateWidgetGeometry() {
   if (!IsRootVM()) {
-    RootViewManager()->UpdateWidgetGeometry();
+    RefPtr<nsViewManager> rootViewManager = RootViewManager();
+    rootViewManager->UpdateWidgetGeometry();
     return;
   }
 
   if (mHasPendingWidgetGeometryChanges) {
     mHasPendingWidgetGeometryChanges = false;
-    RefPtr<nsViewManager> strongThis(this);
     ProcessPendingUpdatesForView(mRootView, false);
   }
 }
 
 void nsViewManager::CallWillPaintOnObservers() {
-  NS_PRECONDITION(IsRootVM(), "Must be root VM for this to be called!");
+  MOZ_ASSERT(IsRootVM(), "Must be root VM for this to be called!");
 
   if (NS_WARN_IF(!gViewManagers)) {
     return;
@@ -1057,9 +980,8 @@ void nsViewManager::CallWillPaintOnObservers() {
     if (vm->RootViewManager() == this) {
       // One of our kids.
       if (vm->mRootView && vm->mRootView->IsEffectivelyVisible()) {
-        nsCOMPtr<nsIPresShell> shell = vm->GetPresShell();
-        if (shell) {
-          shell->WillPaint();
+        if (RefPtr<PresShell> presShell = vm->GetPresShell()) {
+          presShell->WillPaint();
         }
       }
     }
@@ -1072,17 +994,13 @@ void nsViewManager::GetLastUserEventTime(uint32_t& aTime) {
 
 void nsViewManager::InvalidateHierarchy() {
   if (mRootView) {
-    if (!IsRootVM()) {
-      NS_RELEASE(mRootViewManager);
-    }
+    mRootViewManager = nullptr;
     nsView* parent = mRootView->GetParent();
     if (parent) {
       mRootViewManager = parent->GetViewManager()->RootViewManager();
-      NS_ADDREF(mRootViewManager);
       NS_ASSERTION(mRootViewManager != this,
                    "Root view had a parent, but it has the same view manager");
-    } else {
-      mRootViewManager = this;
     }
+    // else, we are implicitly our own root view manager.
   }
 }
